/*********************************************************************
*                SEGGER Microcontroller GmbH & Co. KG                *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2016 SEGGER Microcontroller GmbH & Co. KG         *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 4.24                                             *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : BSP.c
Purpose : BSP for TI MP432 Launch Pad
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "BSP.h"
#include "msp.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
  P1OUT &= ~BIT0;                  // Initially clear P1.0 (LED1)
  P2OUT &= ~(BIT0 | BIT1 | BIT2);  // Initially clear P2.0, P2.1, P2.3 (RGBLED_RED, RGBLED_GREEN, RGBLED_BLUE)
  P1DIR |= BIT0;                   // Configure P1.0 (LED1) as output
  P2DIR |= (BIT0 | BIT1 | BIT2);   // Configure P2.0, P2.1, P2.3 (RGBLED_RED, RGBLED_GREEN, RGBLED_BLUE) as output
}

/*********************************************************************
*
*       BSP_SetLED()
*/
void BSP_SetLED(int Index) {
  if        (Index == 0) {
    P1OUT |= BIT0;  // Set P1.0 (LED1)
  } else if (Index == 1) {
	P2OUT |= BIT0;  // Set P2.0 (RGBLED_RED)
  } else if (Index == 2) {
	P2OUT |= BIT1;  // Set P2.1 (RGBLED_GREEN)
  } else if (Index == 3) {
	P2OUT |= BIT2;  // Set P2.2 (RGBLED_BLUE)
  }
}

/*********************************************************************
*
*       BSP_ClrLED()
*/
void BSP_ClrLED(int Index) {
  if        (Index == 0) {
    P1OUT &= ~BIT0;  // Clear P1.0 (LED1)
  } else if (Index == 1) {
	P2OUT &= ~BIT0;  // Clear P2.0 (RGBLED_RED)
  } else if (Index == 2) {
	P2OUT &= ~BIT1;  // Clear P2.1 (RGBLED_GREEN)
  } else if (Index == 3) {
	P2OUT &= ~BIT2;  // Clear P2.2 (RGBLED_BLUE)
  }
}

/*********************************************************************
*
*       BSP_ToggleLED()
*/
void BSP_ToggleLED(int Index) {
  if        (Index == 0) {
    P1OUT ^= BIT0;  // Toggle P1.0 (LED1)
  } else if (Index == 1) {
	P2OUT ^= BIT0;  // Toggle P2.0 (RGBLED_RED)
  } else if (Index == 2) {
	P2OUT ^= BIT1;  // Toggle P2.1 (RGBLED_GREEN)
  } else if (Index == 3) {
	P2OUT ^= BIT2;  // Toggle P2.2 (RGBLED_BLUE)
  }
}

/****** End Of File *************************************************/
