;/*********************************************************************
;*                SEGGER Microcontroller GmbH & Co. KG                *
;*                        The Embedded Experts                        *
;**********************************************************************
;*                                                                    *
;*       (c) 1995 - 2016 SEGGER Microcontroller GmbH & Co. KG         *
;*                                                                    *
;*       Internet: segger.com  Support: support_embos@segger.com      *
;*                                                                    *
;**********************************************************************
;*                                                                    *
;*       embOS * Real time operating system for microcontrollers      *
;*                                                                    *
;*       Please note:                                                 *
;*                                                                    *
;*       Knowledge of this file may under no circumstances            *
;*       be used to write a similar product or a real-time            *
;*       operating system for in-house use.                           *
;*                                                                    *
;*       Thank you for your fairness !                                *
;*                                                                    *
;**********************************************************************
;*                                                                    *
;*       OS version: Internal                                         *
;*                                                                    *
;*       Current version number will be inserted here                 *
;*       when shipment is built.                                      *
;*                                                                    *
;**********************************************************************
;
;----------------------------------------------------------------------
;File    : HardFaultHandler.asm
;Purpose : HardFault exception handler for TI assembler.
;          Evaluates used stack (MSP, PSP) and passes appropiate stack
;          pointer to the HardFaultHandler "C"-routine.
;------------- END-OF-HEADER ------------------------------------------
;*/

;/*********************************************************************
;*
;*     Forward declarations of segments used
;*
;**********************************************************************
;*/

;/*********************************************************************
;*
;*     Publics
;*
;**********************************************************************
;*/
        .global HardFault_Handler

;/*********************************************************************
;*
;*     Externals, code
;*
;**********************************************************************
;*/

        .ref HardFaultHandler

;/*********************************************************************
;*
;*     CODE segment
;*
;**********************************************************************
;*/

        .sect   ".text"
        .thumb
        .align  4
        
||$C$HardFaultHandler||:              .bits   HardFaultHandler,32

;/*********************************************************************
;*
;*       Global functions
;*
;**********************************************************************
;*/

;/*********************************************************************
;*
;*      HardFault_Handler()
;*
;*  Function description
;*    Evaluates the used stack (MSP, PSP) and passes the appropiate
;*    stack pointer to the HardFaultHandler "C"-routine.
;*
;*  Notes
;*    (1) Ensure that HardFault_Handler is part of the exception table
;*/
HardFault_Handler:
  .if $$defined  (__TI_ARM_V6M0__)
        ;// This version is for Cortex M0
        movs   R0, #4
        mov    R1, LR
        tst    R0, R1            ;// Check EXC_RETURN in Link register bit 2.
        bne    Uses_PSP
        mrs    R0, MSP           ;// Stacking was using MSP.
        b      Pass_StackPtr
Uses_PSP:
        mrs    R0, PSP           ;// Stacking was using PSP.
Pass_StackPtr:
        ldr    R2, $C$HardFaultHandler
        bx     R2                ;// Stack pointer passed through R0.

  .else
        ;// This version is for Cortex M3, Cortex M4 and Cortex M4F
        tst    LR, #4            ;// Check EXC_RETURN in Link register bit 2.
        ite    EQ
        mrseq  R0, MSP           ;// Stacking was using MSP.
        mrsne  R0, PSP           ;// Stacking was using PSP.
        b      HardFaultHandler  ;// Stack pointer passed through R0.
  .endif

; /********************************************************************/

        .end

;/****** End Of File *************************************************/
