/*********************************************************************
*                SEGGER Microcontroller GmbH & Co. KG                *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2016 SEGGER Microcontroller GmbH & Co. KG         *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 4.24                                             *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : RTOSInit_TM4C129.c

Purpose : Initializes and handles the hardware for embOS as far
          as required by embOS
          Feel free to modify this file acc. to your target system.
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "RTOS.h"
#include "SEGGER_SYSVIEW.h"

/*********************************************************************
*
*       Configuration
*
**********************************************************************
*/

/*********************************************************************
*
*       Clock frequency settings (configuration)
*/
#ifndef   OS_FSYS                   /* CPU Main clock frequency     */
  #define OS_FSYS 120000000u
#endif

#ifndef   OS_PCLK_TIMER             /* Peripheral clock for timer   */
  #define OS_PCLK_TIMER (OS_FSYS)   /* May vary from CPU clock      */
#endif                              /* depending on CPU             */

#ifndef   OS_PCLK_UART              /* Peripheral clock for UART    */
  #define OS_PCLK_UART (OS_FSYS)    /* May vary from CPU clock      */
#endif                              /* depending on CPU             */

#ifndef   OS_TICK_FREQ
  #define OS_TICK_FREQ (1000u)
#endif

/*********************************************************************
*
*       Configuration of communication to embOSView
*/
#ifndef   OS_VIEW_ENABLE            // Global enable of communication
  #define OS_VIEW_ENABLE    (1)     // Default: on
#endif

#ifndef   OS_VIEW_USE_UART          // If set, UART will be used for communication
  #define OS_VIEW_USE_UART  (0)     // Default: 0 => memory access is used
#endif                              // if OS_VIEW_ENABLE is on

/*********************************************************************
*
*       Check configuration
*
**********************************************************************
*/
#ifndef   DEBUG     /* Should normally be defined as project option */
  #define DEBUG  (0)
#endif

/*********************************************************************
*
*       Configuration of systemView
*/
#ifndef   SEGGER_SYSVIEW_SINGLE_SHOT     /* Should normally be defined as project option */
  #define SEGGER_SYSVIEW_SINGLE_SHOT  (1)
#endif

/****** End of configurable options *********************************/
/*********************************************************************
*
*       JLINKMEM and UART settings for embOSView
*
*       Automatically generated from configuration settings
*/
#define OS_USE_JLINKMEM   (OS_VIEW_ENABLE && (OS_VIEW_USE_UART == 0))

#define OS_UART_USED      (OS_VIEW_ENABLE && (OS_VIEW_USE_UART != 0) && (0))

#if OS_USE_JLINKMEM
  #include "JLINKMEM.h"
#endif

/****** MAP UART initialization function ****************************/

#if  (OS_UART_USED || OS_USE_JLINKMEM)
  #define OS_COM_INIT() OS_COM_Init()
#else
  #define OS_COM_INIT()
#endif

/*********************************************************************
*
*       Vector table
*/
#if (defined __SES_ARM)           // SEGGER Embedded Studio
  extern int _vectors;
  #define __Vectors    _vectors
#elif (defined __CROSSWORKS_ARM)  // Rowley CrossStudio
  extern int _vectors;
  #define __Vectors    _vectors
#elif (defined __ICCARM__)        // IAR
  #define __Vectors    __vector_table
#elif (defined __TI_ARM__)        // TI CodeComposer
  extern unsigned char g_pfnVectors;
  #define __Vectors    g_pfnVectors
#elif (defined __GNUC__)          // GCC
  extern unsigned char __Vectors;
#elif (defined __CC_ARM)          // KEIL
  extern unsigned char __Vectors;
#endif

/*********************************************************************
*
*       Local defines (sfrs and addresses used in RTOSInit.c)
*
**********************************************************************
*/
#define NUM_INTERRUPTS                 154

#define NVIC_VTOR                      (*(volatile OS_U32*) (0xE000ED08u))

#define SYS_INT_CTRL_STATE             (*(volatile OS_U32*)(0xE000ED04u))
#define SYS_PENDSTSET                  26u

#define SYSPRI1_ADDR                   (0xE000ED18u)
#define SYSHND_CTRL_ADDR               (0xE000ED24u)  // System Handler Control and State
#define SYSHND_CTRL                    (*(volatile OS_U32*) (SYSHND_CTRL_ADDR))

#define NVIC_SYS_HND_CTRL_MEM          (0x00010000u)  // Mem manage fault enable
#define NVIC_SYS_HND_CTRL_BUS          (0x00020000u)  // Bus fault enable
#define NVIC_SYS_HND_CTRL_USAGE        (0x00040000u)  // Usage fault enable

#define NVIC_PRIOBASE_ADDR             (0xE000E400u)
#define NVIC_ENABLE_ADDR               (0xE000E100u)
#define NVIC_DISABLE_ADDR              (0xE000E180u)
#define NVIC_VTOREG_ADDR               (0xE000ED08u)

#define SYS_TICK_BASE_ADDR             (0xE000E010u)
#define SYS_TICK_CONTROL               (*(volatile OS_U32*)(SYS_TICK_BASE_ADDR + 0x00))
#define SYS_TICK_RELOAD                (*(volatile OS_U32*)(SYS_TICK_BASE_ADDR + 0x04))
#define SYS_TICK_VALUE                 (*(volatile OS_U32*)(SYS_TICK_BASE_ADDR + 0x08))
#define SYS_TICK_CALIBRATION           (*(volatile OS_U32*)(SYS_TICK_BASE_ADDR + 0x0C))
#define SYS_TICK_ENABLE_BIT            (0u)
#define SYS_TICK_INT_ENABLE_BIT        (1u)
#define SYS_TICK_CLK_SOURCE_BIT        (2u)

#define ISR_ID_MPU                     (4u)                // MPU fault
#define ISR_ID_BUS                     (5u)                // Bus fault
#define ISR_ID_USAGE                   (6u)                // Usage fault
#define ISR_ID_SYSTICK                 (15u)               // System Tick
#define OS_ISR_ID_TICK                 ISR_ID_SYSTICK      // We use Sys-Timer

// PLL defines
#define SYSCTL_MOSCCTL                 (*(volatile unsigned int*)(0x400FE07Cu))
#define SYSCTL_MEMTIM0                 (*(volatile unsigned int*)(0x400FE0C0u))
#define SYSCTL_RSCLKCFG                (*(volatile unsigned int*)(0x400FE0B0u))
#define SYSCTL_PLLFREQ0                (*(volatile unsigned int*)(0x400FE160u))
#define SYSCTL_PLLFREQ1                (*(volatile unsigned int*)(0x400FE164u))
#define SYSCTL_PLLSTAT                 (*(volatile unsigned int*)(0x400FE168u))
#define SysCtlXtalCfgToIndex(a)        ((a & 0x7c0) >> 6)
#define PLL_M_TO_REG(mi, mf)           ((unsigned int)mi | (unsigned int)(mf << SYSCTL_PLLFREQ0_MFRAC_S))
#define PLL_N_TO_REG(n)                ((unsigned int)(n - 1) << SYSCTL_PLLFREQ1_N_S)
#define MAX_VCO_ENTRIES                2
#define MAX_XTAL_ENTRIES               18
#define SYSCTL_MEMTIM0_EBCHT_M         0x03C00000uL  // EEPROM Clock High Time
#define SYSCTL_MEMTIM0_EBCHT_0_5       0x00000000uL  // System clock high
#define SYSCTL_MEMTIM0_EBCHT_1         0x00400000uL  // 1 system clock period
#define SYSCTL_MEMTIM0_EBCHT_1_5       0x00800000uL  // 1.5 system clock periods
#define SYSCTL_MEMTIM0_EBCHT_2         0x00C00000uL  // 2 system clock periods
#define SYSCTL_MEMTIM0_EBCHT_2_5       0x01000000uL  // 2.5 system clock periods
#define SYSCTL_MEMTIM0_EBCHT_3         0x01400000uL  // 3 system clock periods
#define SYSCTL_MEMTIM0_EBCHT_3_5       0x01800000uL  // 3.5 system clock periods
#define SYSCTL_MEMTIM0_EBCHT_4         0x01C00000uL  // 4 system clock periods
#define SYSCTL_MEMTIM0_EBCHT_4_5       0x02000000uL  // 4.5 system clock periods
#define SYSCTL_MEMTIM0_EBCE            0x00200000uL  // EEPROM Bank Clock Edge
#define SYSCTL_MEMTIM0_MB1             0x00100010uL  // Must be one
#define SYSCTL_MEMTIM0_EWS_M           0x000F0000uL  // EEPROM Wait State
#define SYSCTL_MEMTIM0_FBCHT_M         0x000003C0uL  // Flash Bank Clock High Time
#define SYSCTL_MEMTIM0_FBCHT_0_5       0x00000000uL  // System clock high
#define SYSCTL_MEMTIM0_FBCHT_1         0x00000040uL  // 1 system clock period
#define SYSCTL_MEMTIM0_FBCHT_1_5       0x00000080uL  // 1.5 system clock periods
#define SYSCTL_MEMTIM0_FBCHT_2         0x000000C0uL  // 2 system clock periods
#define SYSCTL_MEMTIM0_FBCHT_2_5       0x00000100uL  // 2.5 system clock periods
#define SYSCTL_MEMTIM0_FBCHT_3         0x00000140uL  // 3 system clock periods
#define SYSCTL_MEMTIM0_FBCHT_3_5       0x00000180uL  // 3.5 system clock periods
#define SYSCTL_MEMTIM0_FBCHT_4         0x000001C0uL  // 4 system clock periods
#define SYSCTL_MEMTIM0_FBCHT_4_5       0x00000200uL  // 4.5 system clock periods
#define SYSCTL_MEMTIM0_FBCE            0x00000020uL  // Flash Bank Clock Edge
#define SYSCTL_MEMTIM0_FWS_M           0x0000000FuL  // Flash Wait State
#define SYSCTL_MEMTIM0_EWS_S           16
#define SYSCTL_MEMTIM0_FWS_S           0
#define SYSCTL_XTAL_10MHZ              0x00000400    // External crystal is 10 MHz
#define SYSCTL_XTAL_16MHZ              0x00000540    // External crystal is 16 MHz
#define SYSCTL_XTAL_25MHZ              0x00000680    // External crystal is 25.0 MHz
#define SYSCTL_XTAL_5MHZ               0x00000240    // External crystal is 5MHz
#define SYSCTL_USE_PLL                 0x00000000uL  // System clock is the PLL clock
#define SYSCTL_USE_OSC                 0x00003800uL  // System clock is the osc clock
#define SYSCTL_RSCLKCFG_USEPLL         0x10000000uL  // Use PLL
#define SYSCTL_RSCLKCFG_PSYSDIV_S      0
#define SYSCTL_RSCLKCFG_PSYSDIV_M      0x000003FFuL  // PLL System Clock Divisor
#define SYSCTL_RSCLKCFG_PLLSRC_PIOSC   0x00000000uL  // PIOSC is PLL input clock source
#define SYSCTL_RSCLKCFG_PLLSRC_MOSC    0x03000000uL  // MOSC is PLL input clock source
#define SYSCTL_RSCLKCFG_PLLSRC_M       0x0F000000uL  // PLL Source
#define SYSCTL_RSCLKCFG_OSYSDIV_S      10
#define SYSCTL_RSCLKCFG_OSYSDIV_M      0x000FFC00uL  // Oscillator System Clock Divisor
#define SYSCTL_RSCLKCFG_OSCSRC_M       0x00F00000uL  // Oscillator Source
#define SYSCTL_RSCLKCFG_OSCSRC_RTC     0x00400000uL  // RTC is oscillator source
#define SYSCTL_RSCLKCFG_OSCSRC_PIOSC   0x00000000uL  // PIOSC is oscillator source
#define SYSCTL_RSCLKCFG_OSCSRC_LFIOSC  0x00200000uL  // LFIOSC is oscillator source
#define SYSCTL_RSCLKCFG_OSCSRC_MOSC    0x00300000uL  // MOSC is oscillator source
#define SYSCTL_RSCLKCFG_OSCSRC_RTC     0x00400000uL  // RTC is oscillator source
#define SYSCTL_RSCLKCFG_MEMTIMU        0x80000000uL  // Memory Timing Register Update
#define SYSCTL_RSCLKCFG_NEWFREQ        0x40000000uL  // New PLLFREQ Accept
#define SYSCTL_RSCLKCFG_ACG            0x20000000uL  // Auto Clock Gating Enable
#define SYSCTL_RSCLKCFG_USEPLL         0x10000000uL  // Use PLL
#define SYSCTL_PLLSTAT_LOCK            0x00000001uL  // PLL Lock
#define SYSCTL_PLLFREQ0_PLLPWR         0x00800000uL  // PLL Power
#define SYSCTL_PLLFREQ0_MFRAC_M        0x000FFC00uL  // PLL M Fractional Value
#define SYSCTL_PLLFREQ0_MINT_M         0x000003FFuL  // PLL M Integer Value
#define SYSCTL_PLLFREQ0_MFRAC_S        10
#define SYSCTL_PLLFREQ0_MINT_S         0
#define SYSCTL_PLLFREQ1_Q_M            0x00001F00uL  // PLL Q Value
#define SYSCTL_PLLFREQ1_N_M            0x0000001FuL  // PLL N Value
#define SYSCTL_PLLFREQ1_Q_S            8
#define SYSCTL_PLLFREQ1_N_S            0
#define SYSCTL_CFG_VCO_480             0xF1000000uL  // VCO is 480 MHz
#define SYSCTL_MOSCCTL_NOXTAL          0x00000004uL  // No Crystal Connected
#define SYSCTL_MOSCCTL_OSCRNG          0x00000010uL  // Oscillator Range
#define SYSCTL_MOSCCTL_PWRDN           0x00000008uL  // Power Down
#define SYSCTL_OSC_INT30               0x00000030uL  // Osc source is int. 30 KHz
#define SYSCTL_OSC_INT                 0x00000010uL  // Osc source is int. osc
#define SYSCTL_OSC_EXT32               0x80000038uL  // Osc source is ext. 32 KHz
#define SYSCTL_OSC_MAIN                0x00000000uL  // Osc source is main osc

/****** OS timer configuration **************************************/
#define OS_TIMER_RELOAD           (OS_PCLK_TIMER / OS_TICK_FREQ - 1)
#if (OS_TIMER_RELOAD >= 0x100000000u)
  #error "Systick can not be used, please check configuration"
#endif

/*********************************************************************
*
*       Const data (for PLL init)
*
**********************************************************************
*/
static const unsigned int g_pppui32XTALtoVCO[MAX_VCO_ENTRIES][MAX_XTAL_ENTRIES][2] = {
  {
    //
    // VCO 320 MHz
    //
    { PLL_M_TO_REG(64, 0),   PLL_N_TO_REG(1) },     // 5 MHz
    { PLL_M_TO_REG(62, 512), PLL_N_TO_REG(1) },     // 5.12 MHz
    { PLL_M_TO_REG(160, 0),  PLL_N_TO_REG(3) },     // 6 MHz
    { PLL_M_TO_REG(52, 85),  PLL_N_TO_REG(1) },     // 6.144 MHz
    { PLL_M_TO_REG(43, 412), PLL_N_TO_REG(1) },     // 7.3728 MHz
    { PLL_M_TO_REG(40, 0),   PLL_N_TO_REG(1) },     // 8 MHz
    { PLL_M_TO_REG(39, 64),  PLL_N_TO_REG(1) },     // 8.192 MHz
    { PLL_M_TO_REG(32, 0),   PLL_N_TO_REG(1) },     // 10 MHz
    { PLL_M_TO_REG(80, 0),   PLL_N_TO_REG(3) },     // 12 MHz
    { PLL_M_TO_REG(26, 43),  PLL_N_TO_REG(1) },     // 12.288 MHz
    { PLL_M_TO_REG(23, 613), PLL_N_TO_REG(1) },     // 13.56 MHz
    { PLL_M_TO_REG(22, 358), PLL_N_TO_REG(1) },     // 14.318180 MHz
    { PLL_M_TO_REG(20, 0),   PLL_N_TO_REG(1) },     // 16 MHz
    { PLL_M_TO_REG(19, 544), PLL_N_TO_REG(1) },     // 16.384 MHz
    { PLL_M_TO_REG(160, 0),  PLL_N_TO_REG(9) },     // 18 MHz
    { PLL_M_TO_REG(16, 0),   PLL_N_TO_REG(1) },     // 20 MHz
    { PLL_M_TO_REG(40, 0),   PLL_N_TO_REG(3) },     // 24 MHz
    { PLL_M_TO_REG(64, 0),   PLL_N_TO_REG(5) },     // 25 MHz
  },
  {
    //
    // VCO 480 MHz
    //
    { PLL_M_TO_REG(96, 0),   PLL_N_TO_REG(1) },     // 5 MHz
    { PLL_M_TO_REG(93, 768), PLL_N_TO_REG(1) },     // 5.12 MHz
    { PLL_M_TO_REG(80, 0),   PLL_N_TO_REG(1) },     // 6 MHz
    { PLL_M_TO_REG(78, 128), PLL_N_TO_REG(1) },     // 6.144 MHz
    { PLL_M_TO_REG(65, 107), PLL_N_TO_REG(1) },     // 7.3728 MHz
    { PLL_M_TO_REG(60, 0),   PLL_N_TO_REG(1) },     // 8 MHz
    { PLL_M_TO_REG(58, 608), PLL_N_TO_REG(1) },     // 8.192 MHz
    { PLL_M_TO_REG(48, 0),   PLL_N_TO_REG(1) },     // 10 MHz
    { PLL_M_TO_REG(40, 0),   PLL_N_TO_REG(1) },     // 12 MHz
    { PLL_M_TO_REG(39, 64),  PLL_N_TO_REG(1) },     // 12.288 MHz
    { PLL_M_TO_REG(35, 408), PLL_N_TO_REG(1) },     // 13.56 MHz
    { PLL_M_TO_REG(33, 536), PLL_N_TO_REG(1) },     // 14.318180 MHz
    { PLL_M_TO_REG(30, 0),   PLL_N_TO_REG(1) },     // 16 MHz
    { PLL_M_TO_REG(29, 304), PLL_N_TO_REG(1) },     // 16.384 MHz
    { PLL_M_TO_REG(80, 0),   PLL_N_TO_REG(3) },     // 18 MHz
    { PLL_M_TO_REG(24, 0),   PLL_N_TO_REG(1) },     // 20 MHz
    { PLL_M_TO_REG(20, 0),   PLL_N_TO_REG(1) },     // 24 MHz
    { PLL_M_TO_REG(96, 0),   PLL_N_TO_REG(5) },     // 25 MHz
  },
};

static const unsigned int g_pui32Xtals[] = {
  1000000,
  1843200,
  2000000,
  2457600,
  3579545,
  3686400,
  4000000,
  4096000,
  4915200,
  5000000,
  5120000,
  6000000,
  6144000,
  7372800,
  8000000,
  8192000,
  10000000,
  12000000,
  12288000,
  13560000,
  14318180,
  16000000,
  16384000,
  18000000,
  20000000,
  24000000,
  25000000
};

static const unsigned int g_pui32VCOFrequencies[MAX_VCO_ENTRIES] = {
  320000000,                              // VCO 320
  480000000,                              // VCO 480
};

static const struct {
  unsigned int ui32Frequency;
  unsigned int ui32MemTiming;
}
g_sXTALtoMEMTIM[] = {
  { 16000000, (SYSCTL_MEMTIM0_FBCHT_0_5 | SYSCTL_MEMTIM0_FBCE |
               (0 << SYSCTL_MEMTIM0_FWS_S) |
               SYSCTL_MEMTIM0_EBCHT_0_5 | SYSCTL_MEMTIM0_EBCE |
               (0 << SYSCTL_MEMTIM0_EWS_S) |
               SYSCTL_MEMTIM0_MB1) },
  { 30000000, (SYSCTL_MEMTIM0_FBCHT_1 | SYSCTL_MEMTIM0_FBCE |
               (1 << SYSCTL_MEMTIM0_FWS_S) |
               SYSCTL_MEMTIM0_EBCHT_1 | SYSCTL_MEMTIM0_EBCE |
               (1 << SYSCTL_MEMTIM0_EWS_S) |
               SYSCTL_MEMTIM0_MB1) },
  { 50000000, (SYSCTL_MEMTIM0_FBCHT_1_5 | SYSCTL_MEMTIM0_FBCE |
               (2 << SYSCTL_MEMTIM0_FWS_S) |
               SYSCTL_MEMTIM0_EBCHT_1_5 | SYSCTL_MEMTIM0_EBCE |
               (2 << SYSCTL_MEMTIM0_EWS_S) |
               SYSCTL_MEMTIM0_MB1) },
  { 60000000, (SYSCTL_MEMTIM0_FBCHT_2 | (3 << SYSCTL_MEMTIM0_FWS_S) |
               SYSCTL_MEMTIM0_EBCHT_2 | (3 << SYSCTL_MEMTIM0_EWS_S) |
               SYSCTL_MEMTIM0_MB1) },
  { 80000000, (SYSCTL_MEMTIM0_FBCHT_2_5 | (4 << SYSCTL_MEMTIM0_FWS_S) |
               SYSCTL_MEMTIM0_EBCHT_2_5 | (4 << SYSCTL_MEMTIM0_EWS_S) |
               SYSCTL_MEMTIM0_MB1) },
  { 100000000, (SYSCTL_MEMTIM0_FBCHT_3 | (5 << SYSCTL_MEMTIM0_FWS_S) |
                SYSCTL_MEMTIM0_EBCHT_3 | (5 << SYSCTL_MEMTIM0_EWS_S) |
                SYSCTL_MEMTIM0_MB1) },
  { 120000000, (SYSCTL_MEMTIM0_FBCHT_3_5 | (6 << SYSCTL_MEMTIM0_FWS_S) |
                SYSCTL_MEMTIM0_EBCHT_3_5 | (6 << SYSCTL_MEMTIM0_EWS_S) |
                SYSCTL_MEMTIM0_MB1) },
};

/*********************************************************************
*
*       Static data
*
**********************************************************************
*/

#if OS_USE_JLINKMEM
  // Size of the communication buffer for JLINKMEM
const OS_U32 OS_JLINKMEM_BufferSize = 32u;
#else
const OS_U32 OS_JLINKMEM_BufferSize = 0;     // Communication not used
#endif

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

/*********************************************************************
*
*       _OS_GetHWTimerCycles()
*
* Function description
*   Returns the current hardware timer count value
*
* Return value
*   Current timer count value
*/
static unsigned int _OS_GetHWTimerCycles(void) {
  return SYS_TICK_VALUE;
}

/*********************************************************************
*
*       _OS_GetHWTimer_IntPending()
*
* Function description
*   Returns if the hardware timer interrupt pending flag is set
*
* Return value
*   == 0; Interrupt pending flag not set
*   != 0: Interrupt pending flag set
*/
static unsigned int _OS_GetHWTimer_IntPending(void) {
  return SYS_INT_CTRL_STATE & (1uL << SYS_PENDSTSET);
}

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_GetPClkTimer()
*
* Function description
*   Helper function for Percepio embOS-Trace
*/
OS_U32 OS_GetPClkTimer(void) {
  return OS_PCLK_TIMER;
}

/*********************************************************************
*
*       OS_GetTickFreq()
*
* Function description
*   Helper function for Percepio embOS-Trace
*/
OS_U32 OS_GetTickFreq(void) {
  return OS_TICK_FREQ;
}

/*********************************************************************
*
*       SysTick_Handler()
*
* Function description
*   This is the code that gets called when the processor receives a
*   _SysTick exception. SysTick is used as OS timer tick.
*/
#ifdef __cplusplus
extern "C" {
#endif
void SysTick_Handler(void);      // Avoid warning, Systick_Handler is not prototyped in any CMSIS header
#ifdef __cplusplus
}
#endif
void SysTick_Handler(void) {
  OS_EnterNestableInterrupt();
  OS_TICK_Handle();
  #if OS_USE_JLINKMEM
    JLINKMEM_Process();
  #endif
  OS_LeaveNestableInterrupt();
}

/*********************************************************************
*
*       _SysCtlFrequencyGet()
*
*       Returns PLL frequency
*/
static unsigned int _SysCtlFrequencyGet(unsigned int ui32Xtal) {
  unsigned int ui32Result;
  unsigned int ui16F1, ui16F2;
  unsigned int ui16PInt, ui16PFract;
  unsigned int ui8Q, ui8N;

  //
  // Extract all of the values from the hardware registers.
  //
  ui16PFract = ((SYSCTL_PLLFREQ0 & SYSCTL_PLLFREQ0_MFRAC_M) >>
                SYSCTL_PLLFREQ0_MFRAC_S);
  ui16PInt = SYSCTL_PLLFREQ0 & SYSCTL_PLLFREQ0_MINT_M;
  ui8Q = (((SYSCTL_PLLFREQ1 & SYSCTL_PLLFREQ1_Q_M) >>
           SYSCTL_PLLFREQ1_Q_S) + 1);
  ui8N = (((SYSCTL_PLLFREQ1 & SYSCTL_PLLFREQ1_N_M) >>
           SYSCTL_PLLFREQ1_N_S) + 1);

  //
  // Divide the crystal value by N.
  //
  ui32Xtal /= (unsigned int)ui8N;

  //
  // Calculate the multiplier for bits 9:5.
  //
  ui16F1 = ui16PFract / 32;

  //
  // Calculate the multiplier for bits 4:0.
  //
  ui16F2 = ui16PFract - (ui16F1 * 32);

  //
  // Get the integer portion.
  //
  ui32Result = ui32Xtal * (unsigned int)ui16PInt;

  //
  // Add first fractional bits portion(9:0).
  //
  ui32Result += (ui32Xtal * (unsigned int)ui16F1) / 32;

  //
  // Add the second fractional bits portion(4:0).
  //
  ui32Result += (ui32Xtal * (unsigned int)ui16F2) / 1024;

  //
  // Divide the result by Q.
  //
  ui32Result = ui32Result / (unsigned int)ui8Q;

  //
  // Return the resulting PLL frequency.
  //
  return(ui32Result);
}


/*********************************************************************
*
*       _SysCtlMemTimingGet()
*
*       Returns an appropriate flash memory timing
*/
static unsigned int _SysCtlMemTimingGet(unsigned int ui32SysClock) {
  unsigned int ui8Idx;

  //
  // Loop through the flash memory timings.
  //
  for(ui8Idx = 0;
      ui8Idx < (sizeof(g_sXTALtoMEMTIM) / sizeof(g_sXTALtoMEMTIM[0]));
      ui8Idx++) {
      //
      // See if the system clock frequency is less than the maximum frequency
      // for this flash memory timing.
      //
      if(ui32SysClock <= g_sXTALtoMEMTIM[ui8Idx].ui32Frequency) {
          //
          // This flash memory timing is the best choice for the system clock
          // frequency, so return it now.
          //
          return(g_sXTALtoMEMTIM[ui8Idx].ui32MemTiming);
      }
  }

  //
  // An appropriate flash memory timing could not be found, so the device is
  // being clocked too fast.  Return the default flash memory timing.
  //
  return (0);
}

/*********************************************************************
*
*       _SysCtlClockFreqSet()
*
*       Initializes the PLL
*/
static void _SysCtlClockFreqSet(unsigned int ui32Config, unsigned int ui32SysClock) {
  int          i32Timeout, i32VCOIdx, i32XtalIdx;
  unsigned int ui32MOSCCTL;
  unsigned int ui32SysDiv, ui32Osc, ui32OscSelect, ui32RSClkConfig;
  char         bNewPLL;

  //
  // Get the index of the crystal from the ui32Config parameter.
  //
  i32XtalIdx = SysCtlXtalCfgToIndex(ui32Config);

  //
  // Determine which non-PLL source was selected.
  //
  if((ui32Config & 0x38) == SYSCTL_OSC_INT) {
    //
    // Use the nominal frequency for the PIOSC oscillator and set the
    // crystal select.
    //
    ui32Osc = 16000000;
    ui32OscSelect = SYSCTL_RSCLKCFG_OSCSRC_PIOSC;
    ui32OscSelect |= SYSCTL_RSCLKCFG_PLLSRC_PIOSC;

    //
    // Force the crystal index to the value for 16-MHz.
    //
    i32XtalIdx = SysCtlXtalCfgToIndex(SYSCTL_XTAL_16MHZ);
  } else if((ui32Config & 0x38) == SYSCTL_OSC_INT30) {
    //
    // Use the nominal frequency for the low frequency oscillator.
    //
    ui32Osc = 30000;
    ui32OscSelect = SYSCTL_RSCLKCFG_OSCSRC_LFIOSC;
  }
  else if((ui32Config & 0x38) == (SYSCTL_OSC_EXT32 & 0x38)) {
    //
    // Use the RTC frequency.
    //
    ui32Osc = 32768;
    ui32OscSelect = SYSCTL_RSCLKCFG_OSCSRC_RTC;
  } else if((ui32Config & 0x38) == SYSCTL_OSC_MAIN) {
    //
    // Bounds check the source frequency for the main oscillator.  The is
    // because the PLL tables in the g_pppui32XTALtoVCO structure range
    // from 5MHz to 25MHz.
    //
    if ((i32XtalIdx > (SysCtlXtalCfgToIndex(SYSCTL_XTAL_25MHZ))) ||
       (i32XtalIdx < (SysCtlXtalCfgToIndex(SYSCTL_XTAL_5MHZ)))) {
      return;
    }

    ui32Osc = g_pui32Xtals[i32XtalIdx];

    //
    // Set the PLL source select to MOSC.
    //
    ui32OscSelect = SYSCTL_RSCLKCFG_OSCSRC_MOSC;
    ui32OscSelect |= SYSCTL_RSCLKCFG_PLLSRC_MOSC;

    //
    // Clear MOSC power down, high oscillator range setting, and no crystal
    // present setting.
    //
    ui32MOSCCTL = SYSCTL_MOSCCTL &
                  ~(SYSCTL_MOSCCTL_OSCRNG | SYSCTL_MOSCCTL_PWRDN |
                    SYSCTL_MOSCCTL_NOXTAL);

    //
    // Increase the drive strength for MOSC of 10 MHz and above.
    //
    if (i32XtalIdx >= (SysCtlXtalCfgToIndex(SYSCTL_XTAL_10MHZ) -
                      (SysCtlXtalCfgToIndex(SYSCTL_XTAL_5MHZ)))) {
        ui32MOSCCTL |= SYSCTL_MOSCCTL_OSCRNG;
    }

    SYSCTL_MOSCCTL = ui32MOSCCTL;
  } else {
    //
    // This was an invalid request because no oscillator source was
    // indicated.
    //
    ui32Osc = 0;
    ui32OscSelect = SYSCTL_RSCLKCFG_OSCSRC_PIOSC;
  }

  //
  // Check if the running with the PLL enabled was requested.
  //
  if((ui32Config & SYSCTL_USE_OSC) == SYSCTL_USE_PLL) {
    //
    // ui32Config must be SYSCTL_OSC_MAIN or SYSCTL_OSC_INT.
    //
    if(((ui32Config & 0x38) != SYSCTL_OSC_MAIN) &&
       ((ui32Config & 0x38) != SYSCTL_OSC_INT)) {
        return;
    }

    //
    // Get the VCO index out of the ui32Config parameter.
    //
    i32VCOIdx = (ui32Config >> 24) & 7;

    //
    // Set the memory timings for the PIOSC since we are about to switch
    // back to PIOSC.
    //
    SYSCTL_MEMTIM0 = _SysCtlMemTimingGet(16000000);

    //
    // Clear the old PLL divider and source in case it was set.
    //
    ui32RSClkConfig = SYSCTL_RSCLKCFG &
                      ~(SYSCTL_RSCLKCFG_PSYSDIV_M |
                        SYSCTL_RSCLKCFG_OSCSRC_M |
                        SYSCTL_RSCLKCFG_PLLSRC_M | SYSCTL_RSCLKCFG_USEPLL);

    //
    // Update the memory timings to match running from PIOSC.
    //
    ui32RSClkConfig |= SYSCTL_RSCLKCFG_MEMTIMU;

    //
    // Update clock configuration to switch back to PIOSC.
    //
    SYSCTL_RSCLKCFG = ui32RSClkConfig;

    //
    // The table starts at 5 MHz so modify the index to match this.
    //
    i32XtalIdx -= SysCtlXtalCfgToIndex(SYSCTL_XTAL_5MHZ);

    //
    // If there were no changes to the PLL do not force the PLL to lock by
    // writing the PLL settings.
    //
    if((SYSCTL_PLLFREQ1 !=
      g_pppui32XTALtoVCO[i32VCOIdx][i32XtalIdx][1]) ||
     (SYSCTL_PLLFREQ0 !=
      (g_pppui32XTALtoVCO[i32VCOIdx][i32XtalIdx][0] |
       SYSCTL_PLLFREQ0_PLLPWR))) {
      bNewPLL = 1;
    } else {
      bNewPLL = 0;
    }

    //
    // If there are new PLL settings write them.
    //
    if(bNewPLL) {
      //
      // Set the oscillator source.
      //
      SYSCTL_RSCLKCFG |= ui32OscSelect;

      //
      // Set the M, N and Q values provided from the table and preserve
      // the power state of the main PLL.
      //
      SYSCTL_PLLFREQ1 = g_pppui32XTALtoVCO[i32VCOIdx][i32XtalIdx][1];
      SYSCTL_PLLFREQ0 = (g_pppui32XTALtoVCO[i32VCOIdx][i32XtalIdx][0] |(SYSCTL_PLLFREQ0 & SYSCTL_PLLFREQ0_PLLPWR));
    }

    //
    // Calculate the System divider such that we get a frequency that is
    // the closest to the requested frequency without going over.
    //
    ui32SysDiv = (g_pui32VCOFrequencies[i32VCOIdx] + ui32SysClock - 1) / ui32SysClock;
    //
    // Calculate the actual system clock.
    //
    ui32SysClock = _SysCtlFrequencyGet(ui32Osc) / ui32SysDiv;

    //
    // Set the Flash and EEPROM timing values.
    //
    SYSCTL_MEMTIM0 = _SysCtlMemTimingGet(ui32SysClock);

    //
    // Check if the PLL is already powered up.
    //
    if(SYSCTL_PLLFREQ0 & SYSCTL_PLLFREQ0_PLLPWR) {
      if(bNewPLL == 1) {
        //
        // Trigger the PLL to lock to the new frequency.
        //
        SYSCTL_RSCLKCFG |= SYSCTL_RSCLKCFG_NEWFREQ;
      }
    } else {
      //
      // Power up the PLL.
      //
      SYSCTL_PLLFREQ0 |= SYSCTL_PLLFREQ0_PLLPWR;
    }

    //
    // Wait until the PLL has locked.
    //
    for(i32Timeout = 32768; i32Timeout > 0; i32Timeout--) {
      if(SYSCTL_PLLSTAT & SYSCTL_PLLSTAT_LOCK) {
        break;
      }
    }

    //
    // If the loop above did not timeout then switch over to the PLL
    //
    if(i32Timeout) {
      ui32RSClkConfig = SYSCTL_RSCLKCFG;
      ui32RSClkConfig |= ((ui32SysDiv - 1) <<
                          SYSCTL_RSCLKCFG_PSYSDIV_S) | ui32OscSelect |
                         SYSCTL_RSCLKCFG_USEPLL;
      ui32RSClkConfig |= SYSCTL_RSCLKCFG_MEMTIMU;

      //
      // Set the new clock configuration.
      //
      SYSCTL_RSCLKCFG = ui32RSClkConfig;
    } else {
      ui32SysClock = 0;
    }
  } else {
    //
    // Set the Flash and EEPROM timing values for PIOSC.
    //
    SYSCTL_MEMTIM0 = _SysCtlMemTimingGet(16000000);

    //
    // Make sure that the PLL is powered down since it is not being used.
    //
    SYSCTL_PLLFREQ0 &= ~SYSCTL_PLLFREQ0_PLLPWR;

    //
    // Clear the old PLL divider and source in case it was set.
    //
    ui32RSClkConfig = SYSCTL_RSCLKCFG;
    ui32RSClkConfig &= ~(SYSCTL_RSCLKCFG_OSYSDIV_M |
                         SYSCTL_RSCLKCFG_OSCSRC_M |
                         SYSCTL_RSCLKCFG_USEPLL);

    //
    // Update the memory timings.
    //
    ui32RSClkConfig |= SYSCTL_RSCLKCFG_MEMTIMU;

    //
    // Set the new clock configuration.
    //
    SYSCTL_RSCLKCFG = ui32RSClkConfig;

    //
    // If zero given as the system clock then default to divide by 1.
    //
    if(ui32SysClock == 0) {
      ui32SysDiv = 0;
    } else {
      //
      // Calculate the System divider based on the requested
      // frequency.
      //
      ui32SysDiv = ui32Osc / ui32SysClock;

      //
      // If the system divisor is not already zero, subtract one to
      // set the value in the register which requires the value to
      // be n-1.
      //
      if(ui32SysDiv != 0) {
        ui32SysDiv -= 1;
      }

      //
      // Calculate the system clock.
      //
      ui32SysClock = ui32Osc / (ui32SysDiv + 1);
    }

    //
    // Set the memory timing values for the new system clock.
    //
    SYSCTL_MEMTIM0 = _SysCtlMemTimingGet(ui32SysClock);

    //
    // Set the new system clock values.
    //
    ui32RSClkConfig = SYSCTL_RSCLKCFG;
    ui32RSClkConfig |= (ui32SysDiv << SYSCTL_RSCLKCFG_OSYSDIV_S) |
                       ui32OscSelect;

    //
    // Update the memory timings.
    //
    ui32RSClkConfig |= SYSCTL_RSCLKCFG_MEMTIMU;

    //
    // Set the new clock configuration.
    //
    SYSCTL_RSCLKCFG = ui32RSClkConfig;
  }
}

/*********************************************************************
*
*       OS_InitHW()
*
*       Initialize the hardware (timer) required for the OS to run.
*       May be modified, if an other timer should be used
*/
void OS_InitHW(void) {
  OS_U8  TickPrio;

  //
  // Structure with information about timer frequency, tick frequency, etc.
  // for micro second precise system time.
  //
  OS_SYSTIMER_CONFIG SysTimerConfig = {OS_PCLK_TIMER, OS_TICK_FREQ, 0, _OS_GetHWTimerCycles, _OS_GetHWTimer_IntPending};

  OS_IncDI();
  //
  // Run from the PLL at 120 MHz
  //
  _SysCtlClockFreqSet((SYSCTL_XTAL_25MHZ | SYSCTL_OSC_MAIN | SYSCTL_USE_PLL | SYSCTL_CFG_VCO_480), OS_FSYS);
  //
  // Initialize OS timer, clock source = core clock
  //
  SYS_TICK_RELOAD  = OS_TIMER_RELOAD;
  SYS_TICK_CONTROL = (1uL << SYS_TICK_ENABLE_BIT) | (1uL << SYS_TICK_CLK_SOURCE_BIT);
  //
  // Initialize NVIC vector base address. Might be necessary for RAM targets or application not running from 0
  //
  NVIC_VTOR = (OS_U32)&__Vectors;
  //
  // Setup values for usec precise system time functions
  //
  OS_Config_SysTimer(&SysTimerConfig);
  //
  // Set the interrupt priority for the system timer to 2nd lowest level to ensure the timer can preempt PendSV handler
  //
  OS_ARM_ISRSetPrio(OS_ISR_ID_TICK, 0xFFu);              // Set to lowest level, ALL BITS set
  TickPrio  = OS_ARM_ISRSetPrio(OS_ISR_ID_TICK, 0xFFu);  // Read priority back to examine relevant preemption-level-bits
  TickPrio -= 1;                                         // Set to low preemption level, 1 level higher than lowest
  OS_ARM_ISRSetPrio(OS_ISR_ID_TICK, TickPrio);
  OS_ARM_EnableISR(OS_ISR_ID_TICK);
  //
  // Configure and initialize SEGGER SystemView
  //
#if OS_PROFILE
  SEGGER_SYSVIEW_Conf();
  //
  // The following call is required when using SEGGER SystemView
  // in single-shot mode, e.g. when using ST-Link instead of J-Link
  //
#if SEGGER_SYSVIEW_SINGLE_SHOT
  SEGGER_SYSVIEW_Start();
#endif
#endif

  OS_COM_INIT();
  OS_DecRI();
}

/*********************************************************************
*
*       OS_Idle()
*
*       Please note:
*       This is basically the "core" of the idle loop.
*       This core loop can be changed, but:
*       The idle loop does not have a stack of its own, therefore no
*       functionality should be implemented that relies on the stack
*       to be preserved. However, a simple program loop can be programmed
*       (like toggling an output or incrementing a counter)
*/
void OS_Idle(void) {     // Idle loop: No task is ready to execute
  while (1) {
    #if ((OS_USE_JLINKMEM == 0) && (OS_DEBUG == 0))     // Enter CPU halt mode when not in DEBUG build and J-Link communication not used
      #ifdef __ICCARM__  // IAR
        __asm(" wfi");
      #endif
      #ifdef __CC_ARM    // KEIL
        __wfi();
      #endif
      #ifdef __GNUC__    // GCC
        __asm(" wfi");
      #endif
    #endif
  }
}

/*********************************************************************
*
*       OS_GetTime_Cycles()
*
*       This routine is required for task-info via embOSView or high
*       resolution time measurement functions.
*       It returns the system time in timer clock cycles.
*/
OS_U32 OS_GetTime_Cycles(void) {
  OS_U32 Time;
  OS_U32 Cnt;

  Time = OS_GetTime32();
  Cnt  = (OS_TIMER_RELOAD) - SYS_TICK_VALUE;
  //
  // Check if timer interrupt pending ...
  //
  if (SYS_INT_CTRL_STATE & (1uL << SYS_PENDSTSET)) {
    Cnt = OS_TIMER_RELOAD - SYS_TICK_VALUE;          // Interrupt pending, re-read timer and adjust result
    Time++;
  }
  return (OS_TIMER_RELOAD * Time) + Cnt;
}

/*********************************************************************
*
*       OS_ConvertCycles2us()
*
*       Convert Cycles into micro seconds.
*
*       If your clock frequency is not a multiple of 1 MHz,
*       you may have to modify this routine in order to get proper
*       diagnostics.
*
*       This routine is required for profiling or high resolution time
*       measurement only. It does not affect operation of the OS.
*/
OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  return Cycles/(OS_PCLK_TIMER/1000000u);
}

/*********************************************************************
*
*       Optional communication with embOSView
*
**********************************************************************
*/

#if OS_USE_JLINKMEM                    // Communication via JTAG / SWD

static void _JLINKMEM_OnRx(OS_U8 Data);
static void _JLINKMEM_OnTx(void);
static OS_INT _JLINKMEM_GetNextChar(void);

/*********************************************************************
*
*       _JLINKMEM_OnRx()
*/
static void _JLINKMEM_OnRx(OS_U8 Data) {
  OS_OnRx(Data);
}

/*********************************************************************
*
*       _JLINKMEM_OnTx()
*/
static void _JLINKMEM_OnTx(void) {
  OS_OnTx();
}

/*********************************************************************
*
*       _JLINKMEM_GetNextChar()
*/
static OS_INT _JLINKMEM_GetNextChar(void) {
  return OS_COM_GetNextChar();
}

/*********************************************************************
*
*       OS_COM_Init()
*       Initialize memory access for embOSView
*/
void OS_COM_Init(void) {
  JLINKMEM_SetpfOnRx(_JLINKMEM_OnRx);
  JLINKMEM_SetpfOnTx(_JLINKMEM_OnTx);
  JLINKMEM_SetpfGetNextChar(_JLINKMEM_GetNextChar);
}

/*********************************************************************
*
*       OS_COM_Send1()
*       Send one character via memory
*/
void OS_COM_Send1(OS_U8 c) {
  JLINKMEM_SendChar(c);
}

#elif OS_UART_USED   // Communication via UART, can not be implemented generic

  #error "OS_UART currently not supported."

  //
  // Device specific versions of the 3 communication functions
  // OS_COM_IsrHandler(void)
  // OS_COM_Send1(OS_U8 c)
  // OS_COM_Init(void)
  // have to be implemented when a UART shall be used for communication to embOSView.
  // Samples can be found in our device specific projects.

#else  /*  No communication or UART selected, using dummies */

void OS_COM_Send1(OS_U8 c) {
  OS_USEPARA(c);           /* Avoid compiler warning */
  OS_COM_ClearTxActive();  /* Let the OS know that Tx is not busy */
}

#endif /*  OS_UART_USED  */

/****** Final check of configuration ********************************/

#ifndef OS_UART_USED
  #error "OS_UART_USED has to be defined"
#endif

#if (OS_TICK_FREQ != 1000u)
  #error "OS_TICK_FREQ != 1000, ensure OS_GetTime_Cycles() and OS_ConvertCycles2us() work."
#endif

/*********************************************************************
*
*       OS interrupt handler and ISR specific functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_ARM_EnableISR
*/
void OS_ARM_EnableISR(int ISRIndex) {
  OS_DI();
  if (ISRIndex < NUM_INTERRUPTS) {
    if (ISRIndex >= 16) {
      //
      // Enable standard "external" interrupts, starting at index 16
      //
      ISRIndex -= 16;
      *(((OS_U32*) NVIC_ENABLE_ADDR) + (ISRIndex >> 5)) = (1uL << (ISRIndex & 0x1Fu));
    } else if (ISRIndex == ISR_ID_MPU) {
      //
      // Enable the MemManage interrupt.
      //
      SYSHND_CTRL |= NVIC_SYS_HND_CTRL_MEM;
    } else if (ISRIndex == ISR_ID_BUS) {
      //
      // Enable the bus fault interrupt.
      //
      SYSHND_CTRL |= NVIC_SYS_HND_CTRL_BUS;
    } else if (ISRIndex == ISR_ID_USAGE) {
      //
      // Enable the usage fault interrupt.
      //
      SYSHND_CTRL |= NVIC_SYS_HND_CTRL_USAGE;
    } else if (ISRIndex == ISR_ID_SYSTICK) {
      //
      // Enable the System Tick interrupt.
      //
      SYS_TICK_CONTROL |= (1uL << SYS_TICK_INT_ENABLE_BIT);
    }
  }
  OS_RestoreI();
}

/*********************************************************************
*
*       OS_ARM_DisableISR
*/
void OS_ARM_DisableISR(int ISRIndex) {
  OS_DI();
  if (ISRIndex < NUM_INTERRUPTS) {
    if (ISRIndex >= 16) {
      //
      // Disable standard "external" interrupts
      //
      ISRIndex -= 16;
      *(((OS_U32*) NVIC_DISABLE_ADDR) + (ISRIndex >> 5)) = (1uL << (ISRIndex & 0x1Fu));
    } else if (ISRIndex == ISR_ID_MPU) {
      //
      // Disable the MemManage interrupt.
      //
      SYSHND_CTRL &= ~NVIC_SYS_HND_CTRL_MEM;
    } else if (ISRIndex == ISR_ID_BUS) {
      //
      // Disable the bus fault interrupt.
      //
      SYSHND_CTRL &= ~NVIC_SYS_HND_CTRL_BUS;
    } else if (ISRIndex == ISR_ID_USAGE) {
      //
      // Disable the usage fault interrupt.
      //
      SYSHND_CTRL &= ~NVIC_SYS_HND_CTRL_USAGE;
    } else if (ISRIndex == ISR_ID_SYSTICK) {
      //
      // Enable the System Tick interrupt.
      //
      SYS_TICK_CONTROL &= ~(1uL << SYS_TICK_INT_ENABLE_BIT);
    }
  }
  OS_RestoreI();
}

/*********************************************************************
*
*       OS_ARM_ISRSetPrio
*
*   Notes:
*     (1) Some priorities of system handler are reserved
*         0..3 : Priority can not be set
*         7..10: Reserved
*         13   : Reserved
*     (2) System handler use different control register. This affects
*         ISRIndex 0..15
*/
int OS_ARM_ISRSetPrio(int ISRIndex, int Prio) {
  OS_U8* pPrio;
  int    OldPrio;

  OldPrio = 0;
  if (ISRIndex < NUM_INTERRUPTS) {
    if (ISRIndex >= 16) {
      //
      // Handle standard "external" interrupts
      //
      ISRIndex -= 16;                   // Adjust index
      OS_DI();
      pPrio   = (OS_U8*)(NVIC_PRIOBASE_ADDR + ISRIndex);
      OldPrio = *pPrio;
      *pPrio  = Prio;
      OS_RestoreI();
    } else {
      //
      // Handle System Interrupt controller
      //
      if ((ISRIndex < 4) | ((ISRIndex >= 7) && (ISRIndex <= 10)) | (ISRIndex == 13)) {
        //
        // Reserved ISR channel, do nothing
        //
      } else {
        //
        // Set priority in system interrupt priority control register
        //
        OS_DI();
        pPrio = (OS_U8*)(SYSPRI1_ADDR);
        ISRIndex -= 4;                  // Adjust Index
        OldPrio = pPrio[ISRIndex];
        pPrio[ISRIndex] = Prio;
        OS_RestoreI();
      }
    }
  }
  return OldPrio;
}

/****** End Of File *************************************************/
