//*****************************************************************************
//
// Copyright (C) 2012 - 2014 Texas Instruments Incorporated - http://www.ti.com/
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
//  Redistributions of source code must retain the above copyright
//  notice, this list of conditions and the following disclaimer.
//
//  Redistributions in binary form must reproduce the above copyright
//  notice, this list of conditions and the following disclaimer in the
//  documentation and/or other materials provided with the
//  distribution.
//
//  Neither the name of Texas Instruments Incorporated nor the names of
//  its contributors may be used to endorse or promote products derived
//  from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// MSP432 CMSIS compliant Register Definitions
//
// The register and bit defines in this file have been formatted to
// follow a unified naming style.
//
// - Registers:   ModuleName[ModuleInstance][ModuleVariant]->RegisterName.reg
// - Bits:        ModuleName[ModuleInstance][ModuleVariant]->RegisterName.bit.BitName
//
// Where the user guide lists registers or bits with leading module
// names, the module names have been removed for better readability.
//
// File creation date: 2014-05-23
//
//****************************************************************************


#ifndef MSP432P401R_CMSIS_H
#define MSP432P401R_CMSIS_H

// Use standard integer types with explicit width
#include <stdint.h>

// MSP430 intrinsic mapping to ARM equivalents
//#include "msp430dna.h"

// Include CMSIS Cortex-M4 Core Peripheral Access Layer Header File
#include "core_cm4.h"

//*****************************************************************************
// Definition of standard bits
//*****************************************************************************
#define BIT0                            (0x0001u)
#define BIT1                            (0x0002u)
#define BIT2                            (0x0004u)
#define BIT3                            (0x0008u)
#define BIT4                            (0x0010u)
#define BIT5                            (0x0020u)
#define BIT6                            (0x0040u)
#define BIT7                            (0x0080u)
#define BIT8                            (0x0100u)
#define BIT9                            (0x0200u)
#define BITA                            (0x0400u)
#define BITB                            (0x0800u)
#define BITC                            (0x1000u)
#define BITD                            (0x2000u)
#define BITE                            (0x4000u)
#define BITF                            (0x8000u)
#define BIT(x)                          (1 << (x))


//*****************************************************************************
// CMSIS-compatible Interrupt Number Definition
//*****************************************************************************
#ifndef __CMSIS_CONFIG__
#define __CMSIS_CONFIG__

typedef enum IRQn
{
  // Cortex-M4 Processor Exceptions Numbers
  NonMaskableInt_IRQn         = -14,    /*  2 Non Maskable Interrupt */
  HardFault_IRQn              = -13,    /*  3 Hard Fault Interrupt */
  MemoryManagement_IRQn       = -12,    /*  4 Memory Management Interrupt */
  BusFault_IRQn               = -11,    /*  5 Bus Fault Interrupt */
  UsageFault_IRQn             = -10,    /*  6 Usage Fault Interrupt */
  SVCall_IRQn                 = -5,     /* 11 SV Call Interrupt */
  DebugMonitor_IRQn           = -4,     /* 12 Debug Monitor Interrupt */
  PendSV_IRQn                 = -2,     /* 14 Pend SV Interrupt */
  SysTick_IRQn                = -1,     /* 15 System Tick Interrupt */
  //  Peripheral Exceptions Numbers
  PSS_IRQn                    =  0,     /* 16 PSS Interrupt */
  CS_IRQn                     =  1,     /* 17 CS Interrupt */
  PCM_IRQn                    =  2,     /* 18 PCM Interrupt */
  WDT_A_IRQn                  =  3,     /* 19 WDT_A Interrupt */
  FPU_IRQn                    =  4,     /* 20 FPU Interrupt */
  FLCTL_IRQn                  =  5,     /* 21 FLCTL Interrupt */
  COMP_E0_IRQn                =  6,     /* 22 COMP_E0 Interrupt */
  COMP_E1_IRQn                =  7,     /* 23 COMP_E1 Interrupt */
  TA0_0_IRQn                  =  8,     /* 24 TA0_0 Interrupt */
  TA0_N_IRQn                  =  9,     /* 25 TA0_N Interrupt */
  TA1_0_IRQn                  = 10,     /* 26 TA1_0 Interrupt */
  TA1_N_IRQn                  = 11,     /* 27 TA1_N Interrupt */
  TA2_0_IRQn                  = 12,     /* 28 TA2_0 Interrupt */
  TA2_N_IRQn                  = 13,     /* 29 TA2_N Interrupt */
  TA3_0_IRQn                  = 14,     /* 30 TA3_0 Interrupt */
  TA3_N_IRQn                  = 15,     /* 31 TA3_N Interrupt */
  EUSCIA0_IRQn                = 16,     /* 32 EUSCIA0 Interrupt */
  EUSCIA1_IRQn                = 17,     /* 33 EUSCIA1 Interrupt */
  EUSCIA2_IRQn                = 18,     /* 34 EUSCIA2 Interrupt */
  EUSCIA3_IRQn                = 19,     /* 35 EUSCIA3 Interrupt */
  EUSCIB0_IRQn                = 20,     /* 36 EUSCIB0 Interrupt */
  EUSCIB1_IRQn                = 21,     /* 37 EUSCIB1 Interrupt */
  EUSCIB2_IRQn                = 22,     /* 38 EUSCIB2 Interrupt */
  EUSCIB3_IRQn                = 23,     /* 39 EUSCIB3 Interrupt */
  ADC14_IRQn                  = 24,     /* 40 ADC14 Interrupt */
  T32_INT1_IRQn               = 25,     /* 41 T32_INT1 Interrupt */
  T32_INT2_IRQn               = 26,     /* 42 T32_INT2 Interrupt */
  T32_INTC_IRQn               = 27,     /* 43 T32_INTC Interrupt */
  AES256_IRQn                 = 28,     /* 44 AES256 Interrupt */
  RTC_C_IRQn                  = 29,     /* 45 RTC_C Interrupt */
  DMA_ERR_IRQn                = 30,     /* 46 DMA_ERR Interrupt */
  DMA_INT3_IRQn               = 31,     /* 47 DMA_INT3 Interrupt */
  DMA_INT2_IRQn               = 32,     /* 48 DMA_INT2 Interrupt */
  DMA_INT1_IRQn               = 33,     /* 49 DMA_INT1 Interrupt */
  DMA_INT0_IRQn               = 34,     /* 50 DMA_INT0 Interrupt */
  PORT1_IRQn                  = 35,     /* 51 PORT1 Interrupt */
  PORT2_IRQn                  = 36,     /* 52 PORT2 Interrupt */
  PORT3_IRQn                  = 37,     /* 53 PORT3 Interrupt */
  PORT4_IRQn                  = 38,     /* 54 PORT4 Interrupt */
  PORT5_IRQn                  = 39,     /* 55 PORT5 Interrupt */
  PORT6_IRQn                  = 40      /* 56 PORT6 Interrupt */
} IRQn_Type;

//*****************************************************************************
// CMSIS-compatible configuration of the Cortex-M4 Processor and Core Peripherals
//*****************************************************************************
#define __MPU_PRESENT           1     // MPU present or not
#define __NVIC_PRIO_BITS        3     // Number of Bits used for Prio Levels
#define __FPU_PRESENT           1     // FPU present or not

#endif // __CMSIS_CONFIG__

//*****************************************************************************
// Definitions for 8/16/32-bit wide bit band access
//*****************************************************************************
#define HWREGBIT8(x, b)                                    (HWREG8(((uint32_t)(x) & 0xF0000000) | 0x02000000 | (((uint32_t)(x) & 0x000FFFFF) << 5) | ((b) << 2)))
#define HWREGBIT16(x, b)                                   (HWREG16(((uint32_t)(x) & 0xF0000000) | 0x02000000 | (((uint32_t)(x) & 0x000FFFFF) << 5) | ((b) << 2)))
#define HWREGBIT32(x, b)                                   (HWREG32(((uint32_t)(x) & 0xF0000000) | 0x02000000 | (((uint32_t)(x) & 0x000FFFFF) << 5) | ((b) << 2)))
#define BITBAND_SRAM(x, b)                                 (*((volatile uint8_t *) (0x22000000 +  (((uint32_t)(uint32_t *)&x) - 0x20000000)*32 + b*4)))
#define BITBAND_PERI(x, b)                                 (*((volatile uint8_t *) (0x42000000 +  (((uint32_t)(uint32_t *)&x) - 0x40000000)*32 + b*4)))

//*****************************************************************************
// Peripheral register map
//*****************************************************************************

//*****************************************************************************
// ADC14 Registers
//*****************************************************************************
typedef struct {
  union {    /* ADC14_CTL0 Register */
    __IO uint32_t reg;
    struct { /* ADC14_CTL0 Control Bits */
      __IO uint32_t SC                  	:	1;  /* ADC14 start conversion. Software-controlled sample-and-conversion start.ADC14SC  */
                                                /* and ADC14ENC may be set together with one instruction. ADC14SC is reset  */
                                                /* automatically. */
      __IO uint32_t ENC                 	:	1;  /* ADC14 enable conversion ADC14ENC low pulse width must be at-least 3 ADC14CLK  */
                                                /* cycles. */
      __I  uint32_t RESERVED_2          	:	2;  /* Reserved */
      __IO uint32_t ON                  	:	1;  /* ADC14 on */
      __I  uint32_t RESERVED_1          	:	2;  /* Reserved */
      __IO uint32_t MSC                 	:	1;  /* ADC14 multiple sample and conversion. Valid only for sequence or repeated modes. */
      __IO uint32_t SHT0                	:	4;  /* ADC14 sample-and-hold time. These bits define the number of ADC14CLK cycles in  */
                                                /* the sampling period for registers ADC14MEM0 to ADC14MEM7 and ADC14MEM24 to  */
                                                /* ADC14MEM31. Can be modified only when ADC14ENC = 0. 1000b to 1111b = Reserved */
      __IO uint32_t SHT1                	:	4;  /* ADC14 sample-and-hold time. These bits define the number of ADC14CLK cycles in  */
                                                /* the sampling period for registers ADC14MEM8 to ADC14MEM23. Can be modified only  */
                                                /* when ADC14ENC = 0. 1000b to 1111b = Reserved */
      __I  uint32_t BUSY                	:	1;  /* ADC14 busy. This bit indicates an active sample or conversion operation. */
      __IO uint32_t CONSEQ              	:	2;  /* ADC14 conversion sequence mode select Can be modified only when ADC14ENC=0. */
      __IO uint32_t SSEL                	:	3;  /* ADC14 clock source select. Can be modified only when ADC14ENC = 0. */
      __IO uint32_t DIV                 	:	3;  /* ADC14 clock divider. Can be modified only when ADC14ENC = 0. */
      __IO uint32_t ISSH                	:	1;  /* ADC14 invert signal sample-and-hold. Can be modified only when ADC14ENC = 0.  */
                                                /* Setting ADC14ISSH=1 and triggering a conversion using ADC14SC is not recommended */
                                                /* . ADC14SC bit gets reset to 0 automatically at the end of conversion and if  */
                                                /* ADC14ISSH=1, the 1->0 transition on ADC14SC will trigger another conversion. */
      __IO uint32_t SHP                 	:	1;  /* ADC14 sample-and-hold pulse-mode select. This bit selects the source of the  */
                                                /* sampling signal (SAMPCON) to be either the output of the sampling timer or the  */
                                                /* sample-input signal directly. Can be modified only when ADC14ENC = 0. */
      __IO uint32_t SHS                 	:	3;  /* ADC14 sample-and-hold source select. Can be modified only when ADC14ENC = 0. */
      __IO uint32_t PDIV                	:	2;  /* ADC14 predivider. This bit predivides the selected ADC14 clock source. Can be  */
                                                /* modified only when ADC14ENC = 0. */

    } bit;
  } CTL0;
  union {    /* ADC14_CTL1 Register */
    __IO uint32_t reg;
    struct { /* ADC14_CTL1 Control Bits */
      __IO uint32_t PWRMD               	:	2;  /* ADC power modes. Can be modified only when ADC14ENC = 0. 01b = Reserved 11b =  */
                                                /* Reserved   */
      __IO uint32_t REFBURST            	:	1;  /* ADC reference buffer burst. Can be modified only when ADC14ENC = 0. */
      __IO uint32_t DF                  	:	1;  /* ADC14 data read-back format. Data is always stored in the binary unsigned format */
                                                /* . */
      __IO uint32_t RES                 	:	2;  /* ADC14 resolution. This bit defines the conversion result resolution. Can be  */
                                                /* modified only when ADC14ENC = 0. */
      __I  uint32_t RESERVED_3          	:	10;  /* Reserved */
      __IO uint32_t CSTARTADD           	:	5;  /* ADC14 conversion start address. These bits select which ADC14 conversion memory  */
                                                /* register is used for a single conversion or for the first conversion in a  */
                                                /* sequence. The value of CSTARTADDx is 0h to 1Fh, corresponding to ADC14MEM0 to  */
                                                /* ADC14MEM31 */
      __I  uint32_t RESERVED_2          	:	1;  /* Reserved */
      __IO uint32_t BATMAP              	:	1;  /* Controls 1/2 AVCC ADC input channel selection */
      __IO uint32_t TCMAP               	:	1;  /* Controls temperature sensor ADC input channel selection */
      __IO uint32_t CH0MAP              	:	1;  /* Controls internal channel 0 selection to ADC input channel MAX-2 */
      __IO uint32_t CH1MAP              	:	1;  /* Controls internal channel 1 selection to ADC input channel MAX-3 */
      __IO uint32_t CH2MAP              	:	1;  /* Controls internal channel 2 selection to ADC input channel MAX-4    */
      __IO uint32_t CH3MAP              	:	1;  /* Controls internal channel 3 selection to ADC input channel MAX-5    */
      __I  uint32_t RESERVED_1          	:	4;  /* Reserved */

    } bit;
  } CTL1;
  union {    /* ADC14_LO0 Register */
    __IO uint32_t reg;
    struct { /* ADC14_LO0 Control Bits */
      __IO uint32_t LO0                 	:	16;  /* Low threshold 0. Can be modified only when ADC14ENC = 0. If ADC14DF = 0,  */
                                                /* unsigned binary format:  The 14-bit threshold value must be right aligned. Bit  */
                                                /* 13 is the MSB. Bits 15-14 are 0 in 14-bit mode, bits 15-12 are 0 in 12-bit mode, */
                                                /* bits 15-10 are 0 in 10-bit mode, and bits 15-8 are 0 in 8-bit mode. The reset  */
                                                /* value is: 0h If ADC14DF = 1, 2s-complement format:  The 14-bit threshold value  */
                                                /* must be left aligned. Bit 15 is the MSB. Bits 1-0 are 0 in 14-bit mode, bits 3-0 */
                                                /* are 0 in 12-bit mode, bits 5-0 are 0 in 10-bit mode, and bits 7-0 are 0 in 8- */
                                                /* bit mode.  The reset value is: 8000h */
      __I  uint32_t RESERVED_1          	:	16;  /* Reserved */

    } bit;
  } LO0;
  union {    /* ADC14_HI0 Register */
    __IO uint32_t reg;
    struct { /* ADC14_HI0 Control Bits */
      __IO uint32_t HI0                 	:	16;  /* High threshold 0. Can be modified only when ADC14ENC = 0. If ADC14DF = 0,  */
                                                /* unsigned binary format:  The 14-bit threshold value must be right aligned. Bit  */
                                                /* 13 is the MSB. Bits 15-14 are 0 in 14-bit mode, bits 15-12 are 0 in 12-bit mode, */
                                                /* bits 15-10 are 0 in 10-bit mode, and bits 15-8 are 0 in 8-bit mode. The reset  */
                                                /* value is: 3FFFh (14 bit), 0FFFh (12 bit), 03FFh (10 bit), or 00FFh (8 bit) If  */
                                                /* ADC14DF = 1, 2s-complement format:  The 14-bit threshold value must be left  */
                                                /* aligned. Bit 15 is the MSB. Bits 1-0 are 0 in 14-bit mode, bits 3-0 are 0 in 12- */
                                                /* bit mode, bits 5-0 are 0 in 10-bit mode, and bits 7-0 are 0 in 8-bit mode.  The  */
                                                /* reset value is: 7FFCh (14 bit), 7FF0h (12 bit), 7FC0h (10 bit), or 7F00h (8 bit) */
      __I  uint32_t RESERVED_1          	:	16;  /* Reserved */

    } bit;
  } HI0;
  union {    /* ADC14_LO1 Register */
    __IO uint32_t reg;
    struct { /* ADC14_LO1 Control Bits */
      __IO uint32_t LO1                 	:	16;  /* Low threshold 1. Can be modified only when ADC14ENC = 0. If ADC14DF = 0,  */
                                                /* unsigned binary format:  The 14-bit threshold value must be right aligned. Bit  */
                                                /* 13 is the MSB. Bits 15-14 are 0 in 14-bit mode, bits 15-12 are 0 in 12-bit mode, */
                                                /* bits 15-10 are 0 in 10-bit mode, and bits 15-8 are 0 in 8-bit mode. The reset  */
                                                /* value is: 0h If ADC14DF = 1, 2s-complement format:  The 14-bit threshold value  */
                                                /* must be left aligned. Bit 15 is the MSB. Bits 1-0 are 0 in 14-bit mode, bits 3-0 */
                                                /* are 0 in 12-bit mode, bits 5-0 are 0 in 10-bit mode, and bits 7-0 are 0 in 8- */
                                                /* bit mode.  The reset value is: 8000h */
      __I  uint32_t RESERVED_1          	:	16;  /* Reserved */

    } bit;
  } LO1;
  union {    /* ADC14_HI1 Register */
    __IO uint32_t reg;
    struct { /* ADC14_HI1 Control Bits */
      __IO uint32_t HI1                 	:	16;  /* High threshold 1. Can be modified only when ADC14ENC = 0. If ADC14DF = 0,  */
                                                /* unsigned binary format:  The 14-bit threshold value must be right aligned. Bit  */
                                                /* 13 is the MSB. Bits 15-14 are 0 in 14-bit mode, bits 15-12 are 0 in 12-bit mode, */
                                                /* bits 15-10 are 0 in 10-bit mode, and bits 15-8 are 0 in 8-bit mode. The reset  */
                                                /* value is: 3FFFh (14 bit), 0FFFh (12 bit), 03FFh (10 bit), or 00FFh (8 bit) If  */
                                                /* ADC14DF = 1, 2s-complement format:  The 14-bit threshold value must be left  */
                                                /* aligned. Bit 15 is the MSB. Bits 1-0 are 0 in 14-bit mode, bits 3-0 are 0 in 12- */
                                                /* bit mode, bits 5-0 are 0 in 10-bit mode, and bits 7-0 are 0 in 8-bit mode.  The  */
                                                /* reset value is: 7FFCh (14 bit), 7FF0h (12 bit), 7FC0h (10 bit), or 7F00h (8 bit) */
      __I  uint32_t RESERVED_1          	:	16;  /* Reserved */

    } bit;
  } HI1;
  union {    /* ADC14_MCTL Register */
    __IO uint32_t reg;
    struct { /* ADC14_MCTL Control Bits */
      __IO uint32_t INCH                	:	5;  /* Input channel select. If even channels are set as differential then odd channel  */
                                                /* configuration is ignored. Can be modified only when ADC14ENC = 0. */
      __I  uint32_t RESERVED_3          	:	2;  /* Reserved */
      __IO uint32_t EOS                 	:	1;  /* End of sequence. Indicates the last conversion in a sequence. Can be modified  */
                                                /* only when ADC14ENC = 0. */
      __IO uint32_t VRSEL               	:	4;  /* Selects combinations of V(R+) and V(R-) sources as well as the buffer selection  */
                                                /* and buffer on or off. When REFOUT = 1, VeREF+ buffered configuration is not  */
                                                /* available. Can be modified only when ADC14ENC = 0. 0010b to 1101b = Reserved It  */
                                                /* is recommended to connect VeREF- to on-board ground when VeREF- is selected for  */
                                                /* V(R-). */
      __I  uint32_t RESERVED_2          	:	1;  /* Reserved */
      __IO uint32_t DIF                 	:	1;  /* Differential mode.  Can be modified only when ADC14ENC = 0. */
      __IO uint32_t WINC                	:	1;  /* Comparator window enable Can be modified only when ADC14ENC = 0. */
      __IO uint32_t WINCTH              	:	1;  /* Window comparator threshold register selection Can be modified only when  */
                                                /* ADC14ENC = 0. */
      __I  uint32_t RESERVED_1          	:	16;  /* Reserved */

    } bit;
  } MCTL;
  uint8_t RESERVED0[124];
  union {    /* ADC14_MEM Register */
    __IO uint32_t reg;
    struct { /* ADC14_MEM Control Bits */
      __IO uint32_t CONVERSION_RESULTS  	:	16;  /* If ADC14DF = 0, unsigned binary: The 14-bit conversion results are right aligned */
                                                /* . Bit 13 is the MSB. Bits 15-14 are 0 in 14-bit mode, bits 15-12 are 0 in 12-bit */
                                                /* mode, bits 15-10 are 0 in 10-bit mode, and bits 15-8 are 0 in 8-bit mode. If  */
                                                /* the user writes to the conversion memory registers, the results are corrupted.   */
                                                /* If ADC14DF = 1, 2s-complement format: The 14-bit conversion results are left  */
                                                /* aligned. Bit 15 is the MSB. Bits 1-0 are 0 in 14-bit mode, bits 3-0 are 0 in 12- */
                                                /* bit mode, bits 5-0 are 0 in 10-bit mode, and bits 7-0 are 0 in 8-bit mode. The  */
                                                /* data is stored in the right-justified format and is converted to the left- */
                                                /* justified 2s-complement format during read back.  If the user writes to the  */
                                                /* conversion memory registers, the results are corrupted.  Reading this register  */
                                                /* clears the corresponding bit in ADC14IFG0. */
      __I  uint32_t RESERVED_1          	:	16;  /* Reserved */

    } bit;
  } MEM;
  uint8_t RESERVED1[160];
  union {    /* ADC14_IER0 Register */
    __IO uint32_t reg;
    struct { /* ADC14_IER0 Control Bits */
      __IO uint32_t IE0                 	:	1;  /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG0  */
                                                /* bit. */
      __IO uint32_t IE1                 	:	1;  /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG1  */
                                                /* bit */
      __IO uint32_t IE2                 	:	1;  /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG2  */
                                                /* bit. */
      __IO uint32_t IE3                 	:	1;  /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG3  */
                                                /* bit. */
      __IO uint32_t IE4                 	:	1;  /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG4  */
                                                /* bit. */
      __IO uint32_t IE5                 	:	1;  /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG5  */
                                                /* bit. */
      __IO uint32_t IE6                 	:	1;  /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG6  */
                                                /* bit. */
      __IO uint32_t IE7                 	:	1;  /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG7  */
                                                /* bit. */
      __IO uint32_t IE8                 	:	1;  /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG8  */
                                                /* bit. */
      __IO uint32_t IE9                 	:	1;  /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG9  */
                                                /* bit. */
      __IO uint32_t IE10                	:	1;  /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG10  */
                                                /* bit. */
      __IO uint32_t IE11                	:	1;  /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG11  */
                                                /* bit. */
      __IO uint32_t IE12                	:	1;  /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG12  */
                                                /* bit. */
      __IO uint32_t IE13                	:	1;  /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG13  */
                                                /* bit. */
      __IO uint32_t IE14                	:	1;  /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG14  */
                                                /* bit. */
      __IO uint32_t IE15                	:	1;  /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG15  */
                                                /* bit. */
      __IO uint32_t IE16                	:	1;  /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG16  */
                                                /* bit. */
      __IO uint32_t IE17                	:	1;  /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG17  */
                                                /* bit. */
      __IO uint32_t IE18                	:	1;  /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG18  */
                                                /* bit. */
      __IO uint32_t IE19                	:	1;  /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG19  */
                                                /* bit. */
      __IO uint32_t IE20                	:	1;  /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG20  */
                                                /* bit. */
      __IO uint32_t IE21                	:	1;  /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG21  */
                                                /* bit. */
      __IO uint32_t IE22                	:	1;  /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG22  */
                                                /* bit. */
      __IO uint32_t IE23                	:	1;  /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG23  */
                                                /* bit. */
      __IO uint32_t IE24                	:	1;  /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG24  */
                                                /* bit. */
      __IO uint32_t IE25                	:	1;  /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG25  */
                                                /* bit. */
      __IO uint32_t IE26                	:	1;  /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG26  */
                                                /* bit. */
      __IO uint32_t IE27                	:	1;  /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG27  */
                                                /* bit. */
      __IO uint32_t IE28                	:	1;  /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG28  */
                                                /* bit. */
      __IO uint32_t IE29                	:	1;  /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG29  */
                                                /* bit. */
      __IO uint32_t IE30                	:	1;  /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG30  */
                                                /* bit. */
      __IO uint32_t IE31                	:	1;  /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG31  */
                                                /* bit. */

    } bit;
  } IER0;
  union {    /* ADC14_IER1 Register */
    __IO uint32_t reg;
    struct { /* ADC14_IER1 Control Bits */
      __I  uint32_t RESERVED_2          	:	1;  /* Reserved */
      __IO uint32_t INIE                	:	1;  /* Interrupt enable for the ADC14MEMx result register being greater than the  */
                                                /* ADC14LO threshold and below the ADC14HI threshold. */
      __IO uint32_t LOIE                	:	1;  /* Interrupt enable for the falling short of the lower limit interrupt of the  */
                                                /* window comparator for the ADC14MEMx result register. */
      __IO uint32_t HIIE                	:	1;  /* Interrupt enable for the exceeding the upper limit interrupt of the window  */
                                                /* comparator for ADC14MEMx result register. */
      __IO uint32_t OVIE                	:	1;  /* ADC14MEMx overflow-interrupt enable.  */
      __IO uint32_t TOVIE               	:	1;  /* ADC14 conversion-time-overflow interrupt enable.   */
      __IO uint32_t RDYIE               	:	1;  /* ADC14 local buffered reference ready interrupt enable.   */
      __I  uint32_t RESERVED_1          	:	25;  /* Reserved */

    } bit;
  } IER1;
  union {    /* ADC14_IFGR0 Register */
    __IO uint32_t reg;
    struct { /* ADC14_IFGR0 Control Bits */
      __I  uint32_t IFG0                	:	1;  /* ADC14MEM0 interrupt flag. This bit is set to 1 when ADC14MEM0 is loaded with a  */
                                                /* conversion result. This bit is reset to 0 when the ADC14MEM0 register is read,  */
                                                /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                /* ADC14CLRIFGR0 register is set to 1. */
      __I  uint32_t IFG1                	:	1;  /* ADC14MEM1 interrupt flag. This bit is set to 1 when ADC14MEM1 is loaded with a  */
                                                /* conversion result. This bit is reset to 0 when the ADC14MEM1 register is read,  */
                                                /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                /* ADC14CLRIFGR0 register is set to 1. */
      __I  uint32_t IFG2                	:	1;  /* ADC14MEM2 interrupt flag. This bit is set to 1 when ADC14MEM2 is loaded with a  */
                                                /* conversion result. This bit is reset to 0 when the ADC14MEM2 register is read,  */
                                                /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                /* ADC14CLRIFGR0 register is set to 1. */
      __I  uint32_t IFG3                	:	1;  /* ADC14MEM3 interrupt flag. This bit is set to 1 when ADC14MEM3 is loaded with a  */
                                                /* conversion result. This bit is reset to 0 when the ADC14MEM3 register is read,  */
                                                /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                /* ADC14CLRIFGR0 register is set to 1. */
      __I  uint32_t IFG4                	:	1;  /* ADC14MEM4 interrupt flag. This bit is set to 1 when ADC14MEM4 is loaded with a  */
                                                /* conversion result. This bit is reset to 0 when the ADC14MEM4 register is read,  */
                                                /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                /* ADC14CLRIFGR0 register is set to 1. */
      __I  uint32_t IFG5                	:	1;  /* ADC14MEM5 interrupt flag. This bit is set to 1 when ADC14MEM5 is loaded with a  */
                                                /* conversion result. This bit is reset to 0 when the ADC14MEM5 register is read,  */
                                                /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                /* ADC14CLRIFGR0 register is set to 1. */
      __I  uint32_t IFG6                	:	1;  /* ADC14MEM6 interrupt flag. This bit is set to 1 when ADC14MEM6 is loaded with a  */
                                                /* conversion result. This bit is reset to 0 when the ADC14MEM6 register is read,  */
                                                /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                /* ADC14CLRIFGR0 register is set to 1. */
      __I  uint32_t IFG7                	:	1;  /* ADC14MEM7 interrupt flag. This bit is set to 1 when ADC14MEM7 is loaded with a  */
                                                /* conversion result. This bit is reset to 0 when the ADC14MEM7 register is read,  */
                                                /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                /* ADC14CLRIFGR0 register is set to 1.. */
      __I  uint32_t IFG8                	:	1;  /* ADC14MEM8 interrupt flag. This bit is set to 1 when ADC14MEM8 is loaded with a  */
                                                /* conversion result. This bit is reset to 0 when the ADC14MEM8 register is read,  */
                                                /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                /* ADC14CLRIFGR0 register is set to 1. */
      __I  uint32_t IFG9                	:	1;  /* ADC14MEM9 interrupt flag. This bit is set to 1 when ADC14MEM9 is loaded with a  */
                                                /* conversion result. This bit is reset to 0 when the ADC14MEM9 register is read,  */
                                                /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                /* ADC14CLRIFGR0 register is set to  1. */
      __I  uint32_t IFG10               	:	1;  /* ADC14MEM10 interrupt flag. This bit is set to 1 when ADC14MEM10 is loaded with a */
                                                /* conversion result. This bit is reset to 0 when the ADC14MEM10 register is read, */
                                                /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                /* ADC14CLRIFGR0 register is set to 1. */
      __I  uint32_t IFG11               	:	1;  /* ADC14MEM11 interrupt flag. This bit is set to 1 when ADC14MEM11 is loaded with a */
                                                /* conversion result. This bit is reset to 0 when the ADC14MEM11 register is read, */
                                                /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                /* ADC14CLRIFGR0 register is set to 1. */
      __I  uint32_t IFG12               	:	1;  /* ADC14MEM12 interrupt flag. This bit is set to 1 when ADC14MEM12 is loaded with a */
                                                /* conversion result. This bit is reset to 0 when the ADC14MEM12 register is read, */
                                                /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                /* ADC14CLRIFGR0 register is set to 1. */
      __I  uint32_t IFG13               	:	1;  /* ADC14MEM13 interrupt flag. This bit is set to 1 when ADC14MEM13 is loaded with a */
                                                /* conversion result. This bit is reset to 0 when the ADC14MEM13 register is read, */
                                                /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                /* ADC14CLRIFGR0 register is set to 1. */
      __I  uint32_t IFG14               	:	1;  /* ADC14MEM14 interrupt flag. This bit is set to 1 when ADC14MEM14 is loaded with a */
                                                /* conversion result. This bit is reset to 0 when the ADC14MEM14 register is read, */
                                                /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                /* ADC14CLRIFGR0 register is set to 1. */
      __I  uint32_t IFG15               	:	1;  /* ADC14MEM15 interrupt flag. This bit is set to 1 when ADC14MEM15 is loaded with a */
                                                /* conversion result. This bit is reset to 0 when the ADC14MEM15 register is read, */
                                                /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                /* ADC14CLRIFGR0 register is set to 1. */
      __I  uint32_t IFG16               	:	1;  /* ADC14MEM16 interrupt flag. This bit is set to 1 when ADC14MEM16 is loaded with a */
                                                /* conversion result. This bit is reset to 0 when the ADC14MEM16 register is read, */
                                                /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                /* ADC14CLRIFGR0 register is set to 1. */
      __I  uint32_t IFG17               	:	1;  /* ADC14MEM17 interrupt flag. This bit is set to 1 when ADC14MEM17 is loaded with a */
                                                /* conversion result. This bit is reset to 0 when the ADC14MEM17 register is read, */
                                                /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                /* ADC14CLRIFGR0 register is set to 1. */
      __I  uint32_t IFG18               	:	1;  /* ADC14MEM18 interrupt flag. This bit is set to 1 when ADC14MEM18 is loaded with a */
                                                /* conversion result. This bit is reset to 0 when the ADC14MEM18 register is read, */
                                                /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                /* ADC14CLRIFGR0 register is set to 1. */
      __I  uint32_t IFG19               	:	1;  /* ADC14MEM19 interrupt flag. This bit is set to 1 when ADC14MEM19 is loaded with a */
                                                /* conversion result. This bit is reset to 0 when the ADC14MEM19 register is read, */
                                                /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                /* ADC14CLRIFGR0 register is set to 1. */
      __I  uint32_t IFG20               	:	1;  /* ADC14MEM20 interrupt flag. This bit is set to 1 when ADC14MEM20 is loaded with a */
                                                /* conversion result. This bit is reset to 0 when the ADC14MEM20 register is read, */
                                                /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                /* ADC14CLRIFGR0 register is set to 1. */
      __I  uint32_t IFG21               	:	1;  /* ADC14MEM21 interrupt flag. This bit is set to 1 when ADC14MEM21 is loaded with a */
                                                /* conversion result. This bit is reset to 0 when the ADC14MEM21 register is read, */
                                                /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                /* ADC14CLRIFGR0 register is set to 1. */
      __I  uint32_t IFG22               	:	1;  /* ADC14MEM22 interrupt flag. This bit is set to 1 when ADC14MEM22 is loaded with a */
                                                /* conversion result. This bit is reset to 0 when the ADC14MEM22 register is read, */
                                                /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                /* ADC14CLRIFGR0 register is set to 1. */
      __I  uint32_t IFG23               	:	1;  /* ADC14MEM23 interrupt flag. This bit is set to 1 when ADC14MEM23 is loaded with a */
                                                /* conversion result. This bit is reset to 0 when the ADC14MEM23 register is read, */
                                                /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                /* ADC14CLRIFGR0 register is set to 1. */
      __I  uint32_t IFG24               	:	1;  /* ADC14MEM24 interrupt flag. This bit is set to 1 when ADC14MEM24 is loaded with a */
                                                /* conversion result. This bit is reset to 0 when the ADC14MEM24 register is read, */
                                                /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                /* ADC14CLRIFGR0 register is set to 1. */
      __I  uint32_t IFG25               	:	1;  /* ADC14MEM25 interrupt flag. This bit is set to 1 when ADC14MEM25 is loaded with a */
                                                /* conversion result. This bit is reset to 0 when the ADC14MEM25 register is read, */
                                                /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                /* ADC14CLRIFGR0 register is set to 1. */
      __I  uint32_t IFG26               	:	1;  /* ADC14MEM26 interrupt flag. This bit is set to 1 when ADC14MEM26 is loaded with a */
                                                /* conversion result.This bit is reset to 0 when the ADC14MEM26 register is read,  */
                                                /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                /* ADC14CLRIFGR0 register is set to 1. */
      __I  uint32_t IFG27               	:	1;  /* ADC14MEM27 interrupt flag. This bit is set to 1 when ADC14MEM27 is loaded with a */
                                                /* conversion result. This bit is reset to 0 when the ADC14MEM27 register is read, */
                                                /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                /* ADC14CLRIFGR0 register is set to 1. */
      __I  uint32_t IFG28               	:	1;  /* ADC14MEM28 interrupt flag. This bit is set to 1 when ADC14MEM28 is loaded with a */
                                                /* conversion result. This bit is reset to 0 when the ADC14MEM28 register is read, */
                                                /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                /* ADC14CLRIFGR0 register is set to 1.  */
      __I  uint32_t IFG29               	:	1;  /* ADC14MEM29 interrupt flag. This bit is set to 1 when ADC14MEM29 is loaded with a */
                                                /* conversion result. This bit is reset to 0 when the ADC14MEM29 register is read, */
                                                /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                /* ADC14CLRIFGR0 register is set to 1. */
      __I  uint32_t IFG30               	:	1;  /* ADC14MEM30 interrupt flag. This bit is set to 1 when ADC14MEM30 is loaded with a */
                                                /* conversion result. This bit is reset to 0 when the ADC14MEM30 register is read, */
                                                /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                /* ADC14CLRIFGR0 register is set to 1. */
      __I  uint32_t IFG31               	:	1;  /* ADC14MEM31 interrupt flag. This bit is set to 1 when ADC14MEM31 is loaded with a */
                                                /* conversion result. This bit is reset to 0 when the ADC14MEM31 register is read, */
                                                /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                /* ADC14CLRIFGR0 register is set to 1. 0b = No interrupt pending  */

    } bit;
  } IFGR0;
  union {    /* ADC14_IFGR1 Register */
    __IO uint32_t reg;
    struct { /* ADC14_IFGR1 Control Bits */
      __I  uint32_t RESERVED_2          	:	1;  /* Reserved */
      __I  uint32_t INIFG               	:	1;  /* Interrupt flag for the ADC14MEMx result register being greater than the ADC14LO  */
                                                /* threshold and below the ADC14HI threshold interrupt. This bit is reset to 0 by  */
                                                /* IV register read or when corresponding bit in CLRIFGR1 is set to 1. */
      __I  uint32_t LOIFG               	:	1;  /* comparator for the ADC14MEMx result register. This bit is reset to 0 by IV  */
                                                /* register read or when corresponding bit in CLRIFGR1 is set to 1. */
      __I  uint32_t HIIFG               	:	1;  /* Interrupt flag for exceeding the upper limit interrupt of the window comparator  */
                                                /* for ADC14MEMx result register. This bit is reset to 0 by IV register read or  */
                                                /* when corresponding bit in CLRIFGR1 is set to 1. */
      __I  uint32_t OVIFG               	:	1;  /*  ADC14MEMx overflow interrupt flag. This bit is reset to 0 by IV register read  */
                                                /* or when corresponding bit in CLRIFGR1 is set to 1. */
      __I  uint32_t TOVIFG              	:	1;  /* ADC14 conversion time overflow interrupt flag. This bit is reset to 0 by IV  */
                                                /* register read or when corresponding bit in CLRIFGR1 is set to 1. */
      __I  uint32_t RDYIFG              	:	1;  /* ADC14 local buffered reference ready interrupt flag. This bit is reset to 0  by  */
                                                /* IV register read () or when corresponding bit in CLRIFGR1 is set to 1. */
      __I  uint32_t RESERVED_1          	:	25;  /* Reserved */

    } bit;
  } IFGR1;
  union {    /* ADC14_CLRIFGR0 Register */
    __IO uint32_t reg;
    struct { /* ADC14_CLRIFGR0 Control Bits */
      __O  uint32_t CLRADC14IFG0        	:	1;  /* clear ADC14IFG0 */
      __O  uint32_t CLRADC14IFG1        	:	1;  /* clear ADC14IFG1 */
      __O  uint32_t CLRADC14IFG2        	:	1;  /* clear ADC14IFG2 */
      __O  uint32_t CLRADC14IFG3        	:	1;  /* clear ADC14IFG3 */
      __O  uint32_t CLRADC14IFG4        	:	1;  /* clear ADC14IFG4 */
      __O  uint32_t CLRADC14IFG5        	:	1;  /* clear ADC14IFG5 */
      __O  uint32_t CLRADC14IFG6        	:	1;  /* clear ADC14IFG6 */
      __O  uint32_t CLRADC14IFG7        	:	1;  /* clear ADC14IFG7 */
      __O  uint32_t CLRADC14IFG8        	:	1;  /* clear ADC14IFG8 */
      __O  uint32_t CLRADC14IFG9        	:	1;  /* clear ADC14IFG9 */
      __O  uint32_t CLRADC14IFG10       	:	1;  /* clear ADC14IFG10 */
      __O  uint32_t CLRADC14IFG11       	:	1;  /* clear ADC14IFG11 */
      __O  uint32_t CLRADC14IFG12       	:	1;  /* clear ADC14IFG12 */
      __O  uint32_t CLRADC14IFG13       	:	1;  /* clear ADC14IFG13 */
      __O  uint32_t CLRADC14IFG14       	:	1;  /* clear ADC14IFG14 */
      __O  uint32_t CLRADC14IFG15       	:	1;  /* clear ADC14IFG15 */
      __O  uint32_t CLRADC14IFG16       	:	1;  /* clear ADC14IFG16 */
      __O  uint32_t CLRADC14IFG17       	:	1;  /* clear ADC14IFG17 */
      __O  uint32_t CLRADC14IFG18       	:	1;  /* clear ADC14IFG18 */
      __O  uint32_t CLRADC14IFG19       	:	1;  /* clear ADC14IFG19 */
      __O  uint32_t CLRADC14IFG20       	:	1;  /* clear DC12IFG20 */
      __O  uint32_t CLRADC14IFG21       	:	1;  /* clear ADC14IFG21   */
      __O  uint32_t CLRADC14IFG22       	:	1;  /* clear ADC14IFG22 */
      __O  uint32_t CLRADC14IFG23       	:	1;  /* clear ADC14IFG23 */
      __O  uint32_t CLRADC14IFG24       	:	1;  /* clear ADC14IFG24 */
      __O  uint32_t CLRADC14IFG25       	:	1;  /* clear ADC14IFG25 */
      __O  uint32_t CLRADC14IFG26       	:	1;  /* clear ADC14IFG26   */
      __O  uint32_t CLRADC14IFG27       	:	1;  /* clear ADC14IFG27 */
      __O  uint32_t CLRADC14IFG28       	:	1;  /* clear ADC14IFG28 */
      __O  uint32_t CLRADC14IFG29       	:	1;  /* clear ADC14IFG29 */
      __O  uint32_t CLRADC14IFG30       	:	1;  /* clear ADC14IFG30 */
      __O  uint32_t CLRADC14IFG31       	:	1;  /* clear ADC14IFG31 */

    } bit;
  } CLRIFGR0;
  union {    /* ADC14_CLRIFGR1 Register */
    __IO uint32_t reg;
    struct { /* ADC14_CLRIFGR1 Control Bits */
      __I  uint32_t RESERVED_2          	:	1;  /* Reserved */
      __O  uint32_t CLRADC14INIFG       	:	1;  /* clear ADC14INIFG */
      __O  uint32_t CLRADC14LOIFG       	:	1;  /* clear ADC14LOIFG */
      __O  uint32_t CLRADC14HIIFG       	:	1;  /* clear ADC14HIIFG */
      __O  uint32_t CLRADC14OVIFG       	:	1;  /* clear ADC14OVIFG */
      __O  uint32_t CLRADC14TOVIFG      	:	1;  /* clear ADC14TOVIFG   */
      __O  uint32_t CLRADC14RDYIFG      	:	1;  /* clear ADC14RDYIFG */
      __I  uint32_t RESERVED_1          	:	25;  /* Reserved */

    } bit;
  } CLRIFGR1;
  uint32_t IV; /* ADC14 Interrupt Vector */
} ADC14_Type;


/* ADC14_CTL0 Control Bits */
#define ADC14_CTL0_SC                                      (0x00000001)          /* ADC14 start conversion. Software-controlled sample-and-conversion start.ADC14SC  */
                                                                                 /* and ADC14ENC may be set together with one instruction. ADC14SC is reset  */
                                                                                 /* automatically. */
#define ADC14_CTL0_ENC                                     (0x00000002)          /* ADC14 enable conversion ADC14ENC low pulse width must be at-least 3 ADC14CLK  */
                                                                                 /* cycles. */
#define ADC14_CTL0_RESERVED_2__M                           (0x0000000c)          /* Reserved. Always reads as 0. */
#define ADC14_CTL0_ON                                      (0x00000010)          /* ADC14 on */
#define ADC14_CTL0_RESERVED_1__M                           (0x00000060)          /* Reserved. Always reads as 0. */
#define ADC14_CTL0_MSC                                     (0x00000080)          /* ADC14 multiple sample and conversion. Valid only for sequence or repeated modes. */
#define ADC14_CTL0_SHT0__M                                 (0x00000f00)          /* ADC14 sample-and-hold time. These bits define the number of ADC14CLK cycles in  */
                                                                                 /* the sampling period for registers ADC14MEM0 to ADC14MEM7 and ADC14MEM24 to  */
                                                                                 /* ADC14MEM31. Can be modified only when ADC14ENC = 0. 1000b to 1111b = Reserved */
#define ADC14_CTL0_SHT0__0                                 (0x00000000)          /* 4 */
#define ADC14_CTL0_SHT0__0_4                               (0x00000000)          /* 4 */
#define ADC14_CTL0_SHT0__1                                 (0x00000100)          /* 8 */
#define ADC14_CTL0_SHT0__1_8                               (0x00000100)          /* 8 */
#define ADC14_CTL0_SHT0__2                                 (0x00000200)          /* 16 */
#define ADC14_CTL0_SHT0__2_16                              (0x00000200)          /* 16 */
#define ADC14_CTL0_SHT0__3                                 (0x00000300)          /* 32 */
#define ADC14_CTL0_SHT0__3_32                              (0x00000300)          /* 32 */
#define ADC14_CTL0_SHT0__4                                 (0x00000400)          /* 64 */
#define ADC14_CTL0_SHT0__4_64                              (0x00000400)          /* 64 */
#define ADC14_CTL0_SHT0__5                                 (0x00000500)          /* 96 */
#define ADC14_CTL0_SHT0__5_96                              (0x00000500)          /* 96 */
#define ADC14_CTL0_SHT0__6                                 (0x00000600)          /* 128 */
#define ADC14_CTL0_SHT0__6_128                             (0x00000600)          /* 128 */
#define ADC14_CTL0_SHT0__7                                 (0x00000700)          /* 192 */
#define ADC14_CTL0_SHT0__7_192                             (0x00000700)          /* 192 */
#define ADC14_CTL0_SHT1__M                                 (0x0000f000)          /* ADC14 sample-and-hold time. These bits define the number of ADC14CLK cycles in  */
                                                                                 /* the sampling period for registers ADC14MEM8 to ADC14MEM23. Can be modified only  */
                                                                                 /* when ADC14ENC = 0. 1000b to 1111b = Reserved */
#define ADC14_CTL0_SHT1__0                                 (0x00000000)          /* 4 */
#define ADC14_CTL0_SHT1__0_4                               (0x00000000)          /* 4 */
#define ADC14_CTL0_SHT1__1                                 (0x00001000)          /* 8 */
#define ADC14_CTL0_SHT1__1_8                               (0x00001000)          /* 8 */
#define ADC14_CTL0_SHT1__2                                 (0x00002000)          /* 16 */
#define ADC14_CTL0_SHT1__2_16                              (0x00002000)          /* 16 */
#define ADC14_CTL0_SHT1__3                                 (0x00003000)          /* 32 */
#define ADC14_CTL0_SHT1__3_32                              (0x00003000)          /* 32 */
#define ADC14_CTL0_SHT1__4                                 (0x00004000)          /* 64 */
#define ADC14_CTL0_SHT1__4_64                              (0x00004000)          /* 64 */
#define ADC14_CTL0_SHT1__5                                 (0x00005000)          /* 96 */
#define ADC14_CTL0_SHT1__5_96                              (0x00005000)          /* 96 */
#define ADC14_CTL0_SHT1__6                                 (0x00006000)          /* 128 */
#define ADC14_CTL0_SHT1__6_128                             (0x00006000)          /* 128 */
#define ADC14_CTL0_SHT1__7                                 (0x00007000)          /* 192 */
#define ADC14_CTL0_SHT1__7_192                             (0x00007000)          /* 192 */
#define ADC14_CTL0_BUSY                                    (0x00010000)          /* ADC14 busy. This bit indicates an active sample or conversion operation. */
#define ADC14_CTL0_CONSEQ__M                               (0x00060000)          /* ADC14 conversion sequence mode select Can be modified only when ADC14ENC=0. */
#define ADC14_CTL0_CONSEQ__0                               (0x00000000)          /* Single-channel, single-conversion */
#define ADC14_CTL0_CONSEQ__1                               (0x00020000)          /* Sequence-of-channels */
#define ADC14_CTL0_CONSEQ__2                               (0x00040000)          /* Repeat-single-channel */
#define ADC14_CTL0_CONSEQ__3                               (0x00060000)          /* Repeat-sequence-of-channels */
#define ADC14_CTL0_SSEL__M                                 (0x00380000)          /* ADC14 clock source select. Can be modified only when ADC14ENC = 0. */
#define ADC14_CTL0_SSEL__0                                 (0x00000000)          /* MODCLK */
#define ADC14_CTL0_SSEL__0_MODCLK                          (0x00000000)          /* MODCLK */
#define ADC14_CTL0_SSEL__1                                 (0x00080000)          /* SYSCLK */
#define ADC14_CTL0_SSEL__1_SYSCLK                          (0x00080000)          /* SYSCLK */
#define ADC14_CTL0_SSEL__2                                 (0x00100000)          /* ACLK */
#define ADC14_CTL0_SSEL__2_ACLK                            (0x00100000)          /* ACLK */
#define ADC14_CTL0_SSEL__3                                 (0x00180000)          /* MCLK */
#define ADC14_CTL0_SSEL__3_MCLK                            (0x00180000)          /* MCLK */
#define ADC14_CTL0_SSEL__4                                 (0x00200000)          /* SMCLK */
#define ADC14_CTL0_SSEL__4_SMCLK                           (0x00200000)          /* SMCLK */
#define ADC14_CTL0_SSEL__5                                 (0x00280000)          /* HSMCLK */
#define ADC14_CTL0_SSEL__5_HSMCLK                          (0x00280000)          /* HSMCLK */
#define ADC14_CTL0_DIV__M                                  (0x01c00000)          /* ADC14 clock divider. Can be modified only when ADC14ENC = 0. */
#define ADC14_CTL0_DIV__0                                  (0x00000000)          /* /1 */
#define ADC14_CTL0_DIV__0__1                               (0x00000000)          /* /1 */
#define ADC14_CTL0_DIV__1                                  (0x00400000)          /* /2 */
#define ADC14_CTL0_DIV__1__2                               (0x00400000)          /* /2 */
#define ADC14_CTL0_DIV__2                                  (0x00800000)          /* /3 */
#define ADC14_CTL0_DIV__2__3                               (0x00800000)          /* /3 */
#define ADC14_CTL0_DIV__3                                  (0x00c00000)          /* /4 */
#define ADC14_CTL0_DIV__3__4                               (0x00c00000)          /* /4 */
#define ADC14_CTL0_DIV__4                                  (0x01000000)          /* /5 */
#define ADC14_CTL0_DIV__4__5                               (0x01000000)          /* /5 */
#define ADC14_CTL0_DIV__5                                  (0x01400000)          /* /6 */
#define ADC14_CTL0_DIV__5__6                               (0x01400000)          /* /6 */
#define ADC14_CTL0_DIV__6                                  (0x01800000)          /* /7 */
#define ADC14_CTL0_DIV__6__7                               (0x01800000)          /* /7 */
#define ADC14_CTL0_DIV__7                                  (0x01c00000)          /* /8 */
#define ADC14_CTL0_DIV__7__8                               (0x01c00000)          /* /8 */
#define ADC14_CTL0_ISSH                                    (0x02000000)          /* ADC14 invert signal sample-and-hold. Can be modified only when ADC14ENC = 0.  */
                                                                                 /* Setting ADC14ISSH=1 and triggering a conversion using ADC14SC is not recommended */
                                                                                 /* . ADC14SC bit gets reset to 0 automatically at the end of conversion and if  */
                                                                                 /* ADC14ISSH=1, the 1->0 transition on ADC14SC will trigger another conversion. */
#define ADC14_CTL0_SHP                                     (0x04000000)          /* ADC14 sample-and-hold pulse-mode select. This bit selects the source of the  */
                                                                                 /* sampling signal (SAMPCON) to be either the output of the sampling timer or the  */
                                                                                 /* sample-input signal directly. Can be modified only when ADC14ENC = 0. */
#define ADC14_CTL0_SHS__M                                  (0x38000000)          /* ADC14 sample-and-hold source select. Can be modified only when ADC14ENC = 0. */
#define ADC14_CTL0_SHS__0                                  (0x00000000)          /* ADC14SC bit */
#define ADC14_CTL0_SHS__0_ADC14SC_BIT                      (0x00000000)          /* ADC14SC bit */
#define ADC14_CTL0_SHS__1                                  (0x08000000)          /* See device-specific data sheet for source */
#define ADC14_CTL0_SHS__2                                  (0x10000000)          /* See device-specific data sheet for source */
#define ADC14_CTL0_SHS__3                                  (0x18000000)          /* See device-specific data sheet for source */
#define ADC14_CTL0_SHS__4                                  (0x20000000)          /* See device-specific data sheet for source */
#define ADC14_CTL0_SHS__5                                  (0x28000000)          /* See device-specific data sheet for source */
#define ADC14_CTL0_SHS__6                                  (0x30000000)          /* See device-specific data sheet for source */
#define ADC14_CTL0_SHS__7                                  (0x38000000)          /* See device-specific data sheet for source */
#define ADC14_CTL0_PDIV__M                                 (0xc0000000)          /* ADC14 predivider. This bit predivides the selected ADC14 clock source. Can be  */
                                                                                 /* modified only when ADC14ENC = 0. */
#define ADC14_CTL0_PDIV__0                                 (0x00000000)          /* Predivide by 1 */
#define ADC14_CTL0_PDIV__0_PREDIVIDE_BY_1                  (0x00000000)          /* Predivide by 1 */
#define ADC14_CTL0_PDIV__1                                 (0x40000000)          /* Predivide by 4 */
#define ADC14_CTL0_PDIV__1_PREDIVIDE_BY_4                  (0x40000000)          /* Predivide by 4 */
#define ADC14_CTL0_PDIV__2                                 (0x80000000)          /* Predivide by 32 */
#define ADC14_CTL0_PDIV__2_PREDIVIDE_BY_32                 (0x80000000)          /* Predivide by 32 */
#define ADC14_CTL0_PDIV__3                                 (0xc0000000)          /* Predivide by 64 */
#define ADC14_CTL0_PDIV__3_PREDIVIDE_BY_64                 (0xc0000000)          /* Predivide by 64 */

/* ADC14_CTL1 Control Bits */
#define ADC14_CTL1_PWRMD__M                                (0x00000003)          /* ADC power modes. Can be modified only when ADC14ENC = 0. 01b = Reserved 11b =  */
                                                                                 /* Reserved   */
#define ADC14_CTL1_PWRMD__0                                (0x00000000)          /* Regular power mode for use with any resolution setting. Sample rate can be up to 1 Msps. */
#define ADC14_CTL1_PWRMD__2                                (0x00000002)          /* Low-power mode for 12-bit, 10-bit, and 8-bit resolution settings. Sample rate must not exceed 200 ksps. */
#define ADC14_CTL1_REFBURST                                (0x00000004)          /* ADC reference buffer burst. Can be modified only when ADC14ENC = 0. */
#define ADC14_CTL1_DF                                      (0x00000008)          /* ADC14 data read-back format. Data is always stored in the binary unsigned format */
                                                                                 /* . */
#define ADC14_CTL1_RES__M                                  (0x00000030)          /* ADC14 resolution. This bit defines the conversion result resolution. Can be  */
                                                                                 /* modified only when ADC14ENC = 0. */
#define ADC14_CTL1_RES__0                                  (0x00000000)          /* 8 bit (9 clock cycle conversion time) */
#define ADC14_CTL1_RES__1                                  (0x00000010)          /* 10 bit (11 clock cycle conversion time) */
#define ADC14_CTL1_RES__2                                  (0x00000020)          /* 12 bit (14 clock cycle conversion time) */
#define ADC14_CTL1_RES__3                                  (0x00000030)          /* 14 bit (16 clock cycle conversion time) */
#define ADC14_CTL1_RESERVED_3__M                           (0x0000ffc0)          /* Reserved. Always reads as 0. */
#define ADC14_CTL1_CSTARTADD__M                            (0x001f0000)          /* ADC14 conversion start address. These bits select which ADC14 conversion memory  */
                                                                                 /* register is used for a single conversion or for the first conversion in a  */
                                                                                 /* sequence. The value of CSTARTADDx is 0h to 1Fh, corresponding to ADC14MEM0 to  */
                                                                                 /* ADC14MEM31 */
#define ADC14_CTL1_RESERVED_2                              (0x00200000)          /* Reserved. Always reads as 0. */
#define ADC14_CTL1_BATMAP                                  (0x00400000)          /* Controls 1/2 AVCC ADC input channel selection */
#define ADC14_CTL1_TCMAP                                   (0x00800000)          /* Controls temperature sensor ADC input channel selection */
#define ADC14_CTL1_CH0MAP                                  (0x01000000)          /* Controls internal channel 0 selection to ADC input channel MAX-2 */
#define ADC14_CTL1_CH1MAP                                  (0x02000000)          /* Controls internal channel 1 selection to ADC input channel MAX-3 */
#define ADC14_CTL1_CH2MAP                                  (0x04000000)          /* Controls internal channel 2 selection to ADC input channel MAX-4    */
#define ADC14_CTL1_CH3MAP                                  (0x08000000)          /* Controls internal channel 3 selection to ADC input channel MAX-5    */
#define ADC14_CTL1_RESERVED_1__M                           (0xf0000000)          /* Reserved. Always reads as 0. */

/* ADC14_LO0 Control Bits */
#define ADC14_LO0_LO0__M                                   (0x0000ffff)          /* Low threshold 0. Can be modified only when ADC14ENC = 0. If ADC14DF = 0,  */
                                                                                 /* unsigned binary format:  The 14-bit threshold value must be right aligned. Bit  */
                                                                                 /* 13 is the MSB. Bits 15-14 are 0 in 14-bit mode, bits 15-12 are 0 in 12-bit mode, */
                                                                                 /* bits 15-10 are 0 in 10-bit mode, and bits 15-8 are 0 in 8-bit mode. The reset  */
                                                                                 /* value is: 0h If ADC14DF = 1, 2s-complement format:  The 14-bit threshold value  */
                                                                                 /* must be left aligned. Bit 15 is the MSB. Bits 1-0 are 0 in 14-bit mode, bits 3-0 */
                                                                                 /* are 0 in 12-bit mode, bits 5-0 are 0 in 10-bit mode, and bits 7-0 are 0 in 8- */
                                                                                 /* bit mode.  The reset value is: 8000h */
#define ADC14_LO0_RESERVED_1__M                            (0xffff0000)          /* Reserved. Always reads as 0. */

/* ADC14_HI0 Control Bits */
#define ADC14_HI0_HI0__M                                   (0x0000ffff)          /* High threshold 0. Can be modified only when ADC14ENC = 0. If ADC14DF = 0,  */
                                                                                 /* unsigned binary format:  The 14-bit threshold value must be right aligned. Bit  */
                                                                                 /* 13 is the MSB. Bits 15-14 are 0 in 14-bit mode, bits 15-12 are 0 in 12-bit mode, */
                                                                                 /* bits 15-10 are 0 in 10-bit mode, and bits 15-8 are 0 in 8-bit mode. The reset  */
                                                                                 /* value is: 3FFFh (14 bit), 0FFFh (12 bit), 03FFh (10 bit), or 00FFh (8 bit) If  */
                                                                                 /* ADC14DF = 1, 2s-complement format:  The 14-bit threshold value must be left  */
                                                                                 /* aligned. Bit 15 is the MSB. Bits 1-0 are 0 in 14-bit mode, bits 3-0 are 0 in 12- */
                                                                                 /* bit mode, bits 5-0 are 0 in 10-bit mode, and bits 7-0 are 0 in 8-bit mode.  The  */
                                                                                 /* reset value is: 7FFCh (14 bit), 7FF0h (12 bit), 7FC0h (10 bit), or 7F00h (8 bit) */
#define ADC14_HI0_RESERVED_1__M                            (0xffff0000)          /* Reserved. Always reads as 0. */

/* ADC14_LO1 Control Bits */
#define ADC14_LO1_LO1__M                                   (0x0000ffff)          /* Low threshold 1. Can be modified only when ADC14ENC = 0. If ADC14DF = 0,  */
                                                                                 /* unsigned binary format:  The 14-bit threshold value must be right aligned. Bit  */
                                                                                 /* 13 is the MSB. Bits 15-14 are 0 in 14-bit mode, bits 15-12 are 0 in 12-bit mode, */
                                                                                 /* bits 15-10 are 0 in 10-bit mode, and bits 15-8 are 0 in 8-bit mode. The reset  */
                                                                                 /* value is: 0h If ADC14DF = 1, 2s-complement format:  The 14-bit threshold value  */
                                                                                 /* must be left aligned. Bit 15 is the MSB. Bits 1-0 are 0 in 14-bit mode, bits 3-0 */
                                                                                 /* are 0 in 12-bit mode, bits 5-0 are 0 in 10-bit mode, and bits 7-0 are 0 in 8- */
                                                                                 /* bit mode.  The reset value is: 8000h */
#define ADC14_LO1_RESERVED_1__M                            (0xffff0000)          /* Reserved. Always reads as 0. */

/* ADC14_HI1 Control Bits */
#define ADC14_HI1_HI1__M                                   (0x0000ffff)          /* High threshold 1. Can be modified only when ADC14ENC = 0. If ADC14DF = 0,  */
                                                                                 /* unsigned binary format:  The 14-bit threshold value must be right aligned. Bit  */
                                                                                 /* 13 is the MSB. Bits 15-14 are 0 in 14-bit mode, bits 15-12 are 0 in 12-bit mode, */
                                                                                 /* bits 15-10 are 0 in 10-bit mode, and bits 15-8 are 0 in 8-bit mode. The reset  */
                                                                                 /* value is: 3FFFh (14 bit), 0FFFh (12 bit), 03FFh (10 bit), or 00FFh (8 bit) If  */
                                                                                 /* ADC14DF = 1, 2s-complement format:  The 14-bit threshold value must be left  */
                                                                                 /* aligned. Bit 15 is the MSB. Bits 1-0 are 0 in 14-bit mode, bits 3-0 are 0 in 12- */
                                                                                 /* bit mode, bits 5-0 are 0 in 10-bit mode, and bits 7-0 are 0 in 8-bit mode.  The  */
                                                                                 /* reset value is: 7FFCh (14 bit), 7FF0h (12 bit), 7FC0h (10 bit), or 7F00h (8 bit) */
#define ADC14_HI1_RESERVED_1__M                            (0xffff0000)          /* Reserved. Always reads as 0. */

/* ADC14_MCTL Control Bits */
#define ADC14_MCTL_INCH__M                                 (0x0000001f)          /* Input channel select. If even channels are set as differential then odd channel  */
                                                                                 /* configuration is ignored. Can be modified only when ADC14ENC = 0. */
#define ADC14_MCTL_INCH__0                                 (0x00000000)          /* If ADC14DIF = 0: A0; If ADC14DIF = 1: Ain+ = A0, Ain- = A1 */
#define ADC14_MCTL_INCH__1                                 (0x00000001)          /* If ADC14DIF = 0: A1; If ADC14DIF = 1: Ain+ = A0, Ain- = A1 */
#define ADC14_MCTL_INCH__2                                 (0x00000002)          /* If ADC14DIF = 0: A2; If ADC14DIF = 1: Ain+ = A2, Ain- = A3 */
#define ADC14_MCTL_INCH__3                                 (0x00000003)          /* If ADC14DIF = 0: A3; If ADC14DIF = 1: Ain+ = A2, Ain- = A3 */
#define ADC14_MCTL_INCH__4                                 (0x00000004)          /* If ADC14DIF = 0: A4; If ADC14DIF = 1: Ain+ = A4, Ain- = A5 */
#define ADC14_MCTL_INCH__5                                 (0x00000005)          /* If ADC14DIF = 0: A5; If ADC14DIF = 1: Ain+ = A4, Ain- = A5 */
#define ADC14_MCTL_INCH__6                                 (0x00000006)          /* If ADC14DIF = 0: A6; If ADC14DIF = 1: Ain+ = A6, Ain- = A7 */
#define ADC14_MCTL_INCH__7                                 (0x00000007)          /* If ADC14DIF = 0: A7; If ADC14DIF = 1: Ain+ = A6, Ain- = A7 */
#define ADC14_MCTL_INCH__8                                 (0x00000008)          /* If ADC14DIF = 0: A8; If ADC14DIF = 1: Ain+ = A8, Ain- = A9 */
#define ADC14_MCTL_INCH__9                                 (0x00000009)          /* If ADC14DIF = 0: A9; If ADC14DIF = 1: Ain+ = A8, Ain- = A9 */
#define ADC14_MCTL_INCH__10                                (0x0000000a)          /* If ADC14DIF = 0: A10; If ADC14DIF = 1: Ain+ = A10, Ain- = A11 */
#define ADC14_MCTL_INCH__11                                (0x0000000b)          /* If ADC14DIF = 0: A11; If ADC14DIF = 1: Ain+ = A10, Ain- = A11 */
#define ADC14_MCTL_INCH__12                                (0x0000000c)          /* If ADC14DIF = 0: A12; If ADC14DIF = 1: Ain+ = A12, Ain- = A13 */
#define ADC14_MCTL_INCH__13                                (0x0000000d)          /* If ADC14DIF = 0: A13; If ADC14DIF = 1: Ain+ = A12, Ain- = A13 */
#define ADC14_MCTL_INCH__14                                (0x0000000e)          /* If ADC14DIF = 0: A14; If ADC14DIF = 1: Ain+ = A14, Ain- = A15 */
#define ADC14_MCTL_INCH__15                                (0x0000000f)          /* If ADC14DIF = 0: A15; If ADC14DIF = 1: Ain+ = A14, Ain- = A15 */
#define ADC14_MCTL_INCH__16                                (0x00000010)          /* If ADC14DIF = 0: A16; If ADC14DIF = 1: Ain+ = A16, Ain- = A17 */
#define ADC14_MCTL_INCH__17                                (0x00000011)          /* If ADC14DIF = 0: A17; If ADC14DIF = 1: Ain+ = A16, Ain- = A17 */
#define ADC14_MCTL_INCH__18                                (0x00000012)          /* If ADC14DIF = 0: A18; If ADC14DIF = 1: Ain+ = A18, Ain- = A19 */
#define ADC14_MCTL_INCH__19                                (0x00000013)          /* If ADC14DIF = 0: A19; If ADC14DIF = 1: Ain+ = A18, Ain- = A19 */
#define ADC14_MCTL_INCH__20                                (0x00000014)          /* If ADC14DIF = 0: A20; If ADC14DIF = 1: Ain+ = A20, Ain- = A21 */
#define ADC14_MCTL_INCH__21                                (0x00000015)          /* If ADC14DIF = 0: A21; If ADC14DIF = 1: Ain+ = A20, Ain- = A21 */
#define ADC14_MCTL_INCH__22                                (0x00000016)          /* If ADC14DIF = 0: A22; If ADC14DIF = 1: Ain+ = A22, Ain- = A23 */
#define ADC14_MCTL_INCH__23                                (0x00000017)          /* If ADC14DIF = 0: A23; If ADC14DIF = 1: Ain+ = A22, Ain- = A23 */
#define ADC14_MCTL_INCH__24                                (0x00000018)          /* If ADC14DIF = 0: A24; If ADC14DIF = 1: Ain+ = A24, Ain- = A25 */
#define ADC14_MCTL_INCH__25                                (0x00000019)          /* If ADC14DIF = 0: A25; If ADC14DIF = 1: Ain+ = A24, Ain- = A25 */
#define ADC14_MCTL_INCH__26                                (0x0000001a)          /* If ADC14DIF = 0: A26; If ADC14DIF = 1: Ain+ = A26, Ain- = A27 */
#define ADC14_MCTL_INCH__27                                (0x0000001b)          /* If ADC14DIF = 0: A27; If ADC14DIF = 1: Ain+ = A26, Ain- = A27 */
#define ADC14_MCTL_INCH__28                                (0x0000001c)          /* If ADC14DIF = 0: A28; If ADC14DIF = 1: Ain+ = A28, Ain- = A29 */
#define ADC14_MCTL_INCH__29                                (0x0000001d)          /* If ADC14DIF = 0: A29; If ADC14DIF = 1: Ain+ = A28, Ain- = A29 */
#define ADC14_MCTL_INCH__30                                (0x0000001e)          /* If ADC14DIF = 0: A30; If ADC14DIF = 1: Ain+ = A30, Ain- = A31 */
#define ADC14_MCTL_INCH__31                                (0x0000001f)          /* If ADC14DIF = 0: A31; If ADC14DIF = 1: Ain+ = A30, Ain- = A31 */
#define ADC14_MCTL_RESERVED_3__M                           (0x00000060)          /* Reserved. Always reads as 0. */
#define ADC14_MCTL_EOS                                     (0x00000080)          /* End of sequence. Indicates the last conversion in a sequence. Can be modified  */
                                                                                 /* only when ADC14ENC = 0. */
#define ADC14_MCTL_VRSEL__M                                (0x00000f00)          /* Selects combinations of V(R+) and V(R-) sources as well as the buffer selection  */
                                                                                 /* and buffer on or off. When REFOUT = 1, VeREF+ buffered configuration is not  */
                                                                                 /* available. Can be modified only when ADC14ENC = 0. 0010b to 1101b = Reserved It  */
                                                                                 /* is recommended to connect VeREF- to on-board ground when VeREF- is selected for  */
                                                                                 /* V(R-). */
#define ADC14_MCTL_VRSEL__0                                (0x00000000)          /* V(R+) = AVCC, V(R-) = AVSS */
#define ADC14_MCTL_VRSEL__1                                (0x00000100)          /* V(R+) = VREF buffered, V(R-) = AVSS */
#define ADC14_MCTL_VRSEL__14                               (0x00000e00)          /* V(R+) = VeREF+, V(R-) = VeREF- */
#define ADC14_MCTL_VRSEL__15                               (0x00000f00)          /* V(R+) = VeREF+ buffered, V(R-) = VeREF- */
#define ADC14_MCTL_RESERVED_2                              (0x00001000)          /* Reserved. Always reads as 0. */
#define ADC14_MCTL_DIF                                     (0x00002000)          /* Differential mode.  Can be modified only when ADC14ENC = 0. */
#define ADC14_MCTL_WINC                                    (0x00004000)          /* Comparator window enable Can be modified only when ADC14ENC = 0. */
#define ADC14_MCTL_WINCTH                                  (0x00008000)          /* Window comparator threshold register selection Can be modified only when  */
                                                                                 /* ADC14ENC = 0. */
#define ADC14_MCTL_RESERVED_1__M                           (0xffff0000)          /* Reserved. Always reads as 0. */

/* ADC14_MEM Control Bits */
#define ADC14_MEM_CONVERSION_RESULTS__M                    (0x0000ffff)          /* If ADC14DF = 0, unsigned binary: The 14-bit conversion results are right aligned */
                                                                                 /* . Bit 13 is the MSB. Bits 15-14 are 0 in 14-bit mode, bits 15-12 are 0 in 12-bit */
                                                                                 /* mode, bits 15-10 are 0 in 10-bit mode, and bits 15-8 are 0 in 8-bit mode. If  */
                                                                                 /* the user writes to the conversion memory registers, the results are corrupted.   */
                                                                                 /* If ADC14DF = 1, 2s-complement format: The 14-bit conversion results are left  */
                                                                                 /* aligned. Bit 15 is the MSB. Bits 1-0 are 0 in 14-bit mode, bits 3-0 are 0 in 12- */
                                                                                 /* bit mode, bits 5-0 are 0 in 10-bit mode, and bits 7-0 are 0 in 8-bit mode. The  */
                                                                                 /* data is stored in the right-justified format and is converted to the left- */
                                                                                 /* justified 2s-complement format during read back.  If the user writes to the  */
                                                                                 /* conversion memory registers, the results are corrupted.  Reading this register  */
                                                                                 /* clears the corresponding bit in ADC14IFG0. */
#define ADC14_MEM_RESERVED_1__M                            (0xffff0000)          /*  Reserved. */

/* ADC14_IER0 Control Bits */
#define ADC14_IER0_IE0                                     (0x00000001)          /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG0  */
                                                                                 /* bit. */
#define ADC14_IER0_IE1                                     (0x00000002)          /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG1  */
                                                                                 /* bit */
#define ADC14_IER0_IE2                                     (0x00000004)          /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG2  */
                                                                                 /* bit. */
#define ADC14_IER0_IE3                                     (0x00000008)          /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG3  */
                                                                                 /* bit. */
#define ADC14_IER0_IE4                                     (0x00000010)          /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG4  */
                                                                                 /* bit. */
#define ADC14_IER0_IE5                                     (0x00000020)          /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG5  */
                                                                                 /* bit. */
#define ADC14_IER0_IE6                                     (0x00000040)          /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG6  */
                                                                                 /* bit. */
#define ADC14_IER0_IE7                                     (0x00000080)          /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG7  */
                                                                                 /* bit. */
#define ADC14_IER0_IE8                                     (0x00000100)          /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG8  */
                                                                                 /* bit. */
#define ADC14_IER0_IE9                                     (0x00000200)          /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG9  */
                                                                                 /* bit. */
#define ADC14_IER0_IE10                                    (0x00000400)          /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG10  */
                                                                                 /* bit. */
#define ADC14_IER0_IE11                                    (0x00000800)          /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG11  */
                                                                                 /* bit. */
#define ADC14_IER0_IE12                                    (0x00001000)          /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG12  */
                                                                                 /* bit. */
#define ADC14_IER0_IE13                                    (0x00002000)          /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG13  */
                                                                                 /* bit. */
#define ADC14_IER0_IE14                                    (0x00004000)          /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG14  */
                                                                                 /* bit. */
#define ADC14_IER0_IE15                                    (0x00008000)          /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG15  */
                                                                                 /* bit. */
#define ADC14_IER0_IE16                                    (0x00010000)          /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG16  */
                                                                                 /* bit. */
#define ADC14_IER0_IE17                                    (0x00020000)          /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG17  */
                                                                                 /* bit. */
#define ADC14_IER0_IE18                                    (0x00040000)          /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG18  */
                                                                                 /* bit. */
#define ADC14_IER0_IE19                                    (0x00080000)          /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG19  */
                                                                                 /* bit. */
#define ADC14_IER0_IE20                                    (0x00100000)          /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG20  */
                                                                                 /* bit. */
#define ADC14_IER0_IE21                                    (0x00200000)          /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG21  */
                                                                                 /* bit. */
#define ADC14_IER0_IE22                                    (0x00400000)          /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG22  */
                                                                                 /* bit. */
#define ADC14_IER0_IE23                                    (0x00800000)          /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG23  */
                                                                                 /* bit. */
#define ADC14_IER0_IE24                                    (0x01000000)          /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG24  */
                                                                                 /* bit. */
#define ADC14_IER0_IE25                                    (0x02000000)          /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG25  */
                                                                                 /* bit. */
#define ADC14_IER0_IE26                                    (0x04000000)          /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG26  */
                                                                                 /* bit. */
#define ADC14_IER0_IE27                                    (0x08000000)          /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG27  */
                                                                                 /* bit. */
#define ADC14_IER0_IE28                                    (0x10000000)          /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG28  */
                                                                                 /* bit. */
#define ADC14_IER0_IE29                                    (0x20000000)          /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG29  */
                                                                                 /* bit. */
#define ADC14_IER0_IE30                                    (0x40000000)          /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG30  */
                                                                                 /* bit. */
#define ADC14_IER0_IE31                                    (0x80000000)          /* Interrupt enable. Enables or disables the interrupt request for the ADC14IFG31  */
                                                                                 /* bit. */

/* ADC14_IER1 Control Bits */
#define ADC14_IER1_RESERVED_2                              (0x00000001)          /* Reserved. Always reads as 0. */
#define ADC14_IER1_INIE                                    (0x00000002)          /* Interrupt enable for the ADC14MEMx result register being greater than the  */
                                                                                 /* ADC14LO threshold and below the ADC14HI threshold. */
#define ADC14_IER1_LOIE                                    (0x00000004)          /* Interrupt enable for the falling short of the lower limit interrupt of the  */
                                                                                 /* window comparator for the ADC14MEMx result register. */
#define ADC14_IER1_HIIE                                    (0x00000008)          /* Interrupt enable for the exceeding the upper limit interrupt of the window  */
                                                                                 /* comparator for ADC14MEMx result register. */
#define ADC14_IER1_OVIE                                    (0x00000010)          /* ADC14MEMx overflow-interrupt enable.  */
#define ADC14_IER1_TOVIE                                   (0x00000020)          /* ADC14 conversion-time-overflow interrupt enable.   */
#define ADC14_IER1_RDYIE                                   (0x00000040)          /* ADC14 local buffered reference ready interrupt enable.   */
#define ADC14_IER1_RESERVED_1__M                           (0xffffff80)          /* Reserved. Always reads as 0. */

/* ADC14_IFGR0 Control Bits */
#define ADC14_IFGR0_IFG0                                   (0x00000001)          /* ADC14MEM0 interrupt flag. This bit is set to 1 when ADC14MEM0 is loaded with a  */
                                                                                 /* conversion result. This bit is reset to 0 when the ADC14MEM0 register is read,  */
                                                                                 /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                                                 /* ADC14CLRIFGR0 register is set to 1. */
#define ADC14_IFGR0_IFG1                                   (0x00000002)          /* ADC14MEM1 interrupt flag. This bit is set to 1 when ADC14MEM1 is loaded with a  */
                                                                                 /* conversion result. This bit is reset to 0 when the ADC14MEM1 register is read,  */
                                                                                 /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                                                 /* ADC14CLRIFGR0 register is set to 1. */
#define ADC14_IFGR0_IFG2                                   (0x00000004)          /* ADC14MEM2 interrupt flag. This bit is set to 1 when ADC14MEM2 is loaded with a  */
                                                                                 /* conversion result. This bit is reset to 0 when the ADC14MEM2 register is read,  */
                                                                                 /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                                                 /* ADC14CLRIFGR0 register is set to 1. */
#define ADC14_IFGR0_IFG3                                   (0x00000008)          /* ADC14MEM3 interrupt flag. This bit is set to 1 when ADC14MEM3 is loaded with a  */
                                                                                 /* conversion result. This bit is reset to 0 when the ADC14MEM3 register is read,  */
                                                                                 /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                                                 /* ADC14CLRIFGR0 register is set to 1. */
#define ADC14_IFGR0_IFG4                                   (0x00000010)          /* ADC14MEM4 interrupt flag. This bit is set to 1 when ADC14MEM4 is loaded with a  */
                                                                                 /* conversion result. This bit is reset to 0 when the ADC14MEM4 register is read,  */
                                                                                 /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                                                 /* ADC14CLRIFGR0 register is set to 1. */
#define ADC14_IFGR0_IFG5                                   (0x00000020)          /* ADC14MEM5 interrupt flag. This bit is set to 1 when ADC14MEM5 is loaded with a  */
                                                                                 /* conversion result. This bit is reset to 0 when the ADC14MEM5 register is read,  */
                                                                                 /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                                                 /* ADC14CLRIFGR0 register is set to 1. */
#define ADC14_IFGR0_IFG6                                   (0x00000040)          /* ADC14MEM6 interrupt flag. This bit is set to 1 when ADC14MEM6 is loaded with a  */
                                                                                 /* conversion result. This bit is reset to 0 when the ADC14MEM6 register is read,  */
                                                                                 /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                                                 /* ADC14CLRIFGR0 register is set to 1. */
#define ADC14_IFGR0_IFG7                                   (0x00000080)          /* ADC14MEM7 interrupt flag. This bit is set to 1 when ADC14MEM7 is loaded with a  */
                                                                                 /* conversion result. This bit is reset to 0 when the ADC14MEM7 register is read,  */
                                                                                 /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                                                 /* ADC14CLRIFGR0 register is set to 1.. */
#define ADC14_IFGR0_IFG8                                   (0x00000100)          /* ADC14MEM8 interrupt flag. This bit is set to 1 when ADC14MEM8 is loaded with a  */
                                                                                 /* conversion result. This bit is reset to 0 when the ADC14MEM8 register is read,  */
                                                                                 /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                                                 /* ADC14CLRIFGR0 register is set to 1. */
#define ADC14_IFGR0_IFG9                                   (0x00000200)          /* ADC14MEM9 interrupt flag. This bit is set to 1 when ADC14MEM9 is loaded with a  */
                                                                                 /* conversion result. This bit is reset to 0 when the ADC14MEM9 register is read,  */
                                                                                 /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                                                 /* ADC14CLRIFGR0 register is set to  1. */
#define ADC14_IFGR0_IFG10                                  (0x00000400)          /* ADC14MEM10 interrupt flag. This bit is set to 1 when ADC14MEM10 is loaded with a */
                                                                                 /* conversion result. This bit is reset to 0 when the ADC14MEM10 register is read, */
                                                                                 /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                                                 /* ADC14CLRIFGR0 register is set to 1. */
#define ADC14_IFGR0_IFG11                                  (0x00000800)          /* ADC14MEM11 interrupt flag. This bit is set to 1 when ADC14MEM11 is loaded with a */
                                                                                 /* conversion result. This bit is reset to 0 when the ADC14MEM11 register is read, */
                                                                                 /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                                                 /* ADC14CLRIFGR0 register is set to 1. */
#define ADC14_IFGR0_IFG12                                  (0x00001000)          /* ADC14MEM12 interrupt flag. This bit is set to 1 when ADC14MEM12 is loaded with a */
                                                                                 /* conversion result. This bit is reset to 0 when the ADC14MEM12 register is read, */
                                                                                 /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                                                 /* ADC14CLRIFGR0 register is set to 1. */
#define ADC14_IFGR0_IFG13                                  (0x00002000)          /* ADC14MEM13 interrupt flag. This bit is set to 1 when ADC14MEM13 is loaded with a */
                                                                                 /* conversion result. This bit is reset to 0 when the ADC14MEM13 register is read, */
                                                                                 /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                                                 /* ADC14CLRIFGR0 register is set to 1. */
#define ADC14_IFGR0_IFG14                                  (0x00004000)          /* ADC14MEM14 interrupt flag. This bit is set to 1 when ADC14MEM14 is loaded with a */
                                                                                 /* conversion result. This bit is reset to 0 when the ADC14MEM14 register is read, */
                                                                                 /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                                                 /* ADC14CLRIFGR0 register is set to 1. */
#define ADC14_IFGR0_IFG15                                  (0x00008000)          /* ADC14MEM15 interrupt flag. This bit is set to 1 when ADC14MEM15 is loaded with a */
                                                                                 /* conversion result. This bit is reset to 0 when the ADC14MEM15 register is read, */
                                                                                 /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                                                 /* ADC14CLRIFGR0 register is set to 1. */
#define ADC14_IFGR0_IFG16                                  (0x00010000)          /* ADC14MEM16 interrupt flag. This bit is set to 1 when ADC14MEM16 is loaded with a */
                                                                                 /* conversion result. This bit is reset to 0 when the ADC14MEM16 register is read, */
                                                                                 /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                                                 /* ADC14CLRIFGR0 register is set to 1. */
#define ADC14_IFGR0_IFG17                                  (0x00020000)          /* ADC14MEM17 interrupt flag. This bit is set to 1 when ADC14MEM17 is loaded with a */
                                                                                 /* conversion result. This bit is reset to 0 when the ADC14MEM17 register is read, */
                                                                                 /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                                                 /* ADC14CLRIFGR0 register is set to 1. */
#define ADC14_IFGR0_IFG18                                  (0x00040000)          /* ADC14MEM18 interrupt flag. This bit is set to 1 when ADC14MEM18 is loaded with a */
                                                                                 /* conversion result. This bit is reset to 0 when the ADC14MEM18 register is read, */
                                                                                 /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                                                 /* ADC14CLRIFGR0 register is set to 1. */
#define ADC14_IFGR0_IFG19                                  (0x00080000)          /* ADC14MEM19 interrupt flag. This bit is set to 1 when ADC14MEM19 is loaded with a */
                                                                                 /* conversion result. This bit is reset to 0 when the ADC14MEM19 register is read, */
                                                                                 /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                                                 /* ADC14CLRIFGR0 register is set to 1. */
#define ADC14_IFGR0_IFG20                                  (0x00100000)          /* ADC14MEM20 interrupt flag. This bit is set to 1 when ADC14MEM20 is loaded with a */
                                                                                 /* conversion result. This bit is reset to 0 when the ADC14MEM20 register is read, */
                                                                                 /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                                                 /* ADC14CLRIFGR0 register is set to 1. */
#define ADC14_IFGR0_IFG21                                  (0x00200000)          /* ADC14MEM21 interrupt flag. This bit is set to 1 when ADC14MEM21 is loaded with a */
                                                                                 /* conversion result. This bit is reset to 0 when the ADC14MEM21 register is read, */
                                                                                 /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                                                 /* ADC14CLRIFGR0 register is set to 1. */
#define ADC14_IFGR0_IFG22                                  (0x00400000)          /* ADC14MEM22 interrupt flag. This bit is set to 1 when ADC14MEM22 is loaded with a */
                                                                                 /* conversion result. This bit is reset to 0 when the ADC14MEM22 register is read, */
                                                                                 /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                                                 /* ADC14CLRIFGR0 register is set to 1. */
#define ADC14_IFGR0_IFG23                                  (0x00800000)          /* ADC14MEM23 interrupt flag. This bit is set to 1 when ADC14MEM23 is loaded with a */
                                                                                 /* conversion result. This bit is reset to 0 when the ADC14MEM23 register is read, */
                                                                                 /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                                                 /* ADC14CLRIFGR0 register is set to 1. */
#define ADC14_IFGR0_IFG24                                  (0x01000000)          /* ADC14MEM24 interrupt flag. This bit is set to 1 when ADC14MEM24 is loaded with a */
                                                                                 /* conversion result. This bit is reset to 0 when the ADC14MEM24 register is read, */
                                                                                 /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                                                 /* ADC14CLRIFGR0 register is set to 1. */
#define ADC14_IFGR0_IFG25                                  (0x02000000)          /* ADC14MEM25 interrupt flag. This bit is set to 1 when ADC14MEM25 is loaded with a */
                                                                                 /* conversion result. This bit is reset to 0 when the ADC14MEM25 register is read, */
                                                                                 /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                                                 /* ADC14CLRIFGR0 register is set to 1. */
#define ADC14_IFGR0_IFG26                                  (0x04000000)          /* ADC14MEM26 interrupt flag. This bit is set to 1 when ADC14MEM26 is loaded with a */
                                                                                 /* conversion result.This bit is reset to 0 when the ADC14MEM26 register is read,  */
                                                                                 /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                                                 /* ADC14CLRIFGR0 register is set to 1. */
#define ADC14_IFGR0_IFG27                                  (0x08000000)          /* ADC14MEM27 interrupt flag. This bit is set to 1 when ADC14MEM27 is loaded with a */
                                                                                 /* conversion result. This bit is reset to 0 when the ADC14MEM27 register is read, */
                                                                                 /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                                                 /* ADC14CLRIFGR0 register is set to 1. */
#define ADC14_IFGR0_IFG28                                  (0x10000000)          /* ADC14MEM28 interrupt flag. This bit is set to 1 when ADC14MEM28 is loaded with a */
                                                                                 /* conversion result. This bit is reset to 0 when the ADC14MEM28 register is read, */
                                                                                 /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                                                 /* ADC14CLRIFGR0 register is set to 1.  */
#define ADC14_IFGR0_IFG29                                  (0x20000000)          /* ADC14MEM29 interrupt flag. This bit is set to 1 when ADC14MEM29 is loaded with a */
                                                                                 /* conversion result. This bit is reset to 0 when the ADC14MEM29 register is read, */
                                                                                 /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                                                 /* ADC14CLRIFGR0 register is set to 1. */
#define ADC14_IFGR0_IFG30                                  (0x40000000)          /* ADC14MEM30 interrupt flag. This bit is set to 1 when ADC14MEM30 is loaded with a */
                                                                                 /* conversion result. This bit is reset to 0 when the ADC14MEM30 register is read, */
                                                                                 /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                                                 /* ADC14CLRIFGR0 register is set to 1. */
#define ADC14_IFGR0_IFG31                                  (0x80000000)          /* ADC14MEM31 interrupt flag. This bit is set to 1 when ADC14MEM31 is loaded with a */
                                                                                 /* conversion result. This bit is reset to 0 when the ADC14MEM31 register is read, */
                                                                                 /* or when the ADC14IV register is read, or when the corresponding bit in the  */
                                                                                 /* ADC14CLRIFGR0 register is set to 1. 0b = No interrupt pending  */

/* ADC14_IFGR1 Control Bits */
#define ADC14_IFGR1_RESERVED_2                             (0x00000001)          /* Reserved. Always reads as 0. */
#define ADC14_IFGR1_INIFG                                  (0x00000002)          /* Interrupt flag for the ADC14MEMx result register being greater than the ADC14LO  */
                                                                                 /* threshold and below the ADC14HI threshold interrupt. This bit is reset to 0 by  */
                                                                                 /* IV register read or when corresponding bit in CLRIFGR1 is set to 1. */
#define ADC14_IFGR1_LOIFG                                  (0x00000004)          /* comparator for the ADC14MEMx result register. This bit is reset to 0 by IV  */
                                                                                 /* register read or when corresponding bit in CLRIFGR1 is set to 1. */
#define ADC14_IFGR1_HIIFG                                  (0x00000008)          /* Interrupt flag for exceeding the upper limit interrupt of the window comparator  */
                                                                                 /* for ADC14MEMx result register. This bit is reset to 0 by IV register read or  */
                                                                                 /* when corresponding bit in CLRIFGR1 is set to 1. */
#define ADC14_IFGR1_OVIFG                                  (0x00000010)          /*  ADC14MEMx overflow interrupt flag. This bit is reset to 0 by IV register read  */
                                                                                 /* or when corresponding bit in CLRIFGR1 is set to 1. */
#define ADC14_IFGR1_TOVIFG                                 (0x00000020)          /* ADC14 conversion time overflow interrupt flag. This bit is reset to 0 by IV  */
                                                                                 /* register read or when corresponding bit in CLRIFGR1 is set to 1. */
#define ADC14_IFGR1_RDYIFG                                 (0x00000040)          /* ADC14 local buffered reference ready interrupt flag. This bit is reset to 0  by  */
                                                                                 /* IV register read () or when corresponding bit in CLRIFGR1 is set to 1. */
#define ADC14_IFGR1_RESERVED_1__M                          (0xffffff80)          /* Reserved. Always reads as 0. */

/* ADC14_CLRIFGR0 Control Bits */
#define ADC14_CLRIFGR0_CLRADC14IFG0                        (0x00000001)          /* clear ADC14IFG0 */
#define ADC14_CLRIFGR0_CLRADC14IFG1                        (0x00000002)          /* clear ADC14IFG1 */
#define ADC14_CLRIFGR0_CLRADC14IFG2                        (0x00000004)          /* clear ADC14IFG2 */
#define ADC14_CLRIFGR0_CLRADC14IFG3                        (0x00000008)          /* clear ADC14IFG3 */
#define ADC14_CLRIFGR0_CLRADC14IFG4                        (0x00000010)          /* clear ADC14IFG4 */
#define ADC14_CLRIFGR0_CLRADC14IFG5                        (0x00000020)          /* clear ADC14IFG5 */
#define ADC14_CLRIFGR0_CLRADC14IFG6                        (0x00000040)          /* clear ADC14IFG6 */
#define ADC14_CLRIFGR0_CLRADC14IFG7                        (0x00000080)          /* clear ADC14IFG7 */
#define ADC14_CLRIFGR0_CLRADC14IFG8                        (0x00000100)          /* clear ADC14IFG8 */
#define ADC14_CLRIFGR0_CLRADC14IFG9                        (0x00000200)          /* clear ADC14IFG9 */
#define ADC14_CLRIFGR0_CLRADC14IFG10                       (0x00000400)          /* clear ADC14IFG10 */
#define ADC14_CLRIFGR0_CLRADC14IFG11                       (0x00000800)          /* clear ADC14IFG11 */
#define ADC14_CLRIFGR0_CLRADC14IFG12                       (0x00001000)          /* clear ADC14IFG12 */
#define ADC14_CLRIFGR0_CLRADC14IFG13                       (0x00002000)          /* clear ADC14IFG13 */
#define ADC14_CLRIFGR0_CLRADC14IFG14                       (0x00004000)          /* clear ADC14IFG14 */
#define ADC14_CLRIFGR0_CLRADC14IFG15                       (0x00008000)          /* clear ADC14IFG15 */
#define ADC14_CLRIFGR0_CLRADC14IFG16                       (0x00010000)          /* clear ADC14IFG16 */
#define ADC14_CLRIFGR0_CLRADC14IFG17                       (0x00020000)          /* clear ADC14IFG17 */
#define ADC14_CLRIFGR0_CLRADC14IFG18                       (0x00040000)          /* clear ADC14IFG18 */
#define ADC14_CLRIFGR0_CLRADC14IFG19                       (0x00080000)          /* clear ADC14IFG19 */
#define ADC14_CLRIFGR0_CLRADC14IFG20                       (0x00100000)          /* clear DC12IFG20 */
#define ADC14_CLRIFGR0_CLRADC14IFG21                       (0x00200000)          /* clear ADC14IFG21   */
#define ADC14_CLRIFGR0_CLRADC14IFG22                       (0x00400000)          /* clear ADC14IFG22 */
#define ADC14_CLRIFGR0_CLRADC14IFG23                       (0x00800000)          /* clear ADC14IFG23 */
#define ADC14_CLRIFGR0_CLRADC14IFG24                       (0x01000000)          /* clear ADC14IFG24 */
#define ADC14_CLRIFGR0_CLRADC14IFG25                       (0x02000000)          /* clear ADC14IFG25 */
#define ADC14_CLRIFGR0_CLRADC14IFG26                       (0x04000000)          /* clear ADC14IFG26   */
#define ADC14_CLRIFGR0_CLRADC14IFG27                       (0x08000000)          /* clear ADC14IFG27 */
#define ADC14_CLRIFGR0_CLRADC14IFG28                       (0x10000000)          /* clear ADC14IFG28 */
#define ADC14_CLRIFGR0_CLRADC14IFG29                       (0x20000000)          /* clear ADC14IFG29 */
#define ADC14_CLRIFGR0_CLRADC14IFG30                       (0x40000000)          /* clear ADC14IFG30 */
#define ADC14_CLRIFGR0_CLRADC14IFG31                       (0x80000000)          /* clear ADC14IFG31 */

/* ADC14_CLRIFGR1 Control Bits */
#define ADC14_CLRIFGR1_RESERVED_2                          (0x00000001)          /* Reserved. Always reads as 0. */
#define ADC14_CLRIFGR1_CLRADC14INIFG                       (0x00000002)          /* clear ADC14INIFG */
#define ADC14_CLRIFGR1_CLRADC14LOIFG                       (0x00000004)          /* clear ADC14LOIFG */
#define ADC14_CLRIFGR1_CLRADC14HIIFG                       (0x00000008)          /* clear ADC14HIIFG */
#define ADC14_CLRIFGR1_CLRADC14OVIFG                       (0x00000010)          /* clear ADC14OVIFG */
#define ADC14_CLRIFGR1_CLRADC14TOVIFG                      (0x00000020)          /* clear ADC14TOVIFG   */
#define ADC14_CLRIFGR1_CLRADC14RDYIFG                      (0x00000040)          /* clear ADC14RDYIFG */
#define ADC14_CLRIFGR1_RESERVED_1__M                       (0xffffff80)          /* Reserved. Always reads as 0. */


//*****************************************************************************
// AES256 Registers
//*****************************************************************************
typedef struct {
  union {    /* AES256_ACTL0 Register */
    __IO uint16_t reg;
    struct { /* AES256_ACTL0 Control Bits */
      __IO uint16_t OP                  	:	2;  /* AES operation. The AESOPx bits are not reset by AESSWRST = 1. Writes are ignored */
                                                /* when AESCMEN = 1 and AESBLKCNTx > 0. */
      __IO uint16_t KL                  	:	2;  /* AES key length. These bits define which of the 3 AES standards is performed. The */
                                                /* AESKLx bits are not reset by AESSWRST = 1. Writes are ignored when AESCMEN = 1  */
                                                /* and AESBLKCNTx > 0. */
      __I  uint16_t RESERVED3           	:	1;  /* Reserved */
      __I  uint16_t CM                  	:	2;  /* AES cipher mode select. These bits are ignored for AESCMEN=0. Writes are ignored */
                                                /* when AESCMEN = 1 and AESBLKCNTx > 0. */
      __IO uint16_t SWRST               	:	1;  /* AES software reset. Immediately resets the complete AES accelerator module even  */
                                                /* when busy except for the AESRDYIE, the AESKLx and the AESOPx bits. It also  */
                                                /* clears the (internal) state memory. The AESSWRST bit is automatically reset and  */
                                                /* is always read as zero. */
      __IO uint16_t RDYIFG              	:	1;  /* AES ready interrupt flag. Set when the selected AES operation was completed and  */
                                                /* the result can be read from AESADOUT. Automatically cleared when AESADOUT is  */
                                                /* read or AESAKEY or AESADIN is written. */
      __I  uint16_t RESERVED2           	:	2;  /* Reserved */
      __IO uint16_t ERRFG               	:	1;  /* AES error flag. AESAKEY or AESADIN were written while an AES operation was in  */
                                                /* progress. The bit must be cleared by software. */
      __IO uint16_t RDYIE               	:	1;  /* AES ready interrupt enable. AESRDYIE is not reset by AESSWRST = 1. */
      __I  uint16_t RESERVED1           	:	2;  /* Reserved */
      __IO uint16_t CMEN                	:	1;  /* AESCMEN enables the support of the ciphermodes ECB, CBC, OFB and CFB together  */
                                                /* with the DMA. Writes are ignored when AESCMEN = 1 and AESBLKCNTx > 0. */

    } bit;
  } ACTL0;
  union {    /* AES256_ACTL1 Register */
    __IO uint16_t reg;
    struct { /* AES256_ACTL1 Control Bits */
      __IO uint16_t BLKCNT              	:	8;  /* Cipher Block Counter. Number of blocks to be encrypted or decrypted with block  */
                                                /* cipher modes enabled (AESCMEN=1). Ignored if AESCMEN=0. The block counter  */
                                                /* decrements with each performed encryption or decryption. Writes are ignored when */
                                                /* AESCMEN = 1 and AESBLKCNTx > 0. */
      __I  uint16_t RESERVED1           	:	8;  /* Reserved */

    } bit;
  } ACTL1;
  union {    /* AES256_ASTAT Register */
    __IO uint16_t reg;
    struct { /* AES256_ASTAT Control Bits */
      __I  uint16_t BUSY                	:	1;  /* AES accelerator module busy; encryption, decryption, or key generation in  */
                                                /* progress. */
      __IO uint16_t KEYWR               	:	1;  /* All 16 bytes written to AESAKEY. This bit can be modified by software but it  */
                                                /* must not be reset by software (1 '0) if AESCMEN=1. Changing its state by  */
                                                /* software also resets the AESKEYCNTx bits. AESKEYWR is reset by PUC, AESSWRST, an */
                                                /* error condition, changing AESOPx, changing AESKLx, and the start to (over)write */
                                                /* a new key. Because it is reset when AESOPx is changed it can be set by software */
                                                /* again to indicate that the loaded key is still valid. */
      __IO uint16_t DINWR               	:	1;  /* All 16 bytes written to AESADIN, AESAXDIN or AESAXIN. This bit can be modified  */
                                                /* by software only if AESCMEN=0. Changing its state by software also resets the  */
                                                /* AESDINCNTx bits. AESDINWR is reset by PUC, AESSWRST, an error condition,  */
                                                /* changing AESOPx, changing AESKLx, the start to (over)write the data, and when  */
                                                /* the AES accelerator is busy. Because it is reset when AESOPx or AESKLx is  */
                                                /* changed it can be set by software again to indicate that the current data is  */
                                                /* still valid. */
      __I  uint16_t DOUTRD              	:	1;  /* All 16 bytes read from AESADOUT. AESDOUTRD is reset by PUC, AESSWRST, an error  */
                                                /* condition, changing AESOPx, changing AESKLx, when the AES accelerator is busy,  */
                                                /* and when the output data is read again. */
      __I  uint16_t KEYCNT              	:	4;  /* Bytes written via AESAKEY for AESKLx=00, words written via AESAKEY if AESKLx=01, */
                                                /* 10,11. Reset when AESKEYWR is reset. If AESKEYCNTx = 0 and AESKEYWR = 0, no  */
                                                /* bytes were written. If AESKEYCNTx = 0 and AESKEYWR = 1, all bytes were written. */
      __I  uint16_t DINCNT              	:	4;  /* Bytes written via AESADIN, AESAXDIN or AESAXIN. Reset when AESDINWR is reset. If */
                                                /* AESDINCNTx = 0 and AESDINWR = 0, no bytes were written. If AESDINCNTx = 0 and  */
                                                /* AESDINWR = 1, all bytes were written. */
      __I  uint16_t DOUTCNT             	:	4;  /* Bytes read via AESADOUT. Reset when AESDOUTRD is reset. If AESDOUTCNTx = 0 and  */
                                                /* AESDOUTRD = 0, no bytes were read. If AESDOUTCNTx = 0 and AESDOUTRD = 1, all  */
                                                /* bytes were read. */

    } bit;
  } ASTAT;
  union {    /* AES256_AKEY Register */
    __IO uint16_t reg;
    struct { /* AES256_AKEY Control Bits */
      __O  uint16_t KEY0                	:	8;  /* AES key byte n when AESAKEY is written as word. AES next key byte when AESAKEY_L */
                                                /* is written as byte. Do not mix word and byte access. Always reads as zero. The  */
                                                /* key is reset by PUC or by AESSWRST = 1. */
      __O  uint16_t KEY1                	:	8;  /* AES key byte n+1 when AESAKEY is written as word. Do not use these bits for byte */
                                                /* access. Do not mix word and byte access. Always reads as zero. The key is reset */
                                                /* by PUC or by AESSWRST = 1. */

    } bit;
  } AKEY;
  union {    /* AES256_ADIN Register */
    __IO uint16_t reg;
    struct { /* AES256_ADIN Control Bits */
      __O  uint16_t DIN0                	:	8;  /* AES data in byte n when AESADIN is written as word. AES next data in byte when  */
                                                /* AESADIN_L is written as byte. Do not mix word and byte access. Always reads as  */
                                                /* zero. */
      __O  uint16_t DIN1                	:	8;  /* AES data in byte n+1 when AESADIN is written as word. Do not use these bits for  */
                                                /* byte access. Do not mix word and byte access. Always reads as zero. */

    } bit;
  } ADIN;
  union {    /* AES256_ADOUT Register */
    __IO uint16_t reg;
    struct { /* AES256_ADOUT Control Bits */
      __I  uint16_t DOUT0               	:	8;  /* AES data out byte n when AESADOUT is read as word. AES next data out byte when  */
                                                /* AESADOUT_L is read as byte. Do not mix word and byte access. */
      __I  uint16_t DOUT1               	:	8;  /* AES data out byte n+1 when AESADOUT is read as word. Do not use these bits for  */
                                                /* byte access. Do not mix word and byte access. */

    } bit;
  } ADOUT;
  union {    /* AES256_AXDIN Register */
    __IO uint16_t reg;
    struct { /* AES256_AXDIN Control Bits */
      __O  uint16_t XDIN0               	:	8;  /* AES data in byte n when AESAXDIN is written as word. AES next data in byte when  */
                                                /* AESAXDIN_L is written as byte. Do not mix word and byte access. Always reads as  */
                                                /* zero. */
      __O  uint16_t XDIN1               	:	8;  /* AES data in byte n+1 when AESAXDIN is written as word. Do not use these bits for */
                                                /* byte access. Do not mix word and byte access. Always reads as zero. */

    } bit;
  } AXDIN;
  union {    /* AES256_AXIN Register */
    __IO uint16_t reg;
    struct { /* AES256_AXIN Control Bits */
      __O  uint16_t XIN0                	:	8;  /* AES data in byte n when AESAXIN is written as word. AES next data in byte when  */
                                                /* AESAXIN_L is written as byte. Do not mix word and byte access. Always reads as  */
                                                /* zero. */
      __O  uint16_t XIN1                	:	8;  /* AES data in byte n+1 when AESAXIN is written as word. Do not use these bits for  */
                                                /* byte access. Do not mix word and byte access. Always reads as zero. */

    } bit;
  } AXIN;
} AES256_Type;


/* AES256_ACTL0 Control Bits */
#define AES256_ACTL0_OP__M                                 (0x0003)              /* AES operation. The AESOPx bits are not reset by AESSWRST = 1. Writes are ignored */
                                                                                 /* when AESCMEN = 1 and AESBLKCNTx > 0. */
#define AES256_ACTL0_OP__0                                 (0x0000)              /* Encryption */
#define AES256_ACTL0_OP__0_ENCRYPTION                      (0x0000)              /* Encryption */
#define AES256_ACTL0_OP__1                                 (0x0001)              /* Decryption. The provided key is the same key used for encryption. */
#define AES256_ACTL0_OP__2                                 (0x0002)              /* Generate first round key required for decryption. */
#define AES256_ACTL0_OP__3                                 (0x0003)              /* Decryption. The provided key is the first round key required for decryption. */
#define AES256_ACTL0_KL__M                                 (0x000c)              /* AES key length. These bits define which of the 3 AES standards is performed. The */
                                                                                 /* AESKLx bits are not reset by AESSWRST = 1. Writes are ignored when AESCMEN = 1  */
                                                                                 /* and AESBLKCNTx > 0. */
#define AES256_ACTL0_KL__0                                 (0x0000)              /* AES128. The keysize is 128 bit. */
#define AES256_ACTL0_KL__1                                 (0x0004)              /* AES192. The keysize is 192 bit. */
#define AES256_ACTL0_KL__2                                 (0x0008)              /* AES256. The keysize is 256 bit. */
#define AES256_ACTL0_RESERVED3                             (0x0010)              /*  */
#define AES256_ACTL0_CM__M                                 (0x0060)              /* AES cipher mode select. These bits are ignored for AESCMEN=0. Writes are ignored */
                                                                                 /* when AESCMEN = 1 and AESBLKCNTx > 0. */
#define AES256_ACTL0_CM__0                                 (0x0000)              /* ECB */
#define AES256_ACTL0_CM__0_ECB                             (0x0000)              /* ECB */
#define AES256_ACTL0_CM__1                                 (0x0020)              /* CBC */
#define AES256_ACTL0_CM__1_CBC                             (0x0020)              /* CBC */
#define AES256_ACTL0_CM__2                                 (0x0040)              /* OFB */
#define AES256_ACTL0_CM__2_OFB                             (0x0040)              /* OFB */
#define AES256_ACTL0_CM__3                                 (0x0060)              /* CFB */
#define AES256_ACTL0_CM__3_CFB                             (0x0060)              /* CFB */
#define AES256_ACTL0_SWRST                                 (0x0080)              /* AES software reset. Immediately resets the complete AES accelerator module even  */
                                                                                 /* when busy except for the AESRDYIE, the AESKLx and the AESOPx bits. It also  */
                                                                                 /* clears the (internal) state memory. The AESSWRST bit is automatically reset and  */
                                                                                 /* is always read as zero. */
#define AES256_ACTL0_RDYIFG                                (0x0100)              /* AES ready interrupt flag. Set when the selected AES operation was completed and  */
                                                                                 /* the result can be read from AESADOUT. Automatically cleared when AESADOUT is  */
                                                                                 /* read or AESAKEY or AESADIN is written. */
#define AES256_ACTL0_RESERVED2__M                          (0x0600)              /*  */
#define AES256_ACTL0_ERRFG                                 (0x0800)              /* AES error flag. AESAKEY or AESADIN were written while an AES operation was in  */
                                                                                 /* progress. The bit must be cleared by software. */
#define AES256_ACTL0_RDYIE                                 (0x1000)              /* AES ready interrupt enable. AESRDYIE is not reset by AESSWRST = 1. */
#define AES256_ACTL0_RESERVED1__M                          (0x6000)              /*  */
#define AES256_ACTL0_CMEN                                  (0x8000)              /* AESCMEN enables the support of the ciphermodes ECB, CBC, OFB and CFB together  */
                                                                                 /* with the DMA. Writes are ignored when AESCMEN = 1 and AESBLKCNTx > 0. */

/* AES256_ACTL1 Control Bits */
#define AES256_ACTL1_BLKCNT__M                             (0x00ff)              /* Cipher Block Counter. Number of blocks to be encrypted or decrypted with block  */
                                                                                 /* cipher modes enabled (AESCMEN=1). Ignored if AESCMEN=0. The block counter  */
                                                                                 /* decrements with each performed encryption or decryption. Writes are ignored when */
                                                                                 /* AESCMEN = 1 and AESBLKCNTx > 0. */
#define AES256_ACTL1_RESERVED1__M                          (0xff00)              /*  */

/* AES256_ASTAT Control Bits */
#define AES256_ASTAT_BUSY                                  (0x0001)              /* AES accelerator module busy; encryption, decryption, or key generation in  */
                                                                                 /* progress. */
#define AES256_ASTAT_KEYWR                                 (0x0002)              /* All 16 bytes written to AESAKEY. This bit can be modified by software but it  */
                                                                                 /* must not be reset by software (1 '0) if AESCMEN=1. Changing its state by  */
                                                                                 /* software also resets the AESKEYCNTx bits. AESKEYWR is reset by PUC, AESSWRST, an */
                                                                                 /* error condition, changing AESOPx, changing AESKLx, and the start to (over)write */
                                                                                 /* a new key. Because it is reset when AESOPx is changed it can be set by software */
                                                                                 /* again to indicate that the loaded key is still valid. */
#define AES256_ASTAT_DINWR                                 (0x0004)              /* All 16 bytes written to AESADIN, AESAXDIN or AESAXIN. This bit can be modified  */
                                                                                 /* by software only if AESCMEN=0. Changing its state by software also resets the  */
                                                                                 /* AESDINCNTx bits. AESDINWR is reset by PUC, AESSWRST, an error condition,  */
                                                                                 /* changing AESOPx, changing AESKLx, the start to (over)write the data, and when  */
                                                                                 /* the AES accelerator is busy. Because it is reset when AESOPx or AESKLx is  */
                                                                                 /* changed it can be set by software again to indicate that the current data is  */
                                                                                 /* still valid. */
#define AES256_ASTAT_DOUTRD                                (0x0008)              /* All 16 bytes read from AESADOUT. AESDOUTRD is reset by PUC, AESSWRST, an error  */
                                                                                 /* condition, changing AESOPx, changing AESKLx, when the AES accelerator is busy,  */
                                                                                 /* and when the output data is read again. */
#define AES256_ASTAT_KEYCNT__M                             (0x00f0)              /* Bytes written via AESAKEY for AESKLx=00, words written via AESAKEY if AESKLx=01, */
                                                                                 /* 10,11. Reset when AESKEYWR is reset. If AESKEYCNTx = 0 and AESKEYWR = 0, no  */
                                                                                 /* bytes were written. If AESKEYCNTx = 0 and AESKEYWR = 1, all bytes were written. */
#define AES256_ASTAT_DINCNT__M                             (0x0f00)              /* Bytes written via AESADIN, AESAXDIN or AESAXIN. Reset when AESDINWR is reset. If */
                                                                                 /* AESDINCNTx = 0 and AESDINWR = 0, no bytes were written. If AESDINCNTx = 0 and  */
                                                                                 /* AESDINWR = 1, all bytes were written. */
#define AES256_ASTAT_DOUTCNT__M                            (0xf000)              /* Bytes read via AESADOUT. Reset when AESDOUTRD is reset. If AESDOUTCNTx = 0 and  */
                                                                                 /* AESDOUTRD = 0, no bytes were read. If AESDOUTCNTx = 0 and AESDOUTRD = 1, all  */
                                                                                 /* bytes were read. */

/* AES256_AKEY Control Bits */
#define AES256_AKEY_KEY0__M                                (0x00ff)              /* AES key byte n when AESAKEY is written as word. AES next key byte when AESAKEY_L */
                                                                                 /* is written as byte. Do not mix word and byte access. Always reads as zero. The  */
                                                                                 /* key is reset by PUC or by AESSWRST = 1. */
#define AES256_AKEY_KEY1__M                                (0xff00)              /* AES key byte n+1 when AESAKEY is written as word. Do not use these bits for byte */
                                                                                 /* access. Do not mix word and byte access. Always reads as zero. The key is reset */
                                                                                 /* by PUC or by AESSWRST = 1. */

/* AES256_ADIN Control Bits */
#define AES256_ADIN_DIN0__M                                (0x00ff)              /* AES data in byte n when AESADIN is written as word. AES next data in byte when  */
                                                                                 /* AESADIN_L is written as byte. Do not mix word and byte access. Always reads as  */
                                                                                 /* zero. */
#define AES256_ADIN_DIN1__M                                (0xff00)              /* AES data in byte n+1 when AESADIN is written as word. Do not use these bits for  */
                                                                                 /* byte access. Do not mix word and byte access. Always reads as zero. */

/* AES256_ADOUT Control Bits */
#define AES256_ADOUT_DOUT0__M                              (0x00ff)              /* AES data out byte n when AESADOUT is read as word. AES next data out byte when  */
                                                                                 /* AESADOUT_L is read as byte. Do not mix word and byte access. */
#define AES256_ADOUT_DOUT1__M                              (0xff00)              /* AES data out byte n+1 when AESADOUT is read as word. Do not use these bits for  */
                                                                                 /* byte access. Do not mix word and byte access. */

/* AES256_AXDIN Control Bits */
#define AES256_AXDIN_XDIN0__M                              (0x00ff)              /* AES data in byte n when AESAXDIN is written as word. AES next data in byte when  */
                                                                                 /* AESAXDIN_L is written as byte. Do not mix word and byte access. Always reads as  */
                                                                                 /* zero. */
#define AES256_AXDIN_XDIN1__M                              (0xff00)              /* AES data in byte n+1 when AESAXDIN is written as word. Do not use these bits for */
                                                                                 /* byte access. Do not mix word and byte access. Always reads as zero. */

/* AES256_AXIN Control Bits */
#define AES256_AXIN_XIN0__M                                (0x00ff)              /* AES data in byte n when AESAXIN is written as word. AES next data in byte when  */
                                                                                 /* AESAXIN_L is written as byte. Do not mix word and byte access. Always reads as  */
                                                                                 /* zero. */
#define AES256_AXIN_XIN1__M                                (0xff00)              /* AES data in byte n+1 when AESAXIN is written as word. Do not use these bits for  */
                                                                                 /* byte access. Do not mix word and byte access. Always reads as zero. */


//*****************************************************************************
// CAPTIO0 Registers
//*****************************************************************************
typedef struct {
  uint8_t RESERVED0[14];
  union {    /* CAPTIO0_CTL Register */
    __IO uint16_t reg;
    struct { /* CAPTIO0_CTL Control Bits */
      __IO uint16_t PISEL               	:	3;  /* Capacitive Touch IO pin select. Selects the pin within selected port Px (see  */
                                                /* CAPTIOPOSELx). Selecting a port pin that is not available on the device in use  */
                                                /* gives unpredictible results. */
      __IO uint16_t POSEL               	:	4;  /* Capacitive Touch IO port select. Selects port Px. Selecting a port pin that is  */
                                                /* not available on the devce in use gives unpredictible results. */
      __IO uint16_t EN                  	:	1;  /* Capacitive Touch IO enable */
      __I  uint16_t STATE               	:	1;  /* Capacitive Touch IO state. Reports the current state of the selected Capacitive  */
                                                /* Touch IO. */

    } bit;
  } CTL;
} CAPTIO0_Type;


/* CAPTIO0_CTL Control Bits */
#define CAPTIO0_CTL_PISEL__M                               (0x000e)              /* Capacitive Touch IO pin select. Selects the pin within selected port Px (see  */
                                                                                 /* CAPTIOPOSELx). Selecting a port pin that is not available on the device in use  */
                                                                                 /* gives unpredictible results. */
#define CAPTIO0_CTL_PISEL__0                               (0x0000)              /* Px.0 */
#define CAPTIO0_CTL_PISEL__0_PX_0                          (0x0000)              /* Px.0 */
#define CAPTIO0_CTL_PISEL__1                               (0x0002)              /* Px.1 */
#define CAPTIO0_CTL_PISEL__1_PX_1                          (0x0002)              /* Px.1 */
#define CAPTIO0_CTL_PISEL__2                               (0x0004)              /* Px.2 */
#define CAPTIO0_CTL_PISEL__2_PX_2                          (0x0004)              /* Px.2 */
#define CAPTIO0_CTL_PISEL__3                               (0x0006)              /* Px.3 */
#define CAPTIO0_CTL_PISEL__3_PX_3                          (0x0006)              /* Px.3 */
#define CAPTIO0_CTL_PISEL__4                               (0x0008)              /* Px.4 */
#define CAPTIO0_CTL_PISEL__4_PX_4                          (0x0008)              /* Px.4 */
#define CAPTIO0_CTL_PISEL__5                               (0x000a)              /* Px.5 */
#define CAPTIO0_CTL_PISEL__5_PX_5                          (0x000a)              /* Px.5 */
#define CAPTIO0_CTL_PISEL__6                               (0x000c)              /* Px.6 */
#define CAPTIO0_CTL_PISEL__6_PX_6                          (0x000c)              /* Px.6 */
#define CAPTIO0_CTL_PISEL__7                               (0x000e)              /* Px.7 */
#define CAPTIO0_CTL_PISEL__7_PX_7                          (0x000e)              /* Px.7 */
#define CAPTIO0_CTL_POSEL__M                               (0x00f0)              /* Capacitive Touch IO port select. Selects port Px. Selecting a port pin that is  */
                                                                                 /* not available on the devce in use gives unpredictible results. */
#define CAPTIO0_CTL_POSEL__0                               (0x0000)              /* Px = PJ */
#define CAPTIO0_CTL_POSEL__0_PX___PJ                       (0x0000)              /* Px = PJ */
#define CAPTIO0_CTL_POSEL__1                               (0x0010)              /* Px = P1 */
#define CAPTIO0_CTL_POSEL__1_PX___P1                       (0x0010)              /* Px = P1 */
#define CAPTIO0_CTL_POSEL__2                               (0x0020)              /* Px = P2 */
#define CAPTIO0_CTL_POSEL__2_PX___P2                       (0x0020)              /* Px = P2 */
#define CAPTIO0_CTL_POSEL__3                               (0x0030)              /* Px = P3 */
#define CAPTIO0_CTL_POSEL__3_PX___P3                       (0x0030)              /* Px = P3 */
#define CAPTIO0_CTL_POSEL__4                               (0x0040)              /* Px = P4 */
#define CAPTIO0_CTL_POSEL__4_PX___P4                       (0x0040)              /* Px = P4 */
#define CAPTIO0_CTL_POSEL__5                               (0x0050)              /* Px = P5 */
#define CAPTIO0_CTL_POSEL__5_PX___P5                       (0x0050)              /* Px = P5 */
#define CAPTIO0_CTL_POSEL__6                               (0x0060)              /* Px = P6 */
#define CAPTIO0_CTL_POSEL__6_PX___P6                       (0x0060)              /* Px = P6 */
#define CAPTIO0_CTL_POSEL__7                               (0x0070)              /* Px = P7 */
#define CAPTIO0_CTL_POSEL__7_PX___P7                       (0x0070)              /* Px = P7 */
#define CAPTIO0_CTL_POSEL__8                               (0x0080)              /* Px = P8 */
#define CAPTIO0_CTL_POSEL__8_PX___P8                       (0x0080)              /* Px = P8 */
#define CAPTIO0_CTL_POSEL__9                               (0x0090)              /* Px = P9 */
#define CAPTIO0_CTL_POSEL__9_PX___P9                       (0x0090)              /* Px = P9 */
#define CAPTIO0_CTL_POSEL__10                              (0x00a0)              /* Px = P10 */
#define CAPTIO0_CTL_POSEL__10_PX___P10                     (0x00a0)              /* Px = P10 */
#define CAPTIO0_CTL_POSEL__11                              (0x00b0)              /* Px = P11 */
#define CAPTIO0_CTL_POSEL__11_PX___P11                     (0x00b0)              /* Px = P11 */
#define CAPTIO0_CTL_POSEL__12                              (0x00c0)              /* Px = P12 */
#define CAPTIO0_CTL_POSEL__12_PX___P12                     (0x00c0)              /* Px = P12 */
#define CAPTIO0_CTL_POSEL__13                              (0x00d0)              /* Px = P13 */
#define CAPTIO0_CTL_POSEL__13_PX___P13                     (0x00d0)              /* Px = P13 */
#define CAPTIO0_CTL_POSEL__14                              (0x00e0)              /* Px = P14 */
#define CAPTIO0_CTL_POSEL__14_PX___P14                     (0x00e0)              /* Px = P14 */
#define CAPTIO0_CTL_POSEL__15                              (0x00f0)              /* Px = P15 */
#define CAPTIO0_CTL_POSEL__15_PX___P15                     (0x00f0)              /* Px = P15 */
#define CAPTIO0_CTL_EN                                     (0x0100)              /* Capacitive Touch IO enable */
#define CAPTIO0_CTL_STATE                                  (0x0200)              /* Capacitive Touch IO state. Reports the current state of the selected Capacitive  */
                                                                                 /* Touch IO. */


//*****************************************************************************
// CAPTIO1 Registers
//*****************************************************************************
typedef struct {
  uint8_t RESERVED0[14];
  union {    /* CAPTIO1_CTL Register */
    __IO uint16_t reg;
    struct { /* CAPTIO1_CTL Control Bits */
      __IO uint16_t PISEL               	:	3;  /* Capacitive Touch IO pin select. Selects the pin within selected port Px (see  */
                                                /* CAPTIOPOSELx). Selecting a port pin that is not available on the device in use  */
                                                /* gives unpredictible results. */
      __IO uint16_t POSEL               	:	4;  /* Capacitive Touch IO port select. Selects port Px. Selecting a port pin that is  */
                                                /* not available on the devce in use gives unpredictible results. */
      __IO uint16_t EN                  	:	1;  /* Capacitive Touch IO enable */
      __I  uint16_t STATE               	:	1;  /* Capacitive Touch IO state. Reports the current state of the selected Capacitive  */
                                                /* Touch IO. */

    } bit;
  } CTL;
} CAPTIO1_Type;


/* CAPTIO1_CTL Control Bits */
#define CAPTIO1_CTL_PISEL__M                               (0x000e)              /* Capacitive Touch IO pin select. Selects the pin within selected port Px (see  */
                                                                                 /* CAPTIOPOSELx). Selecting a port pin that is not available on the device in use  */
                                                                                 /* gives unpredictible results. */
#define CAPTIO1_CTL_PISEL__0                               (0x0000)              /* Px.0 */
#define CAPTIO1_CTL_PISEL__0_PX_0                          (0x0000)              /* Px.0 */
#define CAPTIO1_CTL_PISEL__1                               (0x0002)              /* Px.1 */
#define CAPTIO1_CTL_PISEL__1_PX_1                          (0x0002)              /* Px.1 */
#define CAPTIO1_CTL_PISEL__2                               (0x0004)              /* Px.2 */
#define CAPTIO1_CTL_PISEL__2_PX_2                          (0x0004)              /* Px.2 */
#define CAPTIO1_CTL_PISEL__3                               (0x0006)              /* Px.3 */
#define CAPTIO1_CTL_PISEL__3_PX_3                          (0x0006)              /* Px.3 */
#define CAPTIO1_CTL_PISEL__4                               (0x0008)              /* Px.4 */
#define CAPTIO1_CTL_PISEL__4_PX_4                          (0x0008)              /* Px.4 */
#define CAPTIO1_CTL_PISEL__5                               (0x000a)              /* Px.5 */
#define CAPTIO1_CTL_PISEL__5_PX_5                          (0x000a)              /* Px.5 */
#define CAPTIO1_CTL_PISEL__6                               (0x000c)              /* Px.6 */
#define CAPTIO1_CTL_PISEL__6_PX_6                          (0x000c)              /* Px.6 */
#define CAPTIO1_CTL_PISEL__7                               (0x000e)              /* Px.7 */
#define CAPTIO1_CTL_PISEL__7_PX_7                          (0x000e)              /* Px.7 */
#define CAPTIO1_CTL_POSEL__M                               (0x00f0)              /* Capacitive Touch IO port select. Selects port Px. Selecting a port pin that is  */
                                                                                 /* not available on the devce in use gives unpredictible results. */
#define CAPTIO1_CTL_POSEL__0                               (0x0000)              /* Px = PJ */
#define CAPTIO1_CTL_POSEL__0_PX___PJ                       (0x0000)              /* Px = PJ */
#define CAPTIO1_CTL_POSEL__1                               (0x0010)              /* Px = P1 */
#define CAPTIO1_CTL_POSEL__1_PX___P1                       (0x0010)              /* Px = P1 */
#define CAPTIO1_CTL_POSEL__2                               (0x0020)              /* Px = P2 */
#define CAPTIO1_CTL_POSEL__2_PX___P2                       (0x0020)              /* Px = P2 */
#define CAPTIO1_CTL_POSEL__3                               (0x0030)              /* Px = P3 */
#define CAPTIO1_CTL_POSEL__3_PX___P3                       (0x0030)              /* Px = P3 */
#define CAPTIO1_CTL_POSEL__4                               (0x0040)              /* Px = P4 */
#define CAPTIO1_CTL_POSEL__4_PX___P4                       (0x0040)              /* Px = P4 */
#define CAPTIO1_CTL_POSEL__5                               (0x0050)              /* Px = P5 */
#define CAPTIO1_CTL_POSEL__5_PX___P5                       (0x0050)              /* Px = P5 */
#define CAPTIO1_CTL_POSEL__6                               (0x0060)              /* Px = P6 */
#define CAPTIO1_CTL_POSEL__6_PX___P6                       (0x0060)              /* Px = P6 */
#define CAPTIO1_CTL_POSEL__7                               (0x0070)              /* Px = P7 */
#define CAPTIO1_CTL_POSEL__7_PX___P7                       (0x0070)              /* Px = P7 */
#define CAPTIO1_CTL_POSEL__8                               (0x0080)              /* Px = P8 */
#define CAPTIO1_CTL_POSEL__8_PX___P8                       (0x0080)              /* Px = P8 */
#define CAPTIO1_CTL_POSEL__9                               (0x0090)              /* Px = P9 */
#define CAPTIO1_CTL_POSEL__9_PX___P9                       (0x0090)              /* Px = P9 */
#define CAPTIO1_CTL_POSEL__10                              (0x00a0)              /* Px = P10 */
#define CAPTIO1_CTL_POSEL__10_PX___P10                     (0x00a0)              /* Px = P10 */
#define CAPTIO1_CTL_POSEL__11                              (0x00b0)              /* Px = P11 */
#define CAPTIO1_CTL_POSEL__11_PX___P11                     (0x00b0)              /* Px = P11 */
#define CAPTIO1_CTL_POSEL__12                              (0x00c0)              /* Px = P12 */
#define CAPTIO1_CTL_POSEL__12_PX___P12                     (0x00c0)              /* Px = P12 */
#define CAPTIO1_CTL_POSEL__13                              (0x00d0)              /* Px = P13 */
#define CAPTIO1_CTL_POSEL__13_PX___P13                     (0x00d0)              /* Px = P13 */
#define CAPTIO1_CTL_POSEL__14                              (0x00e0)              /* Px = P14 */
#define CAPTIO1_CTL_POSEL__14_PX___P14                     (0x00e0)              /* Px = P14 */
#define CAPTIO1_CTL_POSEL__15                              (0x00f0)              /* Px = P15 */
#define CAPTIO1_CTL_POSEL__15_PX___P15                     (0x00f0)              /* Px = P15 */
#define CAPTIO1_CTL_EN                                     (0x0100)              /* Capacitive Touch IO enable */
#define CAPTIO1_CTL_STATE                                  (0x0200)              /* Capacitive Touch IO state. Reports the current state of the selected Capacitive  */
                                                                                 /* Touch IO. */


//*****************************************************************************
// COMP_E0 Registers
//*****************************************************************************
typedef struct {
  union {    /* COMP_E0_CTL0 Register */
    __IO uint16_t reg;
    struct { /* COMP_E0_CTL0 Control Bits */
      __IO uint16_t IPSEL               	:	4;  /* Channel input selected for the V+ terminal of the comparator if CEIPEN is set to */
                                                /* 1.   */
      __I  uint16_t RESERVED_2          	:	3;  /* Reserved */
      __IO uint16_t IPEN                	:	1;  /* Channel input enable for the V+ terminal of the comparator. */
      __IO uint16_t IMSEL               	:	4;  /* Channel input selected for the - terminal of the comparator if CEIMEN is set to  */
                                                /* 1.   */
      __I  uint16_t RESERVED_1          	:	3;  /* Reserved */
      __IO uint16_t IMEN                	:	1;  /* Channel input enable for the - terminal of the comparator. */

    } bit;
  } CTL0;
  union {    /* COMP_E0_CTL1 Register */
    __IO uint16_t reg;
    struct { /* COMP_E0_CTL1 Control Bits */
      __IO uint16_t OUT                 	:	1;  /* Output value. This bit reflects the value of the Comparator output. Writing this */
                                                /* bit has no effect on the comparator output.   */
      __IO uint16_t OUTPOL              	:	1;  /* Output polarity. This bit defines the CEOUT polarity. */
      __IO uint16_t F                   	:	1;  /* Output filter. Available if CEPWRMD = 00, 01. */
      __IO uint16_t IES                 	:	1;  /* Interrupt edge select for CEIIFG and CEIFG */
      __IO uint16_t SHORT               	:	1;  /* Input short. This bit shorts the + and - input terminals. */
      __IO uint16_t EX                  	:	1;  /* Exchange. This bit permutes the comparator 0 inputs and inverts the comparator 0 */
                                                /* output.   */
      __IO uint16_t FDLY                	:	2;  /* Filter delay. The filter delay can be selected in four steps. See the device- */
                                                /* specific data sheet for details. */
      __IO uint16_t PWRMD               	:	2;  /* Power Mode */
      __IO uint16_t ON                  	:	1;  /* On. This bit turns the comparator on. When the comparator is turned off the  */
                                                /* Comparator consumes no power. */
      __IO uint16_t MRVL                	:	1;  /* This bit is valid of CEMRVS is set to 1.   */
      __IO uint16_t MRVS                	:	1;  /* This bit defines if the comparator output selects between VREF0 or VREF1 if CERS */
                                                /* = 00, 01, or 10.   */
      __I  uint16_t RESERVED_1          	:	3;  /* Reserved */

    } bit;
  } CTL1;
  union {    /* COMP_E0_CTL2 Register */
    __IO uint16_t reg;
    struct { /* COMP_E0_CTL2 Control Bits */
      __IO uint16_t REF0                	:	5;  /* Reference resistor tap 0. This register defines the tap of the resistor string  */
                                                /* while CEOUT = 0. */
      __IO uint16_t RSEL                	:	1;  /* Reference select. This bit selects which terminal the V#sub#CCREF#/sub# is  */
                                                /* applied to. */
      __IO uint16_t RS                  	:	2;  /* Reference source. This bit define if the reference voltage is derived from VCC  */
                                                /* or from the precise shared reference. */
      __IO uint16_t REF1                	:	5;  /* Reference resistor tap 1. This register defines the tap of the resistor string  */
                                                /* while CEOUT = 1. */
      __IO uint16_t REFL                	:	2;  /* Reference voltage level   */
      __IO uint16_t REFACC              	:	1;  /* Reference accuracy. A reference voltage is requested only if CEREFL > 0.   */

    } bit;
  } CTL2;
  union {    /* COMP_E0_CTL3 Register */
    __IO uint16_t reg;
    struct { /* COMP_E0_CTL3 Control Bits */
      __IO uint16_t PD0                 	:	1;  /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                /* channel x. */
      __IO uint16_t PD1                 	:	1;  /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                /* channel x. */
      __IO uint16_t PD2                 	:	1;  /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                /* channel x. */
      __IO uint16_t PD3                 	:	1;  /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                /* channel x. */
      __IO uint16_t PD4                 	:	1;  /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                /* channel x. */
      __IO uint16_t PD5                 	:	1;  /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                /* channel x. */
      __IO uint16_t PD6                 	:	1;  /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                /* channel x. */
      __IO uint16_t PD7                 	:	1;  /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                /* channel x. */
      __IO uint16_t PD8                 	:	1;  /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                /* channel x. */
      __IO uint16_t PD9                 	:	1;  /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                /* channel x. */
      __IO uint16_t PD10                	:	1;  /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                /* channel x. */
      __IO uint16_t PD11                	:	1;  /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                /* channel x. */
      __IO uint16_t PD12                	:	1;  /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                /* channel x. */
      __IO uint16_t PD13                	:	1;  /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                /* channel x. */
      __IO uint16_t PD14                	:	1;  /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                /* channel x. */
      __IO uint16_t PD15                	:	1;  /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                /* channel x.   */

    } bit;
  } CTL3;
  uint8_t RESERVED0[4];
  union {    /* COMP_E0_INT Register */
    __IO uint16_t reg;
    struct { /* COMP_E0_INT Control Bits */
      __IO uint16_t IFG                 	:	1;  /* Comparator output interrupt flag. The bit CEIES defines the transition of the  */
                                                /* output setting this bit. */
      __IO uint16_t IIFG                	:	1;  /* Comparator output inverted interrupt flag. The bit CEIES defines the transition  */
                                                /* of the output setting this bit. */
      __I  uint16_t RESERVED_4          	:	2;  /* Reserved */
      __IO uint16_t RDYIFG              	:	1;  /* Comparator ready interrupt flag. This bit is set if the Comparator reference  */
                                                /* sources are settled and the Comparator module is operational. This bit has to be */
                                                /* cleared by software. */
      __I  uint16_t RESERVED_3          	:	3;  /* Reserved */
      __IO uint16_t IE                  	:	1;  /* Comparator output interrupt enable */
      __IO uint16_t IIE                 	:	1;  /* Comparator output interrupt enable inverted polarity */
      __I  uint16_t RESERVED_2          	:	2;  /* Reserved */
      __IO uint16_t RDYIE               	:	1;  /* Comparator ready interrupt enable   */
      __I  uint16_t RESERVED_1          	:	3;  /* Reserved */

    } bit;
  } INT;
  uint16_t IV; /*  */
} COMP_E0_Type;


/* COMP_E0_CTL0 Control Bits */
#define COMP_E0_CTL0_IPSEL__M                              (0x000f)              /* Channel input selected for the V+ terminal of the comparator if CEIPEN is set to */
                                                                                 /* 1.   */
#define COMP_E0_CTL0_RESERVED_2__M                         (0x0070)              /* Reserved. Always reads as 0. */
#define COMP_E0_CTL0_IPEN                                  (0x0080)              /* Channel input enable for the V+ terminal of the comparator. */
#define COMP_E0_CTL0_IMSEL__M                              (0x0f00)              /* Channel input selected for the - terminal of the comparator if CEIMEN is set to  */
                                                                                 /* 1.   */
#define COMP_E0_CTL0_RESERVED_1__M                         (0x7000)              /* Reserved. Always reads as 0. */
#define COMP_E0_CTL0_IMEN                                  (0x8000)              /* Channel input enable for the - terminal of the comparator. */

/* COMP_E0_CTL1 Control Bits */
#define COMP_E0_CTL1_OUT                                   (0x0001)              /* Output value. This bit reflects the value of the Comparator output. Writing this */
                                                                                 /* bit has no effect on the comparator output.   */
#define COMP_E0_CTL1_OUTPOL                                (0x0002)              /* Output polarity. This bit defines the CEOUT polarity. */
#define COMP_E0_CTL1_F                                     (0x0004)              /* Output filter. Available if CEPWRMD = 00, 01. */
#define COMP_E0_CTL1_IES                                   (0x0008)              /* Interrupt edge select for CEIIFG and CEIFG */
#define COMP_E0_CTL1_SHORT                                 (0x0010)              /* Input short. This bit shorts the + and - input terminals. */
#define COMP_E0_CTL1_EX                                    (0x0020)              /* Exchange. This bit permutes the comparator 0 inputs and inverts the comparator 0 */
                                                                                 /* output.   */
#define COMP_E0_CTL1_FDLY__M                               (0x00c0)              /* Filter delay. The filter delay can be selected in four steps. See the device- */
                                                                                 /* specific data sheet for details. */
#define COMP_E0_CTL1_FDLY__0                               (0x0000)              /* Typical filter delay of TBD (450) ns */
#define COMP_E0_CTL1_FDLY__1                               (0x0040)              /* Typical filter delay of TBD (900) ns */
#define COMP_E0_CTL1_FDLY__2                               (0x0080)              /* Typical filter delay of TBD (1800) ns */
#define COMP_E0_CTL1_FDLY__3                               (0x00c0)              /* Typical filter delay of TBD (3600) ns */
#define COMP_E0_CTL1_PWRMD__M                              (0x0300)              /* Power Mode */
#define COMP_E0_CTL1_PWRMD__0                              (0x0000)              /* High-speed mode */
#define COMP_E0_CTL1_PWRMD__0_HIGH_SPEED_MODE              (0x0000)              /* High-speed mode */
#define COMP_E0_CTL1_PWRMD__1                              (0x0100)              /* Normal mode */
#define COMP_E0_CTL1_PWRMD__1_NORMAL_MODE                  (0x0100)              /* Normal mode */
#define COMP_E0_CTL1_PWRMD__2                              (0x0200)              /* Ultra-low power mode */
#define COMP_E0_CTL1_ON                                    (0x0400)              /* On. This bit turns the comparator on. When the comparator is turned off the  */
                                                                                 /* Comparator consumes no power. */
#define COMP_E0_CTL1_MRVL                                  (0x0800)              /* This bit is valid of CEMRVS is set to 1.   */
#define COMP_E0_CTL1_MRVS                                  (0x1000)              /* This bit defines if the comparator output selects between VREF0 or VREF1 if CERS */
                                                                                 /* = 00, 01, or 10.   */
#define COMP_E0_CTL1_RESERVED_1__M                         (0xe000)              /* Reserved. Always reads as 0. */

/* COMP_E0_CTL2 Control Bits */
#define COMP_E0_CTL2_REF0__M                               (0x001f)              /* Reference resistor tap 0. This register defines the tap of the resistor string  */
                                                                                 /* while CEOUT = 0. */
#define COMP_E0_CTL2_RSEL                                  (0x0020)              /* Reference select. This bit selects which terminal the V#sub#CCREF#/sub# is  */
                                                                                 /* applied to. */
#define COMP_E0_CTL2_RS__M                                 (0x00c0)              /* Reference source. This bit define if the reference voltage is derived from VCC  */
                                                                                 /* or from the precise shared reference. */
#define COMP_E0_CTL2_RS__0                                 (0x0000)              /* No current is drawn by the reference circuitry. */
#define COMP_E0_CTL2_RS__1                                 (0x0040)              /* VCC applied to the resistor ladder */
#define COMP_E0_CTL2_RS__2                                 (0x0080)              /* Shared reference voltage applied to the resistor ladder. */
#define COMP_E0_CTL2_RS__3                                 (0x00c0)              /* Shared reference voltage supplied to V(CREF). Resistor ladder is off. */
#define COMP_E0_CTL2_REF1__M                               (0x1f00)              /* Reference resistor tap 1. This register defines the tap of the resistor string  */
                                                                                 /* while CEOUT = 1. */
#define COMP_E0_CTL2_REFL__M                               (0x6000)              /* Reference voltage level   */
#define COMP_E0_CTL2_REFL__0                               (0x0000)              /* Reference amplifier is disabled. No reference voltage is requested. */
#define COMP_E0_CTL2_REFL__1                               (0x2000)              /* 1.2 V is selected as shared reference voltage input */
#define COMP_E0_CTL2_REFL__2                               (0x4000)              /* 2.0 V is selected as shared reference voltage input */
#define COMP_E0_CTL2_REFL__3                               (0x6000)              /* 2.5 V is selected as shared reference voltage input */
#define COMP_E0_CTL2_REFACC                                (0x8000)              /* Reference accuracy. A reference voltage is requested only if CEREFL > 0.   */

/* COMP_E0_CTL3 Control Bits */
#define COMP_E0_CTL3_PD0                                   (0x0001)              /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                                                 /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                                                 /* channel x. */
#define COMP_E0_CTL3_PD1                                   (0x0002)              /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                                                 /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                                                 /* channel x. */
#define COMP_E0_CTL3_PD2                                   (0x0004)              /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                                                 /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                                                 /* channel x. */
#define COMP_E0_CTL3_PD3                                   (0x0008)              /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                                                 /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                                                 /* channel x. */
#define COMP_E0_CTL3_PD4                                   (0x0010)              /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                                                 /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                                                 /* channel x. */
#define COMP_E0_CTL3_PD5                                   (0x0020)              /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                                                 /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                                                 /* channel x. */
#define COMP_E0_CTL3_PD6                                   (0x0040)              /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                                                 /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                                                 /* channel x. */
#define COMP_E0_CTL3_PD7                                   (0x0080)              /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                                                 /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                                                 /* channel x. */
#define COMP_E0_CTL3_PD8                                   (0x0100)              /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                                                 /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                                                 /* channel x. */
#define COMP_E0_CTL3_PD9                                   (0x0200)              /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                                                 /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                                                 /* channel x. */
#define COMP_E0_CTL3_PD10                                  (0x0400)              /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                                                 /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                                                 /* channel x. */
#define COMP_E0_CTL3_PD11                                  (0x0800)              /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                                                 /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                                                 /* channel x. */
#define COMP_E0_CTL3_PD12                                  (0x1000)              /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                                                 /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                                                 /* channel x. */
#define COMP_E0_CTL3_PD13                                  (0x2000)              /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                                                 /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                                                 /* channel x. */
#define COMP_E0_CTL3_PD14                                  (0x4000)              /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                                                 /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                                                 /* channel x. */
#define COMP_E0_CTL3_PD15                                  (0x8000)              /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                                                 /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                                                 /* channel x.   */

/* COMP_E0_INT Control Bits */
#define COMP_E0_INT_IFG                                    (0x0001)              /* Comparator output interrupt flag. The bit CEIES defines the transition of the  */
                                                                                 /* output setting this bit. */
#define COMP_E0_INT_IIFG                                   (0x0002)              /* Comparator output inverted interrupt flag. The bit CEIES defines the transition  */
                                                                                 /* of the output setting this bit. */
#define COMP_E0_INT_RESERVED_4__M                          (0x000c)              /* Reserved. Always reads as 0. */
#define COMP_E0_INT_RDYIFG                                 (0x0010)              /* Comparator ready interrupt flag. This bit is set if the Comparator reference  */
                                                                                 /* sources are settled and the Comparator module is operational. This bit has to be */
                                                                                 /* cleared by software. */
#define COMP_E0_INT_RESERVED_3__M                          (0x00e0)              /* Reserved. Always reads as 0. */
#define COMP_E0_INT_IE                                     (0x0100)              /* Comparator output interrupt enable */
#define COMP_E0_INT_IIE                                    (0x0200)              /* Comparator output interrupt enable inverted polarity */
#define COMP_E0_INT_RESERVED_2__M                          (0x0c00)              /* Reserved. Always reads as 0. */
#define COMP_E0_INT_RDYIE                                  (0x1000)              /* Comparator ready interrupt enable   */
#define COMP_E0_INT_RESERVED_1__M                          (0xe000)              /* Reserved. Always reads as 0. */


//*****************************************************************************
// COMP_E1 Registers
//*****************************************************************************
typedef struct {
  union {    /* COMP_E1_CTL0 Register */
    __IO uint16_t reg;
    struct { /* COMP_E1_CTL0 Control Bits */
      __IO uint16_t IPSEL               	:	4;  /* Channel input selected for the V+ terminal of the comparator if CEIPEN is set to */
                                                /* 1.   */
      __I  uint16_t RESERVED_2          	:	3;  /* Reserved */
      __IO uint16_t IPEN                	:	1;  /* Channel input enable for the V+ terminal of the comparator. */
      __IO uint16_t IMSEL               	:	4;  /* Channel input selected for the - terminal of the comparator if CEIMEN is set to  */
                                                /* 1.   */
      __I  uint16_t RESERVED_1          	:	3;  /* Reserved */
      __IO uint16_t IMEN                	:	1;  /* Channel input enable for the - terminal of the comparator. */

    } bit;
  } CTL0;
  union {    /* COMP_E1_CTL1 Register */
    __IO uint16_t reg;
    struct { /* COMP_E1_CTL1 Control Bits */
      __IO uint16_t OUT                 	:	1;  /* Output value. This bit reflects the value of the Comparator output. Writing this */
                                                /* bit has no effect on the comparator output.   */
      __IO uint16_t OUTPOL              	:	1;  /* Output polarity. This bit defines the CEOUT polarity. */
      __IO uint16_t F                   	:	1;  /* Output filter. Available if CEPWRMD = 00, 01. */
      __IO uint16_t IES                 	:	1;  /* Interrupt edge select for CEIIFG and CEIFG */
      __IO uint16_t SHORT               	:	1;  /* Input short. This bit shorts the + and - input terminals. */
      __IO uint16_t EX                  	:	1;  /* Exchange. This bit permutes the comparator 0 inputs and inverts the comparator 0 */
                                                /* output.   */
      __IO uint16_t FDLY                	:	2;  /* Filter delay. The filter delay can be selected in four steps. See the device- */
                                                /* specific data sheet for details. */
      __IO uint16_t PWRMD               	:	2;  /* Power Mode */
      __IO uint16_t ON                  	:	1;  /* On. This bit turns the comparator on. When the comparator is turned off the  */
                                                /* Comparator consumes no power. */
      __IO uint16_t MRVL                	:	1;  /* This bit is valid of CEMRVS is set to 1.   */
      __IO uint16_t MRVS                	:	1;  /* This bit defines if the comparator output selects between VREF0 or VREF1 if CERS */
                                                /* = 00, 01, or 10.   */
      __I  uint16_t RESERVED_1          	:	3;  /* Reserved */

    } bit;
  } CTL1;
  union {    /* COMP_E1_CTL2 Register */
    __IO uint16_t reg;
    struct { /* COMP_E1_CTL2 Control Bits */
      __IO uint16_t REF0                	:	5;  /* Reference resistor tap 0. This register defines the tap of the resistor string  */
                                                /* while CEOUT = 0. */
      __IO uint16_t RSEL                	:	1;  /* Reference select. This bit selects which terminal the V#sub#CCREF#/sub# is  */
                                                /* applied to. */
      __IO uint16_t RS                  	:	2;  /* Reference source. This bit define if the reference voltage is derived from VCC  */
                                                /* or from the precise shared reference. */
      __IO uint16_t REF1                	:	5;  /* Reference resistor tap 1. This register defines the tap of the resistor string  */
                                                /* while CEOUT = 1. */
      __IO uint16_t REFL                	:	2;  /* Reference voltage level   */
      __IO uint16_t REFACC              	:	1;  /* Reference accuracy. A reference voltage is requested only if CEREFL > 0.   */

    } bit;
  } CTL2;
  union {    /* COMP_E1_CTL3 Register */
    __IO uint16_t reg;
    struct { /* COMP_E1_CTL3 Control Bits */
      __IO uint16_t PD0                 	:	1;  /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                /* channel x. */
      __IO uint16_t PD1                 	:	1;  /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                /* channel x. */
      __IO uint16_t PD2                 	:	1;  /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                /* channel x. */
      __IO uint16_t PD3                 	:	1;  /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                /* channel x. */
      __IO uint16_t PD4                 	:	1;  /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                /* channel x. */
      __IO uint16_t PD5                 	:	1;  /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                /* channel x. */
      __IO uint16_t PD6                 	:	1;  /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                /* channel x. */
      __IO uint16_t PD7                 	:	1;  /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                /* channel x. */
      __IO uint16_t PD8                 	:	1;  /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                /* channel x. */
      __IO uint16_t PD9                 	:	1;  /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                /* channel x. */
      __IO uint16_t PD10                	:	1;  /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                /* channel x. */
      __IO uint16_t PD11                	:	1;  /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                /* channel x. */
      __IO uint16_t PD12                	:	1;  /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                /* channel x. */
      __IO uint16_t PD13                	:	1;  /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                /* channel x. */
      __IO uint16_t PD14                	:	1;  /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                /* channel x. */
      __IO uint16_t PD15                	:	1;  /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                /* channel x.   */

    } bit;
  } CTL3;
  uint8_t RESERVED0[4];
  union {    /* COMP_E1_INT Register */
    __IO uint16_t reg;
    struct { /* COMP_E1_INT Control Bits */
      __IO uint16_t IFG                 	:	1;  /* Comparator output interrupt flag. The bit CEIES defines the transition of the  */
                                                /* output setting this bit. */
      __IO uint16_t IIFG                	:	1;  /* Comparator output inverted interrupt flag. The bit CEIES defines the transition  */
                                                /* of the output setting this bit. */
      __I  uint16_t RESERVED_4          	:	2;  /* Reserved */
      __IO uint16_t RDYIFG              	:	1;  /* Comparator ready interrupt flag. This bit is set if the Comparator reference  */
                                                /* sources are settled and the Comparator module is operational. This bit has to be */
                                                /* cleared by software. */
      __I  uint16_t RESERVED_3          	:	3;  /* Reserved */
      __IO uint16_t IE                  	:	1;  /* Comparator output interrupt enable */
      __IO uint16_t IIE                 	:	1;  /* Comparator output interrupt enable inverted polarity */
      __I  uint16_t RESERVED_2          	:	2;  /* Reserved */
      __IO uint16_t RDYIE               	:	1;  /* Comparator ready interrupt enable   */
      __I  uint16_t RESERVED_1          	:	3;  /* Reserved */

    } bit;
  } INT;
  uint16_t IV; /*  */
} COMP_E1_Type;


/* COMP_E1_CTL0 Control Bits */
#define COMP_E1_CTL0_IPSEL__M                              (0x000f)              /* Channel input selected for the V+ terminal of the comparator if CEIPEN is set to */
                                                                                 /* 1.   */
#define COMP_E1_CTL0_RESERVED_2__M                         (0x0070)              /* Reserved. Always reads as 0. */
#define COMP_E1_CTL0_IPEN                                  (0x0080)              /* Channel input enable for the V+ terminal of the comparator. */
#define COMP_E1_CTL0_IMSEL__M                              (0x0f00)              /* Channel input selected for the - terminal of the comparator if CEIMEN is set to  */
                                                                                 /* 1.   */
#define COMP_E1_CTL0_RESERVED_1__M                         (0x7000)              /* Reserved. Always reads as 0. */
#define COMP_E1_CTL0_IMEN                                  (0x8000)              /* Channel input enable for the - terminal of the comparator. */

/* COMP_E1_CTL1 Control Bits */
#define COMP_E1_CTL1_OUT                                   (0x0001)              /* Output value. This bit reflects the value of the Comparator output. Writing this */
                                                                                 /* bit has no effect on the comparator output.   */
#define COMP_E1_CTL1_OUTPOL                                (0x0002)              /* Output polarity. This bit defines the CEOUT polarity. */
#define COMP_E1_CTL1_F                                     (0x0004)              /* Output filter. Available if CEPWRMD = 00, 01. */
#define COMP_E1_CTL1_IES                                   (0x0008)              /* Interrupt edge select for CEIIFG and CEIFG */
#define COMP_E1_CTL1_SHORT                                 (0x0010)              /* Input short. This bit shorts the + and - input terminals. */
#define COMP_E1_CTL1_EX                                    (0x0020)              /* Exchange. This bit permutes the comparator 0 inputs and inverts the comparator 0 */
                                                                                 /* output.   */
#define COMP_E1_CTL1_FDLY__M                               (0x00c0)              /* Filter delay. The filter delay can be selected in four steps. See the device- */
                                                                                 /* specific data sheet for details. */
#define COMP_E1_CTL1_FDLY__0                               (0x0000)              /* Typical filter delay of TBD (450) ns */
#define COMP_E1_CTL1_FDLY__1                               (0x0040)              /* Typical filter delay of TBD (900) ns */
#define COMP_E1_CTL1_FDLY__2                               (0x0080)              /* Typical filter delay of TBD (1800) ns */
#define COMP_E1_CTL1_FDLY__3                               (0x00c0)              /* Typical filter delay of TBD (3600) ns */
#define COMP_E1_CTL1_PWRMD__M                              (0x0300)              /* Power Mode */
#define COMP_E1_CTL1_PWRMD__0                              (0x0000)              /* High-speed mode */
#define COMP_E1_CTL1_PWRMD__0_HIGH_SPEED_MODE              (0x0000)              /* High-speed mode */
#define COMP_E1_CTL1_PWRMD__1                              (0x0100)              /* Normal mode */
#define COMP_E1_CTL1_PWRMD__1_NORMAL_MODE                  (0x0100)              /* Normal mode */
#define COMP_E1_CTL1_PWRMD__2                              (0x0200)              /* Ultra-low power mode */
#define COMP_E1_CTL1_ON                                    (0x0400)              /* On. This bit turns the comparator on. When the comparator is turned off the  */
                                                                                 /* Comparator consumes no power. */
#define COMP_E1_CTL1_MRVL                                  (0x0800)              /* This bit is valid of CEMRVS is set to 1.   */
#define COMP_E1_CTL1_MRVS                                  (0x1000)              /* This bit defines if the comparator output selects between VREF0 or VREF1 if CERS */
                                                                                 /* = 00, 01, or 10.   */
#define COMP_E1_CTL1_RESERVED_1__M                         (0xe000)              /* Reserved. Always reads as 0. */

/* COMP_E1_CTL2 Control Bits */
#define COMP_E1_CTL2_REF0__M                               (0x001f)              /* Reference resistor tap 0. This register defines the tap of the resistor string  */
                                                                                 /* while CEOUT = 0. */
#define COMP_E1_CTL2_RSEL                                  (0x0020)              /* Reference select. This bit selects which terminal the V#sub#CCREF#/sub# is  */
                                                                                 /* applied to. */
#define COMP_E1_CTL2_RS__M                                 (0x00c0)              /* Reference source. This bit define if the reference voltage is derived from VCC  */
                                                                                 /* or from the precise shared reference. */
#define COMP_E1_CTL2_RS__0                                 (0x0000)              /* No current is drawn by the reference circuitry. */
#define COMP_E1_CTL2_RS__1                                 (0x0040)              /* VCC applied to the resistor ladder */
#define COMP_E1_CTL2_RS__2                                 (0x0080)              /* Shared reference voltage applied to the resistor ladder. */
#define COMP_E1_CTL2_RS__3                                 (0x00c0)              /* Shared reference voltage supplied to V(CREF). Resistor ladder is off. */
#define COMP_E1_CTL2_REF1__M                               (0x1f00)              /* Reference resistor tap 1. This register defines the tap of the resistor string  */
                                                                                 /* while CEOUT = 1. */
#define COMP_E1_CTL2_REFL__M                               (0x6000)              /* Reference voltage level   */
#define COMP_E1_CTL2_REFL__0                               (0x0000)              /* Reference amplifier is disabled. No reference voltage is requested. */
#define COMP_E1_CTL2_REFL__1                               (0x2000)              /* 1.2 V is selected as shared reference voltage input */
#define COMP_E1_CTL2_REFL__2                               (0x4000)              /* 2.0 V is selected as shared reference voltage input */
#define COMP_E1_CTL2_REFL__3                               (0x6000)              /* 2.5 V is selected as shared reference voltage input */
#define COMP_E1_CTL2_REFACC                                (0x8000)              /* Reference accuracy. A reference voltage is requested only if CEREFL > 0.   */

/* COMP_E1_CTL3 Control Bits */
#define COMP_E1_CTL3_PD0                                   (0x0001)              /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                                                 /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                                                 /* channel x. */
#define COMP_E1_CTL3_PD1                                   (0x0002)              /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                                                 /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                                                 /* channel x. */
#define COMP_E1_CTL3_PD2                                   (0x0004)              /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                                                 /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                                                 /* channel x. */
#define COMP_E1_CTL3_PD3                                   (0x0008)              /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                                                 /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                                                 /* channel x. */
#define COMP_E1_CTL3_PD4                                   (0x0010)              /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                                                 /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                                                 /* channel x. */
#define COMP_E1_CTL3_PD5                                   (0x0020)              /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                                                 /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                                                 /* channel x. */
#define COMP_E1_CTL3_PD6                                   (0x0040)              /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                                                 /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                                                 /* channel x. */
#define COMP_E1_CTL3_PD7                                   (0x0080)              /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                                                 /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                                                 /* channel x. */
#define COMP_E1_CTL3_PD8                                   (0x0100)              /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                                                 /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                                                 /* channel x. */
#define COMP_E1_CTL3_PD9                                   (0x0200)              /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                                                 /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                                                 /* channel x. */
#define COMP_E1_CTL3_PD10                                  (0x0400)              /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                                                 /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                                                 /* channel x. */
#define COMP_E1_CTL3_PD11                                  (0x0800)              /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                                                 /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                                                 /* channel x. */
#define COMP_E1_CTL3_PD12                                  (0x1000)              /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                                                 /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                                                 /* channel x. */
#define COMP_E1_CTL3_PD13                                  (0x2000)              /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                                                 /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                                                 /* channel x. */
#define COMP_E1_CTL3_PD14                                  (0x4000)              /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                                                 /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                                                 /* channel x. */
#define COMP_E1_CTL3_PD15                                  (0x8000)              /* Port disable. Enables or disables the input buffer for the pin of the port  */
                                                                                 /* associated with Comparator. The bit CEPDx disabled the port of the comparator  */
                                                                                 /* channel x.   */

/* COMP_E1_INT Control Bits */
#define COMP_E1_INT_IFG                                    (0x0001)              /* Comparator output interrupt flag. The bit CEIES defines the transition of the  */
                                                                                 /* output setting this bit. */
#define COMP_E1_INT_IIFG                                   (0x0002)              /* Comparator output inverted interrupt flag. The bit CEIES defines the transition  */
                                                                                 /* of the output setting this bit. */
#define COMP_E1_INT_RESERVED_4__M                          (0x000c)              /* Reserved. Always reads as 0. */
#define COMP_E1_INT_RDYIFG                                 (0x0010)              /* Comparator ready interrupt flag. This bit is set if the Comparator reference  */
                                                                                 /* sources are settled and the Comparator module is operational. This bit has to be */
                                                                                 /* cleared by software. */
#define COMP_E1_INT_RESERVED_3__M                          (0x00e0)              /* Reserved. Always reads as 0. */
#define COMP_E1_INT_IE                                     (0x0100)              /* Comparator output interrupt enable */
#define COMP_E1_INT_IIE                                    (0x0200)              /* Comparator output interrupt enable inverted polarity */
#define COMP_E1_INT_RESERVED_2__M                          (0x0c00)              /* Reserved. Always reads as 0. */
#define COMP_E1_INT_RDYIE                                  (0x1000)              /* Comparator ready interrupt enable   */
#define COMP_E1_INT_RESERVED_1__M                          (0xe000)              /* Reserved. Always reads as 0. */


//*****************************************************************************
// CRC32 Registers
//*****************************************************************************
typedef struct {
  uint16_t CRC32DI; /* CRC32 Data Input Low */
  uint8_t RESERVED0[2];
  uint16_t CRC32DIRB; /* CRC32 Data In Reverse Low */
  uint8_t RESERVED1[2];
  uint16_t CRC32INIRES_LO; /* Data written to this register represents the seed for the CRC calculation. This  */
                                                                                 /* Register always reflects the latest signature of the values collected so far. */
  uint16_t CRC32INIRES_HI; /* Data written to this register represents the seed for the CRC calculation. This  */
                                                                                 /* Register always reflects the latest signature of the values collected so far. */
  uint16_t CRC32RESR_LO; /* Data written to this register represents the seed for the CRC calculation. This  */
                                                                                 /* Register always reflects the latest signature of the values collected so far. */
  uint16_t CRC32RESR_HI; /* Data written to this register represents the seed for the CRC calculation. This  */
                                                                                 /* Register always reflects the latest signature of the values collected so far. */
  uint16_t CRC16DI; /* Data written to this register will be taken into CRC16 signature calculations. */
  uint8_t RESERVED2[2];
  uint16_t CRC16DIRB; /* Data written to this register will be taken into CRC16 signature calculations.  */
                                                                                 /* This register can be accessed 8-bit wide and 16bit wide and on MSP432 families  */
                                                                                 /* 32 bit wide. */
  uint8_t RESERVED3[2];
  uint16_t CRC16INIRES; /* CRC16 Init and Result */
  uint8_t RESERVED4[4];
  uint16_t CRC16RESR; /* Data written to this register represents the seed for the CRC calculation. This  */
                                                                                 /* Register always reflects the latest signature of the values collected so far.  */
                                                                                 /* This register can be accessed 8-bit wide and 16bit wide and on MSP432 families  */
                                                                                 /* 32 bit wide. */
} CRC32_Type;



//*****************************************************************************
// CS Registers
//*****************************************************************************
typedef struct {
  union {    /* CS_ACC Register */
    __IO uint32_t reg;
    struct { /* CS_ACC Control Bits */
      __IO uint32_t KEY                 	:	16;  /* Write CSKEY = xxxx_695Ah to unlock CSCTL0, CSCTL1, CSCTL2, CSCTL3, CSCTL4,  */
                                                /* CSCTL6, CSIE, CSSETIFG, and CSCLRIFG registers. All 16 LSBs need to be written  */
                                                /* together to unlock the registers mentioned above. Writing CSKEY with any other  */
                                                /* value causes CSCTL0, CSCTL1, CSCTL2, CSCTL3, CSCTL4, CSCTL6, CSIE, and CSCLRIFG  */
                                                /* registers to be locked and any writes to these registers is ignored while reads  */
                                                /* will still go through. Always reads back A596h. */
      __I  uint32_t RESERVED1           	:	16;  /* Reserved */

    } bit;
  } ACC;
  union {    /* CS_CTL0 Register */
    __IO uint32_t reg;
    struct { /* CS_CTL0 Control Bits */
      __IO uint32_t DCOTUNE             	:	13;  /* DCO frequency tuning select. Two's complement representation. Value represents  */
                                                /* an offset from the calibrated center frequency for the range selected by the  */
                                                /* DCORSEL bits. */
      __I  uint32_t RESERVED3           	:	3;  /* Reserved */
      __IO uint32_t DCORSEL             	:	3;  /* DCO frequency range select. Selects frequency range settings for the DCO. 110b  */
                                                /* to 111b = Nominal DCO Frequency (MHz): Reserved--defaults to 1.5 when selected;  */
                                                /* Nominal DCO Frequency Range (MHz): Reserved--defaults to 1 to 2 when selected. */
      __I  uint32_t RESERVED2           	:	3;  /* Reserved */
      __IO uint32_t DCORES              	:	1;  /* Enables the DCO external resistor mode. */
      __IO uint32_t DCOEN               	:	1;  /* Enables the DCO oscillator regardless if used as a clock resource. */
      __IO uint32_t DIS_DCO_DELAY_CNT   	:	1;  /* Setting this bit disabes the DCO settling counter value. When this is set the  */
                                                /* clocks are directly fed to the digital system from the DCO while the frequency  */
                                                /* changes are ongoing. 0b = DCO Settling delay counter is enabled. The application */
                                                /* can change to any frequency desired as long as it is supported by the power  */
                                                /* mode. 1b = DCO setlling delay counter disabled. The application should ensure  */
                                                /* that the frequency changes requested are in step size of 5MHz. */
      __I  uint32_t RESERVED1           	:	7;  /* Reserved */

    } bit;
  } CTL0;
  union {    /* CS_CTL1 Register */
    __IO uint32_t reg;
    struct { /* CS_CTL1 Control Bits */
      __IO uint32_t SELM                	:	3;  /* Selects the MCLK source. */
      __I  uint32_t RESERVED8           	:	1;  /* Reserved */
      __IO uint32_t SELS                	:	3;  /* Selects the SMCLK and HSMCLK source. */
      __I  uint32_t RESERVED7           	:	1;  /* Reserved */
      __IO uint32_t SELA                	:	3;  /* Selects the ACLK source. 011b-111b = Reserved for future use. Defaults to  */
                                                /* REFOCLK. Not recommended for use to ensure future compatibilities. */
      __I  uint32_t RESERVED6           	:	1;  /* Reserved */
      __IO uint32_t SELB                	:	1;  /* Selects the BCLK source. */
      __I  uint32_t RESERVED5           	:	3;  /* Reserved */
      __IO uint32_t DIVM                	:	3;  /* MCLK source divider. */
      __I  uint32_t RESERVED4           	:	1;  /* Reserved */
      __IO uint32_t DIVHS               	:	3;  /* HSMCLK source divider. */
      __I  uint32_t RESERVED3           	:	1;  /* Reserved */
      __IO uint32_t DIVA                	:	3;  /* ACLK source divider. */
      __I  uint32_t RESERVED2           	:	1;  /* Reserved */
      __IO uint32_t DIVS                	:	3;  /* SMCLK source divider. */
      __I  uint32_t RESERVED1           	:	1;  /* Reserved */

    } bit;
  } CTL1;
  union {    /* CS_CTL2 Register */
    __IO uint32_t reg;
    struct { /* CS_CTL2 Control Bits */
      __IO uint32_t LFXTDRIVE           	:	3;  /* The LFXT oscillator current can be adjusted to its drive needs. Reset value is  */
                                                /* 7h when LFXT available, and 0h when LFXT not available. */
      __IO uint32_t RESERVED6           	:	4;  /* Reserved */
      __IO uint32_t LFXTAGCOFF          	:	1;  /* LFXT AGC off. Setting this bit disables the automatic gain control of the LFXT  */
                                                /* crystal. This is currently a TI internal register bit. */
      __IO uint32_t LFXT_EN             	:	1;  /* Turns on the LFXT oscillator regardless if used as a clock resource. */
      __IO uint32_t LFXTBYPASS          	:	1;  /* LFXT bypass select. */
      __I  uint32_t RESERVED5           	:	6;  /* Reserved */
      __IO uint32_t HFXTDRIVE           	:	1;  /* HFXT oscillator drive selection. 0b = To be used for HFXTFREQ setting 000b 1b =  */
                                                /* To be used for HFXTFREQ settings 001b to 111b Reset value is 1h when HFXT  */
                                                /* available, and 0h when HFXT not available. */
      __IO uint32_t RESERVED4           	:	2;  /* Reserved */
      __I  uint32_t RESERVED3           	:	1;  /* Reserved */
      __IO uint32_t HFXTFREQ            	:	3;  /* The HFXT frequency selection. These bits must be set to the appropriate  */
                                                /* frequency for crystal or bypass modes of operation. */
      __I  uint32_t RESERVED2           	:	1;  /* Reserved */
      __IO uint32_t HFXT_EN             	:	1;  /* Turns on the HFXT oscillator regardless if used as a clock resource. */
      __IO uint32_t HFXTBYPASS          	:	1;  /* HFXT bypass select. */
      __I  uint32_t RESERVED1           	:	6;  /* Reserved */

    } bit;
  } CTL2;
  union {    /* CS_CTL3 Register */
    __IO uint32_t reg;
    struct { /* CS_CTL3 Control Bits */
      __IO uint32_t FCNTLF              	:	2;  /* Start flag counter for LFXT. Selects number of LFXT cycles before LFXTOFFG can  */
                                                /* be cleared. Only available on devices with LFXT. Reset value is 3h when LFXT  */
                                                /* available, and 0h when LFXT not available. */
      __O  uint32_t RFCNTLF             	:	1;  /* Reset start fault counter for LFXT. Write 1 only. Self clears once written. Only */
                                                /* available on devices with LFXT. */
      __IO uint32_t FCNTLF_EN           	:	1;  /*  */
      __IO uint32_t FCNTHF              	:	2;  /* Start flag counter for HFXT. Selects number of HFXT cycles before HFXTOFFG can  */
                                                /* be cleared. Only available on devices with HFXT. Reset value is 3h when HFXT  */
                                                /* available, and 0h when HFXT not available. */
      __O  uint32_t RFCNTHF             	:	1;  /* Reset start fault counter for HFXT. Write 1 only. Self clears once written. Only */
                                                /* available on devices with HFXT. */
      __IO uint32_t FCNTHF_EN           	:	1;  /* Enable start fault counter for HFXT. Only available on devices with HFXT. Reset  */
                                                /* value is 1h when HFXT available, and 0h when HFXT not available. */
      __IO uint32_t FCNTHF2             	:	2;  /* Start flag counter for HFXT2. Selects number of HFXT2 cycles before HFXT2OFFG  */
                                                /* can be cleared. Only available on devices with HFXT2. Reset value is 3h when  */
                                                /* HFXT2 available, and 0h when HFXT2 not available. */
      __O  uint32_t RFCNTHF2            	:	1;  /* Reset start fault counter for HFXT2. Write 1 only. Self clears once written.  */
                                                /* Only available on devices with HFXT2. */
      __IO uint32_t FCNTHF2_EN          	:	1;  /* Enable start fault counter for HFXT2. Only available on devices with HFXT2.  */
                                                /* Reset value is 1h when HFXT2 available, and 0h when HFXT2 not available. */
      __I  uint32_t RESERVED1           	:	20;  /* Reserved */

    } bit;
  } CTL3;
  union {    /* CS_CTL4 Register */
    __IO uint32_t reg;
    struct { /* CS_CTL4 Control Bits */
      __IO uint32_t HFXT2DRIVE          	:	1;  /* The HFXT2 oscillator current can be adjusted to its drive needs. This in  */
                                                /* combination with the HFXT2FREQ bits can be used for optimizing crystal power  */
                                                /* based on crystal characteristics. Reset value is 7h when HFXT2 available, and 0h */
                                                /* when HFXT2 not available. */
      __IO uint32_t RESERVED4           	:	2;  /* Reserved */
      __I  uint32_t RESERVED3           	:	1;  /* Reserved */
      __IO uint32_t HFXT2FREQ           	:	3;  /* The HFXT2 frequency selection. These bits must be set to the appropriate  */
                                                /* frequency for crystal or bypass modes of operation. */
      __I  uint32_t RESERVED2           	:	1;  /* Reserved */
      __IO uint32_t HFXT2_EN            	:	1;  /* Turns on the HFXT2 oscillator regardless if used as a clock resource. */
      __IO uint32_t HFXT2BYPASS         	:	1;  /* HFXT2 bypass select. */
      __I  uint32_t RESERVED1           	:	22;  /* Reserved */

    } bit;
  } CTL4;
  union {    /* CS_CTL5 Register */
    __IO uint32_t reg;
    struct { /* CS_CTL5 Control Bits */
      __IO uint32_t REFCNTSEL           	:	3;  /* Reference counter source select. 100b-111b = Reserved for future use. Not  */
                                                /* recommended for use to ensure future compatibilities. */
      __IO uint32_t REFCNTPS            	:	3;  /* Reference clock prescaler */
      __I  uint32_t RESERVED2           	:	1;  /* Reserved */
      __O  uint32_t CALSTART            	:	1;  /* Start clock calibration counters. Bit is self clearing. */
      __IO uint32_t PERCNTSEL           	:	3;  /* Period counter source select. 010b-111b = Reserved for future use. Not  */
                                                /* recommended for use to ensure future compatibilities. */
      __I  uint32_t RESERVED1           	:	21;  /* Reserved */

    } bit;
  } CTL5;
  union {    /* CS_CTL6 Register */
    __IO uint32_t reg;
    struct { /* CS_CTL6 Control Bits */
      __I  uint32_t PERCNT              	:	16;  /* Calibration period counter. Read only. */
      __I  uint32_t RESERVED1           	:	16;  /* Reserved */

    } bit;
  } CTL6;
  union {    /* CS_CTL7 Register */
    __IO uint32_t reg;
    struct { /* CS_CTL7 Control Bits */
      __IO uint32_t REFCNT              	:	16;  /* Calibration reference period counter. */
      __I  uint32_t RESERVED1           	:	16;  /* Reserved */

    } bit;
  } CTL7;
  uint8_t RESERVED0[12];
  union {    /* CS_CLKEN Register */
    __IO uint32_t reg;
    struct { /* CS_CLKEN Control Bits */
      __IO uint32_t ACLK_EN             	:	1;  /* ACLK system clock conditional request enable */
      __IO uint32_t MCLK_EN             	:	1;  /* MCLK system clock conditional request enable */
      __IO uint32_t HSMCLK_EN           	:	1;  /* HSMCLK system clock conditional request enable */
      __IO uint32_t SMCLK_EN            	:	1;  /* SMCLK system clock conditional request enable */
      __I  uint32_t RESERVED3           	:	4;  /* Reserved */
      __IO uint32_t VLO_EN              	:	1;  /* Turns on the VLO oscillator regardless if used as a clock resource. */
      __IO uint32_t REFO_EN             	:	1;  /* Turns on the REFO oscillator regardless if used as a clock resource. */
      __IO uint32_t MODOSC_EN           	:	1;  /* Turns on the MODOSC oscillator regardless if used as a clock resource. */
      __I  uint32_t RESERVED2           	:	4;  /* Reserved */
      __IO uint32_t REFOFSEL            	:	1;  /* Selects REFO nominal frequency. */
      __I  uint32_t RESERVED1           	:	16;  /* Reserved */

    } bit;
  } CLKEN;
  union {    /* CS_STAT Register */
    __IO uint32_t reg;
    struct { /* CS_STAT Control Bits */
      __I  uint32_t DCO_ON              	:	1;  /* DCO status */
      __I  uint32_t DCOBIAS_ON          	:	1;  /* DCO bias status */
      __I  uint32_t HFXT_ON             	:	1;  /* HFXT status. Only available on devices with HFXT. */
      __I  uint32_t HFXT2_ON            	:	1;  /* HFXT2 status. Only available on devices with HFXT2. */
      __I  uint32_t MODOSC_ON           	:	1;  /* MODOSC status */
      __I  uint32_t VLO_ON              	:	1;  /* VLO status */
      __I  uint32_t LFXT_ON             	:	1;  /* LFXT status. Only available on devices with LFXT. */
      __I  uint32_t REFO_ON             	:	1;  /* REFO status */
      __I  uint32_t RESERVED2           	:	8;  /* Reserved */
      __I  uint32_t ACLK_ON             	:	1;  /* ACLK system clock status */
      __I  uint32_t MCLK_ON             	:	1;  /* MCLK system clock status */
      __I  uint32_t HSMCLK_ON           	:	1;  /* HSMCLK system clock status */
      __I  uint32_t SMCLK_ON            	:	1;  /* SMCLK system clock status */
      __I  uint32_t MODCLK_ON           	:	1;  /* MODCLK system clock status */
      __I  uint32_t VLOCLK_ON           	:	1;  /* VLOCLK system clock status */
      __I  uint32_t LFXTCLK_ON          	:	1;  /* LFXTCLK system clock status */
      __I  uint32_t REFOCLK_ON          	:	1;  /* REFOCLK system clock status */
      __I  uint32_t ACLK_READY          	:	1;  /* ACLK Ready status. This bit indicates whether the clock is stable after a change */
                                                /* in the frequency/divider settings. 0b = Not ready 1b = Ready */
      __I  uint32_t MCLK_READY          	:	1;  /* MCLK Ready status. This bit indicates whether the clock is stable after a change */
                                                /* in the frequency/divider settings. 0b = Not ready 1b = Ready */
      __I  uint32_t HSMCLK_READY        	:	1;  /* HSMCLK Ready status. This bit indicates whether the clock is stable after a  */
                                                /* change in the frequency/divider settings. 0b = Not ready 1b = Ready */
      __I  uint32_t SMCLK_READY         	:	1;  /* SMCLK Ready status. This bit indicates whether the clock is stable after a  */
                                                /* change in the frequency/divider settings. 0b = Not ready 1b = Ready */
      __I  uint32_t BCLK_READY          	:	1;  /* BCLK Ready status. This bit indicates whether the clock is stable after a change */
                                                /* in the frequency settings. 0b = Not ready 1b = Ready */
      __I  uint32_t RESERVED1           	:	3;  /* Reserved */

    } bit;
  } STAT;
  uint8_t RESERVED1[8];
  union {    /* CS_IE Register */
    __IO uint32_t reg;
    struct { /* CS_IE Control Bits */
      __IO uint32_t LFXTIE              	:	1;  /* LFXT oscillator fault flag interrupt enable. Only availabe on devices with LFXT. */
      __IO uint32_t HFXTIE              	:	1;  /* HFXT oscillator fault flag interrupt enable. Only availabe on devices with HFXT. */
      __IO uint32_t HFXT2IE             	:	1;  /* HFXT2 oscillator fault flag interrupt enable. Only availabe on devices with  */
                                                /* HFXT2. */
      __I  uint32_t RESERVED4           	:	1;  /* Reserved */
      __IO uint32_t DCOMINIE            	:	1;  /* DCO minimum fault flag interrupt enable. */
      __IO uint32_t DCOMAXIE            	:	1;  /* DCO maximum fault flag interrupt enable. */
      __IO uint32_t DCORIE              	:	1;  /* DCO external resistor fault flag interrupt enable. */
      __I  uint32_t RESERVED3           	:	1;  /* Reserved */
      __IO uint32_t FCNTLFIE            	:	1;  /* Start fault counter interrupt enable LFXT. Only availabe on devices with LFXT. */
      __IO uint32_t FCNTHFIE            	:	1;  /* Start fault counter interrupt enable HFXT. Only availabe on devices with HFXT. */
      __I  uint32_t RESERVED1           	:	22;  /* Reserved */
      __IO uint32_t FCNTHF2IE           	:	1;  /* Start fault counter interrupt enable HFXT2. Only availabe on devices with HFXT2. */
      __I  uint32_t RESERVED2           	:	1;  /* Reserved */
      __IO uint32_t PLLOOLIE            	:	1;  /* PLL out-of-lock interrupt enable. Only availabe on devices with a PLL. */
      __IO uint32_t PLLLOSIE            	:	1;  /* PLL loss-of-signal interrupt enable. Only available on devices with a PLL. */
      __IO uint32_t PLLOORIE            	:	1;  /* PLL out-of-range interrupt enable. Only available on devices with a PLL. */
      __IO uint32_t CALIE               	:	1;  /* REFCNT period counter interrupt enable. */

    } bit;
  } IE;
  uint8_t RESERVED2[4];
  union {    /* CS_IFG Register */
    __IO uint32_t reg;
    struct { /* CS_IFG Control Bits */
      __I  uint32_t LFXTIFG             	:	1;  /* LFXT oscillator fault flag. If this bit is set, the OFIFG flag is also set.  */
                                                /* LFXTIFG is set if a LFXT fault condition exists. LFXTIFG can be cleared via  */
                                                /* software by CLR_LFXTIFG. If the LFXT fault condition still remains, LFXTIFG is  */
                                                /* set again. Only available on devices with LFXT. */
      __I  uint32_t HFXTIFG             	:	1;  /* HFXT oscillator fault flag. If this bit is set, the OFIFG flag is also set.  */
                                                /* HFXTIFG is set if a HFXT fault condition exists. HFXTIFG can be cleared via  */
                                                /* software by CLR_HFXTIFG. If the HFXT fault condition still remains, HFXTIFG is  */
                                                /* set again. Only available on devices with HFXT. */
      __I  uint32_t HFXT2IFG            	:	1;  /* NOTE: This bit is reserved for HFXT2 implementations. Do not use for any other  */
                                                /* purpose. HFXT2 oscillator fault flag. If this bit is set, the OFIFG flag is also */
                                                /* set. HFXT2IFG is set if a HFXT2 fault condition exists. HFXT2IFG can be cleared */
                                                /* via software by CLR_HFXT2IFG. If the HFXT2 fault condition still remains,  */
                                                /* HFXT2IFG is set again. Only available on devices with HFXT2. */
      __I  uint32_t RESERVED4           	:	1;  /* Reserved */
      __I  uint32_t DCOMINIFG           	:	1;  /* DCO minimum fault flag. If this bit is set, the OFIFG flag is also set.  */
                                                /* DCOMINIFG can be cleared via software by CLR_DCOMINIFG. If the DCO minimum fault */
                                                /* condition still remains, DCOMINIFG is set again. */
      __I  uint32_t DCOMAXIFG           	:	1;  /* DCO maximum fault flag. If this bit is set, the OFIFG flag is also set.  */
                                                /* DCOMAXIFG can be cleared via software by CLR_DCOMAXIFG. If the DCO maximum fault */
                                                /* condition still remains, DCOMAXIFG is set again. */
      __I  uint32_t DCORIFG             	:	1;  /* DCO external resistor fault flag. If this bit is set, the OFIFG flag is also set */
                                                /* . DCORIFG can be cleared via software by CLR_DCORIFG. If the fault condition  */
                                                /* still remains, DCORIFG is set again. */
      __I  uint32_t RESERVED3           	:	1;  /* Reserved */
      __I  uint32_t FCNTLFIFG           	:	1;  /* Start fault counter interrupt flag LFXT. If this bit is set, the FCNTIFG flag is */
                                                /* also set. Only availabe on devices with a LFXT. */
      __I  uint32_t FCNTHFIFG           	:	1;  /* Start fault counter interrupt flag HFXT. If this bit is set, the FCNTIFG flag is */
                                                /* also set. Only availabe on devices with a HFXT. */
      __I  uint32_t RESERVED1           	:	22;  /* Reserved */
      __I  uint32_t RESERVED2           	:	1;  /* Reserved */
      __I  uint32_t FCNTHF2IFG          	:	1;  /* NOTE: This bit is reserved for HFXT2 implementations. Do not use for any other  */
                                                /* purpose. Start fault counter interrupt flag HFXT2. If this bit is set, the  */
                                                /* FCNTIFG flag is also set. Only availabe on devices with a HFXT2. */
      __I  uint32_t PLLOOLIFG           	:	1;  /* NOTE: This bit is reserved for PLL implementations. Do not use for any other  */
                                                /* purpose. PLL out-of-lock interrupt flag. If this bit is set, the PLLIFG flag is  */
                                                /* also set. Only availabe on devices with a PLL. */
      __I  uint32_t PLLLOSIFG           	:	1;  /* NOTE: This bit is reserved for PLL implementations. Do not use for any other  */
                                                /* purpose. PLL loss-of-signal interrupt flag. If this bit is set, the PLLIFG flag  */
                                                /* is also set. Only available on devices with a PLL. */
      __I  uint32_t PLLOORIFG           	:	1;  /* NOTE: This bit is reserved for PLL implementations. Do not use for any other  */
                                                /* purpose. PLL out-of-range interrupt flag. If this bit is set, the PLLIFG flag is */
                                                /* also set. Only available on devices with a PLL. */
      __I  uint32_t CALIFG              	:	1;  /* REFCNT period counter expired. Contents in PERCNT is available for reading. If  */
                                                /* this bit is set, the FCNTIFG flag is also set. This bit is reserved for CALIFG  */
                                                /* implementations. Do not use for any other purpose. */

    } bit;
  } IFG;
  uint8_t RESERVED3[4];
  union {    /* CS_CLRIFG Register */
    __IO uint32_t reg;
    struct { /* CS_CLRIFG Control Bits */
      __O  uint32_t CLR_LFXTIFG         	:	1;  /* Clear LFXT oscillator fault interrupt flag. Does not clear OFIFG. Only available */
                                                /* on devices with LFXT. */
      __O  uint32_t CLR_HFXTIFG         	:	1;  /* Clear HFXT oscillator fault interrupt flag. Does not clear OFIFG. Only available */
                                                /* on devices with HFXT. */
      __O  uint32_t CLR_HFXT2IFG        	:	1;  /* NOTE: This bit is reserved for HFXT2 implementations. Do not use for any other  */
                                                /* purpose. Clear HFXT2 oscillator fault interrupt flag. Does not clear OFIFG. Only */
                                                /* available on devices with HFXT2. */
      __I  uint32_t RESERVED3           	:	1;  /* Reserved */
      __O  uint32_t CLR_DCOMINIFG       	:	1;  /* Clear DCO minimum fault interrupt flag. Does not clear OFIFG. */
      __O  uint32_t CLR_DCOMAXIFG       	:	1;  /* Clear DCO maximum fault interrupt flag. Does not clear OFIFG. */
      __O  uint32_t CLR_DCORIFG         	:	1;  /* Clear DCO external resistor fault interrupt flag. Does not clear OFIFG. */
      __O  uint32_t CLR_CALIFG          	:	1;  /* NOTE: This bit is reserved for CLR_CALIFG implementations. Do not use for any  */
                                                /* other purpose. REFCNT period counter clear interrupt flag. expired. Does not  */
                                                /* clear FCNTIFG. */
      __O  uint32_t CLR_FCNTLFIFG       	:	1;  /* Start fault counter clear interrupt flag LFXT. Does not clear FCNTIFG. Only  */
                                                /* available on devices with LFXT. */
      __O  uint32_t CLR_FCNTHFIFG       	:	1;  /* Start fault counter clear interrupt flag HFXT. Does not clear FCNTIFG. Only  */
                                                /* available on devices with HFXT. */
      __O  uint32_t RESERVED1           	:	22;  /* Reserved */
      __O  uint32_t CLR_FCNTHF2IFG      	:	1;  /* NOTE: This bit is reserved for HFXT2 implementations. Do not use for any other  */
                                                /* purpose. Start fault counter clear interrupt flag HFXT2. Does not clearFCNTIFG.  */
                                                /* Only available on devices with HFXT2. */
      __O  uint32_t RESERVED2           	:	1;  /* Reserved */
      __O  uint32_t CLR_PLLOOLIFG       	:	1;  /* NOTE: This bit is reserved for PLL implementations. Do not use for any other  */
                                                /* purpose. PLL out-of-lock clear interrupt flag. Does not clear PLLIFG. Only  */
                                                /* available on devices with a PLL. */
      __O  uint32_t CLR_PLLLOSIFG       	:	1;  /* NOTE: This bit is reserved for PLL implementations. Do not use for any other  */
                                                /* purpose. PLL loss-of-signal clear interrupt flag. Does not clear PLLIFG. Only  */
                                                /* available on devices with a PLL. */
      __O  uint32_t CLR_PLLOORIFG       	:	1;  /* NOTE: This bit is reserved for PLL implementations. Do not use for any other  */
                                                /* purpose. PLL out-of-range clear interrupt flag. Does not clear PLLIFG. Only  */
                                                /* available on devices with a PLL. */

    } bit;
  } CLRIFG;
  uint8_t RESERVED4[4];
  union {    /* CS_SETIFG Register */
    __IO uint32_t reg;
    struct { /* CS_SETIFG Control Bits */
      __I  uint32_t SET_LFXTIFG         	:	1;  /* Set LFXT oscillator fault interrupt flag. Only available on devices with LFXT. */
      __I  uint32_t SET_HFXTIFG         	:	1;  /* Set HFXT oscillator fault interrupt flag. Only available on devices with HFXT. */
      __I  uint32_t SET_HFXT2IFG        	:	1;  /* NOTE: This bit is reserved for HFXT2 implementations. Do not use for any other  */
                                                /* purpose. Set HFXT2 oscillator fault interrupt flag. Only available on devices  */
                                                /* with HFXT2. */
      __I  uint32_t RESERVED3           	:	1;  /* Reserved */
      __I  uint32_t SET_DCOMINIFG       	:	1;  /* Set DCO minimum fault interrupt flag. */
      __I  uint32_t SET_DCOMAXIFG       	:	1;  /* Set DCO maximum fault interrupt flag. */
      __I  uint32_t SET_DCORIFG         	:	1;  /* Set DCO external resistor fault interrupt flag. */
      __I  uint32_t SET_CALIFG          	:	1;  /* NOTE: This bit is reserved for SET_CALIFG implementations. Do not use for any  */
                                                /* other purpose. REFCNT period counter set interrupt flag. expired. */
      __I  uint32_t SET_FCNTLFIFG       	:	1;  /* Start fault counter set interrupt flag LFXT. Only available on devices with LFXT */
                                                /* . */
      __I  uint32_t SET_FCNTHFIFG       	:	1;  /* Start fault counter set interrupt flag HFXT. Only available on devices with HFXT */
                                                /* . */
      __I  uint32_t SET_FCNTHF2IFG      	:	1;  /* NOTE: This bit is reserved for HFXT2 implementations. Do not use for any other  */
                                                /* purpose. Start fault counter set interrupt flag HFXT2. Only available on devices */
                                                /* with HFXT2. */
      __I  uint32_t RESERVED2           	:	1;  /* Reserved */
      __I  uint32_t SET_PLLOOLIFG       	:	1;  /* NOTE: This bit is reserved for PLL implementations. Do not use for any other  */
                                                /* purpose. PLL out-of-lock set interrupt flag. Only available on devices with a  */
                                                /* PLL. */
      __I  uint32_t SET_PLLLOSIFG       	:	1;  /* NOTE: This bit is reserved for PLL implementations. Do not use for any other  */
                                                /* purpose. PLL loss-of-signal set interrupt flag. Only available on devices with a */
                                                /* PLL. */
      __I  uint32_t SET_PLLOORIFG       	:	1;  /* NOTE: This bit is reserved for PLL implementations. Do not use for any other  */
                                                /* purpose. PLL out-of-range set interrupt flag. Only available on devices with a  */
                                                /* PLL. */
      __I  uint32_t RESERVED1           	:	17;  /* Reserved */

    } bit;
  } SETIFG;
  uint8_t RESERVED5[4];
  union {    /* CS_DCOERCAL Register */
    __IO uint32_t reg;
    struct { /* CS_DCOERCAL Control Bits */
      __IO uint32_t DCO_TCTRIM          	:	2;  /* DCO Temperature compensation Trim */
      __I  uint32_t RESERVED2           	:	14;  /* Reserved */
      __IO uint32_t DCO_FTRIM           	:	11;  /* DCO frequency trim */
      __I  uint32_t RESERVED1           	:	5;  /* Reserved */

    } bit;
  } DCOERCAL;
} CS_Type;


/* CS_ACC Control Bits */
#define CS_ACC_KEY__M                                      (0x0000ffff)          /* Write CSKEY = xxxx_695Ah to unlock CSCTL0, CSCTL1, CSCTL2, CSCTL3, CSCTL4,  */
                                                                                 /* CSCTL6, CSIE, CSSETIFG, and CSCLRIFG registers. All 16 LSBs need to be written  */
                                                                                 /* together to unlock the registers mentioned above. Writing CSKEY with any other  */
                                                                                 /* value causes CSCTL0, CSCTL1, CSCTL2, CSCTL3, CSCTL4, CSCTL6, CSIE, and CSCLRIFG  */
                                                                                 /* registers to be locked and any writes to these registers is ignored while reads  */
                                                                                 /* will still go through. Always reads back A596h. */
#define CS_ACC_RESERVED1__M                                (0xffff0000)          /*  */

/* CS_CTL0 Control Bits */
#define CS_CTL0_DCOTUNE__M                                 (0x00001fff)          /* DCO frequency tuning select. Two's complement representation. Value represents  */
                                                                                 /* an offset from the calibrated center frequency for the range selected by the  */
                                                                                 /* DCORSEL bits. */
#define CS_CTL0_RESERVED3__M                               (0x0000e000)          /*  */
#define CS_CTL0_DCORSEL__M                                 (0x00070000)          /* DCO frequency range select. Selects frequency range settings for the DCO. 110b  */
                                                                                 /* to 111b = Nominal DCO Frequency (MHz): Reserved--defaults to 1.5 when selected;  */
                                                                                 /* Nominal DCO Frequency Range (MHz): Reserved--defaults to 1 to 2 when selected. */
#define CS_CTL0_DCORSEL__0                                 (0x00000000)          /* Nominal DCO Frequency (MHz): 1.5; Nominal DCO Frequency Range (MHz): 1 to 2 */
#define CS_CTL0_DCORSEL__1                                 (0x00010000)          /* Nominal DCO Frequency (MHz): 3; Nominal DCO Frequency Range (MHz): 2 to 4 */
#define CS_CTL0_DCORSEL__2                                 (0x00020000)          /* Nominal DCO Frequency (MHz): 6; Nominal DCO Frequency Range (MHz): 4 to 8 */
#define CS_CTL0_DCORSEL__3                                 (0x00030000)          /* Nominal DCO Frequency (MHz): 12; Nominal DCO Frequency Range (MHz): 8 to 16 */
#define CS_CTL0_DCORSEL__4                                 (0x00040000)          /* Nominal DCO Frequency (MHz): 24; Nominal DCO Frequency Range (MHz): 16 to 32 */
#define CS_CTL0_DCORSEL__5                                 (0x00050000)          /* Nominal DCO Frequency (MHz): 48; Nominal DCO Frequency Range (MHz): 32 to 64 */
#define CS_CTL0_RESERVED2__M                               (0x00380000)          /*  */
#define CS_CTL0_DCORES                                     (0x00400000)          /* Enables the DCO external resistor mode. */
#define CS_CTL0_DCOEN                                      (0x00800000)          /* Enables the DCO oscillator regardless if used as a clock resource. */
#define CS_CTL0_DIS_DCO_DELAY_CNT                          (0x01000000)          /* Setting this bit disabes the DCO settling counter value. When this is set the  */
                                                                                 /* clocks are directly fed to the digital system from the DCO while the frequency  */
                                                                                 /* changes are ongoing. 0b = DCO Settling delay counter is enabled. The application */
                                                                                 /* can change to any frequency desired as long as it is supported by the power  */
                                                                                 /* mode. 1b = DCO setlling delay counter disabled. The application should ensure  */
                                                                                 /* that the frequency changes requested are in step size of 5MHz. */
#define CS_CTL0_RESERVED1__M                               (0xfe000000)          /*  */

/* CS_CTL1 Control Bits */
#define CS_CTL1_SELM__M                                    (0x00000007)          /* Selects the MCLK source. */
#define CS_CTL1_SELM__0                                    (0x00000000)          /* LFXTCLK when LFXT available, otherwise REFOCLK */
#define CS_CTL1_SELM__1                                    (0x00000001)          /* VLOCLK */
#define CS_CTL1_SELM__1_VLOCLK                             (0x00000001)          /* VLOCLK */
#define CS_CTL1_SELM__2                                    (0x00000002)          /* REFOCLK */
#define CS_CTL1_SELM__2_REFOCLK                            (0x00000002)          /* REFOCLK */
#define CS_CTL1_SELM__3                                    (0x00000003)          /* DCOCLK */
#define CS_CTL1_SELM__3_DCOCLK                             (0x00000003)          /* DCOCLK */
#define CS_CTL1_SELM__4                                    (0x00000004)          /* MODOSC */
#define CS_CTL1_SELM__4_MODOSC                             (0x00000004)          /* MODOSC */
#define CS_CTL1_SELM__5                                    (0x00000005)          /* HFXTCLK when HFXT available, otherwise DCOCLK */
#define CS_CTL1_SELM__6                                    (0x00000006)          /* HFXT2CLK when HFXT2 available, otherwise DCOCLK */
#define CS_CTL1_SELM__7                                    (0x00000007)          /* Reserved for future use. Defaults to DCOCLK. Not recommended for use to ensure future compatibilities. */
#define CS_CTL1_RESERVED8                                  (0x00000008)          /*  */
#define CS_CTL1_SELS__M                                    (0x00000070)          /* Selects the SMCLK and HSMCLK source. */
#define CS_CTL1_SELS__0                                    (0x00000000)          /* LFXTCLK when LFXT available, otherwise REFOCLK */
#define CS_CTL1_SELS__1                                    (0x00000010)          /* VLOCLK */
#define CS_CTL1_SELS__1_VLOCLK                             (0x00000010)          /* VLOCLK */
#define CS_CTL1_SELS__2                                    (0x00000020)          /* REFOCLK */
#define CS_CTL1_SELS__2_REFOCLK                            (0x00000020)          /* REFOCLK */
#define CS_CTL1_SELS__3                                    (0x00000030)          /* DCOCLK */
#define CS_CTL1_SELS__3_DCOCLK                             (0x00000030)          /* DCOCLK */
#define CS_CTL1_SELS__4                                    (0x00000040)          /* MODOSC */
#define CS_CTL1_SELS__4_MODOSC                             (0x00000040)          /* MODOSC */
#define CS_CTL1_SELS__5                                    (0x00000050)          /* HFXTCLK when HFXT available, otherwise DCOCLK */
#define CS_CTL1_SELS__6                                    (0x00000060)          /* HFXT2CLK when HFXT2 available, otherwise DCOCLK */
#define CS_CTL1_SELS__7                                    (0x00000070)          /* Reserved for future use. Defaults to DCOCLK. Not recommended for use to ensure future compatibilities. */
#define CS_CTL1_RESERVED7                                  (0x00000080)          /*  */
#define CS_CTL1_SELA__M                                    (0x00000700)          /* Selects the ACLK source. 011b-111b = Reserved for future use. Defaults to  */
                                                                                 /* REFOCLK. Not recommended for use to ensure future compatibilities. */
#define CS_CTL1_SELA__0                                    (0x00000000)          /* LFXTCLK when LFXT available, otherwise REFOCLK */
#define CS_CTL1_SELA__1                                    (0x00000100)          /* VLOCLK */
#define CS_CTL1_SELA__1_VLOCLK                             (0x00000100)          /* VLOCLK */
#define CS_CTL1_SELA__2                                    (0x00000200)          /* REFOCLK */
#define CS_CTL1_SELA__2_REFOCLK                            (0x00000200)          /* REFOCLK */
#define CS_CTL1_RESERVED6                                  (0x00000800)          /*  */
#define CS_CTL1_SELB                                       (0x00001000)          /* Selects the BCLK source. */
#define CS_CTL1_RESERVED5__M                               (0x0000e000)          /*  */
#define CS_CTL1_DIVM__M                                    (0x00070000)          /* MCLK source divider. */
#define CS_CTL1_DIVM__0                                    (0x00000000)          /* f(MCLK)/1 */
#define CS_CTL1_DIVM__0_F_MCLK__1                          (0x00000000)          /* f(MCLK)/1 */
#define CS_CTL1_DIVM__1                                    (0x00010000)          /* f(MCLK)/2 */
#define CS_CTL1_DIVM__1_F_MCLK__2                          (0x00010000)          /* f(MCLK)/2 */
#define CS_CTL1_DIVM__2                                    (0x00020000)          /* f(MCLK)/4 */
#define CS_CTL1_DIVM__2_F_MCLK__4                          (0x00020000)          /* f(MCLK)/4 */
#define CS_CTL1_DIVM__3                                    (0x00030000)          /* f(MCLK)/8 */
#define CS_CTL1_DIVM__3_F_MCLK__8                          (0x00030000)          /* f(MCLK)/8 */
#define CS_CTL1_DIVM__4                                    (0x00040000)          /* f(MCLK)/16 */
#define CS_CTL1_DIVM__4_F_MCLK__16                         (0x00040000)          /* f(MCLK)/16 */
#define CS_CTL1_DIVM__5                                    (0x00050000)          /* f(MCLK)/32 */
#define CS_CTL1_DIVM__5_F_MCLK__32                         (0x00050000)          /* f(MCLK)/32 */
#define CS_CTL1_DIVM__6                                    (0x00060000)          /* f(MCLK)/64 */
#define CS_CTL1_DIVM__6_F_MCLK__64                         (0x00060000)          /* f(MCLK)/64 */
#define CS_CTL1_DIVM__7                                    (0x00070000)          /* f(MCLK)/128 */
#define CS_CTL1_DIVM__7_F_MCLK__128                        (0x00070000)          /* f(MCLK)/128 */
#define CS_CTL1_RESERVED4                                  (0x00080000)          /*  */
#define CS_CTL1_DIVHS__M                                   (0x00700000)          /* HSMCLK source divider. */
#define CS_CTL1_DIVHS__0                                   (0x00000000)          /* f(HSMCLK)/1 */
#define CS_CTL1_DIVHS__0_F_HSMCLK__1                       (0x00000000)          /* f(HSMCLK)/1 */
#define CS_CTL1_DIVHS__1                                   (0x00100000)          /* f(HSMCLK)/2 */
#define CS_CTL1_DIVHS__1_F_HSMCLK__2                       (0x00100000)          /* f(HSMCLK)/2 */
#define CS_CTL1_DIVHS__2                                   (0x00200000)          /* f(HSMCLK)/4 */
#define CS_CTL1_DIVHS__2_F_HSMCLK__4                       (0x00200000)          /* f(HSMCLK)/4 */
#define CS_CTL1_DIVHS__3                                   (0x00300000)          /* f(HSMCLK)/8 */
#define CS_CTL1_DIVHS__3_F_HSMCLK__8                       (0x00300000)          /* f(HSMCLK)/8 */
#define CS_CTL1_DIVHS__4                                   (0x00400000)          /* f(HSMCLK)/16 */
#define CS_CTL1_DIVHS__4_F_HSMCLK__16                      (0x00400000)          /* f(HSMCLK)/16 */
#define CS_CTL1_DIVHS__5                                   (0x00500000)          /* f(HSMCLK)/32 */
#define CS_CTL1_DIVHS__5_F_HSMCLK__32                      (0x00500000)          /* f(HSMCLK)/32 */
#define CS_CTL1_DIVHS__6                                   (0x00600000)          /* f(HSMCLK)/64 */
#define CS_CTL1_DIVHS__6_F_HSMCLK__64                      (0x00600000)          /* f(HSMCLK)/64 */
#define CS_CTL1_DIVHS__7                                   (0x00700000)          /* f(HSMCLK)/128 */
#define CS_CTL1_DIVHS__7_F_HSMCLK__128                     (0x00700000)          /* f(HSMCLK)/128 */
#define CS_CTL1_RESERVED3                                  (0x00800000)          /*  */
#define CS_CTL1_DIVA__M                                    (0x07000000)          /* ACLK source divider. */
#define CS_CTL1_DIVA__0                                    (0x00000000)          /* f(ACLK)/1 */
#define CS_CTL1_DIVA__0_F_ACLK__1                          (0x00000000)          /* f(ACLK)/1 */
#define CS_CTL1_DIVA__1                                    (0x01000000)          /* f(ACLK)/2 */
#define CS_CTL1_DIVA__1_F_ACLK__2                          (0x01000000)          /* f(ACLK)/2 */
#define CS_CTL1_DIVA__2                                    (0x02000000)          /* f(ACLK)/4 */
#define CS_CTL1_DIVA__2_F_ACLK__4                          (0x02000000)          /* f(ACLK)/4 */
#define CS_CTL1_DIVA__3                                    (0x03000000)          /* f(ACLK)/8 */
#define CS_CTL1_DIVA__3_F_ACLK__8                          (0x03000000)          /* f(ACLK)/8 */
#define CS_CTL1_DIVA__4                                    (0x04000000)          /* f(ACLK)/16 */
#define CS_CTL1_DIVA__4_F_ACLK__16                         (0x04000000)          /* f(ACLK)/16 */
#define CS_CTL1_DIVA__5                                    (0x05000000)          /* f(ACLK)/32 */
#define CS_CTL1_DIVA__5_F_ACLK__32                         (0x05000000)          /* f(ACLK)/32 */
#define CS_CTL1_DIVA__6                                    (0x06000000)          /* f(ACLK)/64 */
#define CS_CTL1_DIVA__6_F_ACLK__64                         (0x06000000)          /* f(ACLK)/64 */
#define CS_CTL1_DIVA__7                                    (0x07000000)          /* f(ACLK)/128 */
#define CS_CTL1_DIVA__7_F_ACLK__128                        (0x07000000)          /* f(ACLK)/128 */
#define CS_CTL1_RESERVED2                                  (0x08000000)          /*  */
#define CS_CTL1_DIVS__M                                    (0x70000000)          /* SMCLK source divider. */
#define CS_CTL1_DIVS__0                                    (0x00000000)          /* f(SMCLK)/1 */
#define CS_CTL1_DIVS__0_F_SMCLK__1                         (0x00000000)          /* f(SMCLK)/1 */
#define CS_CTL1_DIVS__1                                    (0x10000000)          /* f(SMCLK)/2 */
#define CS_CTL1_DIVS__1_F_SMCLK__2                         (0x10000000)          /* f(SMCLK)/2 */
#define CS_CTL1_DIVS__2                                    (0x20000000)          /* f(SMCLK)/4 */
#define CS_CTL1_DIVS__2_F_SMCLK__4                         (0x20000000)          /* f(SMCLK)/4 */
#define CS_CTL1_DIVS__3                                    (0x30000000)          /* f(SMCLK)/8 */
#define CS_CTL1_DIVS__3_F_SMCLK__8                         (0x30000000)          /* f(SMCLK)/8 */
#define CS_CTL1_DIVS__4                                    (0x40000000)          /* f(SMCLK)/16 */
#define CS_CTL1_DIVS__4_F_SMCLK__16                        (0x40000000)          /* f(SMCLK)/16 */
#define CS_CTL1_DIVS__5                                    (0x50000000)          /* f(SMCLK)/32 */
#define CS_CTL1_DIVS__5_F_SMCLK__32                        (0x50000000)          /* f(SMCLK)/32 */
#define CS_CTL1_DIVS__6                                    (0x60000000)          /* f(SMCLK)/64 */
#define CS_CTL1_DIVS__6_F_SMCLK__64                        (0x60000000)          /* f(SMCLK)/64 */
#define CS_CTL1_DIVS__7                                    (0x70000000)          /* f(SMCLK)/128 */
#define CS_CTL1_DIVS__7_F_SMCLK__128                       (0x70000000)          /* f(SMCLK)/128 */
#define CS_CTL1_RESERVED1                                  (0x80000000)          /*  */

/* CS_CTL2 Control Bits */
#define CS_CTL2_LFXTDRIVE__M                               (0x00000007)          /* The LFXT oscillator current can be adjusted to its drive needs. Reset value is  */
                                                                                 /* 7h when LFXT available, and 0h when LFXT not available. */
#define CS_CTL2_LFXTDRIVE__0                               (0x00000000)          /* Lowest current consumption. */
#define CS_CTL2_LFXTDRIVE__1                               (0x00000001)          /* Increased drive strength LFXT oscillator. */
#define CS_CTL2_LFXTDRIVE__2                               (0x00000002)          /* Increased drive strength LFXT oscillator. */
#define CS_CTL2_LFXTDRIVE__3                               (0x00000003)          /* Increased drive strength LFXT oscillator. */
#define CS_CTL2_LFXTDRIVE__4                               (0x00000004)          /* Increased drive strength LFXT oscillator. */
#define CS_CTL2_LFXTDRIVE__5                               (0x00000005)          /* Increased drive strength LFXT oscillator. */
#define CS_CTL2_LFXTDRIVE__6                               (0x00000006)          /* Increased drive strength LFXT oscillator. */
#define CS_CTL2_LFXTDRIVE__7                               (0x00000007)          /* Maximum drive strength LFXT oscillator. */
#define CS_CTL2_RESERVED6__M                               (0x00000078)          /*  */
#define CS_CTL2_LFXTAGCOFF                                 (0x00000080)          /* LFXT AGC off. Setting this bit disables the automatic gain control of the LFXT  */
                                                                                 /* crystal. This is currently a TI internal register bit. */
#define CS_CTL2_LFXT_EN                                    (0x00000100)          /* Turns on the LFXT oscillator regardless if used as a clock resource. */
#define CS_CTL2_LFXTBYPASS                                 (0x00000200)          /* LFXT bypass select. */
#define CS_CTL2_RESERVED5__M                               (0x0000fc00)          /*  */
#define CS_CTL2_HFXTDRIVE                                  (0x00010000)          /* HFXT oscillator drive selection. 0b = To be used for HFXTFREQ setting 000b 1b =  */
                                                                                 /* To be used for HFXTFREQ settings 001b to 111b Reset value is 1h when HFXT  */
                                                                                 /* available, and 0h when HFXT not available. */
#define CS_CTL2_RESERVED4__M                               (0x00060000)          /*  */
#define CS_CTL2_RESERVED3                                  (0x00080000)          /*  */
#define CS_CTL2_HFXTFREQ__M                                (0x00700000)          /* The HFXT frequency selection. These bits must be set to the appropriate  */
                                                                                 /* frequency for crystal or bypass modes of operation. */
#define CS_CTL2_HFXTFREQ__0                                (0x00000000)          /* 1 MHz to 4 MHz */
#define CS_CTL2_HFXTFREQ__0_1_MHZ_TO_4_MHZ                 (0x00000000)          /* 1 MHz to 4 MHz */
#define CS_CTL2_HFXTFREQ__1                                (0x00100000)          /* >4 MHz to 8 MHz */
#define CS_CTL2_HFXTFREQ__1__4_MHZ_TO_8_MHZ                (0x00100000)          /* >4 MHz to 8 MHz */
#define CS_CTL2_HFXTFREQ__2                                (0x00200000)          /* 8 MHz to 16 MHz */
#define CS_CTL2_HFXTFREQ__2_8_MHZ_TO_16_MHZ                (0x00200000)          /* 8 MHz to 16 MHz */
#define CS_CTL2_HFXTFREQ__3                                (0x00300000)          /* >16 MHz to 24 MHz */
#define CS_CTL2_HFXTFREQ__4                                (0x00400000)          /* >24 MHz to 32 MHz */
#define CS_CTL2_HFXTFREQ__5                                (0x00500000)          /* >32 MHz to 40 MHz */
#define CS_CTL2_HFXTFREQ__6                                (0x00600000)          /* >40 MHz to 48 MHz */
#define CS_CTL2_HFXTFREQ__7                                (0x00700000)          /* Reserved for future use. */
#define CS_CTL2_RESERVED2                                  (0x00800000)          /*  */
#define CS_CTL2_HFXT_EN                                    (0x01000000)          /* Turns on the HFXT oscillator regardless if used as a clock resource. */
#define CS_CTL2_HFXTBYPASS                                 (0x02000000)          /* HFXT bypass select. */
#define CS_CTL2_RESERVED1__M                               (0xfc000000)          /*  */

/* CS_CTL3 Control Bits */
#define CS_CTL3_FCNTLF__M                                  (0x00000003)          /* Start flag counter for LFXT. Selects number of LFXT cycles before LFXTOFFG can  */
                                                                                 /* be cleared. Only available on devices with LFXT. Reset value is 3h when LFXT  */
                                                                                 /* available, and 0h when LFXT not available. */
#define CS_CTL3_FCNTLF__0                                  (0x00000000)          /* 4096 cycles */
#define CS_CTL3_FCNTLF__0_4096_CYCLES                      (0x00000000)          /* 4096 cycles */
#define CS_CTL3_FCNTLF__1                                  (0x00000001)          /* 8192 cycles */
#define CS_CTL3_FCNTLF__1_8192_CYCLES                      (0x00000001)          /* 8192 cycles */
#define CS_CTL3_FCNTLF__2                                  (0x00000002)          /* 16384 cycles */
#define CS_CTL3_FCNTLF__2_16384_CYCLES                     (0x00000002)          /* 16384 cycles */
#define CS_CTL3_FCNTLF__3                                  (0x00000003)          /* 32768 cycles */
#define CS_CTL3_FCNTLF__3_32768_CYCLES                     (0x00000003)          /* 32768 cycles */
#define CS_CTL3_RFCNTLF                                    (0x00000004)          /* Reset start fault counter for LFXT. Write 1 only. Self clears once written. Only */
                                                                                 /* available on devices with LFXT. */
#define CS_CTL3_FCNTLF_EN                                  (0x00000008)          /*  */
#define CS_CTL3_FCNTHF__M                                  (0x00000030)          /* Start flag counter for HFXT. Selects number of HFXT cycles before HFXTOFFG can  */
                                                                                 /* be cleared. Only available on devices with HFXT. Reset value is 3h when HFXT  */
                                                                                 /* available, and 0h when HFXT not available. */
#define CS_CTL3_FCNTHF__0                                  (0x00000000)          /* 2048 cycles */
#define CS_CTL3_FCNTHF__0_2048_CYCLES                      (0x00000000)          /* 2048 cycles */
#define CS_CTL3_FCNTHF__1                                  (0x00000010)          /* 4096 cycles */
#define CS_CTL3_FCNTHF__1_4096_CYCLES                      (0x00000010)          /* 4096 cycles */
#define CS_CTL3_FCNTHF__2                                  (0x00000020)          /* 8192 cycles */
#define CS_CTL3_FCNTHF__2_8192_CYCLES                      (0x00000020)          /* 8192 cycles */
#define CS_CTL3_FCNTHF__3                                  (0x00000030)          /* 16384 cycles */
#define CS_CTL3_FCNTHF__3_16384_CYCLES                     (0x00000030)          /* 16384 cycles */
#define CS_CTL3_RFCNTHF                                    (0x00000040)          /* Reset start fault counter for HFXT. Write 1 only. Self clears once written. Only */
                                                                                 /* available on devices with HFXT. */
#define CS_CTL3_FCNTHF_EN                                  (0x00000080)          /* Enable start fault counter for HFXT. Only available on devices with HFXT. Reset  */
                                                                                 /* value is 1h when HFXT available, and 0h when HFXT not available. */
#define CS_CTL3_FCNTHF2__M                                 (0x00000300)          /* Start flag counter for HFXT2. Selects number of HFXT2 cycles before HFXT2OFFG  */
                                                                                 /* can be cleared. Only available on devices with HFXT2. Reset value is 3h when  */
                                                                                 /* HFXT2 available, and 0h when HFXT2 not available. */
#define CS_CTL3_FCNTHF2__0                                 (0x00000000)          /* 2048 cycles */
#define CS_CTL3_FCNTHF2__0_2048_CYCLES                     (0x00000000)          /* 2048 cycles */
#define CS_CTL3_FCNTHF2__1                                 (0x00000100)          /* 4096 cycles */
#define CS_CTL3_FCNTHF2__1_4096_CYCLES                     (0x00000100)          /* 4096 cycles */
#define CS_CTL3_FCNTHF2__2                                 (0x00000200)          /* 8192 cycles */
#define CS_CTL3_FCNTHF2__2_8192_CYCLES                     (0x00000200)          /* 8192 cycles */
#define CS_CTL3_FCNTHF2__3                                 (0x00000300)          /* 16384 cycles */
#define CS_CTL3_FCNTHF2__3_16384_CYCLES                    (0x00000300)          /* 16384 cycles */
#define CS_CTL3_RFCNTHF2                                   (0x00000400)          /* Reset start fault counter for HFXT2. Write 1 only. Self clears once written.  */
                                                                                 /* Only available on devices with HFXT2. */
#define CS_CTL3_FCNTHF2_EN                                 (0x00000800)          /* Enable start fault counter for HFXT2. Only available on devices with HFXT2.  */
                                                                                 /* Reset value is 1h when HFXT2 available, and 0h when HFXT2 not available. */
#define CS_CTL3_RESERVED1__M                               (0xfffff000)          /*  */

/* CS_CTL4 Control Bits */
#define CS_CTL4_HFXT2DRIVE                                 (0x00000001)          /* The HFXT2 oscillator current can be adjusted to its drive needs. This in  */
                                                                                 /* combination with the HFXT2FREQ bits can be used for optimizing crystal power  */
                                                                                 /* based on crystal characteristics. Reset value is 7h when HFXT2 available, and 0h */
                                                                                 /* when HFXT2 not available. */
#define CS_CTL4_RESERVED4__M                               (0x00000006)          /*  */
#define CS_CTL4_RESERVED3                                  (0x00000008)          /*  */
#define CS_CTL4_HFXT2FREQ__M                               (0x00000070)          /* The HFXT2 frequency selection. These bits must be set to the appropriate  */
                                                                                 /* frequency for crystal or bypass modes of operation. */
#define CS_CTL4_HFXT2FREQ__0                               (0x00000000)          /* 1 MHz to 4 MHz */
#define CS_CTL4_HFXT2FREQ__0_1_MHZ_TO_4_MHZ                (0x00000000)          /* 1 MHz to 4 MHz */
#define CS_CTL4_HFXT2FREQ__1                               (0x00000010)          /* >4 MHz to 8 MHz */
#define CS_CTL4_HFXT2FREQ__1__4_MHZ_TO_8_MHZ               (0x00000010)          /* >4 MHz to 8 MHz */
#define CS_CTL4_HFXT2FREQ__2                               (0x00000020)          /* >8 MHz to 16 MHz */
#define CS_CTL4_HFXT2FREQ__3                               (0x00000030)          /* >16 MHz to 24 MHz */
#define CS_CTL4_HFXT2FREQ__4                               (0x00000040)          /* >24 MHz to 32 MHz */
#define CS_CTL4_HFXT2FREQ__5                               (0x00000050)          /* >32 MHz to 40 MHz */
#define CS_CTL4_HFXT2FREQ__6                               (0x00000060)          /* >40 MHz to 48 MHz */
#define CS_CTL4_HFXT2FREQ__7                               (0x00000070)          /* Reserved for future use. */
#define CS_CTL4_RESERVED2                                  (0x00000080)          /*  */
#define CS_CTL4_HFXT2_EN                                   (0x00000100)          /* Turns on the HFXT2 oscillator regardless if used as a clock resource. */
#define CS_CTL4_HFXT2BYPASS                                (0x00000200)          /* HFXT2 bypass select. */
#define CS_CTL4_RESERVED1__M                               (0xfffffc00)          /*  */

/* CS_CTL5 Control Bits */
#define CS_CTL5_REFCNTSEL__M                               (0x00000007)          /* Reference counter source select. 100b-111b = Reserved for future use. Not  */
                                                                                 /* recommended for use to ensure future compatibilities. */
#define CS_CTL5_REFCNTSEL__0                               (0x00000000)          /* REFOCLK */
#define CS_CTL5_REFCNTSEL__0_REFOCLK                       (0x00000000)          /* REFOCLK */
#define CS_CTL5_REFCNTSEL__1                               (0x00000001)          /* LFXTCLK when available, otherwise REFOCLK. */
#define CS_CTL5_REFCNTSEL__2                               (0x00000002)          /* HFXTCLK when available, otherwise REFOCLK. */
#define CS_CTL5_REFCNTSEL__3                               (0x00000003)          /* HFXT2CLK when availabnle, otherwise REFOCLK. */
#define CS_CTL5_REFCNTPS__M                                (0x00000038)          /* Reference clock prescaler */
#define CS_CTL5_REFCNTPS__0                                (0x00000000)          /* /1 */
#define CS_CTL5_REFCNTPS__0__1                             (0x00000000)          /* /1 */
#define CS_CTL5_REFCNTPS__1                                (0x00000008)          /* /16 */
#define CS_CTL5_REFCNTPS__1__16                            (0x00000008)          /* /16 */
#define CS_CTL5_REFCNTPS__2                                (0x00000010)          /* /32 */
#define CS_CTL5_REFCNTPS__2__32                            (0x00000010)          /* /32 */
#define CS_CTL5_REFCNTPS__3                                (0x00000018)          /* /64 */
#define CS_CTL5_REFCNTPS__3__64                            (0x00000018)          /* /64 */
#define CS_CTL5_REFCNTPS__4                                (0x00000020)          /* /128 */
#define CS_CTL5_REFCNTPS__4__128                           (0x00000020)          /* /128 */
#define CS_CTL5_REFCNTPS__5                                (0x00000028)          /* /256 */
#define CS_CTL5_REFCNTPS__5__256                           (0x00000028)          /* /256 */
#define CS_CTL5_REFCNTPS__6                                (0x00000030)          /* /512 */
#define CS_CTL5_REFCNTPS__6__512                           (0x00000030)          /* /512 */
#define CS_CTL5_REFCNTPS__7                                (0x00000038)          /* /1024 */
#define CS_CTL5_REFCNTPS__7__1024                          (0x00000038)          /* /1024 */
#define CS_CTL5_RESERVED2                                  (0x00000040)          /*  */
#define CS_CTL5_CALSTART                                   (0x00000080)          /* Start clock calibration counters. Bit is self clearing. */
#define CS_CTL5_PERCNTSEL__M                               (0x00000700)          /* Period counter source select. 010b-111b = Reserved for future use. Not  */
                                                                                 /* recommended for use to ensure future compatibilities. */
#define CS_CTL5_PERCNTSEL__0                               (0x00000000)          /* DCOCLK */
#define CS_CTL5_PERCNTSEL__0_DCOCLK                        (0x00000000)          /* DCOCLK */
#define CS_CTL5_PERCNTSEL__1                               (0x00000100)          /* REFOCLK */
#define CS_CTL5_PERCNTSEL__1_REFOCLK                       (0x00000100)          /* REFOCLK */
#define CS_CTL5_RESERVED1__M                               (0xfffff800)          /*  */

/* CS_CTL6 Control Bits */
#define CS_CTL6_PERCNT__M                                  (0x0000ffff)          /* Calibration period counter. Read only. */
#define CS_CTL6_RESERVED1__M                               (0xffff0000)          /*  */

/* CS_CTL7 Control Bits */
#define CS_CTL7_REFCNT__M                                  (0x0000ffff)          /* Calibration reference period counter. */
#define CS_CTL7_RESERVED1__M                               (0xffff0000)          /*  */

/* CS_CLKEN Control Bits */
#define CS_CLKEN_ACLK_EN                                   (0x00000001)          /* ACLK system clock conditional request enable */
#define CS_CLKEN_MCLK_EN                                   (0x00000002)          /* MCLK system clock conditional request enable */
#define CS_CLKEN_HSMCLK_EN                                 (0x00000004)          /* HSMCLK system clock conditional request enable */
#define CS_CLKEN_SMCLK_EN                                  (0x00000008)          /* SMCLK system clock conditional request enable */
#define CS_CLKEN_RESERVED3__M                              (0x000000f0)          /*  */
#define CS_CLKEN_VLO_EN                                    (0x00000100)          /* Turns on the VLO oscillator regardless if used as a clock resource. */
#define CS_CLKEN_REFO_EN                                   (0x00000200)          /* Turns on the REFO oscillator regardless if used as a clock resource. */
#define CS_CLKEN_MODOSC_EN                                 (0x00000400)          /* Turns on the MODOSC oscillator regardless if used as a clock resource. */
#define CS_CLKEN_RESERVED2__M                              (0x00007800)          /*  */
#define CS_CLKEN_REFOFSEL                                  (0x00008000)          /* Selects REFO nominal frequency. */
#define CS_CLKEN_RESERVED1__M                              (0xffff0000)          /*  */

/* CS_STAT Control Bits */
#define CS_STAT_DCO_ON                                     (0x00000001)          /* DCO status */
#define CS_STAT_DCOBIAS_ON                                 (0x00000002)          /* DCO bias status */
#define CS_STAT_HFXT_ON                                    (0x00000004)          /* HFXT status. Only available on devices with HFXT. */
#define CS_STAT_HFXT2_ON                                   (0x00000008)          /* HFXT2 status. Only available on devices with HFXT2. */
#define CS_STAT_MODOSC_ON                                  (0x00000010)          /* MODOSC status */
#define CS_STAT_VLO_ON                                     (0x00000020)          /* VLO status */
#define CS_STAT_LFXT_ON                                    (0x00000040)          /* LFXT status. Only available on devices with LFXT. */
#define CS_STAT_REFO_ON                                    (0x00000080)          /* REFO status */
#define CS_STAT_RESERVED2__M                               (0x0000ff00)          /*  */
#define CS_STAT_ACLK_ON                                    (0x00010000)          /* ACLK system clock status */
#define CS_STAT_MCLK_ON                                    (0x00020000)          /* MCLK system clock status */
#define CS_STAT_HSMCLK_ON                                  (0x00040000)          /* HSMCLK system clock status */
#define CS_STAT_SMCLK_ON                                   (0x00080000)          /* SMCLK system clock status */
#define CS_STAT_MODCLK_ON                                  (0x00100000)          /* MODCLK system clock status */
#define CS_STAT_VLOCLK_ON                                  (0x00200000)          /* VLOCLK system clock status */
#define CS_STAT_LFXTCLK_ON                                 (0x00400000)          /* LFXTCLK system clock status */
#define CS_STAT_REFOCLK_ON                                 (0x00800000)          /* REFOCLK system clock status */
#define CS_STAT_ACLK_READY                                 (0x01000000)          /* ACLK Ready status. This bit indicates whether the clock is stable after a change */
                                                                                 /* in the frequency/divider settings. 0b = Not ready 1b = Ready */
#define CS_STAT_MCLK_READY                                 (0x02000000)          /* MCLK Ready status. This bit indicates whether the clock is stable after a change */
                                                                                 /* in the frequency/divider settings. 0b = Not ready 1b = Ready */
#define CS_STAT_HSMCLK_READY                               (0x04000000)          /* HSMCLK Ready status. This bit indicates whether the clock is stable after a  */
                                                                                 /* change in the frequency/divider settings. 0b = Not ready 1b = Ready */
#define CS_STAT_SMCLK_READY                                (0x08000000)          /* SMCLK Ready status. This bit indicates whether the clock is stable after a  */
                                                                                 /* change in the frequency/divider settings. 0b = Not ready 1b = Ready */
#define CS_STAT_BCLK_READY                                 (0x10000000)          /* BCLK Ready status. This bit indicates whether the clock is stable after a change */
                                                                                 /* in the frequency settings. 0b = Not ready 1b = Ready */
#define CS_STAT_RESERVED1__M                               (0xe0000000)          /*  */

/* CS_IE Control Bits */
#define CS_IE_LFXTIE                                       (0x00000001)          /* LFXT oscillator fault flag interrupt enable. Only availabe on devices with LFXT. */
#define CS_IE_HFXTIE                                       (0x00000002)          /* HFXT oscillator fault flag interrupt enable. Only availabe on devices with HFXT. */
#define CS_IE_HFXT2IE                                      (0x00000004)          /* HFXT2 oscillator fault flag interrupt enable. Only availabe on devices with  */
                                                                                 /* HFXT2. */
#define CS_IE_RESERVED4                                    (0x00000008)          /*  */
#define CS_IE_DCOMINIE                                     (0x00000010)          /* DCO minimum fault flag interrupt enable. */
#define CS_IE_DCOMAXIE                                     (0x00000020)          /* DCO maximum fault flag interrupt enable. */
#define CS_IE_DCORIE                                       (0x00000040)          /* DCO external resistor fault flag interrupt enable. */
#define CS_IE_RESERVED3                                    (0x00000080)          /*  */
#define CS_IE_FCNTLFIE                                     (0x00000100)          /* Start fault counter interrupt enable LFXT. Only availabe on devices with LFXT. */
#define CS_IE_FCNTHFIE                                     (0x00000200)          /* Start fault counter interrupt enable HFXT. Only availabe on devices with HFXT. */
#define CS_IE_RESERVED1__M                                 (0xfffffc00)          /*  */
#define CS_IE_FCNTHF2IE                                    (0x00000400)          /* Start fault counter interrupt enable HFXT2. Only availabe on devices with HFXT2. */
#define CS_IE_RESERVED2                                    (0x00000800)          /*  */
#define CS_IE_PLLOOLIE                                     (0x00001000)          /* PLL out-of-lock interrupt enable. Only availabe on devices with a PLL. */
#define CS_IE_PLLLOSIE                                     (0x00002000)          /* PLL loss-of-signal interrupt enable. Only available on devices with a PLL. */
#define CS_IE_PLLOORIE                                     (0x00004000)          /* PLL out-of-range interrupt enable. Only available on devices with a PLL. */
#define CS_IE_CALIE                                        (0x00008000)          /* REFCNT period counter interrupt enable. */

/* CS_IFG Control Bits */
#define CS_IFG_LFXTIFG                                     (0x00000001)          /* LFXT oscillator fault flag. If this bit is set, the OFIFG flag is also set.  */
                                                                                 /* LFXTIFG is set if a LFXT fault condition exists. LFXTIFG can be cleared via  */
                                                                                 /* software by CLR_LFXTIFG. If the LFXT fault condition still remains, LFXTIFG is  */
                                                                                 /* set again. Only available on devices with LFXT. */
#define CS_IFG_HFXTIFG                                     (0x00000002)          /* HFXT oscillator fault flag. If this bit is set, the OFIFG flag is also set.  */
                                                                                 /* HFXTIFG is set if a HFXT fault condition exists. HFXTIFG can be cleared via  */
                                                                                 /* software by CLR_HFXTIFG. If the HFXT fault condition still remains, HFXTIFG is  */
                                                                                 /* set again. Only available on devices with HFXT. */
#define CS_IFG_HFXT2IFG                                    (0x00000004)          /* NOTE: This bit is reserved for HFXT2 implementations. Do not use for any other  */
                                                                                 /* purpose. HFXT2 oscillator fault flag. If this bit is set, the OFIFG flag is also */
                                                                                 /* set. HFXT2IFG is set if a HFXT2 fault condition exists. HFXT2IFG can be cleared */
                                                                                 /* via software by CLR_HFXT2IFG. If the HFXT2 fault condition still remains,  */
                                                                                 /* HFXT2IFG is set again. Only available on devices with HFXT2. */
#define CS_IFG_RESERVED4                                   (0x00000008)          /*  */
#define CS_IFG_DCOMINIFG                                   (0x00000010)          /* DCO minimum fault flag. If this bit is set, the OFIFG flag is also set.  */
                                                                                 /* DCOMINIFG can be cleared via software by CLR_DCOMINIFG. If the DCO minimum fault */
                                                                                 /* condition still remains, DCOMINIFG is set again. */
#define CS_IFG_DCOMAXIFG                                   (0x00000020)          /* DCO maximum fault flag. If this bit is set, the OFIFG flag is also set.  */
                                                                                 /* DCOMAXIFG can be cleared via software by CLR_DCOMAXIFG. If the DCO maximum fault */
                                                                                 /* condition still remains, DCOMAXIFG is set again. */
#define CS_IFG_DCORIFG                                     (0x00000040)          /* DCO external resistor fault flag. If this bit is set, the OFIFG flag is also set */
                                                                                 /* . DCORIFG can be cleared via software by CLR_DCORIFG. If the fault condition  */
                                                                                 /* still remains, DCORIFG is set again. */
#define CS_IFG_RESERVED3                                   (0x00000080)          /*  */
#define CS_IFG_FCNTLFIFG                                   (0x00000100)          /* Start fault counter interrupt flag LFXT. If this bit is set, the FCNTIFG flag is */
                                                                                 /* also set. Only availabe on devices with a LFXT. */
#define CS_IFG_FCNTHFIFG                                   (0x00000200)          /* Start fault counter interrupt flag HFXT. If this bit is set, the FCNTIFG flag is */
                                                                                 /* also set. Only availabe on devices with a HFXT. */
#define CS_IFG_RESERVED1__M                                (0xfffffc00)          /*  */
#define CS_IFG_RESERVED2                                   (0x00000400)          /*  */
#define CS_IFG_FCNTHF2IFG                                  (0x00000800)          /* NOTE: This bit is reserved for HFXT2 implementations. Do not use for any other  */
                                                                                 /* purpose. Start fault counter interrupt flag HFXT2. If this bit is set, the  */
                                                                                 /* FCNTIFG flag is also set. Only availabe on devices with a HFXT2. */
#define CS_IFG_PLLOOLIFG                                   (0x00001000)          /* NOTE: This bit is reserved for PLL implementations. Do not use for any other  */
                                                                                 /* purpose. PLL out-of-lock interrupt flag. If this bit is set, the PLLIFG flag is  */
                                                                                 /* also set. Only availabe on devices with a PLL. */
#define CS_IFG_PLLLOSIFG                                   (0x00002000)          /* NOTE: This bit is reserved for PLL implementations. Do not use for any other  */
                                                                                 /* purpose. PLL loss-of-signal interrupt flag. If this bit is set, the PLLIFG flag  */
                                                                                 /* is also set. Only available on devices with a PLL. */
#define CS_IFG_PLLOORIFG                                   (0x00004000)          /* NOTE: This bit is reserved for PLL implementations. Do not use for any other  */
                                                                                 /* purpose. PLL out-of-range interrupt flag. If this bit is set, the PLLIFG flag is */
                                                                                 /* also set. Only available on devices with a PLL. */
#define CS_IFG_CALIFG                                      (0x00008000)          /* REFCNT period counter expired. Contents in PERCNT is available for reading. If  */
                                                                                 /* this bit is set, the FCNTIFG flag is also set. This bit is reserved for CALIFG  */
                                                                                 /* implementations. Do not use for any other purpose. */

/* CS_CLRIFG Control Bits */
#define CS_CLRIFG_CLR_LFXTIFG                              (0x00000001)          /* Clear LFXT oscillator fault interrupt flag. Does not clear OFIFG. Only available */
                                                                                 /* on devices with LFXT. */
#define CS_CLRIFG_CLR_HFXTIFG                              (0x00000002)          /* Clear HFXT oscillator fault interrupt flag. Does not clear OFIFG. Only available */
                                                                                 /* on devices with HFXT. */
#define CS_CLRIFG_CLR_HFXT2IFG                             (0x00000004)          /* NOTE: This bit is reserved for HFXT2 implementations. Do not use for any other  */
                                                                                 /* purpose. Clear HFXT2 oscillator fault interrupt flag. Does not clear OFIFG. Only */
                                                                                 /* available on devices with HFXT2. */
#define CS_CLRIFG_RESERVED3                                (0x00000008)          /*  */
#define CS_CLRIFG_CLR_DCOMINIFG                            (0x00000010)          /* Clear DCO minimum fault interrupt flag. Does not clear OFIFG. */
#define CS_CLRIFG_CLR_DCOMAXIFG                            (0x00000020)          /* Clear DCO maximum fault interrupt flag. Does not clear OFIFG. */
#define CS_CLRIFG_CLR_DCORIFG                              (0x00000040)          /* Clear DCO external resistor fault interrupt flag. Does not clear OFIFG. */
#define CS_CLRIFG_CLR_CALIFG                               (0x00000080)          /* NOTE: This bit is reserved for CLR_CALIFG implementations. Do not use for any  */
                                                                                 /* other purpose. REFCNT period counter clear interrupt flag. expired. Does not  */
                                                                                 /* clear FCNTIFG. */
#define CS_CLRIFG_CLR_FCNTLFIFG                            (0x00000100)          /* Start fault counter clear interrupt flag LFXT. Does not clear FCNTIFG. Only  */
                                                                                 /* available on devices with LFXT. */
#define CS_CLRIFG_CLR_FCNTHFIFG                            (0x00000200)          /* Start fault counter clear interrupt flag HFXT. Does not clear FCNTIFG. Only  */
                                                                                 /* available on devices with HFXT. */
#define CS_CLRIFG_RESERVED1__M                             (0xfffffc00)          /*  */
#define CS_CLRIFG_CLR_FCNTHF2IFG                           (0x00000400)          /* NOTE: This bit is reserved for HFXT2 implementations. Do not use for any other  */
                                                                                 /* purpose. Start fault counter clear interrupt flag HFXT2. Does not clearFCNTIFG.  */
                                                                                 /* Only available on devices with HFXT2. */
#define CS_CLRIFG_RESERVED2                                (0x00000800)          /*  */
#define CS_CLRIFG_CLR_PLLOOLIFG                            (0x00001000)          /* NOTE: This bit is reserved for PLL implementations. Do not use for any other  */
                                                                                 /* purpose. PLL out-of-lock clear interrupt flag. Does not clear PLLIFG. Only  */
                                                                                 /* available on devices with a PLL. */
#define CS_CLRIFG_CLR_PLLLOSIFG                            (0x00002000)          /* NOTE: This bit is reserved for PLL implementations. Do not use for any other  */
                                                                                 /* purpose. PLL loss-of-signal clear interrupt flag. Does not clear PLLIFG. Only  */
                                                                                 /* available on devices with a PLL. */
#define CS_CLRIFG_CLR_PLLOORIFG                            (0x00004000)          /* NOTE: This bit is reserved for PLL implementations. Do not use for any other  */
                                                                                 /* purpose. PLL out-of-range clear interrupt flag. Does not clear PLLIFG. Only  */
                                                                                 /* available on devices with a PLL. */

/* CS_SETIFG Control Bits */
#define CS_SETIFG_SET_LFXTIFG                              (0x00000001)          /* Set LFXT oscillator fault interrupt flag. Only available on devices with LFXT. */
#define CS_SETIFG_SET_HFXTIFG                              (0x00000002)          /* Set HFXT oscillator fault interrupt flag. Only available on devices with HFXT. */
#define CS_SETIFG_SET_HFXT2IFG                             (0x00000004)          /* NOTE: This bit is reserved for HFXT2 implementations. Do not use for any other  */
                                                                                 /* purpose. Set HFXT2 oscillator fault interrupt flag. Only available on devices  */
                                                                                 /* with HFXT2. */
#define CS_SETIFG_RESERVED3                                (0x00000008)          /*  */
#define CS_SETIFG_SET_DCOMINIFG                            (0x00000010)          /* Set DCO minimum fault interrupt flag. */
#define CS_SETIFG_SET_DCOMAXIFG                            (0x00000020)          /* Set DCO maximum fault interrupt flag. */
#define CS_SETIFG_SET_DCORIFG                              (0x00000040)          /* Set DCO external resistor fault interrupt flag. */
#define CS_SETIFG_SET_CALIFG                               (0x00000080)          /* NOTE: This bit is reserved for SET_CALIFG implementations. Do not use for any  */
                                                                                 /* other purpose. REFCNT period counter set interrupt flag. expired. */
#define CS_SETIFG_SET_FCNTLFIFG                            (0x00000100)          /* Start fault counter set interrupt flag LFXT. Only available on devices with LFXT */
                                                                                 /* . */
#define CS_SETIFG_SET_FCNTHFIFG                            (0x00000200)          /* Start fault counter set interrupt flag HFXT. Only available on devices with HFXT */
                                                                                 /* . */
#define CS_SETIFG_SET_FCNTHF2IFG                           (0x00000400)          /* NOTE: This bit is reserved for HFXT2 implementations. Do not use for any other  */
                                                                                 /* purpose. Start fault counter set interrupt flag HFXT2. Only available on devices */
                                                                                 /* with HFXT2. */
#define CS_SETIFG_RESERVED2                                (0x00000800)          /*  */
#define CS_SETIFG_SET_PLLOOLIFG                            (0x00001000)          /* NOTE: This bit is reserved for PLL implementations. Do not use for any other  */
                                                                                 /* purpose. PLL out-of-lock set interrupt flag. Only available on devices with a  */
                                                                                 /* PLL. */
#define CS_SETIFG_SET_PLLLOSIFG                            (0x00002000)          /* NOTE: This bit is reserved for PLL implementations. Do not use for any other  */
                                                                                 /* purpose. PLL loss-of-signal set interrupt flag. Only available on devices with a */
                                                                                 /* PLL. */
#define CS_SETIFG_SET_PLLOORIFG                            (0x00004000)          /* NOTE: This bit is reserved for PLL implementations. Do not use for any other  */
                                                                                 /* purpose. PLL out-of-range set interrupt flag. Only available on devices with a  */
                                                                                 /* PLL. */
#define CS_SETIFG_RESERVED1__M                             (0xffff8000)          /*  */

/* CS_DCOERCAL Control Bits */
#define CS_DCOERCAL_DCO_TCTRIM__M                          (0x00000003)          /* DCO Temperature compensation Trim */
#define CS_DCOERCAL_RESERVED2__M                           (0x0000fffc)          /*  */
#define CS_DCOERCAL_DCO_FTRIM__M                           (0x07ff0000)          /* DCO frequency trim */
#define CS_DCOERCAL_RESERVED1__M                           (0xf8000000)          /*  */


//*****************************************************************************
// DDDS Registers
//*****************************************************************************
typedef struct {
  uint32_t DDDS_CHECKSUM; /* DDDS checksum */
  uint32_t MODID_DEVINFO; /* Module ID Tag (Module ID_DEVINFO + Length (3)) */
  uint32_t DEVID; /* Device ID */
  uint32_t HWREV; /* Hardware revision */
  uint32_t BCREV; /* Bootcode revision */
  uint32_t MODID_DIEREC; /* Module ID Tag (Module ID_DIEREC + Length (4)) */
  uint32_t LOT_ID; /* Lot/Wafer ID */
  uint32_t DIE_POSITION; /* Die X/Y position */
  uint32_t TEST_RESULTS; /* Test results */
  uint32_t MODID_CS; /* Module ID Tag (Module ID_CS + Length (2)) */
  uint32_t CSDCOIRCAL; /* DCO Calibration value in Internal resistor mode */
  uint32_t CSDCOERCAL; /* DCO Calibration value in External resistor mode */
  uint32_t CSDCOCONST; /* DCO Constant (K) */
  uint32_t MODID_ADC14; /* Module ID Tag (ID_ADC + Length (5)) */
  uint32_t ADC14_PARAM0; /* ADC14 Parameters 0 (Gain/Offset) */
  uint32_t ADC14_REFTEMP0; /* ADC14 Reference Temp. Sensor for 1.2V reference */
  uint32_t ADC14_REFTEMP1; /* ADC14 Reference Temp Sensor for 1.45V reference */
  uint32_t ADC14_REFTEMP2; /* ADC14 Reference Temp Sensor for 2V reference */
  uint32_t ADC14_REFTEMP3; /* ADC14 Reference Temp Sensor for 2.5V reference */
  uint32_t MODID_REF; /* Module ID tag (ID_REF + Length (2)) */
  uint32_t REF_PARAM0; /* Reference params for 1.2V and 1.45V reference */
  uint32_t REF_PARAM1; /* Reference params for 2V and 2.5V reference */
  uint32_t MODID_RANDNUM; /* Module ID tag (ID_RANDNUM + Length(4)) */
  uint32_t RAND0; /* Random number 0 */
  uint32_t RAND1; /* Random number 1 */
  uint32_t RAND2; /* Random number 2 */
  uint32_t RAND3; /* Random number 3 */
  uint32_t DDDS_ENDWORD; /* DDDS end marker */
} DDDS_Type;



//*****************************************************************************
// DMA Registers
//*****************************************************************************
typedef struct {
  union {    /* DMA_DEVCONFIG Register */
    __IO uint32_t reg;
    struct { /* DMA_DEVCONFIG Control Bits */
      __I  uint32_t NUM_DMA_CHANNELS    	:	8;  /* Reflects the number of DMA channels available on the device */
      __I  uint32_t NUM_SRC_PER_CHANNEL 	:	8;  /* Reflects the number of DMA sources per channel */
      __I  uint32_t RESERVED1           	:	16;  /* Reserved */

    } bit;
  } DEVCONFIG;
  union {    /* DMA_SW_CHTRIG Register */
    __IO uint32_t reg;
    struct { /* DMA_SW_CHTRIG Control Bits */
      __IO uint32_t CH0                 	:	1;  /* Write 1 triggers DMA_CHANNEL0. Bit is auto-cleared when channel goes active. */
      __IO uint32_t CH1                 	:	1;  /* Write 1 triggers DMA_CHANNEL1. Bit is auto-cleared when channel goes active. */
      __IO uint32_t CH2                 	:	1;  /* Write 1 triggers DMA_CHANNEL2. Bit is auto-cleared when channel goes active. */
      __IO uint32_t CH3                 	:	1;  /* Write 1 triggers DMA_CHANNEL3. Bit is auto-cleared when channel goes active. */
      __IO uint32_t CH4                 	:	1;  /* Write 1 triggers DMA_CHANNEL4. Bit is auto-cleared when channel goes active. */
      __IO uint32_t CH5                 	:	1;  /* Write 1 triggers DMA_CHANNEL5. Bit is auto-cleared when channel goes active. */
      __IO uint32_t CH6                 	:	1;  /* Write 1 triggers DMA_CHANNEL6. Bit is auto-cleared when channel goes active. */
      __IO uint32_t CH7                 	:	1;  /* Write 1 triggers DMA_CHANNEL7. Bit is auto-cleared when channel goes active. */
      __IO uint32_t CH8                 	:	1;  /* Write 1 triggers DMA_CHANNEL8. Bit is auto-cleared when channel goes active. */
      __IO uint32_t CH9                 	:	1;  /* Write 1 triggers DMA_CHANNEL9. Bit is auto-cleared when channel goes active. */
      __IO uint32_t CH10                	:	1;  /* Write 1 triggers DMA_CHANNEL10. Bit is auto-cleared when channel goes active. */
      __IO uint32_t CH11                	:	1;  /* Write 1 triggers DMA_CHANNEL11. Bit is auto-cleared when channel goes active. */
      __IO uint32_t CH12                	:	1;  /* Write 1 triggers DMA_CHANNEL12. Bit is auto-cleared when channel goes active. */
      __IO uint32_t CH13                	:	1;  /* Write 1 triggers DMA_CHANNEL13. Bit is auto-cleared when channel goes active. */
      __IO uint32_t CH14                	:	1;  /* Write 1 triggers DMA_CHANNEL14. Bit is auto-cleared when channel goes active. */
      __IO uint32_t CH15                	:	1;  /* Write 1 triggers DMA_CHANNEL15. Bit is auto-cleared when channel goes active. */
      __IO uint32_t CH16                	:	1;  /* Write 1 triggers DMA_CHANNEL16. Bit is auto-cleared when channel goes active. */
      __IO uint32_t CH17                	:	1;  /* Write 1 triggers DMA_CHANNEL17. Bit is auto-cleared when channel goes active. */
      __IO uint32_t CH18                	:	1;  /* Write 1 triggers DMA_CHANNEL18. Bit is auto-cleared when channel goes active. */
      __IO uint32_t CH19                	:	1;  /* Write 1 triggers DMA_CHANNEL19. Bit is auto-cleared when channel goes active. */
      __IO uint32_t CH20                	:	1;  /* Write 1 triggers DMA_CHANNEL20. Bit is auto-cleared when channel goes active. */
      __IO uint32_t CH21                	:	1;  /* Write 1 triggers DMA_CHANNEL21. Bit is auto-cleared when channel goes active. */
      __IO uint32_t CH22                	:	1;  /* Write 1 triggers DMA_CHANNEL22. Bit is auto-cleared when channel goes active. */
      __IO uint32_t CH23                	:	1;  /* Write 1 triggers DMA_CHANNEL23. Bit is auto-cleared when channel goes active. */
      __IO uint32_t CH24                	:	1;  /* Write 1 triggers DMA_CHANNEL24. Bit is auto-cleared when channel goes active. */
      __IO uint32_t CH25                	:	1;  /* Write 1 triggers DMA_CHANNEL25. Bit is auto-cleared when channel goes active. */
      __IO uint32_t CH26                	:	1;  /* Write 1 triggers DMA_CHANNEL26. Bit is auto-cleared when channel goes active. */
      __IO uint32_t CH27                	:	1;  /* Write 1 triggers DMA_CHANNEL27. Bit is auto-cleared when channel goes active. */
      __IO uint32_t CH28                	:	1;  /* Write 1 triggers DMA_CHANNEL28. Bit is auto-cleared when channel goes active. */
      __IO uint32_t CH29                	:	1;  /* Write 1 triggers DMA_CHANNEL29. Bit is auto-cleared when channel goes active. */
      __IO uint32_t CH30                	:	1;  /* Write 1 triggers DMA_CHANNEL30. Bit is auto-cleared when channel goes active. */
      __IO uint32_t CH31                	:	1;  /* Write 1 triggers DMA_CHANNEL31. Bit is auto-cleared when channel goes active. */

    } bit;
  } SW_CHTRIG;
  uint8_t RESERVED0[8];
  union {    /* DMA_CH0_SRCCFG Register */
    __IO uint32_t reg;
    struct { /* DMA_CH0_SRCCFG Control Bits */
      __IO uint32_t SRC                 	:	8;  /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                /* higher than the number of available sources will be forced to r mode) */
      __I  uint32_t RESERVED1           	:	24;  /* Reserved */

    } bit;
  } CH0_SRCCFG;
  union {    /* DMA_CH1_SRCCFG Register */
    __IO uint32_t reg;
    struct { /* DMA_CH1_SRCCFG Control Bits */
      __IO uint32_t SRC                 	:	8;  /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                /* higher than the number of available sources will be forced to r mode) */
      __I  uint32_t RESERVED1           	:	24;  /* Reserved */

    } bit;
  } CH1_SRCCFG;
  union {    /* DMA_CH2_SRCCFG Register */
    __IO uint32_t reg;
    struct { /* DMA_CH2_SRCCFG Control Bits */
      __IO uint32_t SRC                 	:	8;  /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                /* higher than the number of available sources will be forced to r mode) */
      __I  uint32_t RESERVED1           	:	24;  /* Reserved */

    } bit;
  } CH2_SRCCFG;
  union {    /* DMA_CH3_SRCCFG Register */
    __IO uint32_t reg;
    struct { /* DMA_CH3_SRCCFG Control Bits */
      __IO uint32_t SRC                 	:	8;  /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                /* higher than the number of available sources will be forced to r mode) */
      __I  uint32_t RESERVED1           	:	24;  /* Reserved */

    } bit;
  } CH3_SRCCFG;
  union {    /* DMA_CH4_SRCCFG Register */
    __IO uint32_t reg;
    struct { /* DMA_CH4_SRCCFG Control Bits */
      __IO uint32_t SRC                 	:	8;  /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                /* higher than the number of available sources will be forced to r mode) */
      __I  uint32_t RESERVED1           	:	24;  /* Reserved */

    } bit;
  } CH4_SRCCFG;
  union {    /* DMA_CH5_SRCCFG Register */
    __IO uint32_t reg;
    struct { /* DMA_CH5_SRCCFG Control Bits */
      __IO uint32_t SRC                 	:	8;  /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                /* higher than the number of available sources will be forced to r mode) */
      __I  uint32_t RESERVED1           	:	24;  /* Reserved */

    } bit;
  } CH5_SRCCFG;
  union {    /* DMA_CH6_SRCCFG Register */
    __IO uint32_t reg;
    struct { /* DMA_CH6_SRCCFG Control Bits */
      __IO uint32_t SRC                 	:	8;  /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                /* higher than the number of available sources will be forced to r mode) */
      __I  uint32_t RESERVED1           	:	24;  /* Reserved */

    } bit;
  } CH6_SRCCFG;
  union {    /* DMA_CH7_SRCCFG Register */
    __IO uint32_t reg;
    struct { /* DMA_CH7_SRCCFG Control Bits */
      __IO uint32_t SRC                 	:	8;  /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                /* higher than the number of available sources will be forced to r mode) */
      __I  uint32_t RESERVED1           	:	24;  /* Reserved */

    } bit;
  } CH7_SRCCFG;
  union {    /* DMA_CH8_SRCCFG Register */
    __IO uint32_t reg;
    struct { /* DMA_CH8_SRCCFG Control Bits */
      __IO uint32_t SRC                 	:	8;  /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                /* higher than the number of available sources will be forced to r mode) */
      __I  uint32_t RESERVED1           	:	24;  /* Reserved */

    } bit;
  } CH8_SRCCFG;
  union {    /* DMA_CH9_SRCCFG Register */
    __IO uint32_t reg;
    struct { /* DMA_CH9_SRCCFG Control Bits */
      __IO uint32_t SRC                 	:	8;  /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                /* higher than the number of available sources will be forced to r mode) */
      __I  uint32_t RESERVED1           	:	24;  /* Reserved */

    } bit;
  } CH9_SRCCFG;
  union {    /* DMA_CH10_SRCCFG Register */
    __IO uint32_t reg;
    struct { /* DMA_CH10_SRCCFG Control Bits */
      __IO uint32_t SRC                 	:	8;  /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                /* higher than the number of available sources will be forced to r mode) */
      __I  uint32_t RESERVED1           	:	24;  /* Reserved */

    } bit;
  } CH10_SRCCFG;
  union {    /* DMA_CH11_SRCCFG Register */
    __IO uint32_t reg;
    struct { /* DMA_CH11_SRCCFG Control Bits */
      __IO uint32_t SRC                 	:	8;  /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                /* higher than the number of available sources will be forced to r mode) */
      __I  uint32_t RESERVED1           	:	24;  /* Reserved */

    } bit;
  } CH11_SRCCFG;
  union {    /* DMA_CH12_SRCCFG Register */
    __IO uint32_t reg;
    struct { /* DMA_CH12_SRCCFG Control Bits */
      __IO uint32_t SRC                 	:	8;  /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                /* higher than the number of available sources will be forced to r mode) */
      __I  uint32_t RESERVED1           	:	24;  /* Reserved */

    } bit;
  } CH12_SRCCFG;
  union {    /* DMA_CH13_SRCCFG Register */
    __IO uint32_t reg;
    struct { /* DMA_CH13_SRCCFG Control Bits */
      __IO uint32_t SRC                 	:	8;  /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                /* higher than the number of available sources will be forced to r mode) */
      __I  uint32_t RESERVED1           	:	24;  /* Reserved */

    } bit;
  } CH13_SRCCFG;
  union {    /* DMA_CH14_SRCCFG Register */
    __IO uint32_t reg;
    struct { /* DMA_CH14_SRCCFG Control Bits */
      __IO uint32_t SRC                 	:	8;  /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                /* higher than the number of available sources will be forced to r mode) */
      __I  uint32_t RESERVED1           	:	24;  /* Reserved */

    } bit;
  } CH14_SRCCFG;
  union {    /* DMA_CH15_SRCCFG Register */
    __IO uint32_t reg;
    struct { /* DMA_CH15_SRCCFG Control Bits */
      __IO uint32_t SRC                 	:	8;  /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                /* higher than the number of available sources will be forced to r mode) */
      __I  uint32_t RESERVED1           	:	24;  /* Reserved */

    } bit;
  } CH15_SRCCFG;
  union {    /* DMA_CH16_SRCCFG Register */
    __IO uint32_t reg;
    struct { /* DMA_CH16_SRCCFG Control Bits */
      __IO uint32_t SRC                 	:	8;  /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                /* higher than the number of available sources will be forced to r mode) */
      __I  uint32_t RESERVED1           	:	24;  /* Reserved */

    } bit;
  } CH16_SRCCFG;
  union {    /* DMA_CH17_SRCCFG Register */
    __IO uint32_t reg;
    struct { /* DMA_CH17_SRCCFG Control Bits */
      __IO uint32_t SRC                 	:	8;  /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                /* higher than the number of available sources will be forced to r mode) */
      __I  uint32_t RESERVED1           	:	24;  /* Reserved */

    } bit;
  } CH17_SRCCFG;
  union {    /* DMA_CH18_SRCCFG Register */
    __IO uint32_t reg;
    struct { /* DMA_CH18_SRCCFG Control Bits */
      __IO uint32_t SRC                 	:	8;  /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                /* higher than the number of available sources will be forced to r mode) */
      __I  uint32_t RESERVED1           	:	24;  /* Reserved */

    } bit;
  } CH18_SRCCFG;
  union {    /* DMA_CH19_SRCCFG Register */
    __IO uint32_t reg;
    struct { /* DMA_CH19_SRCCFG Control Bits */
      __IO uint32_t SRC                 	:	8;  /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                /* higher than the number of available sources will be forced to r mode) */
      __I  uint32_t RESERVED1           	:	24;  /* Reserved */

    } bit;
  } CH19_SRCCFG;
  union {    /* DMA_CH20_SRCCFG Register */
    __IO uint32_t reg;
    struct { /* DMA_CH20_SRCCFG Control Bits */
      __IO uint32_t SRC                 	:	8;  /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                /* higher than the number of available sources will be forced to r mode) */
      __I  uint32_t RESERVED1           	:	24;  /* Reserved */

    } bit;
  } CH20_SRCCFG;
  union {    /* DMA_CH21_SRCCFG Register */
    __IO uint32_t reg;
    struct { /* DMA_CH21_SRCCFG Control Bits */
      __IO uint32_t SRC                 	:	8;  /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                /* higher than the number of available sources will be forced to r mode) */
      __I  uint32_t RESERVED1           	:	24;  /* Reserved */

    } bit;
  } CH21_SRCCFG;
  union {    /* DMA_CH22_SRCCFG Register */
    __IO uint32_t reg;
    struct { /* DMA_CH22_SRCCFG Control Bits */
      __IO uint32_t SRC                 	:	8;  /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                /* higher than the number of available sources will be forced to r mode) */
      __I  uint32_t RESERVED1           	:	24;  /* Reserved */

    } bit;
  } CH22_SRCCFG;
  union {    /* DMA_CH23_SRCCFG Register */
    __IO uint32_t reg;
    struct { /* DMA_CH23_SRCCFG Control Bits */
      __IO uint32_t SRC                 	:	8;  /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                /* higher than the number of available sources will be forced to r mode) */
      __I  uint32_t RESERVED1           	:	24;  /* Reserved */

    } bit;
  } CH23_SRCCFG;
  union {    /* DMA_CH24_SRCCFG Register */
    __IO uint32_t reg;
    struct { /* DMA_CH24_SRCCFG Control Bits */
      __IO uint32_t SRC                 	:	8;  /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                /* higher than the number of available sources will be forced to r mode) */
      __I  uint32_t RESERVED1           	:	24;  /* Reserved */

    } bit;
  } CH24_SRCCFG;
  union {    /* DMA_CH25_SRCCFG Register */
    __IO uint32_t reg;
    struct { /* DMA_CH25_SRCCFG Control Bits */
      __IO uint32_t SRC                 	:	8;  /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                /* higher than the number of available sources will be forced to r mode) */
      __I  uint32_t RESERVED1           	:	24;  /* Reserved */

    } bit;
  } CH25_SRCCFG;
  union {    /* DMA_CH26_SRCCFG Register */
    __IO uint32_t reg;
    struct { /* DMA_CH26_SRCCFG Control Bits */
      __IO uint32_t SRC                 	:	8;  /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                /* higher than the number of available sources will be forced to r mode) */
      __I  uint32_t RESERVED1           	:	24;  /* Reserved */

    } bit;
  } CH26_SRCCFG;
  union {    /* DMA_CH27_SRCCFG Register */
    __IO uint32_t reg;
    struct { /* DMA_CH27_SRCCFG Control Bits */
      __IO uint32_t SRC                 	:	8;  /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                /* higher than the number of available sources will be forced to r mode) */
      __I  uint32_t RESERVED1           	:	24;  /* Reserved */

    } bit;
  } CH27_SRCCFG;
  union {    /* DMA_CH28_SRCCFG Register */
    __IO uint32_t reg;
    struct { /* DMA_CH28_SRCCFG Control Bits */
      __IO uint32_t SRC                 	:	8;  /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                /* higher than the number of available sources will be forced to r mode) */
      __I  uint32_t RESERVED1           	:	24;  /* Reserved */

    } bit;
  } CH28_SRCCFG;
  union {    /* DMA_CH29_SRCCFG Register */
    __IO uint32_t reg;
    struct { /* DMA_CH29_SRCCFG Control Bits */
      __IO uint32_t SRC                 	:	8;  /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                /* higher than the number of available sources will be forced to r mode) */
      __I  uint32_t RESERVED1           	:	24;  /* Reserved */

    } bit;
  } CH29_SRCCFG;
  union {    /* DMA_CH30_SRCCFG Register */
    __IO uint32_t reg;
    struct { /* DMA_CH30_SRCCFG Control Bits */
      __IO uint32_t SRC                 	:	8;  /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                /* higher than the number of available sources will be forced to r mode) */
      __I  uint32_t RESERVED1           	:	24;  /* Reserved */

    } bit;
  } CH30_SRCCFG;
  union {    /* DMA_CH31_SRCCFG Register */
    __IO uint32_t reg;
    struct { /* DMA_CH31_SRCCFG Control Bits */
      __IO uint32_t SRC                 	:	8;  /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                /* higher than the number of available sources will be forced to r mode) */
      __I  uint32_t RESERVED1           	:	24;  /* Reserved */

    } bit;
  } CH31_SRCCFG;
  uint8_t RESERVED1[112];
  union {    /* DMA_INT1_SRCCFG Register */
    __IO uint32_t reg;
    struct { /* DMA_INT1_SRCCFG Control Bits */
      __IO uint32_t INT_SRC             	:	5;  /* Controls which channel's completion event is mapped as a source of this  */
                                                /* Interrupt (bits higher than number of channels will be forced to r) */
      __IO uint32_t EN                  	:	1;  /* When 1, enables the DMA_INT1 mapping. */
      __I  uint32_t RESERVED1           	:	26;  /* Reserved */

    } bit;
  } INT1_SRCCFG;
  union {    /* DMA_INT2_SRCCFG Register */
    __IO uint32_t reg;
    struct { /* DMA_INT2_SRCCFG Control Bits */
      __IO uint32_t INT_SRC             	:	5;  /* Controls which channel's completion event is mapped as a source of this  */
                                                /* Interrupt (bits higher than number of channels will be forced to r) */
      __IO uint32_t EN                  	:	1;  /* When 1, enables the DMA_INT2 mapping. */
      __I  uint32_t RESERVED1           	:	26;  /* Reserved */

    } bit;
  } INT2_SRCCFG;
  union {    /* DMA_INT3_SRCCFG Register */
    __IO uint32_t reg;
    struct { /* DMA_INT3_SRCCFG Control Bits */
      __IO uint32_t INT_SRC             	:	5;  /* Controls which channel's completion event is mapped as a source of this  */
                                                /* Interrupt */
      __IO uint32_t EN                  	:	1;  /* When 1, enables the DMA_INT3 mapping. */
      __I  uint32_t RESERVED1           	:	26;  /* Reserved */

    } bit;
  } INT3_SRCCFG;
  uint8_t RESERVED2[4];
  union {    /* DMA_INT0_SRCFLG Register */
    __IO uint32_t reg;
    struct { /* DMA_INT0_SRCFLG Control Bits */
      __I  uint32_t CH0                 	:	1;  /* If 1, indicates that Channel 0 was the source of DMA_INT0 */
      __I  uint32_t CH1                 	:	1;  /* If 1, indicates that Channel 1 was the source of DMA_INT0 */
      __I  uint32_t CH2                 	:	1;  /* If 1, indicates that Channel 2 was the source of DMA_INT0 */
      __I  uint32_t CH3                 	:	1;  /* If 1, indicates that Channel 3 was the source of DMA_INT0 */
      __I  uint32_t CH4                 	:	1;  /* If 1, indicates that Channel 4 was the source of DMA_INT0 */
      __I  uint32_t CH5                 	:	1;  /* If 1, indicates that Channel 5 was the source of DMA_INT0 */
      __I  uint32_t CH6                 	:	1;  /* If 1, indicates that Channel 6 was the source of DMA_INT0 */
      __I  uint32_t CH7                 	:	1;  /* If 1, indicates that Channel 7 was the source of DMA_INT0 */
      __I  uint32_t CH8                 	:	1;  /* If 1, indicates that Channel 8 was the source of DMA_INT0 */
      __I  uint32_t CH9                 	:	1;  /* If 1, indicates that Channel 9 was the source of DMA_INT0 */
      __I  uint32_t CH10                	:	1;  /* If 1, indicates that Channel 10 was the source of DMA_INT0 */
      __I  uint32_t CH11                	:	1;  /* If 1, indicates that Channel 11 was the source of DMA_INT0 */
      __I  uint32_t CH12                	:	1;  /* If 1, indicates that Channel 12 was the source of DMA_INT0 */
      __I  uint32_t CH13                	:	1;  /* If 1, indicates that Channel 13 was the source of DMA_INT0 */
      __I  uint32_t CH14                	:	1;  /* If 1, indicates that Channel 14 was the source of DMA_INT0 */
      __I  uint32_t CH15                	:	1;  /* If 1, indicates that Channel 15 was the source of DMA_INT0 */
      __I  uint32_t CH16                	:	1;  /* If 1, indicates that Channel 16 was the source of DMA_INT0 */
      __I  uint32_t CH17                	:	1;  /* If 1, indicates that Channel 17 was the source of DMA_INT0 */
      __I  uint32_t CH18                	:	1;  /* If 1, indicates that Channel 18 was the source of DMA_INT0 */
      __I  uint32_t CH19                	:	1;  /* If 1, indicates that Channel 19 was the source of DMA_INT0 */
      __I  uint32_t CH20                	:	1;  /* If 1, indicates that Channel 20 was the source of DMA_INT0 */
      __I  uint32_t CH21                	:	1;  /* If 1, indicates that Channel 21 was the source of DMA_INT0 */
      __I  uint32_t CH22                	:	1;  /* If 1, indicates that Channel 22 was the source of DMA_INT0 */
      __I  uint32_t CH23                	:	1;  /* If 1, indicates that Channel 23 was the source of DMA_INT0 */
      __I  uint32_t CH24                	:	1;  /* If 1, indicates that Channel 24 was the source of DMA_INT0 */
      __I  uint32_t CH25                	:	1;  /* If 1, indicates that Channel 25 was the source of DMA_INT0 */
      __I  uint32_t CH26                	:	1;  /* If 1, indicates that Channel 26 was the source of DMA_INT0 */
      __I  uint32_t CH27                	:	1;  /* If 1, indicates that Channel 27 was the source of DMA_INT0 */
      __I  uint32_t CH28                	:	1;  /* If 1, indicates that Channel 28 was the source of DMA_INT0 */
      __I  uint32_t CH29                	:	1;  /* If 1, indicates that Channel 29 was the source of DMA_INT0 */
      __I  uint32_t CH30                	:	1;  /* If 1, indicates that Channel 30 was the source of DMA_INT0 */
      __I  uint32_t CH31                	:	1;  /* If 1, indicates that Channel 31 was the source of DMA_INT0 */

    } bit;
  } INT0_SRCFLG;
  union {    /* DMA_INT0_CLRFLG Register */
    __IO uint32_t reg;
    struct { /* DMA_INT0_CLRFLG Control Bits */
      __O  uint32_t CH0                 	:	1;  /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                /* Write 0 has no effect */
      __O  uint32_t CH1                 	:	1;  /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                /* Write 0 has no effect */
      __O  uint32_t CH2                 	:	1;  /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                /* Write 0 has no effect */
      __O  uint32_t CH3                 	:	1;  /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                /* Write 0 has no effect */
      __O  uint32_t CH4                 	:	1;  /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                /* Write 0 has no effect */
      __O  uint32_t CH5                 	:	1;  /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                /* Write 0 has no effect */
      __O  uint32_t CH6                 	:	1;  /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                /* Write 0 has no effect */
      __O  uint32_t CH7                 	:	1;  /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                /* Write 0 has no effect */
      __O  uint32_t CH8                 	:	1;  /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                /* Write 0 has no effect */
      __O  uint32_t CH9                 	:	1;  /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                /* Write 0 has no effect */
      __O  uint32_t CH10                	:	1;  /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                /* Write 0 has no effect */
      __O  uint32_t CH11                	:	1;  /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                /* Write 0 has no effect */
      __O  uint32_t CH12                	:	1;  /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                /* Write 0 has no effect */
      __O  uint32_t CH13                	:	1;  /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                /* Write 0 has no effect */
      __O  uint32_t CH14                	:	1;  /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                /* Write 0 has no effect */
      __O  uint32_t CH15                	:	1;  /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                /* Write 0 has no effect */
      __O  uint32_t CH16                	:	1;  /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                /* Write 0 has no effect */
      __O  uint32_t CH17                	:	1;  /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                /* Write 0 has no effect */
      __O  uint32_t CH18                	:	1;  /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                /* Write 0 has no effect */
      __O  uint32_t CH19                	:	1;  /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                /* Write 0 has no effect */
      __O  uint32_t CH20                	:	1;  /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                /* Write 0 has no effect */
      __O  uint32_t CH21                	:	1;  /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                /* Write 0 has no effect */
      __O  uint32_t CH22                	:	1;  /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                /* Write 0 has no effect */
      __O  uint32_t CH23                	:	1;  /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                /* Write 0 has no effect */
      __O  uint32_t CH24                	:	1;  /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                /* Write 0 has no effect */
      __O  uint32_t CH25                	:	1;  /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                /* Write 0 has no effect */
      __O  uint32_t CH26                	:	1;  /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                /* Write 0 has no effect */
      __O  uint32_t CH27                	:	1;  /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                /* Write 0 has no effect */
      __O  uint32_t CH28                	:	1;  /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                /* Write 0 has no effect */
      __O  uint32_t CH29                	:	1;  /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                /* Write 0 has no effect */
      __O  uint32_t CH30                	:	1;  /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                /* Write 0 has no effect */
      __O  uint32_t CH31                	:	1;  /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                /* Write 0 has no effect */

    } bit;
  } INT0_CLRFLG;
} DMA_Type;


/* DMA_DEVCONFIG Control Bits */
#define DMA_DEVCONFIG_NUM_DMA_CHANNELS__M                  (0x000000ff)          /* Reflects the number of DMA channels available on the device */
#define DMA_DEVCONFIG_NUM_SRC_PER_CHANNEL__M               (0x0000ff00)          /* Reflects the number of DMA sources per channel */
#define DMA_DEVCONFIG_RESERVED1__M                         (0xffff0000)          /*  */

/* DMA_SW_CHTRIG Control Bits */
#define DMA_SW_CHTRIG_CH0                                  (0x00000001)          /* Write 1 triggers DMA_CHANNEL0. Bit is auto-cleared when channel goes active. */
#define DMA_SW_CHTRIG_CH1                                  (0x00000002)          /* Write 1 triggers DMA_CHANNEL1. Bit is auto-cleared when channel goes active. */
#define DMA_SW_CHTRIG_CH2                                  (0x00000004)          /* Write 1 triggers DMA_CHANNEL2. Bit is auto-cleared when channel goes active. */
#define DMA_SW_CHTRIG_CH3                                  (0x00000008)          /* Write 1 triggers DMA_CHANNEL3. Bit is auto-cleared when channel goes active. */
#define DMA_SW_CHTRIG_CH4                                  (0x00000010)          /* Write 1 triggers DMA_CHANNEL4. Bit is auto-cleared when channel goes active. */
#define DMA_SW_CHTRIG_CH5                                  (0x00000020)          /* Write 1 triggers DMA_CHANNEL5. Bit is auto-cleared when channel goes active. */
#define DMA_SW_CHTRIG_CH6                                  (0x00000040)          /* Write 1 triggers DMA_CHANNEL6. Bit is auto-cleared when channel goes active. */
#define DMA_SW_CHTRIG_CH7                                  (0x00000080)          /* Write 1 triggers DMA_CHANNEL7. Bit is auto-cleared when channel goes active. */
#define DMA_SW_CHTRIG_CH8                                  (0x00000100)          /* Write 1 triggers DMA_CHANNEL8. Bit is auto-cleared when channel goes active. */
#define DMA_SW_CHTRIG_CH9                                  (0x00000200)          /* Write 1 triggers DMA_CHANNEL9. Bit is auto-cleared when channel goes active. */
#define DMA_SW_CHTRIG_CH10                                 (0x00000400)          /* Write 1 triggers DMA_CHANNEL10. Bit is auto-cleared when channel goes active. */
#define DMA_SW_CHTRIG_CH11                                 (0x00000800)          /* Write 1 triggers DMA_CHANNEL11. Bit is auto-cleared when channel goes active. */
#define DMA_SW_CHTRIG_CH12                                 (0x00001000)          /* Write 1 triggers DMA_CHANNEL12. Bit is auto-cleared when channel goes active. */
#define DMA_SW_CHTRIG_CH13                                 (0x00002000)          /* Write 1 triggers DMA_CHANNEL13. Bit is auto-cleared when channel goes active. */
#define DMA_SW_CHTRIG_CH14                                 (0x00004000)          /* Write 1 triggers DMA_CHANNEL14. Bit is auto-cleared when channel goes active. */
#define DMA_SW_CHTRIG_CH15                                 (0x00008000)          /* Write 1 triggers DMA_CHANNEL15. Bit is auto-cleared when channel goes active. */
#define DMA_SW_CHTRIG_CH16                                 (0x00010000)          /* Write 1 triggers DMA_CHANNEL16. Bit is auto-cleared when channel goes active. */
#define DMA_SW_CHTRIG_CH17                                 (0x00020000)          /* Write 1 triggers DMA_CHANNEL17. Bit is auto-cleared when channel goes active. */
#define DMA_SW_CHTRIG_CH18                                 (0x00040000)          /* Write 1 triggers DMA_CHANNEL18. Bit is auto-cleared when channel goes active. */
#define DMA_SW_CHTRIG_CH19                                 (0x00080000)          /* Write 1 triggers DMA_CHANNEL19. Bit is auto-cleared when channel goes active. */
#define DMA_SW_CHTRIG_CH20                                 (0x00100000)          /* Write 1 triggers DMA_CHANNEL20. Bit is auto-cleared when channel goes active. */
#define DMA_SW_CHTRIG_CH21                                 (0x00200000)          /* Write 1 triggers DMA_CHANNEL21. Bit is auto-cleared when channel goes active. */
#define DMA_SW_CHTRIG_CH22                                 (0x00400000)          /* Write 1 triggers DMA_CHANNEL22. Bit is auto-cleared when channel goes active. */
#define DMA_SW_CHTRIG_CH23                                 (0x00800000)          /* Write 1 triggers DMA_CHANNEL23. Bit is auto-cleared when channel goes active. */
#define DMA_SW_CHTRIG_CH24                                 (0x01000000)          /* Write 1 triggers DMA_CHANNEL24. Bit is auto-cleared when channel goes active. */
#define DMA_SW_CHTRIG_CH25                                 (0x02000000)          /* Write 1 triggers DMA_CHANNEL25. Bit is auto-cleared when channel goes active. */
#define DMA_SW_CHTRIG_CH26                                 (0x04000000)          /* Write 1 triggers DMA_CHANNEL26. Bit is auto-cleared when channel goes active. */
#define DMA_SW_CHTRIG_CH27                                 (0x08000000)          /* Write 1 triggers DMA_CHANNEL27. Bit is auto-cleared when channel goes active. */
#define DMA_SW_CHTRIG_CH28                                 (0x10000000)          /* Write 1 triggers DMA_CHANNEL28. Bit is auto-cleared when channel goes active. */
#define DMA_SW_CHTRIG_CH29                                 (0x20000000)          /* Write 1 triggers DMA_CHANNEL29. Bit is auto-cleared when channel goes active. */
#define DMA_SW_CHTRIG_CH30                                 (0x40000000)          /* Write 1 triggers DMA_CHANNEL30. Bit is auto-cleared when channel goes active. */
#define DMA_SW_CHTRIG_CH31                                 (0x80000000)          /* Write 1 triggers DMA_CHANNEL31. Bit is auto-cleared when channel goes active. */

/* DMA_CH0_SRCCFG Control Bits */
#define DMA_CH0_SRCCFG_SRC__M                              (0x000000ff)          /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                                                 /* higher than the number of available sources will be forced to r mode) */
#define DMA_CH0_SRCCFG_RESERVED1__M                        (0xffffff00)          /*  */

/* DMA_CH1_SRCCFG Control Bits */
#define DMA_CH1_SRCCFG_SRC__M                              (0x000000ff)          /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                                                 /* higher than the number of available sources will be forced to r mode) */
#define DMA_CH1_SRCCFG_RESERVED1__M                        (0xffffff00)          /*  */

/* DMA_CH2_SRCCFG Control Bits */
#define DMA_CH2_SRCCFG_SRC__M                              (0x000000ff)          /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                                                 /* higher than the number of available sources will be forced to r mode) */
#define DMA_CH2_SRCCFG_RESERVED1__M                        (0xffffff00)          /*  */

/* DMA_CH3_SRCCFG Control Bits */
#define DMA_CH3_SRCCFG_SRC__M                              (0x000000ff)          /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                                                 /* higher than the number of available sources will be forced to r mode) */
#define DMA_CH3_SRCCFG_RESERVED1__M                        (0xffffff00)          /*  */

/* DMA_CH4_SRCCFG Control Bits */
#define DMA_CH4_SRCCFG_SRC__M                              (0x000000ff)          /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                                                 /* higher than the number of available sources will be forced to r mode) */
#define DMA_CH4_SRCCFG_RESERVED1__M                        (0xffffff00)          /*  */

/* DMA_CH5_SRCCFG Control Bits */
#define DMA_CH5_SRCCFG_SRC__M                              (0x000000ff)          /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                                                 /* higher than the number of available sources will be forced to r mode) */
#define DMA_CH5_SRCCFG_RESERVED1__M                        (0xffffff00)          /*  */

/* DMA_CH6_SRCCFG Control Bits */
#define DMA_CH6_SRCCFG_SRC__M                              (0x000000ff)          /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                                                 /* higher than the number of available sources will be forced to r mode) */
#define DMA_CH6_SRCCFG_RESERVED1__M                        (0xffffff00)          /*  */

/* DMA_CH7_SRCCFG Control Bits */
#define DMA_CH7_SRCCFG_SRC__M                              (0x000000ff)          /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                                                 /* higher than the number of available sources will be forced to r mode) */
#define DMA_CH7_SRCCFG_RESERVED1__M                        (0xffffff00)          /*  */

/* DMA_CH8_SRCCFG Control Bits */
#define DMA_CH8_SRCCFG_SRC__M                              (0x000000ff)          /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                                                 /* higher than the number of available sources will be forced to r mode) */
#define DMA_CH8_SRCCFG_RESERVED1__M                        (0xffffff00)          /*  */

/* DMA_CH9_SRCCFG Control Bits */
#define DMA_CH9_SRCCFG_SRC__M                              (0x000000ff)          /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                                                 /* higher than the number of available sources will be forced to r mode) */
#define DMA_CH9_SRCCFG_RESERVED1__M                        (0xffffff00)          /*  */

/* DMA_CH10_SRCCFG Control Bits */
#define DMA_CH10_SRCCFG_SRC__M                             (0x000000ff)          /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                                                 /* higher than the number of available sources will be forced to r mode) */
#define DMA_CH10_SRCCFG_RESERVED1__M                       (0xffffff00)          /*  */

/* DMA_CH11_SRCCFG Control Bits */
#define DMA_CH11_SRCCFG_SRC__M                             (0x000000ff)          /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                                                 /* higher than the number of available sources will be forced to r mode) */
#define DMA_CH11_SRCCFG_RESERVED1__M                       (0xffffff00)          /*  */

/* DMA_CH12_SRCCFG Control Bits */
#define DMA_CH12_SRCCFG_SRC__M                             (0x000000ff)          /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                                                 /* higher than the number of available sources will be forced to r mode) */
#define DMA_CH12_SRCCFG_RESERVED1__M                       (0xffffff00)          /*  */

/* DMA_CH13_SRCCFG Control Bits */
#define DMA_CH13_SRCCFG_SRC__M                             (0x000000ff)          /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                                                 /* higher than the number of available sources will be forced to r mode) */
#define DMA_CH13_SRCCFG_RESERVED1__M                       (0xffffff00)          /*  */

/* DMA_CH14_SRCCFG Control Bits */
#define DMA_CH14_SRCCFG_SRC__M                             (0x000000ff)          /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                                                 /* higher than the number of available sources will be forced to r mode) */
#define DMA_CH14_SRCCFG_RESERVED1__M                       (0xffffff00)          /*  */

/* DMA_CH15_SRCCFG Control Bits */
#define DMA_CH15_SRCCFG_SRC__M                             (0x000000ff)          /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                                                 /* higher than the number of available sources will be forced to r mode) */
#define DMA_CH15_SRCCFG_RESERVED1__M                       (0xffffff00)          /*  */

/* DMA_CH16_SRCCFG Control Bits */
#define DMA_CH16_SRCCFG_SRC__M                             (0x000000ff)          /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                                                 /* higher than the number of available sources will be forced to r mode) */
#define DMA_CH16_SRCCFG_RESERVED1__M                       (0xffffff00)          /*  */

/* DMA_CH17_SRCCFG Control Bits */
#define DMA_CH17_SRCCFG_SRC__M                             (0x000000ff)          /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                                                 /* higher than the number of available sources will be forced to r mode) */
#define DMA_CH17_SRCCFG_RESERVED1__M                       (0xffffff00)          /*  */

/* DMA_CH18_SRCCFG Control Bits */
#define DMA_CH18_SRCCFG_SRC__M                             (0x000000ff)          /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                                                 /* higher than the number of available sources will be forced to r mode) */
#define DMA_CH18_SRCCFG_RESERVED1__M                       (0xffffff00)          /*  */

/* DMA_CH19_SRCCFG Control Bits */
#define DMA_CH19_SRCCFG_SRC__M                             (0x000000ff)          /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                                                 /* higher than the number of available sources will be forced to r mode) */
#define DMA_CH19_SRCCFG_RESERVED1__M                       (0xffffff00)          /*  */

/* DMA_CH20_SRCCFG Control Bits */
#define DMA_CH20_SRCCFG_SRC__M                             (0x000000ff)          /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                                                 /* higher than the number of available sources will be forced to r mode) */
#define DMA_CH20_SRCCFG_RESERVED1__M                       (0xffffff00)          /*  */

/* DMA_CH21_SRCCFG Control Bits */
#define DMA_CH21_SRCCFG_SRC__M                             (0x000000ff)          /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                                                 /* higher than the number of available sources will be forced to r mode) */
#define DMA_CH21_SRCCFG_RESERVED1__M                       (0xffffff00)          /*  */

/* DMA_CH22_SRCCFG Control Bits */
#define DMA_CH22_SRCCFG_SRC__M                             (0x000000ff)          /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                                                 /* higher than the number of available sources will be forced to r mode) */
#define DMA_CH22_SRCCFG_RESERVED1__M                       (0xffffff00)          /*  */

/* DMA_CH23_SRCCFG Control Bits */
#define DMA_CH23_SRCCFG_SRC__M                             (0x000000ff)          /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                                                 /* higher than the number of available sources will be forced to r mode) */
#define DMA_CH23_SRCCFG_RESERVED1__M                       (0xffffff00)          /*  */

/* DMA_CH24_SRCCFG Control Bits */
#define DMA_CH24_SRCCFG_SRC__M                             (0x000000ff)          /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                                                 /* higher than the number of available sources will be forced to r mode) */
#define DMA_CH24_SRCCFG_RESERVED1__M                       (0xffffff00)          /*  */

/* DMA_CH25_SRCCFG Control Bits */
#define DMA_CH25_SRCCFG_SRC__M                             (0x000000ff)          /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                                                 /* higher than the number of available sources will be forced to r mode) */
#define DMA_CH25_SRCCFG_RESERVED1__M                       (0xffffff00)          /*  */

/* DMA_CH26_SRCCFG Control Bits */
#define DMA_CH26_SRCCFG_SRC__M                             (0x000000ff)          /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                                                 /* higher than the number of available sources will be forced to r mode) */
#define DMA_CH26_SRCCFG_RESERVED1__M                       (0xffffff00)          /*  */

/* DMA_CH27_SRCCFG Control Bits */
#define DMA_CH27_SRCCFG_SRC__M                             (0x000000ff)          /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                                                 /* higher than the number of available sources will be forced to r mode) */
#define DMA_CH27_SRCCFG_RESERVED1__M                       (0xffffff00)          /*  */

/* DMA_CH28_SRCCFG Control Bits */
#define DMA_CH28_SRCCFG_SRC__M                             (0x000000ff)          /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                                                 /* higher than the number of available sources will be forced to r mode) */
#define DMA_CH28_SRCCFG_RESERVED1__M                       (0xffffff00)          /*  */

/* DMA_CH29_SRCCFG Control Bits */
#define DMA_CH29_SRCCFG_SRC__M                             (0x000000ff)          /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                                                 /* higher than the number of available sources will be forced to r mode) */
#define DMA_CH29_SRCCFG_RESERVED1__M                       (0xffffff00)          /*  */

/* DMA_CH30_SRCCFG Control Bits */
#define DMA_CH30_SRCCFG_SRC__M                             (0x000000ff)          /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                                                 /* higher than the number of available sources will be forced to r mode) */
#define DMA_CH30_SRCCFG_RESERVED1__M                       (0xffffff00)          /*  */

/* DMA_CH31_SRCCFG Control Bits */
#define DMA_CH31_SRCCFG_SRC__M                             (0x000000ff)          /* Controls which device level DMA source is mapped to the channel input (bits  */
                                                                                 /* higher than the number of available sources will be forced to r mode) */
#define DMA_CH31_SRCCFG_RESERVED1__M                       (0xffffff00)          /*  */

/* DMA_INT1_SRCCFG Control Bits */
#define DMA_INT1_SRCCFG_INT_SRC__M                         (0x0000001f)          /* Controls which channel's completion event is mapped as a source of this  */
                                                                                 /* Interrupt (bits higher than number of channels will be forced to r) */
#define DMA_INT1_SRCCFG_EN                                 (0x00000020)          /* When 1, enables the DMA_INT1 mapping. */
#define DMA_INT1_SRCCFG_RESERVED1__M                       (0xffffffc0)          /*  */

/* DMA_INT2_SRCCFG Control Bits */
#define DMA_INT2_SRCCFG_INT_SRC__M                         (0x0000001f)          /* Controls which channel's completion event is mapped as a source of this  */
                                                                                 /* Interrupt (bits higher than number of channels will be forced to r) */
#define DMA_INT2_SRCCFG_EN                                 (0x00000020)          /* When 1, enables the DMA_INT2 mapping. */
#define DMA_INT2_SRCCFG_RESERVED1__M                       (0xffffffc0)          /*  */

/* DMA_INT3_SRCCFG Control Bits */
#define DMA_INT3_SRCCFG_INT_SRC__M                         (0x0000001f)          /* Controls which channel's completion event is mapped as a source of this  */
                                                                                 /* Interrupt */
#define DMA_INT3_SRCCFG_EN                                 (0x00000020)          /* When 1, enables the DMA_INT3 mapping. */
#define DMA_INT3_SRCCFG_RESERVED1__M                       (0xffffffc0)          /*  */

/* DMA_INT0_SRCFLG Control Bits */
#define DMA_INT0_SRCFLG_CH0                                (0x00000001)          /* If 1, indicates that Channel 0 was the source of DMA_INT0 */
#define DMA_INT0_SRCFLG_CH1                                (0x00000002)          /* If 1, indicates that Channel 1 was the source of DMA_INT0 */
#define DMA_INT0_SRCFLG_CH2                                (0x00000004)          /* If 1, indicates that Channel 2 was the source of DMA_INT0 */
#define DMA_INT0_SRCFLG_CH3                                (0x00000008)          /* If 1, indicates that Channel 3 was the source of DMA_INT0 */
#define DMA_INT0_SRCFLG_CH4                                (0x00000010)          /* If 1, indicates that Channel 4 was the source of DMA_INT0 */
#define DMA_INT0_SRCFLG_CH5                                (0x00000020)          /* If 1, indicates that Channel 5 was the source of DMA_INT0 */
#define DMA_INT0_SRCFLG_CH6                                (0x00000040)          /* If 1, indicates that Channel 6 was the source of DMA_INT0 */
#define DMA_INT0_SRCFLG_CH7                                (0x00000080)          /* If 1, indicates that Channel 7 was the source of DMA_INT0 */
#define DMA_INT0_SRCFLG_CH8                                (0x00000100)          /* If 1, indicates that Channel 8 was the source of DMA_INT0 */
#define DMA_INT0_SRCFLG_CH9                                (0x00000200)          /* If 1, indicates that Channel 9 was the source of DMA_INT0 */
#define DMA_INT0_SRCFLG_CH10                               (0x00000400)          /* If 1, indicates that Channel 10 was the source of DMA_INT0 */
#define DMA_INT0_SRCFLG_CH11                               (0x00000800)          /* If 1, indicates that Channel 11 was the source of DMA_INT0 */
#define DMA_INT0_SRCFLG_CH12                               (0x00001000)          /* If 1, indicates that Channel 12 was the source of DMA_INT0 */
#define DMA_INT0_SRCFLG_CH13                               (0x00002000)          /* If 1, indicates that Channel 13 was the source of DMA_INT0 */
#define DMA_INT0_SRCFLG_CH14                               (0x00004000)          /* If 1, indicates that Channel 14 was the source of DMA_INT0 */
#define DMA_INT0_SRCFLG_CH15                               (0x00008000)          /* If 1, indicates that Channel 15 was the source of DMA_INT0 */
#define DMA_INT0_SRCFLG_CH16                               (0x00010000)          /* If 1, indicates that Channel 16 was the source of DMA_INT0 */
#define DMA_INT0_SRCFLG_CH17                               (0x00020000)          /* If 1, indicates that Channel 17 was the source of DMA_INT0 */
#define DMA_INT0_SRCFLG_CH18                               (0x00040000)          /* If 1, indicates that Channel 18 was the source of DMA_INT0 */
#define DMA_INT0_SRCFLG_CH19                               (0x00080000)          /* If 1, indicates that Channel 19 was the source of DMA_INT0 */
#define DMA_INT0_SRCFLG_CH20                               (0x00100000)          /* If 1, indicates that Channel 20 was the source of DMA_INT0 */
#define DMA_INT0_SRCFLG_CH21                               (0x00200000)          /* If 1, indicates that Channel 21 was the source of DMA_INT0 */
#define DMA_INT0_SRCFLG_CH22                               (0x00400000)          /* If 1, indicates that Channel 22 was the source of DMA_INT0 */
#define DMA_INT0_SRCFLG_CH23                               (0x00800000)          /* If 1, indicates that Channel 23 was the source of DMA_INT0 */
#define DMA_INT0_SRCFLG_CH24                               (0x01000000)          /* If 1, indicates that Channel 24 was the source of DMA_INT0 */
#define DMA_INT0_SRCFLG_CH25                               (0x02000000)          /* If 1, indicates that Channel 25 was the source of DMA_INT0 */
#define DMA_INT0_SRCFLG_CH26                               (0x04000000)          /* If 1, indicates that Channel 26 was the source of DMA_INT0 */
#define DMA_INT0_SRCFLG_CH27                               (0x08000000)          /* If 1, indicates that Channel 27 was the source of DMA_INT0 */
#define DMA_INT0_SRCFLG_CH28                               (0x10000000)          /* If 1, indicates that Channel 28 was the source of DMA_INT0 */
#define DMA_INT0_SRCFLG_CH29                               (0x20000000)          /* If 1, indicates that Channel 29 was the source of DMA_INT0 */
#define DMA_INT0_SRCFLG_CH30                               (0x40000000)          /* If 1, indicates that Channel 30 was the source of DMA_INT0 */
#define DMA_INT0_SRCFLG_CH31                               (0x80000000)          /* If 1, indicates that Channel 31 was the source of DMA_INT0 */

/* DMA_INT0_CLRFLG Control Bits */
#define DMA_INT0_CLRFLG_CH0                                (0x00000001)          /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                                                 /* Write 0 has no effect */
#define DMA_INT0_CLRFLG_CH1                                (0x00000002)          /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                                                 /* Write 0 has no effect */
#define DMA_INT0_CLRFLG_CH2                                (0x00000004)          /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                                                 /* Write 0 has no effect */
#define DMA_INT0_CLRFLG_CH3                                (0x00000008)          /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                                                 /* Write 0 has no effect */
#define DMA_INT0_CLRFLG_CH4                                (0x00000010)          /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                                                 /* Write 0 has no effect */
#define DMA_INT0_CLRFLG_CH5                                (0x00000020)          /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                                                 /* Write 0 has no effect */
#define DMA_INT0_CLRFLG_CH6                                (0x00000040)          /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                                                 /* Write 0 has no effect */
#define DMA_INT0_CLRFLG_CH7                                (0x00000080)          /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                                                 /* Write 0 has no effect */
#define DMA_INT0_CLRFLG_CH8                                (0x00000100)          /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                                                 /* Write 0 has no effect */
#define DMA_INT0_CLRFLG_CH9                                (0x00000200)          /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                                                 /* Write 0 has no effect */
#define DMA_INT0_CLRFLG_CH10                               (0x00000400)          /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                                                 /* Write 0 has no effect */
#define DMA_INT0_CLRFLG_CH11                               (0x00000800)          /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                                                 /* Write 0 has no effect */
#define DMA_INT0_CLRFLG_CH12                               (0x00001000)          /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                                                 /* Write 0 has no effect */
#define DMA_INT0_CLRFLG_CH13                               (0x00002000)          /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                                                 /* Write 0 has no effect */
#define DMA_INT0_CLRFLG_CH14                               (0x00004000)          /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                                                 /* Write 0 has no effect */
#define DMA_INT0_CLRFLG_CH15                               (0x00008000)          /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                                                 /* Write 0 has no effect */
#define DMA_INT0_CLRFLG_CH16                               (0x00010000)          /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                                                 /* Write 0 has no effect */
#define DMA_INT0_CLRFLG_CH17                               (0x00020000)          /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                                                 /* Write 0 has no effect */
#define DMA_INT0_CLRFLG_CH18                               (0x00040000)          /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                                                 /* Write 0 has no effect */
#define DMA_INT0_CLRFLG_CH19                               (0x00080000)          /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                                                 /* Write 0 has no effect */
#define DMA_INT0_CLRFLG_CH20                               (0x00100000)          /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                                                 /* Write 0 has no effect */
#define DMA_INT0_CLRFLG_CH21                               (0x00200000)          /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                                                 /* Write 0 has no effect */
#define DMA_INT0_CLRFLG_CH22                               (0x00400000)          /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                                                 /* Write 0 has no effect */
#define DMA_INT0_CLRFLG_CH23                               (0x00800000)          /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                                                 /* Write 0 has no effect */
#define DMA_INT0_CLRFLG_CH24                               (0x01000000)          /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                                                 /* Write 0 has no effect */
#define DMA_INT0_CLRFLG_CH25                               (0x02000000)          /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                                                 /* Write 0 has no effect */
#define DMA_INT0_CLRFLG_CH26                               (0x04000000)          /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                                                 /* Write 0 has no effect */
#define DMA_INT0_CLRFLG_CH27                               (0x08000000)          /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                                                 /* Write 0 has no effect */
#define DMA_INT0_CLRFLG_CH28                               (0x10000000)          /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                                                 /* Write 0 has no effect */
#define DMA_INT0_CLRFLG_CH29                               (0x20000000)          /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                                                 /* Write 0 has no effect */
#define DMA_INT0_CLRFLG_CH30                               (0x40000000)          /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                                                 /* Write 0 has no effect */
#define DMA_INT0_CLRFLG_CH31                               (0x80000000)          /* Write 1 clears the corresponding flag bit in the DMA_INT0_SRCFLG_REG register.  */
                                                                                 /* Write 0 has no effect */


//*****************************************************************************
// EUSCI_A0_SPI Registers
//*****************************************************************************
typedef struct {
  union {    /* EUSCI_A0_SPI_CTLW0 Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A0_SPI_CTLW0 Control Bits */
      __IO uint16_t UCSWRST             	:	1;  /* Software reset enable */
      __IO uint16_t UCSTEM              	:	1;  /* STE mode select in master mode. This byte is ignored in slave or 3-wire mode. */
      __I  uint16_t RESERVED1           	:	4;  /* Reserved */
      __IO uint16_t UCSSEL              	:	2;  /* eUSCI clock source select. These bits select the BRCLK source clock in master  */
                                                /* mode. UCxCLK is always used in slave mode. */
      __IO uint16_t UCSYNC              	:	1;  /* Synchronous mode enable */
      __IO uint16_t UCMODE              	:	2;  /* eUSCI mode. The UCMODEx bits select the synchronous mode when UCSYNC = 1. */
      __IO uint16_t UCMST               	:	1;  /* Master mode select */
      __IO uint16_t UC7BIT              	:	1;  /* Character length. Selects 7-bit or 8-bit character length. */
      __IO uint16_t UCMSB               	:	1;  /* MSB first select. Controls the direction of the receive and transmit shift  */
                                                /* register. */
      __IO uint16_t UCCKPL              	:	1;  /* Clock polarity select */
      __IO uint16_t UCCKPH              	:	1;  /* Clock phase select */

    } bit;
  } CTLW0;
  uint8_t RESERVED0[4];
  uint16_t BRW; /* eUSCI_Ax Bit Rate Control Word */
  uint8_t RESERVED1[2];
  union {    /* EUSCI_A0_SPI_STATW Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A0_SPI_STATW Control Bits */
      __I  uint16_t UCBUSY              	:	1;  /* eUSCI busy. This bit indicates if a transmit or receive operation is in progress */
                                                /* . */
      __IO uint16_t RESERVED2           	:	4;  /* Reserved */
      __IO uint16_t UCOE                	:	1;  /* Overrun error flag. This bit is set when a character is transferred into  */
                                                /* UCxRXBUF before the previous character was read. UCOE is cleared automatically  */
                                                /* when UCxRXBUF is read, and must not be cleared by software. Otherwise, it does  */
                                                /* not function correctly. */
      __IO uint16_t UCFE                	:	1;  /* Framing error flag. This bit indicates a bus conflict in 4-wire master mode.  */
                                                /* UCFE is not used in 3-wire master or any slave mode. */
      __IO uint16_t UCLISTEN            	:	1;  /* Listen enable. The UCLISTEN bit selects loopback mode. */
      __I  uint16_t RESERVED1           	:	8;  /* Reserved */

    } bit;
  } STATW;
  union {    /* EUSCI_A0_SPI_RXBUF Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A0_SPI_RXBUF Control Bits */
      __I  uint16_t UCRXBUF             	:	8;  /* The receive-data buffer is user accessible and contains the last received  */
                                                /* character from the receive shift register. Reading UCxRXBUF resets the receive- */
                                                /* error bits and UCRXIFG. In 7-bit data mode, UCxRXBUF is LSB justified and the  */
                                                /* MSB is always reset. */
      __I  uint16_t RESERVED1           	:	8;  /* Reserved */

    } bit;
  } RXBUF;
  union {    /* EUSCI_A0_SPI_TXBUF Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A0_SPI_TXBUF Control Bits */
      __IO uint16_t UCTXBUF             	:	8;  /* The transmit data buffer is user accessible and holds the data waiting to be  */
                                                /* moved into the transmit shift register and transmitted. Writing to the transmit  */
                                                /* data buffer clears UCTXIFG. The MSB of UCxTXBUF is not used for 7-bit data and  */
                                                /* is reset. */
      __I  uint16_t RESERVED1           	:	8;  /* Reserved */

    } bit;
  } TXBUF;
  uint8_t RESERVED2[10];
  union {    /* EUSCI_A0_SPI_IE Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A0_SPI_IE Control Bits */
      __IO uint16_t UCRXIE              	:	1;  /* Receive interrupt enable */
      __IO uint16_t UCTXIE              	:	1;  /* Transmit interrupt enable */
      __I  uint16_t RESERVED1           	:	14;  /* Reserved */

    } bit;
  } IE;
  union {    /* EUSCI_A0_SPI_IFG Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A0_SPI_IFG Control Bits */
      __IO uint16_t UCRXIFG             	:	1;  /* Receive interrupt flag. UCRXIFG is set when UCxxRXBUF has received a complete  */
                                                /* character. */
      __IO uint16_t UCTXIFG             	:	1;  /* Transmit interrupt flag. UCTXIFG is set when UCxxTXBUF empty. */
      __I  uint16_t RESERVED1           	:	14;  /* Reserved */

    } bit;
  } IFG;
  uint16_t IV; /* eUSCI_Ax Interrupt Vector */
} EUSCI_A0_SPI_Type;


/* EUSCI_A0_SPI_CTLW0 Control Bits */
#define EUSCI_A0_SPI_CTLW0_UCSWRST                         (0x0001)              /* Software reset enable */
#define EUSCI_A0_SPI_CTLW0_UCSTEM                          (0x0002)              /* STE mode select in master mode. This byte is ignored in slave or 3-wire mode. */
#define EUSCI_A0_SPI_CTLW0_RESERVED1__M                    (0x003c)              /*  */
#define EUSCI_A0_SPI_CTLW0_UCSSEL__M                       (0x00c0)              /* eUSCI clock source select. These bits select the BRCLK source clock in master  */
                                                                                 /* mode. UCxCLK is always used in slave mode. */
#define EUSCI_A0_SPI_CTLW0_UCSSEL__1                       (0x0040)              /* ACLK */
#define EUSCI_A0_SPI_CTLW0_UCSSEL__1_ACLK                  (0x0040)              /* ACLK */
#define EUSCI_A0_SPI_CTLW0_UCSSEL__2                       (0x0080)              /* SMCLK */
#define EUSCI_A0_SPI_CTLW0_UCSSEL__2_SMCLK                 (0x0080)              /* SMCLK */
#define EUSCI_A0_SPI_CTLW0_UCSSEL__3                       (0x00c0)              /* SMCLK */
#define EUSCI_A0_SPI_CTLW0_UCSSEL__3_SMCLK                 (0x00c0)              /* SMCLK */
#define EUSCI_A0_SPI_CTLW0_UCSYNC                          (0x0100)              /* Synchronous mode enable */
#define EUSCI_A0_SPI_CTLW0_UCMODE__M                       (0x0600)              /* eUSCI mode. The UCMODEx bits select the synchronous mode when UCSYNC = 1. */
#define EUSCI_A0_SPI_CTLW0_UCMODE__0                       (0x0000)              /* 3-pin SPI */
#define EUSCI_A0_SPI_CTLW0_UCMODE__0_3_PIN_SPI             (0x0000)              /* 3-pin SPI */
#define EUSCI_A0_SPI_CTLW0_UCMODE__1                       (0x0200)              /* 4-pin SPI with UCxSTE active high: Slave enabled when UCxSTE = 1 */
#define EUSCI_A0_SPI_CTLW0_UCMODE__2                       (0x0400)              /* 4-pin SPI with UCxSTE active low: Slave enabled when UCxSTE = 0 */
#define EUSCI_A0_SPI_CTLW0_UCMODE__3                       (0x0600)              /* I2C mode */
#define EUSCI_A0_SPI_CTLW0_UCMODE__3_I2C_MODE              (0x0600)              /* I2C mode */
#define EUSCI_A0_SPI_CTLW0_UCMST                           (0x0800)              /* Master mode select */
#define EUSCI_A0_SPI_CTLW0_UC7BIT                          (0x1000)              /* Character length. Selects 7-bit or 8-bit character length. */
#define EUSCI_A0_SPI_CTLW0_UCMSB                           (0x2000)              /* MSB first select. Controls the direction of the receive and transmit shift  */
                                                                                 /* register. */
#define EUSCI_A0_SPI_CTLW0_UCCKPL                          (0x4000)              /* Clock polarity select */
#define EUSCI_A0_SPI_CTLW0_UCCKPH                          (0x8000)              /* Clock phase select */

/* EUSCI_A0_SPI_STATW Control Bits */
#define EUSCI_A0_SPI_STATW_UCBUSY                          (0x0001)              /* eUSCI busy. This bit indicates if a transmit or receive operation is in progress */
                                                                                 /* . */
#define EUSCI_A0_SPI_STATW_RESERVED2__M                    (0x001e)              /*  */
#define EUSCI_A0_SPI_STATW_UCOE                            (0x0020)              /* Overrun error flag. This bit is set when a character is transferred into  */
                                                                                 /* UCxRXBUF before the previous character was read. UCOE is cleared automatically  */
                                                                                 /* when UCxRXBUF is read, and must not be cleared by software. Otherwise, it does  */
                                                                                 /* not function correctly. */
#define EUSCI_A0_SPI_STATW_UCFE                            (0x0040)              /* Framing error flag. This bit indicates a bus conflict in 4-wire master mode.  */
                                                                                 /* UCFE is not used in 3-wire master or any slave mode. */
#define EUSCI_A0_SPI_STATW_UCLISTEN                        (0x0080)              /* Listen enable. The UCLISTEN bit selects loopback mode. */
#define EUSCI_A0_SPI_STATW_RESERVED1__M                    (0xff00)              /*  */

/* EUSCI_A0_SPI_RXBUF Control Bits */
#define EUSCI_A0_SPI_RXBUF_UCRXBUF__M                      (0x00ff)              /* The receive-data buffer is user accessible and contains the last received  */
                                                                                 /* character from the receive shift register. Reading UCxRXBUF resets the receive- */
                                                                                 /* error bits and UCRXIFG. In 7-bit data mode, UCxRXBUF is LSB justified and the  */
                                                                                 /* MSB is always reset. */
#define EUSCI_A0_SPI_RXBUF_RESERVED1__M                    (0xff00)              /*  */

/* EUSCI_A0_SPI_TXBUF Control Bits */
#define EUSCI_A0_SPI_TXBUF_UCTXBUF__M                      (0x00ff)              /* The transmit data buffer is user accessible and holds the data waiting to be  */
                                                                                 /* moved into the transmit shift register and transmitted. Writing to the transmit  */
                                                                                 /* data buffer clears UCTXIFG. The MSB of UCxTXBUF is not used for 7-bit data and  */
                                                                                 /* is reset. */
#define EUSCI_A0_SPI_TXBUF_RESERVED1__M                    (0xff00)              /*  */

/* EUSCI_A0_SPI_IE Control Bits */
#define EUSCI_A0_SPI_IE_UCRXIE                             (0x0001)              /* Receive interrupt enable */
#define EUSCI_A0_SPI_IE_UCTXIE                             (0x0002)              /* Transmit interrupt enable */
#define EUSCI_A0_SPI_IE_RESERVED1__M                       (0xfffc)              /*  */

/* EUSCI_A0_SPI_IFG Control Bits */
#define EUSCI_A0_SPI_IFG_UCRXIFG                           (0x0001)              /* Receive interrupt flag. UCRXIFG is set when UCxxRXBUF has received a complete  */
                                                                                 /* character. */
#define EUSCI_A0_SPI_IFG_UCTXIFG                           (0x0002)              /* Transmit interrupt flag. UCTXIFG is set when UCxxTXBUF empty. */
#define EUSCI_A0_SPI_IFG_RESERVED1__M                      (0xfffc)              /*  */


//*****************************************************************************
// EUSCI_A0_UART Registers
//*****************************************************************************
typedef struct {
  union {    /* EUSCI_A0_UART_CTLW0 Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A0_UART_CTLW0 Control Bits */
      __IO uint16_t UCSWRST             	:	1;  /* Software reset enable */
      __IO uint16_t UCTXBRK             	:	1;  /* Transmit break. Transmits a break with the next write to the transmit buffer. In */
                                                /* UART mode with automatic baud-rate detection, 055h must be written into  */
                                                /* UCAxTXBUF to generate the required break/synch fields. Otherwise, 0h must be  */
                                                /* written into the transmit buffer. */
      __IO uint16_t UCTXADDR            	:	1;  /* Transmit address. Next frame to be transmitted is marked as address, depending  */
                                                /* on the selected multiprocessor mode. */
      __IO uint16_t UCDORM              	:	1;  /* Dormant. Puts eUSCI_A into sleep mode. */
      __IO uint16_t UCBRKIE             	:	1;  /* Receive break character interrupt enable */
      __IO uint16_t UCRXEIE             	:	1;  /* Receive erroneous-character interrupt enable */
      __IO uint16_t UCSSEL              	:	2;  /* eUSCI_A clock source select. These bits select the BRCLK source clock. */
      __IO uint16_t UCSYNC              	:	1;  /* Synchronous mode enable */
      __IO uint16_t UCMODE              	:	2;  /* eUSCI_A mode. The UCMODEx bits select the asynchronous mode when UCSYNC = 0. */
      __IO uint16_t UCSPB               	:	1;  /* Stop bit select. Number of stop bits. */
      __IO uint16_t UC7BIT              	:	1;  /* Character length. Selects 7-bit or 8-bit character length. */
      __IO uint16_t UCMSB               	:	1;  /* MSB first select. Controls the direction of the receive and transmit shift  */
                                                /* register. */
      __IO uint16_t UCPAR               	:	1;  /* Parity select. UCPAR is not used when parity is disabled. */
      __IO uint16_t UCPEN               	:	1;  /* Parity enable */

    } bit;
  } CTLW0;
  union {    /* EUSCI_A0_UART_CTLW1 Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A0_UART_CTLW1 Control Bits */
      __IO uint16_t UCGLIT              	:	2;  /* Deglitch time */
      __I  uint16_t RESERVED1           	:	14;  /* Reserved */

    } bit;
  } CTLW1;
  uint8_t RESERVED0[2];
  uint16_t BRW; /* eUSCI_Ax Baud Rate Control Word */
  union {    /* EUSCI_A0_UART_MCTLW Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A0_UART_MCTLW Control Bits */
      __IO uint16_t UCOS16              	:	1;  /* Oversampling mode enabled */
      __I  uint16_t RESERVED1           	:	3;  /* Reserved */
      __IO uint16_t UCBRF               	:	4;  /* First modulation stage select. These bits determine the modulation pattern for  */
                                                /* BITCLK16 when UCOS16 = 1. Ignored with UCOS16 = 0. The "Oversampling Baud-Rate  */
                                                /* Generation" section shows the modulation pattern. */
      __IO uint16_t UCBRS               	:	8;  /* Second modulation stage select. These bits hold a free modulation pattern for  */
                                                /* BITCLK. */

    } bit;
  } MCTLW;
  union {    /* EUSCI_A0_UART_STATW Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A0_UART_STATW Control Bits */
      __I  uint16_t UCBUSY              	:	1;  /* eUSCI_A busy. This bit indicates if a transmit or receive operation is in  */
                                                /* progress. */
      __IO uint16_t UCADDR_UCIDLE       	:	1;  /* UCADDR: Address received in address-bit multiprocessor mode. UCADDR is cleared  */
                                                /* when UCAxRXBUF is read. UCIDLE: Idle line detected in idle-line multiprocessor  */
                                                /* mode. UCIDLE is cleared when UCAxRXBUF is read. */
      __IO uint16_t UCRXERR             	:	1;  /* Receive error flag. This bit indicates a character was received with error(s).  */
                                                /* When UCRXERR = 1, on or more error flags, UCFE, UCPE, or UCOE is also set.  */
                                                /* UCRXERR is cleared when UCAxRXBUF is read. */
      __IO uint16_t UCBRK               	:	1;  /* Break detect flag */
      __IO uint16_t UCPE                	:	1;  /* Parity error flag. When UCPEN = 0, UCPE is read as 0. */
      __IO uint16_t UCOE                	:	1;  /* Overrun error flag. This bit is set when a character is transferred into  */
                                                /* UCAxRXBUF before the previous character was read. UCOE is cleared automatically  */
                                                /* when UCxRXBUF is read, and must not be cleared by software. Otherwise, it does  */
                                                /* not function correctly. */
      __IO uint16_t UCFE                	:	1;  /* Framing error flag */
      __IO uint16_t UCLISTEN            	:	1;  /* Listen enable. The UCLISTEN bit selects loopback mode. */
      __I  uint16_t RESERVED1           	:	8;  /* Reserved */

    } bit;
  } STATW;
  union {    /* EUSCI_A0_UART_RXBUF Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A0_UART_RXBUF Control Bits */
      __I  uint16_t UCRXBUF             	:	8;  /* The receive-data buffer is user accessible and contains the last received  */
                                                /* character from the receive shift register. Reading UCAxRXBUF resets the receive- */
                                                /* error bits, the UCADDR or UCIDLE bit, and UCRXIFG. In 7-bit data mode, UCAxRXBUF */
                                                /* is LSB justified and the MSB is always reset. */
      __I  uint16_t RESERVED1           	:	8;  /* Reserved */

    } bit;
  } RXBUF;
  union {    /* EUSCI_A0_UART_TXBUF Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A0_UART_TXBUF Control Bits */
      __IO uint16_t UCTXBUF             	:	8;  /* The transmit data buffer is user accessible and holds the data waiting to be  */
                                                /* moved into the transmit shift register and transmitted on UCAxTXD. Writing to  */
                                                /* the transmit data buffer clears UCTXIFG. The MSB of UCAxTXBUF is not used for 7- */
                                                /* bit data and is reset. */
      __I  uint16_t RESERVED1           	:	8;  /* Reserved */

    } bit;
  } TXBUF;
  union {    /* EUSCI_A0_UART_ABCTL Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A0_UART_ABCTL Control Bits */
      __IO uint16_t UCABDEN             	:	1;  /* Automatic baud-rate detect enable */
      __I  uint16_t RESERVED2           	:	1;  /* Reserved */
      __IO uint16_t UCBTOE              	:	1;  /* Break time out error */
      __IO uint16_t UCSTOE              	:	1;  /* Synch field time out error */
      __IO uint16_t UCDELIM             	:	2;  /* Break/synch delimiter length */
      __I  uint16_t RESERVED1           	:	10;  /* Reserved */

    } bit;
  } ABCTL;
  union {    /* EUSCI_A0_UART_IRCTL Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A0_UART_IRCTL Control Bits */
      __IO uint16_t UCIREN              	:	1;  /* IrDA encoder/decoder enable */
      __IO uint16_t UCIRTXCLK           	:	1;  /* IrDA transmit pulse clock select */
      __IO uint16_t UCIRTXPL            	:	6;  /* Transmit pulse length. Pulse length t(PULSE) = (UCIRTXPLx + 1) / [2 - f(IRTXCLK) */
                                                /* ] */
      __IO uint16_t UCIRRXFE            	:	1;  /* IrDA receive filter enabled */
      __IO uint16_t UCIRRXPL            	:	1;  /* IrDA receive input UCAxRXD polarity */
      __IO uint16_t UCIRRXFL            	:	6;  /* Receive filter length. The minimum pulse length for receive is given by: t(MIN)  */
                                                /* = (UCIRRXFLx + 4) / [2 - f(IRTXCLK)] */

    } bit;
  } IRCTL;
  uint8_t RESERVED1[6];
  union {    /* EUSCI_A0_UART_IE Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A0_UART_IE Control Bits */
      __IO uint16_t UCRXIE              	:	1;  /* Receive interrupt enable */
      __IO uint16_t UCTXIE              	:	1;  /* Transmit interrupt enable */
      __IO uint16_t UCSTTIE             	:	1;  /* Start bit interrupt enable */
      __IO uint16_t UCTXCPTIE           	:	1;  /* Transmit complete interrupt enable */
      __I  uint16_t RESERVED1           	:	12;  /* Reserved */

    } bit;
  } IE;
  union {    /* EUSCI_A0_UART_IFG Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A0_UART_IFG Control Bits */
      __IO uint16_t UCRXIFG             	:	1;  /* Receive interrupt flag. UCRXIFG is set when UCAxRXBUF has received a complete  */
                                                /* character. */
      __IO uint16_t UCTXIFG             	:	1;  /* Transmit interrupt flag. UCTXIFG is set when UCAxTXBUF empty. */
      __IO uint16_t UCSTTIFG            	:	1;  /* Start bit interrupt flag. UCSTTIFG is set after a Start bit was received */
      __IO uint16_t UCTXCPTIFG          	:	1;  /* Transmit ready interrupt enable. UCTXRDYIFG is set when the entire byte in the  */
                                                /* internal shift register got shifted out and UCAxTXBUF is empty. */
      __I  uint16_t RESERVED1           	:	12;  /* Reserved */

    } bit;
  } IFG;
  uint16_t IV; /* eUSCI_Ax Interrupt Vector */
} EUSCI_A0_UART_Type;


/* EUSCI_A0_UART_CTLW0 Control Bits */
#define EUSCI_A0_UART_CTLW0_UCSWRST                        (0x0001)              /* Software reset enable */
#define EUSCI_A0_UART_CTLW0_UCTXBRK                        (0x0002)              /* Transmit break. Transmits a break with the next write to the transmit buffer. In */
                                                                                 /* UART mode with automatic baud-rate detection, 055h must be written into  */
                                                                                 /* UCAxTXBUF to generate the required break/synch fields. Otherwise, 0h must be  */
                                                                                 /* written into the transmit buffer. */
#define EUSCI_A0_UART_CTLW0_UCTXADDR                       (0x0004)              /* Transmit address. Next frame to be transmitted is marked as address, depending  */
                                                                                 /* on the selected multiprocessor mode. */
#define EUSCI_A0_UART_CTLW0_UCDORM                         (0x0008)              /* Dormant. Puts eUSCI_A into sleep mode. */
#define EUSCI_A0_UART_CTLW0_UCBRKIE                        (0x0010)              /* Receive break character interrupt enable */
#define EUSCI_A0_UART_CTLW0_UCRXEIE                        (0x0020)              /* Receive erroneous-character interrupt enable */
#define EUSCI_A0_UART_CTLW0_UCSSEL__M                      (0x00c0)              /* eUSCI_A clock source select. These bits select the BRCLK source clock. */
#define EUSCI_A0_UART_CTLW0_UCSSEL__0                      (0x0000)              /* UCLK */
#define EUSCI_A0_UART_CTLW0_UCSSEL__0_UCLK                 (0x0000)              /* UCLK */
#define EUSCI_A0_UART_CTLW0_UCSSEL__1                      (0x0040)              /* ACLK */
#define EUSCI_A0_UART_CTLW0_UCSSEL__1_ACLK                 (0x0040)              /* ACLK */
#define EUSCI_A0_UART_CTLW0_UCSSEL__2                      (0x0080)              /* SMCLK */
#define EUSCI_A0_UART_CTLW0_UCSSEL__2_SMCLK                (0x0080)              /* SMCLK */
#define EUSCI_A0_UART_CTLW0_UCSSEL__3                      (0x00c0)              /* SMCLK */
#define EUSCI_A0_UART_CTLW0_UCSSEL__3_SMCLK                (0x00c0)              /* SMCLK */
#define EUSCI_A0_UART_CTLW0_UCSYNC                         (0x0100)              /* Synchronous mode enable */
#define EUSCI_A0_UART_CTLW0_UCMODE__M                      (0x0600)              /* eUSCI_A mode. The UCMODEx bits select the asynchronous mode when UCSYNC = 0. */
#define EUSCI_A0_UART_CTLW0_UCMODE__0                      (0x0000)              /* UART mode */
#define EUSCI_A0_UART_CTLW0_UCMODE__0_UART_MODE            (0x0000)              /* UART mode */
#define EUSCI_A0_UART_CTLW0_UCMODE__1                      (0x0200)              /* Idle-line multiprocessor mode */
#define EUSCI_A0_UART_CTLW0_UCMODE__2                      (0x0400)              /* Address-bit multiprocessor mode */
#define EUSCI_A0_UART_CTLW0_UCMODE__3                      (0x0600)              /* UART mode with automatic baud-rate detection */
#define EUSCI_A0_UART_CTLW0_UCSPB                          (0x0800)              /* Stop bit select. Number of stop bits. */
#define EUSCI_A0_UART_CTLW0_UC7BIT                         (0x1000)              /* Character length. Selects 7-bit or 8-bit character length. */
#define EUSCI_A0_UART_CTLW0_UCMSB                          (0x2000)              /* MSB first select. Controls the direction of the receive and transmit shift  */
                                                                                 /* register. */
#define EUSCI_A0_UART_CTLW0_UCPAR                          (0x4000)              /* Parity select. UCPAR is not used when parity is disabled. */
#define EUSCI_A0_UART_CTLW0_UCPEN                          (0x8000)              /* Parity enable */

/* EUSCI_A0_UART_CTLW1 Control Bits */
#define EUSCI_A0_UART_CTLW1_UCGLIT__M                      (0x0003)              /* Deglitch time */
#define EUSCI_A0_UART_CTLW1_UCGLIT__0                      (0x0000)              /* ~2 ns */
#define EUSCI_A0_UART_CTLW1_UCGLIT__0__2_NS                (0x0000)              /* ~2 ns */
#define EUSCI_A0_UART_CTLW1_UCGLIT__1                      (0x0001)              /* ~50 ns */
#define EUSCI_A0_UART_CTLW1_UCGLIT__1__50_NS               (0x0001)              /* ~50 ns */
#define EUSCI_A0_UART_CTLW1_UCGLIT__2                      (0x0002)              /* ~100 ns */
#define EUSCI_A0_UART_CTLW1_UCGLIT__2__100_NS              (0x0002)              /* ~100 ns */
#define EUSCI_A0_UART_CTLW1_UCGLIT__3                      (0x0003)              /* ~200 ns */
#define EUSCI_A0_UART_CTLW1_UCGLIT__3__200_NS              (0x0003)              /* ~200 ns */
#define EUSCI_A0_UART_CTLW1_RESERVED1__M                   (0xfffc)              /*  */

/* EUSCI_A0_UART_MCTLW Control Bits */
#define EUSCI_A0_UART_MCTLW_UCOS16                         (0x0001)              /* Oversampling mode enabled */
#define EUSCI_A0_UART_MCTLW_RESERVED1__M                   (0x000e)              /*  */
#define EUSCI_A0_UART_MCTLW_UCBRF__M                       (0x00f0)              /* First modulation stage select. These bits determine the modulation pattern for  */
                                                                                 /* BITCLK16 when UCOS16 = 1. Ignored with UCOS16 = 0. The "Oversampling Baud-Rate  */
                                                                                 /* Generation" section shows the modulation pattern. */
#define EUSCI_A0_UART_MCTLW_UCBRS__M                       (0xff00)              /* Second modulation stage select. These bits hold a free modulation pattern for  */
                                                                                 /* BITCLK. */

/* EUSCI_A0_UART_STATW Control Bits */
#define EUSCI_A0_UART_STATW_UCBUSY                         (0x0001)              /* eUSCI_A busy. This bit indicates if a transmit or receive operation is in  */
                                                                                 /* progress. */
#define EUSCI_A0_UART_STATW_UCADDR_UCIDLE                  (0x0002)              /* UCADDR: Address received in address-bit multiprocessor mode. UCADDR is cleared  */
                                                                                 /* when UCAxRXBUF is read. UCIDLE: Idle line detected in idle-line multiprocessor  */
                                                                                 /* mode. UCIDLE is cleared when UCAxRXBUF is read. */
#define EUSCI_A0_UART_STATW_UCRXERR                        (0x0004)              /* Receive error flag. This bit indicates a character was received with error(s).  */
                                                                                 /* When UCRXERR = 1, on or more error flags, UCFE, UCPE, or UCOE is also set.  */
                                                                                 /* UCRXERR is cleared when UCAxRXBUF is read. */
#define EUSCI_A0_UART_STATW_UCBRK                          (0x0008)              /* Break detect flag */
#define EUSCI_A0_UART_STATW_UCPE                           (0x0010)              /* Parity error flag. When UCPEN = 0, UCPE is read as 0. */
#define EUSCI_A0_UART_STATW_UCOE                           (0x0020)              /* Overrun error flag. This bit is set when a character is transferred into  */
                                                                                 /* UCAxRXBUF before the previous character was read. UCOE is cleared automatically  */
                                                                                 /* when UCxRXBUF is read, and must not be cleared by software. Otherwise, it does  */
                                                                                 /* not function correctly. */
#define EUSCI_A0_UART_STATW_UCFE                           (0x0040)              /* Framing error flag */
#define EUSCI_A0_UART_STATW_UCLISTEN                       (0x0080)              /* Listen enable. The UCLISTEN bit selects loopback mode. */
#define EUSCI_A0_UART_STATW_RESERVED1__M                   (0xff00)              /*  */

/* EUSCI_A0_UART_RXBUF Control Bits */
#define EUSCI_A0_UART_RXBUF_UCRXBUF__M                     (0x00ff)              /* The receive-data buffer is user accessible and contains the last received  */
                                                                                 /* character from the receive shift register. Reading UCAxRXBUF resets the receive- */
                                                                                 /* error bits, the UCADDR or UCIDLE bit, and UCRXIFG. In 7-bit data mode, UCAxRXBUF */
                                                                                 /* is LSB justified and the MSB is always reset. */
#define EUSCI_A0_UART_RXBUF_RESERVED1__M                   (0xff00)              /*  */

/* EUSCI_A0_UART_TXBUF Control Bits */
#define EUSCI_A0_UART_TXBUF_UCTXBUF__M                     (0x00ff)              /* The transmit data buffer is user accessible and holds the data waiting to be  */
                                                                                 /* moved into the transmit shift register and transmitted on UCAxTXD. Writing to  */
                                                                                 /* the transmit data buffer clears UCTXIFG. The MSB of UCAxTXBUF is not used for 7- */
                                                                                 /* bit data and is reset. */
#define EUSCI_A0_UART_TXBUF_RESERVED1__M                   (0xff00)              /*  */

/* EUSCI_A0_UART_ABCTL Control Bits */
#define EUSCI_A0_UART_ABCTL_UCABDEN                        (0x0001)              /* Automatic baud-rate detect enable */
#define EUSCI_A0_UART_ABCTL_RESERVED2                      (0x0002)              /*  */
#define EUSCI_A0_UART_ABCTL_UCBTOE                         (0x0004)              /* Break time out error */
#define EUSCI_A0_UART_ABCTL_UCSTOE                         (0x0008)              /* Synch field time out error */
#define EUSCI_A0_UART_ABCTL_UCDELIM__M                     (0x0030)              /* Break/synch delimiter length */
#define EUSCI_A0_UART_ABCTL_UCDELIM__0                     (0x0000)              /* 1 bit time */
#define EUSCI_A0_UART_ABCTL_UCDELIM__0_1_BIT_TIME          (0x0000)              /* 1 bit time */
#define EUSCI_A0_UART_ABCTL_UCDELIM__1                     (0x0010)              /* 2 bit times */
#define EUSCI_A0_UART_ABCTL_UCDELIM__1_2_BIT_TIMES         (0x0010)              /* 2 bit times */
#define EUSCI_A0_UART_ABCTL_UCDELIM__2                     (0x0020)              /* 3 bit times */
#define EUSCI_A0_UART_ABCTL_UCDELIM__2_3_BIT_TIMES         (0x0020)              /* 3 bit times */
#define EUSCI_A0_UART_ABCTL_UCDELIM__3                     (0x0030)              /* 4 bit times */
#define EUSCI_A0_UART_ABCTL_UCDELIM__3_4_BIT_TIMES         (0x0030)              /* 4 bit times */
#define EUSCI_A0_UART_ABCTL_RESERVED1__M                   (0xffc0)              /*  */

/* EUSCI_A0_UART_IRCTL Control Bits */
#define EUSCI_A0_UART_IRCTL_UCIREN                         (0x0001)              /* IrDA encoder/decoder enable */
#define EUSCI_A0_UART_IRCTL_UCIRTXCLK                      (0x0002)              /* IrDA transmit pulse clock select */
#define EUSCI_A0_UART_IRCTL_UCIRTXPL__M                    (0x00fc)              /* Transmit pulse length. Pulse length t(PULSE) = (UCIRTXPLx + 1) / [2 - f(IRTXCLK) */
                                                                                 /* ] */
#define EUSCI_A0_UART_IRCTL_UCIRRXFE                       (0x0100)              /* IrDA receive filter enabled */
#define EUSCI_A0_UART_IRCTL_UCIRRXPL                       (0x0200)              /* IrDA receive input UCAxRXD polarity */
#define EUSCI_A0_UART_IRCTL_UCIRRXFL__M                    (0xfc00)              /* Receive filter length. The minimum pulse length for receive is given by: t(MIN)  */
                                                                                 /* = (UCIRRXFLx + 4) / [2 - f(IRTXCLK)] */

/* EUSCI_A0_UART_IE Control Bits */
#define EUSCI_A0_UART_IE_UCRXIE                            (0x0001)              /* Receive interrupt enable */
#define EUSCI_A0_UART_IE_UCTXIE                            (0x0002)              /* Transmit interrupt enable */
#define EUSCI_A0_UART_IE_UCSTTIE                           (0x0004)              /* Start bit interrupt enable */
#define EUSCI_A0_UART_IE_UCTXCPTIE                         (0x0008)              /* Transmit complete interrupt enable */
#define EUSCI_A0_UART_IE_RESERVED1__M                      (0xfff0)              /*  */

/* EUSCI_A0_UART_IFG Control Bits */
#define EUSCI_A0_UART_IFG_UCRXIFG                          (0x0001)              /* Receive interrupt flag. UCRXIFG is set when UCAxRXBUF has received a complete  */
                                                                                 /* character. */
#define EUSCI_A0_UART_IFG_UCTXIFG                          (0x0002)              /* Transmit interrupt flag. UCTXIFG is set when UCAxTXBUF empty. */
#define EUSCI_A0_UART_IFG_UCSTTIFG                         (0x0004)              /* Start bit interrupt flag. UCSTTIFG is set after a Start bit was received */
#define EUSCI_A0_UART_IFG_UCTXCPTIFG                       (0x0008)              /* Transmit ready interrupt enable. UCTXRDYIFG is set when the entire byte in the  */
                                                                                 /* internal shift register got shifted out and UCAxTXBUF is empty. */
#define EUSCI_A0_UART_IFG_RESERVED1__M                     (0xfff0)              /*  */


//*****************************************************************************
// EUSCI_A1_SPI Registers
//*****************************************************************************
typedef struct {
  union {    /* EUSCI_A1_SPI_CTLW0 Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A1_SPI_CTLW0 Control Bits */
      __IO uint16_t UCSWRST             	:	1;  /* Software reset enable */
      __IO uint16_t UCSTEM              	:	1;  /* STE mode select in master mode. This byte is ignored in slave or 3-wire mode. */
      __I  uint16_t RESERVED1           	:	4;  /* Reserved */
      __IO uint16_t UCSSEL              	:	2;  /* eUSCI clock source select. These bits select the BRCLK source clock in master  */
                                                /* mode. UCxCLK is always used in slave mode. */
      __IO uint16_t UCSYNC              	:	1;  /* Synchronous mode enable */
      __IO uint16_t UCMODE              	:	2;  /* eUSCI mode. The UCMODEx bits select the synchronous mode when UCSYNC = 1. */
      __IO uint16_t UCMST               	:	1;  /* Master mode select */
      __IO uint16_t UC7BIT              	:	1;  /* Character length. Selects 7-bit or 8-bit character length. */
      __IO uint16_t UCMSB               	:	1;  /* MSB first select. Controls the direction of the receive and transmit shift  */
                                                /* register. */
      __IO uint16_t UCCKPL              	:	1;  /* Clock polarity select */
      __IO uint16_t UCCKPH              	:	1;  /* Clock phase select */

    } bit;
  } CTLW0;
  uint8_t RESERVED0[4];
  uint16_t BRW; /* eUSCI_Ax Bit Rate Control Word */
  uint8_t RESERVED1[2];
  union {    /* EUSCI_A1_SPI_STATW Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A1_SPI_STATW Control Bits */
      __I  uint16_t UCBUSY              	:	1;  /* eUSCI busy. This bit indicates if a transmit or receive operation is in progress */
                                                /* . */
      __IO uint16_t RESERVED2           	:	4;  /* Reserved */
      __IO uint16_t UCOE                	:	1;  /* Overrun error flag. This bit is set when a character is transferred into  */
                                                /* UCxRXBUF before the previous character was read. UCOE is cleared automatically  */
                                                /* when UCxRXBUF is read, and must not be cleared by software. Otherwise, it does  */
                                                /* not function correctly. */
      __IO uint16_t UCFE                	:	1;  /* Framing error flag. This bit indicates a bus conflict in 4-wire master mode.  */
                                                /* UCFE is not used in 3-wire master or any slave mode. */
      __IO uint16_t UCLISTEN            	:	1;  /* Listen enable. The UCLISTEN bit selects loopback mode. */
      __I  uint16_t RESERVED1           	:	8;  /* Reserved */

    } bit;
  } STATW;
  union {    /* EUSCI_A1_SPI_RXBUF Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A1_SPI_RXBUF Control Bits */
      __I  uint16_t UCRXBUF             	:	8;  /* The receive-data buffer is user accessible and contains the last received  */
                                                /* character from the receive shift register. Reading UCxRXBUF resets the receive- */
                                                /* error bits and UCRXIFG. In 7-bit data mode, UCxRXBUF is LSB justified and the  */
                                                /* MSB is always reset. */
      __I  uint16_t RESERVED1           	:	8;  /* Reserved */

    } bit;
  } RXBUF;
  union {    /* EUSCI_A1_SPI_TXBUF Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A1_SPI_TXBUF Control Bits */
      __IO uint16_t UCTXBUF             	:	8;  /* The transmit data buffer is user accessible and holds the data waiting to be  */
                                                /* moved into the transmit shift register and transmitted. Writing to the transmit  */
                                                /* data buffer clears UCTXIFG. The MSB of UCxTXBUF is not used for 7-bit data and  */
                                                /* is reset. */
      __I  uint16_t RESERVED1           	:	8;  /* Reserved */

    } bit;
  } TXBUF;
  uint8_t RESERVED2[10];
  union {    /* EUSCI_A1_SPI_IE Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A1_SPI_IE Control Bits */
      __IO uint16_t UCRXIE              	:	1;  /* Receive interrupt enable */
      __IO uint16_t UCTXIE              	:	1;  /* Transmit interrupt enable */
      __I  uint16_t RESERVED1           	:	14;  /* Reserved */

    } bit;
  } IE;
  union {    /* EUSCI_A1_SPI_IFG Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A1_SPI_IFG Control Bits */
      __IO uint16_t UCRXIFG             	:	1;  /* Receive interrupt flag. UCRXIFG is set when UCxxRXBUF has received a complete  */
                                                /* character. */
      __IO uint16_t UCTXIFG             	:	1;  /* Transmit interrupt flag. UCTXIFG is set when UCxxTXBUF empty. */
      __I  uint16_t RESERVED1           	:	14;  /* Reserved */

    } bit;
  } IFG;
  uint16_t IV; /* eUSCI_Ax Interrupt Vector */
} EUSCI_A1_SPI_Type;


/* EUSCI_A1_SPI_CTLW0 Control Bits */
#define EUSCI_A1_SPI_CTLW0_UCSWRST                         (0x0001)              /* Software reset enable */
#define EUSCI_A1_SPI_CTLW0_UCSTEM                          (0x0002)              /* STE mode select in master mode. This byte is ignored in slave or 3-wire mode. */
#define EUSCI_A1_SPI_CTLW0_RESERVED1__M                    (0x003c)              /*  */
#define EUSCI_A1_SPI_CTLW0_UCSSEL__M                       (0x00c0)              /* eUSCI clock source select. These bits select the BRCLK source clock in master  */
                                                                                 /* mode. UCxCLK is always used in slave mode. */
#define EUSCI_A1_SPI_CTLW0_UCSSEL__1                       (0x0040)              /* ACLK */
#define EUSCI_A1_SPI_CTLW0_UCSSEL__1_ACLK                  (0x0040)              /* ACLK */
#define EUSCI_A1_SPI_CTLW0_UCSSEL__2                       (0x0080)              /* SMCLK */
#define EUSCI_A1_SPI_CTLW0_UCSSEL__2_SMCLK                 (0x0080)              /* SMCLK */
#define EUSCI_A1_SPI_CTLW0_UCSSEL__3                       (0x00c0)              /* SMCLK */
#define EUSCI_A1_SPI_CTLW0_UCSSEL__3_SMCLK                 (0x00c0)              /* SMCLK */
#define EUSCI_A1_SPI_CTLW0_UCSYNC                          (0x0100)              /* Synchronous mode enable */
#define EUSCI_A1_SPI_CTLW0_UCMODE__M                       (0x0600)              /* eUSCI mode. The UCMODEx bits select the synchronous mode when UCSYNC = 1. */
#define EUSCI_A1_SPI_CTLW0_UCMODE__0                       (0x0000)              /* 3-pin SPI */
#define EUSCI_A1_SPI_CTLW0_UCMODE__0_3_PIN_SPI             (0x0000)              /* 3-pin SPI */
#define EUSCI_A1_SPI_CTLW0_UCMODE__1                       (0x0200)              /* 4-pin SPI with UCxSTE active high: Slave enabled when UCxSTE = 1 */
#define EUSCI_A1_SPI_CTLW0_UCMODE__2                       (0x0400)              /* 4-pin SPI with UCxSTE active low: Slave enabled when UCxSTE = 0 */
#define EUSCI_A1_SPI_CTLW0_UCMODE__3                       (0x0600)              /* I2C mode */
#define EUSCI_A1_SPI_CTLW0_UCMODE__3_I2C_MODE              (0x0600)              /* I2C mode */
#define EUSCI_A1_SPI_CTLW0_UCMST                           (0x0800)              /* Master mode select */
#define EUSCI_A1_SPI_CTLW0_UC7BIT                          (0x1000)              /* Character length. Selects 7-bit or 8-bit character length. */
#define EUSCI_A1_SPI_CTLW0_UCMSB                           (0x2000)              /* MSB first select. Controls the direction of the receive and transmit shift  */
                                                                                 /* register. */
#define EUSCI_A1_SPI_CTLW0_UCCKPL                          (0x4000)              /* Clock polarity select */
#define EUSCI_A1_SPI_CTLW0_UCCKPH                          (0x8000)              /* Clock phase select */

/* EUSCI_A1_SPI_STATW Control Bits */
#define EUSCI_A1_SPI_STATW_UCBUSY                          (0x0001)              /* eUSCI busy. This bit indicates if a transmit or receive operation is in progress */
                                                                                 /* . */
#define EUSCI_A1_SPI_STATW_RESERVED2__M                    (0x001e)              /*  */
#define EUSCI_A1_SPI_STATW_UCOE                            (0x0020)              /* Overrun error flag. This bit is set when a character is transferred into  */
                                                                                 /* UCxRXBUF before the previous character was read. UCOE is cleared automatically  */
                                                                                 /* when UCxRXBUF is read, and must not be cleared by software. Otherwise, it does  */
                                                                                 /* not function correctly. */
#define EUSCI_A1_SPI_STATW_UCFE                            (0x0040)              /* Framing error flag. This bit indicates a bus conflict in 4-wire master mode.  */
                                                                                 /* UCFE is not used in 3-wire master or any slave mode. */
#define EUSCI_A1_SPI_STATW_UCLISTEN                        (0x0080)              /* Listen enable. The UCLISTEN bit selects loopback mode. */
#define EUSCI_A1_SPI_STATW_RESERVED1__M                    (0xff00)              /*  */

/* EUSCI_A1_SPI_RXBUF Control Bits */
#define EUSCI_A1_SPI_RXBUF_UCRXBUF__M                      (0x00ff)              /* The receive-data buffer is user accessible and contains the last received  */
                                                                                 /* character from the receive shift register. Reading UCxRXBUF resets the receive- */
                                                                                 /* error bits and UCRXIFG. In 7-bit data mode, UCxRXBUF is LSB justified and the  */
                                                                                 /* MSB is always reset. */
#define EUSCI_A1_SPI_RXBUF_RESERVED1__M                    (0xff00)              /*  */

/* EUSCI_A1_SPI_TXBUF Control Bits */
#define EUSCI_A1_SPI_TXBUF_UCTXBUF__M                      (0x00ff)              /* The transmit data buffer is user accessible and holds the data waiting to be  */
                                                                                 /* moved into the transmit shift register and transmitted. Writing to the transmit  */
                                                                                 /* data buffer clears UCTXIFG. The MSB of UCxTXBUF is not used for 7-bit data and  */
                                                                                 /* is reset. */
#define EUSCI_A1_SPI_TXBUF_RESERVED1__M                    (0xff00)              /*  */

/* EUSCI_A1_SPI_IE Control Bits */
#define EUSCI_A1_SPI_IE_UCRXIE                             (0x0001)              /* Receive interrupt enable */
#define EUSCI_A1_SPI_IE_UCTXIE                             (0x0002)              /* Transmit interrupt enable */
#define EUSCI_A1_SPI_IE_RESERVED1__M                       (0xfffc)              /*  */

/* EUSCI_A1_SPI_IFG Control Bits */
#define EUSCI_A1_SPI_IFG_UCRXIFG                           (0x0001)              /* Receive interrupt flag. UCRXIFG is set when UCxxRXBUF has received a complete  */
                                                                                 /* character. */
#define EUSCI_A1_SPI_IFG_UCTXIFG                           (0x0002)              /* Transmit interrupt flag. UCTXIFG is set when UCxxTXBUF empty. */
#define EUSCI_A1_SPI_IFG_RESERVED1__M                      (0xfffc)              /*  */


//*****************************************************************************
// EUSCI_A1_UART Registers
//*****************************************************************************
typedef struct {
  union {    /* EUSCI_A1_UART_CTLW0 Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A1_UART_CTLW0 Control Bits */
      __IO uint16_t UCSWRST             	:	1;  /* Software reset enable */
      __IO uint16_t UCTXBRK             	:	1;  /* Transmit break. Transmits a break with the next write to the transmit buffer. In */
                                                /* UART mode with automatic baud-rate detection, 055h must be written into  */
                                                /* UCAxTXBUF to generate the required break/synch fields. Otherwise, 0h must be  */
                                                /* written into the transmit buffer. */
      __IO uint16_t UCTXADDR            	:	1;  /* Transmit address. Next frame to be transmitted is marked as address, depending  */
                                                /* on the selected multiprocessor mode. */
      __IO uint16_t UCDORM              	:	1;  /* Dormant. Puts eUSCI_A into sleep mode. */
      __IO uint16_t UCBRKIE             	:	1;  /* Receive break character interrupt enable */
      __IO uint16_t UCRXEIE             	:	1;  /* Receive erroneous-character interrupt enable */
      __IO uint16_t UCSSEL              	:	2;  /* eUSCI_A clock source select. These bits select the BRCLK source clock. */
      __IO uint16_t UCSYNC              	:	1;  /* Synchronous mode enable */
      __IO uint16_t UCMODE              	:	2;  /* eUSCI_A mode. The UCMODEx bits select the asynchronous mode when UCSYNC = 0. */
      __IO uint16_t UCSPB               	:	1;  /* Stop bit select. Number of stop bits. */
      __IO uint16_t UC7BIT              	:	1;  /* Character length. Selects 7-bit or 8-bit character length. */
      __IO uint16_t UCMSB               	:	1;  /* MSB first select. Controls the direction of the receive and transmit shift  */
                                                /* register. */
      __IO uint16_t UCPAR               	:	1;  /* Parity select. UCPAR is not used when parity is disabled. */
      __IO uint16_t UCPEN               	:	1;  /* Parity enable */

    } bit;
  } CTLW0;
  union {    /* EUSCI_A1_UART_CTLW1 Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A1_UART_CTLW1 Control Bits */
      __IO uint16_t UCGLIT              	:	2;  /* Deglitch time */
      __I  uint16_t RESERVED1           	:	14;  /* Reserved */

    } bit;
  } CTLW1;
  uint8_t RESERVED0[2];
  uint16_t BRW; /* eUSCI_Ax Baud Rate Control Word */
  union {    /* EUSCI_A1_UART_MCTLW Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A1_UART_MCTLW Control Bits */
      __IO uint16_t UCOS16              	:	1;  /* Oversampling mode enabled */
      __I  uint16_t RESERVED1           	:	3;  /* Reserved */
      __IO uint16_t UCBRF               	:	4;  /* First modulation stage select. These bits determine the modulation pattern for  */
                                                /* BITCLK16 when UCOS16 = 1. Ignored with UCOS16 = 0. The "Oversampling Baud-Rate  */
                                                /* Generation" section shows the modulation pattern. */
      __IO uint16_t UCBRS               	:	8;  /* Second modulation stage select. These bits hold a free modulation pattern for  */
                                                /* BITCLK. */

    } bit;
  } MCTLW;
  union {    /* EUSCI_A1_UART_STATW Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A1_UART_STATW Control Bits */
      __I  uint16_t UCBUSY              	:	1;  /* eUSCI_A busy. This bit indicates if a transmit or receive operation is in  */
                                                /* progress. */
      __IO uint16_t UCADDR_UCIDLE       	:	1;  /* UCADDR: Address received in address-bit multiprocessor mode. UCADDR is cleared  */
                                                /* when UCAxRXBUF is read. UCIDLE: Idle line detected in idle-line multiprocessor  */
                                                /* mode. UCIDLE is cleared when UCAxRXBUF is read. */
      __IO uint16_t UCRXERR             	:	1;  /* Receive error flag. This bit indicates a character was received with error(s).  */
                                                /* When UCRXERR = 1, on or more error flags, UCFE, UCPE, or UCOE is also set.  */
                                                /* UCRXERR is cleared when UCAxRXBUF is read. */
      __IO uint16_t UCBRK               	:	1;  /* Break detect flag */
      __IO uint16_t UCPE                	:	1;  /* Parity error flag. When UCPEN = 0, UCPE is read as 0. */
      __IO uint16_t UCOE                	:	1;  /* Overrun error flag. This bit is set when a character is transferred into  */
                                                /* UCAxRXBUF before the previous character was read. UCOE is cleared automatically  */
                                                /* when UCxRXBUF is read, and must not be cleared by software. Otherwise, it does  */
                                                /* not function correctly. */
      __IO uint16_t UCFE                	:	1;  /* Framing error flag */
      __IO uint16_t UCLISTEN            	:	1;  /* Listen enable. The UCLISTEN bit selects loopback mode. */
      __I  uint16_t RESERVED1           	:	8;  /* Reserved */

    } bit;
  } STATW;
  union {    /* EUSCI_A1_UART_RXBUF Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A1_UART_RXBUF Control Bits */
      __I  uint16_t UCRXBUF             	:	8;  /* The receive-data buffer is user accessible and contains the last received  */
                                                /* character from the receive shift register. Reading UCAxRXBUF resets the receive- */
                                                /* error bits, the UCADDR or UCIDLE bit, and UCRXIFG. In 7-bit data mode, UCAxRXBUF */
                                                /* is LSB justified and the MSB is always reset. */
      __I  uint16_t RESERVED1           	:	8;  /* Reserved */

    } bit;
  } RXBUF;
  union {    /* EUSCI_A1_UART_TXBUF Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A1_UART_TXBUF Control Bits */
      __IO uint16_t UCTXBUF             	:	8;  /* The transmit data buffer is user accessible and holds the data waiting to be  */
                                                /* moved into the transmit shift register and transmitted on UCAxTXD. Writing to  */
                                                /* the transmit data buffer clears UCTXIFG. The MSB of UCAxTXBUF is not used for 7- */
                                                /* bit data and is reset. */
      __I  uint16_t RESERVED1           	:	8;  /* Reserved */

    } bit;
  } TXBUF;
  union {    /* EUSCI_A1_UART_ABCTL Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A1_UART_ABCTL Control Bits */
      __IO uint16_t UCABDEN             	:	1;  /* Automatic baud-rate detect enable */
      __I  uint16_t RESERVED2           	:	1;  /* Reserved */
      __IO uint16_t UCBTOE              	:	1;  /* Break time out error */
      __IO uint16_t UCSTOE              	:	1;  /* Synch field time out error */
      __IO uint16_t UCDELIM             	:	2;  /* Break/synch delimiter length */
      __I  uint16_t RESERVED1           	:	10;  /* Reserved */

    } bit;
  } ABCTL;
  union {    /* EUSCI_A1_UART_IRCTL Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A1_UART_IRCTL Control Bits */
      __IO uint16_t UCIREN              	:	1;  /* IrDA encoder/decoder enable */
      __IO uint16_t UCIRTXCLK           	:	1;  /* IrDA transmit pulse clock select */
      __IO uint16_t UCIRTXPL            	:	6;  /* Transmit pulse length. Pulse length t(PULSE) = (UCIRTXPLx + 1) / [2 - f(IRTXCLK) */
                                                /* ] */
      __IO uint16_t UCIRRXFE            	:	1;  /* IrDA receive filter enabled */
      __IO uint16_t UCIRRXPL            	:	1;  /* IrDA receive input UCAxRXD polarity */
      __IO uint16_t UCIRRXFL            	:	6;  /* Receive filter length. The minimum pulse length for receive is given by: t(MIN)  */
                                                /* = (UCIRRXFLx + 4) / [2 - f(IRTXCLK)] */

    } bit;
  } IRCTL;
  uint8_t RESERVED1[6];
  union {    /* EUSCI_A1_UART_IE Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A1_UART_IE Control Bits */
      __IO uint16_t UCRXIE              	:	1;  /* Receive interrupt enable */
      __IO uint16_t UCTXIE              	:	1;  /* Transmit interrupt enable */
      __IO uint16_t UCSTTIE             	:	1;  /* Start bit interrupt enable */
      __IO uint16_t UCTXCPTIE           	:	1;  /* Transmit complete interrupt enable */
      __I  uint16_t RESERVED1           	:	12;  /* Reserved */

    } bit;
  } IE;
  union {    /* EUSCI_A1_UART_IFG Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A1_UART_IFG Control Bits */
      __IO uint16_t UCRXIFG             	:	1;  /* Receive interrupt flag. UCRXIFG is set when UCAxRXBUF has received a complete  */
                                                /* character. */
      __IO uint16_t UCTXIFG             	:	1;  /* Transmit interrupt flag. UCTXIFG is set when UCAxTXBUF empty. */
      __IO uint16_t UCSTTIFG            	:	1;  /* Start bit interrupt flag. UCSTTIFG is set after a Start bit was received */
      __IO uint16_t UCTXCPTIFG          	:	1;  /* Transmit ready interrupt enable. UCTXRDYIFG is set when the entire byte in the  */
                                                /* internal shift register got shifted out and UCAxTXBUF is empty. */
      __I  uint16_t RESERVED1           	:	12;  /* Reserved */

    } bit;
  } IFG;
  uint16_t IV; /* eUSCI_Ax Interrupt Vector */
} EUSCI_A1_UART_Type;


/* EUSCI_A1_UART_CTLW0 Control Bits */
#define EUSCI_A1_UART_CTLW0_UCSWRST                        (0x0001)              /* Software reset enable */
#define EUSCI_A1_UART_CTLW0_UCTXBRK                        (0x0002)              /* Transmit break. Transmits a break with the next write to the transmit buffer. In */
                                                                                 /* UART mode with automatic baud-rate detection, 055h must be written into  */
                                                                                 /* UCAxTXBUF to generate the required break/synch fields. Otherwise, 0h must be  */
                                                                                 /* written into the transmit buffer. */
#define EUSCI_A1_UART_CTLW0_UCTXADDR                       (0x0004)              /* Transmit address. Next frame to be transmitted is marked as address, depending  */
                                                                                 /* on the selected multiprocessor mode. */
#define EUSCI_A1_UART_CTLW0_UCDORM                         (0x0008)              /* Dormant. Puts eUSCI_A into sleep mode. */
#define EUSCI_A1_UART_CTLW0_UCBRKIE                        (0x0010)              /* Receive break character interrupt enable */
#define EUSCI_A1_UART_CTLW0_UCRXEIE                        (0x0020)              /* Receive erroneous-character interrupt enable */
#define EUSCI_A1_UART_CTLW0_UCSSEL__M                      (0x00c0)              /* eUSCI_A clock source select. These bits select the BRCLK source clock. */
#define EUSCI_A1_UART_CTLW0_UCSSEL__0                      (0x0000)              /* UCLK */
#define EUSCI_A1_UART_CTLW0_UCSSEL__0_UCLK                 (0x0000)              /* UCLK */
#define EUSCI_A1_UART_CTLW0_UCSSEL__1                      (0x0040)              /* ACLK */
#define EUSCI_A1_UART_CTLW0_UCSSEL__1_ACLK                 (0x0040)              /* ACLK */
#define EUSCI_A1_UART_CTLW0_UCSSEL__2                      (0x0080)              /* SMCLK */
#define EUSCI_A1_UART_CTLW0_UCSSEL__2_SMCLK                (0x0080)              /* SMCLK */
#define EUSCI_A1_UART_CTLW0_UCSSEL__3                      (0x00c0)              /* SMCLK */
#define EUSCI_A1_UART_CTLW0_UCSSEL__3_SMCLK                (0x00c0)              /* SMCLK */
#define EUSCI_A1_UART_CTLW0_UCSYNC                         (0x0100)              /* Synchronous mode enable */
#define EUSCI_A1_UART_CTLW0_UCMODE__M                      (0x0600)              /* eUSCI_A mode. The UCMODEx bits select the asynchronous mode when UCSYNC = 0. */
#define EUSCI_A1_UART_CTLW0_UCMODE__0                      (0x0000)              /* UART mode */
#define EUSCI_A1_UART_CTLW0_UCMODE__0_UART_MODE            (0x0000)              /* UART mode */
#define EUSCI_A1_UART_CTLW0_UCMODE__1                      (0x0200)              /* Idle-line multiprocessor mode */
#define EUSCI_A1_UART_CTLW0_UCMODE__2                      (0x0400)              /* Address-bit multiprocessor mode */
#define EUSCI_A1_UART_CTLW0_UCMODE__3                      (0x0600)              /* UART mode with automatic baud-rate detection */
#define EUSCI_A1_UART_CTLW0_UCSPB                          (0x0800)              /* Stop bit select. Number of stop bits. */
#define EUSCI_A1_UART_CTLW0_UC7BIT                         (0x1000)              /* Character length. Selects 7-bit or 8-bit character length. */
#define EUSCI_A1_UART_CTLW0_UCMSB                          (0x2000)              /* MSB first select. Controls the direction of the receive and transmit shift  */
                                                                                 /* register. */
#define EUSCI_A1_UART_CTLW0_UCPAR                          (0x4000)              /* Parity select. UCPAR is not used when parity is disabled. */
#define EUSCI_A1_UART_CTLW0_UCPEN                          (0x8000)              /* Parity enable */

/* EUSCI_A1_UART_CTLW1 Control Bits */
#define EUSCI_A1_UART_CTLW1_UCGLIT__M                      (0x0003)              /* Deglitch time */
#define EUSCI_A1_UART_CTLW1_UCGLIT__0                      (0x0000)              /* ~2 ns */
#define EUSCI_A1_UART_CTLW1_UCGLIT__0__2_NS                (0x0000)              /* ~2 ns */
#define EUSCI_A1_UART_CTLW1_UCGLIT__1                      (0x0001)              /* ~50 ns */
#define EUSCI_A1_UART_CTLW1_UCGLIT__1__50_NS               (0x0001)              /* ~50 ns */
#define EUSCI_A1_UART_CTLW1_UCGLIT__2                      (0x0002)              /* ~100 ns */
#define EUSCI_A1_UART_CTLW1_UCGLIT__2__100_NS              (0x0002)              /* ~100 ns */
#define EUSCI_A1_UART_CTLW1_UCGLIT__3                      (0x0003)              /* ~200 ns */
#define EUSCI_A1_UART_CTLW1_UCGLIT__3__200_NS              (0x0003)              /* ~200 ns */
#define EUSCI_A1_UART_CTLW1_RESERVED1__M                   (0xfffc)              /*  */

/* EUSCI_A1_UART_MCTLW Control Bits */
#define EUSCI_A1_UART_MCTLW_UCOS16                         (0x0001)              /* Oversampling mode enabled */
#define EUSCI_A1_UART_MCTLW_RESERVED1__M                   (0x000e)              /*  */
#define EUSCI_A1_UART_MCTLW_UCBRF__M                       (0x00f0)              /* First modulation stage select. These bits determine the modulation pattern for  */
                                                                                 /* BITCLK16 when UCOS16 = 1. Ignored with UCOS16 = 0. The "Oversampling Baud-Rate  */
                                                                                 /* Generation" section shows the modulation pattern. */
#define EUSCI_A1_UART_MCTLW_UCBRS__M                       (0xff00)              /* Second modulation stage select. These bits hold a free modulation pattern for  */
                                                                                 /* BITCLK. */

/* EUSCI_A1_UART_STATW Control Bits */
#define EUSCI_A1_UART_STATW_UCBUSY                         (0x0001)              /* eUSCI_A busy. This bit indicates if a transmit or receive operation is in  */
                                                                                 /* progress. */
#define EUSCI_A1_UART_STATW_UCADDR_UCIDLE                  (0x0002)              /* UCADDR: Address received in address-bit multiprocessor mode. UCADDR is cleared  */
                                                                                 /* when UCAxRXBUF is read. UCIDLE: Idle line detected in idle-line multiprocessor  */
                                                                                 /* mode. UCIDLE is cleared when UCAxRXBUF is read. */
#define EUSCI_A1_UART_STATW_UCRXERR                        (0x0004)              /* Receive error flag. This bit indicates a character was received with error(s).  */
                                                                                 /* When UCRXERR = 1, on or more error flags, UCFE, UCPE, or UCOE is also set.  */
                                                                                 /* UCRXERR is cleared when UCAxRXBUF is read. */
#define EUSCI_A1_UART_STATW_UCBRK                          (0x0008)              /* Break detect flag */
#define EUSCI_A1_UART_STATW_UCPE                           (0x0010)              /* Parity error flag. When UCPEN = 0, UCPE is read as 0. */
#define EUSCI_A1_UART_STATW_UCOE                           (0x0020)              /* Overrun error flag. This bit is set when a character is transferred into  */
                                                                                 /* UCAxRXBUF before the previous character was read. UCOE is cleared automatically  */
                                                                                 /* when UCxRXBUF is read, and must not be cleared by software. Otherwise, it does  */
                                                                                 /* not function correctly. */
#define EUSCI_A1_UART_STATW_UCFE                           (0x0040)              /* Framing error flag */
#define EUSCI_A1_UART_STATW_UCLISTEN                       (0x0080)              /* Listen enable. The UCLISTEN bit selects loopback mode. */
#define EUSCI_A1_UART_STATW_RESERVED1__M                   (0xff00)              /*  */

/* EUSCI_A1_UART_RXBUF Control Bits */
#define EUSCI_A1_UART_RXBUF_UCRXBUF__M                     (0x00ff)              /* The receive-data buffer is user accessible and contains the last received  */
                                                                                 /* character from the receive shift register. Reading UCAxRXBUF resets the receive- */
                                                                                 /* error bits, the UCADDR or UCIDLE bit, and UCRXIFG. In 7-bit data mode, UCAxRXBUF */
                                                                                 /* is LSB justified and the MSB is always reset. */
#define EUSCI_A1_UART_RXBUF_RESERVED1__M                   (0xff00)              /*  */

/* EUSCI_A1_UART_TXBUF Control Bits */
#define EUSCI_A1_UART_TXBUF_UCTXBUF__M                     (0x00ff)              /* The transmit data buffer is user accessible and holds the data waiting to be  */
                                                                                 /* moved into the transmit shift register and transmitted on UCAxTXD. Writing to  */
                                                                                 /* the transmit data buffer clears UCTXIFG. The MSB of UCAxTXBUF is not used for 7- */
                                                                                 /* bit data and is reset. */
#define EUSCI_A1_UART_TXBUF_RESERVED1__M                   (0xff00)              /*  */

/* EUSCI_A1_UART_ABCTL Control Bits */
#define EUSCI_A1_UART_ABCTL_UCABDEN                        (0x0001)              /* Automatic baud-rate detect enable */
#define EUSCI_A1_UART_ABCTL_RESERVED2                      (0x0002)              /*  */
#define EUSCI_A1_UART_ABCTL_UCBTOE                         (0x0004)              /* Break time out error */
#define EUSCI_A1_UART_ABCTL_UCSTOE                         (0x0008)              /* Synch field time out error */
#define EUSCI_A1_UART_ABCTL_UCDELIM__M                     (0x0030)              /* Break/synch delimiter length */
#define EUSCI_A1_UART_ABCTL_UCDELIM__0                     (0x0000)              /* 1 bit time */
#define EUSCI_A1_UART_ABCTL_UCDELIM__0_1_BIT_TIME          (0x0000)              /* 1 bit time */
#define EUSCI_A1_UART_ABCTL_UCDELIM__1                     (0x0010)              /* 2 bit times */
#define EUSCI_A1_UART_ABCTL_UCDELIM__1_2_BIT_TIMES         (0x0010)              /* 2 bit times */
#define EUSCI_A1_UART_ABCTL_UCDELIM__2                     (0x0020)              /* 3 bit times */
#define EUSCI_A1_UART_ABCTL_UCDELIM__2_3_BIT_TIMES         (0x0020)              /* 3 bit times */
#define EUSCI_A1_UART_ABCTL_UCDELIM__3                     (0x0030)              /* 4 bit times */
#define EUSCI_A1_UART_ABCTL_UCDELIM__3_4_BIT_TIMES         (0x0030)              /* 4 bit times */
#define EUSCI_A1_UART_ABCTL_RESERVED1__M                   (0xffc0)              /*  */

/* EUSCI_A1_UART_IRCTL Control Bits */
#define EUSCI_A1_UART_IRCTL_UCIREN                         (0x0001)              /* IrDA encoder/decoder enable */
#define EUSCI_A1_UART_IRCTL_UCIRTXCLK                      (0x0002)              /* IrDA transmit pulse clock select */
#define EUSCI_A1_UART_IRCTL_UCIRTXPL__M                    (0x00fc)              /* Transmit pulse length. Pulse length t(PULSE) = (UCIRTXPLx + 1) / [2 - f(IRTXCLK) */
                                                                                 /* ] */
#define EUSCI_A1_UART_IRCTL_UCIRRXFE                       (0x0100)              /* IrDA receive filter enabled */
#define EUSCI_A1_UART_IRCTL_UCIRRXPL                       (0x0200)              /* IrDA receive input UCAxRXD polarity */
#define EUSCI_A1_UART_IRCTL_UCIRRXFL__M                    (0xfc00)              /* Receive filter length. The minimum pulse length for receive is given by: t(MIN)  */
                                                                                 /* = (UCIRRXFLx + 4) / [2 - f(IRTXCLK)] */

/* EUSCI_A1_UART_IE Control Bits */
#define EUSCI_A1_UART_IE_UCRXIE                            (0x0001)              /* Receive interrupt enable */
#define EUSCI_A1_UART_IE_UCTXIE                            (0x0002)              /* Transmit interrupt enable */
#define EUSCI_A1_UART_IE_UCSTTIE                           (0x0004)              /* Start bit interrupt enable */
#define EUSCI_A1_UART_IE_UCTXCPTIE                         (0x0008)              /* Transmit complete interrupt enable */
#define EUSCI_A1_UART_IE_RESERVED1__M                      (0xfff0)              /*  */

/* EUSCI_A1_UART_IFG Control Bits */
#define EUSCI_A1_UART_IFG_UCRXIFG                          (0x0001)              /* Receive interrupt flag. UCRXIFG is set when UCAxRXBUF has received a complete  */
                                                                                 /* character. */
#define EUSCI_A1_UART_IFG_UCTXIFG                          (0x0002)              /* Transmit interrupt flag. UCTXIFG is set when UCAxTXBUF empty. */
#define EUSCI_A1_UART_IFG_UCSTTIFG                         (0x0004)              /* Start bit interrupt flag. UCSTTIFG is set after a Start bit was received */
#define EUSCI_A1_UART_IFG_UCTXCPTIFG                       (0x0008)              /* Transmit ready interrupt enable. UCTXRDYIFG is set when the entire byte in the  */
                                                                                 /* internal shift register got shifted out and UCAxTXBUF is empty. */
#define EUSCI_A1_UART_IFG_RESERVED1__M                     (0xfff0)              /*  */


//*****************************************************************************
// EUSCI_A2_SPI Registers
//*****************************************************************************
typedef struct {
  union {    /* EUSCI_A2_SPI_CTLW0 Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A2_SPI_CTLW0 Control Bits */
      __IO uint16_t UCSWRST             	:	1;  /* Software reset enable */
      __IO uint16_t UCSTEM              	:	1;  /* STE mode select in master mode. This byte is ignored in slave or 3-wire mode. */
      __I  uint16_t RESERVED1           	:	4;  /* Reserved */
      __IO uint16_t UCSSEL              	:	2;  /* eUSCI clock source select. These bits select the BRCLK source clock in master  */
                                                /* mode. UCxCLK is always used in slave mode. */
      __IO uint16_t UCSYNC              	:	1;  /* Synchronous mode enable */
      __IO uint16_t UCMODE              	:	2;  /* eUSCI mode. The UCMODEx bits select the synchronous mode when UCSYNC = 1. */
      __IO uint16_t UCMST               	:	1;  /* Master mode select */
      __IO uint16_t UC7BIT              	:	1;  /* Character length. Selects 7-bit or 8-bit character length. */
      __IO uint16_t UCMSB               	:	1;  /* MSB first select. Controls the direction of the receive and transmit shift  */
                                                /* register. */
      __IO uint16_t UCCKPL              	:	1;  /* Clock polarity select */
      __IO uint16_t UCCKPH              	:	1;  /* Clock phase select */

    } bit;
  } CTLW0;
  uint8_t RESERVED0[4];
  uint16_t BRW; /* eUSCI_Ax Bit Rate Control Word */
  uint8_t RESERVED1[2];
  union {    /* EUSCI_A2_SPI_STATW Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A2_SPI_STATW Control Bits */
      __I  uint16_t UCBUSY              	:	1;  /* eUSCI busy. This bit indicates if a transmit or receive operation is in progress */
                                                /* . */
      __IO uint16_t RESERVED2           	:	4;  /* Reserved */
      __IO uint16_t UCOE                	:	1;  /* Overrun error flag. This bit is set when a character is transferred into  */
                                                /* UCxRXBUF before the previous character was read. UCOE is cleared automatically  */
                                                /* when UCxRXBUF is read, and must not be cleared by software. Otherwise, it does  */
                                                /* not function correctly. */
      __IO uint16_t UCFE                	:	1;  /* Framing error flag. This bit indicates a bus conflict in 4-wire master mode.  */
                                                /* UCFE is not used in 3-wire master or any slave mode. */
      __IO uint16_t UCLISTEN            	:	1;  /* Listen enable. The UCLISTEN bit selects loopback mode. */
      __I  uint16_t RESERVED1           	:	8;  /* Reserved */

    } bit;
  } STATW;
  union {    /* EUSCI_A2_SPI_RXBUF Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A2_SPI_RXBUF Control Bits */
      __I  uint16_t UCRXBUF             	:	8;  /* The receive-data buffer is user accessible and contains the last received  */
                                                /* character from the receive shift register. Reading UCxRXBUF resets the receive- */
                                                /* error bits and UCRXIFG. In 7-bit data mode, UCxRXBUF is LSB justified and the  */
                                                /* MSB is always reset. */
      __I  uint16_t RESERVED1           	:	8;  /* Reserved */

    } bit;
  } RXBUF;
  union {    /* EUSCI_A2_SPI_TXBUF Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A2_SPI_TXBUF Control Bits */
      __IO uint16_t UCTXBUF             	:	8;  /* The transmit data buffer is user accessible and holds the data waiting to be  */
                                                /* moved into the transmit shift register and transmitted. Writing to the transmit  */
                                                /* data buffer clears UCTXIFG. The MSB of UCxTXBUF is not used for 7-bit data and  */
                                                /* is reset. */
      __I  uint16_t RESERVED1           	:	8;  /* Reserved */

    } bit;
  } TXBUF;
  uint8_t RESERVED2[10];
  union {    /* EUSCI_A2_SPI_IE Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A2_SPI_IE Control Bits */
      __IO uint16_t UCRXIE              	:	1;  /* Receive interrupt enable */
      __IO uint16_t UCTXIE              	:	1;  /* Transmit interrupt enable */
      __I  uint16_t RESERVED1           	:	14;  /* Reserved */

    } bit;
  } IE;
  union {    /* EUSCI_A2_SPI_IFG Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A2_SPI_IFG Control Bits */
      __IO uint16_t UCRXIFG             	:	1;  /* Receive interrupt flag. UCRXIFG is set when UCxxRXBUF has received a complete  */
                                                /* character. */
      __IO uint16_t UCTXIFG             	:	1;  /* Transmit interrupt flag. UCTXIFG is set when UCxxTXBUF empty. */
      __I  uint16_t RESERVED1           	:	14;  /* Reserved */

    } bit;
  } IFG;
  uint16_t IV; /* eUSCI_Ax Interrupt Vector */
} EUSCI_A2_SPI_Type;


/* EUSCI_A2_SPI_CTLW0 Control Bits */
#define EUSCI_A2_SPI_CTLW0_UCSWRST                         (0x0001)              /* Software reset enable */
#define EUSCI_A2_SPI_CTLW0_UCSTEM                          (0x0002)              /* STE mode select in master mode. This byte is ignored in slave or 3-wire mode. */
#define EUSCI_A2_SPI_CTLW0_RESERVED1__M                    (0x003c)              /*  */
#define EUSCI_A2_SPI_CTLW0_UCSSEL__M                       (0x00c0)              /* eUSCI clock source select. These bits select the BRCLK source clock in master  */
                                                                                 /* mode. UCxCLK is always used in slave mode. */
#define EUSCI_A2_SPI_CTLW0_UCSSEL__1                       (0x0040)              /* ACLK */
#define EUSCI_A2_SPI_CTLW0_UCSSEL__1_ACLK                  (0x0040)              /* ACLK */
#define EUSCI_A2_SPI_CTLW0_UCSSEL__2                       (0x0080)              /* SMCLK */
#define EUSCI_A2_SPI_CTLW0_UCSSEL__2_SMCLK                 (0x0080)              /* SMCLK */
#define EUSCI_A2_SPI_CTLW0_UCSSEL__3                       (0x00c0)              /* SMCLK */
#define EUSCI_A2_SPI_CTLW0_UCSSEL__3_SMCLK                 (0x00c0)              /* SMCLK */
#define EUSCI_A2_SPI_CTLW0_UCSYNC                          (0x0100)              /* Synchronous mode enable */
#define EUSCI_A2_SPI_CTLW0_UCMODE__M                       (0x0600)              /* eUSCI mode. The UCMODEx bits select the synchronous mode when UCSYNC = 1. */
#define EUSCI_A2_SPI_CTLW0_UCMODE__0                       (0x0000)              /* 3-pin SPI */
#define EUSCI_A2_SPI_CTLW0_UCMODE__0_3_PIN_SPI             (0x0000)              /* 3-pin SPI */
#define EUSCI_A2_SPI_CTLW0_UCMODE__1                       (0x0200)              /* 4-pin SPI with UCxSTE active high: Slave enabled when UCxSTE = 1 */
#define EUSCI_A2_SPI_CTLW0_UCMODE__2                       (0x0400)              /* 4-pin SPI with UCxSTE active low: Slave enabled when UCxSTE = 0 */
#define EUSCI_A2_SPI_CTLW0_UCMODE__3                       (0x0600)              /* I2C mode */
#define EUSCI_A2_SPI_CTLW0_UCMODE__3_I2C_MODE              (0x0600)              /* I2C mode */
#define EUSCI_A2_SPI_CTLW0_UCMST                           (0x0800)              /* Master mode select */
#define EUSCI_A2_SPI_CTLW0_UC7BIT                          (0x1000)              /* Character length. Selects 7-bit or 8-bit character length. */
#define EUSCI_A2_SPI_CTLW0_UCMSB                           (0x2000)              /* MSB first select. Controls the direction of the receive and transmit shift  */
                                                                                 /* register. */
#define EUSCI_A2_SPI_CTLW0_UCCKPL                          (0x4000)              /* Clock polarity select */
#define EUSCI_A2_SPI_CTLW0_UCCKPH                          (0x8000)              /* Clock phase select */

/* EUSCI_A2_SPI_STATW Control Bits */
#define EUSCI_A2_SPI_STATW_UCBUSY                          (0x0001)              /* eUSCI busy. This bit indicates if a transmit or receive operation is in progress */
                                                                                 /* . */
#define EUSCI_A2_SPI_STATW_RESERVED2__M                    (0x001e)              /*  */
#define EUSCI_A2_SPI_STATW_UCOE                            (0x0020)              /* Overrun error flag. This bit is set when a character is transferred into  */
                                                                                 /* UCxRXBUF before the previous character was read. UCOE is cleared automatically  */
                                                                                 /* when UCxRXBUF is read, and must not be cleared by software. Otherwise, it does  */
                                                                                 /* not function correctly. */
#define EUSCI_A2_SPI_STATW_UCFE                            (0x0040)              /* Framing error flag. This bit indicates a bus conflict in 4-wire master mode.  */
                                                                                 /* UCFE is not used in 3-wire master or any slave mode. */
#define EUSCI_A2_SPI_STATW_UCLISTEN                        (0x0080)              /* Listen enable. The UCLISTEN bit selects loopback mode. */
#define EUSCI_A2_SPI_STATW_RESERVED1__M                    (0xff00)              /*  */

/* EUSCI_A2_SPI_RXBUF Control Bits */
#define EUSCI_A2_SPI_RXBUF_UCRXBUF__M                      (0x00ff)              /* The receive-data buffer is user accessible and contains the last received  */
                                                                                 /* character from the receive shift register. Reading UCxRXBUF resets the receive- */
                                                                                 /* error bits and UCRXIFG. In 7-bit data mode, UCxRXBUF is LSB justified and the  */
                                                                                 /* MSB is always reset. */
#define EUSCI_A2_SPI_RXBUF_RESERVED1__M                    (0xff00)              /*  */

/* EUSCI_A2_SPI_TXBUF Control Bits */
#define EUSCI_A2_SPI_TXBUF_UCTXBUF__M                      (0x00ff)              /* The transmit data buffer is user accessible and holds the data waiting to be  */
                                                                                 /* moved into the transmit shift register and transmitted. Writing to the transmit  */
                                                                                 /* data buffer clears UCTXIFG. The MSB of UCxTXBUF is not used for 7-bit data and  */
                                                                                 /* is reset. */
#define EUSCI_A2_SPI_TXBUF_RESERVED1__M                    (0xff00)              /*  */

/* EUSCI_A2_SPI_IE Control Bits */
#define EUSCI_A2_SPI_IE_UCRXIE                             (0x0001)              /* Receive interrupt enable */
#define EUSCI_A2_SPI_IE_UCTXIE                             (0x0002)              /* Transmit interrupt enable */
#define EUSCI_A2_SPI_IE_RESERVED1__M                       (0xfffc)              /*  */

/* EUSCI_A2_SPI_IFG Control Bits */
#define EUSCI_A2_SPI_IFG_UCRXIFG                           (0x0001)              /* Receive interrupt flag. UCRXIFG is set when UCxxRXBUF has received a complete  */
                                                                                 /* character. */
#define EUSCI_A2_SPI_IFG_UCTXIFG                           (0x0002)              /* Transmit interrupt flag. UCTXIFG is set when UCxxTXBUF empty. */
#define EUSCI_A2_SPI_IFG_RESERVED1__M                      (0xfffc)              /*  */


//*****************************************************************************
// EUSCI_A2_UART Registers
//*****************************************************************************
typedef struct {
  union {    /* EUSCI_A2_UART_CTLW0 Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A2_UART_CTLW0 Control Bits */
      __IO uint16_t UCSWRST             	:	1;  /* Software reset enable */
      __IO uint16_t UCTXBRK             	:	1;  /* Transmit break. Transmits a break with the next write to the transmit buffer. In */
                                                /* UART mode with automatic baud-rate detection, 055h must be written into  */
                                                /* UCAxTXBUF to generate the required break/synch fields. Otherwise, 0h must be  */
                                                /* written into the transmit buffer. */
      __IO uint16_t UCTXADDR            	:	1;  /* Transmit address. Next frame to be transmitted is marked as address, depending  */
                                                /* on the selected multiprocessor mode. */
      __IO uint16_t UCDORM              	:	1;  /* Dormant. Puts eUSCI_A into sleep mode. */
      __IO uint16_t UCBRKIE             	:	1;  /* Receive break character interrupt enable */
      __IO uint16_t UCRXEIE             	:	1;  /* Receive erroneous-character interrupt enable */
      __IO uint16_t UCSSEL              	:	2;  /* eUSCI_A clock source select. These bits select the BRCLK source clock. */
      __IO uint16_t UCSYNC              	:	1;  /* Synchronous mode enable */
      __IO uint16_t UCMODE              	:	2;  /* eUSCI_A mode. The UCMODEx bits select the asynchronous mode when UCSYNC = 0. */
      __IO uint16_t UCSPB               	:	1;  /* Stop bit select. Number of stop bits. */
      __IO uint16_t UC7BIT              	:	1;  /* Character length. Selects 7-bit or 8-bit character length. */
      __IO uint16_t UCMSB               	:	1;  /* MSB first select. Controls the direction of the receive and transmit shift  */
                                                /* register. */
      __IO uint16_t UCPAR               	:	1;  /* Parity select. UCPAR is not used when parity is disabled. */
      __IO uint16_t UCPEN               	:	1;  /* Parity enable */

    } bit;
  } CTLW0;
  union {    /* EUSCI_A2_UART_CTLW1 Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A2_UART_CTLW1 Control Bits */
      __IO uint16_t UCGLIT              	:	2;  /* Deglitch time */
      __I  uint16_t RESERVED1           	:	14;  /* Reserved */

    } bit;
  } CTLW1;
  uint8_t RESERVED0[2];
  uint16_t BRW; /* eUSCI_Ax Baud Rate Control Word */
  union {    /* EUSCI_A2_UART_MCTLW Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A2_UART_MCTLW Control Bits */
      __IO uint16_t UCOS16              	:	1;  /* Oversampling mode enabled */
      __I  uint16_t RESERVED1           	:	3;  /* Reserved */
      __IO uint16_t UCBRF               	:	4;  /* First modulation stage select. These bits determine the modulation pattern for  */
                                                /* BITCLK16 when UCOS16 = 1. Ignored with UCOS16 = 0. The "Oversampling Baud-Rate  */
                                                /* Generation" section shows the modulation pattern. */
      __IO uint16_t UCBRS               	:	8;  /* Second modulation stage select. These bits hold a free modulation pattern for  */
                                                /* BITCLK. */

    } bit;
  } MCTLW;
  union {    /* EUSCI_A2_UART_STATW Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A2_UART_STATW Control Bits */
      __I  uint16_t UCBUSY              	:	1;  /* eUSCI_A busy. This bit indicates if a transmit or receive operation is in  */
                                                /* progress. */
      __IO uint16_t UCADDR_UCIDLE       	:	1;  /* UCADDR: Address received in address-bit multiprocessor mode. UCADDR is cleared  */
                                                /* when UCAxRXBUF is read. UCIDLE: Idle line detected in idle-line multiprocessor  */
                                                /* mode. UCIDLE is cleared when UCAxRXBUF is read. */
      __IO uint16_t UCRXERR             	:	1;  /* Receive error flag. This bit indicates a character was received with error(s).  */
                                                /* When UCRXERR = 1, on or more error flags, UCFE, UCPE, or UCOE is also set.  */
                                                /* UCRXERR is cleared when UCAxRXBUF is read. */
      __IO uint16_t UCBRK               	:	1;  /* Break detect flag */
      __IO uint16_t UCPE                	:	1;  /* Parity error flag. When UCPEN = 0, UCPE is read as 0. */
      __IO uint16_t UCOE                	:	1;  /* Overrun error flag. This bit is set when a character is transferred into  */
                                                /* UCAxRXBUF before the previous character was read. UCOE is cleared automatically  */
                                                /* when UCxRXBUF is read, and must not be cleared by software. Otherwise, it does  */
                                                /* not function correctly. */
      __IO uint16_t UCFE                	:	1;  /* Framing error flag */
      __IO uint16_t UCLISTEN            	:	1;  /* Listen enable. The UCLISTEN bit selects loopback mode. */
      __I  uint16_t RESERVED1           	:	8;  /* Reserved */

    } bit;
  } STATW;
  union {    /* EUSCI_A2_UART_RXBUF Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A2_UART_RXBUF Control Bits */
      __I  uint16_t UCRXBUF             	:	8;  /* The receive-data buffer is user accessible and contains the last received  */
                                                /* character from the receive shift register. Reading UCAxRXBUF resets the receive- */
                                                /* error bits, the UCADDR or UCIDLE bit, and UCRXIFG. In 7-bit data mode, UCAxRXBUF */
                                                /* is LSB justified and the MSB is always reset. */
      __I  uint16_t RESERVED1           	:	8;  /* Reserved */

    } bit;
  } RXBUF;
  union {    /* EUSCI_A2_UART_TXBUF Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A2_UART_TXBUF Control Bits */
      __IO uint16_t UCTXBUF             	:	8;  /* The transmit data buffer is user accessible and holds the data waiting to be  */
                                                /* moved into the transmit shift register and transmitted on UCAxTXD. Writing to  */
                                                /* the transmit data buffer clears UCTXIFG. The MSB of UCAxTXBUF is not used for 7- */
                                                /* bit data and is reset. */
      __I  uint16_t RESERVED1           	:	8;  /* Reserved */

    } bit;
  } TXBUF;
  union {    /* EUSCI_A2_UART_ABCTL Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A2_UART_ABCTL Control Bits */
      __IO uint16_t UCABDEN             	:	1;  /* Automatic baud-rate detect enable */
      __I  uint16_t RESERVED2           	:	1;  /* Reserved */
      __IO uint16_t UCBTOE              	:	1;  /* Break time out error */
      __IO uint16_t UCSTOE              	:	1;  /* Synch field time out error */
      __IO uint16_t UCDELIM             	:	2;  /* Break/synch delimiter length */
      __I  uint16_t RESERVED1           	:	10;  /* Reserved */

    } bit;
  } ABCTL;
  union {    /* EUSCI_A2_UART_IRCTL Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A2_UART_IRCTL Control Bits */
      __IO uint16_t UCIREN              	:	1;  /* IrDA encoder/decoder enable */
      __IO uint16_t UCIRTXCLK           	:	1;  /* IrDA transmit pulse clock select */
      __IO uint16_t UCIRTXPL            	:	6;  /* Transmit pulse length. Pulse length t(PULSE) = (UCIRTXPLx + 1) / [2 - f(IRTXCLK) */
                                                /* ] */
      __IO uint16_t UCIRRXFE            	:	1;  /* IrDA receive filter enabled */
      __IO uint16_t UCIRRXPL            	:	1;  /* IrDA receive input UCAxRXD polarity */
      __IO uint16_t UCIRRXFL            	:	6;  /* Receive filter length. The minimum pulse length for receive is given by: t(MIN)  */
                                                /* = (UCIRRXFLx + 4) / [2 - f(IRTXCLK)] */

    } bit;
  } IRCTL;
  uint8_t RESERVED1[6];
  union {    /* EUSCI_A2_UART_IE Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A2_UART_IE Control Bits */
      __IO uint16_t UCRXIE              	:	1;  /* Receive interrupt enable */
      __IO uint16_t UCTXIE              	:	1;  /* Transmit interrupt enable */
      __IO uint16_t UCSTTIE             	:	1;  /* Start bit interrupt enable */
      __IO uint16_t UCTXCPTIE           	:	1;  /* Transmit complete interrupt enable */
      __I  uint16_t RESERVED1           	:	12;  /* Reserved */

    } bit;
  } IE;
  union {    /* EUSCI_A2_UART_IFG Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A2_UART_IFG Control Bits */
      __IO uint16_t UCRXIFG             	:	1;  /* Receive interrupt flag. UCRXIFG is set when UCAxRXBUF has received a complete  */
                                                /* character. */
      __IO uint16_t UCTXIFG             	:	1;  /* Transmit interrupt flag. UCTXIFG is set when UCAxTXBUF empty. */
      __IO uint16_t UCSTTIFG            	:	1;  /* Start bit interrupt flag. UCSTTIFG is set after a Start bit was received */
      __IO uint16_t UCTXCPTIFG          	:	1;  /* Transmit ready interrupt enable. UCTXRDYIFG is set when the entire byte in the  */
                                                /* internal shift register got shifted out and UCAxTXBUF is empty. */
      __I  uint16_t RESERVED1           	:	12;  /* Reserved */

    } bit;
  } IFG;
  uint16_t IV; /* eUSCI_Ax Interrupt Vector */
} EUSCI_A2_UART_Type;


/* EUSCI_A2_UART_CTLW0 Control Bits */
#define EUSCI_A2_UART_CTLW0_UCSWRST                        (0x0001)              /* Software reset enable */
#define EUSCI_A2_UART_CTLW0_UCTXBRK                        (0x0002)              /* Transmit break. Transmits a break with the next write to the transmit buffer. In */
                                                                                 /* UART mode with automatic baud-rate detection, 055h must be written into  */
                                                                                 /* UCAxTXBUF to generate the required break/synch fields. Otherwise, 0h must be  */
                                                                                 /* written into the transmit buffer. */
#define EUSCI_A2_UART_CTLW0_UCTXADDR                       (0x0004)              /* Transmit address. Next frame to be transmitted is marked as address, depending  */
                                                                                 /* on the selected multiprocessor mode. */
#define EUSCI_A2_UART_CTLW0_UCDORM                         (0x0008)              /* Dormant. Puts eUSCI_A into sleep mode. */
#define EUSCI_A2_UART_CTLW0_UCBRKIE                        (0x0010)              /* Receive break character interrupt enable */
#define EUSCI_A2_UART_CTLW0_UCRXEIE                        (0x0020)              /* Receive erroneous-character interrupt enable */
#define EUSCI_A2_UART_CTLW0_UCSSEL__M                      (0x00c0)              /* eUSCI_A clock source select. These bits select the BRCLK source clock. */
#define EUSCI_A2_UART_CTLW0_UCSSEL__0                      (0x0000)              /* UCLK */
#define EUSCI_A2_UART_CTLW0_UCSSEL__0_UCLK                 (0x0000)              /* UCLK */
#define EUSCI_A2_UART_CTLW0_UCSSEL__1                      (0x0040)              /* ACLK */
#define EUSCI_A2_UART_CTLW0_UCSSEL__1_ACLK                 (0x0040)              /* ACLK */
#define EUSCI_A2_UART_CTLW0_UCSSEL__2                      (0x0080)              /* SMCLK */
#define EUSCI_A2_UART_CTLW0_UCSSEL__2_SMCLK                (0x0080)              /* SMCLK */
#define EUSCI_A2_UART_CTLW0_UCSSEL__3                      (0x00c0)              /* SMCLK */
#define EUSCI_A2_UART_CTLW0_UCSSEL__3_SMCLK                (0x00c0)              /* SMCLK */
#define EUSCI_A2_UART_CTLW0_UCSYNC                         (0x0100)              /* Synchronous mode enable */
#define EUSCI_A2_UART_CTLW0_UCMODE__M                      (0x0600)              /* eUSCI_A mode. The UCMODEx bits select the asynchronous mode when UCSYNC = 0. */
#define EUSCI_A2_UART_CTLW0_UCMODE__0                      (0x0000)              /* UART mode */
#define EUSCI_A2_UART_CTLW0_UCMODE__0_UART_MODE            (0x0000)              /* UART mode */
#define EUSCI_A2_UART_CTLW0_UCMODE__1                      (0x0200)              /* Idle-line multiprocessor mode */
#define EUSCI_A2_UART_CTLW0_UCMODE__2                      (0x0400)              /* Address-bit multiprocessor mode */
#define EUSCI_A2_UART_CTLW0_UCMODE__3                      (0x0600)              /* UART mode with automatic baud-rate detection */
#define EUSCI_A2_UART_CTLW0_UCSPB                          (0x0800)              /* Stop bit select. Number of stop bits. */
#define EUSCI_A2_UART_CTLW0_UC7BIT                         (0x1000)              /* Character length. Selects 7-bit or 8-bit character length. */
#define EUSCI_A2_UART_CTLW0_UCMSB                          (0x2000)              /* MSB first select. Controls the direction of the receive and transmit shift  */
                                                                                 /* register. */
#define EUSCI_A2_UART_CTLW0_UCPAR                          (0x4000)              /* Parity select. UCPAR is not used when parity is disabled. */
#define EUSCI_A2_UART_CTLW0_UCPEN                          (0x8000)              /* Parity enable */

/* EUSCI_A2_UART_CTLW1 Control Bits */
#define EUSCI_A2_UART_CTLW1_UCGLIT__M                      (0x0003)              /* Deglitch time */
#define EUSCI_A2_UART_CTLW1_UCGLIT__0                      (0x0000)              /* ~2 ns */
#define EUSCI_A2_UART_CTLW1_UCGLIT__0__2_NS                (0x0000)              /* ~2 ns */
#define EUSCI_A2_UART_CTLW1_UCGLIT__1                      (0x0001)              /* ~50 ns */
#define EUSCI_A2_UART_CTLW1_UCGLIT__1__50_NS               (0x0001)              /* ~50 ns */
#define EUSCI_A2_UART_CTLW1_UCGLIT__2                      (0x0002)              /* ~100 ns */
#define EUSCI_A2_UART_CTLW1_UCGLIT__2__100_NS              (0x0002)              /* ~100 ns */
#define EUSCI_A2_UART_CTLW1_UCGLIT__3                      (0x0003)              /* ~200 ns */
#define EUSCI_A2_UART_CTLW1_UCGLIT__3__200_NS              (0x0003)              /* ~200 ns */
#define EUSCI_A2_UART_CTLW1_RESERVED1__M                   (0xfffc)              /*  */

/* EUSCI_A2_UART_MCTLW Control Bits */
#define EUSCI_A2_UART_MCTLW_UCOS16                         (0x0001)              /* Oversampling mode enabled */
#define EUSCI_A2_UART_MCTLW_RESERVED1__M                   (0x000e)              /*  */
#define EUSCI_A2_UART_MCTLW_UCBRF__M                       (0x00f0)              /* First modulation stage select. These bits determine the modulation pattern for  */
                                                                                 /* BITCLK16 when UCOS16 = 1. Ignored with UCOS16 = 0. The "Oversampling Baud-Rate  */
                                                                                 /* Generation" section shows the modulation pattern. */
#define EUSCI_A2_UART_MCTLW_UCBRS__M                       (0xff00)              /* Second modulation stage select. These bits hold a free modulation pattern for  */
                                                                                 /* BITCLK. */

/* EUSCI_A2_UART_STATW Control Bits */
#define EUSCI_A2_UART_STATW_UCBUSY                         (0x0001)              /* eUSCI_A busy. This bit indicates if a transmit or receive operation is in  */
                                                                                 /* progress. */
#define EUSCI_A2_UART_STATW_UCADDR_UCIDLE                  (0x0002)              /* UCADDR: Address received in address-bit multiprocessor mode. UCADDR is cleared  */
                                                                                 /* when UCAxRXBUF is read. UCIDLE: Idle line detected in idle-line multiprocessor  */
                                                                                 /* mode. UCIDLE is cleared when UCAxRXBUF is read. */
#define EUSCI_A2_UART_STATW_UCRXERR                        (0x0004)              /* Receive error flag. This bit indicates a character was received with error(s).  */
                                                                                 /* When UCRXERR = 1, on or more error flags, UCFE, UCPE, or UCOE is also set.  */
                                                                                 /* UCRXERR is cleared when UCAxRXBUF is read. */
#define EUSCI_A2_UART_STATW_UCBRK                          (0x0008)              /* Break detect flag */
#define EUSCI_A2_UART_STATW_UCPE                           (0x0010)              /* Parity error flag. When UCPEN = 0, UCPE is read as 0. */
#define EUSCI_A2_UART_STATW_UCOE                           (0x0020)              /* Overrun error flag. This bit is set when a character is transferred into  */
                                                                                 /* UCAxRXBUF before the previous character was read. UCOE is cleared automatically  */
                                                                                 /* when UCxRXBUF is read, and must not be cleared by software. Otherwise, it does  */
                                                                                 /* not function correctly. */
#define EUSCI_A2_UART_STATW_UCFE                           (0x0040)              /* Framing error flag */
#define EUSCI_A2_UART_STATW_UCLISTEN                       (0x0080)              /* Listen enable. The UCLISTEN bit selects loopback mode. */
#define EUSCI_A2_UART_STATW_RESERVED1__M                   (0xff00)              /*  */

/* EUSCI_A2_UART_RXBUF Control Bits */
#define EUSCI_A2_UART_RXBUF_UCRXBUF__M                     (0x00ff)              /* The receive-data buffer is user accessible and contains the last received  */
                                                                                 /* character from the receive shift register. Reading UCAxRXBUF resets the receive- */
                                                                                 /* error bits, the UCADDR or UCIDLE bit, and UCRXIFG. In 7-bit data mode, UCAxRXBUF */
                                                                                 /* is LSB justified and the MSB is always reset. */
#define EUSCI_A2_UART_RXBUF_RESERVED1__M                   (0xff00)              /*  */

/* EUSCI_A2_UART_TXBUF Control Bits */
#define EUSCI_A2_UART_TXBUF_UCTXBUF__M                     (0x00ff)              /* The transmit data buffer is user accessible and holds the data waiting to be  */
                                                                                 /* moved into the transmit shift register and transmitted on UCAxTXD. Writing to  */
                                                                                 /* the transmit data buffer clears UCTXIFG. The MSB of UCAxTXBUF is not used for 7- */
                                                                                 /* bit data and is reset. */
#define EUSCI_A2_UART_TXBUF_RESERVED1__M                   (0xff00)              /*  */

/* EUSCI_A2_UART_ABCTL Control Bits */
#define EUSCI_A2_UART_ABCTL_UCABDEN                        (0x0001)              /* Automatic baud-rate detect enable */
#define EUSCI_A2_UART_ABCTL_RESERVED2                      (0x0002)              /*  */
#define EUSCI_A2_UART_ABCTL_UCBTOE                         (0x0004)              /* Break time out error */
#define EUSCI_A2_UART_ABCTL_UCSTOE                         (0x0008)              /* Synch field time out error */
#define EUSCI_A2_UART_ABCTL_UCDELIM__M                     (0x0030)              /* Break/synch delimiter length */
#define EUSCI_A2_UART_ABCTL_UCDELIM__0                     (0x0000)              /* 1 bit time */
#define EUSCI_A2_UART_ABCTL_UCDELIM__0_1_BIT_TIME          (0x0000)              /* 1 bit time */
#define EUSCI_A2_UART_ABCTL_UCDELIM__1                     (0x0010)              /* 2 bit times */
#define EUSCI_A2_UART_ABCTL_UCDELIM__1_2_BIT_TIMES         (0x0010)              /* 2 bit times */
#define EUSCI_A2_UART_ABCTL_UCDELIM__2                     (0x0020)              /* 3 bit times */
#define EUSCI_A2_UART_ABCTL_UCDELIM__2_3_BIT_TIMES         (0x0020)              /* 3 bit times */
#define EUSCI_A2_UART_ABCTL_UCDELIM__3                     (0x0030)              /* 4 bit times */
#define EUSCI_A2_UART_ABCTL_UCDELIM__3_4_BIT_TIMES         (0x0030)              /* 4 bit times */
#define EUSCI_A2_UART_ABCTL_RESERVED1__M                   (0xffc0)              /*  */

/* EUSCI_A2_UART_IRCTL Control Bits */
#define EUSCI_A2_UART_IRCTL_UCIREN                         (0x0001)              /* IrDA encoder/decoder enable */
#define EUSCI_A2_UART_IRCTL_UCIRTXCLK                      (0x0002)              /* IrDA transmit pulse clock select */
#define EUSCI_A2_UART_IRCTL_UCIRTXPL__M                    (0x00fc)              /* Transmit pulse length. Pulse length t(PULSE) = (UCIRTXPLx + 1) / [2 - f(IRTXCLK) */
                                                                                 /* ] */
#define EUSCI_A2_UART_IRCTL_UCIRRXFE                       (0x0100)              /* IrDA receive filter enabled */
#define EUSCI_A2_UART_IRCTL_UCIRRXPL                       (0x0200)              /* IrDA receive input UCAxRXD polarity */
#define EUSCI_A2_UART_IRCTL_UCIRRXFL__M                    (0xfc00)              /* Receive filter length. The minimum pulse length for receive is given by: t(MIN)  */
                                                                                 /* = (UCIRRXFLx + 4) / [2 - f(IRTXCLK)] */

/* EUSCI_A2_UART_IE Control Bits */
#define EUSCI_A2_UART_IE_UCRXIE                            (0x0001)              /* Receive interrupt enable */
#define EUSCI_A2_UART_IE_UCTXIE                            (0x0002)              /* Transmit interrupt enable */
#define EUSCI_A2_UART_IE_UCSTTIE                           (0x0004)              /* Start bit interrupt enable */
#define EUSCI_A2_UART_IE_UCTXCPTIE                         (0x0008)              /* Transmit complete interrupt enable */
#define EUSCI_A2_UART_IE_RESERVED1__M                      (0xfff0)              /*  */

/* EUSCI_A2_UART_IFG Control Bits */
#define EUSCI_A2_UART_IFG_UCRXIFG                          (0x0001)              /* Receive interrupt flag. UCRXIFG is set when UCAxRXBUF has received a complete  */
                                                                                 /* character. */
#define EUSCI_A2_UART_IFG_UCTXIFG                          (0x0002)              /* Transmit interrupt flag. UCTXIFG is set when UCAxTXBUF empty. */
#define EUSCI_A2_UART_IFG_UCSTTIFG                         (0x0004)              /* Start bit interrupt flag. UCSTTIFG is set after a Start bit was received */
#define EUSCI_A2_UART_IFG_UCTXCPTIFG                       (0x0008)              /* Transmit ready interrupt enable. UCTXRDYIFG is set when the entire byte in the  */
                                                                                 /* internal shift register got shifted out and UCAxTXBUF is empty. */
#define EUSCI_A2_UART_IFG_RESERVED1__M                     (0xfff0)              /*  */


//*****************************************************************************
// EUSCI_A3_SPI Registers
//*****************************************************************************
typedef struct {
  union {    /* EUSCI_A3_SPI_CTLW0 Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A3_SPI_CTLW0 Control Bits */
      __IO uint16_t UCSWRST             	:	1;  /* Software reset enable */
      __IO uint16_t UCSTEM              	:	1;  /* STE mode select in master mode. This byte is ignored in slave or 3-wire mode. */
      __I  uint16_t RESERVED1           	:	4;  /* Reserved */
      __IO uint16_t UCSSEL              	:	2;  /* eUSCI clock source select. These bits select the BRCLK source clock in master  */
                                                /* mode. UCxCLK is always used in slave mode. */
      __IO uint16_t UCSYNC              	:	1;  /* Synchronous mode enable */
      __IO uint16_t UCMODE              	:	2;  /* eUSCI mode. The UCMODEx bits select the synchronous mode when UCSYNC = 1. */
      __IO uint16_t UCMST               	:	1;  /* Master mode select */
      __IO uint16_t UC7BIT              	:	1;  /* Character length. Selects 7-bit or 8-bit character length. */
      __IO uint16_t UCMSB               	:	1;  /* MSB first select. Controls the direction of the receive and transmit shift  */
                                                /* register. */
      __IO uint16_t UCCKPL              	:	1;  /* Clock polarity select */
      __IO uint16_t UCCKPH              	:	1;  /* Clock phase select */

    } bit;
  } CTLW0;
  uint8_t RESERVED0[4];
  uint16_t BRW; /* eUSCI_Ax Bit Rate Control Word */
  uint8_t RESERVED1[2];
  union {    /* EUSCI_A3_SPI_STATW Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A3_SPI_STATW Control Bits */
      __I  uint16_t UCBUSY              	:	1;  /* eUSCI busy. This bit indicates if a transmit or receive operation is in progress */
                                                /* . */
      __IO uint16_t RESERVED2           	:	4;  /* Reserved */
      __IO uint16_t UCOE                	:	1;  /* Overrun error flag. This bit is set when a character is transferred into  */
                                                /* UCxRXBUF before the previous character was read. UCOE is cleared automatically  */
                                                /* when UCxRXBUF is read, and must not be cleared by software. Otherwise, it does  */
                                                /* not function correctly. */
      __IO uint16_t UCFE                	:	1;  /* Framing error flag. This bit indicates a bus conflict in 4-wire master mode.  */
                                                /* UCFE is not used in 3-wire master or any slave mode. */
      __IO uint16_t UCLISTEN            	:	1;  /* Listen enable. The UCLISTEN bit selects loopback mode. */
      __I  uint16_t RESERVED1           	:	8;  /* Reserved */

    } bit;
  } STATW;
  union {    /* EUSCI_A3_SPI_RXBUF Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A3_SPI_RXBUF Control Bits */
      __I  uint16_t UCRXBUF             	:	8;  /* The receive-data buffer is user accessible and contains the last received  */
                                                /* character from the receive shift register. Reading UCxRXBUF resets the receive- */
                                                /* error bits and UCRXIFG. In 7-bit data mode, UCxRXBUF is LSB justified and the  */
                                                /* MSB is always reset. */
      __I  uint16_t RESERVED1           	:	8;  /* Reserved */

    } bit;
  } RXBUF;
  union {    /* EUSCI_A3_SPI_TXBUF Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A3_SPI_TXBUF Control Bits */
      __IO uint16_t UCTXBUF             	:	8;  /* The transmit data buffer is user accessible and holds the data waiting to be  */
                                                /* moved into the transmit shift register and transmitted. Writing to the transmit  */
                                                /* data buffer clears UCTXIFG. The MSB of UCxTXBUF is not used for 7-bit data and  */
                                                /* is reset. */
      __I  uint16_t RESERVED1           	:	8;  /* Reserved */

    } bit;
  } TXBUF;
  uint8_t RESERVED2[10];
  union {    /* EUSCI_A3_SPI_IE Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A3_SPI_IE Control Bits */
      __IO uint16_t UCRXIE              	:	1;  /* Receive interrupt enable */
      __IO uint16_t UCTXIE              	:	1;  /* Transmit interrupt enable */
      __I  uint16_t RESERVED1           	:	14;  /* Reserved */

    } bit;
  } IE;
  union {    /* EUSCI_A3_SPI_IFG Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A3_SPI_IFG Control Bits */
      __IO uint16_t UCRXIFG             	:	1;  /* Receive interrupt flag. UCRXIFG is set when UCxxRXBUF has received a complete  */
                                                /* character. */
      __IO uint16_t UCTXIFG             	:	1;  /* Transmit interrupt flag. UCTXIFG is set when UCxxTXBUF empty. */
      __I  uint16_t RESERVED1           	:	14;  /* Reserved */

    } bit;
  } IFG;
  uint16_t IV; /* eUSCI_Ax Interrupt Vector */
} EUSCI_A3_SPI_Type;


/* EUSCI_A3_SPI_CTLW0 Control Bits */
#define EUSCI_A3_SPI_CTLW0_UCSWRST                         (0x0001)              /* Software reset enable */
#define EUSCI_A3_SPI_CTLW0_UCSTEM                          (0x0002)              /* STE mode select in master mode. This byte is ignored in slave or 3-wire mode. */
#define EUSCI_A3_SPI_CTLW0_RESERVED1__M                    (0x003c)              /*  */
#define EUSCI_A3_SPI_CTLW0_UCSSEL__M                       (0x00c0)              /* eUSCI clock source select. These bits select the BRCLK source clock in master  */
                                                                                 /* mode. UCxCLK is always used in slave mode. */
#define EUSCI_A3_SPI_CTLW0_UCSSEL__1                       (0x0040)              /* ACLK */
#define EUSCI_A3_SPI_CTLW0_UCSSEL__1_ACLK                  (0x0040)              /* ACLK */
#define EUSCI_A3_SPI_CTLW0_UCSSEL__2                       (0x0080)              /* SMCLK */
#define EUSCI_A3_SPI_CTLW0_UCSSEL__2_SMCLK                 (0x0080)              /* SMCLK */
#define EUSCI_A3_SPI_CTLW0_UCSSEL__3                       (0x00c0)              /* SMCLK */
#define EUSCI_A3_SPI_CTLW0_UCSSEL__3_SMCLK                 (0x00c0)              /* SMCLK */
#define EUSCI_A3_SPI_CTLW0_UCSYNC                          (0x0100)              /* Synchronous mode enable */
#define EUSCI_A3_SPI_CTLW0_UCMODE__M                       (0x0600)              /* eUSCI mode. The UCMODEx bits select the synchronous mode when UCSYNC = 1. */
#define EUSCI_A3_SPI_CTLW0_UCMODE__0                       (0x0000)              /* 3-pin SPI */
#define EUSCI_A3_SPI_CTLW0_UCMODE__0_3_PIN_SPI             (0x0000)              /* 3-pin SPI */
#define EUSCI_A3_SPI_CTLW0_UCMODE__1                       (0x0200)              /* 4-pin SPI with UCxSTE active high: Slave enabled when UCxSTE = 1 */
#define EUSCI_A3_SPI_CTLW0_UCMODE__2                       (0x0400)              /* 4-pin SPI with UCxSTE active low: Slave enabled when UCxSTE = 0 */
#define EUSCI_A3_SPI_CTLW0_UCMODE__3                       (0x0600)              /* I2C mode */
#define EUSCI_A3_SPI_CTLW0_UCMODE__3_I2C_MODE              (0x0600)              /* I2C mode */
#define EUSCI_A3_SPI_CTLW0_UCMST                           (0x0800)              /* Master mode select */
#define EUSCI_A3_SPI_CTLW0_UC7BIT                          (0x1000)              /* Character length. Selects 7-bit or 8-bit character length. */
#define EUSCI_A3_SPI_CTLW0_UCMSB                           (0x2000)              /* MSB first select. Controls the direction of the receive and transmit shift  */
                                                                                 /* register. */
#define EUSCI_A3_SPI_CTLW0_UCCKPL                          (0x4000)              /* Clock polarity select */
#define EUSCI_A3_SPI_CTLW0_UCCKPH                          (0x8000)              /* Clock phase select */

/* EUSCI_A3_SPI_STATW Control Bits */
#define EUSCI_A3_SPI_STATW_UCBUSY                          (0x0001)              /* eUSCI busy. This bit indicates if a transmit or receive operation is in progress */
                                                                                 /* . */
#define EUSCI_A3_SPI_STATW_RESERVED2__M                    (0x001e)              /*  */
#define EUSCI_A3_SPI_STATW_UCOE                            (0x0020)              /* Overrun error flag. This bit is set when a character is transferred into  */
                                                                                 /* UCxRXBUF before the previous character was read. UCOE is cleared automatically  */
                                                                                 /* when UCxRXBUF is read, and must not be cleared by software. Otherwise, it does  */
                                                                                 /* not function correctly. */
#define EUSCI_A3_SPI_STATW_UCFE                            (0x0040)              /* Framing error flag. This bit indicates a bus conflict in 4-wire master mode.  */
                                                                                 /* UCFE is not used in 3-wire master or any slave mode. */
#define EUSCI_A3_SPI_STATW_UCLISTEN                        (0x0080)              /* Listen enable. The UCLISTEN bit selects loopback mode. */
#define EUSCI_A3_SPI_STATW_RESERVED1__M                    (0xff00)              /*  */

/* EUSCI_A3_SPI_RXBUF Control Bits */
#define EUSCI_A3_SPI_RXBUF_UCRXBUF__M                      (0x00ff)              /* The receive-data buffer is user accessible and contains the last received  */
                                                                                 /* character from the receive shift register. Reading UCxRXBUF resets the receive- */
                                                                                 /* error bits and UCRXIFG. In 7-bit data mode, UCxRXBUF is LSB justified and the  */
                                                                                 /* MSB is always reset. */
#define EUSCI_A3_SPI_RXBUF_RESERVED1__M                    (0xff00)              /*  */

/* EUSCI_A3_SPI_TXBUF Control Bits */
#define EUSCI_A3_SPI_TXBUF_UCTXBUF__M                      (0x00ff)              /* The transmit data buffer is user accessible and holds the data waiting to be  */
                                                                                 /* moved into the transmit shift register and transmitted. Writing to the transmit  */
                                                                                 /* data buffer clears UCTXIFG. The MSB of UCxTXBUF is not used for 7-bit data and  */
                                                                                 /* is reset. */
#define EUSCI_A3_SPI_TXBUF_RESERVED1__M                    (0xff00)              /*  */

/* EUSCI_A3_SPI_IE Control Bits */
#define EUSCI_A3_SPI_IE_UCRXIE                             (0x0001)              /* Receive interrupt enable */
#define EUSCI_A3_SPI_IE_UCTXIE                             (0x0002)              /* Transmit interrupt enable */
#define EUSCI_A3_SPI_IE_RESERVED1__M                       (0xfffc)              /*  */

/* EUSCI_A3_SPI_IFG Control Bits */
#define EUSCI_A3_SPI_IFG_UCRXIFG                           (0x0001)              /* Receive interrupt flag. UCRXIFG is set when UCxxRXBUF has received a complete  */
                                                                                 /* character. */
#define EUSCI_A3_SPI_IFG_UCTXIFG                           (0x0002)              /* Transmit interrupt flag. UCTXIFG is set when UCxxTXBUF empty. */
#define EUSCI_A3_SPI_IFG_RESERVED1__M                      (0xfffc)              /*  */


//*****************************************************************************
// EUSCI_A3_UART Registers
//*****************************************************************************
typedef struct {
  union {    /* EUSCI_A3_UART_CTLW0 Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A3_UART_CTLW0 Control Bits */
      __IO uint16_t UCSWRST             	:	1;  /* Software reset enable */
      __IO uint16_t UCTXBRK             	:	1;  /* Transmit break. Transmits a break with the next write to the transmit buffer. In */
                                                /* UART mode with automatic baud-rate detection, 055h must be written into  */
                                                /* UCAxTXBUF to generate the required break/synch fields. Otherwise, 0h must be  */
                                                /* written into the transmit buffer. */
      __IO uint16_t UCTXADDR            	:	1;  /* Transmit address. Next frame to be transmitted is marked as address, depending  */
                                                /* on the selected multiprocessor mode. */
      __IO uint16_t UCDORM              	:	1;  /* Dormant. Puts eUSCI_A into sleep mode. */
      __IO uint16_t UCBRKIE             	:	1;  /* Receive break character interrupt enable */
      __IO uint16_t UCRXEIE             	:	1;  /* Receive erroneous-character interrupt enable */
      __IO uint16_t UCSSEL              	:	2;  /* eUSCI_A clock source select. These bits select the BRCLK source clock. */
      __IO uint16_t UCSYNC              	:	1;  /* Synchronous mode enable */
      __IO uint16_t UCMODE              	:	2;  /* eUSCI_A mode. The UCMODEx bits select the asynchronous mode when UCSYNC = 0. */
      __IO uint16_t UCSPB               	:	1;  /* Stop bit select. Number of stop bits. */
      __IO uint16_t UC7BIT              	:	1;  /* Character length. Selects 7-bit or 8-bit character length. */
      __IO uint16_t UCMSB               	:	1;  /* MSB first select. Controls the direction of the receive and transmit shift  */
                                                /* register. */
      __IO uint16_t UCPAR               	:	1;  /* Parity select. UCPAR is not used when parity is disabled. */
      __IO uint16_t UCPEN               	:	1;  /* Parity enable */

    } bit;
  } CTLW0;
  union {    /* EUSCI_A3_UART_CTLW1 Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A3_UART_CTLW1 Control Bits */
      __IO uint16_t UCGLIT              	:	2;  /* Deglitch time */
      __I  uint16_t RESERVED1           	:	14;  /* Reserved */

    } bit;
  } CTLW1;
  uint8_t RESERVED0[2];
  uint16_t BRW; /* eUSCI_Ax Baud Rate Control Word */
  union {    /* EUSCI_A3_UART_MCTLW Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A3_UART_MCTLW Control Bits */
      __IO uint16_t UCOS16              	:	1;  /* Oversampling mode enabled */
      __I  uint16_t RESERVED1           	:	3;  /* Reserved */
      __IO uint16_t UCBRF               	:	4;  /* First modulation stage select. These bits determine the modulation pattern for  */
                                                /* BITCLK16 when UCOS16 = 1. Ignored with UCOS16 = 0. The "Oversampling Baud-Rate  */
                                                /* Generation" section shows the modulation pattern. */
      __IO uint16_t UCBRS               	:	8;  /* Second modulation stage select. These bits hold a free modulation pattern for  */
                                                /* BITCLK. */

    } bit;
  } MCTLW;
  union {    /* EUSCI_A3_UART_STATW Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A3_UART_STATW Control Bits */
      __I  uint16_t UCBUSY              	:	1;  /* eUSCI_A busy. This bit indicates if a transmit or receive operation is in  */
                                                /* progress. */
      __IO uint16_t UCADDR_UCIDLE       	:	1;  /* UCADDR: Address received in address-bit multiprocessor mode. UCADDR is cleared  */
                                                /* when UCAxRXBUF is read. UCIDLE: Idle line detected in idle-line multiprocessor  */
                                                /* mode. UCIDLE is cleared when UCAxRXBUF is read. */
      __IO uint16_t UCRXERR             	:	1;  /* Receive error flag. This bit indicates a character was received with error(s).  */
                                                /* When UCRXERR = 1, on or more error flags, UCFE, UCPE, or UCOE is also set.  */
                                                /* UCRXERR is cleared when UCAxRXBUF is read. */
      __IO uint16_t UCBRK               	:	1;  /* Break detect flag */
      __IO uint16_t UCPE                	:	1;  /* Parity error flag. When UCPEN = 0, UCPE is read as 0. */
      __IO uint16_t UCOE                	:	1;  /* Overrun error flag. This bit is set when a character is transferred into  */
                                                /* UCAxRXBUF before the previous character was read. UCOE is cleared automatically  */
                                                /* when UCxRXBUF is read, and must not be cleared by software. Otherwise, it does  */
                                                /* not function correctly. */
      __IO uint16_t UCFE                	:	1;  /* Framing error flag */
      __IO uint16_t UCLISTEN            	:	1;  /* Listen enable. The UCLISTEN bit selects loopback mode. */
      __I  uint16_t RESERVED1           	:	8;  /* Reserved */

    } bit;
  } STATW;
  union {    /* EUSCI_A3_UART_RXBUF Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A3_UART_RXBUF Control Bits */
      __I  uint16_t UCRXBUF             	:	8;  /* The receive-data buffer is user accessible and contains the last received  */
                                                /* character from the receive shift register. Reading UCAxRXBUF resets the receive- */
                                                /* error bits, the UCADDR or UCIDLE bit, and UCRXIFG. In 7-bit data mode, UCAxRXBUF */
                                                /* is LSB justified and the MSB is always reset. */
      __I  uint16_t RESERVED1           	:	8;  /* Reserved */

    } bit;
  } RXBUF;
  union {    /* EUSCI_A3_UART_TXBUF Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A3_UART_TXBUF Control Bits */
      __IO uint16_t UCTXBUF             	:	8;  /* The transmit data buffer is user accessible and holds the data waiting to be  */
                                                /* moved into the transmit shift register and transmitted on UCAxTXD. Writing to  */
                                                /* the transmit data buffer clears UCTXIFG. The MSB of UCAxTXBUF is not used for 7- */
                                                /* bit data and is reset. */
      __I  uint16_t RESERVED1           	:	8;  /* Reserved */

    } bit;
  } TXBUF;
  union {    /* EUSCI_A3_UART_ABCTL Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A3_UART_ABCTL Control Bits */
      __IO uint16_t UCABDEN             	:	1;  /* Automatic baud-rate detect enable */
      __I  uint16_t RESERVED2           	:	1;  /* Reserved */
      __IO uint16_t UCBTOE              	:	1;  /* Break time out error */
      __IO uint16_t UCSTOE              	:	1;  /* Synch field time out error */
      __IO uint16_t UCDELIM             	:	2;  /* Break/synch delimiter length */
      __I  uint16_t RESERVED1           	:	10;  /* Reserved */

    } bit;
  } ABCTL;
  union {    /* EUSCI_A3_UART_IRCTL Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A3_UART_IRCTL Control Bits */
      __IO uint16_t UCIREN              	:	1;  /* IrDA encoder/decoder enable */
      __IO uint16_t UCIRTXCLK           	:	1;  /* IrDA transmit pulse clock select */
      __IO uint16_t UCIRTXPL            	:	6;  /* Transmit pulse length. Pulse length t(PULSE) = (UCIRTXPLx + 1) / [2 - f(IRTXCLK) */
                                                /* ] */
      __IO uint16_t UCIRRXFE            	:	1;  /* IrDA receive filter enabled */
      __IO uint16_t UCIRRXPL            	:	1;  /* IrDA receive input UCAxRXD polarity */
      __IO uint16_t UCIRRXFL            	:	6;  /* Receive filter length. The minimum pulse length for receive is given by: t(MIN)  */
                                                /* = (UCIRRXFLx + 4) / [2 - f(IRTXCLK)] */

    } bit;
  } IRCTL;
  uint8_t RESERVED1[6];
  union {    /* EUSCI_A3_UART_IE Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A3_UART_IE Control Bits */
      __IO uint16_t UCRXIE              	:	1;  /* Receive interrupt enable */
      __IO uint16_t UCTXIE              	:	1;  /* Transmit interrupt enable */
      __IO uint16_t UCSTTIE             	:	1;  /* Start bit interrupt enable */
      __IO uint16_t UCTXCPTIE           	:	1;  /* Transmit complete interrupt enable */
      __I  uint16_t RESERVED1           	:	12;  /* Reserved */

    } bit;
  } IE;
  union {    /* EUSCI_A3_UART_IFG Register */
    __IO uint16_t reg;
    struct { /* EUSCI_A3_UART_IFG Control Bits */
      __IO uint16_t UCRXIFG             	:	1;  /* Receive interrupt flag. UCRXIFG is set when UCAxRXBUF has received a complete  */
                                                /* character. */
      __IO uint16_t UCTXIFG             	:	1;  /* Transmit interrupt flag. UCTXIFG is set when UCAxTXBUF empty. */
      __IO uint16_t UCSTTIFG            	:	1;  /* Start bit interrupt flag. UCSTTIFG is set after a Start bit was received */
      __IO uint16_t UCTXCPTIFG          	:	1;  /* Transmit ready interrupt enable. UCTXRDYIFG is set when the entire byte in the  */
                                                /* internal shift register got shifted out and UCAxTXBUF is empty. */
      __I  uint16_t RESERVED1           	:	12;  /* Reserved */

    } bit;
  } IFG;
  uint16_t IV; /* eUSCI_Ax Interrupt Vector */
} EUSCI_A3_UART_Type;


/* EUSCI_A3_UART_CTLW0 Control Bits */
#define EUSCI_A3_UART_CTLW0_UCSWRST                        (0x0001)              /* Software reset enable */
#define EUSCI_A3_UART_CTLW0_UCTXBRK                        (0x0002)              /* Transmit break. Transmits a break with the next write to the transmit buffer. In */
                                                                                 /* UART mode with automatic baud-rate detection, 055h must be written into  */
                                                                                 /* UCAxTXBUF to generate the required break/synch fields. Otherwise, 0h must be  */
                                                                                 /* written into the transmit buffer. */
#define EUSCI_A3_UART_CTLW0_UCTXADDR                       (0x0004)              /* Transmit address. Next frame to be transmitted is marked as address, depending  */
                                                                                 /* on the selected multiprocessor mode. */
#define EUSCI_A3_UART_CTLW0_UCDORM                         (0x0008)              /* Dormant. Puts eUSCI_A into sleep mode. */
#define EUSCI_A3_UART_CTLW0_UCBRKIE                        (0x0010)              /* Receive break character interrupt enable */
#define EUSCI_A3_UART_CTLW0_UCRXEIE                        (0x0020)              /* Receive erroneous-character interrupt enable */
#define EUSCI_A3_UART_CTLW0_UCSSEL__M                      (0x00c0)              /* eUSCI_A clock source select. These bits select the BRCLK source clock. */
#define EUSCI_A3_UART_CTLW0_UCSSEL__0                      (0x0000)              /* UCLK */
#define EUSCI_A3_UART_CTLW0_UCSSEL__0_UCLK                 (0x0000)              /* UCLK */
#define EUSCI_A3_UART_CTLW0_UCSSEL__1                      (0x0040)              /* ACLK */
#define EUSCI_A3_UART_CTLW0_UCSSEL__1_ACLK                 (0x0040)              /* ACLK */
#define EUSCI_A3_UART_CTLW0_UCSSEL__2                      (0x0080)              /* SMCLK */
#define EUSCI_A3_UART_CTLW0_UCSSEL__2_SMCLK                (0x0080)              /* SMCLK */
#define EUSCI_A3_UART_CTLW0_UCSSEL__3                      (0x00c0)              /* SMCLK */
#define EUSCI_A3_UART_CTLW0_UCSSEL__3_SMCLK                (0x00c0)              /* SMCLK */
#define EUSCI_A3_UART_CTLW0_UCSYNC                         (0x0100)              /* Synchronous mode enable */
#define EUSCI_A3_UART_CTLW0_UCMODE__M                      (0x0600)              /* eUSCI_A mode. The UCMODEx bits select the asynchronous mode when UCSYNC = 0. */
#define EUSCI_A3_UART_CTLW0_UCMODE__0                      (0x0000)              /* UART mode */
#define EUSCI_A3_UART_CTLW0_UCMODE__0_UART_MODE            (0x0000)              /* UART mode */
#define EUSCI_A3_UART_CTLW0_UCMODE__1                      (0x0200)              /* Idle-line multiprocessor mode */
#define EUSCI_A3_UART_CTLW0_UCMODE__2                      (0x0400)              /* Address-bit multiprocessor mode */
#define EUSCI_A3_UART_CTLW0_UCMODE__3                      (0x0600)              /* UART mode with automatic baud-rate detection */
#define EUSCI_A3_UART_CTLW0_UCSPB                          (0x0800)              /* Stop bit select. Number of stop bits. */
#define EUSCI_A3_UART_CTLW0_UC7BIT                         (0x1000)              /* Character length. Selects 7-bit or 8-bit character length. */
#define EUSCI_A3_UART_CTLW0_UCMSB                          (0x2000)              /* MSB first select. Controls the direction of the receive and transmit shift  */
                                                                                 /* register. */
#define EUSCI_A3_UART_CTLW0_UCPAR                          (0x4000)              /* Parity select. UCPAR is not used when parity is disabled. */
#define EUSCI_A3_UART_CTLW0_UCPEN                          (0x8000)              /* Parity enable */

/* EUSCI_A3_UART_CTLW1 Control Bits */
#define EUSCI_A3_UART_CTLW1_UCGLIT__M                      (0x0003)              /* Deglitch time */
#define EUSCI_A3_UART_CTLW1_UCGLIT__0                      (0x0000)              /* ~2 ns */
#define EUSCI_A3_UART_CTLW1_UCGLIT__0__2_NS                (0x0000)              /* ~2 ns */
#define EUSCI_A3_UART_CTLW1_UCGLIT__1                      (0x0001)              /* ~50 ns */
#define EUSCI_A3_UART_CTLW1_UCGLIT__1__50_NS               (0x0001)              /* ~50 ns */
#define EUSCI_A3_UART_CTLW1_UCGLIT__2                      (0x0002)              /* ~100 ns */
#define EUSCI_A3_UART_CTLW1_UCGLIT__2__100_NS              (0x0002)              /* ~100 ns */
#define EUSCI_A3_UART_CTLW1_UCGLIT__3                      (0x0003)              /* ~200 ns */
#define EUSCI_A3_UART_CTLW1_UCGLIT__3__200_NS              (0x0003)              /* ~200 ns */
#define EUSCI_A3_UART_CTLW1_RESERVED1__M                   (0xfffc)              /*  */

/* EUSCI_A3_UART_MCTLW Control Bits */
#define EUSCI_A3_UART_MCTLW_UCOS16                         (0x0001)              /* Oversampling mode enabled */
#define EUSCI_A3_UART_MCTLW_RESERVED1__M                   (0x000e)              /*  */
#define EUSCI_A3_UART_MCTLW_UCBRF__M                       (0x00f0)              /* First modulation stage select. These bits determine the modulation pattern for  */
                                                                                 /* BITCLK16 when UCOS16 = 1. Ignored with UCOS16 = 0. The "Oversampling Baud-Rate  */
                                                                                 /* Generation" section shows the modulation pattern. */
#define EUSCI_A3_UART_MCTLW_UCBRS__M                       (0xff00)              /* Second modulation stage select. These bits hold a free modulation pattern for  */
                                                                                 /* BITCLK. */

/* EUSCI_A3_UART_STATW Control Bits */
#define EUSCI_A3_UART_STATW_UCBUSY                         (0x0001)              /* eUSCI_A busy. This bit indicates if a transmit or receive operation is in  */
                                                                                 /* progress. */
#define EUSCI_A3_UART_STATW_UCADDR_UCIDLE                  (0x0002)              /* UCADDR: Address received in address-bit multiprocessor mode. UCADDR is cleared  */
                                                                                 /* when UCAxRXBUF is read. UCIDLE: Idle line detected in idle-line multiprocessor  */
                                                                                 /* mode. UCIDLE is cleared when UCAxRXBUF is read. */
#define EUSCI_A3_UART_STATW_UCRXERR                        (0x0004)              /* Receive error flag. This bit indicates a character was received with error(s).  */
                                                                                 /* When UCRXERR = 1, on or more error flags, UCFE, UCPE, or UCOE is also set.  */
                                                                                 /* UCRXERR is cleared when UCAxRXBUF is read. */
#define EUSCI_A3_UART_STATW_UCBRK                          (0x0008)              /* Break detect flag */
#define EUSCI_A3_UART_STATW_UCPE                           (0x0010)              /* Parity error flag. When UCPEN = 0, UCPE is read as 0. */
#define EUSCI_A3_UART_STATW_UCOE                           (0x0020)              /* Overrun error flag. This bit is set when a character is transferred into  */
                                                                                 /* UCAxRXBUF before the previous character was read. UCOE is cleared automatically  */
                                                                                 /* when UCxRXBUF is read, and must not be cleared by software. Otherwise, it does  */
                                                                                 /* not function correctly. */
#define EUSCI_A3_UART_STATW_UCFE                           (0x0040)              /* Framing error flag */
#define EUSCI_A3_UART_STATW_UCLISTEN                       (0x0080)              /* Listen enable. The UCLISTEN bit selects loopback mode. */
#define EUSCI_A3_UART_STATW_RESERVED1__M                   (0xff00)              /*  */

/* EUSCI_A3_UART_RXBUF Control Bits */
#define EUSCI_A3_UART_RXBUF_UCRXBUF__M                     (0x00ff)              /* The receive-data buffer is user accessible and contains the last received  */
                                                                                 /* character from the receive shift register. Reading UCAxRXBUF resets the receive- */
                                                                                 /* error bits, the UCADDR or UCIDLE bit, and UCRXIFG. In 7-bit data mode, UCAxRXBUF */
                                                                                 /* is LSB justified and the MSB is always reset. */
#define EUSCI_A3_UART_RXBUF_RESERVED1__M                   (0xff00)              /*  */

/* EUSCI_A3_UART_TXBUF Control Bits */
#define EUSCI_A3_UART_TXBUF_UCTXBUF__M                     (0x00ff)              /* The transmit data buffer is user accessible and holds the data waiting to be  */
                                                                                 /* moved into the transmit shift register and transmitted on UCAxTXD. Writing to  */
                                                                                 /* the transmit data buffer clears UCTXIFG. The MSB of UCAxTXBUF is not used for 7- */
                                                                                 /* bit data and is reset. */
#define EUSCI_A3_UART_TXBUF_RESERVED1__M                   (0xff00)              /*  */

/* EUSCI_A3_UART_ABCTL Control Bits */
#define EUSCI_A3_UART_ABCTL_UCABDEN                        (0x0001)              /* Automatic baud-rate detect enable */
#define EUSCI_A3_UART_ABCTL_RESERVED2                      (0x0002)              /*  */
#define EUSCI_A3_UART_ABCTL_UCBTOE                         (0x0004)              /* Break time out error */
#define EUSCI_A3_UART_ABCTL_UCSTOE                         (0x0008)              /* Synch field time out error */
#define EUSCI_A3_UART_ABCTL_UCDELIM__M                     (0x0030)              /* Break/synch delimiter length */
#define EUSCI_A3_UART_ABCTL_UCDELIM__0                     (0x0000)              /* 1 bit time */
#define EUSCI_A3_UART_ABCTL_UCDELIM__0_1_BIT_TIME          (0x0000)              /* 1 bit time */
#define EUSCI_A3_UART_ABCTL_UCDELIM__1                     (0x0010)              /* 2 bit times */
#define EUSCI_A3_UART_ABCTL_UCDELIM__1_2_BIT_TIMES         (0x0010)              /* 2 bit times */
#define EUSCI_A3_UART_ABCTL_UCDELIM__2                     (0x0020)              /* 3 bit times */
#define EUSCI_A3_UART_ABCTL_UCDELIM__2_3_BIT_TIMES         (0x0020)              /* 3 bit times */
#define EUSCI_A3_UART_ABCTL_UCDELIM__3                     (0x0030)              /* 4 bit times */
#define EUSCI_A3_UART_ABCTL_UCDELIM__3_4_BIT_TIMES         (0x0030)              /* 4 bit times */
#define EUSCI_A3_UART_ABCTL_RESERVED1__M                   (0xffc0)              /*  */

/* EUSCI_A3_UART_IRCTL Control Bits */
#define EUSCI_A3_UART_IRCTL_UCIREN                         (0x0001)              /* IrDA encoder/decoder enable */
#define EUSCI_A3_UART_IRCTL_UCIRTXCLK                      (0x0002)              /* IrDA transmit pulse clock select */
#define EUSCI_A3_UART_IRCTL_UCIRTXPL__M                    (0x00fc)              /* Transmit pulse length. Pulse length t(PULSE) = (UCIRTXPLx + 1) / [2 - f(IRTXCLK) */
                                                                                 /* ] */
#define EUSCI_A3_UART_IRCTL_UCIRRXFE                       (0x0100)              /* IrDA receive filter enabled */
#define EUSCI_A3_UART_IRCTL_UCIRRXPL                       (0x0200)              /* IrDA receive input UCAxRXD polarity */
#define EUSCI_A3_UART_IRCTL_UCIRRXFL__M                    (0xfc00)              /* Receive filter length. The minimum pulse length for receive is given by: t(MIN)  */
                                                                                 /* = (UCIRRXFLx + 4) / [2 - f(IRTXCLK)] */

/* EUSCI_A3_UART_IE Control Bits */
#define EUSCI_A3_UART_IE_UCRXIE                            (0x0001)              /* Receive interrupt enable */
#define EUSCI_A3_UART_IE_UCTXIE                            (0x0002)              /* Transmit interrupt enable */
#define EUSCI_A3_UART_IE_UCSTTIE                           (0x0004)              /* Start bit interrupt enable */
#define EUSCI_A3_UART_IE_UCTXCPTIE                         (0x0008)              /* Transmit complete interrupt enable */
#define EUSCI_A3_UART_IE_RESERVED1__M                      (0xfff0)              /*  */

/* EUSCI_A3_UART_IFG Control Bits */
#define EUSCI_A3_UART_IFG_UCRXIFG                          (0x0001)              /* Receive interrupt flag. UCRXIFG is set when UCAxRXBUF has received a complete  */
                                                                                 /* character. */
#define EUSCI_A3_UART_IFG_UCTXIFG                          (0x0002)              /* Transmit interrupt flag. UCTXIFG is set when UCAxTXBUF empty. */
#define EUSCI_A3_UART_IFG_UCSTTIFG                         (0x0004)              /* Start bit interrupt flag. UCSTTIFG is set after a Start bit was received */
#define EUSCI_A3_UART_IFG_UCTXCPTIFG                       (0x0008)              /* Transmit ready interrupt enable. UCTXRDYIFG is set when the entire byte in the  */
                                                                                 /* internal shift register got shifted out and UCAxTXBUF is empty. */
#define EUSCI_A3_UART_IFG_RESERVED1__M                     (0xfff0)              /*  */


//*****************************************************************************
// EUSCI_B0_I2C Registers
//*****************************************************************************
typedef struct {
  union {    /* EUSCI_B0_I2C_CTLW0 Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B0_I2C_CTLW0 Control Bits */
      __IO uint16_t UCSWRST             	:	1;  /* Software reset enable. Can be modified only when eUSCI_B is in reset state ( */
                                                /* UCSWRST = 1). */
      __IO uint16_t UCTXSTT             	:	1;  /* Transmit START condition in master mode. Ignored in slave mode. In master  */
                                                /* receiver mode, a repeated START condition is preceded by a NACK. UCTXSTT is  */
                                                /* automatically cleared after START condition and address information is  */
                                                /* transmitted. Ignored in slave mode. */
      __IO uint16_t UCTXSTP             	:	1;  /* Transmit STOP condition in master mode. Ignored in slave mode. In master  */
                                                /* receiver mode, the STOP condition is preceded by a NACK. UCTXSTP is  */
                                                /* automatically cleared after STOP is generated. This bit is a don't care, if  */
                                                /* automatic UCASTPx is different from 01 or 10.  */
      __IO uint16_t UCTXNACK            	:	1;  /* Transmit a NACK. UCTXNACK is automatically cleared after a NACK is transmitted.  */
                                                /* Only for slave receiver mode. */
      __IO uint16_t UCTR                	:	1;  /* Transmitter/receiver */
      __IO uint16_t UCTXACK             	:	1;  /* Transmit ACK condition in slave mode with enabled address mask register. After  */
                                                /* the UCSTTIFG has been set, the user needs to set or reset the UCTXACK flag to  */
                                                /* continue with the I2C protocol. The clock is stretched until the UCBxCTL1  */
                                                /* register has been written. This bit is cleared automatically after the ACK has  */
                                                /* been send. */
      __IO uint16_t UCSSEL              	:	2;  /* eUSCI_B clock source select. These bits select the BRCLK source clock. These  */
                                                /* bits are ignored in slave mode. Can be modified only when eUSCI_B is in reset  */
                                                /* state (UCSWRST = 1). */
      __IO uint16_t UCSYNC              	:	1;  /* Synchronous mode enable. For eUSCI_B always read and write as 1. */
      __IO uint16_t UCMODE              	:	2;  /* eUSCI_B mode. The UCMODEx bits select the synchronous mode when UCSYNC = 1. Can  */
                                                /* be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
      __IO uint16_t UCMST               	:	1;  /* Master mode select. When a master loses arbitration in a multi-master  */
                                                /* environment (UCMM = 1), the UCMST bit is automatically cleared and the module  */
                                                /* acts as slave. */
      __I  uint16_t RESERVED1           	:	1;  /* Reserved */
      __IO uint16_t UCMM                	:	1;  /* Multi-master environment select. Can be modified only when eUSCI_B is in reset  */
                                                /* state (UCSWRST = 1). */
      __IO uint16_t UCSLA10             	:	1;  /* Slave addressing mode select */
      __IO uint16_t UCA10               	:	1;  /* Own addressing mode select. Can be modified only when eUSCI_B is in reset state  */
                                                /* (UCSWRST = 1). */

    } bit;
  } CTLW0;
  union {    /* EUSCI_B0_I2C_CTLW1 Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B0_I2C_CTLW1 Control Bits */
      __IO uint16_t UCGLIT              	:	2;  /* Deglitch time */
      __IO uint16_t UCASTP              	:	2;  /* Automatic STOP condition generation. In slave mode only UCBCNTIFG is available.  */
                                                /* Can be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
      __IO uint16_t UCSWACK             	:	1;  /* Using this bit it is possible to select, whether the eUSCI_B module triggers the */
                                                /* sending of the ACK of the address or if it is controlled by software. */
      __IO uint16_t UCSTPNACK           	:	1;  /* The UCSTPNACK bit allows to make the eUSCI_B master acknowledge the last byte in */
                                                /* master receiver mode as well. This is not conform to the I2C specification and  */
                                                /* should only be used for slaves, which automatically release the SDA after a  */
                                                /* fixed packet length. Can be modified only when eUSCI_B is in reset state ( */
                                                /* UCSWRST = 1). */
      __IO uint16_t UCCLTO              	:	2;  /* Clock low timeout select. Modify only when UCSWRST = 1. */
      __IO uint16_t UCETXINT            	:	1;  /* Early UCTXIFG0. Only in slave mode. When this bit is set, the slave addresses  */
                                                /* defined in UCxI2COA1 to UCxI2COA3 must be disabled. Can be modified only when  */
                                                /* eUSCI_B is in reset state (UCSWRST = 1). */
      __I  uint16_t RESERVED1           	:	7;  /* Reserved */

    } bit;
  } CTLW1;
  uint8_t RESERVED0[2];
  uint16_t BRW; /* eUSCI_Bx Bit Rate Control Word */
  union {    /* EUSCI_B0_I2C_STATW Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B0_I2C_STATW Control Bits */
      __I  uint16_t RESERVED2           	:	4;  /* Reserved */
      __I  uint16_t UCBBUSY             	:	1;  /* Bus busy */
      __I  uint16_t UCGC                	:	1;  /* General call address received. UCGC is automatically cleared when a START  */
                                                /* condition is received. */
      __I  uint16_t UCSCLLOW            	:	1;  /* SCL low */
      __I  uint16_t RESERVED1           	:	1;  /* Reserved */
      __I  uint16_t UCBCNT              	:	8;  /* Hardware byte counter value. Reading this register returns the number of bytes  */
                                                /* received or transmitted on the I2C-Bus since the last START or RESTART. There is */
                                                /* no synchronization of this register done. When reading UCBxBCNT during the  */
                                                /* first bit position, a faulty readback can occur. */

    } bit;
  } STATW;
  union {    /* EUSCI_B0_I2C_TBCNT Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B0_I2C_TBCNT Control Bits */
      __IO uint16_t UCTBCNT             	:	8;  /* The byte counter threshold value is used to set the number of I2C data bytes  */
                                                /* after which the automatic STOP or the UCSTPIFG should occur. This value is  */
                                                /* evaluated only if UCASTPx is different from 00. Can be modified only when  */
                                                /* eUSCI_B is in reset state (UCSWRST = 1). */
      __I  uint16_t RESERVED1           	:	8;  /* Reserved */

    } bit;
  } TBCNT;
  union {    /* EUSCI_B0_I2C_RXBUF Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B0_I2C_RXBUF Control Bits */
      __I  uint16_t UCRXBUF             	:	8;  /* The receive-data buffer is user accessible and contains the last received  */
                                                /* character from the receive shift register. Reading UCBxRXBUF resets the UCRXIFGx */
                                                /* flags. */
      __I  uint16_t RESERVED1           	:	8;  /* Reserved */

    } bit;
  } RXBUF;
  union {    /* EUSCI_B0_I2C_TXBUF Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B0_I2C_TXBUF Control Bits */
      __IO uint16_t UCTXBUF             	:	8;  /* The transmit data buffer is user accessible and holds the data waiting to be  */
                                                /* moved into the transmit shift register and transmitted. Writing to the transmit  */
                                                /* data buffer clears the UCTXIFGx flags.  */
      __I  uint16_t RESERVED1           	:	8;  /* Reserved */

    } bit;
  } TXBUF;
  uint8_t RESERVED1[4];
  union {    /* EUSCI_B0_I2C_I2COA0 Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B0_I2C_I2COA0 Control Bits */
      __IO uint16_t I2COA               	:	10;  /* I2C own address. The I2COA0 bits contain the local address of the eUSCIx_B I2C  */
                                                /* controller. The address is right justified. In 7-bit addressing mode, bit 6 is  */
                                                /* the MSB and bits 9-7 are ignored. In 10-bit addressing mode, bit 9 is the MSB.  */
                                                /* Can be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
      __IO uint16_t UCOAEN              	:	1;  /* Own Address enable register. With this register it can be selected if the I2C  */
                                                /* slave-address related to this register UCBxI2COA0 is evaluated or not. Can be  */
                                                /* modified only when eUSCI_B is in reset state (UCSWRST = 1). */
      __I  uint16_t RESERVED1           	:	4;  /* Reserved */
      __IO uint16_t UCGCEN              	:	1;  /* General call response enable. This bit is only available in UCBxI2COA0. Can be  */
                                                /* modified only when eUSCI_B is in reset state (UCSWRST = 1). */

    } bit;
  } I2COA0;
  union {    /* EUSCI_B0_I2C_I2COA1 Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B0_I2C_I2COA1 Control Bits */
      __IO uint16_t I2COA1              	:	10;  /* I2C own address. The I2COAx bits contain the local address of the eUSCIx_B I2C  */
                                                /* controller. The address is right justified. In 7-bit addressing mode, bit 6 is  */
                                                /* the MSB and bits 9-7 are ignored. In 10-bit addressing mode, bit 9 is the MSB.  */
                                                /* Can be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
      __IO uint16_t UCOAEN              	:	1;  /* Own Address enable register. With this register it can be selected if the I2C  */
                                                /* slave-address related to this register UCBxI2COA1 is evaluated or not. Can be  */
                                                /* modified only when eUSCI_B is in reset state (UCSWRST = 1). */
      __I  uint16_t RESERVED1           	:	5;  /* Reserved */

    } bit;
  } I2COA1;
  union {    /* EUSCI_B0_I2C_I2COA2 Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B0_I2C_I2COA2 Control Bits */
      __IO uint16_t I2COA2              	:	10;  /* I2C own address. The I2COAx bits contain the local address of the eUSCIx_B I2C  */
                                                /* controller. The address is right justified. In 7-bit addressing mode, bit 6 is  */
                                                /* the MSB and bits 9-7 are ignored. In 10-bit addressing mode, bit 9 is the MSB.  */
                                                /* Can be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
      __IO uint16_t UCOAEN              	:	1;  /* Own Address enable register. With this register it can be selected if the I2C  */
                                                /* slave-address related to this register UCBxI2COA2 is evaluated or not. Can be  */
                                                /* modified only when eUSCI_B is in reset state (UCSWRST = 1). */
      __I  uint16_t RESERVED1           	:	5;  /* Reserved */

    } bit;
  } I2COA2;
  union {    /* EUSCI_B0_I2C_I2COA3 Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B0_I2C_I2COA3 Control Bits */
      __IO uint16_t I2COA3              	:	10;  /* I2C own address. The I2COA3 bits contain the local address of the eUSCIx_B I2C  */
                                                /* controller. The address is right justified. In 7-bit addressing mode, bit 6 is  */
                                                /* the MSB and bits 9-7 are ignored. In 10-bit addressing mode, bit 9 is the MSB.  */
                                                /* Can be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
      __IO uint16_t UCOAEN              	:	1;  /* Own Address enable register. With this register it can be selected if the I2C  */
                                                /* slave-address related to this register UCBxI2COA3 is evaluated or not. Can be  */
                                                /* modified only when eUSCI_B is in reset state (UCSWRST = 1). */
      __I  uint16_t RESERVED1           	:	5;  /* Reserved */

    } bit;
  } I2COA3;
  union {    /* EUSCI_B0_I2C_ADDRX Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B0_I2C_ADDRX Control Bits */
      __I  uint16_t ADDRX               	:	10;  /* Received Address Register. This register contains the last received slave  */
                                                /* address on the bus. Using this register and the address mask register it is  */
                                                /* possible to react on more than one slave address using one eUSCI_B module. */
      __I  uint16_t RESERVED1           	:	6;  /* Reserved */

    } bit;
  } ADDRX;
  union {    /* EUSCI_B0_I2C_ADDMASK Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B0_I2C_ADDMASK Control Bits */
      __IO uint16_t ADDMASK             	:	10;  /* Address Mask Register. By clearing the corresponding bit of the own address,  */
                                                /* this bit is a don't care when comparing the address on the bus to the own  */
                                                /* address. Using this method, it is possible to react on more than one slave  */
                                                /* address. When all bits of ADDMASKx are set, the address mask feature is  */
                                                /* deactivated. Can be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
      __I  uint16_t RESERVED1           	:	6;  /* Reserved */

    } bit;
  } ADDMASK;
  union {    /* EUSCI_B0_I2C_I2CSA Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B0_I2C_I2CSA Control Bits */
      __IO uint16_t I2CSA               	:	10;  /* I2C slave address. The I2CSAx bits contain the slave address of the external  */
                                                /* device to be addressed by the eUSCIx_B module. It is only used in master mode.  */
                                                /* The address is right justified. In 7-bit slave addressing mode, bit 6 is the MSB */
                                                /* and bits 9-7 are ignored. In 10-bit slave addressing mode, bit 9 is the MSB. */
      __I  uint16_t RESERVED1           	:	6;  /* Reserved */

    } bit;
  } I2CSA;
  uint8_t RESERVED2[8];
  union {    /* EUSCI_B0_I2C_IE Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B0_I2C_IE Control Bits */
      __IO uint16_t UCRXIE0             	:	1;  /* Receive interrupt enable 0 */
      __IO uint16_t UCTXIE0             	:	1;  /* Transmit interrupt enable 0 */
      __IO uint16_t UCSTTIE             	:	1;  /* START condition interrupt enable */
      __IO uint16_t UCSTPIE             	:	1;  /* STOP condition interrupt enable */
      __IO uint16_t UCALIE              	:	1;  /* Arbitration lost interrupt enable */
      __IO uint16_t UCNACKIE            	:	1;  /* Not-acknowledge interrupt enable */
      __IO uint16_t UCBCNTIE            	:	1;  /* Byte counter interrupt enable. */
      __IO uint16_t UCCLTOIE            	:	1;  /* Clock low timeout interrupt enable. */
      __IO uint16_t UCRXIE1             	:	1;  /* Receive interrupt enable 1 */
      __IO uint16_t UCTXIE1             	:	1;  /* Transmit interrupt enable 1 */
      __IO uint16_t UCRXIE2             	:	1;  /* Receive interrupt enable 2 */
      __IO uint16_t UCTXIE2             	:	1;  /* Transmit interrupt enable 2 */
      __IO uint16_t UCRXIE3             	:	1;  /* Receive interrupt enable 3 */
      __IO uint16_t UCTXIE3             	:	1;  /* Transmit interrupt enable 3 */
      __IO uint16_t UCBIT9IE            	:	1;  /* Bit position 9 interrupt enable */
      __I  uint16_t RESERVED1           	:	1;  /* Reserved */

    } bit;
  } IE;
  union {    /* EUSCI_B0_I2C_IFG Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B0_I2C_IFG Control Bits */
      __IO uint16_t UCRXIFG0            	:	1;  /* eUSCI_B receive interrupt flag 0. UCRXIFG0 is set when UCBxRXBUF has received a  */
                                                /* complete character in master mode or in slave mode, if the slave address defined */
                                                /* in UCBxI2COA0 was on the bus in the same frame. */
      __IO uint16_t UCTXIFG0            	:	1;  /* eUSCI_B transmit interrupt flag 0. UCTXIFG0 is set when UCBxTXBUF is empty in  */
                                                /* master mode or in slave mode, if the slave address defined in UCBxI2COA0 was on  */
                                                /* the bus in the same frame. */
      __IO uint16_t UCSTTIFG            	:	1;  /* START condition interrupt flag */
      __IO uint16_t UCSTPIFG            	:	1;  /* STOP condition interrupt flag */
      __IO uint16_t UCALIFG             	:	1;  /* Arbitration lost interrupt flag */
      __IO uint16_t UCNACKIFG           	:	1;  /* Not-acknowledge received interrupt flag. This flag only is updated when  */
                                                /* operating in master mode. */
      __IO uint16_t UCBCNTIFG           	:	1;  /* Byte counter interrupt flag. When using this interrupt the user needs to ensure  */
                                                /* enough processing bandwidth (see the Byte Counter Interrupt section). */
      __IO uint16_t UCCLTOIFG           	:	1;  /* Clock low timeout interrupt flag */
      __IO uint16_t UCRXIFG1            	:	1;  /* Receive interrupt flag 1. UCRXIFG1 is set when UCBxRXBUF has received a complete */
                                                /* byte in slave mode and if the slave address defined in UCBxI2COA1 was on the  */
                                                /* bus in the same frame. */
      __IO uint16_t UCTXIFG1            	:	1;  /* eUSCI_B transmit interrupt flag 1. UCTXIFG1 is set when UCBxTXBUF is empty in  */
                                                /* slave mode, if the slave address defined in UCBxI2COA1 was on the bus in the  */
                                                /* same frame. */
      __IO uint16_t UCRXIFG2            	:	1;  /* Receive interrupt flag 2. UCRXIFG2 is set when UCBxRXBUF has received a complete */
                                                /* byte in slave mode and if the slave address defined in UCBxI2COA2 was on the  */
                                                /* bus in the same frame. */
      __IO uint16_t UCTXIFG2            	:	1;  /* eUSCI_B transmit interrupt flag 2. UCTXIFG2 is set when UCBxTXBUF is empty in  */
                                                /* slave mode, if the slave address defined in UCBxI2COA2 was on the bus in the  */
                                                /* same frame. */
      __IO uint16_t UCRXIFG3            	:	1;  /* Receive interrupt flag 2. UCRXIFG2 is set when UCBxRXBUF has received a complete */
                                                /* byte in slave mode and if the slave address defined in UCBxI2COA2 was on the  */
                                                /* bus in the same frame. */
      __IO uint16_t UCTXIFG3            	:	1;  /* eUSCI_B transmit interrupt flag 3. UCTXIFG3 is set when UCBxTXBUF is empty in  */
                                                /* slave mode, if the slave address defined in UCBxI2COA3 was on the bus in the  */
                                                /* same frame. */
      __IO uint16_t UCBIT9IFG           	:	1;  /* Bit position 9 interrupt flag */
      __I  uint16_t RESERVED1           	:	1;  /* Reserved */

    } bit;
  } IFG;
  uint16_t IV; /* eUSCI_Bx Interrupt Vector */
} EUSCI_B0_I2C_Type;


/* EUSCI_B0_I2C_CTLW0 Control Bits */
#define EUSCI_B0_I2C_CTLW0_UCSWRST                         (0x0001)              /* Software reset enable. Can be modified only when eUSCI_B is in reset state ( */
                                                                                 /* UCSWRST = 1). */
#define EUSCI_B0_I2C_CTLW0_UCTXSTT                         (0x0002)              /* Transmit START condition in master mode. Ignored in slave mode. In master  */
                                                                                 /* receiver mode, a repeated START condition is preceded by a NACK. UCTXSTT is  */
                                                                                 /* automatically cleared after START condition and address information is  */
                                                                                 /* transmitted. Ignored in slave mode. */
#define EUSCI_B0_I2C_CTLW0_UCTXSTP                         (0x0004)              /* Transmit STOP condition in master mode. Ignored in slave mode. In master  */
                                                                                 /* receiver mode, the STOP condition is preceded by a NACK. UCTXSTP is  */
                                                                                 /* automatically cleared after STOP is generated. This bit is a don't care, if  */
                                                                                 /* automatic UCASTPx is different from 01 or 10.  */
#define EUSCI_B0_I2C_CTLW0_UCTXNACK                        (0x0008)              /* Transmit a NACK. UCTXNACK is automatically cleared after a NACK is transmitted.  */
                                                                                 /* Only for slave receiver mode. */
#define EUSCI_B0_I2C_CTLW0_UCTR                            (0x0010)              /* Transmitter/receiver */
#define EUSCI_B0_I2C_CTLW0_UCTXACK                         (0x0020)              /* Transmit ACK condition in slave mode with enabled address mask register. After  */
                                                                                 /* the UCSTTIFG has been set, the user needs to set or reset the UCTXACK flag to  */
                                                                                 /* continue with the I2C protocol. The clock is stretched until the UCBxCTL1  */
                                                                                 /* register has been written. This bit is cleared automatically after the ACK has  */
                                                                                 /* been send. */
#define EUSCI_B0_I2C_CTLW0_UCSSEL__M                       (0x00c0)              /* eUSCI_B clock source select. These bits select the BRCLK source clock. These  */
                                                                                 /* bits are ignored in slave mode. Can be modified only when eUSCI_B is in reset  */
                                                                                 /* state (UCSWRST = 1). */
#define EUSCI_B0_I2C_CTLW0_UCSSEL__0                       (0x0000)              /* UCLKI */
#define EUSCI_B0_I2C_CTLW0_UCSSEL__0_UCLKI                 (0x0000)              /* UCLKI */
#define EUSCI_B0_I2C_CTLW0_UCSSEL__1                       (0x0040)              /* ACLK */
#define EUSCI_B0_I2C_CTLW0_UCSSEL__1_ACLK                  (0x0040)              /* ACLK */
#define EUSCI_B0_I2C_CTLW0_UCSSEL__2                       (0x0080)              /* SMCLK */
#define EUSCI_B0_I2C_CTLW0_UCSSEL__2_SMCLK                 (0x0080)              /* SMCLK */
#define EUSCI_B0_I2C_CTLW0_UCSSEL__3                       (0x00c0)              /* SMCLK */
#define EUSCI_B0_I2C_CTLW0_UCSSEL__3_SMCLK                 (0x00c0)              /* SMCLK */
#define EUSCI_B0_I2C_CTLW0_UCSYNC                          (0x0100)              /* Synchronous mode enable. For eUSCI_B always read and write as 1. */
#define EUSCI_B0_I2C_CTLW0_UCMODE__M                       (0x0600)              /* eUSCI_B mode. The UCMODEx bits select the synchronous mode when UCSYNC = 1. Can  */
                                                                                 /* be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
#define EUSCI_B0_I2C_CTLW0_UCMODE__0                       (0x0000)              /* 3-pin SPI */
#define EUSCI_B0_I2C_CTLW0_UCMODE__0_3_PIN_SPI             (0x0000)              /* 3-pin SPI */
#define EUSCI_B0_I2C_CTLW0_UCMODE__1                       (0x0200)              /* 4-pin SPI (master/slave enabled if STE = 1) */
#define EUSCI_B0_I2C_CTLW0_UCMODE__2                       (0x0400)              /* 4-pin SPI (master/slave enabled if STE = 0) */
#define EUSCI_B0_I2C_CTLW0_UCMODE__3                       (0x0600)              /* I2C mode */
#define EUSCI_B0_I2C_CTLW0_UCMODE__3_I2C_MODE              (0x0600)              /* I2C mode */
#define EUSCI_B0_I2C_CTLW0_UCMST                           (0x0800)              /* Master mode select. When a master loses arbitration in a multi-master  */
                                                                                 /* environment (UCMM = 1), the UCMST bit is automatically cleared and the module  */
                                                                                 /* acts as slave. */
#define EUSCI_B0_I2C_CTLW0_RESERVED1                       (0x1000)              /*  */
#define EUSCI_B0_I2C_CTLW0_UCMM                            (0x2000)              /* Multi-master environment select. Can be modified only when eUSCI_B is in reset  */
                                                                                 /* state (UCSWRST = 1). */
#define EUSCI_B0_I2C_CTLW0_UCSLA10                         (0x4000)              /* Slave addressing mode select */
#define EUSCI_B0_I2C_CTLW0_UCA10                           (0x8000)              /* Own addressing mode select. Can be modified only when eUSCI_B is in reset state  */
                                                                                 /* (UCSWRST = 1). */

/* EUSCI_B0_I2C_CTLW1 Control Bits */
#define EUSCI_B0_I2C_CTLW1_UCGLIT__M                       (0x0003)              /* Deglitch time */
#define EUSCI_B0_I2C_CTLW1_UCGLIT__0                       (0x0000)              /* 50 ns */
#define EUSCI_B0_I2C_CTLW1_UCGLIT__0_50_NS                 (0x0000)              /* 50 ns */
#define EUSCI_B0_I2C_CTLW1_UCGLIT__1                       (0x0001)              /* 25 ns */
#define EUSCI_B0_I2C_CTLW1_UCGLIT__1_25_NS                 (0x0001)              /* 25 ns */
#define EUSCI_B0_I2C_CTLW1_UCGLIT__2                       (0x0002)              /* 12.5 ns */
#define EUSCI_B0_I2C_CTLW1_UCGLIT__2_12_5_NS               (0x0002)              /* 12.5 ns */
#define EUSCI_B0_I2C_CTLW1_UCGLIT__3                       (0x0003)              /* 6.25 ns */
#define EUSCI_B0_I2C_CTLW1_UCGLIT__3_6_25_NS               (0x0003)              /* 6.25 ns */
#define EUSCI_B0_I2C_CTLW1_UCASTP__M                       (0x000c)              /* Automatic STOP condition generation. In slave mode only UCBCNTIFG is available.  */
                                                                                 /* Can be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
#define EUSCI_B0_I2C_CTLW1_UCASTP__0                       (0x0000)              /* No automatic STOP generation. The STOP condition is generated after the user sets the UCTXSTP bit. The value in UCBxTBCNT is a don't care. */
#define EUSCI_B0_I2C_CTLW1_UCASTP__1                       (0x0004)              /* UCBCNTIFG is set with the byte counter reaches the threshold defined in UCBxTBCNT. */
#define EUSCI_B0_I2C_CTLW1_UCASTP__2                       (0x0008)              /* A STOP condition is generated automatically after the byte counter value reached UCBxTBCNT. UCBCNTIFG is set with the byte counter reaching the threshold. */
#define EUSCI_B0_I2C_CTLW1_UCSWACK                         (0x0010)              /* Using this bit it is possible to select, whether the eUSCI_B module triggers the */
                                                                                 /* sending of the ACK of the address or if it is controlled by software. */
#define EUSCI_B0_I2C_CTLW1_UCSTPNACK                       (0x0020)              /* The UCSTPNACK bit allows to make the eUSCI_B master acknowledge the last byte in */
                                                                                 /* master receiver mode as well. This is not conform to the I2C specification and  */
                                                                                 /* should only be used for slaves, which automatically release the SDA after a  */
                                                                                 /* fixed packet length. Can be modified only when eUSCI_B is in reset state ( */
                                                                                 /* UCSWRST = 1). */
#define EUSCI_B0_I2C_CTLW1_UCCLTO__M                       (0x00c0)              /* Clock low timeout select. Modify only when UCSWRST = 1. */
#define EUSCI_B0_I2C_CTLW1_UCCLTO__0                       (0x0000)              /* Disable clock low timeout counter */
#define EUSCI_B0_I2C_CTLW1_UCCLTO__1                       (0x0040)              /* 135 000 SYSCLK cycles (approximately 28 ms) */
#define EUSCI_B0_I2C_CTLW1_UCCLTO__2                       (0x0080)              /* 150 000 SYSCLK cycles (approximately 31 ms) */
#define EUSCI_B0_I2C_CTLW1_UCCLTO__3                       (0x00c0)              /* 165 000 SYSCLK cycles (approximately 34 ms) */
#define EUSCI_B0_I2C_CTLW1_UCETXINT                        (0x0100)              /* Early UCTXIFG0. Only in slave mode. When this bit is set, the slave addresses  */
                                                                                 /* defined in UCxI2COA1 to UCxI2COA3 must be disabled. Can be modified only when  */
                                                                                 /* eUSCI_B is in reset state (UCSWRST = 1). */
#define EUSCI_B0_I2C_CTLW1_RESERVED1__M                    (0xfe00)              /*  */

/* EUSCI_B0_I2C_STATW Control Bits */
#define EUSCI_B0_I2C_STATW_RESERVED2__M                    (0x000f)              /*  */
#define EUSCI_B0_I2C_STATW_UCBBUSY                         (0x0010)              /* Bus busy */
#define EUSCI_B0_I2C_STATW_UCGC                            (0x0020)              /* General call address received. UCGC is automatically cleared when a START  */
                                                                                 /* condition is received. */
#define EUSCI_B0_I2C_STATW_UCSCLLOW                        (0x0040)              /* SCL low */
#define EUSCI_B0_I2C_STATW_RESERVED1                       (0x0080)              /*  */
#define EUSCI_B0_I2C_STATW_UCBCNT__M                       (0xff00)              /* Hardware byte counter value. Reading this register returns the number of bytes  */
                                                                                 /* received or transmitted on the I2C-Bus since the last START or RESTART. There is */
                                                                                 /* no synchronization of this register done. When reading UCBxBCNT during the  */
                                                                                 /* first bit position, a faulty readback can occur. */

/* EUSCI_B0_I2C_TBCNT Control Bits */
#define EUSCI_B0_I2C_TBCNT_UCTBCNT__M                      (0x00ff)              /* The byte counter threshold value is used to set the number of I2C data bytes  */
                                                                                 /* after which the automatic STOP or the UCSTPIFG should occur. This value is  */
                                                                                 /* evaluated only if UCASTPx is different from 00. Can be modified only when  */
                                                                                 /* eUSCI_B is in reset state (UCSWRST = 1). */
#define EUSCI_B0_I2C_TBCNT_RESERVED1__M                    (0xff00)              /*  */

/* EUSCI_B0_I2C_RXBUF Control Bits */
#define EUSCI_B0_I2C_RXBUF_UCRXBUF__M                      (0x00ff)              /* The receive-data buffer is user accessible and contains the last received  */
                                                                                 /* character from the receive shift register. Reading UCBxRXBUF resets the UCRXIFGx */
                                                                                 /* flags. */
#define EUSCI_B0_I2C_RXBUF_RESERVED1__M                    (0xff00)              /*  */

/* EUSCI_B0_I2C_TXBUF Control Bits */
#define EUSCI_B0_I2C_TXBUF_UCTXBUF__M                      (0x00ff)              /* The transmit data buffer is user accessible and holds the data waiting to be  */
                                                                                 /* moved into the transmit shift register and transmitted. Writing to the transmit  */
                                                                                 /* data buffer clears the UCTXIFGx flags.  */
#define EUSCI_B0_I2C_TXBUF_RESERVED1__M                    (0xff00)              /*  */

/* EUSCI_B0_I2C_I2COA0 Control Bits */
#define EUSCI_B0_I2C_I2COA0_I2COA__M                       (0x03ff)              /* I2C own address. The I2COA0 bits contain the local address of the eUSCIx_B I2C  */
                                                                                 /* controller. The address is right justified. In 7-bit addressing mode, bit 6 is  */
                                                                                 /* the MSB and bits 9-7 are ignored. In 10-bit addressing mode, bit 9 is the MSB.  */
                                                                                 /* Can be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
#define EUSCI_B0_I2C_I2COA0_UCOAEN                         (0x0400)              /* Own Address enable register. With this register it can be selected if the I2C  */
                                                                                 /* slave-address related to this register UCBxI2COA0 is evaluated or not. Can be  */
                                                                                 /* modified only when eUSCI_B is in reset state (UCSWRST = 1). */
#define EUSCI_B0_I2C_I2COA0_RESERVED1__M                   (0x7800)              /*  */
#define EUSCI_B0_I2C_I2COA0_UCGCEN                         (0x8000)              /* General call response enable. This bit is only available in UCBxI2COA0. Can be  */
                                                                                 /* modified only when eUSCI_B is in reset state (UCSWRST = 1). */

/* EUSCI_B0_I2C_I2COA1 Control Bits */
#define EUSCI_B0_I2C_I2COA1_I2COA1__M                      (0x03ff)              /* I2C own address. The I2COAx bits contain the local address of the eUSCIx_B I2C  */
                                                                                 /* controller. The address is right justified. In 7-bit addressing mode, bit 6 is  */
                                                                                 /* the MSB and bits 9-7 are ignored. In 10-bit addressing mode, bit 9 is the MSB.  */
                                                                                 /* Can be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
#define EUSCI_B0_I2C_I2COA1_UCOAEN                         (0x0400)              /* Own Address enable register. With this register it can be selected if the I2C  */
                                                                                 /* slave-address related to this register UCBxI2COA1 is evaluated or not. Can be  */
                                                                                 /* modified only when eUSCI_B is in reset state (UCSWRST = 1). */
#define EUSCI_B0_I2C_I2COA1_RESERVED1__M                   (0xf800)              /*  */

/* EUSCI_B0_I2C_I2COA2 Control Bits */
#define EUSCI_B0_I2C_I2COA2_I2COA2__M                      (0x03ff)              /* I2C own address. The I2COAx bits contain the local address of the eUSCIx_B I2C  */
                                                                                 /* controller. The address is right justified. In 7-bit addressing mode, bit 6 is  */
                                                                                 /* the MSB and bits 9-7 are ignored. In 10-bit addressing mode, bit 9 is the MSB.  */
                                                                                 /* Can be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
#define EUSCI_B0_I2C_I2COA2_UCOAEN                         (0x0400)              /* Own Address enable register. With this register it can be selected if the I2C  */
                                                                                 /* slave-address related to this register UCBxI2COA2 is evaluated or not. Can be  */
                                                                                 /* modified only when eUSCI_B is in reset state (UCSWRST = 1). */
#define EUSCI_B0_I2C_I2COA2_RESERVED1__M                   (0xf800)              /*  */

/* EUSCI_B0_I2C_I2COA3 Control Bits */
#define EUSCI_B0_I2C_I2COA3_I2COA3__M                      (0x03ff)              /* I2C own address. The I2COA3 bits contain the local address of the eUSCIx_B I2C  */
                                                                                 /* controller. The address is right justified. In 7-bit addressing mode, bit 6 is  */
                                                                                 /* the MSB and bits 9-7 are ignored. In 10-bit addressing mode, bit 9 is the MSB.  */
                                                                                 /* Can be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
#define EUSCI_B0_I2C_I2COA3_UCOAEN                         (0x0400)              /* Own Address enable register. With this register it can be selected if the I2C  */
                                                                                 /* slave-address related to this register UCBxI2COA3 is evaluated or not. Can be  */
                                                                                 /* modified only when eUSCI_B is in reset state (UCSWRST = 1). */
#define EUSCI_B0_I2C_I2COA3_RESERVED1__M                   (0xf800)              /*  */

/* EUSCI_B0_I2C_ADDRX Control Bits */
#define EUSCI_B0_I2C_ADDRX_ADDRX__M                        (0x03ff)              /* Received Address Register. This register contains the last received slave  */
                                                                                 /* address on the bus. Using this register and the address mask register it is  */
                                                                                 /* possible to react on more than one slave address using one eUSCI_B module. */
#define EUSCI_B0_I2C_ADDRX_RESERVED1__M                    (0xfc00)              /*  */

/* EUSCI_B0_I2C_ADDMASK Control Bits */
#define EUSCI_B0_I2C_ADDMASK_ADDMASK__M                    (0x03ff)              /* Address Mask Register. By clearing the corresponding bit of the own address,  */
                                                                                 /* this bit is a don't care when comparing the address on the bus to the own  */
                                                                                 /* address. Using this method, it is possible to react on more than one slave  */
                                                                                 /* address. When all bits of ADDMASKx are set, the address mask feature is  */
                                                                                 /* deactivated. Can be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
#define EUSCI_B0_I2C_ADDMASK_RESERVED1__M                  (0xfc00)              /*  */

/* EUSCI_B0_I2C_I2CSA Control Bits */
#define EUSCI_B0_I2C_I2CSA_I2CSA__M                        (0x03ff)              /* I2C slave address. The I2CSAx bits contain the slave address of the external  */
                                                                                 /* device to be addressed by the eUSCIx_B module. It is only used in master mode.  */
                                                                                 /* The address is right justified. In 7-bit slave addressing mode, bit 6 is the MSB */
                                                                                 /* and bits 9-7 are ignored. In 10-bit slave addressing mode, bit 9 is the MSB. */
#define EUSCI_B0_I2C_I2CSA_RESERVED1__M                    (0xfc00)              /*  */

/* EUSCI_B0_I2C_IE Control Bits */
#define EUSCI_B0_I2C_IE_UCRXIE0                            (0x0001)              /* Receive interrupt enable 0 */
#define EUSCI_B0_I2C_IE_UCTXIE0                            (0x0002)              /* Transmit interrupt enable 0 */
#define EUSCI_B0_I2C_IE_UCSTTIE                            (0x0004)              /* START condition interrupt enable */
#define EUSCI_B0_I2C_IE_UCSTPIE                            (0x0008)              /* STOP condition interrupt enable */
#define EUSCI_B0_I2C_IE_UCALIE                             (0x0010)              /* Arbitration lost interrupt enable */
#define EUSCI_B0_I2C_IE_UCNACKIE                           (0x0020)              /* Not-acknowledge interrupt enable */
#define EUSCI_B0_I2C_IE_UCBCNTIE                           (0x0040)              /* Byte counter interrupt enable. */
#define EUSCI_B0_I2C_IE_UCCLTOIE                           (0x0080)              /* Clock low timeout interrupt enable. */
#define EUSCI_B0_I2C_IE_UCRXIE1                            (0x0100)              /* Receive interrupt enable 1 */
#define EUSCI_B0_I2C_IE_UCTXIE1                            (0x0200)              /* Transmit interrupt enable 1 */
#define EUSCI_B0_I2C_IE_UCRXIE2                            (0x0400)              /* Receive interrupt enable 2 */
#define EUSCI_B0_I2C_IE_UCTXIE2                            (0x0800)              /* Transmit interrupt enable 2 */
#define EUSCI_B0_I2C_IE_UCRXIE3                            (0x1000)              /* Receive interrupt enable 3 */
#define EUSCI_B0_I2C_IE_UCTXIE3                            (0x2000)              /* Transmit interrupt enable 3 */
#define EUSCI_B0_I2C_IE_UCBIT9IE                           (0x4000)              /* Bit position 9 interrupt enable */
#define EUSCI_B0_I2C_IE_RESERVED1                          (0x8000)              /*  */

/* EUSCI_B0_I2C_IFG Control Bits */
#define EUSCI_B0_I2C_IFG_UCRXIFG0                          (0x0001)              /* eUSCI_B receive interrupt flag 0. UCRXIFG0 is set when UCBxRXBUF has received a  */
                                                                                 /* complete character in master mode or in slave mode, if the slave address defined */
                                                                                 /* in UCBxI2COA0 was on the bus in the same frame. */
#define EUSCI_B0_I2C_IFG_UCTXIFG0                          (0x0002)              /* eUSCI_B transmit interrupt flag 0. UCTXIFG0 is set when UCBxTXBUF is empty in  */
                                                                                 /* master mode or in slave mode, if the slave address defined in UCBxI2COA0 was on  */
                                                                                 /* the bus in the same frame. */
#define EUSCI_B0_I2C_IFG_UCSTTIFG                          (0x0004)              /* START condition interrupt flag */
#define EUSCI_B0_I2C_IFG_UCSTPIFG                          (0x0008)              /* STOP condition interrupt flag */
#define EUSCI_B0_I2C_IFG_UCALIFG                           (0x0010)              /* Arbitration lost interrupt flag */
#define EUSCI_B0_I2C_IFG_UCNACKIFG                         (0x0020)              /* Not-acknowledge received interrupt flag. This flag only is updated when  */
                                                                                 /* operating in master mode. */
#define EUSCI_B0_I2C_IFG_UCBCNTIFG                         (0x0040)              /* Byte counter interrupt flag. When using this interrupt the user needs to ensure  */
                                                                                 /* enough processing bandwidth (see the Byte Counter Interrupt section). */
#define EUSCI_B0_I2C_IFG_UCCLTOIFG                         (0x0080)              /* Clock low timeout interrupt flag */
#define EUSCI_B0_I2C_IFG_UCRXIFG1                          (0x0100)              /* Receive interrupt flag 1. UCRXIFG1 is set when UCBxRXBUF has received a complete */
                                                                                 /* byte in slave mode and if the slave address defined in UCBxI2COA1 was on the  */
                                                                                 /* bus in the same frame. */
#define EUSCI_B0_I2C_IFG_UCTXIFG1                          (0x0200)              /* eUSCI_B transmit interrupt flag 1. UCTXIFG1 is set when UCBxTXBUF is empty in  */
                                                                                 /* slave mode, if the slave address defined in UCBxI2COA1 was on the bus in the  */
                                                                                 /* same frame. */
#define EUSCI_B0_I2C_IFG_UCRXIFG2                          (0x0400)              /* Receive interrupt flag 2. UCRXIFG2 is set when UCBxRXBUF has received a complete */
                                                                                 /* byte in slave mode and if the slave address defined in UCBxI2COA2 was on the  */
                                                                                 /* bus in the same frame. */
#define EUSCI_B0_I2C_IFG_UCTXIFG2                          (0x0800)              /* eUSCI_B transmit interrupt flag 2. UCTXIFG2 is set when UCBxTXBUF is empty in  */
                                                                                 /* slave mode, if the slave address defined in UCBxI2COA2 was on the bus in the  */
                                                                                 /* same frame. */
#define EUSCI_B0_I2C_IFG_UCRXIFG3                          (0x1000)              /* Receive interrupt flag 2. UCRXIFG2 is set when UCBxRXBUF has received a complete */
                                                                                 /* byte in slave mode and if the slave address defined in UCBxI2COA2 was on the  */
                                                                                 /* bus in the same frame. */
#define EUSCI_B0_I2C_IFG_UCTXIFG3                          (0x2000)              /* eUSCI_B transmit interrupt flag 3. UCTXIFG3 is set when UCBxTXBUF is empty in  */
                                                                                 /* slave mode, if the slave address defined in UCBxI2COA3 was on the bus in the  */
                                                                                 /* same frame. */
#define EUSCI_B0_I2C_IFG_UCBIT9IFG                         (0x4000)              /* Bit position 9 interrupt flag */
#define EUSCI_B0_I2C_IFG_RESERVED1                         (0x8000)              /*  */


//*****************************************************************************
// EUSCI_B0_SPI Registers
//*****************************************************************************
typedef struct {
  union {    /* EUSCI_B0_SPI_CTLW0 Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B0_SPI_CTLW0 Control Bits */
      __IO uint16_t UCSWRST             	:	1;  /* Software reset enable */
      __IO uint16_t UCSTEM              	:	1;  /* STE mode select in master mode. This byte is ignored in slave or 3-wire mode. */
      __I  uint16_t RESERVED1           	:	4;  /* Reserved */
      __IO uint16_t UCSSEL              	:	2;  /* eUSCI clock source select. These bits select the BRCLK source clock in master  */
                                                /* mode. UCxCLK is always used in slave mode. */
      __IO uint16_t UCSYNC              	:	1;  /* Synchronous mode enable */
      __IO uint16_t UCMODE              	:	2;  /* eUSCI mode. The UCMODEx bits select the synchronous mode when UCSYNC = 1. */
      __IO uint16_t UCMST               	:	1;  /* Master mode select */
      __IO uint16_t UC7BIT              	:	1;  /* Character length. Selects 7-bit or 8-bit character length. */
      __IO uint16_t UCMSB               	:	1;  /* MSB first select. Controls the direction of the receive and transmit shift  */
                                                /* register. */
      __IO uint16_t UCCKPL              	:	1;  /* Clock polarity select */
      __IO uint16_t UCCKPH              	:	1;  /* Clock phase select */

    } bit;
  } CTLW0;
  uint8_t RESERVED0[4];
  uint16_t BRW; /* eUSCI_Bx Bit Rate Control Word */
  union {    /* EUSCI_B0_SPI_STATW Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B0_SPI_STATW Control Bits */
      __I  uint16_t UCBUSY              	:	1;  /* eUSCI busy. This bit indicates if a transmit or receive operation is in progress */
                                                /* . */
      __I  uint16_t RESERVED2           	:	4;  /* Reserved */
      __IO uint16_t UCOE                	:	1;  /* Overrun error flag. This bit is set when a character is transferred into  */
                                                /* UCxRXBUF before the previous character was read. UCOE is cleared automatically  */
                                                /* when UCxRXBUF is read, and must not be cleared by software. Otherwise, it does  */
                                                /* not function correctly. */
      __IO uint16_t UCFE                	:	1;  /* Framing error flag. This bit indicates a bus conflict in 4-wire master mode.  */
                                                /* UCFE is not used in 3-wire master or any slave mode. */
      __IO uint16_t UCLISTEN            	:	1;  /* Listen enable. The UCLISTEN bit selects loopback mode. */
      __I  uint16_t RESERVED1           	:	8;  /* Reserved */

    } bit;
  } STATW;
  uint8_t RESERVED1[2];
  union {    /* EUSCI_B0_SPI_RXBUF Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B0_SPI_RXBUF Control Bits */
      __I  uint16_t UCRXBUF             	:	8;  /* The receive-data buffer is user accessible and contains the last received  */
                                                /* character from the receive shift register. Reading UCxRXBUF resets the receive- */
                                                /* error bits and UCRXIFG. In 7-bit data mode, UCxRXBUF is LSB justified and the  */
                                                /* MSB is always reset. */
      __I  uint16_t RESERVED1           	:	8;  /* Reserved */

    } bit;
  } RXBUF;
  union {    /* EUSCI_B0_SPI_TXBUF Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B0_SPI_TXBUF Control Bits */
      __IO uint16_t UCTXBUF             	:	8;  /* The transmit data buffer is user accessible and holds the data waiting to be  */
                                                /* moved into the transmit shift register and transmitted. Writing to the transmit  */
                                                /* data buffer clears UCTXIFG. The MSB of UCxTXBUF is not used for 7-bit data and  */
                                                /* is reset. */
      __I  uint16_t RESERVED1           	:	8;  /* Reserved */

    } bit;
  } TXBUF;
  uint8_t RESERVED2[26];
  union {    /* EUSCI_B0_SPI_IE Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B0_SPI_IE Control Bits */
      __IO uint16_t UCRXIE              	:	1;  /* Receive interrupt enable */
      __IO uint16_t UCTXIE              	:	1;  /* Transmit interrupt enable */
      __I  uint16_t RESERVED1           	:	14;  /* Reserved */

    } bit;
  } IE;
  union {    /* EUSCI_B0_SPI_IFG Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B0_SPI_IFG Control Bits */
      __IO uint16_t UCRXIFG             	:	1;  /* Receive interrupt flag. UCRXIFG is set when UCxxRXBUF has received a complete  */
                                                /* character. */
      __IO uint16_t UCTXIFG             	:	1;  /* Transmit interrupt flag. UCTXIFG is set when UCxxTXBUF empty. */
      __I  uint16_t RESERVED1           	:	14;  /* Reserved */

    } bit;
  } IFG;
  uint16_t IV; /* eUSCI_Bx Interrupt Vector */
} EUSCI_B0_SPI_Type;


/* EUSCI_B0_SPI_CTLW0 Control Bits */
#define EUSCI_B0_SPI_CTLW0_UCSWRST                         (0x0001)              /* Software reset enable */
#define EUSCI_B0_SPI_CTLW0_UCSTEM                          (0x0002)              /* STE mode select in master mode. This byte is ignored in slave or 3-wire mode. */
#define EUSCI_B0_SPI_CTLW0_RESERVED1__M                    (0x003c)              /*  */
#define EUSCI_B0_SPI_CTLW0_UCSSEL__M                       (0x00c0)              /* eUSCI clock source select. These bits select the BRCLK source clock in master  */
                                                                                 /* mode. UCxCLK is always used in slave mode. */
#define EUSCI_B0_SPI_CTLW0_UCSSEL__1                       (0x0040)              /* ACLK */
#define EUSCI_B0_SPI_CTLW0_UCSSEL__1_ACLK                  (0x0040)              /* ACLK */
#define EUSCI_B0_SPI_CTLW0_UCSSEL__2                       (0x0080)              /* SMCLK */
#define EUSCI_B0_SPI_CTLW0_UCSSEL__2_SMCLK                 (0x0080)              /* SMCLK */
#define EUSCI_B0_SPI_CTLW0_UCSSEL__3                       (0x00c0)              /* SMCLK */
#define EUSCI_B0_SPI_CTLW0_UCSSEL__3_SMCLK                 (0x00c0)              /* SMCLK */
#define EUSCI_B0_SPI_CTLW0_UCSYNC                          (0x0100)              /* Synchronous mode enable */
#define EUSCI_B0_SPI_CTLW0_UCMODE__M                       (0x0600)              /* eUSCI mode. The UCMODEx bits select the synchronous mode when UCSYNC = 1. */
#define EUSCI_B0_SPI_CTLW0_UCMODE__0                       (0x0000)              /* 3-pin SPI */
#define EUSCI_B0_SPI_CTLW0_UCMODE__0_3_PIN_SPI             (0x0000)              /* 3-pin SPI */
#define EUSCI_B0_SPI_CTLW0_UCMODE__1                       (0x0200)              /* 4-pin SPI with UCxSTE active high: Slave enabled when UCxSTE = 1 */
#define EUSCI_B0_SPI_CTLW0_UCMODE__2                       (0x0400)              /* 4-pin SPI with UCxSTE active low: Slave enabled when UCxSTE = 0 */
#define EUSCI_B0_SPI_CTLW0_UCMODE__3                       (0x0600)              /* I2C mode */
#define EUSCI_B0_SPI_CTLW0_UCMODE__3_I2C_MODE              (0x0600)              /* I2C mode */
#define EUSCI_B0_SPI_CTLW0_UCMST                           (0x0800)              /* Master mode select */
#define EUSCI_B0_SPI_CTLW0_UC7BIT                          (0x1000)              /* Character length. Selects 7-bit or 8-bit character length. */
#define EUSCI_B0_SPI_CTLW0_UCMSB                           (0x2000)              /* MSB first select. Controls the direction of the receive and transmit shift  */
                                                                                 /* register. */
#define EUSCI_B0_SPI_CTLW0_UCCKPL                          (0x4000)              /* Clock polarity select */
#define EUSCI_B0_SPI_CTLW0_UCCKPH                          (0x8000)              /* Clock phase select */

/* EUSCI_B0_SPI_STATW Control Bits */
#define EUSCI_B0_SPI_STATW_UCBUSY                          (0x0001)              /* eUSCI busy. This bit indicates if a transmit or receive operation is in progress */
                                                                                 /* . */
#define EUSCI_B0_SPI_STATW_RESERVED2__M                    (0x001e)              /*  */
#define EUSCI_B0_SPI_STATW_UCOE                            (0x0020)              /* Overrun error flag. This bit is set when a character is transferred into  */
                                                                                 /* UCxRXBUF before the previous character was read. UCOE is cleared automatically  */
                                                                                 /* when UCxRXBUF is read, and must not be cleared by software. Otherwise, it does  */
                                                                                 /* not function correctly. */
#define EUSCI_B0_SPI_STATW_UCFE                            (0x0040)              /* Framing error flag. This bit indicates a bus conflict in 4-wire master mode.  */
                                                                                 /* UCFE is not used in 3-wire master or any slave mode. */
#define EUSCI_B0_SPI_STATW_UCLISTEN                        (0x0080)              /* Listen enable. The UCLISTEN bit selects loopback mode. */
#define EUSCI_B0_SPI_STATW_RESERVED1__M                    (0xff00)              /*  */

/* EUSCI_B0_SPI_RXBUF Control Bits */
#define EUSCI_B0_SPI_RXBUF_UCRXBUF__M                      (0x00ff)              /* The receive-data buffer is user accessible and contains the last received  */
                                                                                 /* character from the receive shift register. Reading UCxRXBUF resets the receive- */
                                                                                 /* error bits and UCRXIFG. In 7-bit data mode, UCxRXBUF is LSB justified and the  */
                                                                                 /* MSB is always reset. */
#define EUSCI_B0_SPI_RXBUF_RESERVED1__M                    (0xff00)              /*  */

/* EUSCI_B0_SPI_TXBUF Control Bits */
#define EUSCI_B0_SPI_TXBUF_UCTXBUF__M                      (0x00ff)              /* The transmit data buffer is user accessible and holds the data waiting to be  */
                                                                                 /* moved into the transmit shift register and transmitted. Writing to the transmit  */
                                                                                 /* data buffer clears UCTXIFG. The MSB of UCxTXBUF is not used for 7-bit data and  */
                                                                                 /* is reset. */
#define EUSCI_B0_SPI_TXBUF_RESERVED1__M                    (0xff00)              /*  */

/* EUSCI_B0_SPI_IE Control Bits */
#define EUSCI_B0_SPI_IE_UCRXIE                             (0x0001)              /* Receive interrupt enable */
#define EUSCI_B0_SPI_IE_UCTXIE                             (0x0002)              /* Transmit interrupt enable */
#define EUSCI_B0_SPI_IE_RESERVED1__M                       (0xfffc)              /*  */

/* EUSCI_B0_SPI_IFG Control Bits */
#define EUSCI_B0_SPI_IFG_UCRXIFG                           (0x0001)              /* Receive interrupt flag. UCRXIFG is set when UCxxRXBUF has received a complete  */
                                                                                 /* character. */
#define EUSCI_B0_SPI_IFG_UCTXIFG                           (0x0002)              /* Transmit interrupt flag. UCTXIFG is set when UCxxTXBUF empty. */
#define EUSCI_B0_SPI_IFG_RESERVED1__M                      (0xfffc)              /*  */


//*****************************************************************************
// EUSCI_B1_I2C Registers
//*****************************************************************************
typedef struct {
  union {    /* EUSCI_B1_I2C_CTLW0 Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B1_I2C_CTLW0 Control Bits */
      __IO uint16_t UCSWRST             	:	1;  /* Software reset enable. Can be modified only when eUSCI_B is in reset state ( */
                                                /* UCSWRST = 1). */
      __IO uint16_t UCTXSTT             	:	1;  /* Transmit START condition in master mode. Ignored in slave mode. In master  */
                                                /* receiver mode, a repeated START condition is preceded by a NACK. UCTXSTT is  */
                                                /* automatically cleared after START condition and address information is  */
                                                /* transmitted. Ignored in slave mode. */
      __IO uint16_t UCTXSTP             	:	1;  /* Transmit STOP condition in master mode. Ignored in slave mode. In master  */
                                                /* receiver mode, the STOP condition is preceded by a NACK. UCTXSTP is  */
                                                /* automatically cleared after STOP is generated. This bit is a don't care, if  */
                                                /* automatic UCASTPx is different from 01 or 10.  */
      __IO uint16_t UCTXNACK            	:	1;  /* Transmit a NACK. UCTXNACK is automatically cleared after a NACK is transmitted.  */
                                                /* Only for slave receiver mode. */
      __IO uint16_t UCTR                	:	1;  /* Transmitter/receiver */
      __IO uint16_t UCTXACK             	:	1;  /* Transmit ACK condition in slave mode with enabled address mask register. After  */
                                                /* the UCSTTIFG has been set, the user needs to set or reset the UCTXACK flag to  */
                                                /* continue with the I2C protocol. The clock is stretched until the UCBxCTL1  */
                                                /* register has been written. This bit is cleared automatically after the ACK has  */
                                                /* been send. */
      __IO uint16_t UCSSEL              	:	2;  /* eUSCI_B clock source select. These bits select the BRCLK source clock. These  */
                                                /* bits are ignored in slave mode. Can be modified only when eUSCI_B is in reset  */
                                                /* state (UCSWRST = 1). */
      __IO uint16_t UCSYNC              	:	1;  /* Synchronous mode enable. For eUSCI_B always read and write as 1. */
      __IO uint16_t UCMODE              	:	2;  /* eUSCI_B mode. The UCMODEx bits select the synchronous mode when UCSYNC = 1. Can  */
                                                /* be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
      __IO uint16_t UCMST               	:	1;  /* Master mode select. When a master loses arbitration in a multi-master  */
                                                /* environment (UCMM = 1), the UCMST bit is automatically cleared and the module  */
                                                /* acts as slave. */
      __I  uint16_t RESERVED1           	:	1;  /* Reserved */
      __IO uint16_t UCMM                	:	1;  /* Multi-master environment select. Can be modified only when eUSCI_B is in reset  */
                                                /* state (UCSWRST = 1). */
      __IO uint16_t UCSLA10             	:	1;  /* Slave addressing mode select */
      __IO uint16_t UCA10               	:	1;  /* Own addressing mode select. Can be modified only when eUSCI_B is in reset state  */
                                                /* (UCSWRST = 1). */

    } bit;
  } CTLW0;
  union {    /* EUSCI_B1_I2C_CTLW1 Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B1_I2C_CTLW1 Control Bits */
      __IO uint16_t UCGLIT              	:	2;  /* Deglitch time */
      __IO uint16_t UCASTP              	:	2;  /* Automatic STOP condition generation. In slave mode only UCBCNTIFG is available.  */
                                                /* Can be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
      __IO uint16_t UCSWACK             	:	1;  /* Using this bit it is possible to select, whether the eUSCI_B module triggers the */
                                                /* sending of the ACK of the address or if it is controlled by software. */
      __IO uint16_t UCSTPNACK           	:	1;  /* The UCSTPNACK bit allows to make the eUSCI_B master acknowledge the last byte in */
                                                /* master receiver mode as well. This is not conform to the I2C specification and  */
                                                /* should only be used for slaves, which automatically release the SDA after a  */
                                                /* fixed packet length. Can be modified only when eUSCI_B is in reset state ( */
                                                /* UCSWRST = 1). */
      __IO uint16_t UCCLTO              	:	2;  /* Clock low timeout select. Modify only when UCSWRST = 1. */
      __IO uint16_t UCETXINT            	:	1;  /* Early UCTXIFG0. Only in slave mode. When this bit is set, the slave addresses  */
                                                /* defined in UCxI2COA1 to UCxI2COA3 must be disabled. Can be modified only when  */
                                                /* eUSCI_B is in reset state (UCSWRST = 1). */
      __I  uint16_t RESERVED1           	:	7;  /* Reserved */

    } bit;
  } CTLW1;
  uint8_t RESERVED0[2];
  uint16_t BRW; /* eUSCI_Bx Bit Rate Control Word */
  union {    /* EUSCI_B1_I2C_STATW Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B1_I2C_STATW Control Bits */
      __I  uint16_t RESERVED2           	:	4;  /* Reserved */
      __I  uint16_t UCBBUSY             	:	1;  /* Bus busy */
      __I  uint16_t UCGC                	:	1;  /* General call address received. UCGC is automatically cleared when a START  */
                                                /* condition is received. */
      __I  uint16_t UCSCLLOW            	:	1;  /* SCL low */
      __I  uint16_t RESERVED1           	:	1;  /* Reserved */
      __I  uint16_t UCBCNT              	:	8;  /* Hardware byte counter value. Reading this register returns the number of bytes  */
                                                /* received or transmitted on the I2C-Bus since the last START or RESTART. There is */
                                                /* no synchronization of this register done. When reading UCBxBCNT during the  */
                                                /* first bit position, a faulty readback can occur. */

    } bit;
  } STATW;
  union {    /* EUSCI_B1_I2C_TBCNT Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B1_I2C_TBCNT Control Bits */
      __IO uint16_t UCTBCNT             	:	8;  /* The byte counter threshold value is used to set the number of I2C data bytes  */
                                                /* after which the automatic STOP or the UCSTPIFG should occur. This value is  */
                                                /* evaluated only if UCASTPx is different from 00. Can be modified only when  */
                                                /* eUSCI_B is in reset state (UCSWRST = 1). */
      __I  uint16_t RESERVED1           	:	8;  /* Reserved */

    } bit;
  } TBCNT;
  union {    /* EUSCI_B1_I2C_RXBUF Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B1_I2C_RXBUF Control Bits */
      __I  uint16_t UCRXBUF             	:	8;  /* The receive-data buffer is user accessible and contains the last received  */
                                                /* character from the receive shift register. Reading UCBxRXBUF resets the UCRXIFGx */
                                                /* flags. */
      __I  uint16_t RESERVED1           	:	8;  /* Reserved */

    } bit;
  } RXBUF;
  union {    /* EUSCI_B1_I2C_TXBUF Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B1_I2C_TXBUF Control Bits */
      __IO uint16_t UCTXBUF             	:	8;  /* The transmit data buffer is user accessible and holds the data waiting to be  */
                                                /* moved into the transmit shift register and transmitted. Writing to the transmit  */
                                                /* data buffer clears the UCTXIFGx flags.  */
      __I  uint16_t RESERVED1           	:	8;  /* Reserved */

    } bit;
  } TXBUF;
  uint8_t RESERVED1[4];
  union {    /* EUSCI_B1_I2C_I2COA0 Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B1_I2C_I2COA0 Control Bits */
      __IO uint16_t I2COA               	:	10;  /* I2C own address. The I2COA0 bits contain the local address of the eUSCIx_B I2C  */
                                                /* controller. The address is right justified. In 7-bit addressing mode, bit 6 is  */
                                                /* the MSB and bits 9-7 are ignored. In 10-bit addressing mode, bit 9 is the MSB.  */
                                                /* Can be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
      __IO uint16_t UCOAEN              	:	1;  /* Own Address enable register. With this register it can be selected if the I2C  */
                                                /* slave-address related to this register UCBxI2COA0 is evaluated or not. Can be  */
                                                /* modified only when eUSCI_B is in reset state (UCSWRST = 1). */
      __I  uint16_t RESERVED1           	:	4;  /* Reserved */
      __IO uint16_t UCGCEN              	:	1;  /* General call response enable. This bit is only available in UCBxI2COA0. Can be  */
                                                /* modified only when eUSCI_B is in reset state (UCSWRST = 1). */

    } bit;
  } I2COA0;
  union {    /* EUSCI_B1_I2C_I2COA1 Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B1_I2C_I2COA1 Control Bits */
      __IO uint16_t I2COA1              	:	10;  /* I2C own address. The I2COAx bits contain the local address of the eUSCIx_B I2C  */
                                                /* controller. The address is right justified. In 7-bit addressing mode, bit 6 is  */
                                                /* the MSB and bits 9-7 are ignored. In 10-bit addressing mode, bit 9 is the MSB.  */
                                                /* Can be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
      __IO uint16_t UCOAEN              	:	1;  /* Own Address enable register. With this register it can be selected if the I2C  */
                                                /* slave-address related to this register UCBxI2COA1 is evaluated or not. Can be  */
                                                /* modified only when eUSCI_B is in reset state (UCSWRST = 1). */
      __I  uint16_t RESERVED1           	:	5;  /* Reserved */

    } bit;
  } I2COA1;
  union {    /* EUSCI_B1_I2C_I2COA2 Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B1_I2C_I2COA2 Control Bits */
      __IO uint16_t I2COA2              	:	10;  /* I2C own address. The I2COAx bits contain the local address of the eUSCIx_B I2C  */
                                                /* controller. The address is right justified. In 7-bit addressing mode, bit 6 is  */
                                                /* the MSB and bits 9-7 are ignored. In 10-bit addressing mode, bit 9 is the MSB.  */
                                                /* Can be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
      __IO uint16_t UCOAEN              	:	1;  /* Own Address enable register. With this register it can be selected if the I2C  */
                                                /* slave-address related to this register UCBxI2COA2 is evaluated or not. Can be  */
                                                /* modified only when eUSCI_B is in reset state (UCSWRST = 1). */
      __I  uint16_t RESERVED1           	:	5;  /* Reserved */

    } bit;
  } I2COA2;
  union {    /* EUSCI_B1_I2C_I2COA3 Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B1_I2C_I2COA3 Control Bits */
      __IO uint16_t I2COA3              	:	10;  /* I2C own address. The I2COA3 bits contain the local address of the eUSCIx_B I2C  */
                                                /* controller. The address is right justified. In 7-bit addressing mode, bit 6 is  */
                                                /* the MSB and bits 9-7 are ignored. In 10-bit addressing mode, bit 9 is the MSB.  */
                                                /* Can be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
      __IO uint16_t UCOAEN              	:	1;  /* Own Address enable register. With this register it can be selected if the I2C  */
                                                /* slave-address related to this register UCBxI2COA3 is evaluated or not. Can be  */
                                                /* modified only when eUSCI_B is in reset state (UCSWRST = 1). */
      __I  uint16_t RESERVED1           	:	5;  /* Reserved */

    } bit;
  } I2COA3;
  union {    /* EUSCI_B1_I2C_ADDRX Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B1_I2C_ADDRX Control Bits */
      __I  uint16_t ADDRX               	:	10;  /* Received Address Register. This register contains the last received slave  */
                                                /* address on the bus. Using this register and the address mask register it is  */
                                                /* possible to react on more than one slave address using one eUSCI_B module. */
      __I  uint16_t RESERVED1           	:	6;  /* Reserved */

    } bit;
  } ADDRX;
  union {    /* EUSCI_B1_I2C_ADDMASK Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B1_I2C_ADDMASK Control Bits */
      __IO uint16_t ADDMASK             	:	10;  /* Address Mask Register. By clearing the corresponding bit of the own address,  */
                                                /* this bit is a don't care when comparing the address on the bus to the own  */
                                                /* address. Using this method, it is possible to react on more than one slave  */
                                                /* address. When all bits of ADDMASKx are set, the address mask feature is  */
                                                /* deactivated. Can be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
      __I  uint16_t RESERVED1           	:	6;  /* Reserved */

    } bit;
  } ADDMASK;
  union {    /* EUSCI_B1_I2C_I2CSA Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B1_I2C_I2CSA Control Bits */
      __IO uint16_t I2CSA               	:	10;  /* I2C slave address. The I2CSAx bits contain the slave address of the external  */
                                                /* device to be addressed by the eUSCIx_B module. It is only used in master mode.  */
                                                /* The address is right justified. In 7-bit slave addressing mode, bit 6 is the MSB */
                                                /* and bits 9-7 are ignored. In 10-bit slave addressing mode, bit 9 is the MSB. */
      __I  uint16_t RESERVED1           	:	6;  /* Reserved */

    } bit;
  } I2CSA;
  uint8_t RESERVED2[8];
  union {    /* EUSCI_B1_I2C_IE Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B1_I2C_IE Control Bits */
      __IO uint16_t UCRXIE0             	:	1;  /* Receive interrupt enable 0 */
      __IO uint16_t UCTXIE0             	:	1;  /* Transmit interrupt enable 0 */
      __IO uint16_t UCSTTIE             	:	1;  /* START condition interrupt enable */
      __IO uint16_t UCSTPIE             	:	1;  /* STOP condition interrupt enable */
      __IO uint16_t UCALIE              	:	1;  /* Arbitration lost interrupt enable */
      __IO uint16_t UCNACKIE            	:	1;  /* Not-acknowledge interrupt enable */
      __IO uint16_t UCBCNTIE            	:	1;  /* Byte counter interrupt enable. */
      __IO uint16_t UCCLTOIE            	:	1;  /* Clock low timeout interrupt enable. */
      __IO uint16_t UCRXIE1             	:	1;  /* Receive interrupt enable 1 */
      __IO uint16_t UCTXIE1             	:	1;  /* Transmit interrupt enable 1 */
      __IO uint16_t UCRXIE2             	:	1;  /* Receive interrupt enable 2 */
      __IO uint16_t UCTXIE2             	:	1;  /* Transmit interrupt enable 2 */
      __IO uint16_t UCRXIE3             	:	1;  /* Receive interrupt enable 3 */
      __IO uint16_t UCTXIE3             	:	1;  /* Transmit interrupt enable 3 */
      __IO uint16_t UCBIT9IE            	:	1;  /* Bit position 9 interrupt enable */
      __I  uint16_t RESERVED1           	:	1;  /* Reserved */

    } bit;
  } IE;
  union {    /* EUSCI_B1_I2C_IFG Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B1_I2C_IFG Control Bits */
      __IO uint16_t UCRXIFG0            	:	1;  /* eUSCI_B receive interrupt flag 0. UCRXIFG0 is set when UCBxRXBUF has received a  */
                                                /* complete character in master mode or in slave mode, if the slave address defined */
                                                /* in UCBxI2COA0 was on the bus in the same frame. */
      __IO uint16_t UCTXIFG0            	:	1;  /* eUSCI_B transmit interrupt flag 0. UCTXIFG0 is set when UCBxTXBUF is empty in  */
                                                /* master mode or in slave mode, if the slave address defined in UCBxI2COA0 was on  */
                                                /* the bus in the same frame. */
      __IO uint16_t UCSTTIFG            	:	1;  /* START condition interrupt flag */
      __IO uint16_t UCSTPIFG            	:	1;  /* STOP condition interrupt flag */
      __IO uint16_t UCALIFG             	:	1;  /* Arbitration lost interrupt flag */
      __IO uint16_t UCNACKIFG           	:	1;  /* Not-acknowledge received interrupt flag. This flag only is updated when  */
                                                /* operating in master mode. */
      __IO uint16_t UCBCNTIFG           	:	1;  /* Byte counter interrupt flag. When using this interrupt the user needs to ensure  */
                                                /* enough processing bandwidth (see the Byte Counter Interrupt section). */
      __IO uint16_t UCCLTOIFG           	:	1;  /* Clock low timeout interrupt flag */
      __IO uint16_t UCRXIFG1            	:	1;  /* Receive interrupt flag 1. UCRXIFG1 is set when UCBxRXBUF has received a complete */
                                                /* byte in slave mode and if the slave address defined in UCBxI2COA1 was on the  */
                                                /* bus in the same frame. */
      __IO uint16_t UCTXIFG1            	:	1;  /* eUSCI_B transmit interrupt flag 1. UCTXIFG1 is set when UCBxTXBUF is empty in  */
                                                /* slave mode, if the slave address defined in UCBxI2COA1 was on the bus in the  */
                                                /* same frame. */
      __IO uint16_t UCRXIFG2            	:	1;  /* Receive interrupt flag 2. UCRXIFG2 is set when UCBxRXBUF has received a complete */
                                                /* byte in slave mode and if the slave address defined in UCBxI2COA2 was on the  */
                                                /* bus in the same frame. */
      __IO uint16_t UCTXIFG2            	:	1;  /* eUSCI_B transmit interrupt flag 2. UCTXIFG2 is set when UCBxTXBUF is empty in  */
                                                /* slave mode, if the slave address defined in UCBxI2COA2 was on the bus in the  */
                                                /* same frame. */
      __IO uint16_t UCRXIFG3            	:	1;  /* Receive interrupt flag 2. UCRXIFG2 is set when UCBxRXBUF has received a complete */
                                                /* byte in slave mode and if the slave address defined in UCBxI2COA2 was on the  */
                                                /* bus in the same frame. */
      __IO uint16_t UCTXIFG3            	:	1;  /* eUSCI_B transmit interrupt flag 3. UCTXIFG3 is set when UCBxTXBUF is empty in  */
                                                /* slave mode, if the slave address defined in UCBxI2COA3 was on the bus in the  */
                                                /* same frame. */
      __IO uint16_t UCBIT9IFG           	:	1;  /* Bit position 9 interrupt flag */
      __I  uint16_t RESERVED1           	:	1;  /* Reserved */

    } bit;
  } IFG;
  uint16_t IV; /* eUSCI_Bx Interrupt Vector */
} EUSCI_B1_I2C_Type;


/* EUSCI_B1_I2C_CTLW0 Control Bits */
#define EUSCI_B1_I2C_CTLW0_UCSWRST                         (0x0001)              /* Software reset enable. Can be modified only when eUSCI_B is in reset state ( */
                                                                                 /* UCSWRST = 1). */
#define EUSCI_B1_I2C_CTLW0_UCTXSTT                         (0x0002)              /* Transmit START condition in master mode. Ignored in slave mode. In master  */
                                                                                 /* receiver mode, a repeated START condition is preceded by a NACK. UCTXSTT is  */
                                                                                 /* automatically cleared after START condition and address information is  */
                                                                                 /* transmitted. Ignored in slave mode. */
#define EUSCI_B1_I2C_CTLW0_UCTXSTP                         (0x0004)              /* Transmit STOP condition in master mode. Ignored in slave mode. In master  */
                                                                                 /* receiver mode, the STOP condition is preceded by a NACK. UCTXSTP is  */
                                                                                 /* automatically cleared after STOP is generated. This bit is a don't care, if  */
                                                                                 /* automatic UCASTPx is different from 01 or 10.  */
#define EUSCI_B1_I2C_CTLW0_UCTXNACK                        (0x0008)              /* Transmit a NACK. UCTXNACK is automatically cleared after a NACK is transmitted.  */
                                                                                 /* Only for slave receiver mode. */
#define EUSCI_B1_I2C_CTLW0_UCTR                            (0x0010)              /* Transmitter/receiver */
#define EUSCI_B1_I2C_CTLW0_UCTXACK                         (0x0020)              /* Transmit ACK condition in slave mode with enabled address mask register. After  */
                                                                                 /* the UCSTTIFG has been set, the user needs to set or reset the UCTXACK flag to  */
                                                                                 /* continue with the I2C protocol. The clock is stretched until the UCBxCTL1  */
                                                                                 /* register has been written. This bit is cleared automatically after the ACK has  */
                                                                                 /* been send. */
#define EUSCI_B1_I2C_CTLW0_UCSSEL__M                       (0x00c0)              /* eUSCI_B clock source select. These bits select the BRCLK source clock. These  */
                                                                                 /* bits are ignored in slave mode. Can be modified only when eUSCI_B is in reset  */
                                                                                 /* state (UCSWRST = 1). */
#define EUSCI_B1_I2C_CTLW0_UCSSEL__0                       (0x0000)              /* UCLKI */
#define EUSCI_B1_I2C_CTLW0_UCSSEL__0_UCLKI                 (0x0000)              /* UCLKI */
#define EUSCI_B1_I2C_CTLW0_UCSSEL__1                       (0x0040)              /* ACLK */
#define EUSCI_B1_I2C_CTLW0_UCSSEL__1_ACLK                  (0x0040)              /* ACLK */
#define EUSCI_B1_I2C_CTLW0_UCSSEL__2                       (0x0080)              /* SMCLK */
#define EUSCI_B1_I2C_CTLW0_UCSSEL__2_SMCLK                 (0x0080)              /* SMCLK */
#define EUSCI_B1_I2C_CTLW0_UCSSEL__3                       (0x00c0)              /* SMCLK */
#define EUSCI_B1_I2C_CTLW0_UCSSEL__3_SMCLK                 (0x00c0)              /* SMCLK */
#define EUSCI_B1_I2C_CTLW0_UCSYNC                          (0x0100)              /* Synchronous mode enable. For eUSCI_B always read and write as 1. */
#define EUSCI_B1_I2C_CTLW0_UCMODE__M                       (0x0600)              /* eUSCI_B mode. The UCMODEx bits select the synchronous mode when UCSYNC = 1. Can  */
                                                                                 /* be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
#define EUSCI_B1_I2C_CTLW0_UCMODE__0                       (0x0000)              /* 3-pin SPI */
#define EUSCI_B1_I2C_CTLW0_UCMODE__0_3_PIN_SPI             (0x0000)              /* 3-pin SPI */
#define EUSCI_B1_I2C_CTLW0_UCMODE__1                       (0x0200)              /* 4-pin SPI (master/slave enabled if STE = 1) */
#define EUSCI_B1_I2C_CTLW0_UCMODE__2                       (0x0400)              /* 4-pin SPI (master/slave enabled if STE = 0) */
#define EUSCI_B1_I2C_CTLW0_UCMODE__3                       (0x0600)              /* I2C mode */
#define EUSCI_B1_I2C_CTLW0_UCMODE__3_I2C_MODE              (0x0600)              /* I2C mode */
#define EUSCI_B1_I2C_CTLW0_UCMST                           (0x0800)              /* Master mode select. When a master loses arbitration in a multi-master  */
                                                                                 /* environment (UCMM = 1), the UCMST bit is automatically cleared and the module  */
                                                                                 /* acts as slave. */
#define EUSCI_B1_I2C_CTLW0_RESERVED1                       (0x1000)              /*  */
#define EUSCI_B1_I2C_CTLW0_UCMM                            (0x2000)              /* Multi-master environment select. Can be modified only when eUSCI_B is in reset  */
                                                                                 /* state (UCSWRST = 1). */
#define EUSCI_B1_I2C_CTLW0_UCSLA10                         (0x4000)              /* Slave addressing mode select */
#define EUSCI_B1_I2C_CTLW0_UCA10                           (0x8000)              /* Own addressing mode select. Can be modified only when eUSCI_B is in reset state  */
                                                                                 /* (UCSWRST = 1). */

/* EUSCI_B1_I2C_CTLW1 Control Bits */
#define EUSCI_B1_I2C_CTLW1_UCGLIT__M                       (0x0003)              /* Deglitch time */
#define EUSCI_B1_I2C_CTLW1_UCGLIT__0                       (0x0000)              /* 50 ns */
#define EUSCI_B1_I2C_CTLW1_UCGLIT__0_50_NS                 (0x0000)              /* 50 ns */
#define EUSCI_B1_I2C_CTLW1_UCGLIT__1                       (0x0001)              /* 25 ns */
#define EUSCI_B1_I2C_CTLW1_UCGLIT__1_25_NS                 (0x0001)              /* 25 ns */
#define EUSCI_B1_I2C_CTLW1_UCGLIT__2                       (0x0002)              /* 12.5 ns */
#define EUSCI_B1_I2C_CTLW1_UCGLIT__2_12_5_NS               (0x0002)              /* 12.5 ns */
#define EUSCI_B1_I2C_CTLW1_UCGLIT__3                       (0x0003)              /* 6.25 ns */
#define EUSCI_B1_I2C_CTLW1_UCGLIT__3_6_25_NS               (0x0003)              /* 6.25 ns */
#define EUSCI_B1_I2C_CTLW1_UCASTP__M                       (0x000c)              /* Automatic STOP condition generation. In slave mode only UCBCNTIFG is available.  */
                                                                                 /* Can be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
#define EUSCI_B1_I2C_CTLW1_UCASTP__0                       (0x0000)              /* No automatic STOP generation. The STOP condition is generated after the user sets the UCTXSTP bit. The value in UCBxTBCNT is a don't care. */
#define EUSCI_B1_I2C_CTLW1_UCASTP__1                       (0x0004)              /* UCBCNTIFG is set with the byte counter reaches the threshold defined in UCBxTBCNT. */
#define EUSCI_B1_I2C_CTLW1_UCASTP__2                       (0x0008)              /* A STOP condition is generated automatically after the byte counter value reached UCBxTBCNT. UCBCNTIFG is set with the byte counter reaching the threshold. */
#define EUSCI_B1_I2C_CTLW1_UCSWACK                         (0x0010)              /* Using this bit it is possible to select, whether the eUSCI_B module triggers the */
                                                                                 /* sending of the ACK of the address or if it is controlled by software. */
#define EUSCI_B1_I2C_CTLW1_UCSTPNACK                       (0x0020)              /* The UCSTPNACK bit allows to make the eUSCI_B master acknowledge the last byte in */
                                                                                 /* master receiver mode as well. This is not conform to the I2C specification and  */
                                                                                 /* should only be used for slaves, which automatically release the SDA after a  */
                                                                                 /* fixed packet length. Can be modified only when eUSCI_B is in reset state ( */
                                                                                 /* UCSWRST = 1). */
#define EUSCI_B1_I2C_CTLW1_UCCLTO__M                       (0x00c0)              /* Clock low timeout select. Modify only when UCSWRST = 1. */
#define EUSCI_B1_I2C_CTLW1_UCCLTO__0                       (0x0000)              /* Disable clock low timeout counter */
#define EUSCI_B1_I2C_CTLW1_UCCLTO__1                       (0x0040)              /* 135 000 SYSCLK cycles (approximately 28 ms) */
#define EUSCI_B1_I2C_CTLW1_UCCLTO__2                       (0x0080)              /* 150 000 SYSCLK cycles (approximately 31 ms) */
#define EUSCI_B1_I2C_CTLW1_UCCLTO__3                       (0x00c0)              /* 165 000 SYSCLK cycles (approximately 34 ms) */
#define EUSCI_B1_I2C_CTLW1_UCETXINT                        (0x0100)              /* Early UCTXIFG0. Only in slave mode. When this bit is set, the slave addresses  */
                                                                                 /* defined in UCxI2COA1 to UCxI2COA3 must be disabled. Can be modified only when  */
                                                                                 /* eUSCI_B is in reset state (UCSWRST = 1). */
#define EUSCI_B1_I2C_CTLW1_RESERVED1__M                    (0xfe00)              /*  */

/* EUSCI_B1_I2C_STATW Control Bits */
#define EUSCI_B1_I2C_STATW_RESERVED2__M                    (0x000f)              /*  */
#define EUSCI_B1_I2C_STATW_UCBBUSY                         (0x0010)              /* Bus busy */
#define EUSCI_B1_I2C_STATW_UCGC                            (0x0020)              /* General call address received. UCGC is automatically cleared when a START  */
                                                                                 /* condition is received. */
#define EUSCI_B1_I2C_STATW_UCSCLLOW                        (0x0040)              /* SCL low */
#define EUSCI_B1_I2C_STATW_RESERVED1                       (0x0080)              /*  */
#define EUSCI_B1_I2C_STATW_UCBCNT__M                       (0xff00)              /* Hardware byte counter value. Reading this register returns the number of bytes  */
                                                                                 /* received or transmitted on the I2C-Bus since the last START or RESTART. There is */
                                                                                 /* no synchronization of this register done. When reading UCBxBCNT during the  */
                                                                                 /* first bit position, a faulty readback can occur. */

/* EUSCI_B1_I2C_TBCNT Control Bits */
#define EUSCI_B1_I2C_TBCNT_UCTBCNT__M                      (0x00ff)              /* The byte counter threshold value is used to set the number of I2C data bytes  */
                                                                                 /* after which the automatic STOP or the UCSTPIFG should occur. This value is  */
                                                                                 /* evaluated only if UCASTPx is different from 00. Can be modified only when  */
                                                                                 /* eUSCI_B is in reset state (UCSWRST = 1). */
#define EUSCI_B1_I2C_TBCNT_RESERVED1__M                    (0xff00)              /*  */

/* EUSCI_B1_I2C_RXBUF Control Bits */
#define EUSCI_B1_I2C_RXBUF_UCRXBUF__M                      (0x00ff)              /* The receive-data buffer is user accessible and contains the last received  */
                                                                                 /* character from the receive shift register. Reading UCBxRXBUF resets the UCRXIFGx */
                                                                                 /* flags. */
#define EUSCI_B1_I2C_RXBUF_RESERVED1__M                    (0xff00)              /*  */

/* EUSCI_B1_I2C_TXBUF Control Bits */
#define EUSCI_B1_I2C_TXBUF_UCTXBUF__M                      (0x00ff)              /* The transmit data buffer is user accessible and holds the data waiting to be  */
                                                                                 /* moved into the transmit shift register and transmitted. Writing to the transmit  */
                                                                                 /* data buffer clears the UCTXIFGx flags.  */
#define EUSCI_B1_I2C_TXBUF_RESERVED1__M                    (0xff00)              /*  */

/* EUSCI_B1_I2C_I2COA0 Control Bits */
#define EUSCI_B1_I2C_I2COA0_I2COA__M                       (0x03ff)              /* I2C own address. The I2COA0 bits contain the local address of the eUSCIx_B I2C  */
                                                                                 /* controller. The address is right justified. In 7-bit addressing mode, bit 6 is  */
                                                                                 /* the MSB and bits 9-7 are ignored. In 10-bit addressing mode, bit 9 is the MSB.  */
                                                                                 /* Can be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
#define EUSCI_B1_I2C_I2COA0_UCOAEN                         (0x0400)              /* Own Address enable register. With this register it can be selected if the I2C  */
                                                                                 /* slave-address related to this register UCBxI2COA0 is evaluated or not. Can be  */
                                                                                 /* modified only when eUSCI_B is in reset state (UCSWRST = 1). */
#define EUSCI_B1_I2C_I2COA0_RESERVED1__M                   (0x7800)              /*  */
#define EUSCI_B1_I2C_I2COA0_UCGCEN                         (0x8000)              /* General call response enable. This bit is only available in UCBxI2COA0. Can be  */
                                                                                 /* modified only when eUSCI_B is in reset state (UCSWRST = 1). */

/* EUSCI_B1_I2C_I2COA1 Control Bits */
#define EUSCI_B1_I2C_I2COA1_I2COA1__M                      (0x03ff)              /* I2C own address. The I2COAx bits contain the local address of the eUSCIx_B I2C  */
                                                                                 /* controller. The address is right justified. In 7-bit addressing mode, bit 6 is  */
                                                                                 /* the MSB and bits 9-7 are ignored. In 10-bit addressing mode, bit 9 is the MSB.  */
                                                                                 /* Can be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
#define EUSCI_B1_I2C_I2COA1_UCOAEN                         (0x0400)              /* Own Address enable register. With this register it can be selected if the I2C  */
                                                                                 /* slave-address related to this register UCBxI2COA1 is evaluated or not. Can be  */
                                                                                 /* modified only when eUSCI_B is in reset state (UCSWRST = 1). */
#define EUSCI_B1_I2C_I2COA1_RESERVED1__M                   (0xf800)              /*  */

/* EUSCI_B1_I2C_I2COA2 Control Bits */
#define EUSCI_B1_I2C_I2COA2_I2COA2__M                      (0x03ff)              /* I2C own address. The I2COAx bits contain the local address of the eUSCIx_B I2C  */
                                                                                 /* controller. The address is right justified. In 7-bit addressing mode, bit 6 is  */
                                                                                 /* the MSB and bits 9-7 are ignored. In 10-bit addressing mode, bit 9 is the MSB.  */
                                                                                 /* Can be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
#define EUSCI_B1_I2C_I2COA2_UCOAEN                         (0x0400)              /* Own Address enable register. With this register it can be selected if the I2C  */
                                                                                 /* slave-address related to this register UCBxI2COA2 is evaluated or not. Can be  */
                                                                                 /* modified only when eUSCI_B is in reset state (UCSWRST = 1). */
#define EUSCI_B1_I2C_I2COA2_RESERVED1__M                   (0xf800)              /*  */

/* EUSCI_B1_I2C_I2COA3 Control Bits */
#define EUSCI_B1_I2C_I2COA3_I2COA3__M                      (0x03ff)              /* I2C own address. The I2COA3 bits contain the local address of the eUSCIx_B I2C  */
                                                                                 /* controller. The address is right justified. In 7-bit addressing mode, bit 6 is  */
                                                                                 /* the MSB and bits 9-7 are ignored. In 10-bit addressing mode, bit 9 is the MSB.  */
                                                                                 /* Can be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
#define EUSCI_B1_I2C_I2COA3_UCOAEN                         (0x0400)              /* Own Address enable register. With this register it can be selected if the I2C  */
                                                                                 /* slave-address related to this register UCBxI2COA3 is evaluated or not. Can be  */
                                                                                 /* modified only when eUSCI_B is in reset state (UCSWRST = 1). */
#define EUSCI_B1_I2C_I2COA3_RESERVED1__M                   (0xf800)              /*  */

/* EUSCI_B1_I2C_ADDRX Control Bits */
#define EUSCI_B1_I2C_ADDRX_ADDRX__M                        (0x03ff)              /* Received Address Register. This register contains the last received slave  */
                                                                                 /* address on the bus. Using this register and the address mask register it is  */
                                                                                 /* possible to react on more than one slave address using one eUSCI_B module. */
#define EUSCI_B1_I2C_ADDRX_RESERVED1__M                    (0xfc00)              /*  */

/* EUSCI_B1_I2C_ADDMASK Control Bits */
#define EUSCI_B1_I2C_ADDMASK_ADDMASK__M                    (0x03ff)              /* Address Mask Register. By clearing the corresponding bit of the own address,  */
                                                                                 /* this bit is a don't care when comparing the address on the bus to the own  */
                                                                                 /* address. Using this method, it is possible to react on more than one slave  */
                                                                                 /* address. When all bits of ADDMASKx are set, the address mask feature is  */
                                                                                 /* deactivated. Can be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
#define EUSCI_B1_I2C_ADDMASK_RESERVED1__M                  (0xfc00)              /*  */

/* EUSCI_B1_I2C_I2CSA Control Bits */
#define EUSCI_B1_I2C_I2CSA_I2CSA__M                        (0x03ff)              /* I2C slave address. The I2CSAx bits contain the slave address of the external  */
                                                                                 /* device to be addressed by the eUSCIx_B module. It is only used in master mode.  */
                                                                                 /* The address is right justified. In 7-bit slave addressing mode, bit 6 is the MSB */
                                                                                 /* and bits 9-7 are ignored. In 10-bit slave addressing mode, bit 9 is the MSB. */
#define EUSCI_B1_I2C_I2CSA_RESERVED1__M                    (0xfc00)              /*  */

/* EUSCI_B1_I2C_IE Control Bits */
#define EUSCI_B1_I2C_IE_UCRXIE0                            (0x0001)              /* Receive interrupt enable 0 */
#define EUSCI_B1_I2C_IE_UCTXIE0                            (0x0002)              /* Transmit interrupt enable 0 */
#define EUSCI_B1_I2C_IE_UCSTTIE                            (0x0004)              /* START condition interrupt enable */
#define EUSCI_B1_I2C_IE_UCSTPIE                            (0x0008)              /* STOP condition interrupt enable */
#define EUSCI_B1_I2C_IE_UCALIE                             (0x0010)              /* Arbitration lost interrupt enable */
#define EUSCI_B1_I2C_IE_UCNACKIE                           (0x0020)              /* Not-acknowledge interrupt enable */
#define EUSCI_B1_I2C_IE_UCBCNTIE                           (0x0040)              /* Byte counter interrupt enable. */
#define EUSCI_B1_I2C_IE_UCCLTOIE                           (0x0080)              /* Clock low timeout interrupt enable. */
#define EUSCI_B1_I2C_IE_UCRXIE1                            (0x0100)              /* Receive interrupt enable 1 */
#define EUSCI_B1_I2C_IE_UCTXIE1                            (0x0200)              /* Transmit interrupt enable 1 */
#define EUSCI_B1_I2C_IE_UCRXIE2                            (0x0400)              /* Receive interrupt enable 2 */
#define EUSCI_B1_I2C_IE_UCTXIE2                            (0x0800)              /* Transmit interrupt enable 2 */
#define EUSCI_B1_I2C_IE_UCRXIE3                            (0x1000)              /* Receive interrupt enable 3 */
#define EUSCI_B1_I2C_IE_UCTXIE3                            (0x2000)              /* Transmit interrupt enable 3 */
#define EUSCI_B1_I2C_IE_UCBIT9IE                           (0x4000)              /* Bit position 9 interrupt enable */
#define EUSCI_B1_I2C_IE_RESERVED1                          (0x8000)              /*  */

/* EUSCI_B1_I2C_IFG Control Bits */
#define EUSCI_B1_I2C_IFG_UCRXIFG0                          (0x0001)              /* eUSCI_B receive interrupt flag 0. UCRXIFG0 is set when UCBxRXBUF has received a  */
                                                                                 /* complete character in master mode or in slave mode, if the slave address defined */
                                                                                 /* in UCBxI2COA0 was on the bus in the same frame. */
#define EUSCI_B1_I2C_IFG_UCTXIFG0                          (0x0002)              /* eUSCI_B transmit interrupt flag 0. UCTXIFG0 is set when UCBxTXBUF is empty in  */
                                                                                 /* master mode or in slave mode, if the slave address defined in UCBxI2COA0 was on  */
                                                                                 /* the bus in the same frame. */
#define EUSCI_B1_I2C_IFG_UCSTTIFG                          (0x0004)              /* START condition interrupt flag */
#define EUSCI_B1_I2C_IFG_UCSTPIFG                          (0x0008)              /* STOP condition interrupt flag */
#define EUSCI_B1_I2C_IFG_UCALIFG                           (0x0010)              /* Arbitration lost interrupt flag */
#define EUSCI_B1_I2C_IFG_UCNACKIFG                         (0x0020)              /* Not-acknowledge received interrupt flag. This flag only is updated when  */
                                                                                 /* operating in master mode. */
#define EUSCI_B1_I2C_IFG_UCBCNTIFG                         (0x0040)              /* Byte counter interrupt flag. When using this interrupt the user needs to ensure  */
                                                                                 /* enough processing bandwidth (see the Byte Counter Interrupt section). */
#define EUSCI_B1_I2C_IFG_UCCLTOIFG                         (0x0080)              /* Clock low timeout interrupt flag */
#define EUSCI_B1_I2C_IFG_UCRXIFG1                          (0x0100)              /* Receive interrupt flag 1. UCRXIFG1 is set when UCBxRXBUF has received a complete */
                                                                                 /* byte in slave mode and if the slave address defined in UCBxI2COA1 was on the  */
                                                                                 /* bus in the same frame. */
#define EUSCI_B1_I2C_IFG_UCTXIFG1                          (0x0200)              /* eUSCI_B transmit interrupt flag 1. UCTXIFG1 is set when UCBxTXBUF is empty in  */
                                                                                 /* slave mode, if the slave address defined in UCBxI2COA1 was on the bus in the  */
                                                                                 /* same frame. */
#define EUSCI_B1_I2C_IFG_UCRXIFG2                          (0x0400)              /* Receive interrupt flag 2. UCRXIFG2 is set when UCBxRXBUF has received a complete */
                                                                                 /* byte in slave mode and if the slave address defined in UCBxI2COA2 was on the  */
                                                                                 /* bus in the same frame. */
#define EUSCI_B1_I2C_IFG_UCTXIFG2                          (0x0800)              /* eUSCI_B transmit interrupt flag 2. UCTXIFG2 is set when UCBxTXBUF is empty in  */
                                                                                 /* slave mode, if the slave address defined in UCBxI2COA2 was on the bus in the  */
                                                                                 /* same frame. */
#define EUSCI_B1_I2C_IFG_UCRXIFG3                          (0x1000)              /* Receive interrupt flag 2. UCRXIFG2 is set when UCBxRXBUF has received a complete */
                                                                                 /* byte in slave mode and if the slave address defined in UCBxI2COA2 was on the  */
                                                                                 /* bus in the same frame. */
#define EUSCI_B1_I2C_IFG_UCTXIFG3                          (0x2000)              /* eUSCI_B transmit interrupt flag 3. UCTXIFG3 is set when UCBxTXBUF is empty in  */
                                                                                 /* slave mode, if the slave address defined in UCBxI2COA3 was on the bus in the  */
                                                                                 /* same frame. */
#define EUSCI_B1_I2C_IFG_UCBIT9IFG                         (0x4000)              /* Bit position 9 interrupt flag */
#define EUSCI_B1_I2C_IFG_RESERVED1                         (0x8000)              /*  */


//*****************************************************************************
// EUSCI_B1_SPI Registers
//*****************************************************************************
typedef struct {
  union {    /* EUSCI_B1_SPI_CTLW0 Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B1_SPI_CTLW0 Control Bits */
      __IO uint16_t UCSWRST             	:	1;  /* Software reset enable */
      __IO uint16_t UCSTEM              	:	1;  /* STE mode select in master mode. This byte is ignored in slave or 3-wire mode. */
      __I  uint16_t RESERVED1           	:	4;  /* Reserved */
      __IO uint16_t UCSSEL              	:	2;  /* eUSCI clock source select. These bits select the BRCLK source clock in master  */
                                                /* mode. UCxCLK is always used in slave mode. */
      __IO uint16_t UCSYNC              	:	1;  /* Synchronous mode enable */
      __IO uint16_t UCMODE              	:	2;  /* eUSCI mode. The UCMODEx bits select the synchronous mode when UCSYNC = 1. */
      __IO uint16_t UCMST               	:	1;  /* Master mode select */
      __IO uint16_t UC7BIT              	:	1;  /* Character length. Selects 7-bit or 8-bit character length. */
      __IO uint16_t UCMSB               	:	1;  /* MSB first select. Controls the direction of the receive and transmit shift  */
                                                /* register. */
      __IO uint16_t UCCKPL              	:	1;  /* Clock polarity select */
      __IO uint16_t UCCKPH              	:	1;  /* Clock phase select */

    } bit;
  } CTLW0;
  uint8_t RESERVED0[4];
  uint16_t BRW; /* eUSCI_Bx Bit Rate Control Word */
  union {    /* EUSCI_B1_SPI_STATW Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B1_SPI_STATW Control Bits */
      __I  uint16_t UCBUSY              	:	1;  /* eUSCI busy. This bit indicates if a transmit or receive operation is in progress */
                                                /* . */
      __I  uint16_t RESERVED2           	:	4;  /* Reserved */
      __IO uint16_t UCOE                	:	1;  /* Overrun error flag. This bit is set when a character is transferred into  */
                                                /* UCxRXBUF before the previous character was read. UCOE is cleared automatically  */
                                                /* when UCxRXBUF is read, and must not be cleared by software. Otherwise, it does  */
                                                /* not function correctly. */
      __IO uint16_t UCFE                	:	1;  /* Framing error flag. This bit indicates a bus conflict in 4-wire master mode.  */
                                                /* UCFE is not used in 3-wire master or any slave mode. */
      __IO uint16_t UCLISTEN            	:	1;  /* Listen enable. The UCLISTEN bit selects loopback mode. */
      __I  uint16_t RESERVED1           	:	8;  /* Reserved */

    } bit;
  } STATW;
  uint8_t RESERVED1[2];
  union {    /* EUSCI_B1_SPI_RXBUF Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B1_SPI_RXBUF Control Bits */
      __I  uint16_t UCRXBUF             	:	8;  /* The receive-data buffer is user accessible and contains the last received  */
                                                /* character from the receive shift register. Reading UCxRXBUF resets the receive- */
                                                /* error bits and UCRXIFG. In 7-bit data mode, UCxRXBUF is LSB justified and the  */
                                                /* MSB is always reset. */
      __I  uint16_t RESERVED1           	:	8;  /* Reserved */

    } bit;
  } RXBUF;
  union {    /* EUSCI_B1_SPI_TXBUF Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B1_SPI_TXBUF Control Bits */
      __IO uint16_t UCTXBUF             	:	8;  /* The transmit data buffer is user accessible and holds the data waiting to be  */
                                                /* moved into the transmit shift register and transmitted. Writing to the transmit  */
                                                /* data buffer clears UCTXIFG. The MSB of UCxTXBUF is not used for 7-bit data and  */
                                                /* is reset. */
      __I  uint16_t RESERVED1           	:	8;  /* Reserved */

    } bit;
  } TXBUF;
  uint8_t RESERVED2[26];
  union {    /* EUSCI_B1_SPI_IE Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B1_SPI_IE Control Bits */
      __IO uint16_t UCRXIE              	:	1;  /* Receive interrupt enable */
      __IO uint16_t UCTXIE              	:	1;  /* Transmit interrupt enable */
      __I  uint16_t RESERVED1           	:	14;  /* Reserved */

    } bit;
  } IE;
  union {    /* EUSCI_B1_SPI_IFG Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B1_SPI_IFG Control Bits */
      __IO uint16_t UCRXIFG             	:	1;  /* Receive interrupt flag. UCRXIFG is set when UCxxRXBUF has received a complete  */
                                                /* character. */
      __IO uint16_t UCTXIFG             	:	1;  /* Transmit interrupt flag. UCTXIFG is set when UCxxTXBUF empty. */
      __I  uint16_t RESERVED1           	:	14;  /* Reserved */

    } bit;
  } IFG;
  uint16_t IV; /* eUSCI_Bx Interrupt Vector */
} EUSCI_B1_SPI_Type;


/* EUSCI_B1_SPI_CTLW0 Control Bits */
#define EUSCI_B1_SPI_CTLW0_UCSWRST                         (0x0001)              /* Software reset enable */
#define EUSCI_B1_SPI_CTLW0_UCSTEM                          (0x0002)              /* STE mode select in master mode. This byte is ignored in slave or 3-wire mode. */
#define EUSCI_B1_SPI_CTLW0_RESERVED1__M                    (0x003c)              /*  */
#define EUSCI_B1_SPI_CTLW0_UCSSEL__M                       (0x00c0)              /* eUSCI clock source select. These bits select the BRCLK source clock in master  */
                                                                                 /* mode. UCxCLK is always used in slave mode. */
#define EUSCI_B1_SPI_CTLW0_UCSSEL__1                       (0x0040)              /* ACLK */
#define EUSCI_B1_SPI_CTLW0_UCSSEL__1_ACLK                  (0x0040)              /* ACLK */
#define EUSCI_B1_SPI_CTLW0_UCSSEL__2                       (0x0080)              /* SMCLK */
#define EUSCI_B1_SPI_CTLW0_UCSSEL__2_SMCLK                 (0x0080)              /* SMCLK */
#define EUSCI_B1_SPI_CTLW0_UCSSEL__3                       (0x00c0)              /* SMCLK */
#define EUSCI_B1_SPI_CTLW0_UCSSEL__3_SMCLK                 (0x00c0)              /* SMCLK */
#define EUSCI_B1_SPI_CTLW0_UCSYNC                          (0x0100)              /* Synchronous mode enable */
#define EUSCI_B1_SPI_CTLW0_UCMODE__M                       (0x0600)              /* eUSCI mode. The UCMODEx bits select the synchronous mode when UCSYNC = 1. */
#define EUSCI_B1_SPI_CTLW0_UCMODE__0                       (0x0000)              /* 3-pin SPI */
#define EUSCI_B1_SPI_CTLW0_UCMODE__0_3_PIN_SPI             (0x0000)              /* 3-pin SPI */
#define EUSCI_B1_SPI_CTLW0_UCMODE__1                       (0x0200)              /* 4-pin SPI with UCxSTE active high: Slave enabled when UCxSTE = 1 */
#define EUSCI_B1_SPI_CTLW0_UCMODE__2                       (0x0400)              /* 4-pin SPI with UCxSTE active low: Slave enabled when UCxSTE = 0 */
#define EUSCI_B1_SPI_CTLW0_UCMODE__3                       (0x0600)              /* I2C mode */
#define EUSCI_B1_SPI_CTLW0_UCMODE__3_I2C_MODE              (0x0600)              /* I2C mode */
#define EUSCI_B1_SPI_CTLW0_UCMST                           (0x0800)              /* Master mode select */
#define EUSCI_B1_SPI_CTLW0_UC7BIT                          (0x1000)              /* Character length. Selects 7-bit or 8-bit character length. */
#define EUSCI_B1_SPI_CTLW0_UCMSB                           (0x2000)              /* MSB first select. Controls the direction of the receive and transmit shift  */
                                                                                 /* register. */
#define EUSCI_B1_SPI_CTLW0_UCCKPL                          (0x4000)              /* Clock polarity select */
#define EUSCI_B1_SPI_CTLW0_UCCKPH                          (0x8000)              /* Clock phase select */

/* EUSCI_B1_SPI_STATW Control Bits */
#define EUSCI_B1_SPI_STATW_UCBUSY                          (0x0001)              /* eUSCI busy. This bit indicates if a transmit or receive operation is in progress */
                                                                                 /* . */
#define EUSCI_B1_SPI_STATW_RESERVED2__M                    (0x001e)              /*  */
#define EUSCI_B1_SPI_STATW_UCOE                            (0x0020)              /* Overrun error flag. This bit is set when a character is transferred into  */
                                                                                 /* UCxRXBUF before the previous character was read. UCOE is cleared automatically  */
                                                                                 /* when UCxRXBUF is read, and must not be cleared by software. Otherwise, it does  */
                                                                                 /* not function correctly. */
#define EUSCI_B1_SPI_STATW_UCFE                            (0x0040)              /* Framing error flag. This bit indicates a bus conflict in 4-wire master mode.  */
                                                                                 /* UCFE is not used in 3-wire master or any slave mode. */
#define EUSCI_B1_SPI_STATW_UCLISTEN                        (0x0080)              /* Listen enable. The UCLISTEN bit selects loopback mode. */
#define EUSCI_B1_SPI_STATW_RESERVED1__M                    (0xff00)              /*  */

/* EUSCI_B1_SPI_RXBUF Control Bits */
#define EUSCI_B1_SPI_RXBUF_UCRXBUF__M                      (0x00ff)              /* The receive-data buffer is user accessible and contains the last received  */
                                                                                 /* character from the receive shift register. Reading UCxRXBUF resets the receive- */
                                                                                 /* error bits and UCRXIFG. In 7-bit data mode, UCxRXBUF is LSB justified and the  */
                                                                                 /* MSB is always reset. */
#define EUSCI_B1_SPI_RXBUF_RESERVED1__M                    (0xff00)              /*  */

/* EUSCI_B1_SPI_TXBUF Control Bits */
#define EUSCI_B1_SPI_TXBUF_UCTXBUF__M                      (0x00ff)              /* The transmit data buffer is user accessible and holds the data waiting to be  */
                                                                                 /* moved into the transmit shift register and transmitted. Writing to the transmit  */
                                                                                 /* data buffer clears UCTXIFG. The MSB of UCxTXBUF is not used for 7-bit data and  */
                                                                                 /* is reset. */
#define EUSCI_B1_SPI_TXBUF_RESERVED1__M                    (0xff00)              /*  */

/* EUSCI_B1_SPI_IE Control Bits */
#define EUSCI_B1_SPI_IE_UCRXIE                             (0x0001)              /* Receive interrupt enable */
#define EUSCI_B1_SPI_IE_UCTXIE                             (0x0002)              /* Transmit interrupt enable */
#define EUSCI_B1_SPI_IE_RESERVED1__M                       (0xfffc)              /*  */

/* EUSCI_B1_SPI_IFG Control Bits */
#define EUSCI_B1_SPI_IFG_UCRXIFG                           (0x0001)              /* Receive interrupt flag. UCRXIFG is set when UCxxRXBUF has received a complete  */
                                                                                 /* character. */
#define EUSCI_B1_SPI_IFG_UCTXIFG                           (0x0002)              /* Transmit interrupt flag. UCTXIFG is set when UCxxTXBUF empty. */
#define EUSCI_B1_SPI_IFG_RESERVED1__M                      (0xfffc)              /*  */


//*****************************************************************************
// EUSCI_B2_I2C Registers
//*****************************************************************************
typedef struct {
  union {    /* EUSCI_B2_I2C_CTLW0 Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B2_I2C_CTLW0 Control Bits */
      __IO uint16_t UCSWRST             	:	1;  /* Software reset enable. Can be modified only when eUSCI_B is in reset state ( */
                                                /* UCSWRST = 1). */
      __IO uint16_t UCTXSTT             	:	1;  /* Transmit START condition in master mode. Ignored in slave mode. In master  */
                                                /* receiver mode, a repeated START condition is preceded by a NACK. UCTXSTT is  */
                                                /* automatically cleared after START condition and address information is  */
                                                /* transmitted. Ignored in slave mode. */
      __IO uint16_t UCTXSTP             	:	1;  /* Transmit STOP condition in master mode. Ignored in slave mode. In master  */
                                                /* receiver mode, the STOP condition is preceded by a NACK. UCTXSTP is  */
                                                /* automatically cleared after STOP is generated. This bit is a don't care, if  */
                                                /* automatic UCASTPx is different from 01 or 10.  */
      __IO uint16_t UCTXNACK            	:	1;  /* Transmit a NACK. UCTXNACK is automatically cleared after a NACK is transmitted.  */
                                                /* Only for slave receiver mode. */
      __IO uint16_t UCTR                	:	1;  /* Transmitter/receiver */
      __IO uint16_t UCTXACK             	:	1;  /* Transmit ACK condition in slave mode with enabled address mask register. After  */
                                                /* the UCSTTIFG has been set, the user needs to set or reset the UCTXACK flag to  */
                                                /* continue with the I2C protocol. The clock is stretched until the UCBxCTL1  */
                                                /* register has been written. This bit is cleared automatically after the ACK has  */
                                                /* been send. */
      __IO uint16_t UCSSEL              	:	2;  /* eUSCI_B clock source select. These bits select the BRCLK source clock. These  */
                                                /* bits are ignored in slave mode. Can be modified only when eUSCI_B is in reset  */
                                                /* state (UCSWRST = 1). */
      __IO uint16_t UCSYNC              	:	1;  /* Synchronous mode enable. For eUSCI_B always read and write as 1. */
      __IO uint16_t UCMODE              	:	2;  /* eUSCI_B mode. The UCMODEx bits select the synchronous mode when UCSYNC = 1. Can  */
                                                /* be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
      __IO uint16_t UCMST               	:	1;  /* Master mode select. When a master loses arbitration in a multi-master  */
                                                /* environment (UCMM = 1), the UCMST bit is automatically cleared and the module  */
                                                /* acts as slave. */
      __I  uint16_t RESERVED1           	:	1;  /* Reserved */
      __IO uint16_t UCMM                	:	1;  /* Multi-master environment select. Can be modified only when eUSCI_B is in reset  */
                                                /* state (UCSWRST = 1). */
      __IO uint16_t UCSLA10             	:	1;  /* Slave addressing mode select */
      __IO uint16_t UCA10               	:	1;  /* Own addressing mode select. Can be modified only when eUSCI_B is in reset state  */
                                                /* (UCSWRST = 1). */

    } bit;
  } CTLW0;
  union {    /* EUSCI_B2_I2C_CTLW1 Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B2_I2C_CTLW1 Control Bits */
      __IO uint16_t UCGLIT              	:	2;  /* Deglitch time */
      __IO uint16_t UCASTP              	:	2;  /* Automatic STOP condition generation. In slave mode only UCBCNTIFG is available.  */
                                                /* Can be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
      __IO uint16_t UCSWACK             	:	1;  /* Using this bit it is possible to select, whether the eUSCI_B module triggers the */
                                                /* sending of the ACK of the address or if it is controlled by software. */
      __IO uint16_t UCSTPNACK           	:	1;  /* The UCSTPNACK bit allows to make the eUSCI_B master acknowledge the last byte in */
                                                /* master receiver mode as well. This is not conform to the I2C specification and  */
                                                /* should only be used for slaves, which automatically release the SDA after a  */
                                                /* fixed packet length. Can be modified only when eUSCI_B is in reset state ( */
                                                /* UCSWRST = 1). */
      __IO uint16_t UCCLTO              	:	2;  /* Clock low timeout select. Modify only when UCSWRST = 1. */
      __IO uint16_t UCETXINT            	:	1;  /* Early UCTXIFG0. Only in slave mode. When this bit is set, the slave addresses  */
                                                /* defined in UCxI2COA1 to UCxI2COA3 must be disabled. Can be modified only when  */
                                                /* eUSCI_B is in reset state (UCSWRST = 1). */
      __I  uint16_t RESERVED1           	:	7;  /* Reserved */

    } bit;
  } CTLW1;
  uint8_t RESERVED0[2];
  uint16_t BRW; /* eUSCI_Bx Bit Rate Control Word */
  union {    /* EUSCI_B2_I2C_STATW Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B2_I2C_STATW Control Bits */
      __I  uint16_t RESERVED2           	:	4;  /* Reserved */
      __I  uint16_t UCBBUSY             	:	1;  /* Bus busy */
      __I  uint16_t UCGC                	:	1;  /* General call address received. UCGC is automatically cleared when a START  */
                                                /* condition is received. */
      __I  uint16_t UCSCLLOW            	:	1;  /* SCL low */
      __I  uint16_t RESERVED1           	:	1;  /* Reserved */
      __I  uint16_t UCBCNT              	:	8;  /* Hardware byte counter value. Reading this register returns the number of bytes  */
                                                /* received or transmitted on the I2C-Bus since the last START or RESTART. There is */
                                                /* no synchronization of this register done. When reading UCBxBCNT during the  */
                                                /* first bit position, a faulty readback can occur. */

    } bit;
  } STATW;
  union {    /* EUSCI_B2_I2C_TBCNT Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B2_I2C_TBCNT Control Bits */
      __IO uint16_t UCTBCNT             	:	8;  /* The byte counter threshold value is used to set the number of I2C data bytes  */
                                                /* after which the automatic STOP or the UCSTPIFG should occur. This value is  */
                                                /* evaluated only if UCASTPx is different from 00. Can be modified only when  */
                                                /* eUSCI_B is in reset state (UCSWRST = 1). */
      __I  uint16_t RESERVED1           	:	8;  /* Reserved */

    } bit;
  } TBCNT;
  union {    /* EUSCI_B2_I2C_RXBUF Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B2_I2C_RXBUF Control Bits */
      __I  uint16_t UCRXBUF             	:	8;  /* The receive-data buffer is user accessible and contains the last received  */
                                                /* character from the receive shift register. Reading UCBxRXBUF resets the UCRXIFGx */
                                                /* flags. */
      __I  uint16_t RESERVED1           	:	8;  /* Reserved */

    } bit;
  } RXBUF;
  union {    /* EUSCI_B2_I2C_TXBUF Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B2_I2C_TXBUF Control Bits */
      __IO uint16_t UCTXBUF             	:	8;  /* The transmit data buffer is user accessible and holds the data waiting to be  */
                                                /* moved into the transmit shift register and transmitted. Writing to the transmit  */
                                                /* data buffer clears the UCTXIFGx flags.  */
      __I  uint16_t RESERVED1           	:	8;  /* Reserved */

    } bit;
  } TXBUF;
  uint8_t RESERVED1[4];
  union {    /* EUSCI_B2_I2C_I2COA0 Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B2_I2C_I2COA0 Control Bits */
      __IO uint16_t I2COA               	:	10;  /* I2C own address. The I2COA0 bits contain the local address of the eUSCIx_B I2C  */
                                                /* controller. The address is right justified. In 7-bit addressing mode, bit 6 is  */
                                                /* the MSB and bits 9-7 are ignored. In 10-bit addressing mode, bit 9 is the MSB.  */
                                                /* Can be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
      __IO uint16_t UCOAEN              	:	1;  /* Own Address enable register. With this register it can be selected if the I2C  */
                                                /* slave-address related to this register UCBxI2COA0 is evaluated or not. Can be  */
                                                /* modified only when eUSCI_B is in reset state (UCSWRST = 1). */
      __I  uint16_t RESERVED1           	:	4;  /* Reserved */
      __IO uint16_t UCGCEN              	:	1;  /* General call response enable. This bit is only available in UCBxI2COA0. Can be  */
                                                /* modified only when eUSCI_B is in reset state (UCSWRST = 1). */

    } bit;
  } I2COA0;
  union {    /* EUSCI_B2_I2C_I2COA1 Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B2_I2C_I2COA1 Control Bits */
      __IO uint16_t I2COA1              	:	10;  /* I2C own address. The I2COAx bits contain the local address of the eUSCIx_B I2C  */
                                                /* controller. The address is right justified. In 7-bit addressing mode, bit 6 is  */
                                                /* the MSB and bits 9-7 are ignored. In 10-bit addressing mode, bit 9 is the MSB.  */
                                                /* Can be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
      __IO uint16_t UCOAEN              	:	1;  /* Own Address enable register. With this register it can be selected if the I2C  */
                                                /* slave-address related to this register UCBxI2COA1 is evaluated or not. Can be  */
                                                /* modified only when eUSCI_B is in reset state (UCSWRST = 1). */
      __I  uint16_t RESERVED1           	:	5;  /* Reserved */

    } bit;
  } I2COA1;
  union {    /* EUSCI_B2_I2C_I2COA2 Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B2_I2C_I2COA2 Control Bits */
      __IO uint16_t I2COA2              	:	10;  /* I2C own address. The I2COAx bits contain the local address of the eUSCIx_B I2C  */
                                                /* controller. The address is right justified. In 7-bit addressing mode, bit 6 is  */
                                                /* the MSB and bits 9-7 are ignored. In 10-bit addressing mode, bit 9 is the MSB.  */
                                                /* Can be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
      __IO uint16_t UCOAEN              	:	1;  /* Own Address enable register. With this register it can be selected if the I2C  */
                                                /* slave-address related to this register UCBxI2COA2 is evaluated or not. Can be  */
                                                /* modified only when eUSCI_B is in reset state (UCSWRST = 1). */
      __I  uint16_t RESERVED1           	:	5;  /* Reserved */

    } bit;
  } I2COA2;
  union {    /* EUSCI_B2_I2C_I2COA3 Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B2_I2C_I2COA3 Control Bits */
      __IO uint16_t I2COA3              	:	10;  /* I2C own address. The I2COA3 bits contain the local address of the eUSCIx_B I2C  */
                                                /* controller. The address is right justified. In 7-bit addressing mode, bit 6 is  */
                                                /* the MSB and bits 9-7 are ignored. In 10-bit addressing mode, bit 9 is the MSB.  */
                                                /* Can be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
      __IO uint16_t UCOAEN              	:	1;  /* Own Address enable register. With this register it can be selected if the I2C  */
                                                /* slave-address related to this register UCBxI2COA3 is evaluated or not. Can be  */
                                                /* modified only when eUSCI_B is in reset state (UCSWRST = 1). */
      __I  uint16_t RESERVED1           	:	5;  /* Reserved */

    } bit;
  } I2COA3;
  union {    /* EUSCI_B2_I2C_ADDRX Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B2_I2C_ADDRX Control Bits */
      __I  uint16_t ADDRX               	:	10;  /* Received Address Register. This register contains the last received slave  */
                                                /* address on the bus. Using this register and the address mask register it is  */
                                                /* possible to react on more than one slave address using one eUSCI_B module. */
      __I  uint16_t RESERVED1           	:	6;  /* Reserved */

    } bit;
  } ADDRX;
  union {    /* EUSCI_B2_I2C_ADDMASK Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B2_I2C_ADDMASK Control Bits */
      __IO uint16_t ADDMASK             	:	10;  /* Address Mask Register. By clearing the corresponding bit of the own address,  */
                                                /* this bit is a don't care when comparing the address on the bus to the own  */
                                                /* address. Using this method, it is possible to react on more than one slave  */
                                                /* address. When all bits of ADDMASKx are set, the address mask feature is  */
                                                /* deactivated. Can be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
      __I  uint16_t RESERVED1           	:	6;  /* Reserved */

    } bit;
  } ADDMASK;
  union {    /* EUSCI_B2_I2C_I2CSA Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B2_I2C_I2CSA Control Bits */
      __IO uint16_t I2CSA               	:	10;  /* I2C slave address. The I2CSAx bits contain the slave address of the external  */
                                                /* device to be addressed by the eUSCIx_B module. It is only used in master mode.  */
                                                /* The address is right justified. In 7-bit slave addressing mode, bit 6 is the MSB */
                                                /* and bits 9-7 are ignored. In 10-bit slave addressing mode, bit 9 is the MSB. */
      __I  uint16_t RESERVED1           	:	6;  /* Reserved */

    } bit;
  } I2CSA;
  uint8_t RESERVED2[8];
  union {    /* EUSCI_B2_I2C_IE Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B2_I2C_IE Control Bits */
      __IO uint16_t UCRXIE0             	:	1;  /* Receive interrupt enable 0 */
      __IO uint16_t UCTXIE0             	:	1;  /* Transmit interrupt enable 0 */
      __IO uint16_t UCSTTIE             	:	1;  /* START condition interrupt enable */
      __IO uint16_t UCSTPIE             	:	1;  /* STOP condition interrupt enable */
      __IO uint16_t UCALIE              	:	1;  /* Arbitration lost interrupt enable */
      __IO uint16_t UCNACKIE            	:	1;  /* Not-acknowledge interrupt enable */
      __IO uint16_t UCBCNTIE            	:	1;  /* Byte counter interrupt enable. */
      __IO uint16_t UCCLTOIE            	:	1;  /* Clock low timeout interrupt enable. */
      __IO uint16_t UCRXIE1             	:	1;  /* Receive interrupt enable 1 */
      __IO uint16_t UCTXIE1             	:	1;  /* Transmit interrupt enable 1 */
      __IO uint16_t UCRXIE2             	:	1;  /* Receive interrupt enable 2 */
      __IO uint16_t UCTXIE2             	:	1;  /* Transmit interrupt enable 2 */
      __IO uint16_t UCRXIE3             	:	1;  /* Receive interrupt enable 3 */
      __IO uint16_t UCTXIE3             	:	1;  /* Transmit interrupt enable 3 */
      __IO uint16_t UCBIT9IE            	:	1;  /* Bit position 9 interrupt enable */
      __I  uint16_t RESERVED1           	:	1;  /* Reserved */

    } bit;
  } IE;
  union {    /* EUSCI_B2_I2C_IFG Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B2_I2C_IFG Control Bits */
      __IO uint16_t UCRXIFG0            	:	1;  /* eUSCI_B receive interrupt flag 0. UCRXIFG0 is set when UCBxRXBUF has received a  */
                                                /* complete character in master mode or in slave mode, if the slave address defined */
                                                /* in UCBxI2COA0 was on the bus in the same frame. */
      __IO uint16_t UCTXIFG0            	:	1;  /* eUSCI_B transmit interrupt flag 0. UCTXIFG0 is set when UCBxTXBUF is empty in  */
                                                /* master mode or in slave mode, if the slave address defined in UCBxI2COA0 was on  */
                                                /* the bus in the same frame. */
      __IO uint16_t UCSTTIFG            	:	1;  /* START condition interrupt flag */
      __IO uint16_t UCSTPIFG            	:	1;  /* STOP condition interrupt flag */
      __IO uint16_t UCALIFG             	:	1;  /* Arbitration lost interrupt flag */
      __IO uint16_t UCNACKIFG           	:	1;  /* Not-acknowledge received interrupt flag. This flag only is updated when  */
                                                /* operating in master mode. */
      __IO uint16_t UCBCNTIFG           	:	1;  /* Byte counter interrupt flag. When using this interrupt the user needs to ensure  */
                                                /* enough processing bandwidth (see the Byte Counter Interrupt section). */
      __IO uint16_t UCCLTOIFG           	:	1;  /* Clock low timeout interrupt flag */
      __IO uint16_t UCRXIFG1            	:	1;  /* Receive interrupt flag 1. UCRXIFG1 is set when UCBxRXBUF has received a complete */
                                                /* byte in slave mode and if the slave address defined in UCBxI2COA1 was on the  */
                                                /* bus in the same frame. */
      __IO uint16_t UCTXIFG1            	:	1;  /* eUSCI_B transmit interrupt flag 1. UCTXIFG1 is set when UCBxTXBUF is empty in  */
                                                /* slave mode, if the slave address defined in UCBxI2COA1 was on the bus in the  */
                                                /* same frame. */
      __IO uint16_t UCRXIFG2            	:	1;  /* Receive interrupt flag 2. UCRXIFG2 is set when UCBxRXBUF has received a complete */
                                                /* byte in slave mode and if the slave address defined in UCBxI2COA2 was on the  */
                                                /* bus in the same frame. */
      __IO uint16_t UCTXIFG2            	:	1;  /* eUSCI_B transmit interrupt flag 2. UCTXIFG2 is set when UCBxTXBUF is empty in  */
                                                /* slave mode, if the slave address defined in UCBxI2COA2 was on the bus in the  */
                                                /* same frame. */
      __IO uint16_t UCRXIFG3            	:	1;  /* Receive interrupt flag 2. UCRXIFG2 is set when UCBxRXBUF has received a complete */
                                                /* byte in slave mode and if the slave address defined in UCBxI2COA2 was on the  */
                                                /* bus in the same frame. */
      __IO uint16_t UCTXIFG3            	:	1;  /* eUSCI_B transmit interrupt flag 3. UCTXIFG3 is set when UCBxTXBUF is empty in  */
                                                /* slave mode, if the slave address defined in UCBxI2COA3 was on the bus in the  */
                                                /* same frame. */
      __IO uint16_t UCBIT9IFG           	:	1;  /* Bit position 9 interrupt flag */
      __I  uint16_t RESERVED1           	:	1;  /* Reserved */

    } bit;
  } IFG;
  uint16_t IV; /* eUSCI_Bx Interrupt Vector */
} EUSCI_B2_I2C_Type;


/* EUSCI_B2_I2C_CTLW0 Control Bits */
#define EUSCI_B2_I2C_CTLW0_UCSWRST                         (0x0001)              /* Software reset enable. Can be modified only when eUSCI_B is in reset state ( */
                                                                                 /* UCSWRST = 1). */
#define EUSCI_B2_I2C_CTLW0_UCTXSTT                         (0x0002)              /* Transmit START condition in master mode. Ignored in slave mode. In master  */
                                                                                 /* receiver mode, a repeated START condition is preceded by a NACK. UCTXSTT is  */
                                                                                 /* automatically cleared after START condition and address information is  */
                                                                                 /* transmitted. Ignored in slave mode. */
#define EUSCI_B2_I2C_CTLW0_UCTXSTP                         (0x0004)              /* Transmit STOP condition in master mode. Ignored in slave mode. In master  */
                                                                                 /* receiver mode, the STOP condition is preceded by a NACK. UCTXSTP is  */
                                                                                 /* automatically cleared after STOP is generated. This bit is a don't care, if  */
                                                                                 /* automatic UCASTPx is different from 01 or 10.  */
#define EUSCI_B2_I2C_CTLW0_UCTXNACK                        (0x0008)              /* Transmit a NACK. UCTXNACK is automatically cleared after a NACK is transmitted.  */
                                                                                 /* Only for slave receiver mode. */
#define EUSCI_B2_I2C_CTLW0_UCTR                            (0x0010)              /* Transmitter/receiver */
#define EUSCI_B2_I2C_CTLW0_UCTXACK                         (0x0020)              /* Transmit ACK condition in slave mode with enabled address mask register. After  */
                                                                                 /* the UCSTTIFG has been set, the user needs to set or reset the UCTXACK flag to  */
                                                                                 /* continue with the I2C protocol. The clock is stretched until the UCBxCTL1  */
                                                                                 /* register has been written. This bit is cleared automatically after the ACK has  */
                                                                                 /* been send. */
#define EUSCI_B2_I2C_CTLW0_UCSSEL__M                       (0x00c0)              /* eUSCI_B clock source select. These bits select the BRCLK source clock. These  */
                                                                                 /* bits are ignored in slave mode. Can be modified only when eUSCI_B is in reset  */
                                                                                 /* state (UCSWRST = 1). */
#define EUSCI_B2_I2C_CTLW0_UCSSEL__0                       (0x0000)              /* UCLKI */
#define EUSCI_B2_I2C_CTLW0_UCSSEL__0_UCLKI                 (0x0000)              /* UCLKI */
#define EUSCI_B2_I2C_CTLW0_UCSSEL__1                       (0x0040)              /* ACLK */
#define EUSCI_B2_I2C_CTLW0_UCSSEL__1_ACLK                  (0x0040)              /* ACLK */
#define EUSCI_B2_I2C_CTLW0_UCSSEL__2                       (0x0080)              /* SMCLK */
#define EUSCI_B2_I2C_CTLW0_UCSSEL__2_SMCLK                 (0x0080)              /* SMCLK */
#define EUSCI_B2_I2C_CTLW0_UCSSEL__3                       (0x00c0)              /* SMCLK */
#define EUSCI_B2_I2C_CTLW0_UCSSEL__3_SMCLK                 (0x00c0)              /* SMCLK */
#define EUSCI_B2_I2C_CTLW0_UCSYNC                          (0x0100)              /* Synchronous mode enable. For eUSCI_B always read and write as 1. */
#define EUSCI_B2_I2C_CTLW0_UCMODE__M                       (0x0600)              /* eUSCI_B mode. The UCMODEx bits select the synchronous mode when UCSYNC = 1. Can  */
                                                                                 /* be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
#define EUSCI_B2_I2C_CTLW0_UCMODE__0                       (0x0000)              /* 3-pin SPI */
#define EUSCI_B2_I2C_CTLW0_UCMODE__0_3_PIN_SPI             (0x0000)              /* 3-pin SPI */
#define EUSCI_B2_I2C_CTLW0_UCMODE__1                       (0x0200)              /* 4-pin SPI (master/slave enabled if STE = 1) */
#define EUSCI_B2_I2C_CTLW0_UCMODE__2                       (0x0400)              /* 4-pin SPI (master/slave enabled if STE = 0) */
#define EUSCI_B2_I2C_CTLW0_UCMODE__3                       (0x0600)              /* I2C mode */
#define EUSCI_B2_I2C_CTLW0_UCMODE__3_I2C_MODE              (0x0600)              /* I2C mode */
#define EUSCI_B2_I2C_CTLW0_UCMST                           (0x0800)              /* Master mode select. When a master loses arbitration in a multi-master  */
                                                                                 /* environment (UCMM = 1), the UCMST bit is automatically cleared and the module  */
                                                                                 /* acts as slave. */
#define EUSCI_B2_I2C_CTLW0_RESERVED1                       (0x1000)              /*  */
#define EUSCI_B2_I2C_CTLW0_UCMM                            (0x2000)              /* Multi-master environment select. Can be modified only when eUSCI_B is in reset  */
                                                                                 /* state (UCSWRST = 1). */
#define EUSCI_B2_I2C_CTLW0_UCSLA10                         (0x4000)              /* Slave addressing mode select */
#define EUSCI_B2_I2C_CTLW0_UCA10                           (0x8000)              /* Own addressing mode select. Can be modified only when eUSCI_B is in reset state  */
                                                                                 /* (UCSWRST = 1). */

/* EUSCI_B2_I2C_CTLW1 Control Bits */
#define EUSCI_B2_I2C_CTLW1_UCGLIT__M                       (0x0003)              /* Deglitch time */
#define EUSCI_B2_I2C_CTLW1_UCGLIT__0                       (0x0000)              /* 50 ns */
#define EUSCI_B2_I2C_CTLW1_UCGLIT__0_50_NS                 (0x0000)              /* 50 ns */
#define EUSCI_B2_I2C_CTLW1_UCGLIT__1                       (0x0001)              /* 25 ns */
#define EUSCI_B2_I2C_CTLW1_UCGLIT__1_25_NS                 (0x0001)              /* 25 ns */
#define EUSCI_B2_I2C_CTLW1_UCGLIT__2                       (0x0002)              /* 12.5 ns */
#define EUSCI_B2_I2C_CTLW1_UCGLIT__2_12_5_NS               (0x0002)              /* 12.5 ns */
#define EUSCI_B2_I2C_CTLW1_UCGLIT__3                       (0x0003)              /* 6.25 ns */
#define EUSCI_B2_I2C_CTLW1_UCGLIT__3_6_25_NS               (0x0003)              /* 6.25 ns */
#define EUSCI_B2_I2C_CTLW1_UCASTP__M                       (0x000c)              /* Automatic STOP condition generation. In slave mode only UCBCNTIFG is available.  */
                                                                                 /* Can be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
#define EUSCI_B2_I2C_CTLW1_UCASTP__0                       (0x0000)              /* No automatic STOP generation. The STOP condition is generated after the user sets the UCTXSTP bit. The value in UCBxTBCNT is a don't care. */
#define EUSCI_B2_I2C_CTLW1_UCASTP__1                       (0x0004)              /* UCBCNTIFG is set with the byte counter reaches the threshold defined in UCBxTBCNT. */
#define EUSCI_B2_I2C_CTLW1_UCASTP__2                       (0x0008)              /* A STOP condition is generated automatically after the byte counter value reached UCBxTBCNT. UCBCNTIFG is set with the byte counter reaching the threshold. */
#define EUSCI_B2_I2C_CTLW1_UCSWACK                         (0x0010)              /* Using this bit it is possible to select, whether the eUSCI_B module triggers the */
                                                                                 /* sending of the ACK of the address or if it is controlled by software. */
#define EUSCI_B2_I2C_CTLW1_UCSTPNACK                       (0x0020)              /* The UCSTPNACK bit allows to make the eUSCI_B master acknowledge the last byte in */
                                                                                 /* master receiver mode as well. This is not conform to the I2C specification and  */
                                                                                 /* should only be used for slaves, which automatically release the SDA after a  */
                                                                                 /* fixed packet length. Can be modified only when eUSCI_B is in reset state ( */
                                                                                 /* UCSWRST = 1). */
#define EUSCI_B2_I2C_CTLW1_UCCLTO__M                       (0x00c0)              /* Clock low timeout select. Modify only when UCSWRST = 1. */
#define EUSCI_B2_I2C_CTLW1_UCCLTO__0                       (0x0000)              /* Disable clock low timeout counter */
#define EUSCI_B2_I2C_CTLW1_UCCLTO__1                       (0x0040)              /* 135 000 SYSCLK cycles (approximately 28 ms) */
#define EUSCI_B2_I2C_CTLW1_UCCLTO__2                       (0x0080)              /* 150 000 SYSCLK cycles (approximately 31 ms) */
#define EUSCI_B2_I2C_CTLW1_UCCLTO__3                       (0x00c0)              /* 165 000 SYSCLK cycles (approximately 34 ms) */
#define EUSCI_B2_I2C_CTLW1_UCETXINT                        (0x0100)              /* Early UCTXIFG0. Only in slave mode. When this bit is set, the slave addresses  */
                                                                                 /* defined in UCxI2COA1 to UCxI2COA3 must be disabled. Can be modified only when  */
                                                                                 /* eUSCI_B is in reset state (UCSWRST = 1). */
#define EUSCI_B2_I2C_CTLW1_RESERVED1__M                    (0xfe00)              /*  */

/* EUSCI_B2_I2C_STATW Control Bits */
#define EUSCI_B2_I2C_STATW_RESERVED2__M                    (0x000f)              /*  */
#define EUSCI_B2_I2C_STATW_UCBBUSY                         (0x0010)              /* Bus busy */
#define EUSCI_B2_I2C_STATW_UCGC                            (0x0020)              /* General call address received. UCGC is automatically cleared when a START  */
                                                                                 /* condition is received. */
#define EUSCI_B2_I2C_STATW_UCSCLLOW                        (0x0040)              /* SCL low */
#define EUSCI_B2_I2C_STATW_RESERVED1                       (0x0080)              /*  */
#define EUSCI_B2_I2C_STATW_UCBCNT__M                       (0xff00)              /* Hardware byte counter value. Reading this register returns the number of bytes  */
                                                                                 /* received or transmitted on the I2C-Bus since the last START or RESTART. There is */
                                                                                 /* no synchronization of this register done. When reading UCBxBCNT during the  */
                                                                                 /* first bit position, a faulty readback can occur. */

/* EUSCI_B2_I2C_TBCNT Control Bits */
#define EUSCI_B2_I2C_TBCNT_UCTBCNT__M                      (0x00ff)              /* The byte counter threshold value is used to set the number of I2C data bytes  */
                                                                                 /* after which the automatic STOP or the UCSTPIFG should occur. This value is  */
                                                                                 /* evaluated only if UCASTPx is different from 00. Can be modified only when  */
                                                                                 /* eUSCI_B is in reset state (UCSWRST = 1). */
#define EUSCI_B2_I2C_TBCNT_RESERVED1__M                    (0xff00)              /*  */

/* EUSCI_B2_I2C_RXBUF Control Bits */
#define EUSCI_B2_I2C_RXBUF_UCRXBUF__M                      (0x00ff)              /* The receive-data buffer is user accessible and contains the last received  */
                                                                                 /* character from the receive shift register. Reading UCBxRXBUF resets the UCRXIFGx */
                                                                                 /* flags. */
#define EUSCI_B2_I2C_RXBUF_RESERVED1__M                    (0xff00)              /*  */

/* EUSCI_B2_I2C_TXBUF Control Bits */
#define EUSCI_B2_I2C_TXBUF_UCTXBUF__M                      (0x00ff)              /* The transmit data buffer is user accessible and holds the data waiting to be  */
                                                                                 /* moved into the transmit shift register and transmitted. Writing to the transmit  */
                                                                                 /* data buffer clears the UCTXIFGx flags.  */
#define EUSCI_B2_I2C_TXBUF_RESERVED1__M                    (0xff00)              /*  */

/* EUSCI_B2_I2C_I2COA0 Control Bits */
#define EUSCI_B2_I2C_I2COA0_I2COA__M                       (0x03ff)              /* I2C own address. The I2COA0 bits contain the local address of the eUSCIx_B I2C  */
                                                                                 /* controller. The address is right justified. In 7-bit addressing mode, bit 6 is  */
                                                                                 /* the MSB and bits 9-7 are ignored. In 10-bit addressing mode, bit 9 is the MSB.  */
                                                                                 /* Can be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
#define EUSCI_B2_I2C_I2COA0_UCOAEN                         (0x0400)              /* Own Address enable register. With this register it can be selected if the I2C  */
                                                                                 /* slave-address related to this register UCBxI2COA0 is evaluated or not. Can be  */
                                                                                 /* modified only when eUSCI_B is in reset state (UCSWRST = 1). */
#define EUSCI_B2_I2C_I2COA0_RESERVED1__M                   (0x7800)              /*  */
#define EUSCI_B2_I2C_I2COA0_UCGCEN                         (0x8000)              /* General call response enable. This bit is only available in UCBxI2COA0. Can be  */
                                                                                 /* modified only when eUSCI_B is in reset state (UCSWRST = 1). */

/* EUSCI_B2_I2C_I2COA1 Control Bits */
#define EUSCI_B2_I2C_I2COA1_I2COA1__M                      (0x03ff)              /* I2C own address. The I2COAx bits contain the local address of the eUSCIx_B I2C  */
                                                                                 /* controller. The address is right justified. In 7-bit addressing mode, bit 6 is  */
                                                                                 /* the MSB and bits 9-7 are ignored. In 10-bit addressing mode, bit 9 is the MSB.  */
                                                                                 /* Can be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
#define EUSCI_B2_I2C_I2COA1_UCOAEN                         (0x0400)              /* Own Address enable register. With this register it can be selected if the I2C  */
                                                                                 /* slave-address related to this register UCBxI2COA1 is evaluated or not. Can be  */
                                                                                 /* modified only when eUSCI_B is in reset state (UCSWRST = 1). */
#define EUSCI_B2_I2C_I2COA1_RESERVED1__M                   (0xf800)              /*  */

/* EUSCI_B2_I2C_I2COA2 Control Bits */
#define EUSCI_B2_I2C_I2COA2_I2COA2__M                      (0x03ff)              /* I2C own address. The I2COAx bits contain the local address of the eUSCIx_B I2C  */
                                                                                 /* controller. The address is right justified. In 7-bit addressing mode, bit 6 is  */
                                                                                 /* the MSB and bits 9-7 are ignored. In 10-bit addressing mode, bit 9 is the MSB.  */
                                                                                 /* Can be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
#define EUSCI_B2_I2C_I2COA2_UCOAEN                         (0x0400)              /* Own Address enable register. With this register it can be selected if the I2C  */
                                                                                 /* slave-address related to this register UCBxI2COA2 is evaluated or not. Can be  */
                                                                                 /* modified only when eUSCI_B is in reset state (UCSWRST = 1). */
#define EUSCI_B2_I2C_I2COA2_RESERVED1__M                   (0xf800)              /*  */

/* EUSCI_B2_I2C_I2COA3 Control Bits */
#define EUSCI_B2_I2C_I2COA3_I2COA3__M                      (0x03ff)              /* I2C own address. The I2COA3 bits contain the local address of the eUSCIx_B I2C  */
                                                                                 /* controller. The address is right justified. In 7-bit addressing mode, bit 6 is  */
                                                                                 /* the MSB and bits 9-7 are ignored. In 10-bit addressing mode, bit 9 is the MSB.  */
                                                                                 /* Can be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
#define EUSCI_B2_I2C_I2COA3_UCOAEN                         (0x0400)              /* Own Address enable register. With this register it can be selected if the I2C  */
                                                                                 /* slave-address related to this register UCBxI2COA3 is evaluated or not. Can be  */
                                                                                 /* modified only when eUSCI_B is in reset state (UCSWRST = 1). */
#define EUSCI_B2_I2C_I2COA3_RESERVED1__M                   (0xf800)              /*  */

/* EUSCI_B2_I2C_ADDRX Control Bits */
#define EUSCI_B2_I2C_ADDRX_ADDRX__M                        (0x03ff)              /* Received Address Register. This register contains the last received slave  */
                                                                                 /* address on the bus. Using this register and the address mask register it is  */
                                                                                 /* possible to react on more than one slave address using one eUSCI_B module. */
#define EUSCI_B2_I2C_ADDRX_RESERVED1__M                    (0xfc00)              /*  */

/* EUSCI_B2_I2C_ADDMASK Control Bits */
#define EUSCI_B2_I2C_ADDMASK_ADDMASK__M                    (0x03ff)              /* Address Mask Register. By clearing the corresponding bit of the own address,  */
                                                                                 /* this bit is a don't care when comparing the address on the bus to the own  */
                                                                                 /* address. Using this method, it is possible to react on more than one slave  */
                                                                                 /* address. When all bits of ADDMASKx are set, the address mask feature is  */
                                                                                 /* deactivated. Can be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
#define EUSCI_B2_I2C_ADDMASK_RESERVED1__M                  (0xfc00)              /*  */

/* EUSCI_B2_I2C_I2CSA Control Bits */
#define EUSCI_B2_I2C_I2CSA_I2CSA__M                        (0x03ff)              /* I2C slave address. The I2CSAx bits contain the slave address of the external  */
                                                                                 /* device to be addressed by the eUSCIx_B module. It is only used in master mode.  */
                                                                                 /* The address is right justified. In 7-bit slave addressing mode, bit 6 is the MSB */
                                                                                 /* and bits 9-7 are ignored. In 10-bit slave addressing mode, bit 9 is the MSB. */
#define EUSCI_B2_I2C_I2CSA_RESERVED1__M                    (0xfc00)              /*  */

/* EUSCI_B2_I2C_IE Control Bits */
#define EUSCI_B2_I2C_IE_UCRXIE0                            (0x0001)              /* Receive interrupt enable 0 */
#define EUSCI_B2_I2C_IE_UCTXIE0                            (0x0002)              /* Transmit interrupt enable 0 */
#define EUSCI_B2_I2C_IE_UCSTTIE                            (0x0004)              /* START condition interrupt enable */
#define EUSCI_B2_I2C_IE_UCSTPIE                            (0x0008)              /* STOP condition interrupt enable */
#define EUSCI_B2_I2C_IE_UCALIE                             (0x0010)              /* Arbitration lost interrupt enable */
#define EUSCI_B2_I2C_IE_UCNACKIE                           (0x0020)              /* Not-acknowledge interrupt enable */
#define EUSCI_B2_I2C_IE_UCBCNTIE                           (0x0040)              /* Byte counter interrupt enable. */
#define EUSCI_B2_I2C_IE_UCCLTOIE                           (0x0080)              /* Clock low timeout interrupt enable. */
#define EUSCI_B2_I2C_IE_UCRXIE1                            (0x0100)              /* Receive interrupt enable 1 */
#define EUSCI_B2_I2C_IE_UCTXIE1                            (0x0200)              /* Transmit interrupt enable 1 */
#define EUSCI_B2_I2C_IE_UCRXIE2                            (0x0400)              /* Receive interrupt enable 2 */
#define EUSCI_B2_I2C_IE_UCTXIE2                            (0x0800)              /* Transmit interrupt enable 2 */
#define EUSCI_B2_I2C_IE_UCRXIE3                            (0x1000)              /* Receive interrupt enable 3 */
#define EUSCI_B2_I2C_IE_UCTXIE3                            (0x2000)              /* Transmit interrupt enable 3 */
#define EUSCI_B2_I2C_IE_UCBIT9IE                           (0x4000)              /* Bit position 9 interrupt enable */
#define EUSCI_B2_I2C_IE_RESERVED1                          (0x8000)              /*  */

/* EUSCI_B2_I2C_IFG Control Bits */
#define EUSCI_B2_I2C_IFG_UCRXIFG0                          (0x0001)              /* eUSCI_B receive interrupt flag 0. UCRXIFG0 is set when UCBxRXBUF has received a  */
                                                                                 /* complete character in master mode or in slave mode, if the slave address defined */
                                                                                 /* in UCBxI2COA0 was on the bus in the same frame. */
#define EUSCI_B2_I2C_IFG_UCTXIFG0                          (0x0002)              /* eUSCI_B transmit interrupt flag 0. UCTXIFG0 is set when UCBxTXBUF is empty in  */
                                                                                 /* master mode or in slave mode, if the slave address defined in UCBxI2COA0 was on  */
                                                                                 /* the bus in the same frame. */
#define EUSCI_B2_I2C_IFG_UCSTTIFG                          (0x0004)              /* START condition interrupt flag */
#define EUSCI_B2_I2C_IFG_UCSTPIFG                          (0x0008)              /* STOP condition interrupt flag */
#define EUSCI_B2_I2C_IFG_UCALIFG                           (0x0010)              /* Arbitration lost interrupt flag */
#define EUSCI_B2_I2C_IFG_UCNACKIFG                         (0x0020)              /* Not-acknowledge received interrupt flag. This flag only is updated when  */
                                                                                 /* operating in master mode. */
#define EUSCI_B2_I2C_IFG_UCBCNTIFG                         (0x0040)              /* Byte counter interrupt flag. When using this interrupt the user needs to ensure  */
                                                                                 /* enough processing bandwidth (see the Byte Counter Interrupt section). */
#define EUSCI_B2_I2C_IFG_UCCLTOIFG                         (0x0080)              /* Clock low timeout interrupt flag */
#define EUSCI_B2_I2C_IFG_UCRXIFG1                          (0x0100)              /* Receive interrupt flag 1. UCRXIFG1 is set when UCBxRXBUF has received a complete */
                                                                                 /* byte in slave mode and if the slave address defined in UCBxI2COA1 was on the  */
                                                                                 /* bus in the same frame. */
#define EUSCI_B2_I2C_IFG_UCTXIFG1                          (0x0200)              /* eUSCI_B transmit interrupt flag 1. UCTXIFG1 is set when UCBxTXBUF is empty in  */
                                                                                 /* slave mode, if the slave address defined in UCBxI2COA1 was on the bus in the  */
                                                                                 /* same frame. */
#define EUSCI_B2_I2C_IFG_UCRXIFG2                          (0x0400)              /* Receive interrupt flag 2. UCRXIFG2 is set when UCBxRXBUF has received a complete */
                                                                                 /* byte in slave mode and if the slave address defined in UCBxI2COA2 was on the  */
                                                                                 /* bus in the same frame. */
#define EUSCI_B2_I2C_IFG_UCTXIFG2                          (0x0800)              /* eUSCI_B transmit interrupt flag 2. UCTXIFG2 is set when UCBxTXBUF is empty in  */
                                                                                 /* slave mode, if the slave address defined in UCBxI2COA2 was on the bus in the  */
                                                                                 /* same frame. */
#define EUSCI_B2_I2C_IFG_UCRXIFG3                          (0x1000)              /* Receive interrupt flag 2. UCRXIFG2 is set when UCBxRXBUF has received a complete */
                                                                                 /* byte in slave mode and if the slave address defined in UCBxI2COA2 was on the  */
                                                                                 /* bus in the same frame. */
#define EUSCI_B2_I2C_IFG_UCTXIFG3                          (0x2000)              /* eUSCI_B transmit interrupt flag 3. UCTXIFG3 is set when UCBxTXBUF is empty in  */
                                                                                 /* slave mode, if the slave address defined in UCBxI2COA3 was on the bus in the  */
                                                                                 /* same frame. */
#define EUSCI_B2_I2C_IFG_UCBIT9IFG                         (0x4000)              /* Bit position 9 interrupt flag */
#define EUSCI_B2_I2C_IFG_RESERVED1                         (0x8000)              /*  */


//*****************************************************************************
// EUSCI_B2_SPI Registers
//*****************************************************************************
typedef struct {
  union {    /* EUSCI_B2_SPI_CTLW0 Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B2_SPI_CTLW0 Control Bits */
      __IO uint16_t UCSWRST             	:	1;  /* Software reset enable */
      __IO uint16_t UCSTEM              	:	1;  /* STE mode select in master mode. This byte is ignored in slave or 3-wire mode. */
      __I  uint16_t RESERVED1           	:	4;  /* Reserved */
      __IO uint16_t UCSSEL              	:	2;  /* eUSCI clock source select. These bits select the BRCLK source clock in master  */
                                                /* mode. UCxCLK is always used in slave mode. */
      __IO uint16_t UCSYNC              	:	1;  /* Synchronous mode enable */
      __IO uint16_t UCMODE              	:	2;  /* eUSCI mode. The UCMODEx bits select the synchronous mode when UCSYNC = 1. */
      __IO uint16_t UCMST               	:	1;  /* Master mode select */
      __IO uint16_t UC7BIT              	:	1;  /* Character length. Selects 7-bit or 8-bit character length. */
      __IO uint16_t UCMSB               	:	1;  /* MSB first select. Controls the direction of the receive and transmit shift  */
                                                /* register. */
      __IO uint16_t UCCKPL              	:	1;  /* Clock polarity select */
      __IO uint16_t UCCKPH              	:	1;  /* Clock phase select */

    } bit;
  } CTLW0;
  uint8_t RESERVED0[4];
  uint16_t BRW; /* eUSCI_Bx Bit Rate Control Word */
  union {    /* EUSCI_B2_SPI_STATW Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B2_SPI_STATW Control Bits */
      __I  uint16_t UCBUSY              	:	1;  /* eUSCI busy. This bit indicates if a transmit or receive operation is in progress */
                                                /* . */
      __I  uint16_t RESERVED2           	:	4;  /* Reserved */
      __IO uint16_t UCOE                	:	1;  /* Overrun error flag. This bit is set when a character is transferred into  */
                                                /* UCxRXBUF before the previous character was read. UCOE is cleared automatically  */
                                                /* when UCxRXBUF is read, and must not be cleared by software. Otherwise, it does  */
                                                /* not function correctly. */
      __IO uint16_t UCFE                	:	1;  /* Framing error flag. This bit indicates a bus conflict in 4-wire master mode.  */
                                                /* UCFE is not used in 3-wire master or any slave mode. */
      __IO uint16_t UCLISTEN            	:	1;  /* Listen enable. The UCLISTEN bit selects loopback mode. */
      __I  uint16_t RESERVED1           	:	8;  /* Reserved */

    } bit;
  } STATW;
  uint8_t RESERVED1[2];
  union {    /* EUSCI_B2_SPI_RXBUF Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B2_SPI_RXBUF Control Bits */
      __I  uint16_t UCRXBUF             	:	8;  /* The receive-data buffer is user accessible and contains the last received  */
                                                /* character from the receive shift register. Reading UCxRXBUF resets the receive- */
                                                /* error bits and UCRXIFG. In 7-bit data mode, UCxRXBUF is LSB justified and the  */
                                                /* MSB is always reset. */
      __I  uint16_t RESERVED1           	:	8;  /* Reserved */

    } bit;
  } RXBUF;
  union {    /* EUSCI_B2_SPI_TXBUF Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B2_SPI_TXBUF Control Bits */
      __IO uint16_t UCTXBUF             	:	8;  /* The transmit data buffer is user accessible and holds the data waiting to be  */
                                                /* moved into the transmit shift register and transmitted. Writing to the transmit  */
                                                /* data buffer clears UCTXIFG. The MSB of UCxTXBUF is not used for 7-bit data and  */
                                                /* is reset. */
      __I  uint16_t RESERVED1           	:	8;  /* Reserved */

    } bit;
  } TXBUF;
  uint8_t RESERVED2[26];
  union {    /* EUSCI_B2_SPI_IE Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B2_SPI_IE Control Bits */
      __IO uint16_t UCRXIE              	:	1;  /* Receive interrupt enable */
      __IO uint16_t UCTXIE              	:	1;  /* Transmit interrupt enable */
      __I  uint16_t RESERVED1           	:	14;  /* Reserved */

    } bit;
  } IE;
  union {    /* EUSCI_B2_SPI_IFG Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B2_SPI_IFG Control Bits */
      __IO uint16_t UCRXIFG             	:	1;  /* Receive interrupt flag. UCRXIFG is set when UCxxRXBUF has received a complete  */
                                                /* character. */
      __IO uint16_t UCTXIFG             	:	1;  /* Transmit interrupt flag. UCTXIFG is set when UCxxTXBUF empty. */
      __I  uint16_t RESERVED1           	:	14;  /* Reserved */

    } bit;
  } IFG;
  uint16_t IV; /* eUSCI_Bx Interrupt Vector */
} EUSCI_B2_SPI_Type;


/* EUSCI_B2_SPI_CTLW0 Control Bits */
#define EUSCI_B2_SPI_CTLW0_UCSWRST                         (0x0001)              /* Software reset enable */
#define EUSCI_B2_SPI_CTLW0_UCSTEM                          (0x0002)              /* STE mode select in master mode. This byte is ignored in slave or 3-wire mode. */
#define EUSCI_B2_SPI_CTLW0_RESERVED1__M                    (0x003c)              /*  */
#define EUSCI_B2_SPI_CTLW0_UCSSEL__M                       (0x00c0)              /* eUSCI clock source select. These bits select the BRCLK source clock in master  */
                                                                                 /* mode. UCxCLK is always used in slave mode. */
#define EUSCI_B2_SPI_CTLW0_UCSSEL__1                       (0x0040)              /* ACLK */
#define EUSCI_B2_SPI_CTLW0_UCSSEL__1_ACLK                  (0x0040)              /* ACLK */
#define EUSCI_B2_SPI_CTLW0_UCSSEL__2                       (0x0080)              /* SMCLK */
#define EUSCI_B2_SPI_CTLW0_UCSSEL__2_SMCLK                 (0x0080)              /* SMCLK */
#define EUSCI_B2_SPI_CTLW0_UCSSEL__3                       (0x00c0)              /* SMCLK */
#define EUSCI_B2_SPI_CTLW0_UCSSEL__3_SMCLK                 (0x00c0)              /* SMCLK */
#define EUSCI_B2_SPI_CTLW0_UCSYNC                          (0x0100)              /* Synchronous mode enable */
#define EUSCI_B2_SPI_CTLW0_UCMODE__M                       (0x0600)              /* eUSCI mode. The UCMODEx bits select the synchronous mode when UCSYNC = 1. */
#define EUSCI_B2_SPI_CTLW0_UCMODE__0                       (0x0000)              /* 3-pin SPI */
#define EUSCI_B2_SPI_CTLW0_UCMODE__0_3_PIN_SPI             (0x0000)              /* 3-pin SPI */
#define EUSCI_B2_SPI_CTLW0_UCMODE__1                       (0x0200)              /* 4-pin SPI with UCxSTE active high: Slave enabled when UCxSTE = 1 */
#define EUSCI_B2_SPI_CTLW0_UCMODE__2                       (0x0400)              /* 4-pin SPI with UCxSTE active low: Slave enabled when UCxSTE = 0 */
#define EUSCI_B2_SPI_CTLW0_UCMODE__3                       (0x0600)              /* I2C mode */
#define EUSCI_B2_SPI_CTLW0_UCMODE__3_I2C_MODE              (0x0600)              /* I2C mode */
#define EUSCI_B2_SPI_CTLW0_UCMST                           (0x0800)              /* Master mode select */
#define EUSCI_B2_SPI_CTLW0_UC7BIT                          (0x1000)              /* Character length. Selects 7-bit or 8-bit character length. */
#define EUSCI_B2_SPI_CTLW0_UCMSB                           (0x2000)              /* MSB first select. Controls the direction of the receive and transmit shift  */
                                                                                 /* register. */
#define EUSCI_B2_SPI_CTLW0_UCCKPL                          (0x4000)              /* Clock polarity select */
#define EUSCI_B2_SPI_CTLW0_UCCKPH                          (0x8000)              /* Clock phase select */

/* EUSCI_B2_SPI_STATW Control Bits */
#define EUSCI_B2_SPI_STATW_UCBUSY                          (0x0001)              /* eUSCI busy. This bit indicates if a transmit or receive operation is in progress */
                                                                                 /* . */
#define EUSCI_B2_SPI_STATW_RESERVED2__M                    (0x001e)              /*  */
#define EUSCI_B2_SPI_STATW_UCOE                            (0x0020)              /* Overrun error flag. This bit is set when a character is transferred into  */
                                                                                 /* UCxRXBUF before the previous character was read. UCOE is cleared automatically  */
                                                                                 /* when UCxRXBUF is read, and must not be cleared by software. Otherwise, it does  */
                                                                                 /* not function correctly. */
#define EUSCI_B2_SPI_STATW_UCFE                            (0x0040)              /* Framing error flag. This bit indicates a bus conflict in 4-wire master mode.  */
                                                                                 /* UCFE is not used in 3-wire master or any slave mode. */
#define EUSCI_B2_SPI_STATW_UCLISTEN                        (0x0080)              /* Listen enable. The UCLISTEN bit selects loopback mode. */
#define EUSCI_B2_SPI_STATW_RESERVED1__M                    (0xff00)              /*  */

/* EUSCI_B2_SPI_RXBUF Control Bits */
#define EUSCI_B2_SPI_RXBUF_UCRXBUF__M                      (0x00ff)              /* The receive-data buffer is user accessible and contains the last received  */
                                                                                 /* character from the receive shift register. Reading UCxRXBUF resets the receive- */
                                                                                 /* error bits and UCRXIFG. In 7-bit data mode, UCxRXBUF is LSB justified and the  */
                                                                                 /* MSB is always reset. */
#define EUSCI_B2_SPI_RXBUF_RESERVED1__M                    (0xff00)              /*  */

/* EUSCI_B2_SPI_TXBUF Control Bits */
#define EUSCI_B2_SPI_TXBUF_UCTXBUF__M                      (0x00ff)              /* The transmit data buffer is user accessible and holds the data waiting to be  */
                                                                                 /* moved into the transmit shift register and transmitted. Writing to the transmit  */
                                                                                 /* data buffer clears UCTXIFG. The MSB of UCxTXBUF is not used for 7-bit data and  */
                                                                                 /* is reset. */
#define EUSCI_B2_SPI_TXBUF_RESERVED1__M                    (0xff00)              /*  */

/* EUSCI_B2_SPI_IE Control Bits */
#define EUSCI_B2_SPI_IE_UCRXIE                             (0x0001)              /* Receive interrupt enable */
#define EUSCI_B2_SPI_IE_UCTXIE                             (0x0002)              /* Transmit interrupt enable */
#define EUSCI_B2_SPI_IE_RESERVED1__M                       (0xfffc)              /*  */

/* EUSCI_B2_SPI_IFG Control Bits */
#define EUSCI_B2_SPI_IFG_UCRXIFG                           (0x0001)              /* Receive interrupt flag. UCRXIFG is set when UCxxRXBUF has received a complete  */
                                                                                 /* character. */
#define EUSCI_B2_SPI_IFG_UCTXIFG                           (0x0002)              /* Transmit interrupt flag. UCTXIFG is set when UCxxTXBUF empty. */
#define EUSCI_B2_SPI_IFG_RESERVED1__M                      (0xfffc)              /*  */


//*****************************************************************************
// EUSCI_B3_I2C Registers
//*****************************************************************************
typedef struct {
  union {    /* EUSCI_B3_I2C_CTLW0 Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B3_I2C_CTLW0 Control Bits */
      __IO uint16_t UCSWRST             	:	1;  /* Software reset enable. Can be modified only when eUSCI_B is in reset state ( */
                                                /* UCSWRST = 1). */
      __IO uint16_t UCTXSTT             	:	1;  /* Transmit START condition in master mode. Ignored in slave mode. In master  */
                                                /* receiver mode, a repeated START condition is preceded by a NACK. UCTXSTT is  */
                                                /* automatically cleared after START condition and address information is  */
                                                /* transmitted. Ignored in slave mode. */
      __IO uint16_t UCTXSTP             	:	1;  /* Transmit STOP condition in master mode. Ignored in slave mode. In master  */
                                                /* receiver mode, the STOP condition is preceded by a NACK. UCTXSTP is  */
                                                /* automatically cleared after STOP is generated. This bit is a don't care, if  */
                                                /* automatic UCASTPx is different from 01 or 10.  */
      __IO uint16_t UCTXNACK            	:	1;  /* Transmit a NACK. UCTXNACK is automatically cleared after a NACK is transmitted.  */
                                                /* Only for slave receiver mode. */
      __IO uint16_t UCTR                	:	1;  /* Transmitter/receiver */
      __IO uint16_t UCTXACK             	:	1;  /* Transmit ACK condition in slave mode with enabled address mask register. After  */
                                                /* the UCSTTIFG has been set, the user needs to set or reset the UCTXACK flag to  */
                                                /* continue with the I2C protocol. The clock is stretched until the UCBxCTL1  */
                                                /* register has been written. This bit is cleared automatically after the ACK has  */
                                                /* been send. */
      __IO uint16_t UCSSEL              	:	2;  /* eUSCI_B clock source select. These bits select the BRCLK source clock. These  */
                                                /* bits are ignored in slave mode. Can be modified only when eUSCI_B is in reset  */
                                                /* state (UCSWRST = 1). */
      __IO uint16_t UCSYNC              	:	1;  /* Synchronous mode enable. For eUSCI_B always read and write as 1. */
      __IO uint16_t UCMODE              	:	2;  /* eUSCI_B mode. The UCMODEx bits select the synchronous mode when UCSYNC = 1. Can  */
                                                /* be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
      __IO uint16_t UCMST               	:	1;  /* Master mode select. When a master loses arbitration in a multi-master  */
                                                /* environment (UCMM = 1), the UCMST bit is automatically cleared and the module  */
                                                /* acts as slave. */
      __I  uint16_t RESERVED1           	:	1;  /* Reserved */
      __IO uint16_t UCMM                	:	1;  /* Multi-master environment select. Can be modified only when eUSCI_B is in reset  */
                                                /* state (UCSWRST = 1). */
      __IO uint16_t UCSLA10             	:	1;  /* Slave addressing mode select */
      __IO uint16_t UCA10               	:	1;  /* Own addressing mode select. Can be modified only when eUSCI_B is in reset state  */
                                                /* (UCSWRST = 1). */

    } bit;
  } CTLW0;
  union {    /* EUSCI_B3_I2C_CTLW1 Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B3_I2C_CTLW1 Control Bits */
      __IO uint16_t UCGLIT              	:	2;  /* Deglitch time */
      __IO uint16_t UCASTP              	:	2;  /* Automatic STOP condition generation. In slave mode only UCBCNTIFG is available.  */
                                                /* Can be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
      __IO uint16_t UCSWACK             	:	1;  /* Using this bit it is possible to select, whether the eUSCI_B module triggers the */
                                                /* sending of the ACK of the address or if it is controlled by software. */
      __IO uint16_t UCSTPNACK           	:	1;  /* The UCSTPNACK bit allows to make the eUSCI_B master acknowledge the last byte in */
                                                /* master receiver mode as well. This is not conform to the I2C specification and  */
                                                /* should only be used for slaves, which automatically release the SDA after a  */
                                                /* fixed packet length. Can be modified only when eUSCI_B is in reset state ( */
                                                /* UCSWRST = 1). */
      __IO uint16_t UCCLTO              	:	2;  /* Clock low timeout select. Modify only when UCSWRST = 1. */
      __IO uint16_t UCETXINT            	:	1;  /* Early UCTXIFG0. Only in slave mode. When this bit is set, the slave addresses  */
                                                /* defined in UCxI2COA1 to UCxI2COA3 must be disabled. Can be modified only when  */
                                                /* eUSCI_B is in reset state (UCSWRST = 1). */
      __I  uint16_t RESERVED1           	:	7;  /* Reserved */

    } bit;
  } CTLW1;
  uint8_t RESERVED0[2];
  uint16_t BRW; /* eUSCI_Bx Bit Rate Control Word */
  union {    /* EUSCI_B3_I2C_STATW Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B3_I2C_STATW Control Bits */
      __I  uint16_t RESERVED2           	:	4;  /* Reserved */
      __I  uint16_t UCBBUSY             	:	1;  /* Bus busy */
      __I  uint16_t UCGC                	:	1;  /* General call address received. UCGC is automatically cleared when a START  */
                                                /* condition is received. */
      __I  uint16_t UCSCLLOW            	:	1;  /* SCL low */
      __I  uint16_t RESERVED1           	:	1;  /* Reserved */
      __I  uint16_t UCBCNT              	:	8;  /* Hardware byte counter value. Reading this register returns the number of bytes  */
                                                /* received or transmitted on the I2C-Bus since the last START or RESTART. There is */
                                                /* no synchronization of this register done. When reading UCBxBCNT during the  */
                                                /* first bit position, a faulty readback can occur. */

    } bit;
  } STATW;
  union {    /* EUSCI_B3_I2C_TBCNT Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B3_I2C_TBCNT Control Bits */
      __IO uint16_t UCTBCNT             	:	8;  /* The byte counter threshold value is used to set the number of I2C data bytes  */
                                                /* after which the automatic STOP or the UCSTPIFG should occur. This value is  */
                                                /* evaluated only if UCASTPx is different from 00. Can be modified only when  */
                                                /* eUSCI_B is in reset state (UCSWRST = 1). */
      __I  uint16_t RESERVED1           	:	8;  /* Reserved */

    } bit;
  } TBCNT;
  union {    /* EUSCI_B3_I2C_RXBUF Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B3_I2C_RXBUF Control Bits */
      __I  uint16_t UCRXBUF             	:	8;  /* The receive-data buffer is user accessible and contains the last received  */
                                                /* character from the receive shift register. Reading UCBxRXBUF resets the UCRXIFGx */
                                                /* flags. */
      __I  uint16_t RESERVED1           	:	8;  /* Reserved */

    } bit;
  } RXBUF;
  union {    /* EUSCI_B3_I2C_TXBUF Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B3_I2C_TXBUF Control Bits */
      __IO uint16_t UCTXBUF             	:	8;  /* The transmit data buffer is user accessible and holds the data waiting to be  */
                                                /* moved into the transmit shift register and transmitted. Writing to the transmit  */
                                                /* data buffer clears the UCTXIFGx flags.  */
      __I  uint16_t RESERVED1           	:	8;  /* Reserved */

    } bit;
  } TXBUF;
  uint8_t RESERVED1[4];
  union {    /* EUSCI_B3_I2C_I2COA0 Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B3_I2C_I2COA0 Control Bits */
      __IO uint16_t I2COA               	:	10;  /* I2C own address. The I2COA0 bits contain the local address of the eUSCIx_B I2C  */
                                                /* controller. The address is right justified. In 7-bit addressing mode, bit 6 is  */
                                                /* the MSB and bits 9-7 are ignored. In 10-bit addressing mode, bit 9 is the MSB.  */
                                                /* Can be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
      __IO uint16_t UCOAEN              	:	1;  /* Own Address enable register. With this register it can be selected if the I2C  */
                                                /* slave-address related to this register UCBxI2COA0 is evaluated or not. Can be  */
                                                /* modified only when eUSCI_B is in reset state (UCSWRST = 1). */
      __I  uint16_t RESERVED1           	:	4;  /* Reserved */
      __IO uint16_t UCGCEN              	:	1;  /* General call response enable. This bit is only available in UCBxI2COA0. Can be  */
                                                /* modified only when eUSCI_B is in reset state (UCSWRST = 1). */

    } bit;
  } I2COA0;
  union {    /* EUSCI_B3_I2C_I2COA1 Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B3_I2C_I2COA1 Control Bits */
      __IO uint16_t I2COA1              	:	10;  /* I2C own address. The I2COAx bits contain the local address of the eUSCIx_B I2C  */
                                                /* controller. The address is right justified. In 7-bit addressing mode, bit 6 is  */
                                                /* the MSB and bits 9-7 are ignored. In 10-bit addressing mode, bit 9 is the MSB.  */
                                                /* Can be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
      __IO uint16_t UCOAEN              	:	1;  /* Own Address enable register. With this register it can be selected if the I2C  */
                                                /* slave-address related to this register UCBxI2COA1 is evaluated or not. Can be  */
                                                /* modified only when eUSCI_B is in reset state (UCSWRST = 1). */
      __I  uint16_t RESERVED1           	:	5;  /* Reserved */

    } bit;
  } I2COA1;
  union {    /* EUSCI_B3_I2C_I2COA2 Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B3_I2C_I2COA2 Control Bits */
      __IO uint16_t I2COA2              	:	10;  /* I2C own address. The I2COAx bits contain the local address of the eUSCIx_B I2C  */
                                                /* controller. The address is right justified. In 7-bit addressing mode, bit 6 is  */
                                                /* the MSB and bits 9-7 are ignored. In 10-bit addressing mode, bit 9 is the MSB.  */
                                                /* Can be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
      __IO uint16_t UCOAEN              	:	1;  /* Own Address enable register. With this register it can be selected if the I2C  */
                                                /* slave-address related to this register UCBxI2COA2 is evaluated or not. Can be  */
                                                /* modified only when eUSCI_B is in reset state (UCSWRST = 1). */
      __I  uint16_t RESERVED1           	:	5;  /* Reserved */

    } bit;
  } I2COA2;
  union {    /* EUSCI_B3_I2C_I2COA3 Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B3_I2C_I2COA3 Control Bits */
      __IO uint16_t I2COA3              	:	10;  /* I2C own address. The I2COA3 bits contain the local address of the eUSCIx_B I2C  */
                                                /* controller. The address is right justified. In 7-bit addressing mode, bit 6 is  */
                                                /* the MSB and bits 9-7 are ignored. In 10-bit addressing mode, bit 9 is the MSB.  */
                                                /* Can be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
      __IO uint16_t UCOAEN              	:	1;  /* Own Address enable register. With this register it can be selected if the I2C  */
                                                /* slave-address related to this register UCBxI2COA3 is evaluated or not. Can be  */
                                                /* modified only when eUSCI_B is in reset state (UCSWRST = 1). */
      __I  uint16_t RESERVED1           	:	5;  /* Reserved */

    } bit;
  } I2COA3;
  union {    /* EUSCI_B3_I2C_ADDRX Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B3_I2C_ADDRX Control Bits */
      __I  uint16_t ADDRX               	:	10;  /* Received Address Register. This register contains the last received slave  */
                                                /* address on the bus. Using this register and the address mask register it is  */
                                                /* possible to react on more than one slave address using one eUSCI_B module. */
      __I  uint16_t RESERVED1           	:	6;  /* Reserved */

    } bit;
  } ADDRX;
  union {    /* EUSCI_B3_I2C_ADDMASK Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B3_I2C_ADDMASK Control Bits */
      __IO uint16_t ADDMASK             	:	10;  /* Address Mask Register. By clearing the corresponding bit of the own address,  */
                                                /* this bit is a don't care when comparing the address on the bus to the own  */
                                                /* address. Using this method, it is possible to react on more than one slave  */
                                                /* address. When all bits of ADDMASKx are set, the address mask feature is  */
                                                /* deactivated. Can be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
      __I  uint16_t RESERVED1           	:	6;  /* Reserved */

    } bit;
  } ADDMASK;
  union {    /* EUSCI_B3_I2C_I2CSA Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B3_I2C_I2CSA Control Bits */
      __IO uint16_t I2CSA               	:	10;  /* I2C slave address. The I2CSAx bits contain the slave address of the external  */
                                                /* device to be addressed by the eUSCIx_B module. It is only used in master mode.  */
                                                /* The address is right justified. In 7-bit slave addressing mode, bit 6 is the MSB */
                                                /* and bits 9-7 are ignored. In 10-bit slave addressing mode, bit 9 is the MSB. */
      __I  uint16_t RESERVED1           	:	6;  /* Reserved */

    } bit;
  } I2CSA;
  uint8_t RESERVED2[8];
  union {    /* EUSCI_B3_I2C_IE Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B3_I2C_IE Control Bits */
      __IO uint16_t UCRXIE0             	:	1;  /* Receive interrupt enable 0 */
      __IO uint16_t UCTXIE0             	:	1;  /* Transmit interrupt enable 0 */
      __IO uint16_t UCSTTIE             	:	1;  /* START condition interrupt enable */
      __IO uint16_t UCSTPIE             	:	1;  /* STOP condition interrupt enable */
      __IO uint16_t UCALIE              	:	1;  /* Arbitration lost interrupt enable */
      __IO uint16_t UCNACKIE            	:	1;  /* Not-acknowledge interrupt enable */
      __IO uint16_t UCBCNTIE            	:	1;  /* Byte counter interrupt enable. */
      __IO uint16_t UCCLTOIE            	:	1;  /* Clock low timeout interrupt enable. */
      __IO uint16_t UCRXIE1             	:	1;  /* Receive interrupt enable 1 */
      __IO uint16_t UCTXIE1             	:	1;  /* Transmit interrupt enable 1 */
      __IO uint16_t UCRXIE2             	:	1;  /* Receive interrupt enable 2 */
      __IO uint16_t UCTXIE2             	:	1;  /* Transmit interrupt enable 2 */
      __IO uint16_t UCRXIE3             	:	1;  /* Receive interrupt enable 3 */
      __IO uint16_t UCTXIE3             	:	1;  /* Transmit interrupt enable 3 */
      __IO uint16_t UCBIT9IE            	:	1;  /* Bit position 9 interrupt enable */
      __I  uint16_t RESERVED1           	:	1;  /* Reserved */

    } bit;
  } IE;
  union {    /* EUSCI_B3_I2C_IFG Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B3_I2C_IFG Control Bits */
      __IO uint16_t UCRXIFG0            	:	1;  /* eUSCI_B receive interrupt flag 0. UCRXIFG0 is set when UCBxRXBUF has received a  */
                                                /* complete character in master mode or in slave mode, if the slave address defined */
                                                /* in UCBxI2COA0 was on the bus in the same frame. */
      __IO uint16_t UCTXIFG0            	:	1;  /* eUSCI_B transmit interrupt flag 0. UCTXIFG0 is set when UCBxTXBUF is empty in  */
                                                /* master mode or in slave mode, if the slave address defined in UCBxI2COA0 was on  */
                                                /* the bus in the same frame. */
      __IO uint16_t UCSTTIFG            	:	1;  /* START condition interrupt flag */
      __IO uint16_t UCSTPIFG            	:	1;  /* STOP condition interrupt flag */
      __IO uint16_t UCALIFG             	:	1;  /* Arbitration lost interrupt flag */
      __IO uint16_t UCNACKIFG           	:	1;  /* Not-acknowledge received interrupt flag. This flag only is updated when  */
                                                /* operating in master mode. */
      __IO uint16_t UCBCNTIFG           	:	1;  /* Byte counter interrupt flag. When using this interrupt the user needs to ensure  */
                                                /* enough processing bandwidth (see the Byte Counter Interrupt section). */
      __IO uint16_t UCCLTOIFG           	:	1;  /* Clock low timeout interrupt flag */
      __IO uint16_t UCRXIFG1            	:	1;  /* Receive interrupt flag 1. UCRXIFG1 is set when UCBxRXBUF has received a complete */
                                                /* byte in slave mode and if the slave address defined in UCBxI2COA1 was on the  */
                                                /* bus in the same frame. */
      __IO uint16_t UCTXIFG1            	:	1;  /* eUSCI_B transmit interrupt flag 1. UCTXIFG1 is set when UCBxTXBUF is empty in  */
                                                /* slave mode, if the slave address defined in UCBxI2COA1 was on the bus in the  */
                                                /* same frame. */
      __IO uint16_t UCRXIFG2            	:	1;  /* Receive interrupt flag 2. UCRXIFG2 is set when UCBxRXBUF has received a complete */
                                                /* byte in slave mode and if the slave address defined in UCBxI2COA2 was on the  */
                                                /* bus in the same frame. */
      __IO uint16_t UCTXIFG2            	:	1;  /* eUSCI_B transmit interrupt flag 2. UCTXIFG2 is set when UCBxTXBUF is empty in  */
                                                /* slave mode, if the slave address defined in UCBxI2COA2 was on the bus in the  */
                                                /* same frame. */
      __IO uint16_t UCRXIFG3            	:	1;  /* Receive interrupt flag 2. UCRXIFG2 is set when UCBxRXBUF has received a complete */
                                                /* byte in slave mode and if the slave address defined in UCBxI2COA2 was on the  */
                                                /* bus in the same frame. */
      __IO uint16_t UCTXIFG3            	:	1;  /* eUSCI_B transmit interrupt flag 3. UCTXIFG3 is set when UCBxTXBUF is empty in  */
                                                /* slave mode, if the slave address defined in UCBxI2COA3 was on the bus in the  */
                                                /* same frame. */
      __IO uint16_t UCBIT9IFG           	:	1;  /* Bit position 9 interrupt flag */
      __I  uint16_t RESERVED1           	:	1;  /* Reserved */

    } bit;
  } IFG;
  uint16_t IV; /* eUSCI_Bx Interrupt Vector */
} EUSCI_B3_I2C_Type;


/* EUSCI_B3_I2C_CTLW0 Control Bits */
#define EUSCI_B3_I2C_CTLW0_UCSWRST                         (0x0001)              /* Software reset enable. Can be modified only when eUSCI_B is in reset state ( */
                                                                                 /* UCSWRST = 1). */
#define EUSCI_B3_I2C_CTLW0_UCTXSTT                         (0x0002)              /* Transmit START condition in master mode. Ignored in slave mode. In master  */
                                                                                 /* receiver mode, a repeated START condition is preceded by a NACK. UCTXSTT is  */
                                                                                 /* automatically cleared after START condition and address information is  */
                                                                                 /* transmitted. Ignored in slave mode. */
#define EUSCI_B3_I2C_CTLW0_UCTXSTP                         (0x0004)              /* Transmit STOP condition in master mode. Ignored in slave mode. In master  */
                                                                                 /* receiver mode, the STOP condition is preceded by a NACK. UCTXSTP is  */
                                                                                 /* automatically cleared after STOP is generated. This bit is a don't care, if  */
                                                                                 /* automatic UCASTPx is different from 01 or 10.  */
#define EUSCI_B3_I2C_CTLW0_UCTXNACK                        (0x0008)              /* Transmit a NACK. UCTXNACK is automatically cleared after a NACK is transmitted.  */
                                                                                 /* Only for slave receiver mode. */
#define EUSCI_B3_I2C_CTLW0_UCTR                            (0x0010)              /* Transmitter/receiver */
#define EUSCI_B3_I2C_CTLW0_UCTXACK                         (0x0020)              /* Transmit ACK condition in slave mode with enabled address mask register. After  */
                                                                                 /* the UCSTTIFG has been set, the user needs to set or reset the UCTXACK flag to  */
                                                                                 /* continue with the I2C protocol. The clock is stretched until the UCBxCTL1  */
                                                                                 /* register has been written. This bit is cleared automatically after the ACK has  */
                                                                                 /* been send. */
#define EUSCI_B3_I2C_CTLW0_UCSSEL__M                       (0x00c0)              /* eUSCI_B clock source select. These bits select the BRCLK source clock. These  */
                                                                                 /* bits are ignored in slave mode. Can be modified only when eUSCI_B is in reset  */
                                                                                 /* state (UCSWRST = 1). */
#define EUSCI_B3_I2C_CTLW0_UCSSEL__0                       (0x0000)              /* UCLKI */
#define EUSCI_B3_I2C_CTLW0_UCSSEL__0_UCLKI                 (0x0000)              /* UCLKI */
#define EUSCI_B3_I2C_CTLW0_UCSSEL__1                       (0x0040)              /* ACLK */
#define EUSCI_B3_I2C_CTLW0_UCSSEL__1_ACLK                  (0x0040)              /* ACLK */
#define EUSCI_B3_I2C_CTLW0_UCSSEL__2                       (0x0080)              /* SMCLK */
#define EUSCI_B3_I2C_CTLW0_UCSSEL__2_SMCLK                 (0x0080)              /* SMCLK */
#define EUSCI_B3_I2C_CTLW0_UCSSEL__3                       (0x00c0)              /* SMCLK */
#define EUSCI_B3_I2C_CTLW0_UCSSEL__3_SMCLK                 (0x00c0)              /* SMCLK */
#define EUSCI_B3_I2C_CTLW0_UCSYNC                          (0x0100)              /* Synchronous mode enable. For eUSCI_B always read and write as 1. */
#define EUSCI_B3_I2C_CTLW0_UCMODE__M                       (0x0600)              /* eUSCI_B mode. The UCMODEx bits select the synchronous mode when UCSYNC = 1. Can  */
                                                                                 /* be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
#define EUSCI_B3_I2C_CTLW0_UCMODE__0                       (0x0000)              /* 3-pin SPI */
#define EUSCI_B3_I2C_CTLW0_UCMODE__0_3_PIN_SPI             (0x0000)              /* 3-pin SPI */
#define EUSCI_B3_I2C_CTLW0_UCMODE__1                       (0x0200)              /* 4-pin SPI (master/slave enabled if STE = 1) */
#define EUSCI_B3_I2C_CTLW0_UCMODE__2                       (0x0400)              /* 4-pin SPI (master/slave enabled if STE = 0) */
#define EUSCI_B3_I2C_CTLW0_UCMODE__3                       (0x0600)              /* I2C mode */
#define EUSCI_B3_I2C_CTLW0_UCMODE__3_I2C_MODE              (0x0600)              /* I2C mode */
#define EUSCI_B3_I2C_CTLW0_UCMST                           (0x0800)              /* Master mode select. When a master loses arbitration in a multi-master  */
                                                                                 /* environment (UCMM = 1), the UCMST bit is automatically cleared and the module  */
                                                                                 /* acts as slave. */
#define EUSCI_B3_I2C_CTLW0_RESERVED1                       (0x1000)              /*  */
#define EUSCI_B3_I2C_CTLW0_UCMM                            (0x2000)              /* Multi-master environment select. Can be modified only when eUSCI_B is in reset  */
                                                                                 /* state (UCSWRST = 1). */
#define EUSCI_B3_I2C_CTLW0_UCSLA10                         (0x4000)              /* Slave addressing mode select */
#define EUSCI_B3_I2C_CTLW0_UCA10                           (0x8000)              /* Own addressing mode select. Can be modified only when eUSCI_B is in reset state  */
                                                                                 /* (UCSWRST = 1). */

/* EUSCI_B3_I2C_CTLW1 Control Bits */
#define EUSCI_B3_I2C_CTLW1_UCGLIT__M                       (0x0003)              /* Deglitch time */
#define EUSCI_B3_I2C_CTLW1_UCGLIT__0                       (0x0000)              /* 50 ns */
#define EUSCI_B3_I2C_CTLW1_UCGLIT__0_50_NS                 (0x0000)              /* 50 ns */
#define EUSCI_B3_I2C_CTLW1_UCGLIT__1                       (0x0001)              /* 25 ns */
#define EUSCI_B3_I2C_CTLW1_UCGLIT__1_25_NS                 (0x0001)              /* 25 ns */
#define EUSCI_B3_I2C_CTLW1_UCGLIT__2                       (0x0002)              /* 12.5 ns */
#define EUSCI_B3_I2C_CTLW1_UCGLIT__2_12_5_NS               (0x0002)              /* 12.5 ns */
#define EUSCI_B3_I2C_CTLW1_UCGLIT__3                       (0x0003)              /* 6.25 ns */
#define EUSCI_B3_I2C_CTLW1_UCGLIT__3_6_25_NS               (0x0003)              /* 6.25 ns */
#define EUSCI_B3_I2C_CTLW1_UCASTP__M                       (0x000c)              /* Automatic STOP condition generation. In slave mode only UCBCNTIFG is available.  */
                                                                                 /* Can be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
#define EUSCI_B3_I2C_CTLW1_UCASTP__0                       (0x0000)              /* No automatic STOP generation. The STOP condition is generated after the user sets the UCTXSTP bit. The value in UCBxTBCNT is a don't care. */
#define EUSCI_B3_I2C_CTLW1_UCASTP__1                       (0x0004)              /* UCBCNTIFG is set with the byte counter reaches the threshold defined in UCBxTBCNT. */
#define EUSCI_B3_I2C_CTLW1_UCASTP__2                       (0x0008)              /* A STOP condition is generated automatically after the byte counter value reached UCBxTBCNT. UCBCNTIFG is set with the byte counter reaching the threshold. */
#define EUSCI_B3_I2C_CTLW1_UCSWACK                         (0x0010)              /* Using this bit it is possible to select, whether the eUSCI_B module triggers the */
                                                                                 /* sending of the ACK of the address or if it is controlled by software. */
#define EUSCI_B3_I2C_CTLW1_UCSTPNACK                       (0x0020)              /* The UCSTPNACK bit allows to make the eUSCI_B master acknowledge the last byte in */
                                                                                 /* master receiver mode as well. This is not conform to the I2C specification and  */
                                                                                 /* should only be used for slaves, which automatically release the SDA after a  */
                                                                                 /* fixed packet length. Can be modified only when eUSCI_B is in reset state ( */
                                                                                 /* UCSWRST = 1). */
#define EUSCI_B3_I2C_CTLW1_UCCLTO__M                       (0x00c0)              /* Clock low timeout select. Modify only when UCSWRST = 1. */
#define EUSCI_B3_I2C_CTLW1_UCCLTO__0                       (0x0000)              /* Disable clock low timeout counter */
#define EUSCI_B3_I2C_CTLW1_UCCLTO__1                       (0x0040)              /* 135 000 SYSCLK cycles (approximately 28 ms) */
#define EUSCI_B3_I2C_CTLW1_UCCLTO__2                       (0x0080)              /* 150 000 SYSCLK cycles (approximately 31 ms) */
#define EUSCI_B3_I2C_CTLW1_UCCLTO__3                       (0x00c0)              /* 165 000 SYSCLK cycles (approximately 34 ms) */
#define EUSCI_B3_I2C_CTLW1_UCETXINT                        (0x0100)              /* Early UCTXIFG0. Only in slave mode. When this bit is set, the slave addresses  */
                                                                                 /* defined in UCxI2COA1 to UCxI2COA3 must be disabled. Can be modified only when  */
                                                                                 /* eUSCI_B is in reset state (UCSWRST = 1). */
#define EUSCI_B3_I2C_CTLW1_RESERVED1__M                    (0xfe00)              /*  */

/* EUSCI_B3_I2C_STATW Control Bits */
#define EUSCI_B3_I2C_STATW_RESERVED2__M                    (0x000f)              /*  */
#define EUSCI_B3_I2C_STATW_UCBBUSY                         (0x0010)              /* Bus busy */
#define EUSCI_B3_I2C_STATW_UCGC                            (0x0020)              /* General call address received. UCGC is automatically cleared when a START  */
                                                                                 /* condition is received. */
#define EUSCI_B3_I2C_STATW_UCSCLLOW                        (0x0040)              /* SCL low */
#define EUSCI_B3_I2C_STATW_RESERVED1                       (0x0080)              /*  */
#define EUSCI_B3_I2C_STATW_UCBCNT__M                       (0xff00)              /* Hardware byte counter value. Reading this register returns the number of bytes  */
                                                                                 /* received or transmitted on the I2C-Bus since the last START or RESTART. There is */
                                                                                 /* no synchronization of this register done. When reading UCBxBCNT during the  */
                                                                                 /* first bit position, a faulty readback can occur. */

/* EUSCI_B3_I2C_TBCNT Control Bits */
#define EUSCI_B3_I2C_TBCNT_UCTBCNT__M                      (0x00ff)              /* The byte counter threshold value is used to set the number of I2C data bytes  */
                                                                                 /* after which the automatic STOP or the UCSTPIFG should occur. This value is  */
                                                                                 /* evaluated only if UCASTPx is different from 00. Can be modified only when  */
                                                                                 /* eUSCI_B is in reset state (UCSWRST = 1). */
#define EUSCI_B3_I2C_TBCNT_RESERVED1__M                    (0xff00)              /*  */

/* EUSCI_B3_I2C_RXBUF Control Bits */
#define EUSCI_B3_I2C_RXBUF_UCRXBUF__M                      (0x00ff)              /* The receive-data buffer is user accessible and contains the last received  */
                                                                                 /* character from the receive shift register. Reading UCBxRXBUF resets the UCRXIFGx */
                                                                                 /* flags. */
#define EUSCI_B3_I2C_RXBUF_RESERVED1__M                    (0xff00)              /*  */

/* EUSCI_B3_I2C_TXBUF Control Bits */
#define EUSCI_B3_I2C_TXBUF_UCTXBUF__M                      (0x00ff)              /* The transmit data buffer is user accessible and holds the data waiting to be  */
                                                                                 /* moved into the transmit shift register and transmitted. Writing to the transmit  */
                                                                                 /* data buffer clears the UCTXIFGx flags.  */
#define EUSCI_B3_I2C_TXBUF_RESERVED1__M                    (0xff00)              /*  */

/* EUSCI_B3_I2C_I2COA0 Control Bits */
#define EUSCI_B3_I2C_I2COA0_I2COA__M                       (0x03ff)              /* I2C own address. The I2COA0 bits contain the local address of the eUSCIx_B I2C  */
                                                                                 /* controller. The address is right justified. In 7-bit addressing mode, bit 6 is  */
                                                                                 /* the MSB and bits 9-7 are ignored. In 10-bit addressing mode, bit 9 is the MSB.  */
                                                                                 /* Can be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
#define EUSCI_B3_I2C_I2COA0_UCOAEN                         (0x0400)              /* Own Address enable register. With this register it can be selected if the I2C  */
                                                                                 /* slave-address related to this register UCBxI2COA0 is evaluated or not. Can be  */
                                                                                 /* modified only when eUSCI_B is in reset state (UCSWRST = 1). */
#define EUSCI_B3_I2C_I2COA0_RESERVED1__M                   (0x7800)              /*  */
#define EUSCI_B3_I2C_I2COA0_UCGCEN                         (0x8000)              /* General call response enable. This bit is only available in UCBxI2COA0. Can be  */
                                                                                 /* modified only when eUSCI_B is in reset state (UCSWRST = 1). */

/* EUSCI_B3_I2C_I2COA1 Control Bits */
#define EUSCI_B3_I2C_I2COA1_I2COA1__M                      (0x03ff)              /* I2C own address. The I2COAx bits contain the local address of the eUSCIx_B I2C  */
                                                                                 /* controller. The address is right justified. In 7-bit addressing mode, bit 6 is  */
                                                                                 /* the MSB and bits 9-7 are ignored. In 10-bit addressing mode, bit 9 is the MSB.  */
                                                                                 /* Can be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
#define EUSCI_B3_I2C_I2COA1_UCOAEN                         (0x0400)              /* Own Address enable register. With this register it can be selected if the I2C  */
                                                                                 /* slave-address related to this register UCBxI2COA1 is evaluated or not. Can be  */
                                                                                 /* modified only when eUSCI_B is in reset state (UCSWRST = 1). */
#define EUSCI_B3_I2C_I2COA1_RESERVED1__M                   (0xf800)              /*  */

/* EUSCI_B3_I2C_I2COA2 Control Bits */
#define EUSCI_B3_I2C_I2COA2_I2COA2__M                      (0x03ff)              /* I2C own address. The I2COAx bits contain the local address of the eUSCIx_B I2C  */
                                                                                 /* controller. The address is right justified. In 7-bit addressing mode, bit 6 is  */
                                                                                 /* the MSB and bits 9-7 are ignored. In 10-bit addressing mode, bit 9 is the MSB.  */
                                                                                 /* Can be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
#define EUSCI_B3_I2C_I2COA2_UCOAEN                         (0x0400)              /* Own Address enable register. With this register it can be selected if the I2C  */
                                                                                 /* slave-address related to this register UCBxI2COA2 is evaluated or not. Can be  */
                                                                                 /* modified only when eUSCI_B is in reset state (UCSWRST = 1). */
#define EUSCI_B3_I2C_I2COA2_RESERVED1__M                   (0xf800)              /*  */

/* EUSCI_B3_I2C_I2COA3 Control Bits */
#define EUSCI_B3_I2C_I2COA3_I2COA3__M                      (0x03ff)              /* I2C own address. The I2COA3 bits contain the local address of the eUSCIx_B I2C  */
                                                                                 /* controller. The address is right justified. In 7-bit addressing mode, bit 6 is  */
                                                                                 /* the MSB and bits 9-7 are ignored. In 10-bit addressing mode, bit 9 is the MSB.  */
                                                                                 /* Can be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
#define EUSCI_B3_I2C_I2COA3_UCOAEN                         (0x0400)              /* Own Address enable register. With this register it can be selected if the I2C  */
                                                                                 /* slave-address related to this register UCBxI2COA3 is evaluated or not. Can be  */
                                                                                 /* modified only when eUSCI_B is in reset state (UCSWRST = 1). */
#define EUSCI_B3_I2C_I2COA3_RESERVED1__M                   (0xf800)              /*  */

/* EUSCI_B3_I2C_ADDRX Control Bits */
#define EUSCI_B3_I2C_ADDRX_ADDRX__M                        (0x03ff)              /* Received Address Register. This register contains the last received slave  */
                                                                                 /* address on the bus. Using this register and the address mask register it is  */
                                                                                 /* possible to react on more than one slave address using one eUSCI_B module. */
#define EUSCI_B3_I2C_ADDRX_RESERVED1__M                    (0xfc00)              /*  */

/* EUSCI_B3_I2C_ADDMASK Control Bits */
#define EUSCI_B3_I2C_ADDMASK_ADDMASK__M                    (0x03ff)              /* Address Mask Register. By clearing the corresponding bit of the own address,  */
                                                                                 /* this bit is a don't care when comparing the address on the bus to the own  */
                                                                                 /* address. Using this method, it is possible to react on more than one slave  */
                                                                                 /* address. When all bits of ADDMASKx are set, the address mask feature is  */
                                                                                 /* deactivated. Can be modified only when eUSCI_B is in reset state (UCSWRST = 1). */
#define EUSCI_B3_I2C_ADDMASK_RESERVED1__M                  (0xfc00)              /*  */

/* EUSCI_B3_I2C_I2CSA Control Bits */
#define EUSCI_B3_I2C_I2CSA_I2CSA__M                        (0x03ff)              /* I2C slave address. The I2CSAx bits contain the slave address of the external  */
                                                                                 /* device to be addressed by the eUSCIx_B module. It is only used in master mode.  */
                                                                                 /* The address is right justified. In 7-bit slave addressing mode, bit 6 is the MSB */
                                                                                 /* and bits 9-7 are ignored. In 10-bit slave addressing mode, bit 9 is the MSB. */
#define EUSCI_B3_I2C_I2CSA_RESERVED1__M                    (0xfc00)              /*  */

/* EUSCI_B3_I2C_IE Control Bits */
#define EUSCI_B3_I2C_IE_UCRXIE0                            (0x0001)              /* Receive interrupt enable 0 */
#define EUSCI_B3_I2C_IE_UCTXIE0                            (0x0002)              /* Transmit interrupt enable 0 */
#define EUSCI_B3_I2C_IE_UCSTTIE                            (0x0004)              /* START condition interrupt enable */
#define EUSCI_B3_I2C_IE_UCSTPIE                            (0x0008)              /* STOP condition interrupt enable */
#define EUSCI_B3_I2C_IE_UCALIE                             (0x0010)              /* Arbitration lost interrupt enable */
#define EUSCI_B3_I2C_IE_UCNACKIE                           (0x0020)              /* Not-acknowledge interrupt enable */
#define EUSCI_B3_I2C_IE_UCBCNTIE                           (0x0040)              /* Byte counter interrupt enable. */
#define EUSCI_B3_I2C_IE_UCCLTOIE                           (0x0080)              /* Clock low timeout interrupt enable. */
#define EUSCI_B3_I2C_IE_UCRXIE1                            (0x0100)              /* Receive interrupt enable 1 */
#define EUSCI_B3_I2C_IE_UCTXIE1                            (0x0200)              /* Transmit interrupt enable 1 */
#define EUSCI_B3_I2C_IE_UCRXIE2                            (0x0400)              /* Receive interrupt enable 2 */
#define EUSCI_B3_I2C_IE_UCTXIE2                            (0x0800)              /* Transmit interrupt enable 2 */
#define EUSCI_B3_I2C_IE_UCRXIE3                            (0x1000)              /* Receive interrupt enable 3 */
#define EUSCI_B3_I2C_IE_UCTXIE3                            (0x2000)              /* Transmit interrupt enable 3 */
#define EUSCI_B3_I2C_IE_UCBIT9IE                           (0x4000)              /* Bit position 9 interrupt enable */
#define EUSCI_B3_I2C_IE_RESERVED1                          (0x8000)              /*  */

/* EUSCI_B3_I2C_IFG Control Bits */
#define EUSCI_B3_I2C_IFG_UCRXIFG0                          (0x0001)              /* eUSCI_B receive interrupt flag 0. UCRXIFG0 is set when UCBxRXBUF has received a  */
                                                                                 /* complete character in master mode or in slave mode, if the slave address defined */
                                                                                 /* in UCBxI2COA0 was on the bus in the same frame. */
#define EUSCI_B3_I2C_IFG_UCTXIFG0                          (0x0002)              /* eUSCI_B transmit interrupt flag 0. UCTXIFG0 is set when UCBxTXBUF is empty in  */
                                                                                 /* master mode or in slave mode, if the slave address defined in UCBxI2COA0 was on  */
                                                                                 /* the bus in the same frame. */
#define EUSCI_B3_I2C_IFG_UCSTTIFG                          (0x0004)              /* START condition interrupt flag */
#define EUSCI_B3_I2C_IFG_UCSTPIFG                          (0x0008)              /* STOP condition interrupt flag */
#define EUSCI_B3_I2C_IFG_UCALIFG                           (0x0010)              /* Arbitration lost interrupt flag */
#define EUSCI_B3_I2C_IFG_UCNACKIFG                         (0x0020)              /* Not-acknowledge received interrupt flag. This flag only is updated when  */
                                                                                 /* operating in master mode. */
#define EUSCI_B3_I2C_IFG_UCBCNTIFG                         (0x0040)              /* Byte counter interrupt flag. When using this interrupt the user needs to ensure  */
                                                                                 /* enough processing bandwidth (see the Byte Counter Interrupt section). */
#define EUSCI_B3_I2C_IFG_UCCLTOIFG                         (0x0080)              /* Clock low timeout interrupt flag */
#define EUSCI_B3_I2C_IFG_UCRXIFG1                          (0x0100)              /* Receive interrupt flag 1. UCRXIFG1 is set when UCBxRXBUF has received a complete */
                                                                                 /* byte in slave mode and if the slave address defined in UCBxI2COA1 was on the  */
                                                                                 /* bus in the same frame. */
#define EUSCI_B3_I2C_IFG_UCTXIFG1                          (0x0200)              /* eUSCI_B transmit interrupt flag 1. UCTXIFG1 is set when UCBxTXBUF is empty in  */
                                                                                 /* slave mode, if the slave address defined in UCBxI2COA1 was on the bus in the  */
                                                                                 /* same frame. */
#define EUSCI_B3_I2C_IFG_UCRXIFG2                          (0x0400)              /* Receive interrupt flag 2. UCRXIFG2 is set when UCBxRXBUF has received a complete */
                                                                                 /* byte in slave mode and if the slave address defined in UCBxI2COA2 was on the  */
                                                                                 /* bus in the same frame. */
#define EUSCI_B3_I2C_IFG_UCTXIFG2                          (0x0800)              /* eUSCI_B transmit interrupt flag 2. UCTXIFG2 is set when UCBxTXBUF is empty in  */
                                                                                 /* slave mode, if the slave address defined in UCBxI2COA2 was on the bus in the  */
                                                                                 /* same frame. */
#define EUSCI_B3_I2C_IFG_UCRXIFG3                          (0x1000)              /* Receive interrupt flag 2. UCRXIFG2 is set when UCBxRXBUF has received a complete */
                                                                                 /* byte in slave mode and if the slave address defined in UCBxI2COA2 was on the  */
                                                                                 /* bus in the same frame. */
#define EUSCI_B3_I2C_IFG_UCTXIFG3                          (0x2000)              /* eUSCI_B transmit interrupt flag 3. UCTXIFG3 is set when UCBxTXBUF is empty in  */
                                                                                 /* slave mode, if the slave address defined in UCBxI2COA3 was on the bus in the  */
                                                                                 /* same frame. */
#define EUSCI_B3_I2C_IFG_UCBIT9IFG                         (0x4000)              /* Bit position 9 interrupt flag */
#define EUSCI_B3_I2C_IFG_RESERVED1                         (0x8000)              /*  */


//*****************************************************************************
// EUSCI_B3_SPI Registers
//*****************************************************************************
typedef struct {
  union {    /* EUSCI_B3_SPI_CTLW0 Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B3_SPI_CTLW0 Control Bits */
      __IO uint16_t UCSWRST             	:	1;  /* Software reset enable */
      __IO uint16_t UCSTEM              	:	1;  /* STE mode select in master mode. This byte is ignored in slave or 3-wire mode. */
      __I  uint16_t RESERVED1           	:	4;  /* Reserved */
      __IO uint16_t UCSSEL              	:	2;  /* eUSCI clock source select. These bits select the BRCLK source clock in master  */
                                                /* mode. UCxCLK is always used in slave mode. */
      __IO uint16_t UCSYNC              	:	1;  /* Synchronous mode enable */
      __IO uint16_t UCMODE              	:	2;  /* eUSCI mode. The UCMODEx bits select the synchronous mode when UCSYNC = 1. */
      __IO uint16_t UCMST               	:	1;  /* Master mode select */
      __IO uint16_t UC7BIT              	:	1;  /* Character length. Selects 7-bit or 8-bit character length. */
      __IO uint16_t UCMSB               	:	1;  /* MSB first select. Controls the direction of the receive and transmit shift  */
                                                /* register. */
      __IO uint16_t UCCKPL              	:	1;  /* Clock polarity select */
      __IO uint16_t UCCKPH              	:	1;  /* Clock phase select */

    } bit;
  } CTLW0;
  uint8_t RESERVED0[4];
  uint16_t BRW; /* eUSCI_Bx Bit Rate Control Word */
  union {    /* EUSCI_B3_SPI_STATW Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B3_SPI_STATW Control Bits */
      __I  uint16_t UCBUSY              	:	1;  /* eUSCI busy. This bit indicates if a transmit or receive operation is in progress */
                                                /* . */
      __I  uint16_t RESERVED2           	:	4;  /* Reserved */
      __IO uint16_t UCOE                	:	1;  /* Overrun error flag. This bit is set when a character is transferred into  */
                                                /* UCxRXBUF before the previous character was read. UCOE is cleared automatically  */
                                                /* when UCxRXBUF is read, and must not be cleared by software. Otherwise, it does  */
                                                /* not function correctly. */
      __IO uint16_t UCFE                	:	1;  /* Framing error flag. This bit indicates a bus conflict in 4-wire master mode.  */
                                                /* UCFE is not used in 3-wire master or any slave mode. */
      __IO uint16_t UCLISTEN            	:	1;  /* Listen enable. The UCLISTEN bit selects loopback mode. */
      __I  uint16_t RESERVED1           	:	8;  /* Reserved */

    } bit;
  } STATW;
  uint8_t RESERVED1[2];
  union {    /* EUSCI_B3_SPI_RXBUF Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B3_SPI_RXBUF Control Bits */
      __I  uint16_t UCRXBUF             	:	8;  /* The receive-data buffer is user accessible and contains the last received  */
                                                /* character from the receive shift register. Reading UCxRXBUF resets the receive- */
                                                /* error bits and UCRXIFG. In 7-bit data mode, UCxRXBUF is LSB justified and the  */
                                                /* MSB is always reset. */
      __I  uint16_t RESERVED1           	:	8;  /* Reserved */

    } bit;
  } RXBUF;
  union {    /* EUSCI_B3_SPI_TXBUF Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B3_SPI_TXBUF Control Bits */
      __IO uint16_t UCTXBUF             	:	8;  /* The transmit data buffer is user accessible and holds the data waiting to be  */
                                                /* moved into the transmit shift register and transmitted. Writing to the transmit  */
                                                /* data buffer clears UCTXIFG. The MSB of UCxTXBUF is not used for 7-bit data and  */
                                                /* is reset. */
      __I  uint16_t RESERVED1           	:	8;  /* Reserved */

    } bit;
  } TXBUF;
  uint8_t RESERVED2[26];
  union {    /* EUSCI_B3_SPI_IE Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B3_SPI_IE Control Bits */
      __IO uint16_t UCRXIE              	:	1;  /* Receive interrupt enable */
      __IO uint16_t UCTXIE              	:	1;  /* Transmit interrupt enable */
      __I  uint16_t RESERVED1           	:	14;  /* Reserved */

    } bit;
  } IE;
  union {    /* EUSCI_B3_SPI_IFG Register */
    __IO uint16_t reg;
    struct { /* EUSCI_B3_SPI_IFG Control Bits */
      __IO uint16_t UCRXIFG             	:	1;  /* Receive interrupt flag. UCRXIFG is set when UCxxRXBUF has received a complete  */
                                                /* character. */
      __IO uint16_t UCTXIFG             	:	1;  /* Transmit interrupt flag. UCTXIFG is set when UCxxTXBUF empty. */
      __I  uint16_t RESERVED1           	:	14;  /* Reserved */

    } bit;
  } IFG;
  uint16_t IV; /* eUSCI_Bx Interrupt Vector */
} EUSCI_B3_SPI_Type;


/* EUSCI_B3_SPI_CTLW0 Control Bits */
#define EUSCI_B3_SPI_CTLW0_UCSWRST                         (0x0001)              /* Software reset enable */
#define EUSCI_B3_SPI_CTLW0_UCSTEM                          (0x0002)              /* STE mode select in master mode. This byte is ignored in slave or 3-wire mode. */
#define EUSCI_B3_SPI_CTLW0_RESERVED1__M                    (0x003c)              /*  */
#define EUSCI_B3_SPI_CTLW0_UCSSEL__M                       (0x00c0)              /* eUSCI clock source select. These bits select the BRCLK source clock in master  */
                                                                                 /* mode. UCxCLK is always used in slave mode. */
#define EUSCI_B3_SPI_CTLW0_UCSSEL__1                       (0x0040)              /* ACLK */
#define EUSCI_B3_SPI_CTLW0_UCSSEL__1_ACLK                  (0x0040)              /* ACLK */
#define EUSCI_B3_SPI_CTLW0_UCSSEL__2                       (0x0080)              /* SMCLK */
#define EUSCI_B3_SPI_CTLW0_UCSSEL__2_SMCLK                 (0x0080)              /* SMCLK */
#define EUSCI_B3_SPI_CTLW0_UCSSEL__3                       (0x00c0)              /* SMCLK */
#define EUSCI_B3_SPI_CTLW0_UCSSEL__3_SMCLK                 (0x00c0)              /* SMCLK */
#define EUSCI_B3_SPI_CTLW0_UCSYNC                          (0x0100)              /* Synchronous mode enable */
#define EUSCI_B3_SPI_CTLW0_UCMODE__M                       (0x0600)              /* eUSCI mode. The UCMODEx bits select the synchronous mode when UCSYNC = 1. */
#define EUSCI_B3_SPI_CTLW0_UCMODE__0                       (0x0000)              /* 3-pin SPI */
#define EUSCI_B3_SPI_CTLW0_UCMODE__0_3_PIN_SPI             (0x0000)              /* 3-pin SPI */
#define EUSCI_B3_SPI_CTLW0_UCMODE__1                       (0x0200)              /* 4-pin SPI with UCxSTE active high: Slave enabled when UCxSTE = 1 */
#define EUSCI_B3_SPI_CTLW0_UCMODE__2                       (0x0400)              /* 4-pin SPI with UCxSTE active low: Slave enabled when UCxSTE = 0 */
#define EUSCI_B3_SPI_CTLW0_UCMODE__3                       (0x0600)              /* I2C mode */
#define EUSCI_B3_SPI_CTLW0_UCMODE__3_I2C_MODE              (0x0600)              /* I2C mode */
#define EUSCI_B3_SPI_CTLW0_UCMST                           (0x0800)              /* Master mode select */
#define EUSCI_B3_SPI_CTLW0_UC7BIT                          (0x1000)              /* Character length. Selects 7-bit or 8-bit character length. */
#define EUSCI_B3_SPI_CTLW0_UCMSB                           (0x2000)              /* MSB first select. Controls the direction of the receive and transmit shift  */
                                                                                 /* register. */
#define EUSCI_B3_SPI_CTLW0_UCCKPL                          (0x4000)              /* Clock polarity select */
#define EUSCI_B3_SPI_CTLW0_UCCKPH                          (0x8000)              /* Clock phase select */

/* EUSCI_B3_SPI_STATW Control Bits */
#define EUSCI_B3_SPI_STATW_UCBUSY                          (0x0001)              /* eUSCI busy. This bit indicates if a transmit or receive operation is in progress */
                                                                                 /* . */
#define EUSCI_B3_SPI_STATW_RESERVED2__M                    (0x001e)              /*  */
#define EUSCI_B3_SPI_STATW_UCOE                            (0x0020)              /* Overrun error flag. This bit is set when a character is transferred into  */
                                                                                 /* UCxRXBUF before the previous character was read. UCOE is cleared automatically  */
                                                                                 /* when UCxRXBUF is read, and must not be cleared by software. Otherwise, it does  */
                                                                                 /* not function correctly. */
#define EUSCI_B3_SPI_STATW_UCFE                            (0x0040)              /* Framing error flag. This bit indicates a bus conflict in 4-wire master mode.  */
                                                                                 /* UCFE is not used in 3-wire master or any slave mode. */
#define EUSCI_B3_SPI_STATW_UCLISTEN                        (0x0080)              /* Listen enable. The UCLISTEN bit selects loopback mode. */
#define EUSCI_B3_SPI_STATW_RESERVED1__M                    (0xff00)              /*  */

/* EUSCI_B3_SPI_RXBUF Control Bits */
#define EUSCI_B3_SPI_RXBUF_UCRXBUF__M                      (0x00ff)              /* The receive-data buffer is user accessible and contains the last received  */
                                                                                 /* character from the receive shift register. Reading UCxRXBUF resets the receive- */
                                                                                 /* error bits and UCRXIFG. In 7-bit data mode, UCxRXBUF is LSB justified and the  */
                                                                                 /* MSB is always reset. */
#define EUSCI_B3_SPI_RXBUF_RESERVED1__M                    (0xff00)              /*  */

/* EUSCI_B3_SPI_TXBUF Control Bits */
#define EUSCI_B3_SPI_TXBUF_UCTXBUF__M                      (0x00ff)              /* The transmit data buffer is user accessible and holds the data waiting to be  */
                                                                                 /* moved into the transmit shift register and transmitted. Writing to the transmit  */
                                                                                 /* data buffer clears UCTXIFG. The MSB of UCxTXBUF is not used for 7-bit data and  */
                                                                                 /* is reset. */
#define EUSCI_B3_SPI_TXBUF_RESERVED1__M                    (0xff00)              /*  */

/* EUSCI_B3_SPI_IE Control Bits */
#define EUSCI_B3_SPI_IE_UCRXIE                             (0x0001)              /* Receive interrupt enable */
#define EUSCI_B3_SPI_IE_UCTXIE                             (0x0002)              /* Transmit interrupt enable */
#define EUSCI_B3_SPI_IE_RESERVED1__M                       (0xfffc)              /*  */

/* EUSCI_B3_SPI_IFG Control Bits */
#define EUSCI_B3_SPI_IFG_UCRXIFG                           (0x0001)              /* Receive interrupt flag. UCRXIFG is set when UCxxRXBUF has received a complete  */
                                                                                 /* character. */
#define EUSCI_B3_SPI_IFG_UCTXIFG                           (0x0002)              /* Transmit interrupt flag. UCTXIFG is set when UCxxTXBUF empty. */
#define EUSCI_B3_SPI_IFG_RESERVED1__M                      (0xfffc)              /*  */


//*****************************************************************************
// FLCTL Registers
//*****************************************************************************
typedef struct {
  union {    /* FLCTL_PWRSTAT Register */
    __IO uint32_t reg;
    struct { /* FLCTL_PWRSTAT Control Bits */
      __I  uint32_t PSTAT               	:	3;  /* Flash power status */
      __I  uint32_t LDOSTAT             	:	1;  /* PSS FLDO GOOD status */
      __I  uint32_t VREFSTAT            	:	1;  /* PSS VREF stable status */
      __I  uint32_t IREFSTAT            	:	1;  /* PSS IREF stable status */
      __I  uint32_t TRIMSTAT            	:	1;  /* PSS trim done status */
      __I  uint32_t RD_2T               	:	1;  /* Indicates if Flash is being accessed in 2T mode */
      __I  uint32_t RESERVED_1          	:	24;  /* Reserved */

    } bit;
  } PWRSTAT;
  uint8_t RESERVED0[12];
  union {    /* FLCTL_RDCTL_BNK0 Register */
    __IO uint32_t reg;
    struct { /* FLCTL_RDCTL_BNK0 Control Bits */
      __IO uint32_t RD_MODE             	:	4;  /* Flash read mode control setting for Bank 1001 = Read Margin 0B 1010 = Read  */
                                                /* Margin 1B All others = Reserved, bank is set to Normal read mode */
      __IO uint32_t BUFI                	:	1;  /* Enables read buffering feature for instruction fetches to this Bank */
      __IO uint32_t BUFD                	:	1;  /* Enables read buffering feature for data reads to this Bank */
      __I  uint32_t RESERVED_2          	:	2;  /* Reserved */
      __IO uint32_t INFO_PARI           	:	1;  /* Enable parity check on instruction fetches to INFO sectors in this Bank */
      __IO uint32_t INFO_PARD           	:	1;  /* Enable parity check on data reads to INFO sectors in this Bank */
      __IO uint32_t USR_PARI            	:	1;  /* Enable parity check on instruction fetches to User sectors in this Bank */
      __IO uint32_t USR_PARD            	:	1;  /* Enable parity check on data reads to User sectors in this Bank */
      __IO uint32_t WAIT                	:	4;  /* Defines the number of wait states required for a read operation to the bank 0000 */
                                                /* = 0 wait states 0001 = 1 wait states 0010 = 2 wait states 0011 = 3 wait states  */
                                                /* 0100 = 4 wait states 0101 = 5 wait states 0110 = 6 wait states 0111 = 7 wait  */
                                                /* states 1000 = 8 wait states 1001 = 9 wait states 1010 = 10 wait states 1011 = 11 */
                                                /* wait states 1100 = 12 wait states 1101 = 13 wait states 1110 = 14 wait states  */
                                                /* 1111 = 15 wait states */
      __I  uint32_t RD_MODE_STATUS      	:	4;  /* Reflects the current Read Mode of the Bank 1001 = Read Margin 0B 1010 = Read  */
                                                /* Margin 1B All others = Reserved, bank is set to Normal read mode */
      __I  uint32_t RESERVED_1          	:	12;  /* Reserved */

    } bit;
  } RDCTL_BNK0;
  union {    /* FLCTL_RDCTL_BNK1 Register */
    __IO uint32_t reg;
    struct { /* FLCTL_RDCTL_BNK1 Control Bits */
      __IO uint32_t RD_MODE             	:	4;  /* Flash read mode control setting for Bank 1001 = Read Margin 0B 1010 = Read  */
                                                /* Margin 1B All others = Reserved, bank is set to Normal read mode */
      __IO uint32_t BUFI                	:	1;  /* Enables read buffering feature for instruction fetches to this Bank */
      __IO uint32_t BUFD                	:	1;  /* Enables read buffering feature for data reads to this Bank */
      __I  uint32_t RESERVED_2          	:	2;  /* Reserved */
      __IO uint32_t INFO_PARI           	:	1;  /* Enable parity check on instruction fetches to INFO sectors in this Bank */
      __IO uint32_t INFO_PARD           	:	1;  /* Enable parity check on data reads to INFO sectors in this Bank */
      __IO uint32_t USR_PARI            	:	1;  /* Enable parity check on instruction fetches to User sectors in this Bank */
      __IO uint32_t USR_PARD            	:	1;  /* Enable parity check on data reads to User sectors in this Bank */
      __IO uint32_t WAIT                	:	4;  /* Defines the number of wait states required for a read operation to the bank 0000 */
                                                /* = 0 wait states 0001 = 1 wait states 0010 = 2 wait states 0011 = 3 wait states  */
                                                /* 0100 = 4 wait states 0101 = 5 wait states 0110 = 6 wait states 0111 = 7 wait  */
                                                /* states 1000 = 8 wait states 1001 = 9 wait states 1010 = 10 wait states 1011 = 11 */
                                                /* wait states 1100 = 12 wait states 1101 = 13 wait states 1110 = 14 wait states  */
                                                /* 1111 = 15 wait states */
      __I  uint32_t RD_MODE_STATUS      	:	4;  /* Reflects the current Read Mode of the Bank 1001 = Read Margin 0B 1010 = Read  */
                                                /* Margin 1B All others = Reserved, bank is set to Normal read mode */
      __I  uint32_t RESERVED_1          	:	12;  /* Reserved */

    } bit;
  } RDCTL_BNK1;
  uint8_t RESERVED1[8];
  union {    /* FLCTL_RDBRST_CTLSTAT Register */
    __IO uint32_t reg;
    struct { /* FLCTL_RDBRST_CTLSTAT Control Bits */
      __O  uint32_t START               	:	1;  /* Write 1 triggers start of burst/compare    operation */
      __IO uint32_t MEM_TYPE            	:	2;  /* Type of memory that burst is carried out on */
      __IO uint32_t STOP_FAIL           	:	1;  /* If set to 1, causes burst/compare operation to terminate on first compare  */
                                                /* mismatch */
      __IO uint32_t DATA_CMP            	:	1;  /* Data pattern used for comparison against memory read data */
      __IO uint32_t PAR_CMP             	:	1;  /* If set to 1, enables parity compare during the burst operation */
      __IO uint32_t TEST_EN             	:	1;  /* When   set to 1, enables comparison against test data compare registers */
      __I  uint32_t RESERVED_3          	:	9;  /* Reserved */
      __I  uint32_t BRST_STAT           	:	2;  /* Status of Burst/Compare operation */
      __I  uint32_t CMP_ERR             	:	1;  /* if 1, indicates that the Burst/Compare Operation encountered atleast one   data  */
                                                /* comparison error */
      __I  uint32_t ADDR_ERR            	:	1;  /* If 1, indicates that Burst/Compare Operation was terminated due to access to  */
                                                /* reserved memory */
      __I  uint32_t RESERVED_2          	:	3;  /* Reserved */
      __O  uint32_t CLR_STAT            	:	1;  /* Write '1' to clear status bits 19-16 of this register Write '0' has no effect */
      __I  uint32_t RESERVED_1          	:	8;  /* Reserved */

    } bit;
  } RDBRST_CTLSTAT;
  union {    /* FLCTL_RDBRST_STARTADDR Register */
    __IO uint32_t reg;
    struct { /* FLCTL_RDBRST_STARTADDR Control Bits */
      __I  uint32_t START_ADDRESS       	:	21;  /* Start Address of Burst Operation. Offset from 0h, with 0h as start address of  */
                                                /* the type of memory region selected Bits 3-0 are always 0 (forced 128bit boundary */
                                                /* ) */
      __I  uint32_t RESERVED_1          	:	11;  /* Reserved */

    } bit;
  } RDBRST_STARTADDR;
  union {    /* FLCTL_RDBRST_LEN Register */
    __IO uint32_t reg;
    struct { /* FLCTL_RDBRST_LEN Control Bits */
      __I  uint32_t BURST_LENGTH        	:	21;  /* Length of Burst Operation (in bytes). Bits 3-0 are always 0 (forced minimum  */
                                                /* resolution of 128bits) */
      __I  uint32_t RESERVED_1          	:	11;  /* Reserved */

    } bit;
  } RDBRST_LEN;
  uint8_t RESERVED2[16];
  union {    /* FLCTL_RDBRST_FAILADDR Register */
    __IO uint32_t reg;
    struct { /* FLCTL_RDBRST_FAILADDR Control Bits */
      __I  uint32_t FAIL_ADDRESS        	:	21;  /* Reflects address of last failed compare. Offset from 0h, with 0h as start  */
                                                /* address of the type of memory region selected Bits 3-0 are always 0 (forced  */
                                                /* 128bit boundary) */
      __I  uint32_t RESERVED_1          	:	11;  /* Reserved */

    } bit;
  } RDBRST_FAILADDR;
  union {    /* FLCTL_RDBRST_FAILCNT Register */
    __IO uint32_t reg;
    struct { /* FLCTL_RDBRST_FAILCNT Control Bits */
      __IO uint32_t FAIL_COUNT          	:	17;  /* Reflects number of failures encountered in burst operation. */
      __I  uint32_t RESERVED_1          	:	15;  /* Reserved */

    } bit;
  } RDBRST_FAILCNT;
  uint8_t RESERVED3[12];
  union {    /* FLCTL_PRG_CTLSTAT Register */
    __IO uint32_t reg;
    struct { /* FLCTL_PRG_CTLSTAT Control Bits */
      __IO uint32_t ENABLE              	:	1;  /* Master control for all word program operations */
      __IO uint32_t MODE                	:	1;  /* Controls write mode selected by application */
      __IO uint32_t VER_PRE             	:	1;  /* Controls automatic pre program verify operations */
      __IO uint32_t VER_PST             	:	1;  /* Controls automatic post program verify operations */
      __I  uint32_t RESERVED_2          	:	12;  /* Reserved */
      __I  uint32_t STATUS              	:	2;  /* Reflects the status of program operations in the Flash memory */
      __I  uint32_t BNK_ACT             	:	1;  /* Reflects which bank is currently undergoing a program operation (valid only if  */
                                                /* bits 17-16 don't show idle) */
      __I  uint32_t RESERVED_1          	:	13;  /* Reserved */

    } bit;
  } PRG_CTLSTAT;
  union {    /* FLCTL_PRGBRST_CTLSTAT Register */
    __IO uint32_t reg;
    struct { /* FLCTL_PRGBRST_CTLSTAT Control Bits */
      __O  uint32_t START               	:	1;  /* Write 1 triggers start of burst program operation */
      __IO uint32_t TYPE                	:	2;  /* Type of memory that burst program is carried out on */
      __IO uint32_t LEN                 	:	3;  /* Length of burst (in 128 bit granularity) */
      __IO uint32_t AUTO_PRE            	:	1;  /* Controls the Auto-Verify operation before the Burst Program */
      __IO uint32_t AUTO_PST            	:	1;  /* Controls the Auto-Verify operation after the Burst Program */
      __I  uint32_t RESERVED_3          	:	8;  /* Reserved */
      __I  uint32_t BURST_STATUS        	:	3;  /* At any point in time, it reflects the status of a Burst Operation */
      __I  uint32_t PRE_ERR             	:	1;  /* If 1, indicates that Burst Operation encountered preprogram auto-verify     */
                                                /* errors */
      __I  uint32_t PST_ERR             	:	1;  /* if 1, indicates that the Burst Operation encountered postprogram   auto-verify  */
                                                /* errors */
      __I  uint32_t ADDR_ERR            	:	1;  /* If 1, indicates that Burst Operation was terminated due to attempted   program  */
                                                /* of reserved memory */
      __I  uint32_t RESERVED_2          	:	1;  /* Reserved */
      __O  uint32_t CLR_STAT            	:	1;  /* Write '1' to clear status bits 21-16 of this register Write '0' has no effect */
      __I  uint32_t RESERVED_1          	:	8;  /* Reserved */

    } bit;
  } PRGBRST_CTLSTAT;
  union {    /* FLCTL_PRGBRST_STARTADDR Register */
    __IO uint32_t reg;
    struct { /* FLCTL_PRGBRST_STARTADDR Control Bits */
      __IO uint32_t START_ADDRESS       	:	22;  /* Start Address of Program Burst Operation. Offset from 0h, with 0h as start  */
                                                /* address of the type of memory region selected Bits 3-0 are always 0 (forced  */
                                                /* 128bit boundary) */
      __I  uint32_t RESERVED_1          	:	10;  /* Reserved */

    } bit;
  } PRGBRST_STARTADDR;
  uint8_t RESERVED4[4];
  uint32_t PRGBRST_DATA0_0; /* Program Burst Data0 Register0 */
  uint32_t PRGBRST_DATA0_1; /* Program Burst Data0 Register1 */
  uint32_t PRGBRST_DATA0_2; /* Program Burst Data0 Register2 */
  uint32_t PRGBRST_DATA0_3; /* Program Burst Data0 Register3 */
  uint32_t PRGBRST_DATA1_0; /* Program Burst Data1 Register0 */
  uint32_t PRGBRST_DATA1_1; /* Program Burst Data0 Register1 */
  uint32_t PRGBRST_DATA1_2; /* Program Burst Data0 Register2 */
  uint32_t PRGBRST_DATA1_3; /* Program Burst Data0 Register3 */
  uint32_t PRGBRST_DATA2_0; /* Program Burst Data0 Register0 */
  uint32_t PRGBRST_DATA2_1; /* Program Burst Data0 Register1 */
  uint32_t PRGBRST_DATA2_2; /* Program Burst Data0 Register2 */
  uint32_t PRGBRST_DATA2_3; /* Program Burst Data0 Register3 */
  uint32_t PRGBRST_DATA3_0; /* Program Burst Data0 Register0 */
  uint32_t PRGBRST_DATA3_1; /* Program Burst Data0 Register1 */
  uint32_t PRGBRST_DATA3_2; /* Program Burst Data0 Register2 */
  uint32_t PRGBRST_DATA3_3; /* Program Burst Data0 Register3 */
  union {    /* FLCTL_ERASE_CTLSTAT Register */
    __IO uint32_t reg;
    struct { /* FLCTL_ERASE_CTLSTAT Control Bits */
      __O  uint32_t START               	:	1;  /* Write 1 triggers start of Erase operation */
      __IO uint32_t MODE                	:	1;  /* Controls erase mode selected by application */
      __IO uint32_t TYPE                	:	2;  /* Type of memory that erase operation is carried out on (don't care if mass erase  */
                                                /* is set to 1) */
      __I  uint32_t RESERVED_2          	:	12;  /* Reserved */
      __I  uint32_t STATUS              	:	2;  /* Reflects the status of erase operations in the Flash memory */
      __I  uint32_t ADDR_ERR            	:	1;  /* If 1, indicates that Erase Operation was terminated due to attempted erase of  */
                                                /* reserved memory address */
      __O  uint32_t CLR_STAT            	:	1;  /* Write '1' to clear status bits 18-16 of this register Write '0' has no effect */
      __I  uint32_t RESERVED_1          	:	12;  /* Reserved */

    } bit;
  } ERASE_CTLSTAT;
  union {    /* FLCTL_ERASE_SECTADDR Register */
    __IO uint32_t reg;
    struct { /* FLCTL_ERASE_SECTADDR Control Bits */
      __IO uint32_t SECT_ADDRESS        	:	22;  /* Address of Sector being Erased. Offset from 0h, with 0h as start address of the  */
                                                /* type of memory region selected  If memory type is set to INFO/USER, Bits 11-0  */
                                                /* are always 0 (forced sector boundary of 4KB)  If memory type is set to ENGR,  */
                                                /* Bits 10-0 are always 0 (forced sector boundary of 2KB) */
      __I  uint32_t RESERVED_1          	:	10;  /* Reserved */

    } bit;
  } ERASE_SECTADDR;
  uint8_t RESERVED5[8];
  union {    /* FLCTL_INFOWEPROT_BNK0 Register */
    __IO uint32_t reg;
    struct { /* FLCTL_INFOWEPROT_BNK0 Control Bits */
      __IO uint32_t PROT0               	:	1;  /* If set to 1, protects Sector 0 from program or erase operations */
      __IO uint32_t PROT1               	:	1;  /* If set to 1, protects Sector 1 from program or erase operations */
      __I  uint32_t RESERVED_1          	:	30;  /* Reserved */

    } bit;
  } INFOWEPROT_BNK0;
  union {    /* FLCTL_USRWEPROT_BNK0 Register */
    __IO uint32_t reg;
    struct { /* FLCTL_USRWEPROT_BNK0 Control Bits */
      __IO uint32_t PROT0               	:	1;  /* If set to 1, protects Sector 0 from program or erase operations */
      __IO uint32_t PROT1               	:	1;  /* If set to 1, protects Sector 1 from program or erase operations */
      __IO uint32_t PROT2               	:	1;  /* If set to 1, protects Sector 2 from program or erase operations */
      __IO uint32_t PROT3               	:	1;  /* If set to 1, protects Sector 3 from program or erase operations */
      __IO uint32_t PROT4               	:	1;  /* If set to 1, protects Sector 4 from program or erase operations */
      __IO uint32_t PROT5               	:	1;  /* If set to 1, protects Sector 5 from program or erase operations */
      __IO uint32_t PROT6               	:	1;  /* If set to 1, protects Sector 6 from program or erase operations */
      __IO uint32_t PROT7               	:	1;  /* If set to 1, protects Sector 7 from program or erase operations */
      __IO uint32_t PROT8               	:	1;  /* If set to 1, protects Sector 8 from program or erase operations */
      __IO uint32_t PROT9               	:	1;  /* If set to 1, protects Sector 9 from program or erase operations */
      __IO uint32_t PROT10              	:	1;  /* If set to 1, protects Sector 10 from program or erase operations */
      __IO uint32_t PROT11              	:	1;  /* If set to 1, protects Sector 11 from program or erase operations */
      __IO uint32_t PROT12              	:	1;  /* If set to 1, protects Sector 12 from program or erase operations */
      __IO uint32_t PROT13              	:	1;  /* If set to 1, protects Sector 13 from program or erase operations */
      __IO uint32_t PROT14              	:	1;  /* If set to 1, protects Sector 14 from program or erase operations */
      __IO uint32_t PROT15              	:	1;  /* If set to 1, protects Sector 15 from program or erase operations */
      __IO uint32_t PROT16              	:	1;  /* If set to 1, protects Sector 16 from program or erase operations */
      __IO uint32_t PROT17              	:	1;  /* If set to 1, protects Sector 17 from program or erase operations */
      __IO uint32_t PROT18              	:	1;  /* If set to 1, protects Sector 18 from program or erase operations */
      __IO uint32_t PROT19              	:	1;  /* If set to 1, protects Sector 19 from program or erase operations */
      __IO uint32_t PROT20              	:	1;  /* If set to 1, protects Sector 20 from program or erase operations */
      __IO uint32_t PROT21              	:	1;  /* If set to 1, protects Sector 21 from program or erase operations */
      __IO uint32_t PROT22              	:	1;  /* If set to 1, protects Sector 22 from program or erase operations */
      __IO uint32_t PROT23              	:	1;  /* If set to 1, protects Sector 23 from program or erase operations */
      __IO uint32_t PROT24              	:	1;  /* If set to 1, protects Sector 24 from program or erase operations */
      __IO uint32_t PROT25              	:	1;  /* If set to 1, protects Sector 25 from program or erase operations */
      __IO uint32_t PROT26              	:	1;  /* If set to 1, protects Sector 26 from program or erase operations */
      __IO uint32_t PROT27              	:	1;  /* If set to 1, protects Sector 27 from program or erase operations */
      __IO uint32_t PROT28              	:	1;  /* If set to 1, protects Sector 28 from program or erase operations */
      __IO uint32_t PROT29              	:	1;  /* If set to 1, protects Sector 29 from program or erase operations */
      __IO uint32_t PROT30              	:	1;  /* If set to 1, protects Sector 30 from program or erase operations */
      __IO uint32_t PROT31              	:	1;  /* If set to 1, protects Sector 31 from program or erase operations */

    } bit;
  } USRWEPROT_BNK0;
  uint8_t RESERVED6[8];
  union {    /* FLCTL_INFOWEPROT_BNK1 Register */
    __IO uint32_t reg;
    struct { /* FLCTL_INFOWEPROT_BNK1 Control Bits */
      __IO uint32_t PROT0               	:	1;  /* If set to 1, protects Sector 0 from program or erase operations */
      __IO uint32_t PROT1               	:	1;  /* If set to 1, protects Sector 1 from program or erase operations */
      __I  uint32_t RESERVED_1          	:	30;  /* Reserved */

    } bit;
  } INFOWEPROT_BNK1;
  union {    /* FLCTL_USRWEPROT_BNK1 Register */
    __IO uint32_t reg;
    struct { /* FLCTL_USRWEPROT_BNK1 Control Bits */
      __IO uint32_t PROT0               	:	1;  /* If set to 1, protects Sector 0 from program or erase operations */
      __IO uint32_t PROT1               	:	1;  /* If set to 1, protects Sector 1 from program or erase operations */
      __IO uint32_t PROT2               	:	1;  /* If set to 1, protects Sector 2 from program or erase operations */
      __IO uint32_t PROT3               	:	1;  /* If set to 1, protects Sector 3 from program or erase operations */
      __IO uint32_t PROT4               	:	1;  /* If set to 1, protects Sector 4 from program or erase operations */
      __IO uint32_t PROT5               	:	1;  /* If set to 1, protects Sector 5 from program or erase operations */
      __IO uint32_t PROT6               	:	1;  /* If set to 1, protects Sector 6 from program or erase operations */
      __IO uint32_t PROT7               	:	1;  /* If set to 1, protects Sector 7 from program or erase operations */
      __IO uint32_t PROT8               	:	1;  /* If set to 1, protects Sector 8 from program or erase operations */
      __IO uint32_t PROT9               	:	1;  /* If set to 1, protects Sector 9 from program or erase operations */
      __IO uint32_t PROT10              	:	1;  /* If set to 1, protects Sector 10 from program or erase operations */
      __IO uint32_t PROT11              	:	1;  /* If set to 1, protects Sector 11 from program or erase operations */
      __IO uint32_t PROT12              	:	1;  /* If set to 1, protects Sector 12 from program or erase operations */
      __IO uint32_t PROT13              	:	1;  /* If set to 1, protects Sector 13 from program or erase operations */
      __IO uint32_t PROT14              	:	1;  /* If set to 1, protects Sector 14 from program or erase operations */
      __IO uint32_t PROT15              	:	1;  /* If set to 1, protects Sector 15 from program or erase operations */
      __IO uint32_t PROT16              	:	1;  /* If set to 1, protects Sector 16 from program or erase operations */
      __IO uint32_t PROT17              	:	1;  /* If set to 1, protects Sector 17 from program or erase operations */
      __IO uint32_t PROT18              	:	1;  /* If set to 1, protects Sector 18 from program or erase operations */
      __IO uint32_t PROT19              	:	1;  /* If set to 1, protects Sector 19 from program or erase operations */
      __IO uint32_t PROT20              	:	1;  /* If set to 1, protects Sector 20 from program or erase operations */
      __IO uint32_t PROT21              	:	1;  /* If set to 1, protects Sector 21 from program or erase operations */
      __IO uint32_t PROT22              	:	1;  /* If set to 1, protects Sector 22 from program or erase operations */
      __IO uint32_t PROT23              	:	1;  /* If set to 1, protects Sector 23 from program or erase operations */
      __IO uint32_t PROT24              	:	1;  /* If set to 1, protects Sector 24 from program or erase operations */
      __IO uint32_t PROT25              	:	1;  /* If set to 1, protects Sector 25 from program or erase operations */
      __IO uint32_t PROT26              	:	1;  /* If set to 1, protects Sector 26 from program or erase operations */
      __IO uint32_t PROT27              	:	1;  /* If set to 1, protects Sector 27 from program or erase operations */
      __IO uint32_t PROT28              	:	1;  /* If set to 1, protects Sector 28 from program or erase operations */
      __IO uint32_t PROT29              	:	1;  /* If set to 1, protects Sector 29 from program or erase operations */
      __IO uint32_t PROT30              	:	1;  /* If set to 1, protects Sector 30 from program or erase operations */
      __IO uint32_t PROT31              	:	1;  /* If set to 1, protects Sector 31 from program or erase operations */

    } bit;
  } USRWEPROT_BNK1;
  uint8_t RESERVED7[8];
  union {    /* FLCTL_BMRK_CTLSTAT Register */
    __IO uint32_t reg;
    struct { /* FLCTL_BMRK_CTLSTAT Control Bits */
      __IO uint32_t I_BMRK              	:	1;  /* When 1, increments the Instruction Benchmark count register on each     */
                                                /* instruction fetch to the Flash */
      __IO uint32_t D_BMRK              	:	1;  /* When 1, increments the Data Benchmark count register on each data read access to */
                                                /* the Flash */
      __IO uint32_t CMP_EN              	:	1;  /* When 1, enables comparison of the Instruction or Data Benchmark Registers  */
                                                /* against the threshold value */
      __IO uint32_t CMP_SEL             	:	1;  /* Selects which benchmark register should be compared against the threshold */
      __I  uint32_t RESERVED_1          	:	28;  /* Reserved */

    } bit;
  } BMRK_CTLSTAT;
  uint32_t BMRK_IFETCH; /* Flash Benchmark Instruction Fetch Count Register */
  uint32_t BMRK_DREAD; /* Flash Benchmark Data Read Count Register */
  uint32_t BMRK_CMP; /* Flash Benchmark Count Compare Register */
  uint8_t RESERVED8[16];
  union {    /* FLCTL_INTFLAG Register */
    __IO uint32_t reg;
    struct { /* FLCTL_INTFLAG Control Bits */
      __I  uint32_t RDBRST              	:	1;  /* If set to 1, indicates that the Read Burst/Compare operation is complete */
      __I  uint32_t AVPRE               	:	1;  /* If set to 1, indicates that the pre-program verify operation has detected an  */
                                                /* error */
      __I  uint32_t AVPST               	:	1;  /* If set to 1, indicates that the post-program verify operation has failed  */
                                                /* comparison */
      __I  uint32_t PRG                 	:	1;  /* If set to 1, indicates that a word Program operation is complete */
      __I  uint32_t PRGB                	:	1;  /* If set to 1, indicates that the configured Burst Program operation is complete */
      __I  uint32_t ERASE               	:	1;  /* If set to 1, indicates that the Erase operation is complete */
      __I  uint32_t PARBNK0             	:	1;  /* If set to 1, indicates an instruction/data parity error in Bank0 */
      __I  uint32_t PARBNK1             	:	1;  /* If set to 1, indicates an instruction/data parity error in Bank1 */
      __I  uint32_t BMRK                	:	1;  /* If set to 1, indicates that a Benchmark Compare match occurred */
      __I  uint32_t PRG_ERR             	:	1;  /* If set to 1, indicates a word composition error in full word write mode    ( */
                                                /* possible data loss due to writes crossing over to a new 128bit boundary before  */
                                                /* full word has been  composed) */
      __I  uint32_t RESERVED_1          	:	22;  /* Reserved */

    } bit;
  } INTFLAG;
  union {    /* FLCTL_INTEN Register */
    __IO uint32_t reg;
    struct { /* FLCTL_INTEN Control Bits */
      __IO uint32_t RDBRST              	:	1;  /* If set to 1, enables the Controller to generate an interrupt based on the  */
                                                /* corresponding bit in the FLCTL_INTFLAG_REG */
      __IO uint32_t AVPRE               	:	1;  /* If set to 1, enables the Controller to generate an interrupt based on the  */
                                                /* corresponding bit in the FLCTL_INTFLAG_REG */
      __IO uint32_t AVPST               	:	1;  /* If set to 1, enables the Controller to generate an interrupt based on the  */
                                                /* corresponding bit in the FLCTL_INTFLAG_REG */
      __IO uint32_t PRG                 	:	1;  /* If set to 1, enables the Controller to generate an interrupt based on the  */
                                                /* corresponding bit in the FLCTL_INTFLAG_REG */
      __IO uint32_t PRGB                	:	1;  /* If set to 1, enables the Controller to generate an interrupt based on the  */
                                                /* corresponding bit in the FLCTL_INTFLAG_REG */
      __IO uint32_t ERASE               	:	1;  /* If set to 1, enables the Controller to generate an interrupt based on the  */
                                                /* corresponding bit in the FLCTL_INTFLAG_REG */
      __IO uint32_t PARBNK0             	:	1;  /* If set to 1, enables the Controller to generate an interrupt based on the  */
                                                /* corresponding bit in the FLCTL_INTFLAG_REG */
      __IO uint32_t PARBNK1             	:	1;  /* If set to 1, enables the Controller to generate an interrupt based on the  */
                                                /* corresponding bit in the FLCTL_INTFLAG_REG */
      __IO uint32_t BMRK                	:	1;  /* If set to 1, enables the Controller to generate an interrupt based on the  */
                                                /* corresponding bit in the FLCTL_INTFLAG_REG */
      __IO uint32_t PRG_ERR             	:	1;  /* If set to 1, enables the Controller to generate an interrupt based on the  */
                                                /* corresponding bit in the FLCTL_INTFLAG_REG */
      __I  uint32_t RESERVED_1          	:	22;  /* Reserved */

    } bit;
  } INTEN;
  union {    /* FLCTL_INTCLR Register */
    __IO uint32_t reg;
    struct { /* FLCTL_INTCLR Control Bits */
      __O  uint32_t RDBRST              	:	1;  /* Write 1 clears the corresponding interrupt flag bit in the FLCTL_INTFLAG_REG */
      __O  uint32_t AVPRE               	:	1;  /* Write 1 clears the corresponding interrupt flag bit in the FLCTL_INTFLAG_REG */
      __O  uint32_t AVPST               	:	1;  /* Write 1 clears the corresponding interrupt flag bit in the FLCTL_INTFLAG_REG */
      __O  uint32_t PRG                 	:	1;  /* Write 1 clears the corresponding interrupt flag bit in the FLCTL_INTFLAG_REG */
      __O  uint32_t PRGB                	:	1;  /* Write 1 clears the corresponding interrupt flag bit in the FLCTL_INTFLAG_REG */
      __O  uint32_t ERASE               	:	1;  /* Write 1 clears the corresponding interrupt flag bit in the FLCTL_INTFLAG_REG */
      __O  uint32_t PARBNK0             	:	1;  /* Write 1 clears the corresponding interrupt flag bit in the    FLCTL_INTFLAG_REG */
      __O  uint32_t PARBNK1             	:	1;  /* Write 1 clears the corresponding interrupt flag bit in the    FLCTL_INTFLAG_REG */
      __O  uint32_t BMRK                	:	1;  /* Write 1 clears the corresponding interrupt flag bit in the FLCTL_INTFLAG_REG */
      __O  uint32_t PRG_ERR             	:	1;  /* Write 1 clears the corresponding interrupt flag bit in the    FLCTL_INTFLAG_REG */
      __I  uint32_t RESERVED_1          	:	22;  /* Reserved */

    } bit;
  } INTCLR;
  union {    /* FLCTL_INTSET Register */
    __IO uint32_t reg;
    struct { /* FLCTL_INTSET Control Bits */
      __O  uint32_t RDBRST              	:	1;  /* Write 1 sets the corresponding interrupt flag bit in the FLCTL_INTFLAG_REG */
      __O  uint32_t AVPRE               	:	1;  /* Write 1 sets the corresponding interrupt flag bit in the FLCTL_INTFLAG_REG */
      __O  uint32_t AVPST               	:	1;  /* Write 1 sets the corresponding interrupt flag bit in the FLCTL_INTFLAG_REG */
      __O  uint32_t PRG                 	:	1;  /* Write 1 sets the corresponding interrupt flag bit in the FLCTL_INTFLAG_REG */
      __O  uint32_t PRGB                	:	1;  /* Write 1 sets the corresponding interrupt flag bit in the FLCTL_INTFLAG_REG */
      __O  uint32_t ERASE               	:	1;  /* Write 1 sets the corresponding interrupt flag bit in the FLCTL_INTFLAG_REG */
      __O  uint32_t PARBNK0             	:	1;  /* Write 1 sets the corresponding interrupt flag bit in the FLCTL_INTFLAG_REG */
      __O  uint32_t PARBNK1             	:	1;  /* Write 1 sets the corresponding interrupt flag bit in the FLCTL_INTFLAG_REG */
      __O  uint32_t BMRK                	:	1;  /* Write 1 sets the corresponding interrupt flag bit in the FLCTL_INTFLAG_REG */
      __O  uint32_t PRG_ERR             	:	1;  /* Write 1 sets the corresponding interrupt flag bit in the FLCTL_INTFLAG_REG */
      __I  uint32_t RESERVED_1          	:	22;  /* Reserved */

    } bit;
  } INTSET;
  union {    /* FLCTL_READ_TIMCTL Register */
    __IO uint32_t reg;
    struct { /* FLCTL_READ_TIMCTL Control Bits */
      __IO uint32_t SETUP               	:	8;  /* Configures the length of the Setup phase for this operation */
      __I  uint32_t RESERVED_2          	:	4;  /* Reserved */
      __IO uint32_t IREF_BOOST1         	:	4;  /* Configures the length of the IREF_BOOST1 signal of the IP */
      __IO uint32_t SETUP_LONG          	:	8;  /* Configures the length of the Setup time into read mode when the device is  */
                                                /* recovering from one of the following conditions  Moving from Power-down or  */
                                                /* Standby back to Active  #b#and#/b# device is not trimmed Moving from LPR standby */
                                                /* to LPR Active Recovering from the LDO Boost operation after a Mass Erase */
      __I  uint32_t RESERVED_1          	:	8;  /* Reserved */

    } bit;
  } READ_TIMCTL;
  union {    /* FLCTL_READMARGIN_TIMCTL Register */
    __IO uint32_t reg;
    struct { /* FLCTL_READMARGIN_TIMCTL Control Bits */
      __IO uint32_t SETUP               	:	8;  /* Configures the length of the Setup phase for this operation */
      __I  uint32_t RESERVED_1          	:	24;  /* Reserved */

    } bit;
  } READMARGIN_TIMCTL;
  union {    /* FLCTL_PRGVER_TIMCTL Register */
    __IO uint32_t reg;
    struct { /* FLCTL_PRGVER_TIMCTL Control Bits */
      __IO uint32_t SETUP               	:	8;  /* Configures the length of the Setup phase for this operation */
      __IO uint32_t ACTIVE              	:	4;  /* Configures the length of the Active phase for this operation */
      __IO uint32_t HOLD                	:	4;  /* Configures the length of the Hold phase for this operation */
      __I  uint32_t RESERVED_1          	:	16;  /* Reserved */

    } bit;
  } PRGVER_TIMCTL;
  union {    /* FLCTL_ERSVER_TIMCTL Register */
    __IO uint32_t reg;
    struct { /* FLCTL_ERSVER_TIMCTL Control Bits */
      __IO uint32_t SETUP               	:	8;  /* Configures the length of the Setup phase for this operation */
      __I  uint32_t RESERVED_1          	:	24;  /* Reserved */

    } bit;
  } ERSVER_TIMCTL;
  union {    /* FLCTL_LKGVER_TIMCTL Register */
    __IO uint32_t reg;
    struct { /* FLCTL_LKGVER_TIMCTL Control Bits */
      __IO uint32_t SETUP               	:	8;  /* Configures the length of the Setup phase for this operation */
      __I  uint32_t RESERVED_1          	:	24;  /* Reserved */

    } bit;
  } LKGVER_TIMCTL;
  union {    /* FLCTL_PROGRAM_TIMCTL Register */
    __IO uint32_t reg;
    struct { /* FLCTL_PROGRAM_TIMCTL Control Bits */
      __IO uint32_t SETUP               	:	8;  /* Configures the length of the Setup phase for this operation */
      __IO uint32_t ACTIVE              	:	20;  /* Configures the length of the Active phase for this operation */
      __IO uint32_t HOLD                	:	4;  /* Configures the length of the Hold phase for this operation */

    } bit;
  } PROGRAM_TIMCTL;
  union {    /* FLCTL_ERASE_TIMCTL Register */
    __IO uint32_t reg;
    struct { /* FLCTL_ERASE_TIMCTL Control Bits */
      __IO uint32_t SETUP               	:	8;  /* Configures the length of the Setup phase for this operation */
      __IO uint32_t ACTIVE              	:	20;  /* Configures the length of the Active phase for this operation */
      __IO uint32_t HOLD                	:	4;  /* Configures the length of the Hold phase for this operation */

    } bit;
  } ERASE_TIMCTL;
  union {    /* FLCTL_MASSERASE_TIMCTL Register */
    __IO uint32_t reg;
    struct { /* FLCTL_MASSERASE_TIMCTL Control Bits */
      __IO uint32_t BOOST_ACTIVE        	:	8;  /* Configures the length of the time for which LDO Boost Signal is kept active */
      __IO uint32_t BOOST_HOLD          	:	8;  /* Configures the length for which Flash deactivates the LDO Boost signal before  */
                                                /* processing any new commands */
      __I  uint32_t RESERVED_1          	:	16;  /* Reserved */

    } bit;
  } MASSERASE_TIMCTL;
} FLCTL_Type;


/* FLCTL_PWRSTAT Control Bits */
#define FLCTL_PWRSTAT_PSTAT__M                             (0x00000007)          /* Flash power status */
#define FLCTL_PWRSTAT_PSTAT__0                             (0x00000000)          /* Flash IP in power-down mode */
#define FLCTL_PWRSTAT_PSTAT__1                             (0x00000001)          /* Flash IP Vdd domain power-up in progress */
#define FLCTL_PWRSTAT_PSTAT__2                             (0x00000002)          /* PSS LDO_GOOD, IREF_OK and VREF_OK check in progress */
#define FLCTL_PWRSTAT_PSTAT__3                             (0x00000003)          /* Flash IP SAFE_LV check in progress */
#define FLCTL_PWRSTAT_PSTAT__4                             (0x00000004)          /* Flash IP Active */
#define FLCTL_PWRSTAT_PSTAT__4_FLASH_IP_ACTIVE             (0x00000004)          /* Flash IP Active */
#define FLCTL_PWRSTAT_PSTAT__5                             (0x00000005)          /* Flash IP Active in LPR/LPS mode */
#define FLCTL_PWRSTAT_PSTAT__6                             (0x00000006)          /* Flash IP in Standby mode */
#define FLCTL_PWRSTAT_PSTAT__7                             (0x00000007)          /* Flash IP in Current mirror boost state */
#define FLCTL_PWRSTAT_LDOSTAT                              (0x00000008)          /* PSS FLDO GOOD status */
#define FLCTL_PWRSTAT_VREFSTAT                             (0x00000010)          /* PSS VREF stable status */
#define FLCTL_PWRSTAT_IREFSTAT                             (0x00000020)          /* PSS IREF stable status */
#define FLCTL_PWRSTAT_TRIMSTAT                             (0x00000040)          /* PSS trim done status */
#define FLCTL_PWRSTAT_RD_2T                                (0x00000080)          /* Indicates if Flash is being accessed in 2T mode */
#define FLCTL_PWRSTAT_RESERVED_1__M                        (0xffffff00)          /* Reserved. Reads return 0h */

/* FLCTL_RDCTL_BNK0 Control Bits */
#define FLCTL_RDCTL_BNK0_RD_MODE__M                        (0x0000000f)          /* Flash read mode control setting for Bank 1001 = Read Margin 0B 1010 = Read  */
                                                                                 /* Margin 1B All others = Reserved, bank is set to Normal read mode */
#define FLCTL_RDCTL_BNK0_RD_MODE__0                        (0x00000000)          /* Normal read mode */
#define FLCTL_RDCTL_BNK0_RD_MODE__1                        (0x00000001)          /* Read Margin 0 */
#define FLCTL_RDCTL_BNK0_RD_MODE__1_READ_MARGIN_0          (0x00000001)          /* Read Margin 0 */
#define FLCTL_RDCTL_BNK0_RD_MODE__2                        (0x00000002)          /* Read Margin 1 */
#define FLCTL_RDCTL_BNK0_RD_MODE__2_READ_MARGIN_1          (0x00000002)          /* Read Margin 1 */
#define FLCTL_RDCTL_BNK0_RD_MODE__3                        (0x00000003)          /* Program Verify */
#define FLCTL_RDCTL_BNK0_RD_MODE__3_PROGRAM_VERIFY         (0x00000003)          /* Program Verify */
#define FLCTL_RDCTL_BNK0_RD_MODE__4                        (0x00000004)          /* Erase Verify */
#define FLCTL_RDCTL_BNK0_RD_MODE__4_ERASE_VERIFY           (0x00000004)          /* Erase Verify */
#define FLCTL_RDCTL_BNK0_RD_MODE__5                        (0x00000005)          /* Leakage Verify */
#define FLCTL_RDCTL_BNK0_RD_MODE__5_LEAKAGE_VERIFY         (0x00000005)          /* Leakage Verify */
#define FLCTL_RDCTL_BNK0_BUFI                              (0x00000010)          /* Enables read buffering feature for instruction fetches to this Bank */
#define FLCTL_RDCTL_BNK0_BUFD                              (0x00000020)          /* Enables read buffering feature for data reads to this Bank */
#define FLCTL_RDCTL_BNK0_RESERVED_2__M                     (0x000000c0)          /* Reserved. Reads return 0h */
#define FLCTL_RDCTL_BNK0_INFO_PARI                         (0x00000100)          /* Enable parity check on instruction fetches to INFO sectors in this Bank */
#define FLCTL_RDCTL_BNK0_INFO_PARD                         (0x00000200)          /* Enable parity check on data reads to INFO sectors in this Bank */
#define FLCTL_RDCTL_BNK0_USR_PARI                          (0x00000400)          /* Enable parity check on instruction fetches to User sectors in this Bank */
#define FLCTL_RDCTL_BNK0_USR_PARD                          (0x00000800)          /* Enable parity check on data reads to User sectors in this Bank */
#define FLCTL_RDCTL_BNK0_WAIT__M                           (0x0000f000)          /* Defines the number of wait states required for a read operation to the bank 0000 */
                                                                                 /* = 0 wait states 0001 = 1 wait states 0010 = 2 wait states 0011 = 3 wait states  */
                                                                                 /* 0100 = 4 wait states 0101 = 5 wait states 0110 = 6 wait states 0111 = 7 wait  */
                                                                                 /* states 1000 = 8 wait states 1001 = 9 wait states 1010 = 10 wait states 1011 = 11 */
                                                                                 /* wait states 1100 = 12 wait states 1101 = 13 wait states 1110 = 14 wait states  */
                                                                                 /* 1111 = 15 wait states */
#define FLCTL_RDCTL_BNK0_RD_MODE_STATUS__M                 (0x000f0000)          /* Reflects the current Read Mode of the Bank 1001 = Read Margin 0B 1010 = Read  */
                                                                                 /* Margin 1B All others = Reserved, bank is set to Normal read mode */
#define FLCTL_RDCTL_BNK0_RD_MODE_STATUS__0                 (0x00000000)          /* Normal read mode */
#define FLCTL_RDCTL_BNK0_RD_MODE_STATUS__1                 (0x00010000)          /* Read Margin 0 */
#define FLCTL_RDCTL_BNK0_RD_MODE_STATUS__1_READ_MARGIN_0   (0x00010000)          /* Read Margin 0 */
#define FLCTL_RDCTL_BNK0_RD_MODE_STATUS__2                 (0x00020000)          /* Read Margin 1 */
#define FLCTL_RDCTL_BNK0_RD_MODE_STATUS__2_READ_MARGIN_1   (0x00020000)          /* Read Margin 1 */
#define FLCTL_RDCTL_BNK0_RD_MODE_STATUS__3                 (0x00030000)          /* Program Verify */
#define FLCTL_RDCTL_BNK0_RD_MODE_STATUS__3_PROGRAM_VERIFY  (0x00030000)          /* Program Verify */
#define FLCTL_RDCTL_BNK0_RD_MODE_STATUS__4                 (0x00040000)          /* Erase Verify */
#define FLCTL_RDCTL_BNK0_RD_MODE_STATUS__4_ERASE_VERIFY    (0x00040000)          /* Erase Verify */
#define FLCTL_RDCTL_BNK0_RD_MODE_STATUS__5                 (0x00050000)          /* Leakage Verify */
#define FLCTL_RDCTL_BNK0_RD_MODE_STATUS__5_LEAKAGE_VERIFY  (0x00050000)          /* Leakage Verify */
#define FLCTL_RDCTL_BNK0_RESERVED_1__M                     (0xfff00000)          /* Reserved. Reads return 0h */

/* FLCTL_RDCTL_BNK1 Control Bits */
#define FLCTL_RDCTL_BNK1_RD_MODE__M                        (0x0000000f)          /* Flash read mode control setting for Bank 1001 = Read Margin 0B 1010 = Read  */
                                                                                 /* Margin 1B All others = Reserved, bank is set to Normal read mode */
#define FLCTL_RDCTL_BNK1_RD_MODE__0                        (0x00000000)          /* Normal read mode */
#define FLCTL_RDCTL_BNK1_RD_MODE__1                        (0x00000001)          /* Read Margin 0 */
#define FLCTL_RDCTL_BNK1_RD_MODE__1_READ_MARGIN_0          (0x00000001)          /* Read Margin 0 */
#define FLCTL_RDCTL_BNK1_RD_MODE__2                        (0x00000002)          /* Read Margin 1 */
#define FLCTL_RDCTL_BNK1_RD_MODE__2_READ_MARGIN_1          (0x00000002)          /* Read Margin 1 */
#define FLCTL_RDCTL_BNK1_RD_MODE__3                        (0x00000003)          /* Program Verify */
#define FLCTL_RDCTL_BNK1_RD_MODE__3_PROGRAM_VERIFY         (0x00000003)          /* Program Verify */
#define FLCTL_RDCTL_BNK1_RD_MODE__4                        (0x00000004)          /* Erase Verify */
#define FLCTL_RDCTL_BNK1_RD_MODE__4_ERASE_VERIFY           (0x00000004)          /* Erase Verify */
#define FLCTL_RDCTL_BNK1_RD_MODE__5                        (0x00000005)          /* Leakage Verify */
#define FLCTL_RDCTL_BNK1_RD_MODE__5_LEAKAGE_VERIFY         (0x00000005)          /* Leakage Verify */
#define FLCTL_RDCTL_BNK1_BUFI                              (0x00000010)          /* Enables read buffering feature for instruction fetches to this Bank */
#define FLCTL_RDCTL_BNK1_BUFD                              (0x00000020)          /* Enables read buffering feature for data reads to this Bank */
#define FLCTL_RDCTL_BNK1_RESERVED_2__M                     (0x000000c0)          /* Reserved. Reads return 0h */
#define FLCTL_RDCTL_BNK1_INFO_PARI                         (0x00000100)          /* Enable parity check on instruction fetches to INFO sectors in this Bank */
#define FLCTL_RDCTL_BNK1_INFO_PARD                         (0x00000200)          /* Enable parity check on data reads to INFO sectors in this Bank */
#define FLCTL_RDCTL_BNK1_USR_PARI                          (0x00000400)          /* Enable parity check on instruction fetches to User sectors in this Bank */
#define FLCTL_RDCTL_BNK1_USR_PARD                          (0x00000800)          /* Enable parity check on data reads to User sectors in this Bank */
#define FLCTL_RDCTL_BNK1_WAIT__M                           (0x0000f000)          /* Defines the number of wait states required for a read operation to the bank 0000 */
                                                                                 /* = 0 wait states 0001 = 1 wait states 0010 = 2 wait states 0011 = 3 wait states  */
                                                                                 /* 0100 = 4 wait states 0101 = 5 wait states 0110 = 6 wait states 0111 = 7 wait  */
                                                                                 /* states 1000 = 8 wait states 1001 = 9 wait states 1010 = 10 wait states 1011 = 11 */
                                                                                 /* wait states 1100 = 12 wait states 1101 = 13 wait states 1110 = 14 wait states  */
                                                                                 /* 1111 = 15 wait states */
#define FLCTL_RDCTL_BNK1_RD_MODE_STATUS__M                 (0x000f0000)          /* Reflects the current Read Mode of the Bank 1001 = Read Margin 0B 1010 = Read  */
                                                                                 /* Margin 1B All others = Reserved, bank is set to Normal read mode */
#define FLCTL_RDCTL_BNK1_RD_MODE_STATUS__0                 (0x00000000)          /* Normal read mode */
#define FLCTL_RDCTL_BNK1_RD_MODE_STATUS__1                 (0x00010000)          /* Read Margin 0 */
#define FLCTL_RDCTL_BNK1_RD_MODE_STATUS__1_READ_MARGIN_0   (0x00010000)          /* Read Margin 0 */
#define FLCTL_RDCTL_BNK1_RD_MODE_STATUS__2                 (0x00020000)          /* Read Margin 1 */
#define FLCTL_RDCTL_BNK1_RD_MODE_STATUS__2_READ_MARGIN_1   (0x00020000)          /* Read Margin 1 */
#define FLCTL_RDCTL_BNK1_RD_MODE_STATUS__3                 (0x00030000)          /* Program Verify */
#define FLCTL_RDCTL_BNK1_RD_MODE_STATUS__3_PROGRAM_VERIFY  (0x00030000)          /* Program Verify */
#define FLCTL_RDCTL_BNK1_RD_MODE_STATUS__4                 (0x00040000)          /* Erase Verify */
#define FLCTL_RDCTL_BNK1_RD_MODE_STATUS__4_ERASE_VERIFY    (0x00040000)          /* Erase Verify */
#define FLCTL_RDCTL_BNK1_RD_MODE_STATUS__5                 (0x00050000)          /* Leakage Verify */
#define FLCTL_RDCTL_BNK1_RD_MODE_STATUS__5_LEAKAGE_VERIFY  (0x00050000)          /* Leakage Verify */
#define FLCTL_RDCTL_BNK1_RESERVED_1__M                     (0xfff00000)          /* Reserved. Reads return 0h */

/* FLCTL_RDBRST_CTLSTAT Control Bits */
#define FLCTL_RDBRST_CTLSTAT_START                         (0x00000001)          /* Write 1 triggers start of burst/compare    operation */
#define FLCTL_RDBRST_CTLSTAT_MEM_TYPE__M                   (0x00000006)          /* Type of memory that burst is carried out on */
#define FLCTL_RDBRST_CTLSTAT_MEM_TYPE__0                   (0x00000000)          /* USER */
#define FLCTL_RDBRST_CTLSTAT_MEM_TYPE__0_USER              (0x00000000)          /* USER */
#define FLCTL_RDBRST_CTLSTAT_MEM_TYPE__1                   (0x00000002)          /* INFO */
#define FLCTL_RDBRST_CTLSTAT_MEM_TYPE__1_INFO              (0x00000002)          /* INFO */
#define FLCTL_RDBRST_CTLSTAT_MEM_TYPE__3                   (0x00000006)          /* ENGR */
#define FLCTL_RDBRST_CTLSTAT_MEM_TYPE__3_ENGR              (0x00000006)          /* ENGR */
#define FLCTL_RDBRST_CTLSTAT_STOP_FAIL                     (0x00000008)          /* If set to 1, causes burst/compare operation to terminate on first compare  */
                                                                                 /* mismatch */
#define FLCTL_RDBRST_CTLSTAT_DATA_CMP                      (0x00000010)          /* Data pattern used for comparison against memory read data */
#define FLCTL_RDBRST_CTLSTAT_PAR_CMP                       (0x00000020)          /* If set to 1, enables parity compare during the burst operation */
#define FLCTL_RDBRST_CTLSTAT_TEST_EN                       (0x00000040)          /* When   set to 1, enables comparison against test data compare registers */
#define FLCTL_RDBRST_CTLSTAT_RESERVED_3__M                 (0x0000ff80)          /* Reserved. Reads return 0h */
#define FLCTL_RDBRST_CTLSTAT_BRST_STAT__M                  (0x00030000)          /* Status of Burst/Compare operation */
#define FLCTL_RDBRST_CTLSTAT_BRST_STAT__0                  (0x00000000)          /* Idle */
#define FLCTL_RDBRST_CTLSTAT_BRST_STAT__0_IDLE             (0x00000000)          /* Idle */
#define FLCTL_RDBRST_CTLSTAT_BRST_STAT__1                  (0x00010000)          /* Burst/Compare START bit written, but operation pending */
#define FLCTL_RDBRST_CTLSTAT_BRST_STAT__2                  (0x00020000)          /* Burst/Compare in progress */
#define FLCTL_RDBRST_CTLSTAT_BRST_STAT__3                  (0x00030000)          /* Burst complete (status of completed burst remains in this state unless explicitly cleared by SW) */
#define FLCTL_RDBRST_CTLSTAT_CMP_ERR                       (0x00040000)          /* if 1, indicates that the Burst/Compare Operation encountered atleast one   data  */
                                                                                 /* comparison error */
#define FLCTL_RDBRST_CTLSTAT_ADDR_ERR                      (0x00080000)          /* If 1, indicates that Burst/Compare Operation was terminated due to access to  */
                                                                                 /* reserved memory */
#define FLCTL_RDBRST_CTLSTAT_RESERVED_2__M                 (0x00700000)          /* Reserved. Reads return 0h */
#define FLCTL_RDBRST_CTLSTAT_CLR_STAT                      (0x00800000)          /* Write '1' to clear status bits 19-16 of this register Write '0' has no effect */
#define FLCTL_RDBRST_CTLSTAT_RESERVED_1__M                 (0xff000000)          /* Reserved. Reads return 0h */

/* FLCTL_RDBRST_STARTADDR Control Bits */
#define FLCTL_RDBRST_STARTADDR_START_ADDRESS__M            (0x001fffff)          /* Start Address of Burst Operation. Offset from 0h, with 0h as start address of  */
                                                                                 /* the type of memory region selected Bits 3-0 are always 0 (forced 128bit boundary */
                                                                                 /* ) */
#define FLCTL_RDBRST_STARTADDR_RESERVED_1__M               (0xffe00000)          /* Reserved. Reads return 0h */

/* FLCTL_RDBRST_LEN Control Bits */
#define FLCTL_RDBRST_LEN_BURST_LENGTH__M                   (0x001fffff)          /* Length of Burst Operation (in bytes). Bits 3-0 are always 0 (forced minimum  */
                                                                                 /* resolution of 128bits) */
#define FLCTL_RDBRST_LEN_RESERVED_1__M                     (0xffe00000)          /* Reserved. Reads return 0h */

/* FLCTL_RDBRST_FAILADDR Control Bits */
#define FLCTL_RDBRST_FAILADDR_FAIL_ADDRESS__M              (0x001fffff)          /* Reflects address of last failed compare. Offset from 0h, with 0h as start  */
                                                                                 /* address of the type of memory region selected Bits 3-0 are always 0 (forced  */
                                                                                 /* 128bit boundary) */
#define FLCTL_RDBRST_FAILADDR_RESERVED_1__M                (0xffe00000)          /* Reserved. Reads return 0h */

/* FLCTL_RDBRST_FAILCNT Control Bits */
#define FLCTL_RDBRST_FAILCNT_FAIL_COUNT__M                 (0x0001ffff)          /* Reflects number of failures encountered in burst operation. */
#define FLCTL_RDBRST_FAILCNT_RESERVED_1__M                 (0xfffe0000)          /* Reserved. Reads return 0h */

/* FLCTL_PRG_CTLSTAT Control Bits */
#define FLCTL_PRG_CTLSTAT_ENABLE                           (0x00000001)          /* Master control for all word program operations */
#define FLCTL_PRG_CTLSTAT_MODE                             (0x00000002)          /* Controls write mode selected by application */
#define FLCTL_PRG_CTLSTAT_VER_PRE                          (0x00000004)          /* Controls automatic pre program verify operations */
#define FLCTL_PRG_CTLSTAT_VER_PST                          (0x00000008)          /* Controls automatic post program verify operations */
#define FLCTL_PRG_CTLSTAT_RESERVED_2__M                    (0x0000fff0)          /* Reserved. Reads return 0h */
#define FLCTL_PRG_CTLSTAT_STATUS__M                        (0x00030000)          /* Reflects the status of program operations in the Flash memory */
#define FLCTL_PRG_CTLSTAT_STATUS__0                        (0x00000000)          /* Idle (no program operation currently active) */
#define FLCTL_PRG_CTLSTAT_STATUS__1                        (0x00010000)          /* Single word program operation triggered, but pending */
#define FLCTL_PRG_CTLSTAT_STATUS__2                        (0x00020000)          /* Single word program in progress */
#define FLCTL_PRG_CTLSTAT_BNK_ACT                          (0x00040000)          /* Reflects which bank is currently undergoing a program operation (valid only if  */
                                                                                 /* bits 17-16 don't show idle) */
#define FLCTL_PRG_CTLSTAT_RESERVED_1__M                    (0xfff80000)          /* Reserved. Reads return 0h */

/* FLCTL_PRGBRST_CTLSTAT Control Bits */
#define FLCTL_PRGBRST_CTLSTAT_START                        (0x00000001)          /* Write 1 triggers start of burst program operation */
#define FLCTL_PRGBRST_CTLSTAT_TYPE__M                      (0x00000006)          /* Type of memory that burst program is carried out on */
#define FLCTL_PRGBRST_CTLSTAT_TYPE__0                      (0x00000000)          /* USER */
#define FLCTL_PRGBRST_CTLSTAT_TYPE__0_USER                 (0x00000000)          /* USER */
#define FLCTL_PRGBRST_CTLSTAT_TYPE__1                      (0x00000002)          /* INFO */
#define FLCTL_PRGBRST_CTLSTAT_TYPE__1_INFO                 (0x00000002)          /* INFO */
#define FLCTL_PRGBRST_CTLSTAT_TYPE__3                      (0x00000006)          /* ENGR */
#define FLCTL_PRGBRST_CTLSTAT_TYPE__3_ENGR                 (0x00000006)          /* ENGR */
#define FLCTL_PRGBRST_CTLSTAT_LEN__M                       (0x00000038)          /* Length of burst (in 128 bit granularity) */
#define FLCTL_PRGBRST_CTLSTAT_LEN__0                       (0x00000000)          /* No burst operation */
#define FLCTL_PRGBRST_CTLSTAT_LEN__1                       (0x00000008)          /* 1 word burst of 128 bits, starting with address in the FLCTL_PRGBRST_STARTADDR_REG Register */
#define FLCTL_PRGBRST_CTLSTAT_LEN__2                       (0x00000010)          /* 2*128 bits burst write, starting with address in the FLCTL_PRGBRST_STARTADDR_REG Register */
#define FLCTL_PRGBRST_CTLSTAT_LEN__3                       (0x00000018)          /* 3*128 bits burst write, starting with address in the FLCTL_PRGBRST_STARTADDR_REG Register */
#define FLCTL_PRGBRST_CTLSTAT_LEN__4                       (0x00000020)          /* 4*128 bits burst write, starting with address in the FLCTL_PRGBRST_STARTADDR_REG Register */
#define FLCTL_PRGBRST_CTLSTAT_LEN__5                       (0x00000028)          /* Reserved. No burst operation. */
#define FLCTL_PRGBRST_CTLSTAT_LEN__6                       (0x00000030)          /* Reserved. No burst operation. */
#define FLCTL_PRGBRST_CTLSTAT_LEN__7                       (0x00000038)          /* Reserved. No burst operation. */
#define FLCTL_PRGBRST_CTLSTAT_AUTO_PRE                     (0x00000040)          /* Controls the Auto-Verify operation before the Burst Program */
#define FLCTL_PRGBRST_CTLSTAT_AUTO_PST                     (0x00000080)          /* Controls the Auto-Verify operation after the Burst Program */
#define FLCTL_PRGBRST_CTLSTAT_RESERVED_3__M                (0x0000ff00)          /* Reserved. Reads return 0h */
#define FLCTL_PRGBRST_CTLSTAT_BURST_STATUS__M              (0x00070000)          /* At any point in time, it reflects the status of a Burst Operation */
#define FLCTL_PRGBRST_CTLSTAT_BURST_STATUS__0              (0x00000000)          /* Idle (Burst not active) */
#define FLCTL_PRGBRST_CTLSTAT_BURST_STATUS__1              (0x00010000)          /* Burst program started but pending */
#define FLCTL_PRGBRST_CTLSTAT_BURST_STATUS__2              (0x00020000)          /* Burst active, with 1st 128 bit word being written into Flash */
#define FLCTL_PRGBRST_CTLSTAT_BURST_STATUS__3              (0x00030000)          /* Burst active, with 2nd 128 bit word being written into Flash */
#define FLCTL_PRGBRST_CTLSTAT_BURST_STATUS__4              (0x00040000)          /* Burst active, with 3rd 128 bit word being written into Flash */
#define FLCTL_PRGBRST_CTLSTAT_BURST_STATUS__5              (0x00050000)          /* Burst active, with 4th 128 bit word being written into Flash */
#define FLCTL_PRGBRST_CTLSTAT_BURST_STATUS__7              (0x00070000)          /* Burst Complete (status of completed burst remains in this state unless explicitly cleared by SW) */
#define FLCTL_PRGBRST_CTLSTAT_PRE_ERR                      (0x00080000)          /* If 1, indicates that Burst Operation encountered preprogram auto-verify     */
                                                                                 /* errors */
#define FLCTL_PRGBRST_CTLSTAT_PST_ERR                      (0x00100000)          /* if 1, indicates that the Burst Operation encountered postprogram   auto-verify  */
                                                                                 /* errors */
#define FLCTL_PRGBRST_CTLSTAT_ADDR_ERR                     (0x00200000)          /* If 1, indicates that Burst Operation was terminated due to attempted   program  */
                                                                                 /* of reserved memory */
#define FLCTL_PRGBRST_CTLSTAT_RESERVED_2                   (0x00400000)          /* Reserved. Reads return 0h */
#define FLCTL_PRGBRST_CTLSTAT_CLR_STAT                     (0x00800000)          /* Write '1' to clear status bits 21-16 of this register Write '0' has no effect */
#define FLCTL_PRGBRST_CTLSTAT_RESERVED_1__M                (0xff000000)          /* Reserved. Reads return 0h */

/* FLCTL_PRGBRST_STARTADDR Control Bits */
#define FLCTL_PRGBRST_STARTADDR_START_ADDRESS__M           (0x003fffff)          /* Start Address of Program Burst Operation. Offset from 0h, with 0h as start  */
                                                                                 /* address of the type of memory region selected Bits 3-0 are always 0 (forced  */
                                                                                 /* 128bit boundary) */
#define FLCTL_PRGBRST_STARTADDR_RESERVED_1__M              (0xffc00000)          /* Reserved. Reads return 0h */

/* FLCTL_ERASE_CTLSTAT Control Bits */
#define FLCTL_ERASE_CTLSTAT_START                          (0x00000001)          /* Write 1 triggers start of Erase operation */
#define FLCTL_ERASE_CTLSTAT_MODE                           (0x00000002)          /* Controls erase mode selected by application */
#define FLCTL_ERASE_CTLSTAT_TYPE__M                        (0x0000000c)          /* Type of memory that erase operation is carried out on (don't care if mass erase  */
                                                                                 /* is set to 1) */
#define FLCTL_ERASE_CTLSTAT_TYPE__0                        (0x00000000)          /* USER */
#define FLCTL_ERASE_CTLSTAT_TYPE__0_USER                   (0x00000000)          /* USER */
#define FLCTL_ERASE_CTLSTAT_TYPE__1                        (0x00000004)          /* INFO */
#define FLCTL_ERASE_CTLSTAT_TYPE__1_INFO                   (0x00000004)          /* INFO */
#define FLCTL_ERASE_CTLSTAT_TYPE__3                        (0x0000000c)          /* ENGR */
#define FLCTL_ERASE_CTLSTAT_TYPE__3_ENGR                   (0x0000000c)          /* ENGR */
#define FLCTL_ERASE_CTLSTAT_RESERVED_2__M                  (0x0000fff0)          /* Reserved. Reads return 0h */
#define FLCTL_ERASE_CTLSTAT_STATUS__M                      (0x00030000)          /* Reflects the status of erase operations in the Flash memory */
#define FLCTL_ERASE_CTLSTAT_STATUS__0                      (0x00000000)          /* Idle (no program operation currently active) */
#define FLCTL_ERASE_CTLSTAT_STATUS__1                      (0x00010000)          /* Erase operation triggered to START but pending */
#define FLCTL_ERASE_CTLSTAT_STATUS__2                      (0x00020000)          /* Erase operation in progress */
#define FLCTL_ERASE_CTLSTAT_STATUS__3                      (0x00030000)          /* Erase operation completed (status of completed erase remains in this state unless explicitly cleared by SW) */
#define FLCTL_ERASE_CTLSTAT_ADDR_ERR                       (0x00040000)          /* If 1, indicates that Erase Operation was terminated due to attempted erase of  */
                                                                                 /* reserved memory address */
#define FLCTL_ERASE_CTLSTAT_CLR_STAT                       (0x00080000)          /* Write '1' to clear status bits 18-16 of this register Write '0' has no effect */
#define FLCTL_ERASE_CTLSTAT_RESERVED_1__M                  (0xfff00000)          /* Reserved. Reads return 0h */

/* FLCTL_ERASE_SECTADDR Control Bits */
#define FLCTL_ERASE_SECTADDR_SECT_ADDRESS__M               (0x003fffff)          /* Address of Sector being Erased. Offset from 0h, with 0h as start address of the  */
                                                                                 /* type of memory region selected  If memory type is set to INFO/USER, Bits 11-0  */
                                                                                 /* are always 0 (forced sector boundary of 4KB)  If memory type is set to ENGR,  */
                                                                                 /* Bits 10-0 are always 0 (forced sector boundary of 2KB) */
#define FLCTL_ERASE_SECTADDR_RESERVED_1__M                 (0xffc00000)          /* Reserved. Reads return 0h */

/* FLCTL_INFOWEPROT_BNK0 Control Bits */
#define FLCTL_INFOWEPROT_BNK0_PROT0                        (0x00000001)          /* If set to 1, protects Sector 0 from program or erase operations */
#define FLCTL_INFOWEPROT_BNK0_PROT1                        (0x00000002)          /* If set to 1, protects Sector 1 from program or erase operations */
#define FLCTL_INFOWEPROT_BNK0_RESERVED_1__M                (0xfffffffc)          /* Reserved. Reads return 0h */

/* FLCTL_USRWEPROT_BNK0 Control Bits */
#define FLCTL_USRWEPROT_BNK0_PROT0                         (0x00000001)          /* If set to 1, protects Sector 0 from program or erase operations */
#define FLCTL_USRWEPROT_BNK0_PROT1                         (0x00000002)          /* If set to 1, protects Sector 1 from program or erase operations */
#define FLCTL_USRWEPROT_BNK0_PROT2                         (0x00000004)          /* If set to 1, protects Sector 2 from program or erase operations */
#define FLCTL_USRWEPROT_BNK0_PROT3                         (0x00000008)          /* If set to 1, protects Sector 3 from program or erase operations */
#define FLCTL_USRWEPROT_BNK0_PROT4                         (0x00000010)          /* If set to 1, protects Sector 4 from program or erase operations */
#define FLCTL_USRWEPROT_BNK0_PROT5                         (0x00000020)          /* If set to 1, protects Sector 5 from program or erase operations */
#define FLCTL_USRWEPROT_BNK0_PROT6                         (0x00000040)          /* If set to 1, protects Sector 6 from program or erase operations */
#define FLCTL_USRWEPROT_BNK0_PROT7                         (0x00000080)          /* If set to 1, protects Sector 7 from program or erase operations */
#define FLCTL_USRWEPROT_BNK0_PROT8                         (0x00000100)          /* If set to 1, protects Sector 8 from program or erase operations */
#define FLCTL_USRWEPROT_BNK0_PROT9                         (0x00000200)          /* If set to 1, protects Sector 9 from program or erase operations */
#define FLCTL_USRWEPROT_BNK0_PROT10                        (0x00000400)          /* If set to 1, protects Sector 10 from program or erase operations */
#define FLCTL_USRWEPROT_BNK0_PROT11                        (0x00000800)          /* If set to 1, protects Sector 11 from program or erase operations */
#define FLCTL_USRWEPROT_BNK0_PROT12                        (0x00001000)          /* If set to 1, protects Sector 12 from program or erase operations */
#define FLCTL_USRWEPROT_BNK0_PROT13                        (0x00002000)          /* If set to 1, protects Sector 13 from program or erase operations */
#define FLCTL_USRWEPROT_BNK0_PROT14                        (0x00004000)          /* If set to 1, protects Sector 14 from program or erase operations */
#define FLCTL_USRWEPROT_BNK0_PROT15                        (0x00008000)          /* If set to 1, protects Sector 15 from program or erase operations */
#define FLCTL_USRWEPROT_BNK0_PROT16                        (0x00010000)          /* If set to 1, protects Sector 16 from program or erase operations */
#define FLCTL_USRWEPROT_BNK0_PROT17                        (0x00020000)          /* If set to 1, protects Sector 17 from program or erase operations */
#define FLCTL_USRWEPROT_BNK0_PROT18                        (0x00040000)          /* If set to 1, protects Sector 18 from program or erase operations */
#define FLCTL_USRWEPROT_BNK0_PROT19                        (0x00080000)          /* If set to 1, protects Sector 19 from program or erase operations */
#define FLCTL_USRWEPROT_BNK0_PROT20                        (0x00100000)          /* If set to 1, protects Sector 20 from program or erase operations */
#define FLCTL_USRWEPROT_BNK0_PROT21                        (0x00200000)          /* If set to 1, protects Sector 21 from program or erase operations */
#define FLCTL_USRWEPROT_BNK0_PROT22                        (0x00400000)          /* If set to 1, protects Sector 22 from program or erase operations */
#define FLCTL_USRWEPROT_BNK0_PROT23                        (0x00800000)          /* If set to 1, protects Sector 23 from program or erase operations */
#define FLCTL_USRWEPROT_BNK0_PROT24                        (0x01000000)          /* If set to 1, protects Sector 24 from program or erase operations */
#define FLCTL_USRWEPROT_BNK0_PROT25                        (0x02000000)          /* If set to 1, protects Sector 25 from program or erase operations */
#define FLCTL_USRWEPROT_BNK0_PROT26                        (0x04000000)          /* If set to 1, protects Sector 26 from program or erase operations */
#define FLCTL_USRWEPROT_BNK0_PROT27                        (0x08000000)          /* If set to 1, protects Sector 27 from program or erase operations */
#define FLCTL_USRWEPROT_BNK0_PROT28                        (0x10000000)          /* If set to 1, protects Sector 28 from program or erase operations */
#define FLCTL_USRWEPROT_BNK0_PROT29                        (0x20000000)          /* If set to 1, protects Sector 29 from program or erase operations */
#define FLCTL_USRWEPROT_BNK0_PROT30                        (0x40000000)          /* If set to 1, protects Sector 30 from program or erase operations */
#define FLCTL_USRWEPROT_BNK0_PROT31                        (0x80000000)          /* If set to 1, protects Sector 31 from program or erase operations */

/* FLCTL_INFOWEPROT_BNK1 Control Bits */
#define FLCTL_INFOWEPROT_BNK1_PROT0                        (0x00000001)          /* If set to 1, protects Sector 0 from program or erase operations */
#define FLCTL_INFOWEPROT_BNK1_PROT1                        (0x00000002)          /* If set to 1, protects Sector 1 from program or erase operations */
#define FLCTL_INFOWEPROT_BNK1_RESERVED_1__M                (0xfffffffc)          /* Reserved. Reads return 0h */

/* FLCTL_USRWEPROT_BNK1 Control Bits */
#define FLCTL_USRWEPROT_BNK1_PROT0                         (0x00000001)          /* If set to 1, protects Sector 0 from program or erase operations */
#define FLCTL_USRWEPROT_BNK1_PROT1                         (0x00000002)          /* If set to 1, protects Sector 1 from program or erase operations */
#define FLCTL_USRWEPROT_BNK1_PROT2                         (0x00000004)          /* If set to 1, protects Sector 2 from program or erase operations */
#define FLCTL_USRWEPROT_BNK1_PROT3                         (0x00000008)          /* If set to 1, protects Sector 3 from program or erase operations */
#define FLCTL_USRWEPROT_BNK1_PROT4                         (0x00000010)          /* If set to 1, protects Sector 4 from program or erase operations */
#define FLCTL_USRWEPROT_BNK1_PROT5                         (0x00000020)          /* If set to 1, protects Sector 5 from program or erase operations */
#define FLCTL_USRWEPROT_BNK1_PROT6                         (0x00000040)          /* If set to 1, protects Sector 6 from program or erase operations */
#define FLCTL_USRWEPROT_BNK1_PROT7                         (0x00000080)          /* If set to 1, protects Sector 7 from program or erase operations */
#define FLCTL_USRWEPROT_BNK1_PROT8                         (0x00000100)          /* If set to 1, protects Sector 8 from program or erase operations */
#define FLCTL_USRWEPROT_BNK1_PROT9                         (0x00000200)          /* If set to 1, protects Sector 9 from program or erase operations */
#define FLCTL_USRWEPROT_BNK1_PROT10                        (0x00000400)          /* If set to 1, protects Sector 10 from program or erase operations */
#define FLCTL_USRWEPROT_BNK1_PROT11                        (0x00000800)          /* If set to 1, protects Sector 11 from program or erase operations */
#define FLCTL_USRWEPROT_BNK1_PROT12                        (0x00001000)          /* If set to 1, protects Sector 12 from program or erase operations */
#define FLCTL_USRWEPROT_BNK1_PROT13                        (0x00002000)          /* If set to 1, protects Sector 13 from program or erase operations */
#define FLCTL_USRWEPROT_BNK1_PROT14                        (0x00004000)          /* If set to 1, protects Sector 14 from program or erase operations */
#define FLCTL_USRWEPROT_BNK1_PROT15                        (0x00008000)          /* If set to 1, protects Sector 15 from program or erase operations */
#define FLCTL_USRWEPROT_BNK1_PROT16                        (0x00010000)          /* If set to 1, protects Sector 16 from program or erase operations */
#define FLCTL_USRWEPROT_BNK1_PROT17                        (0x00020000)          /* If set to 1, protects Sector 17 from program or erase operations */
#define FLCTL_USRWEPROT_BNK1_PROT18                        (0x00040000)          /* If set to 1, protects Sector 18 from program or erase operations */
#define FLCTL_USRWEPROT_BNK1_PROT19                        (0x00080000)          /* If set to 1, protects Sector 19 from program or erase operations */
#define FLCTL_USRWEPROT_BNK1_PROT20                        (0x00100000)          /* If set to 1, protects Sector 20 from program or erase operations */
#define FLCTL_USRWEPROT_BNK1_PROT21                        (0x00200000)          /* If set to 1, protects Sector 21 from program or erase operations */
#define FLCTL_USRWEPROT_BNK1_PROT22                        (0x00400000)          /* If set to 1, protects Sector 22 from program or erase operations */
#define FLCTL_USRWEPROT_BNK1_PROT23                        (0x00800000)          /* If set to 1, protects Sector 23 from program or erase operations */
#define FLCTL_USRWEPROT_BNK1_PROT24                        (0x01000000)          /* If set to 1, protects Sector 24 from program or erase operations */
#define FLCTL_USRWEPROT_BNK1_PROT25                        (0x02000000)          /* If set to 1, protects Sector 25 from program or erase operations */
#define FLCTL_USRWEPROT_BNK1_PROT26                        (0x04000000)          /* If set to 1, protects Sector 26 from program or erase operations */
#define FLCTL_USRWEPROT_BNK1_PROT27                        (0x08000000)          /* If set to 1, protects Sector 27 from program or erase operations */
#define FLCTL_USRWEPROT_BNK1_PROT28                        (0x10000000)          /* If set to 1, protects Sector 28 from program or erase operations */
#define FLCTL_USRWEPROT_BNK1_PROT29                        (0x20000000)          /* If set to 1, protects Sector 29 from program or erase operations */
#define FLCTL_USRWEPROT_BNK1_PROT30                        (0x40000000)          /* If set to 1, protects Sector 30 from program or erase operations */
#define FLCTL_USRWEPROT_BNK1_PROT31                        (0x80000000)          /* If set to 1, protects Sector 31 from program or erase operations */

/* FLCTL_BMRK_CTLSTAT Control Bits */
#define FLCTL_BMRK_CTLSTAT_I_BMRK                          (0x00000001)          /* When 1, increments the Instruction Benchmark count register on each     */
                                                                                 /* instruction fetch to the Flash */
#define FLCTL_BMRK_CTLSTAT_D_BMRK                          (0x00000002)          /* When 1, increments the Data Benchmark count register on each data read access to */
                                                                                 /* the Flash */
#define FLCTL_BMRK_CTLSTAT_CMP_EN                          (0x00000004)          /* When 1, enables comparison of the Instruction or Data Benchmark Registers  */
                                                                                 /* against the threshold value */
#define FLCTL_BMRK_CTLSTAT_CMP_SEL                         (0x00000008)          /* Selects which benchmark register should be compared against the threshold */
#define FLCTL_BMRK_CTLSTAT_RESERVED_1__M                   (0xfffffff0)          /* Reserved. Reads return 0h */

/* FLCTL_INTFLAG Control Bits */
#define FLCTL_INTFLAG_RDBRST                               (0x00000001)          /* If set to 1, indicates that the Read Burst/Compare operation is complete */
#define FLCTL_INTFLAG_AVPRE                                (0x00000002)          /* If set to 1, indicates that the pre-program verify operation has detected an  */
                                                                                 /* error */
#define FLCTL_INTFLAG_AVPST                                (0x00000004)          /* If set to 1, indicates that the post-program verify operation has failed  */
                                                                                 /* comparison */
#define FLCTL_INTFLAG_PRG                                  (0x00000008)          /* If set to 1, indicates that a word Program operation is complete */
#define FLCTL_INTFLAG_PRGB                                 (0x00000010)          /* If set to 1, indicates that the configured Burst Program operation is complete */
#define FLCTL_INTFLAG_ERASE                                (0x00000020)          /* If set to 1, indicates that the Erase operation is complete */
#define FLCTL_INTFLAG_PARBNK0                              (0x00000040)          /* If set to 1, indicates an instruction/data parity error in Bank0 */
#define FLCTL_INTFLAG_PARBNK1                              (0x00000080)          /* If set to 1, indicates an instruction/data parity error in Bank1 */
#define FLCTL_INTFLAG_BMRK                                 (0x00000100)          /* If set to 1, indicates that a Benchmark Compare match occurred */
#define FLCTL_INTFLAG_PRG_ERR                              (0x00000200)          /* If set to 1, indicates a word composition error in full word write mode    ( */
                                                                                 /* possible data loss due to writes crossing over to a new 128bit boundary before  */
                                                                                 /* full word has been  composed) */
#define FLCTL_INTFLAG_RESERVED_1__M                        (0xfffffc00)          /* Reserved. Reads return 0h */

/* FLCTL_INTEN Control Bits */
#define FLCTL_INTEN_RDBRST                                 (0x00000001)          /* If set to 1, enables the Controller to generate an interrupt based on the  */
                                                                                 /* corresponding bit in the FLCTL_INTFLAG_REG */
#define FLCTL_INTEN_AVPRE                                  (0x00000002)          /* If set to 1, enables the Controller to generate an interrupt based on the  */
                                                                                 /* corresponding bit in the FLCTL_INTFLAG_REG */
#define FLCTL_INTEN_AVPST                                  (0x00000004)          /* If set to 1, enables the Controller to generate an interrupt based on the  */
                                                                                 /* corresponding bit in the FLCTL_INTFLAG_REG */
#define FLCTL_INTEN_PRG                                    (0x00000008)          /* If set to 1, enables the Controller to generate an interrupt based on the  */
                                                                                 /* corresponding bit in the FLCTL_INTFLAG_REG */
#define FLCTL_INTEN_PRGB                                   (0x00000010)          /* If set to 1, enables the Controller to generate an interrupt based on the  */
                                                                                 /* corresponding bit in the FLCTL_INTFLAG_REG */
#define FLCTL_INTEN_ERASE                                  (0x00000020)          /* If set to 1, enables the Controller to generate an interrupt based on the  */
                                                                                 /* corresponding bit in the FLCTL_INTFLAG_REG */
#define FLCTL_INTEN_PARBNK0                                (0x00000040)          /* If set to 1, enables the Controller to generate an interrupt based on the  */
                                                                                 /* corresponding bit in the FLCTL_INTFLAG_REG */
#define FLCTL_INTEN_PARBNK1                                (0x00000080)          /* If set to 1, enables the Controller to generate an interrupt based on the  */
                                                                                 /* corresponding bit in the FLCTL_INTFLAG_REG */
#define FLCTL_INTEN_BMRK                                   (0x00000100)          /* If set to 1, enables the Controller to generate an interrupt based on the  */
                                                                                 /* corresponding bit in the FLCTL_INTFLAG_REG */
#define FLCTL_INTEN_PRG_ERR                                (0x00000200)          /* If set to 1, enables the Controller to generate an interrupt based on the  */
                                                                                 /* corresponding bit in the FLCTL_INTFLAG_REG */
#define FLCTL_INTEN_RESERVED_1__M                          (0xfffffc00)          /* Reserved. Reads return 0h */

/* FLCTL_INTCLR Control Bits */
#define FLCTL_INTCLR_RDBRST                                (0x00000001)          /* Write 1 clears the corresponding interrupt flag bit in the FLCTL_INTFLAG_REG */
#define FLCTL_INTCLR_AVPRE                                 (0x00000002)          /* Write 1 clears the corresponding interrupt flag bit in the FLCTL_INTFLAG_REG */
#define FLCTL_INTCLR_AVPST                                 (0x00000004)          /* Write 1 clears the corresponding interrupt flag bit in the FLCTL_INTFLAG_REG */
#define FLCTL_INTCLR_PRG                                   (0x00000008)          /* Write 1 clears the corresponding interrupt flag bit in the FLCTL_INTFLAG_REG */
#define FLCTL_INTCLR_PRGB                                  (0x00000010)          /* Write 1 clears the corresponding interrupt flag bit in the FLCTL_INTFLAG_REG */
#define FLCTL_INTCLR_ERASE                                 (0x00000020)          /* Write 1 clears the corresponding interrupt flag bit in the FLCTL_INTFLAG_REG */
#define FLCTL_INTCLR_PARBNK0                               (0x00000040)          /* Write 1 clears the corresponding interrupt flag bit in the    FLCTL_INTFLAG_REG */
#define FLCTL_INTCLR_PARBNK1                               (0x00000080)          /* Write 1 clears the corresponding interrupt flag bit in the    FLCTL_INTFLAG_REG */
#define FLCTL_INTCLR_BMRK                                  (0x00000100)          /* Write 1 clears the corresponding interrupt flag bit in the FLCTL_INTFLAG_REG */
#define FLCTL_INTCLR_PRG_ERR                               (0x00000200)          /* Write 1 clears the corresponding interrupt flag bit in the    FLCTL_INTFLAG_REG */
#define FLCTL_INTCLR_RESERVED_1__M                         (0xfffffc00)          /* Reserved. Reads return 0h */

/* FLCTL_INTSET Control Bits */
#define FLCTL_INTSET_RDBRST                                (0x00000001)          /* Write 1 sets the corresponding interrupt flag bit in the FLCTL_INTFLAG_REG */
#define FLCTL_INTSET_AVPRE                                 (0x00000002)          /* Write 1 sets the corresponding interrupt flag bit in the FLCTL_INTFLAG_REG */
#define FLCTL_INTSET_AVPST                                 (0x00000004)          /* Write 1 sets the corresponding interrupt flag bit in the FLCTL_INTFLAG_REG */
#define FLCTL_INTSET_PRG                                   (0x00000008)          /* Write 1 sets the corresponding interrupt flag bit in the FLCTL_INTFLAG_REG */
#define FLCTL_INTSET_PRGB                                  (0x00000010)          /* Write 1 sets the corresponding interrupt flag bit in the FLCTL_INTFLAG_REG */
#define FLCTL_INTSET_ERASE                                 (0x00000020)          /* Write 1 sets the corresponding interrupt flag bit in the FLCTL_INTFLAG_REG */
#define FLCTL_INTSET_PARBNK0                               (0x00000040)          /* Write 1 sets the corresponding interrupt flag bit in the FLCTL_INTFLAG_REG */
#define FLCTL_INTSET_PARBNK1                               (0x00000080)          /* Write 1 sets the corresponding interrupt flag bit in the FLCTL_INTFLAG_REG */
#define FLCTL_INTSET_BMRK                                  (0x00000100)          /* Write 1 sets the corresponding interrupt flag bit in the FLCTL_INTFLAG_REG */
#define FLCTL_INTSET_PRG_ERR                               (0x00000200)          /* Write 1 sets the corresponding interrupt flag bit in the FLCTL_INTFLAG_REG */
#define FLCTL_INTSET_RESERVED_1__M                         (0xfffffc00)          /* Reserved. Reads return 0h */

/* FLCTL_READ_TIMCTL Control Bits */
#define FLCTL_READ_TIMCTL_SETUP__M                         (0x000000ff)          /* Configures the length of the Setup phase for this operation */
#define FLCTL_READ_TIMCTL_RESERVED_2__M                    (0x00000f00)          /* Reserved. Always returns 0h */
#define FLCTL_READ_TIMCTL_IREF_BOOST1__M                   (0x0000f000)          /* Configures the length of the IREF_BOOST1 signal of the IP */
#define FLCTL_READ_TIMCTL_SETUP_LONG__M                    (0x00ff0000)          /* Configures the length of the Setup time into read mode when the device is  */
                                                                                 /* recovering from one of the following conditions  Moving from Power-down or  */
                                                                                 /* Standby back to Active  #b#and#/b# device is not trimmed Moving from LPR standby */
                                                                                 /* to LPR Active Recovering from the LDO Boost operation after a Mass Erase */
#define FLCTL_READ_TIMCTL_RESERVED_1__M                    (0xff000000)          /* Reserved. Always returns 0h */

/* FLCTL_READMARGIN_TIMCTL Control Bits */
#define FLCTL_READMARGIN_TIMCTL_SETUP__M                   (0x000000ff)          /* Configures the length of the Setup phase for this operation */
#define FLCTL_READMARGIN_TIMCTL_RESERVED_1__M              (0xffffff00)          /* Reserved. Always returns 0h */

/* FLCTL_PRGVER_TIMCTL Control Bits */
#define FLCTL_PRGVER_TIMCTL_SETUP__M                       (0x000000ff)          /* Configures the length of the Setup phase for this operation */
#define FLCTL_PRGVER_TIMCTL_ACTIVE__M                      (0x00000f00)          /* Configures the length of the Active phase for this operation */
#define FLCTL_PRGVER_TIMCTL_HOLD__M                        (0x0000f000)          /* Configures the length of the Hold phase for this operation */
#define FLCTL_PRGVER_TIMCTL_RESERVED_1__M                  (0xffff0000)          /* Reserved. Reads return 0h */

/* FLCTL_ERSVER_TIMCTL Control Bits */
#define FLCTL_ERSVER_TIMCTL_SETUP__M                       (0x000000ff)          /* Configures the length of the Setup phase for this operation */
#define FLCTL_ERSVER_TIMCTL_RESERVED_1__M                  (0xffffff00)          /* Reserved. Reads return 0h */

/* FLCTL_LKGVER_TIMCTL Control Bits */
#define FLCTL_LKGVER_TIMCTL_SETUP__M                       (0x000000ff)          /* Configures the length of the Setup phase for this operation */
#define FLCTL_LKGVER_TIMCTL_RESERVED_1__M                  (0xffffff00)          /* Reserved. Reads return 0h */

/* FLCTL_PROGRAM_TIMCTL Control Bits */
#define FLCTL_PROGRAM_TIMCTL_SETUP__M                      (0x000000ff)          /* Configures the length of the Setup phase for this operation */
#define FLCTL_PROGRAM_TIMCTL_ACTIVE__M                     (0x0fffff00)          /* Configures the length of the Active phase for this operation */
#define FLCTL_PROGRAM_TIMCTL_HOLD__M                       (0xf0000000)          /* Configures the length of the Hold phase for this operation */

/* FLCTL_ERASE_TIMCTL Control Bits */
#define FLCTL_ERASE_TIMCTL_SETUP__M                        (0x000000ff)          /* Configures the length of the Setup phase for this operation */
#define FLCTL_ERASE_TIMCTL_ACTIVE__M                       (0x0fffff00)          /* Configures the length of the Active phase for this operation */
#define FLCTL_ERASE_TIMCTL_HOLD__M                         (0xf0000000)          /* Configures the length of the Hold phase for this operation */

/* FLCTL_MASSERASE_TIMCTL Control Bits */
#define FLCTL_MASSERASE_TIMCTL_BOOST_ACTIVE__M             (0x000000ff)          /* Configures the length of the time for which LDO Boost Signal is kept active */
#define FLCTL_MASSERASE_TIMCTL_BOOST_HOLD__M               (0x0000ff00)          /* Configures the length for which Flash deactivates the LDO Boost signal before  */
                                                                                 /* processing any new commands */
#define FLCTL_MASSERASE_TIMCTL_RESERVED_1__M               (0xffff0000)          /* Reserved. Reads return 0h */


//*****************************************************************************
// FPB Registers
//*****************************************************************************
typedef struct {
  union {    /* FPB_FP_CTRL Register */
    __IO uint32_t reg;
    struct { /* FPB_FP_CTRL Control Bits */
      __IO uint32_t ENABLE              	:	1;  /* Flash patch unit enable bit */
      __O  uint32_t KEY                 	:	1;  /* Key field. To write to the Flash Patch Control Register, you must write a 1 to  */
                                                /* this write-only bit. */
      __I  uint32_t NUM_CODE1           	:	4;  /* Number of code slots field. */
      __I  uint32_t NUM_LIT             	:	4;  /* Number of literal slots field. */
      __I  uint32_t NUM_CODE2           	:	2;  /* Number of full banks of code comparators, sixteen comparators per bank. Where  */
                                                /* less than sixteen code comparators are provided, the bank count is zero, and the */
                                                /* number present indicated by NUM_CODE. This read only field contains 3'b000 to  */
                                                /* indicate 0 banks for Cortex-M4 processor. */

    } bit;
  } FP_CTRL;
  union {    /* FPB_FP_REMAP Register */
    __IO uint32_t reg;
    struct { /* FPB_FP_REMAP Control Bits */
      __IO uint32_t REMAP               	:	24;  /* Remap base address field. */

    } bit;
  } FP_REMAP;
  union {    /* FPB_FP_COMP0 Register */
    __IO uint32_t reg;
    struct { /* FPB_FP_COMP0 Control Bits */
      __IO uint32_t ENABLE              	:	1;  /* Compare and remap enable for Flash Patch Comparator Register 0. The ENABLE bit  */
                                                /* of FP_CTRL must also be set to enable comparisons. Reset clears the ENABLE bit. */
      __IO uint32_t COMP                	:	27;  /* Comparison address. */
      __IO uint32_t REPLACE             	:	2;  /* This selects what happens when the COMP address is matched. Settings other than  */
                                                /* b00 are only valid for instruction comparators. Literal comparators ignore non- */
                                                /* b00 settings. Address remapping only takes place for the b00 setting. */

    } bit;
  } FP_COMP0;
  union {    /* FPB_FP_COMP1 Register */
    __IO uint32_t reg;
    struct { /* FPB_FP_COMP1 Control Bits */
      __IO uint32_t ENABLE              	:	1;  /* Compare and remap enable for Flash Patch Comparator Register 1. The ENABLE bit  */
                                                /* of FP_CTRL must also be set to enable comparisons. Reset clears the ENABLE bit. */
      __IO uint32_t COMP                	:	27;  /* Comparison address. */
      __IO uint32_t REPLACE             	:	2;  /* This selects what happens when the COMP address is matched. Settings other than  */
                                                /* b00 are only valid for instruction comparators. Literal comparators ignore non- */
                                                /* b00 settings. Address remapping only takes place for the b00 setting. */

    } bit;
  } FP_COMP1;
  union {    /* FPB_FP_COMP2 Register */
    __IO uint32_t reg;
    struct { /* FPB_FP_COMP2 Control Bits */
      __IO uint32_t ENABLE              	:	1;  /* Compare and remap enable for Flash Patch Comparator Register 2. The ENABLE bit  */
                                                /* of FP_CTRL must also be set to enable comparisons. Reset clears the ENABLE bit. */
      __IO uint32_t COMP                	:	27;  /* Comparison address. */
      __IO uint32_t REPLACE             	:	2;  /* This selects what happens when the COMP address is matched. Settings other than  */
                                                /* b00 are only valid for instruction comparators. Literal comparators ignore non- */
                                                /* b00 settings. Address remapping only takes place for the b00 setting. */

    } bit;
  } FP_COMP2;
  union {    /* FPB_FP_COMP3 Register */
    __IO uint32_t reg;
    struct { /* FPB_FP_COMP3 Control Bits */
      __IO uint32_t ENABLE              	:	1;  /* Compare and remap enable for Flash Patch Comparator Register 3. The ENABLE bit  */
                                                /* of FP_CTRL must also be set to enable comparisons. Reset clears the ENABLE bit. */
      __IO uint32_t COMP                	:	27;  /* Comparison address. */
      __IO uint32_t REPLACE             	:	2;  /* This selects what happens when the COMP address is matched. Settings other than  */
                                                /* b00 are only valid for instruction comparators. Literal comparators ignore non- */
                                                /* b00 settings. Address remapping only takes place for the b00 setting. */

    } bit;
  } FP_COMP3;
  union {    /* FPB_FP_COMP4 Register */
    __IO uint32_t reg;
    struct { /* FPB_FP_COMP4 Control Bits */
      __IO uint32_t ENABLE              	:	1;  /* Compare and remap enable for Flash Patch Comparator Register 4. The ENABLE bit  */
                                                /* of FP_CTRL must also be set to enable comparisons. Reset clears the ENABLE bit. */
      __IO uint32_t COMP                	:	27;  /* Comparison address. */
      __IO uint32_t REPLACE             	:	2;  /* This selects what happens when the COMP address is matched. Settings other than  */
                                                /* b00 are only valid for instruction comparators. Literal comparators ignore non- */
                                                /* b00 settings. Address remapping only takes place for the b00 setting. */

    } bit;
  } FP_COMP4;
  union {    /* FPB_FP_COMP5 Register */
    __IO uint32_t reg;
    struct { /* FPB_FP_COMP5 Control Bits */
      __IO uint32_t ENABLE              	:	1;  /* Compare and remap enable for Flash Patch Comparator Register 5. The ENABLE bit  */
                                                /* of FP_CTRL must also be set to enable comparisons. Reset clears the ENABLE bit. */
      __IO uint32_t COMP                	:	27;  /* Comparison address. */
      __IO uint32_t REPLACE             	:	2;  /* This selects what happens when the COMP address is matched. Settings other than  */
                                                /* b00 are only valid for instruction comparators. Literal comparators ignore non- */
                                                /* b00 settings. Address remapping only takes place for the b00 setting. */

    } bit;
  } FP_COMP5;
  union {    /* FPB_FP_COMP6 Register */
    __IO uint32_t reg;
    struct { /* FPB_FP_COMP6 Control Bits */
      __IO uint32_t ENABLE              	:	1;  /* Compare and remap enable for Flash Patch Comparator Register 6. The ENABLE bit  */
                                                /* of FP_CTRL must also be set to enable comparisons. Reset clears the ENABLE bit. */
      __IO uint32_t COMP                	:	27;  /* Comparison address. */
      __IO uint32_t REPLACE             	:	2;  /* This selects what happens when the COMP address is matched. Settings other than  */
                                                /* b00 are only valid for instruction comparators. Literal comparators ignore non- */
                                                /* b00 settings. Address remapping only takes place for the b00 setting. */

    } bit;
  } FP_COMP6;
  union {    /* FPB_FP_COMP7 Register */
    __IO uint32_t reg;
    struct { /* FPB_FP_COMP7 Control Bits */
      __IO uint32_t ENABLE              	:	1;  /* Compare and remap enable for Flash Patch Comparator Register 7. The ENABLE bit  */
                                                /* of FP_CTRL must also be set to enable comparisons. Reset clears the ENABLE bit. */
      __IO uint32_t COMP                	:	27;  /* Comparison address. */
      __IO uint32_t REPLACE             	:	2;  /* This selects what happens when the COMP address is matched. Settings other than  */
                                                /* b00 are only valid for instruction comparators. Literal comparators ignore non- */
                                                /* b00 settings. Address remapping only takes place for the b00 setting. */

    } bit;
  } FP_COMP7;
  uint8_t RESERVED0[4008];
  uint32_t PID4; /* Peripheral identification register (PID4).  */
  uint32_t PID5; /* Peripheral identification register (PID5).  */
  uint32_t PID6; /* Peripheral identification register (PID6).  */
  uint32_t PID7; /* Peripheral identification register (PID7).  */
  uint32_t PID0; /* Peripheral identification register Bits 7:0 (PID0).  */
  uint32_t PID1; /* Peripheral identification register Bits 15:8 (PID1).  */
  uint32_t PID2; /* Peripheral identification register Bits 23:16 (PID2).  */
  uint32_t PID3; /* Peripheral identification register Bits 31:24 (PID3).  */
  uint32_t CID0; /* Component identification register Bits 7:0 (CID0).  */
  uint32_t CID1; /* Component identification register Bits 15:8 (CID1).  */
  uint32_t CID2; /* Component identification register Bits 23:16 (CID2).  */
  uint32_t CID3; /* Component identification register Bits 31:24 (CID3).  */
} FPB_Type;


/* FPB_FP_CTRL Control Bits */
#define FPB_FP_CTRL_ENABLE                                 (0x00000001)          /* Flash patch unit enable bit */
#define FPB_FP_CTRL_KEY                                    (0x00000002)          /* Key field. To write to the Flash Patch Control Register, you must write a 1 to  */
                                                                                 /* this write-only bit. */
#define FPB_FP_CTRL_NUM_CODE1__M                           (0x000000f0)          /* Number of code slots field. */
#define FPB_FP_CTRL_NUM_CODE1__0                           (0x00000000)          /* no code slots */
#define FPB_FP_CTRL_NUM_CODE1__0_NO_CODE_SLOTS             (0x00000000)          /* no code slots */
#define FPB_FP_CTRL_NUM_CODE1__2                           (0x00000020)          /* two code slots */
#define FPB_FP_CTRL_NUM_CODE1__2_TWO_CODE_SLOTS            (0x00000020)          /* two code slots */
#define FPB_FP_CTRL_NUM_CODE1__6                           (0x00000060)          /* six code slots */
#define FPB_FP_CTRL_NUM_CODE1__6_SIX_CODE_SLOTS            (0x00000060)          /* six code slots */
#define FPB_FP_CTRL_NUM_LIT__M                             (0x00000f00)          /* Number of literal slots field. */
#define FPB_FP_CTRL_NUM_LIT__0                             (0x00000000)          /* no literal slots */
#define FPB_FP_CTRL_NUM_LIT__2                             (0x00000200)          /* two literal slots */
#define FPB_FP_CTRL_NUM_CODE2__M                           (0x00003000)          /* Number of full banks of code comparators, sixteen comparators per bank. Where  */
                                                                                 /* less than sixteen code comparators are provided, the bank count is zero, and the */
                                                                                 /* number present indicated by NUM_CODE. This read only field contains 3'b000 to  */
                                                                                 /* indicate 0 banks for Cortex-M4 processor. */

/* FPB_FP_REMAP Control Bits */
#define FPB_FP_REMAP_REMAP__M                              (0x1fffffe0)          /* Remap base address field. */

/* FPB_FP_COMP0 Control Bits */
#define FPB_FP_COMP0_ENABLE                                (0x00000001)          /* Compare and remap enable for Flash Patch Comparator Register 0. The ENABLE bit  */
                                                                                 /* of FP_CTRL must also be set to enable comparisons. Reset clears the ENABLE bit. */
#define FPB_FP_COMP0_COMP__M                               (0x1ffffffc)          /* Comparison address. */
#define FPB_FP_COMP0_REPLACE__M                            (0xc0000000)          /* This selects what happens when the COMP address is matched. Settings other than  */
                                                                                 /* b00 are only valid for instruction comparators. Literal comparators ignore non- */
                                                                                 /* b00 settings. Address remapping only takes place for the b00 setting. */
#define FPB_FP_COMP0_REPLACE__0                            (0x00000000)          /* remap to remap address. See FP_REMAP */
#define FPB_FP_COMP0_REPLACE__1                            (0x40000000)          /* set BKPT on lower halfword, upper is unaffected */
#define FPB_FP_COMP0_REPLACE__2                            (0x80000000)          /* set BKPT on upper halfword, lower is unaffected */
#define FPB_FP_COMP0_REPLACE__3                            (0xc0000000)          /* set BKPT on both lower and upper halfwords. */

/* FPB_FP_COMP1 Control Bits */
#define FPB_FP_COMP1_ENABLE                                (0x00000001)          /* Compare and remap enable for Flash Patch Comparator Register 1. The ENABLE bit  */
                                                                                 /* of FP_CTRL must also be set to enable comparisons. Reset clears the ENABLE bit. */
#define FPB_FP_COMP1_COMP__M                               (0x1ffffffc)          /* Comparison address. */
#define FPB_FP_COMP1_REPLACE__M                            (0xc0000000)          /* This selects what happens when the COMP address is matched. Settings other than  */
                                                                                 /* b00 are only valid for instruction comparators. Literal comparators ignore non- */
                                                                                 /* b00 settings. Address remapping only takes place for the b00 setting. */
#define FPB_FP_COMP1_REPLACE__0                            (0x00000000)          /* remap to remap address. See FP_REMAP */
#define FPB_FP_COMP1_REPLACE__1                            (0x40000000)          /* set BKPT on lower halfword, upper is unaffected */
#define FPB_FP_COMP1_REPLACE__2                            (0x80000000)          /* set BKPT on upper halfword, lower is unaffected */
#define FPB_FP_COMP1_REPLACE__3                            (0xc0000000)          /* set BKPT on both lower and upper halfwords. */

/* FPB_FP_COMP2 Control Bits */
#define FPB_FP_COMP2_ENABLE                                (0x00000001)          /* Compare and remap enable for Flash Patch Comparator Register 2. The ENABLE bit  */
                                                                                 /* of FP_CTRL must also be set to enable comparisons. Reset clears the ENABLE bit. */
#define FPB_FP_COMP2_COMP__M                               (0x1ffffffc)          /* Comparison address. */
#define FPB_FP_COMP2_REPLACE__M                            (0xc0000000)          /* This selects what happens when the COMP address is matched. Settings other than  */
                                                                                 /* b00 are only valid for instruction comparators. Literal comparators ignore non- */
                                                                                 /* b00 settings. Address remapping only takes place for the b00 setting. */
#define FPB_FP_COMP2_REPLACE__0                            (0x00000000)          /* remap to remap address. See FP_REMAP */
#define FPB_FP_COMP2_REPLACE__1                            (0x40000000)          /* set BKPT on lower halfword, upper is unaffected */
#define FPB_FP_COMP2_REPLACE__2                            (0x80000000)          /* set BKPT on upper halfword, lower is unaffected */
#define FPB_FP_COMP2_REPLACE__3                            (0xc0000000)          /* set BKPT on both lower and upper halfwords. */

/* FPB_FP_COMP3 Control Bits */
#define FPB_FP_COMP3_ENABLE                                (0x00000001)          /* Compare and remap enable for Flash Patch Comparator Register 3. The ENABLE bit  */
                                                                                 /* of FP_CTRL must also be set to enable comparisons. Reset clears the ENABLE bit. */
#define FPB_FP_COMP3_COMP__M                               (0x1ffffffc)          /* Comparison address. */
#define FPB_FP_COMP3_REPLACE__M                            (0xc0000000)          /* This selects what happens when the COMP address is matched. Settings other than  */
                                                                                 /* b00 are only valid for instruction comparators. Literal comparators ignore non- */
                                                                                 /* b00 settings. Address remapping only takes place for the b00 setting. */
#define FPB_FP_COMP3_REPLACE__0                            (0x00000000)          /* remap to remap address. See FP_REMAP */
#define FPB_FP_COMP3_REPLACE__1                            (0x40000000)          /* set BKPT on lower halfword, upper is unaffected */
#define FPB_FP_COMP3_REPLACE__2                            (0x80000000)          /* set BKPT on upper halfword, lower is unaffected */
#define FPB_FP_COMP3_REPLACE__3                            (0xc0000000)          /* set BKPT on both lower and upper halfwords. */

/* FPB_FP_COMP4 Control Bits */
#define FPB_FP_COMP4_ENABLE                                (0x00000001)          /* Compare and remap enable for Flash Patch Comparator Register 4. The ENABLE bit  */
                                                                                 /* of FP_CTRL must also be set to enable comparisons. Reset clears the ENABLE bit. */
#define FPB_FP_COMP4_COMP__M                               (0x1ffffffc)          /* Comparison address. */
#define FPB_FP_COMP4_REPLACE__M                            (0xc0000000)          /* This selects what happens when the COMP address is matched. Settings other than  */
                                                                                 /* b00 are only valid for instruction comparators. Literal comparators ignore non- */
                                                                                 /* b00 settings. Address remapping only takes place for the b00 setting. */
#define FPB_FP_COMP4_REPLACE__0                            (0x00000000)          /* remap to remap address. See FP_REMAP */
#define FPB_FP_COMP4_REPLACE__1                            (0x40000000)          /* set BKPT on lower halfword, upper is unaffected */
#define FPB_FP_COMP4_REPLACE__2                            (0x80000000)          /* set BKPT on upper halfword, lower is unaffected */
#define FPB_FP_COMP4_REPLACE__3                            (0xc0000000)          /* set BKPT on both lower and upper halfwords. */

/* FPB_FP_COMP5 Control Bits */
#define FPB_FP_COMP5_ENABLE                                (0x00000001)          /* Compare and remap enable for Flash Patch Comparator Register 5. The ENABLE bit  */
                                                                                 /* of FP_CTRL must also be set to enable comparisons. Reset clears the ENABLE bit. */
#define FPB_FP_COMP5_COMP__M                               (0x1ffffffc)          /* Comparison address. */
#define FPB_FP_COMP5_REPLACE__M                            (0xc0000000)          /* This selects what happens when the COMP address is matched. Settings other than  */
                                                                                 /* b00 are only valid for instruction comparators. Literal comparators ignore non- */
                                                                                 /* b00 settings. Address remapping only takes place for the b00 setting. */
#define FPB_FP_COMP5_REPLACE__0                            (0x00000000)          /* remap to remap address. See FP_REMAP */
#define FPB_FP_COMP5_REPLACE__1                            (0x40000000)          /* set BKPT on lower halfword, upper is unaffected */
#define FPB_FP_COMP5_REPLACE__2                            (0x80000000)          /* set BKPT on upper halfword, lower is unaffected */
#define FPB_FP_COMP5_REPLACE__3                            (0xc0000000)          /* set BKPT on both lower and upper halfwords. */

/* FPB_FP_COMP6 Control Bits */
#define FPB_FP_COMP6_ENABLE                                (0x00000001)          /* Compare and remap enable for Flash Patch Comparator Register 6. The ENABLE bit  */
                                                                                 /* of FP_CTRL must also be set to enable comparisons. Reset clears the ENABLE bit. */
#define FPB_FP_COMP6_COMP__M                               (0x1ffffffc)          /* Comparison address. */
#define FPB_FP_COMP6_REPLACE__M                            (0xc0000000)          /* This selects what happens when the COMP address is matched. Settings other than  */
                                                                                 /* b00 are only valid for instruction comparators. Literal comparators ignore non- */
                                                                                 /* b00 settings. Address remapping only takes place for the b00 setting. */
#define FPB_FP_COMP6_REPLACE__0                            (0x00000000)          /* remap to remap address. See FP_REMAP */
#define FPB_FP_COMP6_REPLACE__1                            (0x40000000)          /* set BKPT on lower halfword, upper is unaffected */
#define FPB_FP_COMP6_REPLACE__2                            (0x80000000)          /* set BKPT on upper halfword, lower is unaffected */
#define FPB_FP_COMP6_REPLACE__3                            (0xc0000000)          /* set BKPT on both lower and upper halfwords. */

/* FPB_FP_COMP7 Control Bits */
#define FPB_FP_COMP7_ENABLE                                (0x00000001)          /* Compare and remap enable for Flash Patch Comparator Register 7. The ENABLE bit  */
                                                                                 /* of FP_CTRL must also be set to enable comparisons. Reset clears the ENABLE bit. */
#define FPB_FP_COMP7_COMP__M                               (0x1ffffffc)          /* Comparison address. */
#define FPB_FP_COMP7_REPLACE__M                            (0xc0000000)          /* This selects what happens when the COMP address is matched. Settings other than  */
                                                                                 /* b00 are only valid for instruction comparators. Literal comparators ignore non- */
                                                                                 /* b00 settings. Address remapping only takes place for the b00 setting. */
#define FPB_FP_COMP7_REPLACE__0                            (0x00000000)          /* remap to remap address. See FP_REMAP */
#define FPB_FP_COMP7_REPLACE__1                            (0x40000000)          /* set BKPT on lower halfword, upper is unaffected */
#define FPB_FP_COMP7_REPLACE__2                            (0x80000000)          /* set BKPT on upper halfword, lower is unaffected */
#define FPB_FP_COMP7_REPLACE__3                            (0xc0000000)          /* set BKPT on both lower and upper halfwords. */


//*****************************************************************************
// P1 Registers
//*****************************************************************************
typedef struct {
  uint8_t IN; /* Port 1 Input */
  uint8_t RESERVED0[1];
  uint8_t OUT; /* Port 1 Output */
  uint8_t RESERVED1[1];
  uint8_t DIR; /* Port 1 Direction */
  uint8_t RESERVED2[1];
  uint8_t REN; /* Port 1 Resistor Enable */
  uint8_t RESERVED3[1];
  uint8_t DS; /* Port 1 Drive Strength */
  uint8_t RESERVED4[1];
  uint8_t SEL0; /* Port 1 Select 0 */
  uint8_t RESERVED5[1];
  uint8_t SEL1; /* Port 1 Select 1 */
  uint8_t RESERVED6[1];
  uint8_t IV; /* Port 1 Interrupt Vector */
  uint8_t RESERVED7[7];
  uint8_t SELC; /* Port 1 Complement Selection */
  uint8_t RESERVED8[1];
  uint8_t IES; /* Port 1 Interrupt Edge Select */
  uint8_t RESERVED9[1];
  uint8_t IE; /* Port 1 Interrupt Enable */
  uint8_t RESERVED10[1];
  uint8_t IFG; /* Port 1 Interrupt Flag */
} P1_Type;



//*****************************************************************************
// P10 Registers
//*****************************************************************************
typedef struct {
  uint8_t IN; /* Port 10 Input */
  uint8_t RESERVED0[1];
  uint8_t OUT; /* Port 10 Output */
  uint8_t RESERVED1[1];
  uint8_t DIR; /* Port 10 Direction */
  uint8_t RESERVED2[1];
  uint8_t REN; /* Port 10 Resistor Enable */
  uint8_t RESERVED3[1];
  uint8_t DS; /* Port 10 Drive Strength */
  uint8_t RESERVED4[1];
  uint8_t SEL0; /* Port 10 Select 0 */
  uint8_t RESERVED5[1];
  uint8_t SEL1; /* Port 10 Select 1 */
  uint8_t RESERVED6[9];
  uint8_t SELC; /* Port 10 Complement Selection */
  uint8_t RESERVED7[1];
  uint8_t IES; /* Port 10 Interrupt Edge Select */
  uint8_t RESERVED8[1];
  uint8_t IE; /* Port 10 Interrupt Enable */
  uint8_t RESERVED9[1];
  uint8_t IFG; /* Port 10 Interrupt Flag */
  uint8_t IV; /* Port 10 Interrupt Vector */
} P10_Type;



//*****************************************************************************
// P2 Registers
//*****************************************************************************
typedef struct {
  uint8_t IN; /* Port 2 Input */
  uint8_t RESERVED0[1];
  uint8_t OUT; /* Port 2 Output */
  uint8_t RESERVED1[1];
  uint8_t DIR; /* Port 2 Direction */
  uint8_t RESERVED2[1];
  uint8_t REN; /* Port 2 Resistor Enable */
  uint8_t RESERVED3[1];
  uint8_t DS; /* Port 2 Drive Strength */
  uint8_t RESERVED4[1];
  uint8_t SEL0; /* Port 2 Select 0 */
  uint8_t RESERVED5[1];
  uint8_t SEL1; /* Port 2 Select 1 */
  uint8_t RESERVED6[9];
  uint8_t SELC; /* Port 2 Complement Selection */
  uint8_t RESERVED7[1];
  uint8_t IES; /* Port 2 Interrupt Edge Select */
  uint8_t RESERVED8[1];
  uint8_t IE; /* Port 2 Interrupt Enable */
  uint8_t RESERVED9[1];
  uint8_t IFG; /* Port 2 Interrupt Flag */
  uint8_t IV; /* Port 2 Interrupt Vector */
} P2_Type;



//*****************************************************************************
// P3 Registers
//*****************************************************************************
typedef struct {
  uint8_t IN; /* Port 3 Input */
  uint8_t RESERVED0[1];
  uint8_t OUT; /* Port 3 Output */
  uint8_t RESERVED1[1];
  uint8_t DIR; /* Port 3 Direction */
  uint8_t RESERVED2[1];
  uint8_t REN; /* Port 3 Resistor Enable */
  uint8_t RESERVED3[1];
  uint8_t DS; /* Port 3 Drive Strength */
  uint8_t RESERVED4[1];
  uint8_t SEL0; /* Port 3 Select 0 */
  uint8_t RESERVED5[1];
  uint8_t SEL1; /* Port 3 Select 1 */
  uint8_t RESERVED6[1];
  uint8_t IV; /* Port 3 Interrupt Vector */
  uint8_t RESERVED7[7];
  uint8_t SELC; /* Port 3 Complement Selection */
  uint8_t RESERVED8[1];
  uint8_t IES; /* Port 3 Interrupt Edge Select */
  uint8_t RESERVED9[1];
  uint8_t IE; /* Port 3 Interrupt Enable */
  uint8_t RESERVED10[1];
  uint8_t IFG; /* Port 3 Interrupt Flag */
} P3_Type;



//*****************************************************************************
// P4 Registers
//*****************************************************************************
typedef struct {
  uint8_t IN; /* Port 4 Input */
  uint8_t RESERVED0[1];
  uint8_t OUT; /* Port 4 Output */
  uint8_t RESERVED1[1];
  uint8_t DIR; /* Port 4 Direction */
  uint8_t RESERVED2[1];
  uint8_t REN; /* Port 4 Resistor Enable */
  uint8_t RESERVED3[1];
  uint8_t DS; /* Port 4 Drive Strength */
  uint8_t RESERVED4[1];
  uint8_t SEL0; /* Port 4 Select 0 */
  uint8_t RESERVED5[1];
  uint8_t SEL1; /* Port 4 Select 1 */
  uint8_t RESERVED6[9];
  uint8_t SELC; /* Port 4 Complement Selection */
  uint8_t RESERVED7[1];
  uint8_t IES; /* Port 4 Interrupt Edge Select */
  uint8_t RESERVED8[1];
  uint8_t IE; /* Port 4 Interrupt Enable */
  uint8_t RESERVED9[1];
  uint8_t IFG; /* Port 4 Interrupt Flag */
  uint8_t IV; /* Port 4 Interrupt Vector */
} P4_Type;



//*****************************************************************************
// P5 Registers
//*****************************************************************************
typedef struct {
  uint8_t IN; /* Port 5 Input */
  uint8_t RESERVED0[1];
  uint8_t OUT; /* Port 5 Output */
  uint8_t RESERVED1[1];
  uint8_t DIR; /* Port 5 Direction */
  uint8_t RESERVED2[1];
  uint8_t REN; /* Port 5 Resistor Enable */
  uint8_t RESERVED3[1];
  uint8_t DS; /* Port 5 Drive Strength */
  uint8_t RESERVED4[1];
  uint8_t SEL0; /* Port 5 Select 0 */
  uint8_t RESERVED5[1];
  uint8_t SEL1; /* Port 5 Select 1 */
  uint8_t RESERVED6[1];
  uint8_t IV; /* Port 5 Interrupt Vector */
  uint8_t RESERVED7[7];
  uint8_t SELC; /* Port 5 Complement Selection */
  uint8_t RESERVED8[1];
  uint8_t IES; /* Port 5 Interrupt Edge Select */
  uint8_t RESERVED9[1];
  uint8_t IE; /* Port 5 Interrupt Enable */
  uint8_t RESERVED10[1];
  uint8_t IFG; /* Port 5 Interrupt Flag */
} P5_Type;



//*****************************************************************************
// P6 Registers
//*****************************************************************************
typedef struct {
  uint8_t IN; /* Port 6 Input */
  uint8_t RESERVED0[1];
  uint8_t OUT; /* Port 6 Output */
  uint8_t RESERVED1[1];
  uint8_t DIR; /* Port 6 Direction */
  uint8_t RESERVED2[1];
  uint8_t REN; /* Port 6 Resistor Enable */
  uint8_t RESERVED3[1];
  uint8_t DS; /* Port 6 Drive Strength */
  uint8_t RESERVED4[1];
  uint8_t SEL0; /* Port 6 Select 0 */
  uint8_t RESERVED5[1];
  uint8_t SEL1; /* Port 6 Select 1 */
  uint8_t RESERVED6[9];
  uint8_t SELC; /* Port 6 Complement Selection */
  uint8_t RESERVED7[1];
  uint8_t IES; /* Port 6 Interrupt Edge Select */
  uint8_t RESERVED8[1];
  uint8_t IE; /* Port 6 Interrupt Enable */
  uint8_t RESERVED9[1];
  uint8_t IFG; /* Port 6 Interrupt Flag */
  uint8_t IV; /* Port 6 Interrupt Vector */
} P6_Type;



//*****************************************************************************
// P7 Registers
//*****************************************************************************
typedef struct {
  uint8_t IN; /* Port 7 Input */
  uint8_t RESERVED0[1];
  uint8_t OUT; /* Port 7 Output */
  uint8_t RESERVED1[1];
  uint8_t DIR; /* Port 7 Direction */
  uint8_t RESERVED2[1];
  uint8_t REN; /* Port 7 Resistor Enable */
  uint8_t RESERVED3[1];
  uint8_t DS; /* Port 7 Drive Strength */
  uint8_t RESERVED4[1];
  uint8_t SEL0; /* Port 7 Select 0 */
  uint8_t RESERVED5[1];
  uint8_t SEL1; /* Port 7 Select 1 */
  uint8_t RESERVED6[1];
  uint8_t IV; /* Port 7 Interrupt Vector */
  uint8_t RESERVED7[7];
  uint8_t SELC; /* Port 7 Complement Selection */
  uint8_t RESERVED8[1];
  uint8_t IES; /* Port 7 Interrupt Edge Select */
  uint8_t RESERVED9[1];
  uint8_t IE; /* Port 7 Interrupt Enable */
  uint8_t RESERVED10[1];
  uint8_t IFG; /* Port 7 Interrupt Flag */
} P7_Type;



//*****************************************************************************
// P8 Registers
//*****************************************************************************
typedef struct {
  uint8_t IN; /* Port 8 Input */
  uint8_t RESERVED0[1];
  uint8_t OUT; /* Port 8 Output */
  uint8_t RESERVED1[1];
  uint8_t DIR; /* Port 8 Direction */
  uint8_t RESERVED2[1];
  uint8_t REN; /* Port 8 Resistor Enable */
  uint8_t RESERVED3[1];
  uint8_t DS; /* Port 8 Drive Strength */
  uint8_t RESERVED4[1];
  uint8_t SEL0; /* Port 8 Select 0 */
  uint8_t RESERVED5[1];
  uint8_t SEL1; /* Port 8 Select 1 */
  uint8_t RESERVED6[9];
  uint8_t SELC; /* Port 8 Complement Selection */
  uint8_t RESERVED7[1];
  uint8_t IES; /* Port 8 Interrupt Edge Select */
  uint8_t RESERVED8[1];
  uint8_t IE; /* Port 8 Interrupt Enable */
  uint8_t RESERVED9[1];
  uint8_t IFG; /* Port 8 Interrupt Flag */
  uint8_t IV; /* Port 8 Interrupt Vector */
} P8_Type;



//*****************************************************************************
// P9 Registers
//*****************************************************************************
typedef struct {
  uint8_t IN; /* Port 9 Input */
  uint8_t RESERVED0[1];
  uint8_t OUT; /* Port 9 Output */
  uint8_t RESERVED1[1];
  uint8_t DIR; /* Port 9 Direction */
  uint8_t RESERVED2[1];
  uint8_t REN; /* Port 9 Resistor Enable */
  uint8_t RESERVED3[1];
  uint8_t DS; /* Port 9 Drive Strength */
  uint8_t RESERVED4[1];
  uint8_t SEL0; /* Port 9 Select 0 */
  uint8_t RESERVED5[1];
  uint8_t SEL1; /* Port 9 Select 1 */
  uint8_t RESERVED6[1];
  uint8_t IV; /* Port 9 Interrupt Vector */
  uint8_t RESERVED7[7];
  uint8_t SELC; /* Port 9 Complement Selection */
  uint8_t RESERVED8[1];
  uint8_t IES; /* Port 9 Interrupt Edge Select */
  uint8_t RESERVED9[1];
  uint8_t IE; /* Port 9 Interrupt Enable */
  uint8_t RESERVED10[1];
  uint8_t IFG; /* Port 9 Interrupt Flag */
} P9_Type;



//*****************************************************************************
// PA Registers
//*****************************************************************************
typedef struct {
  uint16_t IN; /* Port A Input */
  uint16_t OUT; /* Port A Output */
  uint16_t DIR; /* Port A Direction */
  uint16_t REN; /* Port A Resistor Enable */
  uint16_t DS; /* Port A Drive Strength */
  uint16_t SEL0; /* Port A Select 0 */
  uint16_t SEL1; /* Port A Select 1 */
  uint8_t RESERVED0[8];
  uint16_t SELC; /* Port A Complement Selection */
  uint16_t IES; /* Port A Interrupt Edge Select */
  uint16_t IE; /* Port A Interrupt Enable */
  uint16_t IFG; /* Port A Interrupt Flag */
} PA_Type;



//*****************************************************************************
// PB Registers
//*****************************************************************************
typedef struct {
  uint16_t IN; /* Port B Input */
  uint16_t OUT; /* Port B Output */
  uint16_t DIR; /* Port B Direction */
  uint16_t REN; /* Port B Resistor Enable */
  uint16_t DS; /* Port B Drive Strength */
  uint16_t SEL0; /* Port B Select 0 */
  uint16_t SEL1; /* Port B Select 1 */
  uint8_t RESERVED0[8];
  uint16_t SELC; /* Port B Complement Selection */
  uint16_t IES; /* Port B Interrupt Edge Select */
  uint16_t IE; /* Port B Interrupt Enable */
  uint16_t IFG; /* Port B Interrupt Flag */
} PB_Type;



//*****************************************************************************
// PC Registers
//*****************************************************************************
typedef struct {
  uint16_t IN; /* Port C Input */
  uint16_t OUT; /* Port C Output */
  uint16_t DIR; /* Port C Direction */
  uint16_t REN; /* Port C Resistor Enable */
  uint16_t DS; /* Port C Drive Strength */
  uint16_t SEL0; /* Port C Select 0 */
  uint16_t SEL1; /* Port C Select 1 */
  uint8_t RESERVED0[8];
  uint16_t SELC; /* Port C Complement Selection */
  uint16_t IES; /* Port C Interrupt Edge Select */
  uint16_t IE; /* Port C Interrupt Enable */
  uint16_t IFG; /* Port C Interrupt Flag */
} PC_Type;



//*****************************************************************************
// PCM Registers
//*****************************************************************************
typedef struct {
  union {    /* PCM_PMR Register */
    __IO uint32_t reg;
    struct { /* PCM_PMR Control Bits */
      __IO uint32_t AMR                 	:	4;  /* Active Mode Request. Used to request active modes. These bits can only be  */
                                                /* modified while PMR_BUSY = 0 of the PCM_CTL_REG 2h-3h = Reserved 6h-7h = Reserved */
                                                /* Ah-Fh = Reserved   */
      __IO uint32_t SDR                 	:	4;  /* Shutdown Mode Request. Used to request shutdown modes. These bits can only be  */
                                                /* modified while PMR_BUSY = 0 of the PCM_CTL_REG. 1h-9h = Reserved. Bh = Reserved. */
                                                /* Dh-Fh = Reserved.   */
      __I  uint32_t CPM                 	:	6;  /* Current Power Mode. These bits reflect the current power mode and will be  */
                                                /* updated once the power mode request completes.  2h-3h = Reserved 6h-7h =  */
                                                /* Reserved Ah-Fh = Reserved 12h-13h = Reserved 16h-17h = Reserved 1A-1Fh =  */
                                                /* Reserved   */
      __I  uint32_t RESERVED_1          	:	2;  /* Reserved */
      __IO uint32_t KEY                 	:	16;  /* PCM key. Must write proper key to allow write access to PMR. Any incorrect key  */
                                                /* will not allow PMR to be written and power request will not be generated. Write  */
                                                /* PCMKEY = 695A_xxxxh to unlock for write access. Always reads back A596h. This is */
                                                /* a word accessible register. Any other type of access (half-word or byte access) */
                                                /* will be ignored by PCM. */

    } bit;
  } PMR;
  union {    /* PCM_CTL Register */
    __IO uint32_t reg;
    struct { /* PCM_CTL Control Bits */
      __IO uint32_t LOCKSD              	:	1;  /* Lock shutdown. This bit once set can only be cleared by the user or by a power  */
                                                /* cycle. This bit is automatically set upon SD0/SDC entry. User cannot set this  */
                                                /* bit. In the write mode this bit should always be written with a value of 0 to  */
                                                /* clear it. Writing a value of 1 has no effect on the system and these writes are  */
                                                /* ignored. */
      __IO uint32_t LOCKRTC             	:	1;  /* Lock RTC. This bit once set can only be cleared by the user or by a power cycle. */
                                                /* This bit is automatically set upon SD0 entry. User cannot set this bit. In the  */
                                                /* write mode this bit should always be written with a value of 0 to clear it. */
                                                /* Writing a value of 1 has no effect on the system and these writes are ignored. */
      __IO uint32_t FORCE_DSL_SD_ENTRY  	:	1;  /* Bit selection for the application to determine whether the entry into DSL/SD0/ */
                                                /* SDC should be forced even if there are active system clocks running which do not */
                                                /* meet the DSL/SD0/SDC criteria. This bit by itself does not cause a DSL/SD0/SDC  */
                                                /* by itself, but is a pre-requisite before the CPU executes WFI. 0b = PCM aborts  */
                                                /* DSL/SD0/SDC mode transition. If on DSL/SD0/SDC entry condition, system clocks  */
                                                /* requests are detected from modules other than RTC and WDT (If sourced from the  */
                                                /* BCLK tree, If RTC/WDT are sourcing from any other clock tree they are also  */
                                                /* considered to block DSL/SD0/SDC entry). PCM generates the SM_INVALID_CLK flag on */
                                                /* abort to DSL/SD0/SDC entry.  1b = PCM enters DSL/SD0/SDC after shuting off the  */
                                                /* clocks forcefully. Application needs to ensure RTC and WDT are clocked using  */
                                                /* BCLK tree to keep these modules alive in DSL/SD0. In SDC all clocks are  */
                                                /* forcefully shutoff and the Vcore is turned off. */
      __I  uint32_t RESERVED_2          	:	5;  /* Reserved */
      __I  uint32_t PMR_BUSY            	:	1;  /* Power mode request busy flag. This flag is set while a power change request is  */
                                                /* being processed. The flag is cleared when the request has completed. Writes to  */
                                                /* the PCM_PMR_REG or Clock System registers are ignored while PMR_BUSY = 1. Reads  */
                                                /* to the PMR or CS registers are possible while PMR_BUSY = 1.  */
      __I  uint32_t RESERVED_1          	:	7;  /* Reserved */
      __IO uint32_t KEY                 	:	16;  /* PCM key. Must write proper key to allow write access to PCMCTL . Any incorrect  */
                                                /* key will not allow PCMCTL to be written and the write will be ignored. Write  */
                                                /* PCMKEY = 695A_xxxxh to unlock for write access. Always reads back A596h. This is */
                                                /* a word accessible register. Any other type of access (half-word/byte access)  */
                                                /* will be ignored by PCM. */

    } bit;
  } CTL;
  union {    /* PCM_INTEN Register */
    __IO uint32_t reg;
    struct { /* PCM_INTEN Control Bits */
      __IO uint32_t EN_SM_INVALID_TR    	:	1;  /* Enable sleep mode invalid transition interrupt. Setting this bit enables an  */
                                                /* interrupt/NMI on an invalid transition from AM to DSL (SDx all transitions are  */
                                                /* allowed).    */
      __IO uint32_t EN_SM_INVALID_CLK   	:	1;  /* Enable sleep mode invalid clock interrupt. Setting this bit enables an interrupt */
                                                /* /NMI on an invalid clock setting during a DSL/SD0/SDC transition from an active  */
                                                /* mode when FORCE_DSL_SD_ENTRY = 0. This bit has not effect when  */
                                                /* FORCE_DSL_SD_ENTRY = 1.   */
      __IO uint32_t EN_AM_INVALID_TR    	:	1;  /* Enable active mode invalid transition. Setting this bit enables an interrupt/NMI */
                                                /* on an invalid transition setting during an active power mode request.    */
      __I  uint32_t RESERVED_2          	:	3;  /* Reserved */
      __IO uint32_t EN_DCDC_ERROR       	:	1;  /* Enable DC to DC error. Setting this bit enables an interrupt/NMI when DC to DC  */
                                                /* operation cannot be achieved or maintained.    */
      __I  uint32_t RESERVED_1          	:	25;  /* Reserved */

    } bit;
  } INTEN;
  union {    /* PCM_INTFLAG Register */
    __IO uint32_t reg;
    struct { /* PCM_INTFLAG Control Bits */
      __I  uint32_t SM_INVALID_TR       	:	1;  /* Sleep mode invalid transition flag. This flag is set if the requested AM to DSL  */
                                                /* transition is invalid. Flag will remain set until cleared by software. */
      __I  uint32_t SM_INVALID_CLK      	:	1;  /* Sleep mode invalid clock flag. This flag is set if the deep sleep mode request  */
                                                /* is invalid due to a clock request active before the DSL/SD0/SDC entry while the  */
                                                /* FORCE_DSL_SD_ENTRY = 0. Flag will remain set until cleared by software. */
      __I  uint32_t AM_INVALID_TR       	:	1;  /* Active mode invalid transition flag. This flag is set if the active mode request */
                                                /* is an invalid transition. Flag will remain set until cleared by software. */
      __I  uint32_t RESERVED_2          	:	3;  /* Reserved */
      __I  uint32_t DCDC_ERROR          	:	1;  /* DC to DC error flag. This flag is set if DC to DC operation cannot be achieved  */
                                                /* or maintained. Flag will remain set until cleared by software. */
      __I  uint32_t RESERVED_1          	:	25;  /* Reserved */

    } bit;
  } INTFLAG;
  union {    /* PCM_INTCLR Register */
    __IO uint32_t reg;
    struct { /* PCM_INTCLR Control Bits */
      __O  uint32_t CLR_SM_INVALID_TR   	:	1;  /* Clear sleep mode invalid transition flag. */
      __O  uint32_t CLR_SM_INVALID_CLK  	:	1;  /* Clear sleep mode invalid clock flag. */
      __O  uint32_t CLR_AM_INVALID_TR   	:	1;  /* Clear active mode invalid transition flag. */
      __O  uint32_t RESERVED_2          	:	3;  /* Reserved */
      __O  uint32_t CLR_DCDC_ERROR      	:	1;  /* Clear DC to DC error flag. */
      __O  uint32_t RESERVED_1          	:	25;  /* Reserved */

    } bit;
  } INTCLR;
} PCM_Type;


/* PCM_PMR Control Bits */
#define PCM_PMR_AMR__M                                     (0x0000000f)          /* Active Mode Request. Used to request active modes. These bits can only be  */
                                                                                 /* modified while PMR_BUSY = 0 of the PCM_CTL_REG 2h-3h = Reserved 6h-7h = Reserved */
                                                                                 /* Ah-Fh = Reserved   */
#define PCM_PMR_AMR__0                                     (0x00000000)          /* AM0_LDO */
#define PCM_PMR_AMR__0_AM0_LDO                             (0x00000000)          /* AM0_LDO */
#define PCM_PMR_AMR__1                                     (0x00000001)          /* AM1_LDO */
#define PCM_PMR_AMR__1_AM1_LDO                             (0x00000001)          /* AM1_LDO */
#define PCM_PMR_AMR__4                                     (0x00000004)          /* AM0_DCDC */
#define PCM_PMR_AMR__4_AM0_DCDC                            (0x00000004)          /* AM0_DCDC */
#define PCM_PMR_AMR__5                                     (0x00000005)          /* AM1_DCDC */
#define PCM_PMR_AMR__5_AM1_DCDC                            (0x00000005)          /* AM1_DCDC */
#define PCM_PMR_AMR__8                                     (0x00000008)          /* AM0_LPR */
#define PCM_PMR_AMR__8_AM0_LPR                             (0x00000008)          /* AM0_LPR */
#define PCM_PMR_AMR__9                                     (0x00000009)          /* AM1_LPR */
#define PCM_PMR_AMR__9_AM1_LPR                             (0x00000009)          /* AM1_LPR */
#define PCM_PMR_SDR__M                                     (0x000000f0)          /* Shutdown Mode Request. Used to request shutdown modes. These bits can only be  */
                                                                                 /* modified while PMR_BUSY = 0 of the PCM_CTL_REG. 1h-9h = Reserved. Bh = Reserved. */
                                                                                 /* Dh-Fh = Reserved.   */
#define PCM_PMR_SDR__0                                     (0x00000000)          /* DSL mode. Core voltage setting is similar to the mode from which DSL is entered. */
#define PCM_PMR_SDR__10                                    (0x000000a0)          /* SD0 partial shutdown mode. Core voltage setting 0. */
#define PCM_PMR_SDR__12                                    (0x000000c0)          /* SDC Complete shutdown mode . */
#define PCM_PMR_CPM__M                                     (0x00003f00)          /* Current Power Mode. These bits reflect the current power mode and will be  */
                                                                                 /* updated once the power mode request completes.  2h-3h = Reserved 6h-7h =  */
                                                                                 /* Reserved Ah-Fh = Reserved 12h-13h = Reserved 16h-17h = Reserved 1A-1Fh =  */
                                                                                 /* Reserved   */
#define PCM_PMR_CPM__0                                     (0x00000000)          /* AM0_LDO */
#define PCM_PMR_CPM__0_AM0_LDO                             (0x00000000)          /* AM0_LDO */
#define PCM_PMR_CPM__1                                     (0x00000100)          /* AM1_LDO */
#define PCM_PMR_CPM__1_AM1_LDO                             (0x00000100)          /* AM1_LDO */
#define PCM_PMR_CPM__4                                     (0x00000400)          /* AM0_DCDC */
#define PCM_PMR_CPM__4_AM0_DCDC                            (0x00000400)          /* AM0_DCDC */
#define PCM_PMR_CPM__5                                     (0x00000500)          /* AM1_DCDC */
#define PCM_PMR_CPM__5_AM1_DCDC                            (0x00000500)          /* AM1_DCDC */
#define PCM_PMR_CPM__8                                     (0x00000800)          /* AM0_LPR */
#define PCM_PMR_CPM__8_AM0_LPR                             (0x00000800)          /* AM0_LPR */
#define PCM_PMR_CPM__9                                     (0x00000900)          /* AM1_LPR */
#define PCM_PMR_CPM__9_AM1_LPR                             (0x00000900)          /* AM1_LPR */
#define PCM_PMR_CPM__16                                    (0x00001000)          /* SL0_LDO */
#define PCM_PMR_CPM__16_SL0_LDO                            (0x00001000)          /* SL0_LDO */
#define PCM_PMR_CPM__17                                    (0x00001100)          /* SL1_LDO */
#define PCM_PMR_CPM__17_SL1_LDO                            (0x00001100)          /* SL1_LDO */
#define PCM_PMR_CPM__20                                    (0x00001400)          /* SL0_DCDC */
#define PCM_PMR_CPM__20_SL0_DCDC                           (0x00001400)          /* SL0_DCDC */
#define PCM_PMR_CPM__21                                    (0x00001500)          /* SL1_DCDC */
#define PCM_PMR_CPM__21_SL1_DCDC                           (0x00001500)          /* SL1_DCDC */
#define PCM_PMR_CPM__24                                    (0x00001800)          /* SL0_LPR */
#define PCM_PMR_CPM__24_SL0_LPR                            (0x00001800)          /* SL0_LPR */
#define PCM_PMR_CPM__25                                    (0x00001900)          /* SL1_LPR */
#define PCM_PMR_CPM__25_SL1_LPR                            (0x00001900)          /* SL1_LPR */
#define PCM_PMR_CPM__32                                    (0x00002000)          /* DSL */
#define PCM_PMR_CPM__32_DSL                                (0x00002000)          /* DSL */
#define PCM_PMR_CPM__63                                    (0x00003f00)          /* AM_DEFAULT */
#define PCM_PMR_CPM__63_AM_DEFAULT                         (0x00003f00)          /* AM_DEFAULT */
#define PCM_PMR_RESERVED_1__M                              (0x0000c000)          /* Reserved. Always read as 0. */
#define PCM_PMR_KEY__M                                     (0xffff0000)          /* PCM key. Must write proper key to allow write access to PMR. Any incorrect key  */
                                                                                 /* will not allow PMR to be written and power request will not be generated. Write  */
                                                                                 /* PCMKEY = 695A_xxxxh to unlock for write access. Always reads back A596h. This is */
                                                                                 /* a word accessible register. Any other type of access (half-word or byte access) */
                                                                                 /* will be ignored by PCM. */

/* PCM_CTL Control Bits */
#define PCM_CTL_LOCKSD                                     (0x00000001)          /* Lock shutdown. This bit once set can only be cleared by the user or by a power  */
                                                                                 /* cycle. This bit is automatically set upon SD0/SDC entry. User cannot set this  */
                                                                                 /* bit. In the write mode this bit should always be written with a value of 0 to  */
                                                                                 /* clear it. Writing a value of 1 has no effect on the system and these writes are  */
                                                                                 /* ignored. */
#define PCM_CTL_LOCKRTC                                    (0x00000002)          /* Lock RTC. This bit once set can only be cleared by the user or by a power cycle. */
                                                                                 /* This bit is automatically set upon SD0 entry. User cannot set this bit. In the  */
                                                                                 /* write mode this bit should always be written with a value of 0 to clear it. */
                                                                                 /* Writing a value of 1 has no effect on the system and these writes are ignored. */
#define PCM_CTL_FORCE_DSL_SD_ENTRY                         (0x00000004)          /* Bit selection for the application to determine whether the entry into DSL/SD0/ */
                                                                                 /* SDC should be forced even if there are active system clocks running which do not */
                                                                                 /* meet the DSL/SD0/SDC criteria. This bit by itself does not cause a DSL/SD0/SDC  */
                                                                                 /* by itself, but is a pre-requisite before the CPU executes WFI. 0b = PCM aborts  */
                                                                                 /* DSL/SD0/SDC mode transition. If on DSL/SD0/SDC entry condition, system clocks  */
                                                                                 /* requests are detected from modules other than RTC and WDT (If sourced from the  */
                                                                                 /* BCLK tree, If RTC/WDT are sourcing from any other clock tree they are also  */
                                                                                 /* considered to block DSL/SD0/SDC entry). PCM generates the SM_INVALID_CLK flag on */
                                                                                 /* abort to DSL/SD0/SDC entry.  1b = PCM enters DSL/SD0/SDC after shuting off the  */
                                                                                 /* clocks forcefully. Application needs to ensure RTC and WDT are clocked using  */
                                                                                 /* BCLK tree to keep these modules alive in DSL/SD0. In SDC all clocks are  */
                                                                                 /* forcefully shutoff and the Vcore is turned off. */
#define PCM_CTL_RESERVED_2__M                              (0x000000f8)          /* Reserved. Reads back 0. */
#define PCM_CTL_PMR_BUSY                                   (0x00000100)          /* Power mode request busy flag. This flag is set while a power change request is  */
                                                                                 /* being processed. The flag is cleared when the request has completed. Writes to  */
                                                                                 /* the PCM_PMR_REG or Clock System registers are ignored while PMR_BUSY = 1. Reads  */
                                                                                 /* to the PMR or CS registers are possible while PMR_BUSY = 1.  */
#define PCM_CTL_RESERVED_1__M                              (0x0000fe00)          /* Reserved. Reads back 0. */
#define PCM_CTL_KEY__M                                     (0xffff0000)          /* PCM key. Must write proper key to allow write access to PCMCTL . Any incorrect  */
                                                                                 /* key will not allow PCMCTL to be written and the write will be ignored. Write  */
                                                                                 /* PCMKEY = 695A_xxxxh to unlock for write access. Always reads back A596h. This is */
                                                                                 /* a word accessible register. Any other type of access (half-word/byte access)  */
                                                                                 /* will be ignored by PCM. */

/* PCM_INTEN Control Bits */
#define PCM_INTEN_EN_SM_INVALID_TR                         (0x00000001)          /* Enable sleep mode invalid transition interrupt. Setting this bit enables an  */
                                                                                 /* interrupt/NMI on an invalid transition from AM to DSL (SDx all transitions are  */
                                                                                 /* allowed).    */
#define PCM_INTEN_EN_SM_INVALID_CLK                        (0x00000002)          /* Enable sleep mode invalid clock interrupt. Setting this bit enables an interrupt */
                                                                                 /* /NMI on an invalid clock setting during a DSL/SD0/SDC transition from an active  */
                                                                                 /* mode when FORCE_DSL_SD_ENTRY = 0. This bit has not effect when  */
                                                                                 /* FORCE_DSL_SD_ENTRY = 1.   */
#define PCM_INTEN_EN_AM_INVALID_TR                         (0x00000004)          /* Enable active mode invalid transition. Setting this bit enables an interrupt/NMI */
                                                                                 /* on an invalid transition setting during an active power mode request.    */
#define PCM_INTEN_RESERVED_2__M                            (0x00000038)          /* Reserved. Reads back 0. */
#define PCM_INTEN_EN_DCDC_ERROR                            (0x00000040)          /* Enable DC to DC error. Setting this bit enables an interrupt/NMI when DC to DC  */
                                                                                 /* operation cannot be achieved or maintained.    */
#define PCM_INTEN_RESERVED_1__M                            (0xffffff80)          /* Reserved. Reads back 0. */

/* PCM_INTFLAG Control Bits */
#define PCM_INTFLAG_SM_INVALID_TR                          (0x00000001)          /* Sleep mode invalid transition flag. This flag is set if the requested AM to DSL  */
                                                                                 /* transition is invalid. Flag will remain set until cleared by software. */
#define PCM_INTFLAG_SM_INVALID_CLK                         (0x00000002)          /* Sleep mode invalid clock flag. This flag is set if the deep sleep mode request  */
                                                                                 /* is invalid due to a clock request active before the DSL/SD0/SDC entry while the  */
                                                                                 /* FORCE_DSL_SD_ENTRY = 0. Flag will remain set until cleared by software. */
#define PCM_INTFLAG_AM_INVALID_TR                          (0x00000004)          /* Active mode invalid transition flag. This flag is set if the active mode request */
                                                                                 /* is an invalid transition. Flag will remain set until cleared by software. */
#define PCM_INTFLAG_RESERVED_2__M                          (0x00000038)          /* Reserved. Reads back 0. */
#define PCM_INTFLAG_DCDC_ERROR                             (0x00000040)          /* DC to DC error flag. This flag is set if DC to DC operation cannot be achieved  */
                                                                                 /* or maintained. Flag will remain set until cleared by software. */
#define PCM_INTFLAG_RESERVED_1__M                          (0xffffff80)          /* Reserved. Reads back 0. */

/* PCM_INTCLR Control Bits */
#define PCM_INTCLR_CLR_SM_INVALID_TR                       (0x00000001)          /* Clear sleep mode invalid transition flag. */
#define PCM_INTCLR_CLR_SM_INVALID_CLK                      (0x00000002)          /* Clear sleep mode invalid clock flag. */
#define PCM_INTCLR_CLR_AM_INVALID_TR                       (0x00000004)          /* Clear active mode invalid transition flag. */
#define PCM_INTCLR_RESERVED_2__M                           (0x00000038)          /* Reserved. Write 1 only. Reads back 0. */
#define PCM_INTCLR_CLR_DCDC_ERROR                          (0x00000040)          /* Clear DC to DC error flag. */
#define PCM_INTCLR_RESERVED_1__M                           (0xffffff80)          /* Reserved. Write 1 only. Reads back 0. */


//*****************************************************************************
// PD Registers
//*****************************************************************************
typedef struct {
  uint16_t IN; /* Port D Input */
  uint16_t OUT; /* Port D Output */
  uint16_t DIR; /* Port D Direction */
  uint16_t REN; /* Port D Resistor Enable */
  uint16_t DS; /* Port D Drive Strength */
  uint16_t SEL0; /* Port D Select 0 */
  uint16_t SEL1; /* Port D Select 1 */
  uint8_t RESERVED0[8];
  uint16_t SELC; /* Port D Complement Selection */
  uint16_t IES; /* Port D Interrupt Edge Select */
  uint16_t IE; /* Port D Interrupt Enable */
  uint16_t IFG; /* Port D Interrupt Flag */
} PD_Type;



//*****************************************************************************
// PE Registers
//*****************************************************************************
typedef struct {
  uint16_t IN; /* Port E Input */
  uint16_t OUT; /* Port E Output */
  uint16_t DIR; /* Port E Direction */
  uint16_t REN; /* Port E Resistor Enable */
  uint16_t DS; /* Port E Drive Strength */
  uint16_t SEL0; /* Port E Select 0 */
  uint16_t SEL1; /* Port E Select 1 */
  uint8_t RESERVED0[8];
  uint16_t SELC; /* Port E Complement Selection */
  uint16_t IES; /* Port E Interrupt Edge Select */
  uint16_t IE; /* Port E Interrupt Enable */
  uint16_t IFG; /* Port E Interrupt Flag */
} PE_Type;



//*****************************************************************************
// PJ Registers
//*****************************************************************************
typedef struct {
  uint16_t IN; /* Port J Input */
  uint16_t OUT; /* Port J Output */
  uint16_t DIR; /* Port J Direction */
  uint16_t REN; /* Port J Resistor Enable */
  uint8_t RESERVED0[1];
  uint8_t DS; /* Port J Drive Strength */
  uint16_t SEL0; /* Port J Select 0 */
  uint16_t SEL1; /* Port J Select 1 */
  uint8_t RESERVED1[8];
  uint16_t SELC; /* Port J Complement Select */
} PJ_Type;



//*****************************************************************************
// PMAP Registers
//*****************************************************************************
typedef struct {
  uint16_t KEYID; /* Port mapping key register */
  union {    /* PMAP_CTL Register */
    __IO uint16_t reg;
    struct { /* PMAP_CTL Control Bits */
      __I  uint16_t LOCKED              	:	1;  /*  Port mapping lock bit. Read only */
      __IO uint16_t RECFG               	:	1;  /*  Port mapping reconfiguration control bit */
      __I  uint16_t RESERVED_1          	:	14;  /* Reserved */

    } bit;
  } CTL;
  uint8_t RESERVED0[5];
  uint8_t P1MAP; /* Port mapping register P1.0 to Port mapping register P1.7 */
  uint8_t RESERVED1[7];
  uint8_t P2MAP; /* Port mapping register P2.0 to Port mapping register P2.7 */
  uint8_t RESERVED2[7];
  uint8_t P3MAP; /* Port mapping register P3.0 to Port mapping register P3.7 */
  uint8_t RESERVED3[7];
  uint8_t P4MAP; /* Port mapping register P4.0 to Port mapping register P4.7 */
  uint8_t RESERVED4[7];
  uint8_t P5MAP; /* Port mapping register P5.0 to Port mapping register P5.7 */
  uint8_t RESERVED5[7];
  uint8_t P6MAP; /* Port mapping register P6.0 to Port mapping register P6.7 */
  uint8_t RESERVED6[7];
  uint8_t P7MAP; /* Port mapping register P7.0 to Port mapping register P7.7 */
} PMAP_Type;


/* PMAP_CTL Control Bits */
#define PMAP_CTL_LOCKED                                    (0x0001)              /*  Port mapping lock bit. Read only */
#define PMAP_CTL_RECFG                                     (0x0002)              /*  Port mapping reconfiguration control bit */
#define PMAP_CTL_RESERVED_1__M                             (0xfffc)              /*  Reserved. Always reads as 0. */


//*****************************************************************************
// PSS Registers
//*****************************************************************************
typedef struct {
  union {    /* PSS_KEY Register */
    __IO uint32_t reg;
    struct { /* PSS_KEY Control Bits */
      __IO uint32_t KEY                 	:	16;  /* PSS control key. Always read as A596h. Must be written with 695Ah to unlock the  */
                                                /* PSS registers. Any other write value locks the PSS registers */
      __I  uint32_t RESERVED1           	:	16;  /* Reserved */

    } bit;
  } KEY;
  union {    /* PSS_SVSMCTL Register */
    __IO uint32_t reg;
    struct { /* PSS_SVSMCTL Control Bits */
      __IO uint32_t SVSMHOFF            	:	1;  /* SVSM high-side off */
      __IO uint32_t SVSMHLP             	:	1;  /* SVSM high-side low-power mode */
      __IO uint32_t SVSMHS              	:	1;  /* Supply supervisor or monitor selection for the high-side */
      __IO uint32_t SVSMHTH             	:	3;  /* SVSM high-side reset voltage level. If DVCC falls short of the SVSMH voltage  */
                                                /* level selected by SVSMHTH, a reset is triggered (if SVSMHOFF = 0 and SVSMHS = 0) */
                                                /* or interrupt is triggered (if SVSMHOFF = 0 and SVSMHS = 1). The voltage levels  */
                                                /* are defined in the device-specific data sheet. */
      __IO uint32_t SVMHOE              	:	1;  /* SVM high-side output enable */
      __IO uint32_t SVMHOUTPOLAL        	:	1;  /* SVMHOUT pin polarity active low. */
      __IO uint32_t SVSLOFF             	:	1;  /* SVS low-side off */
      __IO uint32_t SVSLLP              	:	1;  /* SVS low-side low-power mode */
      __IO uint32_t DCDC_FORCE          	:	1;  /* Disables automatic supply voltage detection and possible denial of DC/DC power  */
                                                /* mode if supply voltage is too low. */
      __I  uint32_t RESERVED1           	:	21;  /* Reserved */

    } bit;
  } SVSMCTL;
  union {    /* PSS_DOCMCTL Register */
    __IO uint32_t reg;
    struct { /* PSS_DOCMCTL Control Bits */
      __IO uint32_t DOCMON              	:	1;  /* Turns on/off the DOCM module */
      __IO uint32_t DOCMSAMP            	:	1;  /* DOCM sample current */
      __I  uint32_t RESERVED2           	:	1;  /* Reserved */
      __IO uint32_t DOCMCM              	:	6;  /* Controls current mirrors in DOCM for conversion */
      __I  uint32_t RESERVED1           	:	23;  /* Reserved */

    } bit;
  } DOCMCTL;
  union {    /* PSS_DOCMOUTR Register */
    __IO uint32_t reg;
    struct { /* PSS_DOCMOUTR Control Bits */
      __I  uint32_t DOCMOUT             	:	6;  /* DOCM comparator output */
      __I  uint32_t RESERVED1           	:	26;  /* Reserved */

    } bit;
  } DOCMOUTR;
  uint8_t RESERVED0[36];
  union {    /* PSS_IE Register */
    __IO uint32_t reg;
    struct { /* PSS_IE Control Bits */
      __IO uint32_t SVSLVLRIE           	:	1;  /* SVS low-side voltage level reached interrupt enabled. */
      __IO uint32_t SVSMHIE             	:	1;  /* High-side SVSM interrupt enable, when set as a monitor SVSMH = 1. */
      __I  uint32_t RESERVED1           	:	30;  /* Reserved */

    } bit;
  } IE;
  union {    /* PSS_IFG Register */
    __IO uint32_t reg;
    struct { /* PSS_IFG Control Bits */
      __IO uint32_t SVSLVLRIFG          	:	1;  /* SVS low-side voltage level reached interrupt flag. The bit is cleared by  */
                                                /* software. NOTE: After power up, the reset value depends on the power sequence. */
      __IO uint32_t SVSMHIFG            	:	1;  /* High-side SVSM interrupt flag. SVSMH = 0 (supervisor mode): The SVSMHIFG  */
                                                /* interrupt flag is not active. SVSMH = 1 (monitor mode): The SVSMHIFG interrupt  */
                                                /* flag is set if DVCC drops below the SVSMH power-down level and a system NMI is  */
                                                /* generated. The bit is cleared by software. */
      __I  uint32_t RESERVED1           	:	30;  /* Reserved */

    } bit;
  } IFG;
  union {    /* PSS_CLRIFG Register */
    __IO uint32_t reg;
    struct { /* PSS_CLRIFG Control Bits */
      __O  uint32_t CLRSVSLVLRIFG       	:	1;  /* SVSLVLR clear interrupt flag */
      __O  uint32_t CLRSVSMHIFG         	:	1;  /* SVSMH clear interrupt flag */
      __I  uint32_t RESERVED1           	:	30;  /* Reserved */

    } bit;
  } CLRIFG;
  uint8_t RESERVED1[704];
  uint32_t REFTRIM1; /* Configuration Register 1 for PSS Refsys and SVSL Threshold Levels */
  uint32_t REFTRIM2; /* Configuration Register 2 for PSS Refsys */
  uint32_t SVSCNFG; /* Configuration Register for SVSMH and SVSL */
  uint32_t FLSHCNFG0; /* Configuration Register 1 for PSS Flash */
  union {    /* PSS_FLSHCNFG1 Register */
    __IO uint32_t reg;
    struct { /* PSS_FLSHCNFG1 Control Bits */
      __IO uint32_t VRFFLCNFG           	:	4;  /* [3] Disables Startup Comparator [2] Forces High performance Mode [1] Forces Low  */
                                                /* Power Mode [0] Masks Bias Ready Output to Healthy */
      __IO uint32_t IRFFLCNFG           	:	3;  /* [6] Disables Startup Comparator Loopback [5] Disables Startup Comparator [4]  */
                                                /* Masks Bias Ready Output to Healthy */
      __IO uint32_t IREFFLTREN          	:	1;  /* Routes IREF current to DFT mux */
      __IO uint32_t FLLDOPRG            	:	4;  /* Flash LDO programmation */
      __IO uint32_t RESERVED1           	:	4;  /* Reserved */
      __IO uint32_t FLLDOCNFG           	:	16;  /* Configures Flash LDO */

    } bit;
  } FLSHCNFG1;
  uint32_t MISCCNFG; /* Configuration Register or Core LDO and certain DFT features */
  uint32_t VCOREPROG; /* Allowed VCORE Levels */
  uint32_t OSDLCNFG; /* Configuration Register for Oscillators, Delays and DCDC Supply Monitor (MNK)  */
                                                                                 /* Comparator */
  uint32_t DCDCCNFG1; /* Configuration Register 1 for DCDC */
  uint32_t DCDCCNFG2; /* Configuration Register 2 for DCDC */
  uint32_t DCDCCNFG3; /* Configuration Register 3 for DCDC */
  uint32_t SHCTR; /* Sample, Standby Duration Count Values */
  uint32_t RSTFLG; /* Reset Flags */
  uint8_t RESERVED2[8];
  uint32_t OPTEOP; /* Optimization bits and End of Programming Indicator */
  uint8_t RESERVED3[960];
  uint32_t SVSMHTH01; /* SVSMH Threshold Levels 0 and 1 */
  uint32_t SVSMHTH23; /* SVSMH Threshold Levels 2 and 3 */
  uint32_t SVSMHTH45; /* SVSMH Threshold Levels 4 and 5 */
  uint32_t SVSMHTH67; /* SVSMH Threshold Levels 6 and 7 */
  union {    /* PSS_DFTCNFG Register */
    __IO uint32_t reg;
    struct { /* PSS_DFTCNFG Control Bits */
      __I  uint32_t RESERVED1           	:	16;  /* Reserved */
      __IO uint32_t LOCAL_TP0_MUX_SEL   	:	4;  /* 4-bit select to bring out 1 of the 16 signals from group 0 on test port 0(TP3) */
      __IO uint32_t LOCAL_TP1_MUX_SEL   	:	4;  /* 4-bit select to bring out 1 of the 16 signals from group 1 on test port 1 (TP3) */
      __IO uint32_t LOCAL_TP2_MUX_SEL   	:	4;  /* 4-bit select to bring out 1 of the 16 signals from group 3 on test port 2 (TP3) */
      __IO uint32_t LOCAL_TP3_MUX_SEL   	:	4;  /* 4-bit select to bring out 1 of the 16 signals from group 3 on test port 3 (TP3) */

    } bit;
  } DFTCNFG;
  uint8_t RESERVED4[12];
  union {    /* PSS_DFTCTRL1 Register */
    __IO uint32_t reg;
    struct { /* PSS_DFTCTRL1 Control Bits */
      __IO uint32_t DIS_DFT_RHS         	:	1;  /* Disable DFT controls for reset_high_side and allows func logic to control ( */
                                                /* disables any DFT_mode or HVDET override settings) */
      __IO uint32_t DIS_DFT_RLS         	:	1;  /* Disable DFT controls for reset_low_side and allows func logic to control ( */
                                                /* disables any DFT_mode or HVDET override settings) */
      __IO uint32_t DIS_DFT_FL_LDO      	:	1;  /* Disable DFT controls for flash ldo and allows func logic to control (disables  */
                                                /* any DFT_mode or HVDET override settings) */
      __IO uint32_t DIS_DFT_FL_VREF     	:	1;  /* Disable DFT controls for flash vref and allows func logic to control (disables  */
                                                /* any DFT_mode or HVDET override settings) */
      __IO uint32_t DIS_DFT_FL_IREF     	:	1;  /* Disable DFT controls for flash iref and allows func logic to control (disables  */
                                                /* any DFT_mode or HVDET override settings) */
      __IO uint32_t RESERVED1           	:	3;  /* Reserved */
      __IO uint32_t SHORT_DLY_REQ       	:	1;  /* Short Delay Request for reference settling */
      __IO uint32_t LONG_DLY_REQ        	:	1;  /* Long Delay Request for reference settling */
      __IO uint32_t LPM_SLOW_OSC_EN     	:	1;  /* Enables SLOW_CLOCK - the slower speed LPM oscillator in DFT mode (controls  */
                                                /* REFSYS Sand H for LP performance modes) */
      __IO uint32_t LPM_FAST_OSC_EN     	:	1;  /* Enables FAST_CLOCK - the higher speed LPM oscillator in DFT mode (controls  */
                                                /* REFSYS SandH for LP performance modes) */
      __IO uint32_t FLASH_VREF_DIS      	:	1;  /* Disables Flash VREF in DFT mode. */
      __IO uint32_t FLASH_IREF_DIS      	:	1;  /* Disables Flash IREF in DFT mode. */
      __IO uint32_t FLASH_LDO_TRISTATE_EN	:	1;  /* Tristates (HiZ) the Flash LDO in DFT mode */
      __IO uint32_t FLASH_LPM_LDO_EN    	:	1;  /* Forces Flash LDO in LPM in DFT mode */
      __IO uint32_t FLASH_LDO_DIS       	:	1;  /* Disables Flash LDO in DFT mode */
      __IO uint32_t REF_STDBY_EN        	:	1;  /* Standby control in SandH mode */
      __IO uint32_t REF_SMPL_DIS        	:	1;  /* Sample control in SandH mode */
      __IO uint32_t REF_LPM_EN          	:	1;  /* Enables Reference in LPM mode (SandH mode) */
      __IO uint32_t DCDC_TRISTATE_DIS   	:	1;  /* Disables the tri-stating of the DCDC */
      __IO uint32_t DCDC_MNK_COMP_EN    	:	1;  /* Enables MNK comp in DFT mode */
      __IO uint32_t DCDC_EN             	:	1;  /* Enables DC-DC in DFT mode (by default disabled) */
      __IO uint32_t CORE_LPM_LDO_DIS    	:	1;  /* Disables Core LPM LDO in DFT mode */
      __IO uint32_t SVSL_LPM_EN         	:	1;  /* Enables SVSL in LPM in DFT mode */
      __IO uint32_t SVSL_DIS            	:	1;  /* Disables SVSL in DFT mode */
      __IO uint32_t CORE_LPM_LDO_NO_HIZ_EN	:	1;  /* Removes the tristating (HiZ) the CORE LPM LDO in DFT mode (default LPM LDO is  */
                                                /* tristated) */
      __IO uint32_t VCORE_LDO_DIS       	:	1;  /* Disables VCORE LDO in DFT Mode (If VCORE forced externally during DFT, please  */
                                                /* set this bit before entering PMM DFT mode] */
      __IO uint32_t SVSMH_LPM_EN        	:	1;  /* Enables SVSMH in LPM in DFT mode */
      __IO uint32_t SVSMH_DIS           	:	1;  /* Disables SVSMH in DFT mode */
      __IO uint32_t BG_DIS              	:	1;  /* Disables BG in DFT Mode */
      __IO uint32_t DFT_MODE            	:	1;  /* Puts PSS in DFT mode; Disables Reset due to SVSMH, SVSLite trip (disables high  */
                                                /* side and low side reset to inactive) */

    } bit;
  } DFTCTRL1;
  union {    /* PSS_DFTCTRL2 Register */
    __IO uint32_t reg;
    struct { /* PSS_DFTCTRL2 Control Bits */
      __I  uint32_t RESERVED2           	:	11;  /* Reserved */
      __IO uint32_t RESERVED1           	:	3;  /* Reserved */
      __IO uint32_t SVS_LP_OVRDE        	:	1;  /* Puts SVSL in Low Power Mode */
      __IO uint32_t SVSMH_LP_OVRDE      	:	1;  /* Puts SVSMH in Low Power Mode */
      __IO uint32_t BG_MOD_OVRDE        	:	1;  /* Puts PSS REFSYS in LPM */
      __IO uint32_t FLASH_VREF_BIAS_RDY_OVRDE	:	1;  /* Asserts flash vref bias ready */
      __IO uint32_t FLASH_IREF_BIAS_RDY_OVRDE	:	1;  /* Asserts flash iref bias ready */
      __IO uint32_t FLASH_LDO_PGOOD_OVRDE	:	1;  /* Asserts flash ldo pgood */
      __IO uint32_t FLASH_LDO_BIAS_RDY_OVRDE	:	1;  /* Asserts flash ldo bias ready */
      __IO uint32_t MNK_COMP_OK_OVRDE   	:	1;  /* Asserts MNK comp ok */
      __IO uint32_t SVSL_BIAS_RDY_OVRDE 	:	1;  /* Asserts SVSL bias ready */
      __IO uint32_t TRIM_DONE_OVRDE     	:	1;  /* Asserts PSS trim done */
      __IO uint32_t TRIM_SET_OVRDE      	:	1;  /* Asserts PSS trim set */
      __IO uint32_t DCDC_BIAS_RDY_OVRDE 	:	1;  /* Asserts DC/DC bias ready */
      __IO uint32_t LDO_LPM_BIAS_RDY_OVRDE	:	1;  /* Asserts core lpm ldo bias ready */
      __IO uint32_t VCORE_OK_OVRDE      	:	1;  /* Asserts vcore ok */
      __IO uint32_t LDO_BIAS_RDY_OVRDE  	:	1;  /* Asserts core ldo bias ready */
      __IO uint32_t VCC_OK_OVRDE        	:	1;  /* Asserts vcc ok */
      __IO uint32_t SVSMH_BIAS_RDY_OVRDE	:	1;  /* Asserts SVSMH bias ready */
      __IO uint32_t REF_OK_OVRDE        	:	1;  /* Asserts ref ok */

    } bit;
  } DFTCTRL2;
  union {    /* PSS_DFTATBBF Register */
    __IO uint32_t reg;
    struct { /* PSS_DFTATBBF Control Bits */
      __IO uint32_t ATBBFGEN            	:	1;  /* Set this bit to bring out sense lines from Analog IPs from the rest of the chip  */
                                                /* through the DFT Buffer. Needs top level port configuration and other DFT  */
                                                /* settings as well. */
      __IO uint32_t ATBBFDCDC           	:	1;  /* Set this bit to bring out sense lines from the DCDC through the DFT Buffer.  */
                                                /* Needs top level port configuration and DCDCCNFG1/2/3 register setting as well. */
      __IO uint32_t ATBBFFLSH           	:	1;  /* Set this bit to bring out sense lines from the Flash PSS through the DFT Buffer. */
                                                /* Needs top level port configuration and FLSHCNFG0/1 register setting as well. */
      __IO uint32_t BUFCALBF            	:	1;  /* Set this bit along with corresponding bit in DFTATBUBF register to short  */
                                                /* buffered and unbuffered path to calibrate DFT Buffer offset. Needs top level  */
                                                /* port configuration as well. */
      __IO uint32_t LDOINTSNSBF         	:	1;  /* Set this bit to bring out voltages internal to LDO through the DFT Buffer. Needs */
                                                /* top level port configuration and MISCCNFG register setting as well. */
      __IO uint32_t UNUSED              	:	10;  /* Unused Unused Unused. Unused. */
      __IO uint32_t BGINTSNSBF          	:	1;  /* Set this bit to bring out an internal bias node inside bandgap through the DFT  */
                                                /* Buffer. Needs top level port configuration as well. */
      __IO uint32_t PRERF0P6SNSBF       	:	1;  /* Set this bit to bring out the pre-S/H 0.6V reference voltage through the DFT  */
                                                /* Buffer. Needs top level port configuration as well. */
      __IO uint32_t PRERFVCSNSBF        	:	1;  /* Set this bit to bring out the pre-S/H VCORE reference voltage through the DFT  */
                                                /* Buffer. Needs top level port configuration as well. */
      __IO uint32_t BG1P2SNSBF          	:	1;  /* Set this bit to bring out the internal bandgap 1.2V reference voltage through  */
                                                /* the DFT Buffer. Needs top level port configuration as well. */
      __IO uint32_t BG0P6SNSBF          	:	1;  /* Set this bit to bring out the internal bandgap 0.6V reference voltage through  */
                                                /* the DFT Buffer. Needs top level port configuration as well. */
      __IO uint32_t RFVCSNSBF           	:	1;  /* Set this bit to bring out the VCORE reference voltage through the DFT Buffer.  */
                                                /* Needs top level port configuration as well. */
      __IO uint32_t RF0P6SNSBF          	:	1;  /* Set this bit to bring out the generic 0.6V reference voltage through the DFT  */
                                                /* Buffer. Needs top level port configuration as well. */

    } bit;
  } DFTATBBF;
  union {    /* PSS_DFTATBUBF Register */
    __IO uint32_t reg;
    struct { /* PSS_DFTATBUBF Control Bits */
      __IO uint32_t ATBUBFGEN           	:	1;  /* Set this bit to bring out sense lines from Analog IPs from the rest of the chip  */
                                                /* through the DFT unbuffered path. Needs top level port configuration and other  */
                                                /* DFT settings as well. */
      __IO uint32_t ATBUBFDCDC          	:	1;  /* Set this bit to bring out sense lines from the DCDC through the DFT unbuffered  */
                                                /* path. Needs top level port configuration and DCDCCNFG1/2/3 register setting as  */
                                                /* well. */
      __IO uint32_t ATBUBFFLSH          	:	1;  /* Set this bit to bring out sense lines from the Flash PSS through the DFT  */
                                                /* unbuffered path. Needs top level port configuration and FLSHCNFG0/1 register  */
                                                /* setting as well. */
      __IO uint32_t BUFCALUBF           	:	1;  /* Set this bit along with corresponding bit in DFTATBBF register to short buffered */
                                                /* and unbuffered path to calibrate DFT Buffer offset. Needs top level port  */
                                                /* configuration as well. */
      __IO uint32_t LDOINTSNSUBF        	:	1;  /* Set this bit to bring out voltages internal to LDO through the DFT unbuffered  */
                                                /* path. Needs top level port configuration and MISCCNFG register setting as well. */
      __IO uint32_t UNUSED              	:	4;  /* Unused Unused Unused. Unused. */
      __IO uint32_t SVSLVCCDIVSNS       	:	1;  /* Set this bit to bring out the SVSL resistor divider output through the DFT  */
                                                /* unbuffered path. Needs top level port configuration and MISCCNFG register  */
                                                /* setting as well. */
      __IO uint32_t SVSLIPFRC           	:	1;  /* Set this bit to force the SVSL Input Voltage (instead of VCORE) to test SVSL  */
                                                /* through the DFT unbuffered path. Needs top level port configuration as well.. */
      __IO uint32_t SVSMHVCCDIVSNS      	:	1;  /* Set this bit to bring out the SVSMH resistor divider output through the DFT  */
                                                /* unbuffered path. Needs top level port configuration and MISCCNFG register  */
                                                /* setting as well. */
      __IO uint32_t REFGNDSNS           	:	1;  /* Set this bit to bring out the REFSYS Ground Sense line through the DFT  */
                                                /* unbuffered path. Needs top level port configuration as well. */
      __IO uint32_t SVSMHIPFRC          	:	1;  /* Set this bit to force the SVSMH Input Voltage (instead of VCC / VDDS) to test  */
                                                /* SVSL through the DFT unbuffered path. Needs top level port configuration as well */
                                                /* .. */
      __IO uint32_t REFCTFRC            	:	1;  /* Set this bit to force an external current of 100nA through the DFT unbuffered  */
                                                /* path. Needs top level port configuration as well. */
      __IO uint32_t BGINTSNSUBF         	:	1;  /* Set this bit to bring out an internal bias node inside bandgap through the DFT  */
                                                /* unbuffered path. Needs top level port configuration as well. */
      __IO uint32_t PRERF0P6SNSUBF      	:	1;  /* Set this bit to bring out the pre-S/H 0.6V reference voltage through the DFT  */
                                                /* unbuffered path. Needs top level port configuration as well. */
      __IO uint32_t PRERFVCSNSUBF       	:	1;  /* Set this bit to bring out the pre-S/H VCORE reference voltage through the DFT  */
                                                /* unbuffered path. Needs top level port configuration as well. */
      __IO uint32_t BG1P2SNSUBF         	:	1;  /* Set this bit to bring out the internal bandgap 1.2V reference voltage through  */
                                                /* the DFT unbuffered path. Needs top level port configuration as well. */
      __IO uint32_t BG0P6SNSUBF         	:	1;  /* Set this bit to bring out the internal bandgap 0.6V reference voltage through  */
                                                /* the DFT unbuffered path. Needs top level port configuration as well. */
      __IO uint32_t DBGLDOCT0           	:	1;  /* Set this bit to bring out certain currents internal to VCORE LDO through the DFT */
                                                /* unbuffered path. Needs top level port configuration as well. */
      __IO uint32_t DBGLDOCT1           	:	1;  /* Set this bit to bring out certain currents internal to VCORE LDO through the DFT */
                                                /* unbuffered path. Needs top level port configuration as well. */
      __IO uint32_t DBGLDOCT2           	:	1;  /* Set this bit to bring out certain currents internal to VCORE LDO through the DFT */
                                                /* unbuffered path. Needs top level port configuration as well. */

    } bit;
  } DFTATBUBF;
} PSS_Type;


/* PSS_KEY Control Bits */
#define PSS_KEY_KEY__M                                     (0x0000ffff)          /* PSS control key. Always read as A596h. Must be written with 695Ah to unlock the  */
                                                                                 /* PSS registers. Any other write value locks the PSS registers */
#define PSS_KEY_RESERVED1__M                               (0xffff0000)          /*  */

/* PSS_SVSMCTL Control Bits */
#define PSS_SVSMCTL_SVSMHOFF                               (0x00000001)          /* SVSM high-side off */
#define PSS_SVSMCTL_SVSMHLP                                (0x00000002)          /* SVSM high-side low-power mode */
#define PSS_SVSMCTL_SVSMHS                                 (0x00000004)          /* Supply supervisor or monitor selection for the high-side */
#define PSS_SVSMCTL_SVSMHTH__M                             (0x00000038)          /* SVSM high-side reset voltage level. If DVCC falls short of the SVSMH voltage  */
                                                                                 /* level selected by SVSMHTH, a reset is triggered (if SVSMHOFF = 0 and SVSMHS = 0) */
                                                                                 /* or interrupt is triggered (if SVSMHOFF = 0 and SVSMHS = 1). The voltage levels  */
                                                                                 /* are defined in the device-specific data sheet. */
#define PSS_SVSMCTL_SVMHOE                                 (0x00000040)          /* SVM high-side output enable */
#define PSS_SVSMCTL_SVMHOUTPOLAL                           (0x00000080)          /* SVMHOUT pin polarity active low. */
#define PSS_SVSMCTL_SVSLOFF                                (0x00000100)          /* SVS low-side off */
#define PSS_SVSMCTL_SVSLLP                                 (0x00000200)          /* SVS low-side low-power mode */
#define PSS_SVSMCTL_DCDC_FORCE                             (0x00000400)          /* Disables automatic supply voltage detection and possible denial of DC/DC power  */
                                                                                 /* mode if supply voltage is too low. */
#define PSS_SVSMCTL_RESERVED1__M                           (0xfffff800)          /*  */

/* PSS_DOCMCTL Control Bits */
#define PSS_DOCMCTL_DOCMON                                 (0x00000001)          /* Turns on/off the DOCM module */
#define PSS_DOCMCTL_DOCMSAMP                               (0x00000002)          /* DOCM sample current */
#define PSS_DOCMCTL_RESERVED2                              (0x00000004)          /*  */
#define PSS_DOCMCTL_DOCMCM__M                              (0x000001f8)          /* Controls current mirrors in DOCM for conversion */
#define PSS_DOCMCTL_DOCMCM__0                              (0x00000000)          /* TBD */
#define PSS_DOCMCTL_DOCMCM__0_TBD                          (0x00000000)          /* TBD */
#define PSS_DOCMCTL_DOCMCM__1                              (0x00000008)          /* TBD */
#define PSS_DOCMCTL_DOCMCM__1_TBD                          (0x00000008)          /* TBD */
#define PSS_DOCMCTL_RESERVED1__M                           (0xfffffe00)          /*  */

/* PSS_DOCMOUTR Control Bits */
#define PSS_DOCMOUTR_DOCMOUT__M                            (0x0000003f)          /* DOCM comparator output */
#define PSS_DOCMOUTR_RESERVED1__M                          (0xffffffc0)          /*  */

/* PSS_IE Control Bits */
#define PSS_IE_SVSLVLRIE                                   (0x00000001)          /* SVS low-side voltage level reached interrupt enabled. */
#define PSS_IE_SVSMHIE                                     (0x00000002)          /* High-side SVSM interrupt enable, when set as a monitor SVSMH = 1. */
#define PSS_IE_RESERVED1__M                                (0xfffffffc)          /*  */

/* PSS_IFG Control Bits */
#define PSS_IFG_SVSLVLRIFG                                 (0x00000001)          /* SVS low-side voltage level reached interrupt flag. The bit is cleared by  */
                                                                                 /* software. NOTE: After power up, the reset value depends on the power sequence. */
#define PSS_IFG_SVSMHIFG                                   (0x00000002)          /* High-side SVSM interrupt flag. SVSMH = 0 (supervisor mode): The SVSMHIFG  */
                                                                                 /* interrupt flag is not active. SVSMH = 1 (monitor mode): The SVSMHIFG interrupt  */
                                                                                 /* flag is set if DVCC drops below the SVSMH power-down level and a system NMI is  */
                                                                                 /* generated. The bit is cleared by software. */
#define PSS_IFG_RESERVED1__M                               (0xfffffffc)          /*  */

/* PSS_CLRIFG Control Bits */
#define PSS_CLRIFG_CLRSVSLVLRIFG                           (0x00000001)          /* SVSLVLR clear interrupt flag */
#define PSS_CLRIFG_CLRSVSMHIFG                             (0x00000002)          /* SVSMH clear interrupt flag */
#define PSS_CLRIFG_RESERVED1__M                            (0xfffffffc)          /*  */

/* PSS_FLSHCNFG1 Control Bits */
#define PSS_FLSHCNFG1_VRFFLCNFG__M                         (0x0000000f)          /* [3] Disables Startup Comparator [2] Forces High performance Mode [1] Forces Low  */
                                                                                 /* Power Mode [0] Masks Bias Ready Output to Healthy */
#define PSS_FLSHCNFG1_IRFFLCNFG__M                         (0x00000070)          /* [6] Disables Startup Comparator Loopback [5] Disables Startup Comparator [4]  */
                                                                                 /* Masks Bias Ready Output to Healthy */
#define PSS_FLSHCNFG1_IREFFLTREN                           (0x00000080)          /* Routes IREF current to DFT mux */
#define PSS_FLSHCNFG1_FLLDOPRG__M                          (0x00000f00)          /* Flash LDO programmation */
#define PSS_FLSHCNFG1_RESERVED1__M                         (0x0000f000)          /*  */
#define PSS_FLSHCNFG1_FLLDOCNFG__M                         (0xffff0000)          /* Configures Flash LDO */

/* PSS_DFTCNFG Control Bits */
#define PSS_DFTCNFG_RESERVED1__M                           (0x0000ffff)          /*  */
#define PSS_DFTCNFG_LOCAL_TP0_MUX_SEL__M                   (0x000f0000)          /* 4-bit select to bring out 1 of the 16 signals from group 0 on test port 0(TP3) */
#define PSS_DFTCNFG_LOCAL_TP1_MUX_SEL__M                   (0x00f00000)          /* 4-bit select to bring out 1 of the 16 signals from group 1 on test port 1 (TP3) */
#define PSS_DFTCNFG_LOCAL_TP2_MUX_SEL__M                   (0x0f000000)          /* 4-bit select to bring out 1 of the 16 signals from group 3 on test port 2 (TP3) */
#define PSS_DFTCNFG_LOCAL_TP3_MUX_SEL__M                   (0xf0000000)          /* 4-bit select to bring out 1 of the 16 signals from group 3 on test port 3 (TP3) */

/* PSS_DFTCTRL1 Control Bits */
#define PSS_DFTCTRL1_DIS_DFT_RHS                           (0x00000001)          /* Disable DFT controls for reset_high_side and allows func logic to control ( */
                                                                                 /* disables any DFT_mode or HVDET override settings) */
#define PSS_DFTCTRL1_DIS_DFT_RLS                           (0x00000002)          /* Disable DFT controls for reset_low_side and allows func logic to control ( */
                                                                                 /* disables any DFT_mode or HVDET override settings) */
#define PSS_DFTCTRL1_DIS_DFT_FL_LDO                        (0x00000004)          /* Disable DFT controls for flash ldo and allows func logic to control (disables  */
                                                                                 /* any DFT_mode or HVDET override settings) */
#define PSS_DFTCTRL1_DIS_DFT_FL_VREF                       (0x00000008)          /* Disable DFT controls for flash vref and allows func logic to control (disables  */
                                                                                 /* any DFT_mode or HVDET override settings) */
#define PSS_DFTCTRL1_DIS_DFT_FL_IREF                       (0x00000010)          /* Disable DFT controls for flash iref and allows func logic to control (disables  */
                                                                                 /* any DFT_mode or HVDET override settings) */
#define PSS_DFTCTRL1_RESERVED1__M                          (0x000000e0)          /*  */
#define PSS_DFTCTRL1_SHORT_DLY_REQ                         (0x00000100)          /* Short Delay Request for reference settling */
#define PSS_DFTCTRL1_LONG_DLY_REQ                          (0x00000200)          /* Long Delay Request for reference settling */
#define PSS_DFTCTRL1_LPM_SLOW_OSC_EN                       (0x00000400)          /* Enables SLOW_CLOCK - the slower speed LPM oscillator in DFT mode (controls  */
                                                                                 /* REFSYS Sand H for LP performance modes) */
#define PSS_DFTCTRL1_LPM_FAST_OSC_EN                       (0x00000800)          /* Enables FAST_CLOCK - the higher speed LPM oscillator in DFT mode (controls  */
                                                                                 /* REFSYS SandH for LP performance modes) */
#define PSS_DFTCTRL1_FLASH_VREF_DIS                        (0x00001000)          /* Disables Flash VREF in DFT mode. */
#define PSS_DFTCTRL1_FLASH_IREF_DIS                        (0x00002000)          /* Disables Flash IREF in DFT mode. */
#define PSS_DFTCTRL1_FLASH_LDO_TRISTATE_EN                 (0x00004000)          /* Tristates (HiZ) the Flash LDO in DFT mode */
#define PSS_DFTCTRL1_FLASH_LPM_LDO_EN                      (0x00008000)          /* Forces Flash LDO in LPM in DFT mode */
#define PSS_DFTCTRL1_FLASH_LDO_DIS                         (0x00010000)          /* Disables Flash LDO in DFT mode */
#define PSS_DFTCTRL1_REF_STDBY_EN                          (0x00020000)          /* Standby control in SandH mode */
#define PSS_DFTCTRL1_REF_SMPL_DIS                          (0x00040000)          /* Sample control in SandH mode */
#define PSS_DFTCTRL1_REF_LPM_EN                            (0x00080000)          /* Enables Reference in LPM mode (SandH mode) */
#define PSS_DFTCTRL1_DCDC_TRISTATE_DIS                     (0x00100000)          /* Disables the tri-stating of the DCDC */
#define PSS_DFTCTRL1_DCDC_MNK_COMP_EN                      (0x00200000)          /* Enables MNK comp in DFT mode */
#define PSS_DFTCTRL1_DCDC_EN                               (0x00400000)          /* Enables DC-DC in DFT mode (by default disabled) */
#define PSS_DFTCTRL1_CORE_LPM_LDO_DIS                      (0x00800000)          /* Disables Core LPM LDO in DFT mode */
#define PSS_DFTCTRL1_SVSL_LPM_EN                           (0x01000000)          /* Enables SVSL in LPM in DFT mode */
#define PSS_DFTCTRL1_SVSL_DIS                              (0x02000000)          /* Disables SVSL in DFT mode */
#define PSS_DFTCTRL1_CORE_LPM_LDO_NO_HIZ_EN                (0x04000000)          /* Removes the tristating (HiZ) the CORE LPM LDO in DFT mode (default LPM LDO is  */
                                                                                 /* tristated) */
#define PSS_DFTCTRL1_VCORE_LDO_DIS                         (0x08000000)          /* Disables VCORE LDO in DFT Mode (If VCORE forced externally during DFT, please  */
                                                                                 /* set this bit before entering PMM DFT mode] */
#define PSS_DFTCTRL1_SVSMH_LPM_EN                          (0x10000000)          /* Enables SVSMH in LPM in DFT mode */
#define PSS_DFTCTRL1_SVSMH_DIS                             (0x20000000)          /* Disables SVSMH in DFT mode */
#define PSS_DFTCTRL1_BG_DIS                                (0x40000000)          /* Disables BG in DFT Mode */
#define PSS_DFTCTRL1_DFT_MODE                              (0x80000000)          /* Puts PSS in DFT mode; Disables Reset due to SVSMH, SVSLite trip (disables high  */
                                                                                 /* side and low side reset to inactive) */

/* PSS_DFTCTRL2 Control Bits */
#define PSS_DFTCTRL2_RESERVED2__M                          (0x000007ff)          /*  */
#define PSS_DFTCTRL2_RESERVED1__M                          (0x00003800)          /*  */
#define PSS_DFTCTRL2_SVS_LP_OVRDE                          (0x00004000)          /* Puts SVSL in Low Power Mode */
#define PSS_DFTCTRL2_SVSMH_LP_OVRDE                        (0x00008000)          /* Puts SVSMH in Low Power Mode */
#define PSS_DFTCTRL2_BG_MOD_OVRDE                          (0x00010000)          /* Puts PSS REFSYS in LPM */
#define PSS_DFTCTRL2_FLASH_VREF_BIAS_RDY_OVRDE             (0x00020000)          /* Asserts flash vref bias ready */
#define PSS_DFTCTRL2_FLASH_IREF_BIAS_RDY_OVRDE             (0x00040000)          /* Asserts flash iref bias ready */
#define PSS_DFTCTRL2_FLASH_LDO_PGOOD_OVRDE                 (0x00080000)          /* Asserts flash ldo pgood */
#define PSS_DFTCTRL2_FLASH_LDO_BIAS_RDY_OVRDE              (0x00100000)          /* Asserts flash ldo bias ready */
#define PSS_DFTCTRL2_MNK_COMP_OK_OVRDE                     (0x00200000)          /* Asserts MNK comp ok */
#define PSS_DFTCTRL2_SVSL_BIAS_RDY_OVRDE                   (0x00400000)          /* Asserts SVSL bias ready */
#define PSS_DFTCTRL2_TRIM_DONE_OVRDE                       (0x00800000)          /* Asserts PSS trim done */
#define PSS_DFTCTRL2_TRIM_SET_OVRDE                        (0x01000000)          /* Asserts PSS trim set */
#define PSS_DFTCTRL2_DCDC_BIAS_RDY_OVRDE                   (0x02000000)          /* Asserts DC/DC bias ready */
#define PSS_DFTCTRL2_LDO_LPM_BIAS_RDY_OVRDE                (0x04000000)          /* Asserts core lpm ldo bias ready */
#define PSS_DFTCTRL2_VCORE_OK_OVRDE                        (0x08000000)          /* Asserts vcore ok */
#define PSS_DFTCTRL2_LDO_BIAS_RDY_OVRDE                    (0x10000000)          /* Asserts core ldo bias ready */
#define PSS_DFTCTRL2_VCC_OK_OVRDE                          (0x20000000)          /* Asserts vcc ok */
#define PSS_DFTCTRL2_SVSMH_BIAS_RDY_OVRDE                  (0x40000000)          /* Asserts SVSMH bias ready */
#define PSS_DFTCTRL2_REF_OK_OVRDE                          (0x80000000)          /* Asserts ref ok */

/* PSS_DFTATBBF Control Bits */
#define PSS_DFTATBBF_ATBBFGEN                              (0x00000001)          /* Set this bit to bring out sense lines from Analog IPs from the rest of the chip  */
                                                                                 /* through the DFT Buffer. Needs top level port configuration and other DFT  */
                                                                                 /* settings as well. */
#define PSS_DFTATBBF_ATBBFDCDC                             (0x00000002)          /* Set this bit to bring out sense lines from the DCDC through the DFT Buffer.  */
                                                                                 /* Needs top level port configuration and DCDCCNFG1/2/3 register setting as well. */
#define PSS_DFTATBBF_ATBBFFLSH                             (0x00000004)          /* Set this bit to bring out sense lines from the Flash PSS through the DFT Buffer. */
                                                                                 /* Needs top level port configuration and FLSHCNFG0/1 register setting as well. */
#define PSS_DFTATBBF_BUFCALBF                              (0x00000008)          /* Set this bit along with corresponding bit in DFTATBUBF register to short  */
                                                                                 /* buffered and unbuffered path to calibrate DFT Buffer offset. Needs top level  */
                                                                                 /* port configuration as well. */
#define PSS_DFTATBBF_LDOINTSNSBF                           (0x00000010)          /* Set this bit to bring out voltages internal to LDO through the DFT Buffer. Needs */
                                                                                 /* top level port configuration and MISCCNFG register setting as well. */
#define PSS_DFTATBBF_UNUSED__M                             (0x00007fe0)          /* Unused Unused Unused. Unused. */
#define PSS_DFTATBBF_BGINTSNSBF                            (0x00008000)          /* Set this bit to bring out an internal bias node inside bandgap through the DFT  */
                                                                                 /* Buffer. Needs top level port configuration as well. */
#define PSS_DFTATBBF_PRERF0P6SNSBF                         (0x00080000)          /* Set this bit to bring out the pre-S/H 0.6V reference voltage through the DFT  */
                                                                                 /* Buffer. Needs top level port configuration as well. */
#define PSS_DFTATBBF_PRERFVCSNSBF                          (0x00200000)          /* Set this bit to bring out the pre-S/H VCORE reference voltage through the DFT  */
                                                                                 /* Buffer. Needs top level port configuration as well. */
#define PSS_DFTATBBF_BG1P2SNSBF                            (0x00400000)          /* Set this bit to bring out the internal bandgap 1.2V reference voltage through  */
                                                                                 /* the DFT Buffer. Needs top level port configuration as well. */
#define PSS_DFTATBBF_BG0P6SNSBF                            (0x00800000)          /* Set this bit to bring out the internal bandgap 0.6V reference voltage through  */
                                                                                 /* the DFT Buffer. Needs top level port configuration as well. */
#define PSS_DFTATBBF_RFVCSNSBF                             (0x40000000)          /* Set this bit to bring out the VCORE reference voltage through the DFT Buffer.  */
                                                                                 /* Needs top level port configuration as well. */
#define PSS_DFTATBBF_RF0P6SNSBF                            (0x80000000)          /* Set this bit to bring out the generic 0.6V reference voltage through the DFT  */
                                                                                 /* Buffer. Needs top level port configuration as well. */

/* PSS_DFTATBUBF Control Bits */
#define PSS_DFTATBUBF_ATBUBFGEN                            (0x00000001)          /* Set this bit to bring out sense lines from Analog IPs from the rest of the chip  */
                                                                                 /* through the DFT unbuffered path. Needs top level port configuration and other  */
                                                                                 /* DFT settings as well. */
#define PSS_DFTATBUBF_ATBUBFDCDC                           (0x00000002)          /* Set this bit to bring out sense lines from the DCDC through the DFT unbuffered  */
                                                                                 /* path. Needs top level port configuration and DCDCCNFG1/2/3 register setting as  */
                                                                                 /* well. */
#define PSS_DFTATBUBF_ATBUBFFLSH                           (0x00000004)          /* Set this bit to bring out sense lines from the Flash PSS through the DFT  */
                                                                                 /* unbuffered path. Needs top level port configuration and FLSHCNFG0/1 register  */
                                                                                 /* setting as well. */
#define PSS_DFTATBUBF_BUFCALUBF                            (0x00000008)          /* Set this bit along with corresponding bit in DFTATBBF register to short buffered */
                                                                                 /* and unbuffered path to calibrate DFT Buffer offset. Needs top level port  */
                                                                                 /* configuration as well. */
#define PSS_DFTATBUBF_LDOINTSNSUBF                         (0x00000010)          /* Set this bit to bring out voltages internal to LDO through the DFT unbuffered  */
                                                                                 /* path. Needs top level port configuration and MISCCNFG register setting as well. */
#define PSS_DFTATBUBF_UNUSED__M                            (0x000001e0)          /* Unused Unused Unused. Unused. */
#define PSS_DFTATBUBF_SVSLVCCDIVSNS                        (0x00000200)          /* Set this bit to bring out the SVSL resistor divider output through the DFT  */
                                                                                 /* unbuffered path. Needs top level port configuration and MISCCNFG register  */
                                                                                 /* setting as well. */
#define PSS_DFTATBUBF_SVSLIPFRC                            (0x00000400)          /* Set this bit to force the SVSL Input Voltage (instead of VCORE) to test SVSL  */
                                                                                 /* through the DFT unbuffered path. Needs top level port configuration as well.. */
#define PSS_DFTATBUBF_SVSMHVCCDIVSNS                       (0x00000800)          /* Set this bit to bring out the SVSMH resistor divider output through the DFT  */
                                                                                 /* unbuffered path. Needs top level port configuration and MISCCNFG register  */
                                                                                 /* setting as well. */
#define PSS_DFTATBUBF_REFGNDSNS                            (0x00001000)          /* Set this bit to bring out the REFSYS Ground Sense line through the DFT  */
                                                                                 /* unbuffered path. Needs top level port configuration as well. */
#define PSS_DFTATBUBF_SVSMHIPFRC                           (0x00002000)          /* Set this bit to force the SVSMH Input Voltage (instead of VCC / VDDS) to test  */
                                                                                 /* SVSL through the DFT unbuffered path. Needs top level port configuration as well */
                                                                                 /* .. */
#define PSS_DFTATBUBF_REFCTFRC                             (0x00004000)          /* Set this bit to force an external current of 100nA through the DFT unbuffered  */
                                                                                 /* path. Needs top level port configuration as well. */
#define PSS_DFTATBUBF_BGINTSNSUBF                          (0x00008000)          /* Set this bit to bring out an internal bias node inside bandgap through the DFT  */
                                                                                 /* unbuffered path. Needs top level port configuration as well. */
#define PSS_DFTATBUBF_PRERF0P6SNSUBF                       (0x00080000)          /* Set this bit to bring out the pre-S/H 0.6V reference voltage through the DFT  */
                                                                                 /* unbuffered path. Needs top level port configuration as well. */
#define PSS_DFTATBUBF_PRERFVCSNSUBF                        (0x00200000)          /* Set this bit to bring out the pre-S/H VCORE reference voltage through the DFT  */
                                                                                 /* unbuffered path. Needs top level port configuration as well. */
#define PSS_DFTATBUBF_BG1P2SNSUBF                          (0x00400000)          /* Set this bit to bring out the internal bandgap 1.2V reference voltage through  */
                                                                                 /* the DFT unbuffered path. Needs top level port configuration as well. */
#define PSS_DFTATBUBF_BG0P6SNSUBF                          (0x00800000)          /* Set this bit to bring out the internal bandgap 0.6V reference voltage through  */
                                                                                 /* the DFT unbuffered path. Needs top level port configuration as well. */
#define PSS_DFTATBUBF_DBGLDOCT0                            (0x20000000)          /* Set this bit to bring out certain currents internal to VCORE LDO through the DFT */
                                                                                 /* unbuffered path. Needs top level port configuration as well. */
#define PSS_DFTATBUBF_DBGLDOCT1                            (0x40000000)          /* Set this bit to bring out certain currents internal to VCORE LDO through the DFT */
                                                                                 /* unbuffered path. Needs top level port configuration as well. */
#define PSS_DFTATBUBF_DBGLDOCT2                            (0x80000000)          /* Set this bit to bring out certain currents internal to VCORE LDO through the DFT */
                                                                                 /* unbuffered path. Needs top level port configuration as well. */


//*****************************************************************************
// REF_A Registers
//*****************************************************************************
typedef struct {
  union {    /* REF_A_CTL0 Register */
    __IO uint16_t reg;
    struct { /* REF_A_CTL0 Control Bits */
      __IO uint16_t ON                  	:	1;  /* Reference enable. Can be modified only when REFGENBUSY = 0. */
      __IO uint16_t OUT                 	:	1;  /* Reference output buffer. Can be modified only when REFGENBUSY = 0. */
      __IO uint16_t RESERVED_2          	:	1;  /* Reserved */
      __IO uint16_t TCOFF               	:	1;  /* Temperature sensor disabled. Can be modified only when REFGENBUSY = 0. */
      __IO uint16_t VSEL                	:	2;  /* Reference voltage level select. Can be modified only when REFGENBUSY = 0. */
      __IO uint16_t GENOT               	:	1;  /* Reference generator one-time trigger. If written with a "1" the generation of  */
                                                /* the variable reference voltage is started. Once the reference voltage request is */
                                                /* set this bit is cleared by hardware.  */
      __IO uint16_t BGOT                	:	1;  /* Bandgap and bandgap buffer one-time trigger. If written with a "1" the  */
                                                /* generation of the buffered bandgap voltage is started. Once the bandgap buffer  */
                                                /* voltage request is set this bit is cleared by hardware.  */
      __I  uint16_t GENACT              	:	1;  /* Reference generator active. Read only. */
      __I  uint16_t BGACT               	:	1;  /* Reference bandgap active. Read only. */
      __I  uint16_t GENBUSY             	:	1;  /* Reference generator busy. Read only. */
      __I  uint16_t BGMODE              	:	1;  /* Bandgap mode. Read only. */
      __I  uint16_t GENRDY              	:	1;  /* Variable reference voltage ready status. */
      __I  uint16_t BGRDY               	:	1;  /* Buffered bandgap voltage ready status. */
      __I  uint16_t RESERVED_1          	:	2;  /* Reserved */

    } bit;
  } CTL0;
} REF_A_Type;


/* REF_A_CTL0 Control Bits */
#define REF_A_CTL0_ON                                      (0x0001)              /* Reference enable. Can be modified only when REFGENBUSY = 0. */
#define REF_A_CTL0_OUT                                     (0x0002)              /* Reference output buffer. Can be modified only when REFGENBUSY = 0. */
#define REF_A_CTL0_RESERVED_2                              (0x0004)              /* Reserved. */
#define REF_A_CTL0_TCOFF                                   (0x0008)              /* Temperature sensor disabled. Can be modified only when REFGENBUSY = 0. */
#define REF_A_CTL0_VSEL__M                                 (0x0030)              /* Reference voltage level select. Can be modified only when REFGENBUSY = 0. */
#define REF_A_CTL0_VSEL__0                                 (0x0000)              /* 1.2 V available when reference requested or REFON = 1 */
#define REF_A_CTL0_VSEL__1                                 (0x0010)              /* 1.45 V available when reference requested or REFON = 1 */
#define REF_A_CTL0_VSEL__3                                 (0x0030)              /* 2.5 V available when reference requested or REFON = 1 */
#define REF_A_CTL0_GENOT                                   (0x0040)              /* Reference generator one-time trigger. If written with a "1" the generation of  */
                                                                                 /* the variable reference voltage is started. Once the reference voltage request is */
                                                                                 /* set this bit is cleared by hardware.  */
#define REF_A_CTL0_BGOT                                    (0x0080)              /* Bandgap and bandgap buffer one-time trigger. If written with a "1" the  */
                                                                                 /* generation of the buffered bandgap voltage is started. Once the bandgap buffer  */
                                                                                 /* voltage request is set this bit is cleared by hardware.  */
#define REF_A_CTL0_GENACT                                  (0x0100)              /* Reference generator active. Read only. */
#define REF_A_CTL0_BGACT                                   (0x0200)              /* Reference bandgap active. Read only. */
#define REF_A_CTL0_GENBUSY                                 (0x0400)              /* Reference generator busy. Read only. */
#define REF_A_CTL0_BGMODE                                  (0x0800)              /* Bandgap mode. Read only. */
#define REF_A_CTL0_GENRDY                                  (0x1000)              /* Variable reference voltage ready status. */
#define REF_A_CTL0_BGRDY                                   (0x2000)              /* Buffered bandgap voltage ready status. */
#define REF_A_CTL0_RESERVED_1__M                           (0xc000)              /* Reserved. Always reads as 0. */


//*****************************************************************************
// RSTCTL Registers
//*****************************************************************************
typedef struct {
  union {    /* RSTCTL_RESETREQ Register */
    __IO uint32_t reg;
    struct { /* RSTCTL_RESETREQ Control Bits */
      __O  uint32_t SFT_REQ             	:	1;  /* If written with 1, generates a Soft Reset request to the system Reset Controller */
      __O  uint32_t HRD_REQ             	:	1;  /* If written with 1, generates a Hard Reset request to the system Reset Controller */
      __I  uint32_t RESERVED_2          	:	6;  /* Reserved */
      __O  uint32_t RSTKEY              	:	8;  /* Must be written with 69h to enable writes to bits 1-0 (in the same write  */
                                                /* operation), else writes to bits 1-0 are ignored */
      __I  uint32_t RESERVED_1          	:	16;  /* Reserved */

    } bit;
  } RESETREQ;
  union {    /* RSTCTL_HRDRESETSTAT Register */
    __IO uint32_t reg;
    struct { /* RSTCTL_HRDRESETSTAT Control Bits */
      __I  uint32_t SRC0                	:	1;  /* If 1, indicates that SRC0 was the source of the Hard Reset */
      __I  uint32_t SRC1                	:	1;  /* If 1, indicates that SRC1 was the source of the Hard Reset */
      __I  uint32_t SRC2                	:	1;  /* If 1, indicates that SRC2 was the source of the Hard Reset */
      __I  uint32_t SRC3                	:	1;  /* If 1, indicates that SRC3 was the source of the Hard Reset */
      __I  uint32_t SRC4                	:	1;  /* If 1, indicates that SRC4 was the source of the Hard Reset */
      __I  uint32_t SRC5                	:	1;  /* If 1, indicates that SRC5 was the source of the Hard Reset */
      __I  uint32_t SRC6                	:	1;  /* If 1, indicates that SRC6 was the source of the Hard Reset */
      __I  uint32_t SRC7                	:	1;  /* If 1, indicates that SRC7 was the source of the Hard Reset */
      __I  uint32_t SRC8                	:	1;  /* If 1, indicates that SRC8 was the source of the Hard Reset */
      __I  uint32_t SRC9                	:	1;  /* If 1, indicates that SRC9 was the source of the Hard Reset */
      __I  uint32_t SRC10               	:	1;  /* If 1, indicates that SRC10 was the source of the Hard Reset */
      __I  uint32_t SRC11               	:	1;  /* If 1, indicates that SRC11 was the source of the Hard Reset */
      __I  uint32_t SRC12               	:	1;  /* If 1, indicates that SRC12 was the source of the Hard Reset */
      __I  uint32_t SRC13               	:	1;  /* If 1, indicates that SRC13 was the source of the Hard Reset */
      __I  uint32_t SRC14               	:	1;  /* If 1, indicates that SRC14 was the source of the Hard Reset */
      __I  uint32_t SRC15               	:	1;  /* If 1, indicates that SRC15 was the source of the Hard Reset */
      __I  uint32_t RESERVED_1          	:	16;  /* Reserved */

    } bit;
  } HRDRESETSTAT;
  union {    /* RSTCTL_HRDRESETCLR Register */
    __IO uint32_t reg;
    struct { /* RSTCTL_HRDRESETCLR Control Bits */
      __O  uint32_t SRC0                	:	1;  /* Write 1 clears the corresponding bit in the RSTCTL_HRDRESETSTAT_REG. Write 0 has */
                                                /* no effect. */
      __O  uint32_t SRC1                	:	1;  /* Write 1 clears the corresponding bit in the RSTCTL_HRDRESETSTAT_REG. Write 0 has */
                                                /* no effect. */
      __O  uint32_t SRC2                	:	1;  /* Write 1 clears the corresponding bit in the RSTCTL_HRDRESETSTAT_REG. Write 0 has */
                                                /* no effect. */
      __O  uint32_t SRC3                	:	1;  /* Write 1 clears the corresponding bit in the RSTCTL_HRDRESETSTAT_REG. Write 0 has */
                                                /* no effect. */
      __O  uint32_t SRC4                	:	1;  /* Write 1 clears the corresponding bit in the RSTCTL_HRDRESETSTAT_REG. Write 0 has */
                                                /* no effect. */
      __O  uint32_t SRC5                	:	1;  /* Write 1 clears the corresponding bit in the RSTCTL_HRDRESETSTAT_REG. Write 0 has */
                                                /* no effect. */
      __O  uint32_t SRC6                	:	1;  /* Write 1 clears the corresponding bit in the RSTCTL_HRDRESETSTAT_REG. Write 0 has */
                                                /* no effect. */
      __O  uint32_t SRC7                	:	1;  /* Write 1 clears the corresponding bit in the RSTCTL_HRDRESETSTAT_REG. Write 0 has */
                                                /* no effect. */
      __O  uint32_t SRC8                	:	1;  /* Write 1 clears the corresponding bit in the RSTCTL_HRDRESETSTAT_REG. Write 0 has */
                                                /* no effect. */
      __O  uint32_t SRC9                	:	1;  /* Write 1 clears the corresponding bit in the RSTCTL_HRDRESETSTAT_REG. Write 0 has */
                                                /* no effect. */
      __O  uint32_t SRC10               	:	1;  /* Write 1 clears the corresponding bit in the RSTCTL_HRDRESETSTAT_REG. Write 0 has */
                                                /* no effect. */
      __O  uint32_t SRC11               	:	1;  /* Write 1 clears the corresponding bit in the RSTCTL_HRDRESETSTAT_REG. Write 0 has */
                                                /* no effect. */
      __O  uint32_t SRC12               	:	1;  /* Write 1 clears the corresponding bit in the RSTCTL_HRDRESETSTAT_REG. Write 0 has */
                                                /* no effect. */
      __O  uint32_t SRC13               	:	1;  /* Write 1 clears the corresponding bit in the RSTCTL_HRDRESETSTAT_REG. Write 0 has */
                                                /* no effect. */
      __O  uint32_t SRC14               	:	1;  /* Write 1 clears the corresponding bit in the RSTCTL_HRDRESETSTAT_REG. Write 0 has */
                                                /* no effect. */
      __O  uint32_t SRC15               	:	1;  /* Write 1 clears the corresponding bit in the RSTCTL_HRDRESETSTAT_REG. Write 0 has */
                                                /* no effect. */
      __I  uint32_t RESERVED_1          	:	16;  /* Reserved */

    } bit;
  } HRDRESETCLR;
  union {    /* RSTCTL_HRDRESETSET Register */
    __IO uint32_t reg;
    struct { /* RSTCTL_HRDRESETSET Control Bits */
      __O  uint32_t SRC0                	:	1;  /* Write 1 sets the corresponding bit in the RSTCTL_HRDRESETSTAT_REG (and initiates */
                                                /* a Hard Reset). Write 0 has no effect. */
      __O  uint32_t SRC1                	:	1;  /* Write 1 sets the corresponding bit in the RSTCTL_HRDRESETSTAT_REG (and initiates */
                                                /* a Hard Reset). Write 0 has no effect. */
      __O  uint32_t SRC2                	:	1;  /* Write 1 sets the corresponding bit in the RSTCTL_HRDRESETSTAT_REG (and initiates */
                                                /* a Hard Reset). Write 0 has no effect. */
      __O  uint32_t SRC3                	:	1;  /* Write 1 sets the corresponding bit in the RSTCTL_HRDRESETSTAT_REG (and initiates */
                                                /* a Hard Reset). Write 0 has no effect. */
      __O  uint32_t SRC4                	:	1;  /* Write 1 sets the corresponding bit in the RSTCTL_HRDRESETSTAT_REG (and initiates */
                                                /* a Hard Reset). Write 0 has no effect. */
      __O  uint32_t SRC5                	:	1;  /* Write 1 sets the corresponding bit in the RSTCTL_HRDRESETSTAT_REG (and initiates */
                                                /* a Hard Reset). Write 0 has no effect. */
      __O  uint32_t SRC6                	:	1;  /* Write 1 sets the corresponding bit in the RSTCTL_HRDRESETSTAT_REG (and initiates */
                                                /* a Hard Reset). Write 0 has no effect. */
      __O  uint32_t SRC7                	:	1;  /* Write 1 sets the corresponding bit in the RSTCTL_HRDRESETSTAT_REG (and initiates */
                                                /* a Hard Reset). Write 0 has no effect. */
      __O  uint32_t SRC8                	:	1;  /* Write 1 sets the corresponding bit in the RSTCTL_HRDRESETSTAT_REG (and initiates */
                                                /* a Hard Reset). Write 0 has no effect. */
      __O  uint32_t SRC9                	:	1;  /* Write 1 sets the corresponding bit in the RSTCTL_HRDRESETSTAT_REG (and initiates */
                                                /* a Hard Reset). Write 0 has no effect. */
      __O  uint32_t SRC10               	:	1;  /* Write 1 sets the corresponding bit in the RSTCTL_HRDRESETSTAT_REG (and initiates */
                                                /* a Hard Reset). Write 0 has no effect. */
      __O  uint32_t SRC11               	:	1;  /* Write 1 sets the corresponding bit in the RSTCTL_HRDRESETSTAT_REG (and initiates */
                                                /* a Hard Reset). Write 0 has no effect. */
      __O  uint32_t SRC12               	:	1;  /* Write 1 sets the corresponding bit in the RSTCTL_HRDRESETSTAT_REG (and initiates */
                                                /* a Hard Reset). Write 0 has no effect. */
      __O  uint32_t SRC13               	:	1;  /* Write 1 sets the corresponding bit in the RSTCTL_HRDRESETSTAT_REG (and initiates */
                                                /* a Hard Reset). Write 0 has no effect. */
      __O  uint32_t SRC14               	:	1;  /* Write 1 sets the corresponding bit in the RSTCTL_HRDRESETSTAT_REG (and initiates */
                                                /* a Hard Reset). Write 0 has no effect. */
      __O  uint32_t SRC15               	:	1;  /* Write 1 sets the corresponding bit in the RSTCTL_HRDRESETSTAT_REG (and initiates */
                                                /* a Hard Reset). Write 0 has no effect. */
      __I  uint32_t RESERVED_1          	:	16;  /* Reserved */

    } bit;
  } HRDRESETSET;
  union {    /* RSTCTL_SFTRESETSTAT Register */
    __IO uint32_t reg;
    struct { /* RSTCTL_SFTRESETSTAT Control Bits */
      __I  uint32_t SRC0                	:	1;  /* If 1, indicates that SRC0 was the source of the Soft Reset */
      __I  uint32_t SRC1                	:	1;  /* If 1, indicates that SRC1 was the source of the Soft Reset */
      __I  uint32_t SRC2                	:	1;  /* If 1, indicates that SRC2 was the source of the Soft Reset */
      __I  uint32_t SRC3                	:	1;  /* If 1, indicates that SRC3 was the source of the Soft Reset */
      __I  uint32_t SRC4                	:	1;  /* If 1, indicates that SRC4 was the source of the Soft Reset */
      __I  uint32_t SRC5                	:	1;  /* If 1, indicates that SRC5 was the source of the Soft Reset */
      __I  uint32_t SRC6                	:	1;  /* If 1, indicates that SRC6 was the source of the Soft Reset */
      __I  uint32_t SRC7                	:	1;  /* If 1, indicates that SRC7 was the source of the Soft Reset */
      __I  uint32_t SRC8                	:	1;  /* If 1, indicates that SRC8 was the source of the Soft Reset */
      __I  uint32_t SRC9                	:	1;  /* If 1, indicates that SRC9 was the source of the Soft Reset */
      __I  uint32_t SRC10               	:	1;  /* If 1, indicates that SRC10 was the source of the Soft Reset */
      __I  uint32_t SRC11               	:	1;  /* If 1, indicates that SRC11 was the source of the Soft Reset */
      __I  uint32_t SRC12               	:	1;  /* If 1, indicates that SRC12 was the source of the Soft Reset */
      __I  uint32_t SRC13               	:	1;  /* If 1, indicates that SRC13 was the source of the Soft Reset */
      __I  uint32_t SRC14               	:	1;  /* If 1, indicates that SRC14 was the source of the Soft Reset */
      __I  uint32_t SRC15               	:	1;  /* If 1, indicates that SRC15 was the source of the Soft Reset */
      __I  uint32_t RESERVED_1          	:	16;  /* Reserved */

    } bit;
  } SFTRESETSTAT;
  union {    /* RSTCTL_SFTRESETCLR Register */
    __IO uint32_t reg;
    struct { /* RSTCTL_SFTRESETCLR Control Bits */
      __O  uint32_t SRC0                	:	1;  /* Write 1 clears the corresponding bit in the RSTCTL_SFTRESETSTAT_REG. Write 0 has */
                                                /* no effect. */
      __O  uint32_t SRC1                	:	1;  /* Write 1 clears the corresponding bit in the RSTCTL_SFTRESETSTAT_REG. Write 0 has */
                                                /* no effect. */
      __O  uint32_t SRC2                	:	1;  /* Write 1 clears the corresponding bit in the RSTCTL_SFTRESETSTAT_REG. Write 0 has */
                                                /* no effect. */
      __O  uint32_t SRC3                	:	1;  /* Write 1 clears the corresponding bit in the RSTCTL_SFTRESETSTAT_REG. Write 0 has */
                                                /* no effect. */
      __O  uint32_t SRC4                	:	1;  /* Write 1 clears the corresponding bit in the RSTCTL_SFTRESETSTAT_REG. Write 0 has */
                                                /* no effect. */
      __O  uint32_t SRC5                	:	1;  /* Write 1 clears the corresponding bit in the RSTCTL_SFTRESETSTAT_REG. Write 0 has */
                                                /* no effect. */
      __O  uint32_t SRC6                	:	1;  /* Write 1 clears the corresponding bit in the RSTCTL_SFTRESETSTAT_REG. Write 0 has */
                                                /* no effect. */
      __O  uint32_t SRC7                	:	1;  /* Write 1 clears the corresponding bit in the RSTCTL_SFTRESETSTAT_REG. Write 0 has */
                                                /* no effect. */
      __O  uint32_t SRC8                	:	1;  /* Write 1 clears the corresponding bit in the RSTCTL_SFTRESETSTAT_REG. Write 0 has */
                                                /* no effect. */
      __O  uint32_t SRC9                	:	1;  /* Write 1 clears the corresponding bit in the RSTCTL_SFTRESETSTAT_REG. Write 0 has */
                                                /* no effect. */
      __O  uint32_t SRC10               	:	1;  /* Write 1 clears the corresponding bit in the RSTCTL_SFTRESETSTAT_REG. Write 0 has */
                                                /* no effect. */
      __O  uint32_t SRC11               	:	1;  /* Write 1 clears the corresponding bit in the RSTCTL_SFTRESETSTAT_REG. Write 0 has */
                                                /* no effect. */
      __O  uint32_t SRC12               	:	1;  /* Write 1 clears the corresponding bit in the RSTCTL_SFTRESETSTAT_REG. Write 0 has */
                                                /* no effect. */
      __O  uint32_t SRC13               	:	1;  /* Write 1 clears the corresponding bit in the RSTCTL_SFTRESETSTAT_REG. Write 0 has */
                                                /* no effect. */
      __O  uint32_t SRC14               	:	1;  /* Write 1 clears the corresponding bit in the RSTCTL_SFTRESETSTAT_REG. Write 0 has */
                                                /* no effect. */
      __O  uint32_t SRC15               	:	1;  /* Write 1 clears the corresponding bit in the RSTCTL_SFTRESETSTAT_REG. Write 0 has */
                                                /* no effect. */
      __I  uint32_t RESERVED_1          	:	16;  /* Reserved */

    } bit;
  } SFTRESETCLR;
  union {    /* RSTCTL_SFTRESETSET Register */
    __IO uint32_t reg;
    struct { /* RSTCTL_SFTRESETSET Control Bits */
      __O  uint32_t SRC0                	:	1;  /* Write 1 sets the corresponding bit in the RSTCTL_SFTRESETSTAT_REG (and initiates */
                                                /* a Soft Reset). Write 0 has no effect. */
      __O  uint32_t SRC1                	:	1;  /* Write 1 sets the corresponding bit in the RSTCTL_SFTRESETSTAT_REG (and initiates */
                                                /* a Soft Reset). Write 0 has no effect. */
      __O  uint32_t SRC2                	:	1;  /* Write 1 sets the corresponding bit in the RSTCTL_SFTRESETSTAT_REG (and initiates */
                                                /* a Soft Reset). Write 0 has no effect. */
      __O  uint32_t SRC3                	:	1;  /* Write 1 sets the corresponding bit in the RSTCTL_SFTRESETSTAT_REG (and initiates */
                                                /* a Soft Reset). Write 0 has no effect. */
      __O  uint32_t SRC4                	:	1;  /* Write 1 sets the corresponding bit in the RSTCTL_SFTRESETSTAT_REG (and initiates */
                                                /* a Soft Reset). Write 0 has no effect. */
      __O  uint32_t SRC5                	:	1;  /* Write 1 sets the corresponding bit in the RSTCTL_SFTRESETSTAT_REG (and initiates */
                                                /* a Soft Reset). Write 0 has no effect. */
      __O  uint32_t SRC6                	:	1;  /* Write 1 sets the corresponding bit in the RSTCTL_SFTRESETSTAT_REG (and initiates */
                                                /* a Soft Reset). Write 0 has no effect. */
      __O  uint32_t SRC7                	:	1;  /* Write 1 sets the corresponding bit in the RSTCTL_SFTRESETSTAT_REG (and initiates */
                                                /* a Soft Reset). Write 0 has no effect. */
      __O  uint32_t SRC8                	:	1;  /* Write 1 sets the corresponding bit in the RSTCTL_SFTRESETSTAT_REG (and initiates */
                                                /* a Soft Reset). Write 0 has no effect. */
      __O  uint32_t SRC9                	:	1;  /* Write 1 sets the corresponding bit in the RSTCTL_SFTRESETSTAT_REG (and initiates */
                                                /* a Soft Reset). Write 0 has no effect. */
      __O  uint32_t SRC10               	:	1;  /* Write 1 sets the corresponding bit in the RSTCTL_SFTRESETSTAT_REG (and initiates */
                                                /* a Soft Reset). Write 0 has no effect. */
      __O  uint32_t SRC11               	:	1;  /* Write 1 sets the corresponding bit in the RSTCTL_SFTRESETSTAT_REG (and initiates */
                                                /* a Soft Reset). Write 0 has no effect. */
      __O  uint32_t SRC12               	:	1;  /* Write 1 sets the corresponding bit in the RSTCTL_SFTRESETSTAT_REG (and initiates */
                                                /* a Soft Reset). Write 0 has no effect. */
      __O  uint32_t SRC13               	:	1;  /* Write 1 sets the corresponding bit in the RSTCTL_SFTRESETSTAT_REG (and initiates */
                                                /* a Soft Reset). Write 0 has no effect. */
      __O  uint32_t SRC14               	:	1;  /* Write 1 sets the corresponding bit in the RSTCTL_SFTRESETSTAT_REG (and initiates */
                                                /* a Soft Reset). Write 0 has no effect. */
      __O  uint32_t SRC15               	:	1;  /* Write 1 sets the corresponding bit in the RSTCTL_SFTRESETSTAT_REG (and initiates */
                                                /* a Soft Reset). Write 0 has no effect. */
      __I  uint32_t RESERVED_1          	:	16;  /* Reserved */

    } bit;
  } SFTRESETSET;
  uint8_t RESERVED0[228];
  union {    /* RSTCTL_PSSRSTSTAT Register */
    __IO uint32_t reg;
    struct { /* RSTCTL_PSSRSTSTAT Control Bits */
      __I  uint32_t SVSL                	:	1;  /* Indicates if POR was caused by an SVSL trip condition in the PSS (SVSL is always */
                                                /* enabled) */
      __I  uint32_t SVSH                	:	1;  /* Indicates if POR was caused by an SVSH trip condition int the PSS (with SVSH  */
                                                /* enabled and set as supervisor) */
      __I  uint32_t BGREF               	:	1;  /* Indicates if POR was caused by a BGREF not okay condition in the PSS */
      __I  uint32_t VCCDET              	:	1;  /* Indicates if POR was caused by a VccDET trip condition in the PSS */
      __I  uint32_t RESERVED_1          	:	28;  /* Reserved */

    } bit;
  } PSSRSTSTAT;
  union {    /* RSTCTL_PSSRSTCLR Register */
    __IO uint32_t reg;
    struct { /* RSTCTL_PSSRSTCLR Control Bits */
      __O  uint32_t CLR                 	:	1;  /* Write 1 clears all PSS Reset Flags in the RSTCTL_PSSRSTSTAT_REG */
      __I  uint32_t RESERVED_1          	:	31;  /* Reserved */

    } bit;
  } PSSRSTCLR;
  union {    /* RSTCTL_PCMRSTSTAT Register */
    __IO uint32_t reg;
    struct { /* RSTCTL_PCMRSTSTAT Control Bits */
      __I  uint32_t SD0                 	:	1;  /* Indicates if POR was caused by PCM due to an exit from the SD0 mode */
      __I  uint32_t SDC                 	:	1;  /* Indicates if POR was caused by PCM due to an exit from the SDC mode */
      __I  uint32_t RESERVED_1          	:	30;  /* Reserved */

    } bit;
  } PCMRSTSTAT;
  union {    /* RSTCTL_PCMRSTCLR Register */
    __IO uint32_t reg;
    struct { /* RSTCTL_PCMRSTCLR Control Bits */
      __O  uint32_t CLR                 	:	1;  /* Write 1 clears all PCM Reset Flags in the RSTCTL_PCMRSTSTAT_REG */
      __I  uint32_t RESERVED_1          	:	31;  /* Reserved */

    } bit;
  } PCMRSTCLR;
  union {    /* RSTCTL_PINRSTSTAT Register */
    __IO uint32_t reg;
    struct { /* RSTCTL_PINRSTSTAT Control Bits */
      __I  uint32_t RSTNMI              	:	1;  /* Indicates if POR was caused by RSTNMI pin based reset event in the device. This  */
                                                /* bit comes up by default as 0 in the case of cold power up. This is only set if  */
                                                /* there is a POR caused due to the RSTNMI pin. */
      __I  uint32_t RESERVED_1          	:	31;  /* Reserved */

    } bit;
  } PINRSTSTAT;
  union {    /* RSTCTL_PINRSTCLR Register */
    __IO uint32_t reg;
    struct { /* RSTCTL_PINRSTCLR Control Bits */
      __O  uint32_t CLR                 	:	1;  /* Write 1 clears the RSTNMI Pin Reset Flag in RSTCTL_PINRSTSTAT_REG */
      __I  uint32_t RESERVED_1          	:	31;  /* Reserved */

    } bit;
  } PINRSTCLR;
  union {    /* RSTCTL_RBTRSTSTAT Register */
    __IO uint32_t reg;
    struct { /* RSTCTL_RBTRSTSTAT Control Bits */
      __I  uint32_t REBOOT              	:	1;  /* Indicates if Reboot reset was caused by the CPU_SYS module. */
      __I  uint32_t RESERVED_1          	:	31;  /* Reserved */

    } bit;
  } RBTRSTSTAT;
  union {    /* RSTCTL_RBTRSTCLR Register */
    __IO uint32_t reg;
    struct { /* RSTCTL_RBTRSTCLR Control Bits */
      __O  uint32_t CLR                 	:	1;  /* Write 1 clears the Reboot Reset Flag in RSTCTL_RBTRSTSTAT_REG */
      __I  uint32_t RESERVED_1          	:	31;  /* Reserved */

    } bit;
  } RBTRSTCLR;
} RSTCTL_Type;


/* RSTCTL_RESETREQ Control Bits */
#define RSTCTL_RESETREQ_SFT_REQ                            (0x00000001)          /* If written with 1, generates a Soft Reset request to the system Reset Controller */
#define RSTCTL_RESETREQ_HRD_REQ                            (0x00000002)          /* If written with 1, generates a Hard Reset request to the system Reset Controller */
#define RSTCTL_RESETREQ_RESERVED_2__M                      (0x000000fc)          /* Reserved. Always reads 0h */
#define RSTCTL_RESETREQ_RSTKEY__M                          (0x0000ff00)          /* Must be written with 69h to enable writes to bits 1-0 (in the same write  */
                                                                                 /* operation), else writes to bits 1-0 are ignored */
#define RSTCTL_RESETREQ_RESERVED_1__M                      (0xffff0000)          /* Reserved. Always reads 0h */

/* RSTCTL_HRDRESETSTAT Control Bits */
#define RSTCTL_HRDRESETSTAT_SRC0                           (0x00000001)          /* If 1, indicates that SRC0 was the source of the Hard Reset */
#define RSTCTL_HRDRESETSTAT_SRC1                           (0x00000002)          /* If 1, indicates that SRC1 was the source of the Hard Reset */
#define RSTCTL_HRDRESETSTAT_SRC2                           (0x00000004)          /* If 1, indicates that SRC2 was the source of the Hard Reset */
#define RSTCTL_HRDRESETSTAT_SRC3                           (0x00000008)          /* If 1, indicates that SRC3 was the source of the Hard Reset */
#define RSTCTL_HRDRESETSTAT_SRC4                           (0x00000010)          /* If 1, indicates that SRC4 was the source of the Hard Reset */
#define RSTCTL_HRDRESETSTAT_SRC5                           (0x00000020)          /* If 1, indicates that SRC5 was the source of the Hard Reset */
#define RSTCTL_HRDRESETSTAT_SRC6                           (0x00000040)          /* If 1, indicates that SRC6 was the source of the Hard Reset */
#define RSTCTL_HRDRESETSTAT_SRC7                           (0x00000080)          /* If 1, indicates that SRC7 was the source of the Hard Reset */
#define RSTCTL_HRDRESETSTAT_SRC8                           (0x00000100)          /* If 1, indicates that SRC8 was the source of the Hard Reset */
#define RSTCTL_HRDRESETSTAT_SRC9                           (0x00000200)          /* If 1, indicates that SRC9 was the source of the Hard Reset */
#define RSTCTL_HRDRESETSTAT_SRC10                          (0x00000400)          /* If 1, indicates that SRC10 was the source of the Hard Reset */
#define RSTCTL_HRDRESETSTAT_SRC11                          (0x00000800)          /* If 1, indicates that SRC11 was the source of the Hard Reset */
#define RSTCTL_HRDRESETSTAT_SRC12                          (0x00001000)          /* If 1, indicates that SRC12 was the source of the Hard Reset */
#define RSTCTL_HRDRESETSTAT_SRC13                          (0x00002000)          /* If 1, indicates that SRC13 was the source of the Hard Reset */
#define RSTCTL_HRDRESETSTAT_SRC14                          (0x00004000)          /* If 1, indicates that SRC14 was the source of the Hard Reset */
#define RSTCTL_HRDRESETSTAT_SRC15                          (0x00008000)          /* If 1, indicates that SRC15 was the source of the Hard Reset */
#define RSTCTL_HRDRESETSTAT_RESERVED_1__M                  (0xffff0000)          /* Reserved. Always reads 0h */

/* RSTCTL_HRDRESETCLR Control Bits */
#define RSTCTL_HRDRESETCLR_SRC0                            (0x00000001)          /* Write 1 clears the corresponding bit in the RSTCTL_HRDRESETSTAT_REG. Write 0 has */
                                                                                 /* no effect. */
#define RSTCTL_HRDRESETCLR_SRC1                            (0x00000002)          /* Write 1 clears the corresponding bit in the RSTCTL_HRDRESETSTAT_REG. Write 0 has */
                                                                                 /* no effect. */
#define RSTCTL_HRDRESETCLR_SRC2                            (0x00000004)          /* Write 1 clears the corresponding bit in the RSTCTL_HRDRESETSTAT_REG. Write 0 has */
                                                                                 /* no effect. */
#define RSTCTL_HRDRESETCLR_SRC3                            (0x00000008)          /* Write 1 clears the corresponding bit in the RSTCTL_HRDRESETSTAT_REG. Write 0 has */
                                                                                 /* no effect. */
#define RSTCTL_HRDRESETCLR_SRC4                            (0x00000010)          /* Write 1 clears the corresponding bit in the RSTCTL_HRDRESETSTAT_REG. Write 0 has */
                                                                                 /* no effect. */
#define RSTCTL_HRDRESETCLR_SRC5                            (0x00000020)          /* Write 1 clears the corresponding bit in the RSTCTL_HRDRESETSTAT_REG. Write 0 has */
                                                                                 /* no effect. */
#define RSTCTL_HRDRESETCLR_SRC6                            (0x00000040)          /* Write 1 clears the corresponding bit in the RSTCTL_HRDRESETSTAT_REG. Write 0 has */
                                                                                 /* no effect. */
#define RSTCTL_HRDRESETCLR_SRC7                            (0x00000080)          /* Write 1 clears the corresponding bit in the RSTCTL_HRDRESETSTAT_REG. Write 0 has */
                                                                                 /* no effect. */
#define RSTCTL_HRDRESETCLR_SRC8                            (0x00000100)          /* Write 1 clears the corresponding bit in the RSTCTL_HRDRESETSTAT_REG. Write 0 has */
                                                                                 /* no effect. */
#define RSTCTL_HRDRESETCLR_SRC9                            (0x00000200)          /* Write 1 clears the corresponding bit in the RSTCTL_HRDRESETSTAT_REG. Write 0 has */
                                                                                 /* no effect. */
#define RSTCTL_HRDRESETCLR_SRC10                           (0x00000400)          /* Write 1 clears the corresponding bit in the RSTCTL_HRDRESETSTAT_REG. Write 0 has */
                                                                                 /* no effect. */
#define RSTCTL_HRDRESETCLR_SRC11                           (0x00000800)          /* Write 1 clears the corresponding bit in the RSTCTL_HRDRESETSTAT_REG. Write 0 has */
                                                                                 /* no effect. */
#define RSTCTL_HRDRESETCLR_SRC12                           (0x00001000)          /* Write 1 clears the corresponding bit in the RSTCTL_HRDRESETSTAT_REG. Write 0 has */
                                                                                 /* no effect. */
#define RSTCTL_HRDRESETCLR_SRC13                           (0x00002000)          /* Write 1 clears the corresponding bit in the RSTCTL_HRDRESETSTAT_REG. Write 0 has */
                                                                                 /* no effect. */
#define RSTCTL_HRDRESETCLR_SRC14                           (0x00004000)          /* Write 1 clears the corresponding bit in the RSTCTL_HRDRESETSTAT_REG. Write 0 has */
                                                                                 /* no effect. */
#define RSTCTL_HRDRESETCLR_SRC15                           (0x00008000)          /* Write 1 clears the corresponding bit in the RSTCTL_HRDRESETSTAT_REG. Write 0 has */
                                                                                 /* no effect. */
#define RSTCTL_HRDRESETCLR_RESERVED_1__M                   (0xffff0000)          /* Reserved. Always reads 0h */

/* RSTCTL_HRDRESETSET Control Bits */
#define RSTCTL_HRDRESETSET_SRC0                            (0x00000001)          /* Write 1 sets the corresponding bit in the RSTCTL_HRDRESETSTAT_REG (and initiates */
                                                                                 /* a Hard Reset). Write 0 has no effect. */
#define RSTCTL_HRDRESETSET_SRC1                            (0x00000002)          /* Write 1 sets the corresponding bit in the RSTCTL_HRDRESETSTAT_REG (and initiates */
                                                                                 /* a Hard Reset). Write 0 has no effect. */
#define RSTCTL_HRDRESETSET_SRC2                            (0x00000004)          /* Write 1 sets the corresponding bit in the RSTCTL_HRDRESETSTAT_REG (and initiates */
                                                                                 /* a Hard Reset). Write 0 has no effect. */
#define RSTCTL_HRDRESETSET_SRC3                            (0x00000008)          /* Write 1 sets the corresponding bit in the RSTCTL_HRDRESETSTAT_REG (and initiates */
                                                                                 /* a Hard Reset). Write 0 has no effect. */
#define RSTCTL_HRDRESETSET_SRC4                            (0x00000010)          /* Write 1 sets the corresponding bit in the RSTCTL_HRDRESETSTAT_REG (and initiates */
                                                                                 /* a Hard Reset). Write 0 has no effect. */
#define RSTCTL_HRDRESETSET_SRC5                            (0x00000020)          /* Write 1 sets the corresponding bit in the RSTCTL_HRDRESETSTAT_REG (and initiates */
                                                                                 /* a Hard Reset). Write 0 has no effect. */
#define RSTCTL_HRDRESETSET_SRC6                            (0x00000040)          /* Write 1 sets the corresponding bit in the RSTCTL_HRDRESETSTAT_REG (and initiates */
                                                                                 /* a Hard Reset). Write 0 has no effect. */
#define RSTCTL_HRDRESETSET_SRC7                            (0x00000080)          /* Write 1 sets the corresponding bit in the RSTCTL_HRDRESETSTAT_REG (and initiates */
                                                                                 /* a Hard Reset). Write 0 has no effect. */
#define RSTCTL_HRDRESETSET_SRC8                            (0x00000100)          /* Write 1 sets the corresponding bit in the RSTCTL_HRDRESETSTAT_REG (and initiates */
                                                                                 /* a Hard Reset). Write 0 has no effect. */
#define RSTCTL_HRDRESETSET_SRC9                            (0x00000200)          /* Write 1 sets the corresponding bit in the RSTCTL_HRDRESETSTAT_REG (and initiates */
                                                                                 /* a Hard Reset). Write 0 has no effect. */
#define RSTCTL_HRDRESETSET_SRC10                           (0x00000400)          /* Write 1 sets the corresponding bit in the RSTCTL_HRDRESETSTAT_REG (and initiates */
                                                                                 /* a Hard Reset). Write 0 has no effect. */
#define RSTCTL_HRDRESETSET_SRC11                           (0x00000800)          /* Write 1 sets the corresponding bit in the RSTCTL_HRDRESETSTAT_REG (and initiates */
                                                                                 /* a Hard Reset). Write 0 has no effect. */
#define RSTCTL_HRDRESETSET_SRC12                           (0x00001000)          /* Write 1 sets the corresponding bit in the RSTCTL_HRDRESETSTAT_REG (and initiates */
                                                                                 /* a Hard Reset). Write 0 has no effect. */
#define RSTCTL_HRDRESETSET_SRC13                           (0x00002000)          /* Write 1 sets the corresponding bit in the RSTCTL_HRDRESETSTAT_REG (and initiates */
                                                                                 /* a Hard Reset). Write 0 has no effect. */
#define RSTCTL_HRDRESETSET_SRC14                           (0x00004000)          /* Write 1 sets the corresponding bit in the RSTCTL_HRDRESETSTAT_REG (and initiates */
                                                                                 /* a Hard Reset). Write 0 has no effect. */
#define RSTCTL_HRDRESETSET_SRC15                           (0x00008000)          /* Write 1 sets the corresponding bit in the RSTCTL_HRDRESETSTAT_REG (and initiates */
                                                                                 /* a Hard Reset). Write 0 has no effect. */
#define RSTCTL_HRDRESETSET_RESERVED_1__M                   (0xffff0000)          /* Reserved. Always reads 0h */

/* RSTCTL_SFTRESETSTAT Control Bits */
#define RSTCTL_SFTRESETSTAT_SRC0                           (0x00000001)          /* If 1, indicates that SRC0 was the source of the Soft Reset */
#define RSTCTL_SFTRESETSTAT_SRC1                           (0x00000002)          /* If 1, indicates that SRC1 was the source of the Soft Reset */
#define RSTCTL_SFTRESETSTAT_SRC2                           (0x00000004)          /* If 1, indicates that SRC2 was the source of the Soft Reset */
#define RSTCTL_SFTRESETSTAT_SRC3                           (0x00000008)          /* If 1, indicates that SRC3 was the source of the Soft Reset */
#define RSTCTL_SFTRESETSTAT_SRC4                           (0x00000010)          /* If 1, indicates that SRC4 was the source of the Soft Reset */
#define RSTCTL_SFTRESETSTAT_SRC5                           (0x00000020)          /* If 1, indicates that SRC5 was the source of the Soft Reset */
#define RSTCTL_SFTRESETSTAT_SRC6                           (0x00000040)          /* If 1, indicates that SRC6 was the source of the Soft Reset */
#define RSTCTL_SFTRESETSTAT_SRC7                           (0x00000080)          /* If 1, indicates that SRC7 was the source of the Soft Reset */
#define RSTCTL_SFTRESETSTAT_SRC8                           (0x00000100)          /* If 1, indicates that SRC8 was the source of the Soft Reset */
#define RSTCTL_SFTRESETSTAT_SRC9                           (0x00000200)          /* If 1, indicates that SRC9 was the source of the Soft Reset */
#define RSTCTL_SFTRESETSTAT_SRC10                          (0x00000400)          /* If 1, indicates that SRC10 was the source of the Soft Reset */
#define RSTCTL_SFTRESETSTAT_SRC11                          (0x00000800)          /* If 1, indicates that SRC11 was the source of the Soft Reset */
#define RSTCTL_SFTRESETSTAT_SRC12                          (0x00001000)          /* If 1, indicates that SRC12 was the source of the Soft Reset */
#define RSTCTL_SFTRESETSTAT_SRC13                          (0x00002000)          /* If 1, indicates that SRC13 was the source of the Soft Reset */
#define RSTCTL_SFTRESETSTAT_SRC14                          (0x00004000)          /* If 1, indicates that SRC14 was the source of the Soft Reset */
#define RSTCTL_SFTRESETSTAT_SRC15                          (0x00008000)          /* If 1, indicates that SRC15 was the source of the Soft Reset */
#define RSTCTL_SFTRESETSTAT_RESERVED_1__M                  (0xffff0000)          /* Reserved. Always reads 0h */

/* RSTCTL_SFTRESETCLR Control Bits */
#define RSTCTL_SFTRESETCLR_SRC0                            (0x00000001)          /* Write 1 clears the corresponding bit in the RSTCTL_SFTRESETSTAT_REG. Write 0 has */
                                                                                 /* no effect. */
#define RSTCTL_SFTRESETCLR_SRC1                            (0x00000002)          /* Write 1 clears the corresponding bit in the RSTCTL_SFTRESETSTAT_REG. Write 0 has */
                                                                                 /* no effect. */
#define RSTCTL_SFTRESETCLR_SRC2                            (0x00000004)          /* Write 1 clears the corresponding bit in the RSTCTL_SFTRESETSTAT_REG. Write 0 has */
                                                                                 /* no effect. */
#define RSTCTL_SFTRESETCLR_SRC3                            (0x00000008)          /* Write 1 clears the corresponding bit in the RSTCTL_SFTRESETSTAT_REG. Write 0 has */
                                                                                 /* no effect. */
#define RSTCTL_SFTRESETCLR_SRC4                            (0x00000010)          /* Write 1 clears the corresponding bit in the RSTCTL_SFTRESETSTAT_REG. Write 0 has */
                                                                                 /* no effect. */
#define RSTCTL_SFTRESETCLR_SRC5                            (0x00000020)          /* Write 1 clears the corresponding bit in the RSTCTL_SFTRESETSTAT_REG. Write 0 has */
                                                                                 /* no effect. */
#define RSTCTL_SFTRESETCLR_SRC6                            (0x00000040)          /* Write 1 clears the corresponding bit in the RSTCTL_SFTRESETSTAT_REG. Write 0 has */
                                                                                 /* no effect. */
#define RSTCTL_SFTRESETCLR_SRC7                            (0x00000080)          /* Write 1 clears the corresponding bit in the RSTCTL_SFTRESETSTAT_REG. Write 0 has */
                                                                                 /* no effect. */
#define RSTCTL_SFTRESETCLR_SRC8                            (0x00000100)          /* Write 1 clears the corresponding bit in the RSTCTL_SFTRESETSTAT_REG. Write 0 has */
                                                                                 /* no effect. */
#define RSTCTL_SFTRESETCLR_SRC9                            (0x00000200)          /* Write 1 clears the corresponding bit in the RSTCTL_SFTRESETSTAT_REG. Write 0 has */
                                                                                 /* no effect. */
#define RSTCTL_SFTRESETCLR_SRC10                           (0x00000400)          /* Write 1 clears the corresponding bit in the RSTCTL_SFTRESETSTAT_REG. Write 0 has */
                                                                                 /* no effect. */
#define RSTCTL_SFTRESETCLR_SRC11                           (0x00000800)          /* Write 1 clears the corresponding bit in the RSTCTL_SFTRESETSTAT_REG. Write 0 has */
                                                                                 /* no effect. */
#define RSTCTL_SFTRESETCLR_SRC12                           (0x00001000)          /* Write 1 clears the corresponding bit in the RSTCTL_SFTRESETSTAT_REG. Write 0 has */
                                                                                 /* no effect. */
#define RSTCTL_SFTRESETCLR_SRC13                           (0x00002000)          /* Write 1 clears the corresponding bit in the RSTCTL_SFTRESETSTAT_REG. Write 0 has */
                                                                                 /* no effect. */
#define RSTCTL_SFTRESETCLR_SRC14                           (0x00004000)          /* Write 1 clears the corresponding bit in the RSTCTL_SFTRESETSTAT_REG. Write 0 has */
                                                                                 /* no effect. */
#define RSTCTL_SFTRESETCLR_SRC15                           (0x00008000)          /* Write 1 clears the corresponding bit in the RSTCTL_SFTRESETSTAT_REG. Write 0 has */
                                                                                 /* no effect. */
#define RSTCTL_SFTRESETCLR_RESERVED_1__M                   (0xffff0000)          /* Reserved. Always reads 0h */

/* RSTCTL_SFTRESETSET Control Bits */
#define RSTCTL_SFTRESETSET_SRC0                            (0x00000001)          /* Write 1 sets the corresponding bit in the RSTCTL_SFTRESETSTAT_REG (and initiates */
                                                                                 /* a Soft Reset). Write 0 has no effect. */
#define RSTCTL_SFTRESETSET_SRC1                            (0x00000002)          /* Write 1 sets the corresponding bit in the RSTCTL_SFTRESETSTAT_REG (and initiates */
                                                                                 /* a Soft Reset). Write 0 has no effect. */
#define RSTCTL_SFTRESETSET_SRC2                            (0x00000004)          /* Write 1 sets the corresponding bit in the RSTCTL_SFTRESETSTAT_REG (and initiates */
                                                                                 /* a Soft Reset). Write 0 has no effect. */
#define RSTCTL_SFTRESETSET_SRC3                            (0x00000008)          /* Write 1 sets the corresponding bit in the RSTCTL_SFTRESETSTAT_REG (and initiates */
                                                                                 /* a Soft Reset). Write 0 has no effect. */
#define RSTCTL_SFTRESETSET_SRC4                            (0x00000010)          /* Write 1 sets the corresponding bit in the RSTCTL_SFTRESETSTAT_REG (and initiates */
                                                                                 /* a Soft Reset). Write 0 has no effect. */
#define RSTCTL_SFTRESETSET_SRC5                            (0x00000020)          /* Write 1 sets the corresponding bit in the RSTCTL_SFTRESETSTAT_REG (and initiates */
                                                                                 /* a Soft Reset). Write 0 has no effect. */
#define RSTCTL_SFTRESETSET_SRC6                            (0x00000040)          /* Write 1 sets the corresponding bit in the RSTCTL_SFTRESETSTAT_REG (and initiates */
                                                                                 /* a Soft Reset). Write 0 has no effect. */
#define RSTCTL_SFTRESETSET_SRC7                            (0x00000080)          /* Write 1 sets the corresponding bit in the RSTCTL_SFTRESETSTAT_REG (and initiates */
                                                                                 /* a Soft Reset). Write 0 has no effect. */
#define RSTCTL_SFTRESETSET_SRC8                            (0x00000100)          /* Write 1 sets the corresponding bit in the RSTCTL_SFTRESETSTAT_REG (and initiates */
                                                                                 /* a Soft Reset). Write 0 has no effect. */
#define RSTCTL_SFTRESETSET_SRC9                            (0x00000200)          /* Write 1 sets the corresponding bit in the RSTCTL_SFTRESETSTAT_REG (and initiates */
                                                                                 /* a Soft Reset). Write 0 has no effect. */
#define RSTCTL_SFTRESETSET_SRC10                           (0x00000400)          /* Write 1 sets the corresponding bit in the RSTCTL_SFTRESETSTAT_REG (and initiates */
                                                                                 /* a Soft Reset). Write 0 has no effect. */
#define RSTCTL_SFTRESETSET_SRC11                           (0x00000800)          /* Write 1 sets the corresponding bit in the RSTCTL_SFTRESETSTAT_REG (and initiates */
                                                                                 /* a Soft Reset). Write 0 has no effect. */
#define RSTCTL_SFTRESETSET_SRC12                           (0x00001000)          /* Write 1 sets the corresponding bit in the RSTCTL_SFTRESETSTAT_REG (and initiates */
                                                                                 /* a Soft Reset). Write 0 has no effect. */
#define RSTCTL_SFTRESETSET_SRC13                           (0x00002000)          /* Write 1 sets the corresponding bit in the RSTCTL_SFTRESETSTAT_REG (and initiates */
                                                                                 /* a Soft Reset). Write 0 has no effect. */
#define RSTCTL_SFTRESETSET_SRC14                           (0x00004000)          /* Write 1 sets the corresponding bit in the RSTCTL_SFTRESETSTAT_REG (and initiates */
                                                                                 /* a Soft Reset). Write 0 has no effect. */
#define RSTCTL_SFTRESETSET_SRC15                           (0x00008000)          /* Write 1 sets the corresponding bit in the RSTCTL_SFTRESETSTAT_REG (and initiates */
                                                                                 /* a Soft Reset). Write 0 has no effect. */
#define RSTCTL_SFTRESETSET_RESERVED_1__M                   (0xffff0000)          /* Reserved. Always reads 0h */

/* RSTCTL_PSSRSTSTAT Control Bits */
#define RSTCTL_PSSRSTSTAT_SVSL                             (0x00000001)          /* Indicates if POR was caused by an SVSL trip condition in the PSS (SVSL is always */
                                                                                 /* enabled) */
#define RSTCTL_PSSRSTSTAT_SVSH                             (0x00000002)          /* Indicates if POR was caused by an SVSH trip condition int the PSS (with SVSH  */
                                                                                 /* enabled and set as supervisor) */
#define RSTCTL_PSSRSTSTAT_BGREF                            (0x00000004)          /* Indicates if POR was caused by a BGREF not okay condition in the PSS */
#define RSTCTL_PSSRSTSTAT_VCCDET                           (0x00000008)          /* Indicates if POR was caused by a VccDET trip condition in the PSS */
#define RSTCTL_PSSRSTSTAT_RESERVED_1__M                    (0xfffffff0)          /* Reserved. Always reads 0h */

/* RSTCTL_PSSRSTCLR Control Bits */
#define RSTCTL_PSSRSTCLR_CLR                               (0x00000001)          /* Write 1 clears all PSS Reset Flags in the RSTCTL_PSSRSTSTAT_REG */
#define RSTCTL_PSSRSTCLR_RESERVED_1__M                     (0xfffffffe)          /* Reserved. Always reads 0h */

/* RSTCTL_PCMRSTSTAT Control Bits */
#define RSTCTL_PCMRSTSTAT_SD0                              (0x00000001)          /* Indicates if POR was caused by PCM due to an exit from the SD0 mode */
#define RSTCTL_PCMRSTSTAT_SDC                              (0x00000002)          /* Indicates if POR was caused by PCM due to an exit from the SDC mode */
#define RSTCTL_PCMRSTSTAT_RESERVED_1__M                    (0xfffffffc)          /* Reserved. Always reads 0h */

/* RSTCTL_PCMRSTCLR Control Bits */
#define RSTCTL_PCMRSTCLR_CLR                               (0x00000001)          /* Write 1 clears all PCM Reset Flags in the RSTCTL_PCMRSTSTAT_REG */
#define RSTCTL_PCMRSTCLR_RESERVED_1__M                     (0xfffffffe)          /* Reserved. Always reads 0h */

/* RSTCTL_PINRSTSTAT Control Bits */
#define RSTCTL_PINRSTSTAT_RSTNMI                           (0x00000001)          /* Indicates if POR was caused by RSTNMI pin based reset event in the device. This  */
                                                                                 /* bit comes up by default as 0 in the case of cold power up. This is only set if  */
                                                                                 /* there is a POR caused due to the RSTNMI pin. */
#define RSTCTL_PINRSTSTAT_RESERVED_1__M                    (0xfffffffe)          /* Reserved. Always reads 0h */

/* RSTCTL_PINRSTCLR Control Bits */
#define RSTCTL_PINRSTCLR_CLR                               (0x00000001)          /* Write 1 clears the RSTNMI Pin Reset Flag in RSTCTL_PINRSTSTAT_REG */
#define RSTCTL_PINRSTCLR_RESERVED_1__M                     (0xfffffffe)          /* Reserved. Always reads 0h */

/* RSTCTL_RBTRSTSTAT Control Bits */
#define RSTCTL_RBTRSTSTAT_REBOOT                           (0x00000001)          /* Indicates if Reboot reset was caused by the CPU_SYS module. */
#define RSTCTL_RBTRSTSTAT_RESERVED_1__M                    (0xfffffffe)          /* Reserved. Always reads 0h */

/* RSTCTL_RBTRSTCLR Control Bits */
#define RSTCTL_RBTRSTCLR_CLR                               (0x00000001)          /* Write 1 clears the Reboot Reset Flag in RSTCTL_RBTRSTSTAT_REG */
#define RSTCTL_RBTRSTCLR_RESERVED_1__M                     (0xfffffffe)          /* Reserved. Always reads 0h */


//*****************************************************************************
// RTC_C Registers
//*****************************************************************************
typedef struct {
  union {    /* RTC_C_CTL0 Register */
    __IO uint16_t reg;
    struct { /* RTC_C_CTL0 Control Bits */
      __IO uint16_t RDYIFG              	:	1;  /* Real-time clock ready interrupt flag. */
      __IO uint16_t AIFG                	:	1;  /* Real-time clock alarm interrupt flag. This interrupt can be used as DSL/SD0 wake */
                                                /* -up event */
      __IO uint16_t TEVIFG              	:	1;  /* Real-time clock time event interrupt flag. This interrupt can be used as DSL/SD0 */
                                                /* wake-up event */
      __IO uint16_t OFIFG               	:	1;  /* 32-kHz crystal oscillator fault interrupt flag. This interrupt can be used as  */
                                                /* DSL/SD0 wake-up event. It indicates a clock failure during backup operation */
      __IO uint16_t RDYIE               	:	1;  /* Real-time clock ready interrupt enable */
      __IO uint16_t AIE                 	:	1;  /* Real-time clock alarm interrupt enable. This interrupt can be used as DSL/SD0  */
                                                /* wake-up event. */
      __IO uint16_t TEVIE               	:	1;  /* Real-time clock event interrupt enable. This interrupt can be used as DSL/SD0  */
                                                /* wake-up event. */
      __IO uint16_t OFIE                	:	1;  /* 32-kHz crystal oscillator fault interrupt flag. This interrupt can be used as  */
                                                /* DSL/SD0 wake-up event. */
      __IO uint16_t KEY                 	:	8;  /* Real-time clock key. This register should be written with A5h to unlock RTC. Any */
                                                /* write with value other than A5h will lock the module. Read from this register  */
                                                /* always returns 96h. */

    } bit;
  } CTL0;
  uint8_t RESERVED0[1];
  union {    /* RTC_C_CTL1 Register */
    __IO uint8_t reg;
    struct { /* RTC_C_CTL1 Control Bits */
      __IO uint16_t TEV                 	:	2;  /* Real-time clock time event 00b = Minute changed 01b = Hour changed 10b = Every  */
                                                /* day at midnight (00:00) 11b = Every day at noon (12:00) */
      __IO uint16_t SSEL                	:	2;  /* Real-time clock source select. */
      __I  uint16_t RDY                 	:	1;  /* Real-time clock ready */
      __I  uint16_t MODE                	:	1;  /* Real-time clock mode. */
      __IO uint16_t HOLD                	:	1;  /* Real-time clock hold */
      __IO uint16_t BCD                 	:	1;  /* Real-time clock BCD select. Selects BCD counting for real-time clock. */

    } bit;
  } CTL1;
  union {    /* RTC_C_CTL3 Register */
    __IO uint8_t reg;
    struct { /* RTC_C_CTL3 Control Bits */
      __IO uint16_t CALF                	:	2;  /* Real-time clock calibration frequency. Selects frequency output to RTCCLK pin  */
                                                /* for calibration measurement. The corresponding port must be configured for the  */
                                                /* peripheral module function.  */
      __I  uint16_t RESERVED1           	:	6;  /* Reserved */

    } bit;
  } CTL3;
  union {    /* RTC_C_OCAL Register */
    __IO uint16_t reg;
    struct { /* RTC_C_OCAL Control Bits */
      __IO uint16_t OCAL                	:	8;  /* Real-time clock offset error calibration. Each LSB represents approximately + */
                                                /* 1ppm (RTCOCALS = 1) or -1ppm (RTCOCALS = 0) adjustment in frequency. Maximum  */
                                                /* effective calibration value is +/-240ppm. Excess values written above +/-240ppm  */
                                                /* are ignored by hardware. */
      __I  uint16_t RESERVED            	:	7;  /* Reserved */
      __IO uint16_t OCALS               	:	1;  /* Real-time clock offset error calibration sign. This bit decides the sign of  */
                                                /* offset error calibration */

    } bit;
  } OCAL;
  union {    /* RTC_C_TCMP Register */
    __IO uint16_t reg;
    struct { /* RTC_C_TCMP Control Bits */
      __IO uint16_t TCMP                	:	8;  /* Real-time clock temperature compensation. Value written into this register is  */
                                                /* used for temperature compensation of RTC. Each LSB represents approximately + */
                                                /* 1ppm (RTCTCMPS = 1) or -1ppm (RTCTCMPS = 0) adjustment in frequency. Maximum  */
                                                /* effective calibration value is +/-240ppm. Excess values written above +/-240ppm  */
                                                /* are ignored by hardware. */
      __I  uint16_t RESERVED1           	:	5;  /* Reserved */
      __I  uint16_t TCOK                	:	1;  /* Real-time clock temperature compensation write OK. This is a read-only bit that  */
                                                /* indicates if the write to RTCTCMP is successful or not. */
      __I  uint16_t TCRDY               	:	1;  /* Real-time clock temperature compensation ready. This is a read only bit that  */
                                                /* indicates when the RTCTCMPx can be written. Write to RTCTCMPx should be avoided  */
                                                /* when RTCTCRDY is reset. */
      __IO uint16_t TCMPS               	:	1;  /* Real-time clock temperature compensation sign. This bit decides the sign of  */
                                                /* temperature compensation. */

    } bit;
  } TCMP;
  union {    /* RTC_C_PS0CTL Register */
    __IO uint16_t reg;
    struct { /* RTC_C_PS0CTL Control Bits */
      __IO uint16_t RT0PSIFG            	:	1;  /* Prescale timer 0 interrupt flag */
      __IO uint16_t RT0PSIE             	:	1;  /* Prescale timer 0 interrupt enable */
      __IO uint16_t RT0IP               	:	3;  /* Prescale timer 0 interrupt interval */
      __I  uint16_t RESERVED1           	:	11;  /* Reserved */

    } bit;
  } PS0CTL;
  union {    /* RTC_C_PS1CTL Register */
    __IO uint16_t reg;
    struct { /* RTC_C_PS1CTL Control Bits */
      __IO uint16_t RT1PSIFG            	:	1;  /* Prescale timer 1 interrupt flag */
      __IO uint16_t RT1PSIE             	:	1;  /* Prescale timer 1 interrupt enable */
      __IO uint16_t RT1IP               	:	3;  /* Prescale timer 1 interrupt interval */
      __I  uint16_t RESERVED1           	:	11;  /* Reserved */

    } bit;
  } PS1CTL;
  uint8_t RESERVED1[1];
  uint8_t PS0; /* Real-Time Prescale Timer 0 Counter */
  uint8_t PS1; /* Real-Time Prescale Timer 1 Counter */
  uint16_t IV; /* Real Time Clock Interrupt Vector */
  uint8_t RESERVED2[1];
  union {    /* RTC_C_SEC Register */
    __IO uint8_t reg;
    struct { /* RTC_C_SEC Control Bits */
      __IO uint16_t SECONDS             	:	6;  /* Seconds (0 to 59) */

    } bit;
  } SEC;
  union {    /* RTC_C_MIN Register */
    __IO uint8_t reg;
    struct { /* RTC_C_MIN Control Bits */
      __IO uint16_t MINUTES             	:	6;  /* Minutes (0 to 59) */

    } bit;
  } MIN;
  union {    /* RTC_C_HOUR Register */
    __IO uint8_t reg;
    struct { /* RTC_C_HOUR Control Bits */
      __IO uint16_t HOURS               	:	5;  /* Hours (0 to 23) */

    } bit;
  } HOUR;
  union {    /* RTC_C_DOW Register */
    __IO uint8_t reg;
    struct { /* RTC_C_DOW Control Bits */
      __IO uint16_t DAY_OF_WEEK         	:	3;  /* Day of week (0 to 6) */

    } bit;
  } DOW;
  union {    /* RTC_C_DAY Register */
    __IO uint8_t reg;
    struct { /* RTC_C_DAY Control Bits */
      __IO uint16_t DAY_OF_MONTH        	:	5;  /* Day of month (1 to 28, 29, 30, 31) */

    } bit;
  } DAY;
  union {    /* RTC_C_MON Register */
    __IO uint8_t reg;
    struct { /* RTC_C_MON Control Bits */
      __IO uint16_t MONTH               	:	4;  /* Month (1 to 12) */

    } bit;
  } MON;
  union {    /* RTC_C_YEAR Register */
    __IO uint16_t reg;
    struct { /* RTC_C_YEAR Control Bits */
      __IO uint16_t YEAR_LOW_BYTE       	:	8;  /* Year low byte. Valid values for Year are 0 to 4095. */
      __IO uint16_t YEAR_HIGH_BYTE      	:	4;  /* Year high byte. Valid values for Year are 0 to 4095. */

    } bit;
  } YEAR;
  uint8_t RESERVED3[1];
  union {    /* RTC_C_AMIN Register */
    __IO uint8_t reg;
    struct { /* RTC_C_AMIN Control Bits */
      __IO uint16_t MINUTES             	:	6;  /* Minutes (0 to 59) */
      __IO uint16_t AE                  	:	1;  /* Alarm enable Alarm enable */

    } bit;
  } AMIN;
  union {    /* RTC_C_AHOUR Register */
    __IO uint8_t reg;
    struct { /* RTC_C_AHOUR Control Bits */
      __IO uint16_t HOURS               	:	5;  /* Hours (0 to 23) */
      __IO uint16_t AE                  	:	1;  /* Alarm enable Alarm enable */

    } bit;
  } AHOUR;
  union {    /* RTC_C_ADOW Register */
    __IO uint8_t reg;
    struct { /* RTC_C_ADOW Control Bits */
      __IO uint16_t DAY_OF_WEEK         	:	3;  /* Day of week (0 to 6) */
      __IO uint16_t AE                  	:	1;  /* Alarm enable Alarm enable */

    } bit;
  } ADOW;
  union {    /* RTC_C_ADAY Register */
    __IO uint8_t reg;
    struct { /* RTC_C_ADAY Control Bits */
      __IO uint16_t DAY_OF_MONTH        	:	5;  /* Day of month (1 to 28, 29, 30, 31) */
      __IO uint16_t AE                  	:	1;  /* Alarm enable Alarm enable */

    } bit;
  } ADAY;
  uint16_t BIN2BCD; /* Binary-to-BCD conversion register */
  uint16_t BCD2BIN; /* BCD-to-binary conversion register */
} RTC_C_Type;


/* RTC_C_CTL0 Control Bits */
#define RTC_C_CTL0_RDYIFG                                  (0x0001)              /* Real-time clock ready interrupt flag. */
#define RTC_C_CTL0_AIFG                                    (0x0002)              /* Real-time clock alarm interrupt flag. This interrupt can be used as DSL/SD0 wake */
                                                                                 /* -up event */
#define RTC_C_CTL0_TEVIFG                                  (0x0004)              /* Real-time clock time event interrupt flag. This interrupt can be used as DSL/SD0 */
                                                                                 /* wake-up event */
#define RTC_C_CTL0_OFIFG                                   (0x0008)              /* 32-kHz crystal oscillator fault interrupt flag. This interrupt can be used as  */
                                                                                 /* DSL/SD0 wake-up event. It indicates a clock failure during backup operation */
#define RTC_C_CTL0_RDYIE                                   (0x0010)              /* Real-time clock ready interrupt enable */
#define RTC_C_CTL0_AIE                                     (0x0020)              /* Real-time clock alarm interrupt enable. This interrupt can be used as DSL/SD0  */
                                                                                 /* wake-up event. */
#define RTC_C_CTL0_TEVIE                                   (0x0040)              /* Real-time clock event interrupt enable. This interrupt can be used as DSL/SD0  */
                                                                                 /* wake-up event. */
#define RTC_C_CTL0_OFIE                                    (0x0080)              /* 32-kHz crystal oscillator fault interrupt flag. This interrupt can be used as  */
                                                                                 /* DSL/SD0 wake-up event. */
#define RTC_C_CTL0_KEY__M                                  (0xff00)              /* Real-time clock key. This register should be written with A5h to unlock RTC. Any */
                                                                                 /* write with value other than A5h will lock the module. Read from this register  */
                                                                                 /* always returns 96h. */

/* RTC_C_CTL1 Control Bits */
#define RTC_C_CTL1_TEV__M                                  (0x0003)              /* Real-time clock time event 00b = Minute changed 01b = Hour changed 10b = Every  */
                                                                                 /* day at midnight (00:00) 11b = Every day at noon (12:00) */
#define RTC_C_CTL1_SSEL__M                                 (0x000c)              /* Real-time clock source select. */
#define RTC_C_CTL1_SSEL__0                                 (0x0000)              /* BCLK */
#define RTC_C_CTL1_SSEL__0_BCLK                            (0x0000)              /* BCLK */
#define RTC_C_CTL1_SSEL__1                                 (0x0004)              /* Reserved. Defaults to BCLK. */
#define RTC_C_CTL1_SSEL__2                                 (0x0008)              /* Reserved. Defaults to BCLK. */
#define RTC_C_CTL1_SSEL__3                                 (0x000c)              /* Reserved. Defaults to BCLK */
#define RTC_C_CTL1_RDY                                     (0x0010)              /* Real-time clock ready */
#define RTC_C_CTL1_MODE                                    (0x0020)              /* Real-time clock mode. */
#define RTC_C_CTL1_HOLD                                    (0x0040)              /* Real-time clock hold */
#define RTC_C_CTL1_BCD                                     (0x0080)              /* Real-time clock BCD select. Selects BCD counting for real-time clock. */

/* RTC_C_CTL3 Control Bits */
#define RTC_C_CTL3_CALF__M                                 (0x0003)              /* Real-time clock calibration frequency. Selects frequency output to RTCCLK pin  */
                                                                                 /* for calibration measurement. The corresponding port must be configured for the  */
                                                                                 /* peripheral module function.  */
#define RTC_C_CTL3_CALF__0                                 (0x0000)              /* No frequency output to RTCCLK pin */
#define RTC_C_CTL3_CALF__1                                 (0x0001)              /* 512 Hz */
#define RTC_C_CTL3_CALF__1_512_HZ                          (0x0001)              /* 512 Hz */
#define RTC_C_CTL3_CALF__2                                 (0x0002)              /* 256 Hz */
#define RTC_C_CTL3_CALF__2_256_HZ                          (0x0002)              /* 256 Hz */
#define RTC_C_CTL3_CALF__3                                 (0x0003)              /* 1 Hz */
#define RTC_C_CTL3_CALF__3_1_HZ                            (0x0003)              /* 1 Hz */
#define RTC_C_CTL3_RESERVED1__M                            (0x00fc)              /*  */

/* RTC_C_OCAL Control Bits */
#define RTC_C_OCAL_OCAL__M                                 (0x00ff)              /* Real-time clock offset error calibration. Each LSB represents approximately + */
                                                                                 /* 1ppm (RTCOCALS = 1) or -1ppm (RTCOCALS = 0) adjustment in frequency. Maximum  */
                                                                                 /* effective calibration value is +/-240ppm. Excess values written above +/-240ppm  */
                                                                                 /* are ignored by hardware. */
#define RTC_C_OCAL_RESERVED__M                             (0x7f00)              /*  */
#define RTC_C_OCAL_OCALS                                   (0x8000)              /* Real-time clock offset error calibration sign. This bit decides the sign of  */
                                                                                 /* offset error calibration */

/* RTC_C_TCMP Control Bits */
#define RTC_C_TCMP_TCMP__M                                 (0x00ff)              /* Real-time clock temperature compensation. Value written into this register is  */
                                                                                 /* used for temperature compensation of RTC. Each LSB represents approximately + */
                                                                                 /* 1ppm (RTCTCMPS = 1) or -1ppm (RTCTCMPS = 0) adjustment in frequency. Maximum  */
                                                                                 /* effective calibration value is +/-240ppm. Excess values written above +/-240ppm  */
                                                                                 /* are ignored by hardware. */
#define RTC_C_TCMP_RESERVED1__M                            (0x1f00)              /*  */
#define RTC_C_TCMP_TCOK                                    (0x2000)              /* Real-time clock temperature compensation write OK. This is a read-only bit that  */
                                                                                 /* indicates if the write to RTCTCMP is successful or not. */
#define RTC_C_TCMP_TCRDY                                   (0x4000)              /* Real-time clock temperature compensation ready. This is a read only bit that  */
                                                                                 /* indicates when the RTCTCMPx can be written. Write to RTCTCMPx should be avoided  */
                                                                                 /* when RTCTCRDY is reset. */
#define RTC_C_TCMP_TCMPS                                   (0x8000)              /* Real-time clock temperature compensation sign. This bit decides the sign of  */
                                                                                 /* temperature compensation. */

/* RTC_C_PS0CTL Control Bits */
#define RTC_C_PS0CTL_RT0PSIFG                              (0x0001)              /* Prescale timer 0 interrupt flag */
#define RTC_C_PS0CTL_RT0PSIE                               (0x0002)              /* Prescale timer 0 interrupt enable */
#define RTC_C_PS0CTL_RT0IP__M                              (0x001c)              /* Prescale timer 0 interrupt interval */
#define RTC_C_PS0CTL_RT0IP__0                              (0x0000)              /* Divide by 2 */
#define RTC_C_PS0CTL_RT0IP__0_DIVIDE_BY_2                  (0x0000)              /* Divide by 2 */
#define RTC_C_PS0CTL_RT0IP__1                              (0x0004)              /* Divide by 4 */
#define RTC_C_PS0CTL_RT0IP__1_DIVIDE_BY_4                  (0x0004)              /* Divide by 4 */
#define RTC_C_PS0CTL_RT0IP__2                              (0x0008)              /* Divide by 8 */
#define RTC_C_PS0CTL_RT0IP__2_DIVIDE_BY_8                  (0x0008)              /* Divide by 8 */
#define RTC_C_PS0CTL_RT0IP__3                              (0x000c)              /* Divide by 16 */
#define RTC_C_PS0CTL_RT0IP__3_DIVIDE_BY_16                 (0x000c)              /* Divide by 16 */
#define RTC_C_PS0CTL_RT0IP__4                              (0x0010)              /* Divide by 32 */
#define RTC_C_PS0CTL_RT0IP__4_DIVIDE_BY_32                 (0x0010)              /* Divide by 32 */
#define RTC_C_PS0CTL_RT0IP__5                              (0x0014)              /* Divide by 64 */
#define RTC_C_PS0CTL_RT0IP__5_DIVIDE_BY_64                 (0x0014)              /* Divide by 64 */
#define RTC_C_PS0CTL_RT0IP__6                              (0x0018)              /* Divide by 128 */
#define RTC_C_PS0CTL_RT0IP__6_DIVIDE_BY_128                (0x0018)              /* Divide by 128 */
#define RTC_C_PS0CTL_RT0IP__7                              (0x001c)              /* Divide by 256 */
#define RTC_C_PS0CTL_RT0IP__7_DIVIDE_BY_256                (0x001c)              /* Divide by 256 */
#define RTC_C_PS0CTL_RESERVED1__M                          (0xffe0)              /*  */

/* RTC_C_PS1CTL Control Bits */
#define RTC_C_PS1CTL_RT1PSIFG                              (0x0001)              /* Prescale timer 1 interrupt flag */
#define RTC_C_PS1CTL_RT1PSIE                               (0x0002)              /* Prescale timer 1 interrupt enable */
#define RTC_C_PS1CTL_RT1IP__M                              (0x001c)              /* Prescale timer 1 interrupt interval */
#define RTC_C_PS1CTL_RT1IP__0                              (0x0000)              /* Divide by 2 */
#define RTC_C_PS1CTL_RT1IP__0_DIVIDE_BY_2                  (0x0000)              /* Divide by 2 */
#define RTC_C_PS1CTL_RT1IP__1                              (0x0004)              /* Divide by 4 */
#define RTC_C_PS1CTL_RT1IP__1_DIVIDE_BY_4                  (0x0004)              /* Divide by 4 */
#define RTC_C_PS1CTL_RT1IP__2                              (0x0008)              /* Divide by 8 */
#define RTC_C_PS1CTL_RT1IP__2_DIVIDE_BY_8                  (0x0008)              /* Divide by 8 */
#define RTC_C_PS1CTL_RT1IP__3                              (0x000c)              /* Divide by 16 */
#define RTC_C_PS1CTL_RT1IP__3_DIVIDE_BY_16                 (0x000c)              /* Divide by 16 */
#define RTC_C_PS1CTL_RT1IP__4                              (0x0010)              /* Divide by 32 */
#define RTC_C_PS1CTL_RT1IP__4_DIVIDE_BY_32                 (0x0010)              /* Divide by 32 */
#define RTC_C_PS1CTL_RT1IP__5                              (0x0014)              /* Divide by 64 */
#define RTC_C_PS1CTL_RT1IP__5_DIVIDE_BY_64                 (0x0014)              /* Divide by 64 */
#define RTC_C_PS1CTL_RT1IP__6                              (0x0018)              /* Divide by 128 */
#define RTC_C_PS1CTL_RT1IP__6_DIVIDE_BY_128                (0x0018)              /* Divide by 128 */
#define RTC_C_PS1CTL_RT1IP__7                              (0x001c)              /* Divide by 256 */
#define RTC_C_PS1CTL_RT1IP__7_DIVIDE_BY_256                (0x001c)              /* Divide by 256 */
#define RTC_C_PS1CTL_RESERVED1__M                          (0xffe0)              /*  */

/* RTC_C_SEC Control Bits */
#define RTC_C_SEC_SECONDS__M                               (0x003f)              /* Seconds (0 to 59) */

/* RTC_C_MIN Control Bits */
#define RTC_C_MIN_MINUTES__M                               (0x003f)              /* Minutes (0 to 59) */

/* RTC_C_HOUR Control Bits */
#define RTC_C_HOUR_HOURS__M                                (0x001f)              /* Hours (0 to 23) */

/* RTC_C_DOW Control Bits */
#define RTC_C_DOW_DAY_OF_WEEK__M                           (0x0007)              /* Day of week (0 to 6) */

/* RTC_C_DAY Control Bits */
#define RTC_C_DAY_DAY_OF_MONTH__M                          (0x001f)              /* Day of month (1 to 28, 29, 30, 31) */

/* RTC_C_MON Control Bits */
#define RTC_C_MON_MONTH__M                                 (0x000f)              /* Month (1 to 12) */

/* RTC_C_YEAR Control Bits */
#define RTC_C_YEAR_YEAR_LOW_BYTE__M                        (0x00ff)              /* Year low byte. Valid values for Year are 0 to 4095. */
#define RTC_C_YEAR_YEAR_HIGH_BYTE__M                       (0x0f00)              /* Year high byte. Valid values for Year are 0 to 4095. */

/* RTC_C_AMIN Control Bits */
#define RTC_C_AMIN_MINUTES__M                              (0x003f)              /* Minutes (0 to 59) */
#define RTC_C_AMIN_AE                                      (0x0080)              /* Alarm enable Alarm enable */

/* RTC_C_AHOUR Control Bits */
#define RTC_C_AHOUR_HOURS__M                               (0x001f)              /* Hours (0 to 23) */
#define RTC_C_AHOUR_AE                                     (0x0080)              /* Alarm enable Alarm enable */

/* RTC_C_ADOW Control Bits */
#define RTC_C_ADOW_DAY_OF_WEEK__M                          (0x0007)              /* Day of week (0 to 6) */
#define RTC_C_ADOW_AE                                      (0x0080)              /* Alarm enable Alarm enable */

/* RTC_C_ADAY Control Bits */
#define RTC_C_ADAY_DAY_OF_MONTH__M                         (0x001f)              /* Day of month (1 to 28, 29, 30, 31) */
#define RTC_C_ADAY_AE                                      (0x0080)              /* Alarm enable Alarm enable */


//*****************************************************************************
// SCS Registers
//*****************************************************************************
typedef struct {
  uint8_t RESERVED0[4];
  union {    /* SCS_ICTR Register */
    __I uint32_t reg;
    struct { /* SCS_ICTR Control Bits */
      __I  uint32_t INTLINESNUM         	:	5;  /* Total number of interrupt lines in groups of 32. */

    } bit;
  } ICTR;
  union {    /* SCS_ACTLR Register */
    __IO uint32_t reg;
    struct { /* SCS_ACTLR Control Bits */
      __IO uint32_t DISMCYCINT          	:	1;  /* Disables interruption of multi-cycle instructions. This increases the interrupt  */
                                                /* latency of the processor becuase LDM/STM completes before interrupt stacking  */
                                                /* occurs. */
      __IO uint32_t DISDEFWBUF          	:	1;  /* Disables write buffer us during default memorty map accesses. This causes all  */
                                                /* bus faults to be precise bus faults but decreases the performance of the  */
                                                /* processor because the stores to memory have to complete before the next  */
                                                /* instruction can be executed. */
      __IO uint32_t DISFOLD             	:	1;  /* Disables IT folding. */

    } bit;
  } ACTLR;
  uint8_t RESERVED1[4];
  union {    /* SCS_STCSR Register */
    __IO uint32_t reg;
    struct { /* SCS_STCSR Control Bits */
      __IO uint32_t ENABLE              	:	1;  /* Enable SysTick counter */
      __IO uint32_t TICKINT             	:	1;  /*  */
      __I  uint32_t CLKSOURCE           	:	1;  /* Clock source. If no reference clock is provided, it is held at 1 and so gives  */
                                                /* the same time as the core clock. The core clock must be at least 2.5 times  */
                                                /* faster than the reference clock. If it is not, the count values are  */
                                                /* Unpredictable. */
      __I  uint32_t COUNTFLAG           	:	1;  /* Returns 1 if timer counted to 0 since last time this was read. Clears on read by */
                                                /* application of any part of the SysTick Control and Status Register. If read by  */
                                                /* the debugger using the DAP, this bit is cleared on read-only if the MasterType  */
                                                /* bit in the AHB-AP Control Register is set to 0. Otherwise, the COUNTFLAG bit is  */
                                                /* not changed by the debugger read. */

    } bit;
  } STCSR;
  union {    /* SCS_STRVR Register */
    __IO uint32_t reg;
    struct { /* SCS_STRVR Control Bits */
      __IO uint32_t RELOAD              	:	24;  /* Value to load into the SysTick Current Value Register when the counter reaches 0 */
                                                /* . */

    } bit;
  } STRVR;
  union {    /* SCS_STCVR Register */
    __IO uint32_t reg;
    struct { /* SCS_STCVR Control Bits */
      __IO uint32_t CURRENT             	:	24;  /* Current value at the time the register is accessed. No read-modify-write  */
                                                /* protection is provided, so change with care. Writing to it with any value clears */
                                                /* the register to 0. Clearing this register also clears the COUNTFLAG bit of the  */
                                                /* SysTick Control and Status Register. */

    } bit;
  } STCVR;
  union {    /* SCS_STCR Register */
    __I uint32_t reg;
    struct { /* SCS_STCR Control Bits */
      __I  uint32_t TENMS               	:	24;  /* Reads as zero. Indicates calibration value is not known. */
      __I  uint32_t SKEW                	:	1;  /* Reads as one. The calibration value is not exactly 10ms because of clock  */
                                                /* frequency. This could affect its suitability as a software real time clock. */
      __I  uint32_t NOREF               	:	1;  /* Reads as one. Indicates that no separate reference clock is provided. */

    } bit;
  } STCR;
  uint8_t RESERVED2[224];
  uint32_t NVIC_ISER0; /* Irq 0 to 31 Set Enable Register. Use the Interrupt Set-Enable Registers to  */
                                                                                 /* enable interrupts and determine which interrupts are currently enabled. */
  uint32_t NVIC_ISER1; /* Irq 32 to 63 Set Enable Register. Use the Interrupt Set-Enable Registers to  */
                                                                                 /* enable interrupts and determine which interrupts are currently enabled. */
  uint8_t RESERVED3[120];
  uint32_t NVIC_ICER0; /* Irq 0 to 31 Clear Enable Register. Use the Interrupt Clear-Enable Registers to  */
                                                                                 /* disable interrupts and determine which interrupts are currently enabled. */
  uint32_t NVIC_ICER1; /* Irq 32 to 63 Clear Enable Register. Use the Interrupt Clear-Enable Registers to  */
                                                                                 /* disable interrupts and determine which interrupts are currently enabled. */
  uint8_t RESERVED4[120];
  uint32_t NVIC_ISPR0; /* Irq 0 to 31 Set Pending Register. Use the Interrupt Set-Pending Registers to  */
                                                                                 /* force interrupts into the pending state and determine which interrupts are  */
                                                                                 /* currently pending */
  uint32_t NVIC_ISPR1; /* Irq 32 to 63 Set Pending Register. Use the Interrupt Set-Pending Registers to  */
                                                                                 /* force interrupts into the pending state and determine which interrupts are  */
                                                                                 /* currently pending */
  uint8_t RESERVED5[120];
  uint32_t NVIC_ICPR0; /* Irq 0 to 31 Clear Pending Register. Use the Interrupt Clear-Pending Registers to */
                                                                                 /* clear pending interrupts and determine which interrupts are currently pending. */
  uint32_t NVIC_ICPR1; /* Irq 32 to 63 Clear Pending Register. Use the Interrupt Clear-Pending Registers  */
                                                                                 /* to clear pending interrupts and determine which interrupts are currently pending */
                                                                                 /* . */
  uint8_t RESERVED6[120];
  uint32_t NVIC_IABR0; /* Irq 0 to 31 Active Bit Register. Read the Active Bit Registers to determine  */
                                                                                 /* which interrupts are active. Each flag in the register corresponds to one  */
                                                                                 /* interrupt. */
  uint32_t NVIC_IABR1; /* Irq 32 to 63 Active Bit Register. Read the Active Bit Registers to determine  */
                                                                                 /* which interrupts are active. Each flag in the register corresponds to one  */
                                                                                 /* interrupt. */
  uint8_t RESERVED7[248];
  union {    /* SCS_NVIC_IPR0 Register */
    __IO uint32_t reg;
    struct { /* SCS_NVIC_IPR0 Control Bits */
      __IO uint32_t PRI_0               	:	8;  /* Priority of interrupt 0 */
      __IO uint32_t PRI_1               	:	8;  /* Priority of interrupt 1 */
      __IO uint32_t PRI_2               	:	8;  /* Priority of interrupt 2 */
      __IO uint32_t PRI_3               	:	8;  /* Priority of interrupt 3 */

    } bit;
  } NVIC_IPR0;
  union {    /* SCS_NVIC_IPR1 Register */
    __IO uint32_t reg;
    struct { /* SCS_NVIC_IPR1 Control Bits */
      __IO uint32_t PRI_4               	:	8;  /* Priority of interrupt 4 */
      __IO uint32_t PRI_5               	:	8;  /* Priority of interrupt 5 */
      __IO uint32_t PRI_6               	:	8;  /* Priority of interrupt 6 */
      __IO uint32_t PRI_7               	:	8;  /* Priority of interrupt 7 */

    } bit;
  } NVIC_IPR1;
  union {    /* SCS_NVIC_IPR2 Register */
    __IO uint32_t reg;
    struct { /* SCS_NVIC_IPR2 Control Bits */
      __IO uint32_t PRI_8               	:	8;  /* Priority of interrupt 8 */
      __IO uint32_t PRI_9               	:	8;  /* Priority of interrupt 9 */
      __IO uint32_t PRI_10              	:	8;  /* Priority of interrupt 10 */
      __IO uint32_t PRI_11              	:	8;  /* Priority of interrupt 11 */

    } bit;
  } NVIC_IPR2;
  union {    /* SCS_NVIC_IPR3 Register */
    __IO uint32_t reg;
    struct { /* SCS_NVIC_IPR3 Control Bits */
      __IO uint32_t PRI_12              	:	8;  /* Priority of interrupt 12 */
      __IO uint32_t PRI_13              	:	8;  /* Priority of interrupt 13 */
      __IO uint32_t PRI_14              	:	8;  /* Priority of interrupt 14 */
      __IO uint32_t PRI_15              	:	8;  /* Priority of interrupt 15 */

    } bit;
  } NVIC_IPR3;
  union {    /* SCS_NVIC_IPR4 Register */
    __IO uint32_t reg;
    struct { /* SCS_NVIC_IPR4 Control Bits */
      __IO uint32_t PRI_16              	:	8;  /* Priority of interrupt 16 */
      __IO uint32_t PRI_17              	:	8;  /* Priority of interrupt 17 */
      __IO uint32_t PRI_18              	:	8;  /* Priority of interrupt 18 */
      __IO uint32_t PRI_19              	:	8;  /* Priority of interrupt 19 */

    } bit;
  } NVIC_IPR4;
  union {    /* SCS_NVIC_IPR5 Register */
    __IO uint32_t reg;
    struct { /* SCS_NVIC_IPR5 Control Bits */
      __IO uint32_t PRI_20              	:	8;  /* Priority of interrupt 20 */
      __IO uint32_t PRI_21              	:	8;  /* Priority of interrupt 21 */
      __IO uint32_t PRI_22              	:	8;  /* Priority of interrupt 22 */
      __IO uint32_t PRI_23              	:	8;  /* Priority of interrupt 23 */

    } bit;
  } NVIC_IPR5;
  union {    /* SCS_NVIC_IPR6 Register */
    __IO uint32_t reg;
    struct { /* SCS_NVIC_IPR6 Control Bits */
      __IO uint32_t PRI_24              	:	8;  /* Priority of interrupt 24 */
      __IO uint32_t PRI_25              	:	8;  /* Priority of interrupt 25 */
      __IO uint32_t PRI_26              	:	8;  /* Priority of interrupt 26 */
      __IO uint32_t PRI_27              	:	8;  /* Priority of interrupt 27 */

    } bit;
  } NVIC_IPR6;
  union {    /* SCS_NVIC_IPR7 Register */
    __IO uint32_t reg;
    struct { /* SCS_NVIC_IPR7 Control Bits */
      __IO uint32_t PRI_28              	:	8;  /* Priority of interrupt 28 */
      __IO uint32_t PRI_29              	:	8;  /* Priority of interrupt 29 */
      __IO uint32_t PRI_30              	:	8;  /* Priority of interrupt 30 */
      __IO uint32_t PRI_31              	:	8;  /* Priority of interrupt 31 */

    } bit;
  } NVIC_IPR7;
  union {    /* SCS_NVIC_IPR8 Register */
    __IO uint32_t reg;
    struct { /* SCS_NVIC_IPR8 Control Bits */
      __IO uint32_t PRI_32              	:	8;  /* Priority of interrupt 32 */
      __IO uint32_t PRI_33              	:	8;  /* Priority of interrupt 33 */
      __IO uint32_t PRI_34              	:	8;  /* Priority of interrupt 34 */
      __IO uint32_t PRI_35              	:	8;  /* Priority of interrupt 35 */

    } bit;
  } NVIC_IPR8;
  union {    /* SCS_NVIC_IPR9 Register */
    __IO uint32_t reg;
    struct { /* SCS_NVIC_IPR9 Control Bits */
      __IO uint32_t PRI_36              	:	8;  /* Priority of interrupt 36 */
      __IO uint32_t PRI_37              	:	8;  /* Priority of interrupt 37 */
      __IO uint32_t PRI_38              	:	8;  /* Priority of interrupt 38 */
      __IO uint32_t PRI_39              	:	8;  /* Priority of interrupt 39 */

    } bit;
  } NVIC_IPR9;
  union {    /* SCS_NVIC_IPR10 Register */
    __IO uint32_t reg;
    struct { /* SCS_NVIC_IPR10 Control Bits */
      __IO uint32_t PRI_40              	:	8;  /* Priority of interrupt 40 */
      __IO uint32_t PRI_41              	:	8;  /* Priority of interrupt 41 */
      __IO uint32_t PRI_42              	:	8;  /* Priority of interrupt 42 */
      __IO uint32_t PRI_43              	:	8;  /* Priority of interrupt 43 */

    } bit;
  } NVIC_IPR10;
  union {    /* SCS_NVIC_IPR11 Register */
    __IO uint32_t reg;
    struct { /* SCS_NVIC_IPR11 Control Bits */
      __IO uint32_t PRI_44              	:	8;  /* Priority of interrupt 44 */
      __IO uint32_t PRI_45              	:	8;  /* Priority of interrupt 45 */
      __IO uint32_t PRI_46              	:	8;  /* Priority of interrupt 46 */
      __IO uint32_t PRI_47              	:	8;  /* Priority of interrupt 47 */

    } bit;
  } NVIC_IPR11;
  union {    /* SCS_NVIC_IPR12 Register */
    __IO uint32_t reg;
    struct { /* SCS_NVIC_IPR12 Control Bits */
      __IO uint32_t PRI_48              	:	8;  /* Priority of interrupt 48 */
      __IO uint32_t PRI_49              	:	8;  /* Priority of interrupt 49 */
      __IO uint32_t PRI_50              	:	8;  /* Priority of interrupt 50 */
      __IO uint32_t PRI_51              	:	8;  /* Priority of interrupt 51 */

    } bit;
  } NVIC_IPR12;
  union {    /* SCS_NVIC_IPR13 Register */
    __IO uint32_t reg;
    struct { /* SCS_NVIC_IPR13 Control Bits */
      __IO uint32_t PRI_52              	:	8;  /* Priority of interrupt 52 */
      __IO uint32_t PRI_53              	:	8;  /* Priority of interrupt 53 */
      __IO uint32_t PRI_54              	:	8;  /* Priority of interrupt 54 */
      __IO uint32_t PRI_55              	:	8;  /* Priority of interrupt 55 */

    } bit;
  } NVIC_IPR13;
  union {    /* SCS_NVIC_IPR14 Register */
    __IO uint32_t reg;
    struct { /* SCS_NVIC_IPR14 Control Bits */
      __IO uint32_t PRI_56              	:	8;  /* Priority of interrupt 56 */
      __IO uint32_t PRI_57              	:	8;  /* Priority of interrupt 57 */
      __IO uint32_t PRI_58              	:	8;  /* Priority of interrupt 58 */
      __IO uint32_t PRI_59              	:	8;  /* Priority of interrupt 59 */

    } bit;
  } NVIC_IPR14;
  union {    /* SCS_NVIC_IPR15 Register */
    __IO uint32_t reg;
    struct { /* SCS_NVIC_IPR15 Control Bits */
      __IO uint32_t PRI_60              	:	8;  /* Priority of interrupt 60 */
      __IO uint32_t PRI_61              	:	8;  /* Priority of interrupt 61 */
      __IO uint32_t PRI_62              	:	8;  /* Priority of interrupt 62 */
      __IO uint32_t PRI_63              	:	8;  /* Priority of interrupt 63 */

    } bit;
  } NVIC_IPR15;
  uint8_t RESERVED8[2240];
  union {    /* SCS_CPUID Register */
    __I uint32_t reg;
    struct { /* SCS_CPUID Control Bits */
      __I  uint32_t REVISION            	:	4;  /* Implementation defined revision number. */
      __I  uint32_t PARTNO              	:	12;  /* Number of processor within family. */
      __I  uint32_t CONSTANT            	:	4;  /* Reads as 0xC */
      __I  uint32_t VARIANT             	:	4;  /* Implementation defined variant number. */
      __I  uint32_t IMPLEMENTER         	:	8;  /* Implementor code. */

    } bit;
  } CPUID;
  union {    /* SCS_ICSR Register */
    __IO uint32_t reg;
    struct { /* SCS_ICSR Control Bits */
      __I  uint32_t VECTACTIVE          	:	9;  /* Active ISR number field. Reset clears the VECTACTIVE field. */
      __I  uint32_t RETTOBASE           	:	1;  /* This bit is 1 when the set of all active exceptions minus the  */
                                                /* IPSR_current_exception yields the empty set. */
      __I  uint32_t VECTPENDING         	:	6;  /* Pending ISR number field. VECTPENDING contains the interrupt number of the  */
                                                /* highest priority pending ISR. */
      __I  uint32_t ISRPENDING          	:	1;  /* Interrupt pending flag. Excludes NMI and faults. */
      __I  uint32_t ISRPREEMPT          	:	1;  /* You must only use this at debug time. It indicates that a pending interrupt is  */
                                                /* to be taken in the next running cycle. If C_MASKINTS is clear in the Debug  */
                                                /* Halting Control and Status Register, the interrupt is serviced. */
      __O  uint32_t PENDSTCLR           	:	1;  /* Clear pending SysTick bit */
      __IO uint32_t PENDSTSET           	:	1;  /* Set a pending SysTick bit. */
      __O  uint32_t PENDSVCLR           	:	1;  /* Clear pending pendSV bit */
      __IO uint32_t PENDSVSET           	:	1;  /* Set pending pendSV bit. */
      __IO uint32_t NMIPENDSET          	:	1;  /* Set pending NMI bit. NMIPENDSET pends and activates an NMI. Because NMI is the  */
                                                /* highest-priority interrupt, it takes effect as soon as it registers. */

    } bit;
  } ICSR;
  union {    /* SCS_VTOR Register */
    __IO uint32_t reg;
    struct { /* SCS_VTOR Control Bits */
      __IO uint32_t TBLOFF              	:	22;  /* Vector table base offset field. Contains the offset of the table base from the  */
                                                /* bottom of the SRAM or CODE space. */
      __IO uint32_t TBLBASE             	:	1;  /* Table base is in Code (0) or RAM (1). */

    } bit;
  } VTOR;
  union {    /* SCS_AIRCR Register */
    __IO uint32_t reg;
    struct { /* SCS_AIRCR Control Bits */
      __O  uint32_t VECTRESET           	:	1;  /* System Reset bit. Resets the system, with the exception of debug components. The */
                                                /* VECTRESET bit self-clears. Reset clears the VECTRESET bit. For debugging, only  */
                                                /* write this bit when the core is halted. */
      __O  uint32_t VECTCLRACTIVE       	:	1;  /* Clears all active state information for active NMI, fault, and interrupts.  It  */
                                                /* is the responsibility of the application to reinitialize the stack. The  */
                                                /* VECTCLRACTIVE bit is for returning to a known state during debug. The  */
                                                /* VECTCLRACTIVE bit self-clears. IPSR is not cleared by this operation. So, if  */
                                                /* used by an application, it must only be used at the base level of activation, or */
                                                /* within a system handler whose active bit can be set. */
      __O  uint32_t SYSRESETREQ         	:	1;  /* Causes a signal to be asserted to the outer system that indicates a reset is  */
                                                /* requested. Intended to force a large system reset of all major components except */
                                                /* for debug. Setting this bit does not prevent Halting Debug from running. */
      __IO uint32_t PRIGROUP            	:	3;  /* Interrupt priority grouping field. The PRIGROUP field is a binary point position */
                                                /* indicator for creating subpriorities for exceptions that share the same pre- */
                                                /* emption level. It divides the PRI_n field in the Interrupt Priority Register  */
                                                /* into a pre-emption level and a subpriority level. The binary point is a left-of  */
                                                /* value. This means that the PRIGROUP value represents a point starting at the  */
                                                /* left of the Least Significant Bit (LSB). This is bit [0] of 7:0. The lowest  */
                                                /* value might not be 0 depending on the number of bits allocated for priorities,  */
                                                /* and implementation choices  */
      __I  uint32_t ENDIANESS           	:	1;  /* Data endianness bit. ENDIANNESS is sampled from the BIGEND input port during  */
                                                /* reset. You cannot change ENDIANNESS outside of reset. */
      __O  uint32_t VECTKEY             	:	16;  /* Register key. Writing to this register requires 0x5FA in the VECTKEY field.  */
                                                /* Otherwise the write value is ignored. */

    } bit;
  } AIRCR;
  union {    /* SCS_SCR Register */
    __IO uint32_t reg;
    struct { /* SCS_SCR Control Bits */
      __IO uint32_t SLEEPONEXIT         	:	1;  /* Sleep on exit when returning from Handler mode to Thread mode. Enables interrupt */
                                                /* driven applications to avoid returning to empty main application. */
      __IO uint32_t SLEEPDEEP           	:	1;  /* Sleep deep bit. */
      __IO uint32_t SEVONPEND           	:	1;  /* When enabled, this causes WFE to wake up when an interrupt moves from inactive  */
                                                /* to pended. Otherwise, WFE only wakes up from an event signal, external and SEV  */
                                                /* instruction generated. The event input, RXEV, is registered even when not  */
                                                /* waiting for an event, and so effects the next WFE. */

    } bit;
  } SCR;
  union {    /* SCS_CCR Register */
    __IO uint32_t reg;
    struct { /* SCS_CCR Control Bits */
      __IO uint32_t NONBASETHREDENA     	:	1;  /* When 0, default, It is only possible to enter Thread mode when returning from  */
                                                /* the last exception. When set to 1, Thread mode can be entered from any level in  */
                                                /* Handler mode by controlled return value. */
      __IO uint32_t USERSETMPEND        	:	1;  /* If written as 1, enables user code to write the Software Trigger Interrupt  */
                                                /* register to trigger (pend) a Main exception, which is one associated with the  */
                                                /* Main stack pointer. */
      __IO uint32_t UNALIGN_TRP         	:	1;  /* Trap for unaligned access. This enables faulting/halting on any unaligned half  */
                                                /* or full word access. Unaligned load-store multiples always fault. The relevant  */
                                                /* Usage Fault Status Register bit is UNALIGNED. */
      __IO uint32_t DIV_0_TRP           	:	1;  /* Trap on Divide by 0. This enables faulting/halting when an attempt is made to  */
                                                /* divide by 0. The relevant Usage Fault Status Register bit is DIVBYZERO. */
      __IO uint32_t BFHFNMIGN           	:	1;  /* When enabled, this causes handlers running at priority -1 and -2 (Hard Fault,  */
                                                /* NMI, and FAULTMASK escalated handlers) to ignore Data Bus faults caused by load  */
                                                /* and store instructions. When disabled, these bus faults cause a lock-up. You  */
                                                /* must only use this enable with extreme caution. All data bus faults are ignored  */
                                                /* therefore you must only use it when the handler and its data are in absolutely  */
                                                /* safe memory. Its normal use is to probe system devices and bridges to detect  */
                                                /* control path problems and fix them. */
      __IO uint32_t STKALIGN            	:	1;  /* Stack alignment bit. */

    } bit;
  } CCR;
  union {    /* SCS_SHPR1 Register */
    __IO uint32_t reg;
    struct { /* SCS_SHPR1 Control Bits */
      __IO uint32_t PRI_4               	:	8;  /* Priority of system handler 4. */
      __IO uint32_t PRI_5               	:	8;  /* Priority of system handler 5. */
      __IO uint32_t PRI_6               	:	8;  /* Priority of system handler 6. */
      __IO uint32_t PRI_7               	:	8;  /* Priority of system handler 7. */

    } bit;
  } SHPR1;
  union {    /* SCS_SHPR2 Register */
    __IO uint32_t reg;
    struct { /* SCS_SHPR2 Control Bits */
      __IO uint32_t PRI_8               	:	8;  /* Priority of system handler 8. */
      __IO uint32_t PRI_9               	:	8;  /* Priority of system handler 9. */
      __IO uint32_t PRI_10              	:	8;  /* Priority of system handler 10. */
      __IO uint32_t PRI_11              	:	8;  /* Priority of system handler 11. */

    } bit;
  } SHPR2;
  union {    /* SCS_SHPR3 Register */
    __IO uint32_t reg;
    struct { /* SCS_SHPR3 Control Bits */
      __IO uint32_t PRI_12              	:	8;  /* Priority of system handler 12. */
      __IO uint32_t PRI_13              	:	8;  /* Priority of system handler 13. */
      __IO uint32_t PRI_14              	:	8;  /* Priority of system handler 14. */
      __IO uint32_t PRI_15              	:	8;  /* Priority of system handler 15. */

    } bit;
  } SHPR3;
  union {    /* SCS_SHCSR Register */
    __IO uint32_t reg;
    struct { /* SCS_SHCSR Control Bits */
      __I  uint32_t MEMFAULTACT         	:	1;  /* MemManage active flag. */
      __I  uint32_t BUSFAULTACT         	:	1;  /* BusFault active flag. */
      __I  uint32_t USGFAULTACT         	:	1;  /* UsageFault active flag. */
      __I  uint32_t SVCALLACT           	:	1;  /* SVCall active flag. */
      __I  uint32_t MONITORACT          	:	1;  /* the Monitor active flag. */
      __I  uint32_t PENDSVACT           	:	1;  /* PendSV active flag. */
      __I  uint32_t SYSTICKACT          	:	1;  /* SysTick active flag. */
      __I  uint32_t USGFAULTPENDED      	:	1;  /* usage fault pended flag. */
      __I  uint32_t MEMFAULTPENDED      	:	1;  /* MemManage pended flag. */
      __I  uint32_t BUSFAULTPENDED      	:	1;  /* BusFault pended flag. */
      __I  uint32_t SVCALLPENDED        	:	1;  /* SVCall pended flag. */
      __IO uint32_t MEMFAULTENA         	:	1;  /* MemManage fault system handler enable */
      __IO uint32_t BUSFAULTENA         	:	1;  /* Bus fault system handler enable */
      __IO uint32_t USGFAULTENA         	:	1;  /* Usage fault system handler enable */

    } bit;
  } SHCSR;
  union {    /* SCS_CFSR Register */
    __IO uint32_t reg;
    struct { /* SCS_CFSR Control Bits */
      __IO uint32_t IACCVIOL            	:	1;  /* Instruction access violation flag. Attempting to fetch an instruction from a  */
                                                /* location that does not permit execution sets the IACCVIOL flag. This occurs on  */
                                                /* any access to an XN region, even when the MPU is disabled or not present. The  */
                                                /* return PC points to the faulting instruction. The MMAR is not written. */
      __IO uint32_t DACCVIOL            	:	1;  /* Data access violation flag. Attempting to load or store at a location that does  */
                                                /* not permit the operation sets the DACCVIOL flag. The return PC points to the  */
                                                /* faulting instruction. This error loads MMAR with the address of the attempted  */
                                                /* access. */
      __IO uint32_t MUNSTKERR           	:	1;  /* Unstack from exception return has caused one or more access violations. This is  */
                                                /* chained to the handler, so that the original return stack is still present. SP  */
                                                /* is not adjusted from failing return and new save is not performed. The MMAR is  */
                                                /* not written. */
      __IO uint32_t MSTKERR             	:	1;  /* Stacking from exception has caused one or more access violations. The SP is  */
                                                /* still adjusted and the values in the context area on the stack might be  */
                                                /* incorrect. The MMAR is not written. */
      __IO uint32_t MMARVALID           	:	1;  /* Memory Manage Address Register (MMAR) address valid flag. A later-arriving fault */
                                                /* , such as a bus fault, can clear a memory manage fault.. If a MemManage fault  */
                                                /* occurs that is escalated to a Hard Fault because of priority, the Hard Fault  */
                                                /* handler must clear this bit. This prevents problems on return to a stacked  */
                                                /* active MemManage handler whose MMAR value has been overwritten. */
      __IO uint32_t IBUSERR             	:	1;  /* Instruction bus error flag. The IBUSERR flag is set by a prefetch error. The  */
                                                /* fault stops on the instruction, so if the error occurs under a branch shadow, no */
                                                /* fault occurs. The BFAR is not written. */
      __IO uint32_t PRECISERR           	:	1;  /* Precise data bus error return. */
      __IO uint32_t IMPRECISERR         	:	1;  /* Imprecise data bus error. It is a BusFault, but the Return PC is not related to  */
                                                /* the causing instruction. This is not a synchronous fault. So, if detected when  */
                                                /* the priority of the current activation is higher than the Bus Fault, it only  */
                                                /* pends. Bus fault activates when returning to a lower priority activation. If a  */
                                                /* precise fault occurs before returning to a lower priority exception, the handler */
                                                /* detects both IMPRECISERR set and one of the precise fault status bits set at  */
                                                /* the same time. The BFAR is not written. */
      __IO uint32_t UNSTKERR            	:	1;  /* Unstack from exception return has caused one or more bus faults. This is chained */
                                                /* to the handler, so that the original return stack is still present. SP is not  */
                                                /* adjusted from failing return and new save is not performed. The BFAR is not  */
                                                /* written. */
      __IO uint32_t STKERR              	:	1;  /* Stacking from exception has caused one or more bus faults. The SP is still  */
                                                /* adjusted and the values in the context area on the stack might be incorrect. The */
                                                /* BFAR is not written. */
      __IO uint32_t BFARVALID           	:	1;  /* This bit is set if the Bus Fault Address Register (BFAR) contains a valid  */
                                                /* address. This is true after a bus fault where the address is known. Other faults */
                                                /* can clear this bit, such as a Mem Manage fault occurring later. If a Bus fault  */
                                                /* occurs that is escalated to a Hard Fault because of priority, the Hard Fault  */
                                                /* handler must clear this bit. This prevents problems if returning to a stacked  */
                                                /* active Bus fault handler whose BFAR value has been overwritten. */
      __IO uint32_t UNDEFINSTR          	:	1;  /* The UNDEFINSTR flag is set when the processor attempts to execute an undefined  */
                                                /* instruction. This is an instruction that the processor cannot decode. The return */
                                                /* PC points to the undefined instruction. */
      __IO uint32_t INVSTATE            	:	1;  /* Invalid combination of EPSR and instruction, for reasons other than UNDEFINED  */
                                                /* instruction. Return PC points to faulting instruction, with the invalid state. */
      __IO uint32_t INVPC               	:	1;  /* Attempt to load EXC_RETURN into PC illegally. Invalid instruction, invalid  */
                                                /* context, invalid value. The return PC points to the instruction that tried to  */
                                                /* set the PC. */
      __IO uint32_t NOCP                	:	1;  /* Attempt to use a coprocessor instruction. The processor does not support  */
                                                /* coprocessor instructions. */
      __IO uint32_t UNALIGNED           	:	1;  /* When UNALIGN_TRP is enabled (see Configuration Control Register on page 8-26),  */
                                                /* and there is an attempt to make an unaligned memory access, then this fault  */
                                                /* occurs. Unaligned LDM/STM/LDRD/STRD instructions always fault irrespective of  */
                                                /* the setting of UNALIGN_TRP. */
      __IO uint32_t DIVBYZERO           	:	1;  /* When DIV_0_TRP (see Configuration Control Register on page 8-26) is enabled and  */
                                                /* an SDIV or UDIV instruction is used with a divisor of 0, this fault occurs The  */
                                                /* instruction is executed and the return PC points to it. If DIV_0_TRP is not set, */
                                                /* then the divide returns a quotient of 0. */

    } bit;
  } CFSR;
  union {    /* SCS_HFSR Register */
    __IO uint32_t reg;
    struct { /* SCS_HFSR Control Bits */
      __IO uint32_t VECTTBL             	:	1;  /* This bit is set if there is a fault because of vector table read on exception  */
                                                /* processing (Bus Fault). This case is always a Hard Fault. The return PC points  */
                                                /* to the pre-empted instruction. */
      __IO uint32_t FORCED              	:	1;  /* Hard Fault activated because a Configurable Fault was received and cannot  */
                                                /* activate because of priority or because the Configurable Fault is disabled. The  */
                                                /* Hard Fault handler then has to read the other fault status registers to  */
                                                /* determine cause. */
      __IO uint32_t DEBUGEVT            	:	1;  /* This bit is set if there is a fault related to debug. This is only possible when */
                                                /* halting debug is not enabled. For monitor enabled debug, it only happens for  */
                                                /* BKPT when the current priority is higher than the monitor. When both halting and */
                                                /* monitor debug are disabled, it only happens for debug events that are not  */
                                                /* ignored (minimally, BKPT). The Debug Fault Status Register is updated. */

    } bit;
  } HFSR;
  union {    /* SCS_DFSR Register */
    __IO uint32_t reg;
    struct { /* SCS_DFSR Control Bits */
      __IO uint32_t HALTED              	:	1;  /* Halt request flag. The processor is halted on the next instruction. */
      __IO uint32_t BKPT                	:	1;  /* BKPT flag. The BKPT flag is set by a BKPT instruction in flash patch code, and  */
                                                /* also by normal code. Return PC points to breakpoint containing instruction. */
      __IO uint32_t DWTTRAP             	:	1;  /* Data Watchpoint and Trace (DWT) flag. The processor stops at the current  */
                                                /* instruction or at the next instruction. */
      __IO uint32_t VCATCH              	:	1;  /* Vector catch flag. When the VCATCH flag is set, a flag in one of the local fault */
                                                /* status registers is also set to indicate the type of fault. */
      __IO uint32_t EXTERNAL            	:	1;  /* External debug request flag. The processor stops on next instruction boundary. */

    } bit;
  } DFSR;
  uint32_t MMFAR; /* Mem Manage Fault Address Register. Use the Memory Manage Fault Address Register  */
                                                                                 /* to read the address of the location that caused a Memory Manage Fault. */
  uint32_t BFAR; /* Bus Fault Address Register. Use the Bus Fault Address Register to read the  */
                                                                                 /* address of the location that generated a Bus Fault. */
  uint32_t AFSR; /* Auxiliary Fault Status Register. Use the Auxiliary Fault Status Register (AFSR)  */
                                                                                 /* to determine additional system fault information to software. The AFSR flags map */
                                                                                 /* directly onto the AUXFAULT inputs of the processor, and a single-cycle high  */
                                                                                 /* level on an external pin causes the corresponding AFSR bit to become latched as  */
                                                                                 /* one. The bit can only be cleared by writing a one to the corresponding AFSR bit. */
                                                                                 /* When an AFSR bit is written or latched as one, an exception does not occur. If  */
                                                                                 /* you require an exception, you must use an interrupt. */
  union {    /* SCS_ID_PFR0 Register */
    __I uint32_t reg;
    struct { /* SCS_ID_PFR0 Control Bits */
      __I  uint32_t STATE0              	:	4;  /* State0 (T-bit == 0) */
      __I  uint32_t STATE1              	:	4;  /* State1 (T-bit == 1) */

    } bit;
  } ID_PFR0;
  union {    /* SCS_ID_PFR1 Register */
    __I uint32_t reg;
    struct { /* SCS_ID_PFR1 Control Bits */
      __I  uint32_t MICROCONTROLLER_PROGRAMMERS_MODEL	:	4;  /* Microcontroller programmer's model */

    } bit;
  } ID_PFR1;
  union {    /* SCS_ID_DFR0 Register */
    __I uint32_t reg;
    struct { /* SCS_ID_DFR0 Control Bits */
      __I  uint32_t MICROCONTROLLER_DEBUG_MODEL	:	4;  /* Microcontroller Debug Model - memory mapped */

    } bit;
  } ID_DFR0;
  uint32_t ID_AFR0; /* Auxiliary Feature register0. This register provides some freedom for  */
                                                                                 /* IMPLEMENTATION DEFINED features to be registered against the CPUID. Not used in  */
                                                                                 /* Cortex-M4. */
  union {    /* SCS_ID_MMFR0 Register */
    __I uint32_t reg;
    struct { /* SCS_ID_MMFR0 Control Bits */
      __I  uint32_t PMSA_SUPPORT        	:	4;  /* PMSA support */
      __I  uint32_t CACHE_COHERENCE_SUPPORT	:	4;  /* Cache coherence support */
      __I  uint32_t OUTER_NON_SHARABLE_SUPPORT	:	4;  /* Outer non-sharable support */
      __I  uint32_t AUILIARY_REGISTER_SUPPORT	:	4;  /* Auxiliary register support */

    } bit;
  } ID_MMFR0;
  uint32_t ID_MMFR1; /* Memory Model Feature register1. General information on the memory model and  */
                                                                                 /* memory management support. */
  union {    /* SCS_ID_MMFR2 Register */
    __I uint32_t reg;
    struct { /* SCS_ID_MMFR2 Control Bits */
      __I  uint32_t WAIT_FOR_INTERRUPT_STALLING	:	4;  /* wait for interrupt stalling */

    } bit;
  } ID_MMFR2;
  uint32_t ID_MMFR3; /* Memory Model Feature register3. General information on the memory model and  */
                                                                                 /* memory management support. */
  union {    /* SCS_ISAR0 Register */
    __I uint32_t reg;
    struct { /* SCS_ISAR0 Control Bits */
      __I  uint32_t BITCOUNT_INSTRS     	:	4;  /* BitCount instructions */
      __I  uint32_t BITFIELD_INSTRS     	:	4;  /* BitField instructions */
      __I  uint32_t CMPBRANCH_INSTRS    	:	4;  /* CmpBranch instructions */
      __I  uint32_t COPROC_INSTRS       	:	4;  /* Coprocessor instructions */
      __I  uint32_t DEBUG_INSTRS        	:	4;  /* Debug instructions */
      __I  uint32_t DIVIDE_INSTRS       	:	4;  /* Divide instructions */

    } bit;
  } ISAR0;
  union {    /* SCS_ID_ISAR1 Register */
    __I uint32_t reg;
    struct { /* SCS_ID_ISAR1 Control Bits */
      __I  uint32_t ETEND_INSRS         	:	4;  /* Extend instructions. Note that the shift options on these instructions are also  */
                                                /* controlled by the WithShifts_instrs attribute. */
      __I  uint32_t IFTHEN_INSTRS       	:	4;  /* IfThen instructions */
      __I  uint32_t IMMEDIATE_INSTRS    	:	4;  /* Immediate instructions */
      __I  uint32_t INTERWORK_INSTRS    	:	4;  /* Interwork instructions */

    } bit;
  } ID_ISAR1;
  union {    /* SCS_ID_ISAR2 Register */
    __I uint32_t reg;
    struct { /* SCS_ID_ISAR2 Control Bits */
      __I  uint32_t LOADSTORE_INSTRS    	:	4;  /* LoadStore instructions */
      __I  uint32_t MEMHINT_INSTRS      	:	4;  /* MemoryHint instructions */
      __I  uint32_t MULTIACCESSINT_INSTRS	:	4;  /* Multi-Access interruptible instructions */
      __I  uint32_t MULT_INSTRS         	:	4;  /* Multiply instructions */
      __I  uint32_t MULTS_INSTRS        	:	4;  /* Multiply instructions (advanced, signed) */
      __I  uint32_t MULTU_INSTRS        	:	4;  /* Multiply instructions (advanced, unsigned) */
      __I  uint32_t REVERSAL_INSTRS     	:	4;  /* Reversal instructions */

    } bit;
  } ID_ISAR2;
  union {    /* SCS_ID_ISAR3 Register */
    __I uint32_t reg;
    struct { /* SCS_ID_ISAR3 Control Bits */
      __I  uint32_t SATRUATE_INSTRS     	:	4;  /* Saturate instructions */
      __I  uint32_t SIMD_INSTRS         	:	4;  /* SIMD instructions */
      __I  uint32_t SVC_INSTRS          	:	4;  /* SVC instructions */
      __I  uint32_t SYNCPRIM_INSTRS     	:	4;  /* SyncPrim instructions. Note there are no LDREXD or STREXD in ARMv7-M. This  */
                                                /* attribute is used in conjunction with the SyncPrim_instrs_frac attribute in  */
                                                /* ID_ISAR4[23:20]. */
      __I  uint32_t TABBRANCH_INSTRS    	:	4;  /* TableBranch instructions */
      __I  uint32_t THUMBCOPY_INSTRS    	:	4;  /* ThumbCopy instructions */
      __I  uint32_t TRUENOP_INSTRS      	:	4;  /* TrueNOP instructions */

    } bit;
  } ID_ISAR3;
  union {    /* SCS_ID_ISAR4 Register */
    __I uint32_t reg;
    struct { /* SCS_ID_ISAR4 Control Bits */
      __I  uint32_t UNPRIV_INSTRS       	:	4;  /* Unprivileged instructions */
      __I  uint32_t WITHSHIFTS_INSTRS   	:	4;  /* WithShift instructions. Note that all additions only apply in cases where the  */
                                                /* encoding supports them - e.g. there is no difference between levels 3 and 4 in  */
                                                /* the Thumb-2 instruction set. Also note that MOV instructions with shift options  */
                                                /* should instead be treated as ASR, LSL, LSR, ROR or RRX instructions. */
      __I  uint32_t WRITEBACK_INSTRS    	:	4;  /* Writeback instructions */
      __I  uint32_t BARRIER_INSTRS      	:	4;  /* Barrier instructions */
      __I  uint32_t SYNCPRIM_INSTRS_FRAC	:	4;  /* SyncPrim_instrs_frac */
      __I  uint32_t PSR_M_INSTRS        	:	4;  /* PSR_M_instrs */

    } bit;
  } ID_ISAR4;
  uint8_t RESERVED9[124];
  union {    /* SCS_DHCSR Register */
    __IO uint32_t reg;
    struct { /* SCS_DHCSR Control Bits */
      __IO uint32_t C_DEBUGEN           	:	1;  /* Enables debug. This can only be written by AHB-AP and not by the core. It is  */
                                                /* ignored when written by the core, which cannot set or clear it. The core must  */
                                                /* write a 1 to it when writing C_HALT to halt itself. */
      __IO uint32_t C_HALT              	:	1;  /* Halts the core. This bit is set automatically when the core Halts. For example  */
                                                /* Breakpoint. This bit clears on core reset. This bit can only be written if  */
                                                /* C_DEBUGEN is 1, otherwise it is ignored. When setting this bit to 1, C_DEBUGEN  */
                                                /* must also be written to 1 in the same value (value[1:0] is 2'b11). The core can  */
                                                /* halt itself, but only if C_DEBUGEN is already 1 and only if it writes with b11). */
      __IO uint32_t C_STEP              	:	1;  /* Steps the core in halted debug. When C_DEBUGEN = 0, this bit has no effect. Must */
                                                /* only be modified when the processor is halted (S_HALT == 1). */
      __IO uint32_t C_MASKINTS          	:	1;  /* Mask interrupts when stepping or running in halted debug. Does not affect NMI,  */
                                                /* which is not maskable. Must only be modified when the processor is halted ( */
                                                /* S_HALT == 1). Also does not affect fault exceptions and SVC caused by execution  */
                                                /* of the instructions. CMASKINTS must be set or cleared before halt is released.  */
                                                /* This means that the writes to set or clear C_MASKINTS and to set or clear C_HALT */
                                                /* must be separate. */
      __IO uint32_t C_SNAPSTALL         	:	1;  /* If the core is stalled on a load/store operation the stall ceases and the  */
                                                /* instruction is forced to complete. This enables Halting debug to gain control of */
                                                /* the core. It can only be set if: C_DEBUGEN = 1 and C_HALT = 1. The core reads  */
                                                /* S_RETIRE_ST as 0. This indicates that no instruction has advanced. This prevents */
                                                /* misuse. The bus state is Unpredictable when this is used. S_RETIRE can detect  */
                                                /* core stalls on load/store operations. */
      __I  uint32_t S_REGRDY            	:	1;  /* Register Read/Write on the Debug Core Register Selector register is available.  */
                                                /* Last transfer is complete. */
      __I  uint32_t S_HALT              	:	1;  /* The core is in debug state when S_HALT is set. */
      __I  uint32_t S_SLEEP             	:	1;  /* Indicates that the core is sleeping (WFI, WFE, or SLEEP-ON-EXIT). Must use  */
                                                /* C_HALT to gain control or wait for interrupt to wake-up. */
      __I  uint32_t S_LOCKUP            	:	1;  /* Reads as one if the core is running (not halted) and a lockup condition is  */
                                                /* present. */
      __I  uint32_t S_RETIRE_ST         	:	1;  /* Indicates that an instruction has completed since last read. This is a sticky  */
                                                /* bit that clears on read. This determines if the core is stalled on a load/store  */
                                                /* or fetch. */
      __I  uint32_t S_RESET_ST          	:	1;  /* Indicates that the core has been reset, or is now being reset, since the last  */
                                                /* time this bit was read. This a sticky bit that clears on read. So, reading twice */
                                                /* and getting 1 then 0 means it was reset in the past. Reading twice and getting  */
                                                /* 1 both times means that it is being reset now (held in reset still). */

    } bit;
  } DHCSR;
  union {    /* SCS_DCRSR Register */
    __O uint32_t reg;
    struct { /* SCS_DCRSR Control Bits */
      __O  uint32_t REGSEL              	:	5;  /* Register select */
      __O  uint32_t REGWNR              	:	1;  /* Write = 1, Read = 0 */

    } bit;
  } DCRSR;
  uint32_t DCRDR; /* Debug Core Register Data Register. The purpose of the Debug Core Register Data  */
                                                                                 /* Register (DCRDR) is to hold data for reading and writing registers to and from  */
                                                                                 /* the processor. This is the data value written to the register selected by the  */
                                                                                 /* Debug Register Selector Register. When the processor receives a request from the */
                                                                                 /* Debug Core Register Selector, this register is read or written by the processor */
                                                                                 /* using a normal load-store unit operation. If core register transfers are not  */
                                                                                 /* being performed, software-based debug monitors can use this register for  */
                                                                                 /* communication in non-halting debug. For example, OS RSD and Real View Monitor.  */
                                                                                 /* This enables flags and bits to acknowledge state and indicate if commands have  */
                                                                                 /* been accepted to, replied to, or accepted and replied to. */
  union {    /* SCS_DEMCR Register */
    __IO uint32_t reg;
    struct { /* SCS_DEMCR Control Bits */
      __IO uint32_t VC_CORERESET        	:	1;  /* Reset Vector Catch. Halt running system if Core reset occurs. */
      __IO uint32_t VC_MMERR            	:	1;  /* Debug trap on Memory Management faults. */
      __IO uint32_t VC_NOCPERR          	:	1;  /* Debug trap on Usage Fault access to Coprocessor that is not present or marked as */
                                                /* not present in CAR register. */
      __IO uint32_t VC_CHKERR           	:	1;  /* Debug trap on Usage Fault enabled checking errors. */
      __IO uint32_t VC_STATERR          	:	1;  /* Debug trap on Usage Fault state errors. */
      __IO uint32_t VC_BUSERR           	:	1;  /* Debug Trap on normal Bus error. */
      __IO uint32_t VC_INTERR           	:	1;  /* Debug Trap on interrupt/exception service errors. These are a subset of other  */
                                                /* faults and catches before BUSERR or HARDERR. */
      __IO uint32_t VC_HARDERR          	:	1;  /* Debug trap on Hard Fault. */
      __IO uint32_t MON_EN              	:	1;  /* Enable the debug monitor. When enabled, the System handler priority register  */
                                                /* controls its priority level. If disabled, then all debug events go to Hard fault */
                                                /* . C_DEBUGEN in the Debug Halting Control and Statue register overrides this bit. */
                                                /* Vector catching is semi-synchronous. When a matching event is seen, a Halt is  */
                                                /* requested. Because the processor can only halt on an instruction boundary, it  */
                                                /* must wait until the next instruction boundary. As a result, it stops on the  */
                                                /* first instruction of the exception handler. However, two special cases exist  */
                                                /* when a vector catch has triggered: 1. If a fault is taken during vectoring,  */
                                                /* vector read or stack push error, the halt occurs on the corresponding fault  */
                                                /* handler, for the vector error or stack push. 2. If a late arriving interrupt  */
                                                /* comes in during vectoring, it is not taken. That is, an implementation that  */
                                                /* supports the late arrival optimization must suppress it in this case. */
      __IO uint32_t MON_PEND            	:	1;  /* Pend the monitor to activate when priority permits. This can wake up the monitor */
                                                /* through the AHB-AP port. It is the equivalent to C_HALT for Monitor debug. This */
                                                /* register does not reset on a system reset. It is only reset by a power-on reset */
                                                /* . Software in the reset handler or later, or by the DAP must enable the debug  */
                                                /* monitor. */
      __IO uint32_t MON_STEP            	:	1;  /* When MON_EN = 1, this steps the core. When MON_EN = 0, this bit is ignored. This */
                                                /* is the equivalent to C_STEP. Interrupts are only stepped according to the  */
                                                /* priority of the monitor and settings of PRIMASK, FAULTMASK, or BASEPRI. */
      __IO uint32_t MON_REQ             	:	1;  /* This enables the monitor to identify how it wakes up. This bit clears on a Core  */
                                                /* Reset. */
      __IO uint32_t TRCENA              	:	1;  /* This bit must be set to 1 to enable use of the trace and debug blocks: Data  */
                                                /* Watchpoint and Trace (DWT), Instrumentation Trace Macrocell (ITM), Embedded  */
                                                /* Trace Macrocell (ETM), Trace Port Interface Unit (TPIU). This enables control of */
                                                /* power usage unless tracing is required. The application can enable this, for  */
                                                /* ITM use, or use by a debugger. Note that if no debug or trace components are  */
                                                /* present in the implementation then it is not possible to set TRCENA. */

    } bit;
  } DEMCR;
  uint8_t RESERVED10[256];
  union {    /* SCS_STIR Register */
    __O uint32_t reg;
    struct { /* SCS_STIR Control Bits */
      __O  uint32_t INTID               	:	9;  /* Interrupt ID field. Writing a value to the INTID field is the same as manually  */
                                                /* pending an interrupt by setting the corresponding interrupt bit in an Interrupt  */
                                                /* Set Pending Register. */

    } bit;
  } STIR;
  uint8_t RESERVED11[204];
  uint32_t PID4; /* Peripheral identification register (PID4).  */
  uint32_t PID5; /* Peripheral identification register (PID5).  */
  uint32_t PID6; /* Peripheral identification register (PID6).  */
  uint32_t PID7; /* Peripheral identification register (PID7).  */
  uint32_t PID0; /* Peripheral identification register Bits 7:0 (PID0).  */
  uint32_t PID1; /* Peripheral identification register Bits 15:8 (PID1).  */
  uint32_t PID2; /* Peripheral identification register Bits 23:16 (PID2).  */
  uint32_t PID3; /* Peripheral identification register Bits 31:24 (PID3).  */
  uint32_t CID0; /* Component identification register Bits 7:0 (CID0).  */
  uint32_t CID1; /* Component identification register Bits 15:8 (CID1).  */
  uint32_t CID2; /* Component identification register Bits 23:16 (CID2).  */
  uint32_t CID3; /* Component identification register Bits 31:24 (CID3).  */
} SCS_Type;


/* SCS_ICTR Control Bits */
#define SCS_ICTR_INTLINESNUM__M                            (0x0000001f)          /* Total number of interrupt lines in groups of 32. */

/* SCS_ACTLR Control Bits */
#define SCS_ACTLR_DISMCYCINT                               (0x00000001)          /* Disables interruption of multi-cycle instructions. This increases the interrupt  */
                                                                                 /* latency of the processor becuase LDM/STM completes before interrupt stacking  */
                                                                                 /* occurs. */
#define SCS_ACTLR_DISDEFWBUF                               (0x00000002)          /* Disables write buffer us during default memorty map accesses. This causes all  */
                                                                                 /* bus faults to be precise bus faults but decreases the performance of the  */
                                                                                 /* processor because the stores to memory have to complete before the next  */
                                                                                 /* instruction can be executed. */
#define SCS_ACTLR_DISFOLD                                  (0x00000004)          /* Disables IT folding. */

/* SCS_STCSR Control Bits */
#define SCS_STCSR_ENABLE                                   (0x00000001)          /* Enable SysTick counter */
#define SCS_STCSR_TICKINT                                  (0x00000002)          /*  */
#define SCS_STCSR_CLKSOURCE                                (0x00000004)          /* Clock source. If no reference clock is provided, it is held at 1 and so gives  */
                                                                                 /* the same time as the core clock. The core clock must be at least 2.5 times  */
                                                                                 /* faster than the reference clock. If it is not, the count values are  */
                                                                                 /* Unpredictable. */
#define SCS_STCSR_COUNTFLAG                                (0x00010000)          /* Returns 1 if timer counted to 0 since last time this was read. Clears on read by */
                                                                                 /* application of any part of the SysTick Control and Status Register. If read by  */
                                                                                 /* the debugger using the DAP, this bit is cleared on read-only if the MasterType  */
                                                                                 /* bit in the AHB-AP Control Register is set to 0. Otherwise, the COUNTFLAG bit is  */
                                                                                 /* not changed by the debugger read. */

/* SCS_STRVR Control Bits */
#define SCS_STRVR_RELOAD__M                                (0x00ffffff)          /* Value to load into the SysTick Current Value Register when the counter reaches 0 */
                                                                                 /* . */

/* SCS_STCVR Control Bits */
#define SCS_STCVR_CURRENT__M                               (0x00ffffff)          /* Current value at the time the register is accessed. No read-modify-write  */
                                                                                 /* protection is provided, so change with care. Writing to it with any value clears */
                                                                                 /* the register to 0. Clearing this register also clears the COUNTFLAG bit of the  */
                                                                                 /* SysTick Control and Status Register. */

/* SCS_STCR Control Bits */
#define SCS_STCR_TENMS__M                                  (0x00ffffff)          /* Reads as zero. Indicates calibration value is not known. */
#define SCS_STCR_SKEW                                      (0x40000000)          /* Reads as one. The calibration value is not exactly 10ms because of clock  */
                                                                                 /* frequency. This could affect its suitability as a software real time clock. */
#define SCS_STCR_NOREF                                     (0x80000000)          /* Reads as one. Indicates that no separate reference clock is provided. */

/* SCS_NVIC_IPR0 Control Bits */
#define SCS_NVIC_IPR0_PRI_0__M                             (0x000000ff)          /* Priority of interrupt 0 */
#define SCS_NVIC_IPR0_PRI_1__M                             (0x0000ff00)          /* Priority of interrupt 1 */
#define SCS_NVIC_IPR0_PRI_2__M                             (0x00ff0000)          /* Priority of interrupt 2 */
#define SCS_NVIC_IPR0_PRI_3__M                             (0xff000000)          /* Priority of interrupt 3 */

/* SCS_NVIC_IPR1 Control Bits */
#define SCS_NVIC_IPR1_PRI_4__M                             (0x000000ff)          /* Priority of interrupt 4 */
#define SCS_NVIC_IPR1_PRI_5__M                             (0x0000ff00)          /* Priority of interrupt 5 */
#define SCS_NVIC_IPR1_PRI_6__M                             (0x00ff0000)          /* Priority of interrupt 6 */
#define SCS_NVIC_IPR1_PRI_7__M                             (0xff000000)          /* Priority of interrupt 7 */

/* SCS_NVIC_IPR2 Control Bits */
#define SCS_NVIC_IPR2_PRI_8__M                             (0x000000ff)          /* Priority of interrupt 8 */
#define SCS_NVIC_IPR2_PRI_9__M                             (0x0000ff00)          /* Priority of interrupt 9 */
#define SCS_NVIC_IPR2_PRI_10__M                            (0x00ff0000)          /* Priority of interrupt 10 */
#define SCS_NVIC_IPR2_PRI_11__M                            (0xff000000)          /* Priority of interrupt 11 */

/* SCS_NVIC_IPR3 Control Bits */
#define SCS_NVIC_IPR3_PRI_12__M                            (0x000000ff)          /* Priority of interrupt 12 */
#define SCS_NVIC_IPR3_PRI_13__M                            (0x0000ff00)          /* Priority of interrupt 13 */
#define SCS_NVIC_IPR3_PRI_14__M                            (0x00ff0000)          /* Priority of interrupt 14 */
#define SCS_NVIC_IPR3_PRI_15__M                            (0xff000000)          /* Priority of interrupt 15 */

/* SCS_NVIC_IPR4 Control Bits */
#define SCS_NVIC_IPR4_PRI_16__M                            (0x000000ff)          /* Priority of interrupt 16 */
#define SCS_NVIC_IPR4_PRI_17__M                            (0x0000ff00)          /* Priority of interrupt 17 */
#define SCS_NVIC_IPR4_PRI_18__M                            (0x00ff0000)          /* Priority of interrupt 18 */
#define SCS_NVIC_IPR4_PRI_19__M                            (0xff000000)          /* Priority of interrupt 19 */

/* SCS_NVIC_IPR5 Control Bits */
#define SCS_NVIC_IPR5_PRI_20__M                            (0x000000ff)          /* Priority of interrupt 20 */
#define SCS_NVIC_IPR5_PRI_21__M                            (0x0000ff00)          /* Priority of interrupt 21 */
#define SCS_NVIC_IPR5_PRI_22__M                            (0x00ff0000)          /* Priority of interrupt 22 */
#define SCS_NVIC_IPR5_PRI_23__M                            (0xff000000)          /* Priority of interrupt 23 */

/* SCS_NVIC_IPR6 Control Bits */
#define SCS_NVIC_IPR6_PRI_24__M                            (0x000000ff)          /* Priority of interrupt 24 */
#define SCS_NVIC_IPR6_PRI_25__M                            (0x0000ff00)          /* Priority of interrupt 25 */
#define SCS_NVIC_IPR6_PRI_26__M                            (0x00ff0000)          /* Priority of interrupt 26 */
#define SCS_NVIC_IPR6_PRI_27__M                            (0xff000000)          /* Priority of interrupt 27 */

/* SCS_NVIC_IPR7 Control Bits */
#define SCS_NVIC_IPR7_PRI_28__M                            (0x000000ff)          /* Priority of interrupt 28 */
#define SCS_NVIC_IPR7_PRI_29__M                            (0x0000ff00)          /* Priority of interrupt 29 */
#define SCS_NVIC_IPR7_PRI_30__M                            (0x00ff0000)          /* Priority of interrupt 30 */
#define SCS_NVIC_IPR7_PRI_31__M                            (0xff000000)          /* Priority of interrupt 31 */

/* SCS_NVIC_IPR8 Control Bits */
#define SCS_NVIC_IPR8_PRI_32__M                            (0x000000ff)          /* Priority of interrupt 32 */
#define SCS_NVIC_IPR8_PRI_33__M                            (0x0000ff00)          /* Priority of interrupt 33 */
#define SCS_NVIC_IPR8_PRI_34__M                            (0x00ff0000)          /* Priority of interrupt 34 */
#define SCS_NVIC_IPR8_PRI_35__M                            (0xff000000)          /* Priority of interrupt 35 */

/* SCS_NVIC_IPR9 Control Bits */
#define SCS_NVIC_IPR9_PRI_36__M                            (0x000000ff)          /* Priority of interrupt 36 */
#define SCS_NVIC_IPR9_PRI_37__M                            (0x0000ff00)          /* Priority of interrupt 37 */
#define SCS_NVIC_IPR9_PRI_38__M                            (0x00ff0000)          /* Priority of interrupt 38 */
#define SCS_NVIC_IPR9_PRI_39__M                            (0xff000000)          /* Priority of interrupt 39 */

/* SCS_NVIC_IPR10 Control Bits */
#define SCS_NVIC_IPR10_PRI_40__M                           (0x000000ff)          /* Priority of interrupt 40 */
#define SCS_NVIC_IPR10_PRI_41__M                           (0x0000ff00)          /* Priority of interrupt 41 */
#define SCS_NVIC_IPR10_PRI_42__M                           (0x00ff0000)          /* Priority of interrupt 42 */
#define SCS_NVIC_IPR10_PRI_43__M                           (0xff000000)          /* Priority of interrupt 43 */

/* SCS_NVIC_IPR11 Control Bits */
#define SCS_NVIC_IPR11_PRI_44__M                           (0x000000ff)          /* Priority of interrupt 44 */
#define SCS_NVIC_IPR11_PRI_45__M                           (0x0000ff00)          /* Priority of interrupt 45 */
#define SCS_NVIC_IPR11_PRI_46__M                           (0x00ff0000)          /* Priority of interrupt 46 */
#define SCS_NVIC_IPR11_PRI_47__M                           (0xff000000)          /* Priority of interrupt 47 */

/* SCS_NVIC_IPR12 Control Bits */
#define SCS_NVIC_IPR12_PRI_48__M                           (0x000000ff)          /* Priority of interrupt 48 */
#define SCS_NVIC_IPR12_PRI_49__M                           (0x0000ff00)          /* Priority of interrupt 49 */
#define SCS_NVIC_IPR12_PRI_50__M                           (0x00ff0000)          /* Priority of interrupt 50 */
#define SCS_NVIC_IPR12_PRI_51__M                           (0xff000000)          /* Priority of interrupt 51 */

/* SCS_NVIC_IPR13 Control Bits */
#define SCS_NVIC_IPR13_PRI_52__M                           (0x000000ff)          /* Priority of interrupt 52 */
#define SCS_NVIC_IPR13_PRI_53__M                           (0x0000ff00)          /* Priority of interrupt 53 */
#define SCS_NVIC_IPR13_PRI_54__M                           (0x00ff0000)          /* Priority of interrupt 54 */
#define SCS_NVIC_IPR13_PRI_55__M                           (0xff000000)          /* Priority of interrupt 55 */

/* SCS_NVIC_IPR14 Control Bits */
#define SCS_NVIC_IPR14_PRI_56__M                           (0x000000ff)          /* Priority of interrupt 56 */
#define SCS_NVIC_IPR14_PRI_57__M                           (0x0000ff00)          /* Priority of interrupt 57 */
#define SCS_NVIC_IPR14_PRI_58__M                           (0x00ff0000)          /* Priority of interrupt 58 */
#define SCS_NVIC_IPR14_PRI_59__M                           (0xff000000)          /* Priority of interrupt 59 */

/* SCS_NVIC_IPR15 Control Bits */
#define SCS_NVIC_IPR15_PRI_60__M                           (0x000000ff)          /* Priority of interrupt 60 */
#define SCS_NVIC_IPR15_PRI_61__M                           (0x0000ff00)          /* Priority of interrupt 61 */
#define SCS_NVIC_IPR15_PRI_62__M                           (0x00ff0000)          /* Priority of interrupt 62 */
#define SCS_NVIC_IPR15_PRI_63__M                           (0xff000000)          /* Priority of interrupt 63 */

/* SCS_CPUID Control Bits */
#define SCS_CPUID_REVISION__M                              (0x0000000f)          /* Implementation defined revision number. */
#define SCS_CPUID_PARTNO__M                                (0x0000fff0)          /* Number of processor within family. */
#define SCS_CPUID_CONSTANT__M                              (0x000f0000)          /* Reads as 0xC */
#define SCS_CPUID_VARIANT__M                               (0x00f00000)          /* Implementation defined variant number. */
#define SCS_CPUID_IMPLEMENTER__M                           (0xff000000)          /* Implementor code. */

/* SCS_ICSR Control Bits */
#define SCS_ICSR_VECTACTIVE__M                             (0x000001ff)          /* Active ISR number field. Reset clears the VECTACTIVE field. */
#define SCS_ICSR_RETTOBASE                                 (0x00000800)          /* This bit is 1 when the set of all active exceptions minus the  */
                                                                                 /* IPSR_current_exception yields the empty set. */
#define SCS_ICSR_VECTPENDING__M                            (0x0003f000)          /* Pending ISR number field. VECTPENDING contains the interrupt number of the  */
                                                                                 /* highest priority pending ISR. */
#define SCS_ICSR_ISRPENDING                                (0x00400000)          /* Interrupt pending flag. Excludes NMI and faults. */
#define SCS_ICSR_ISRPREEMPT                                (0x00800000)          /* You must only use this at debug time. It indicates that a pending interrupt is  */
                                                                                 /* to be taken in the next running cycle. If C_MASKINTS is clear in the Debug  */
                                                                                 /* Halting Control and Status Register, the interrupt is serviced. */
#define SCS_ICSR_PENDSTCLR                                 (0x02000000)          /* Clear pending SysTick bit */
#define SCS_ICSR_PENDSTSET                                 (0x04000000)          /* Set a pending SysTick bit. */
#define SCS_ICSR_PENDSVCLR                                 (0x08000000)          /* Clear pending pendSV bit */
#define SCS_ICSR_PENDSVSET                                 (0x10000000)          /* Set pending pendSV bit. */
#define SCS_ICSR_NMIPENDSET                                (0x80000000)          /* Set pending NMI bit. NMIPENDSET pends and activates an NMI. Because NMI is the  */
                                                                                 /* highest-priority interrupt, it takes effect as soon as it registers. */

/* SCS_VTOR Control Bits */
#define SCS_VTOR_TBLOFF__M                                 (0x1fffff80)          /* Vector table base offset field. Contains the offset of the table base from the  */
                                                                                 /* bottom of the SRAM or CODE space. */
#define SCS_VTOR_TBLBASE                                   (0x20000000)          /* Table base is in Code (0) or RAM (1). */

/* SCS_AIRCR Control Bits */
#define SCS_AIRCR_VECTRESET                                (0x00000001)          /* System Reset bit. Resets the system, with the exception of debug components. The */
                                                                                 /* VECTRESET bit self-clears. Reset clears the VECTRESET bit. For debugging, only  */
                                                                                 /* write this bit when the core is halted. */
#define SCS_AIRCR_VECTCLRACTIVE                            (0x00000002)          /* Clears all active state information for active NMI, fault, and interrupts.  It  */
                                                                                 /* is the responsibility of the application to reinitialize the stack. The  */
                                                                                 /* VECTCLRACTIVE bit is for returning to a known state during debug. The  */
                                                                                 /* VECTCLRACTIVE bit self-clears. IPSR is not cleared by this operation. So, if  */
                                                                                 /* used by an application, it must only be used at the base level of activation, or */
                                                                                 /* within a system handler whose active bit can be set. */
#define SCS_AIRCR_SYSRESETREQ                              (0x00000004)          /* Causes a signal to be asserted to the outer system that indicates a reset is  */
                                                                                 /* requested. Intended to force a large system reset of all major components except */
                                                                                 /* for debug. Setting this bit does not prevent Halting Debug from running. */
#define SCS_AIRCR_PRIGROUP__M                              (0x00000700)          /* Interrupt priority grouping field. The PRIGROUP field is a binary point position */
                                                                                 /* indicator for creating subpriorities for exceptions that share the same pre- */
                                                                                 /* emption level. It divides the PRI_n field in the Interrupt Priority Register  */
                                                                                 /* into a pre-emption level and a subpriority level. The binary point is a left-of  */
                                                                                 /* value. This means that the PRIGROUP value represents a point starting at the  */
                                                                                 /* left of the Least Significant Bit (LSB). This is bit [0] of 7:0. The lowest  */
                                                                                 /* value might not be 0 depending on the number of bits allocated for priorities,  */
                                                                                 /* and implementation choices  */
#define SCS_AIRCR_ENDIANESS                                (0x00008000)          /* Data endianness bit. ENDIANNESS is sampled from the BIGEND input port during  */
                                                                                 /* reset. You cannot change ENDIANNESS outside of reset. */
#define SCS_AIRCR_VECTKEY__M                               (0xffff0000)          /* Register key. Writing to this register requires 0x5FA in the VECTKEY field.  */
                                                                                 /* Otherwise the write value is ignored. */

/* SCS_SCR Control Bits */
#define SCS_SCR_SLEEPONEXIT                                (0x00000002)          /* Sleep on exit when returning from Handler mode to Thread mode. Enables interrupt */
                                                                                 /* driven applications to avoid returning to empty main application. */
#define SCS_SCR_SLEEPDEEP                                  (0x00000004)          /* Sleep deep bit. */
#define SCS_SCR_SEVONPEND                                  (0x00000010)          /* When enabled, this causes WFE to wake up when an interrupt moves from inactive  */
                                                                                 /* to pended. Otherwise, WFE only wakes up from an event signal, external and SEV  */
                                                                                 /* instruction generated. The event input, RXEV, is registered even when not  */
                                                                                 /* waiting for an event, and so effects the next WFE. */

/* SCS_CCR Control Bits */
#define SCS_CCR_NONBASETHREDENA                            (0x00000001)          /* When 0, default, It is only possible to enter Thread mode when returning from  */
                                                                                 /* the last exception. When set to 1, Thread mode can be entered from any level in  */
                                                                                 /* Handler mode by controlled return value. */
#define SCS_CCR_USERSETMPEND                               (0x00000002)          /* If written as 1, enables user code to write the Software Trigger Interrupt  */
                                                                                 /* register to trigger (pend) a Main exception, which is one associated with the  */
                                                                                 /* Main stack pointer. */
#define SCS_CCR_UNALIGN_TRP                                (0x00000008)          /* Trap for unaligned access. This enables faulting/halting on any unaligned half  */
                                                                                 /* or full word access. Unaligned load-store multiples always fault. The relevant  */
                                                                                 /* Usage Fault Status Register bit is UNALIGNED. */
#define SCS_CCR_DIV_0_TRP                                  (0x00000010)          /* Trap on Divide by 0. This enables faulting/halting when an attempt is made to  */
                                                                                 /* divide by 0. The relevant Usage Fault Status Register bit is DIVBYZERO. */
#define SCS_CCR_BFHFNMIGN                                  (0x00000100)          /* When enabled, this causes handlers running at priority -1 and -2 (Hard Fault,  */
                                                                                 /* NMI, and FAULTMASK escalated handlers) to ignore Data Bus faults caused by load  */
                                                                                 /* and store instructions. When disabled, these bus faults cause a lock-up. You  */
                                                                                 /* must only use this enable with extreme caution. All data bus faults are ignored  */
                                                                                 /* therefore you must only use it when the handler and its data are in absolutely  */
                                                                                 /* safe memory. Its normal use is to probe system devices and bridges to detect  */
                                                                                 /* control path problems and fix them. */
#define SCS_CCR_STKALIGN                                   (0x00000200)          /* Stack alignment bit. */

/* SCS_SHPR1 Control Bits */
#define SCS_SHPR1_PRI_4__M                                 (0x000000ff)          /* Priority of system handler 4. */
#define SCS_SHPR1_PRI_5__M                                 (0x0000ff00)          /* Priority of system handler 5. */
#define SCS_SHPR1_PRI_6__M                                 (0x00ff0000)          /* Priority of system handler 6. */
#define SCS_SHPR1_PRI_7__M                                 (0xff000000)          /* Priority of system handler 7. */

/* SCS_SHPR2 Control Bits */
#define SCS_SHPR2_PRI_8__M                                 (0x000000ff)          /* Priority of system handler 8. */
#define SCS_SHPR2_PRI_9__M                                 (0x0000ff00)          /* Priority of system handler 9. */
#define SCS_SHPR2_PRI_10__M                                (0x00ff0000)          /* Priority of system handler 10. */
#define SCS_SHPR2_PRI_11__M                                (0xff000000)          /* Priority of system handler 11. */

/* SCS_SHPR3 Control Bits */
#define SCS_SHPR3_PRI_12__M                                (0x000000ff)          /* Priority of system handler 12. */
#define SCS_SHPR3_PRI_13__M                                (0x0000ff00)          /* Priority of system handler 13. */
#define SCS_SHPR3_PRI_14__M                                (0x00ff0000)          /* Priority of system handler 14. */
#define SCS_SHPR3_PRI_15__M                                (0xff000000)          /* Priority of system handler 15. */

/* SCS_SHCSR Control Bits */
#define SCS_SHCSR_MEMFAULTACT                              (0x00000001)          /* MemManage active flag. */
#define SCS_SHCSR_BUSFAULTACT                              (0x00000002)          /* BusFault active flag. */
#define SCS_SHCSR_USGFAULTACT                              (0x00000008)          /* UsageFault active flag. */
#define SCS_SHCSR_SVCALLACT                                (0x00000080)          /* SVCall active flag. */
#define SCS_SHCSR_MONITORACT                               (0x00000100)          /* the Monitor active flag. */
#define SCS_SHCSR_PENDSVACT                                (0x00000400)          /* PendSV active flag. */
#define SCS_SHCSR_SYSTICKACT                               (0x00000800)          /* SysTick active flag. */
#define SCS_SHCSR_USGFAULTPENDED                           (0x00001000)          /* usage fault pended flag. */
#define SCS_SHCSR_MEMFAULTPENDED                           (0x00002000)          /* MemManage pended flag. */
#define SCS_SHCSR_BUSFAULTPENDED                           (0x00004000)          /* BusFault pended flag. */
#define SCS_SHCSR_SVCALLPENDED                             (0x00008000)          /* SVCall pended flag. */
#define SCS_SHCSR_MEMFAULTENA                              (0x00010000)          /* MemManage fault system handler enable */
#define SCS_SHCSR_BUSFAULTENA                              (0x00020000)          /* Bus fault system handler enable */
#define SCS_SHCSR_USGFAULTENA                              (0x00040000)          /* Usage fault system handler enable */

/* SCS_CFSR Control Bits */
#define SCS_CFSR_IACCVIOL                                  (0x00000001)          /* Instruction access violation flag. Attempting to fetch an instruction from a  */
                                                                                 /* location that does not permit execution sets the IACCVIOL flag. This occurs on  */
                                                                                 /* any access to an XN region, even when the MPU is disabled or not present. The  */
                                                                                 /* return PC points to the faulting instruction. The MMAR is not written. */
#define SCS_CFSR_DACCVIOL                                  (0x00000002)          /* Data access violation flag. Attempting to load or store at a location that does  */
                                                                                 /* not permit the operation sets the DACCVIOL flag. The return PC points to the  */
                                                                                 /* faulting instruction. This error loads MMAR with the address of the attempted  */
                                                                                 /* access. */
#define SCS_CFSR_MUNSTKERR                                 (0x00000008)          /* Unstack from exception return has caused one or more access violations. This is  */
                                                                                 /* chained to the handler, so that the original return stack is still present. SP  */
                                                                                 /* is not adjusted from failing return and new save is not performed. The MMAR is  */
                                                                                 /* not written. */
#define SCS_CFSR_MSTKERR                                   (0x00000010)          /* Stacking from exception has caused one or more access violations. The SP is  */
                                                                                 /* still adjusted and the values in the context area on the stack might be  */
                                                                                 /* incorrect. The MMAR is not written. */
#define SCS_CFSR_MMARVALID                                 (0x00000080)          /* Memory Manage Address Register (MMAR) address valid flag. A later-arriving fault */
                                                                                 /* , such as a bus fault, can clear a memory manage fault.. If a MemManage fault  */
                                                                                 /* occurs that is escalated to a Hard Fault because of priority, the Hard Fault  */
                                                                                 /* handler must clear this bit. This prevents problems on return to a stacked  */
                                                                                 /* active MemManage handler whose MMAR value has been overwritten. */
#define SCS_CFSR_IBUSERR                                   (0x00000100)          /* Instruction bus error flag. The IBUSERR flag is set by a prefetch error. The  */
                                                                                 /* fault stops on the instruction, so if the error occurs under a branch shadow, no */
                                                                                 /* fault occurs. The BFAR is not written. */
#define SCS_CFSR_PRECISERR                                 (0x00000200)          /* Precise data bus error return. */
#define SCS_CFSR_IMPRECISERR                               (0x00000400)          /* Imprecise data bus error. It is a BusFault, but the Return PC is not related to  */
                                                                                 /* the causing instruction. This is not a synchronous fault. So, if detected when  */
                                                                                 /* the priority of the current activation is higher than the Bus Fault, it only  */
                                                                                 /* pends. Bus fault activates when returning to a lower priority activation. If a  */
                                                                                 /* precise fault occurs before returning to a lower priority exception, the handler */
                                                                                 /* detects both IMPRECISERR set and one of the precise fault status bits set at  */
                                                                                 /* the same time. The BFAR is not written. */
#define SCS_CFSR_UNSTKERR                                  (0x00000800)          /* Unstack from exception return has caused one or more bus faults. This is chained */
                                                                                 /* to the handler, so that the original return stack is still present. SP is not  */
                                                                                 /* adjusted from failing return and new save is not performed. The BFAR is not  */
                                                                                 /* written. */
#define SCS_CFSR_STKERR                                    (0x00001000)          /* Stacking from exception has caused one or more bus faults. The SP is still  */
                                                                                 /* adjusted and the values in the context area on the stack might be incorrect. The */
                                                                                 /* BFAR is not written. */
#define SCS_CFSR_BFARVALID                                 (0x00008000)          /* This bit is set if the Bus Fault Address Register (BFAR) contains a valid  */
                                                                                 /* address. This is true after a bus fault where the address is known. Other faults */
                                                                                 /* can clear this bit, such as a Mem Manage fault occurring later. If a Bus fault  */
                                                                                 /* occurs that is escalated to a Hard Fault because of priority, the Hard Fault  */
                                                                                 /* handler must clear this bit. This prevents problems if returning to a stacked  */
                                                                                 /* active Bus fault handler whose BFAR value has been overwritten. */
#define SCS_CFSR_UNDEFINSTR                                (0x00010000)          /* The UNDEFINSTR flag is set when the processor attempts to execute an undefined  */
                                                                                 /* instruction. This is an instruction that the processor cannot decode. The return */
                                                                                 /* PC points to the undefined instruction. */
#define SCS_CFSR_INVSTATE                                  (0x00020000)          /* Invalid combination of EPSR and instruction, for reasons other than UNDEFINED  */
                                                                                 /* instruction. Return PC points to faulting instruction, with the invalid state. */
#define SCS_CFSR_INVPC                                     (0x00040000)          /* Attempt to load EXC_RETURN into PC illegally. Invalid instruction, invalid  */
                                                                                 /* context, invalid value. The return PC points to the instruction that tried to  */
                                                                                 /* set the PC. */
#define SCS_CFSR_NOCP                                      (0x00080000)          /* Attempt to use a coprocessor instruction. The processor does not support  */
                                                                                 /* coprocessor instructions. */
#define SCS_CFSR_UNALIGNED                                 (0x01000000)          /* When UNALIGN_TRP is enabled (see Configuration Control Register on page 8-26),  */
                                                                                 /* and there is an attempt to make an unaligned memory access, then this fault  */
                                                                                 /* occurs. Unaligned LDM/STM/LDRD/STRD instructions always fault irrespective of  */
                                                                                 /* the setting of UNALIGN_TRP. */
#define SCS_CFSR_DIVBYZERO                                 (0x02000000)          /* When DIV_0_TRP (see Configuration Control Register on page 8-26) is enabled and  */
                                                                                 /* an SDIV or UDIV instruction is used with a divisor of 0, this fault occurs The  */
                                                                                 /* instruction is executed and the return PC points to it. If DIV_0_TRP is not set, */
                                                                                 /* then the divide returns a quotient of 0. */

/* SCS_HFSR Control Bits */
#define SCS_HFSR_VECTTBL                                   (0x00000002)          /* This bit is set if there is a fault because of vector table read on exception  */
                                                                                 /* processing (Bus Fault). This case is always a Hard Fault. The return PC points  */
                                                                                 /* to the pre-empted instruction. */
#define SCS_HFSR_FORCED                                    (0x40000000)          /* Hard Fault activated because a Configurable Fault was received and cannot  */
                                                                                 /* activate because of priority or because the Configurable Fault is disabled. The  */
                                                                                 /* Hard Fault handler then has to read the other fault status registers to  */
                                                                                 /* determine cause. */
#define SCS_HFSR_DEBUGEVT                                  (0x80000000)          /* This bit is set if there is a fault related to debug. This is only possible when */
                                                                                 /* halting debug is not enabled. For monitor enabled debug, it only happens for  */
                                                                                 /* BKPT when the current priority is higher than the monitor. When both halting and */
                                                                                 /* monitor debug are disabled, it only happens for debug events that are not  */
                                                                                 /* ignored (minimally, BKPT). The Debug Fault Status Register is updated. */

/* SCS_DFSR Control Bits */
#define SCS_DFSR_HALTED                                    (0x00000001)          /* Halt request flag. The processor is halted on the next instruction. */
#define SCS_DFSR_BKPT                                      (0x00000002)          /* BKPT flag. The BKPT flag is set by a BKPT instruction in flash patch code, and  */
                                                                                 /* also by normal code. Return PC points to breakpoint containing instruction. */
#define SCS_DFSR_DWTTRAP                                   (0x00000004)          /* Data Watchpoint and Trace (DWT) flag. The processor stops at the current  */
                                                                                 /* instruction or at the next instruction. */
#define SCS_DFSR_VCATCH                                    (0x00000008)          /* Vector catch flag. When the VCATCH flag is set, a flag in one of the local fault */
                                                                                 /* status registers is also set to indicate the type of fault. */
#define SCS_DFSR_EXTERNAL                                  (0x00000010)          /* External debug request flag. The processor stops on next instruction boundary. */

/* SCS_ID_PFR0 Control Bits */
#define SCS_ID_PFR0_STATE0__M                              (0x0000000f)          /* State0 (T-bit == 0) */
#define SCS_ID_PFR0_STATE0__0                              (0x00000000)          /* no ARM encoding */
#define SCS_ID_PFR0_STATE0__0_NO_ARM_ENCODING              (0x00000000)          /* no ARM encoding */
#define SCS_ID_PFR0_STATE0__1                              (0x00000001)          /* N/A */
#define SCS_ID_PFR0_STATE0__1_N_A                          (0x00000001)          /* N/A */
#define SCS_ID_PFR0_STATE1__M                              (0x000000f0)          /* State1 (T-bit == 1) */
#define SCS_ID_PFR0_STATE1__0                              (0x00000000)          /* N/A */
#define SCS_ID_PFR0_STATE1__0_N_A                          (0x00000000)          /* N/A */
#define SCS_ID_PFR0_STATE1__1                              (0x00000010)          /* N/A */
#define SCS_ID_PFR0_STATE1__1_N_A                          (0x00000010)          /* N/A */
#define SCS_ID_PFR0_STATE1__2                              (0x00000020)          /* Thumb-2 encoding with the 16-bit basic instructions plus 32-bit Buncond/BL but no other 32-bit basic instructions (Note non-basic 32-bit instructions can be added using the appropriate instruction attribute, but other 32-bit basic instructions cannot.) */
#define SCS_ID_PFR0_STATE1__3                              (0x00000030)          /* Thumb-2 encoding with all Thumb-2 basic instructions */

/* SCS_ID_PFR1 Control Bits */
#define SCS_ID_PFR1_MICROCONTROLLER_PROGRAMMERS_MODEL__M   (0x00000f00)          /* Microcontroller programmer's model */
#define SCS_ID_PFR1_MICROCONTROLLER_PROGRAMMERS_MODEL__0   (0x00000000)          /* not supported */
#define SCS_ID_PFR1_MICROCONTROLLER_PROGRAMMERS_MODEL__0_NOT_SUPPORTED (0x00000000)          /* not supported */
#define SCS_ID_PFR1_MICROCONTROLLER_PROGRAMMERS_MODEL__2   (0x00000200)          /* two-stack support */

/* SCS_ID_DFR0 Control Bits */
#define SCS_ID_DFR0_MICROCONTROLLER_DEBUG_MODEL__M         (0x00f00000)          /* Microcontroller Debug Model - memory mapped */
#define SCS_ID_DFR0_MICROCONTROLLER_DEBUG_MODEL__0         (0x00000000)          /* not supported */
#define SCS_ID_DFR0_MICROCONTROLLER_DEBUG_MODEL__0_NOT_SUPPORTED (0x00000000)          /* not supported */
#define SCS_ID_DFR0_MICROCONTROLLER_DEBUG_MODEL__1         (0x00100000)          /* Microcontroller debug v1 (ITMv1, DWTv1, optional ETM) */

/* SCS_ID_MMFR0 Control Bits */
#define SCS_ID_MMFR0_PMSA_SUPPORT__M                       (0x000000f0)          /* PMSA support */
#define SCS_ID_MMFR0_PMSA_SUPPORT__0                       (0x00000000)          /* not supported */
#define SCS_ID_MMFR0_PMSA_SUPPORT__0_NOT_SUPPORTED         (0x00000000)          /* not supported */
#define SCS_ID_MMFR0_PMSA_SUPPORT__1                       (0x00000010)          /* IMPLEMENTATION DEFINED (N/A) */
#define SCS_ID_MMFR0_PMSA_SUPPORT__2                       (0x00000020)          /* PMSA base (features as defined for ARMv6) (N/A) */
#define SCS_ID_MMFR0_PMSA_SUPPORT__3                       (0x00000030)          /* PMSAv7 (base plus subregion support) */
#define SCS_ID_MMFR0_CACHE_COHERENCE_SUPPORT__M            (0x00000f00)          /* Cache coherence support */
#define SCS_ID_MMFR0_CACHE_COHERENCE_SUPPORT__0            (0x00000000)          /* no shared support */
#define SCS_ID_MMFR0_CACHE_COHERENCE_SUPPORT__1            (0x00000100)          /* partial-inner-shared coherency (coherency amongst some - but not all - of the entities within an inner-coherent domain) */
#define SCS_ID_MMFR0_CACHE_COHERENCE_SUPPORT__2            (0x00000200)          /* full-inner-shared coherency (coherency amongst all of the entities within an inner-coherent domain) */
#define SCS_ID_MMFR0_CACHE_COHERENCE_SUPPORT__3            (0x00000300)          /* full coherency (coherency amongst all of the entities) */
#define SCS_ID_MMFR0_OUTER_NON_SHARABLE_SUPPORT__M         (0x0000f000)          /* Outer non-sharable support */
#define SCS_ID_MMFR0_OUTER_NON_SHARABLE_SUPPORT__0         (0x00000000)          /* Outer non-sharable not supported */
#define SCS_ID_MMFR0_OUTER_NON_SHARABLE_SUPPORT__1         (0x00001000)          /* Outer sharable supported */
#define SCS_ID_MMFR0_AUILIARY_REGISTER_SUPPORT__M          (0x00f00000)          /* Auxiliary register support */
#define SCS_ID_MMFR0_AUILIARY_REGISTER_SUPPORT__0          (0x00000000)          /* not supported */
#define SCS_ID_MMFR0_AUILIARY_REGISTER_SUPPORT__0_NOT_SUPPORTED (0x00000000)          /* not supported */
#define SCS_ID_MMFR0_AUILIARY_REGISTER_SUPPORT__1          (0x00100000)          /* Auxiliary control register */

/* SCS_ID_MMFR2 Control Bits */
#define SCS_ID_MMFR2_WAIT_FOR_INTERRUPT_STALLING__M        (0x0f000000)          /* wait for interrupt stalling */
#define SCS_ID_MMFR2_WAIT_FOR_INTERRUPT_STALLING__0        (0x00000000)          /* not supported */
#define SCS_ID_MMFR2_WAIT_FOR_INTERRUPT_STALLING__0_NOT_SUPPORTED (0x00000000)          /* not supported */
#define SCS_ID_MMFR2_WAIT_FOR_INTERRUPT_STALLING__1        (0x01000000)          /* wait for interrupt supported */

/* SCS_ISAR0 Control Bits */
#define SCS_ISAR0_BITCOUNT_INSTRS__M                       (0x000000f0)          /* BitCount instructions */
#define SCS_ISAR0_BITCOUNT_INSTRS__0                       (0x00000000)          /* no bit-counting instructions present */
#define SCS_ISAR0_BITCOUNT_INSTRS__1                       (0x00000010)          /* adds CLZ */
#define SCS_ISAR0_BITCOUNT_INSTRS__1_ADDS_CLZ              (0x00000010)          /* adds CLZ */
#define SCS_ISAR0_BITFIELD_INSTRS__M                       (0x00000f00)          /* BitField instructions */
#define SCS_ISAR0_BITFIELD_INSTRS__0                       (0x00000000)          /* no bitfield instructions present */
#define SCS_ISAR0_BITFIELD_INSTRS__1                       (0x00000100)          /* adds BFC, BFI, SBFX, UBFX */
#define SCS_ISAR0_CMPBRANCH_INSTRS__M                      (0x0000f000)          /* CmpBranch instructions */
#define SCS_ISAR0_CMPBRANCH_INSTRS__0                      (0x00000000)          /* no combined compare-and-branch instructions present */
#define SCS_ISAR0_CMPBRANCH_INSTRS__1                      (0x00001000)          /* adds CB{N}Z */
#define SCS_ISAR0_CMPBRANCH_INSTRS__1_ADDS_CB_N_Z          (0x00001000)          /* adds CB{N}Z */
#define SCS_ISAR0_COPROC_INSTRS__M                         (0x000f0000)          /* Coprocessor instructions */
#define SCS_ISAR0_COPROC_INSTRS__0                         (0x00000000)          /* no coprocessor support, other than for separately attributed architectures such as CP15 or VFP */
#define SCS_ISAR0_COPROC_INSTRS__1                         (0x00010000)          /* adds generic CDP, LDC, MCR, MRC, STC */
#define SCS_ISAR0_COPROC_INSTRS__2                         (0x00020000)          /* adds generic CDP2, LDC2, MCR2, MRC2, STC2 */
#define SCS_ISAR0_COPROC_INSTRS__3                         (0x00030000)          /* adds generic MCRR, MRRC */
#define SCS_ISAR0_COPROC_INSTRS__4                         (0x00040000)          /* adds generic MCRR2, MRRC2 */
#define SCS_ISAR0_DEBUG_INSTRS__M                          (0x00f00000)          /* Debug instructions */
#define SCS_ISAR0_DEBUG_INSTRS__0                          (0x00000000)          /* no debug instructions present */
#define SCS_ISAR0_DEBUG_INSTRS__1                          (0x00100000)          /* adds BKPT */
#define SCS_ISAR0_DEBUG_INSTRS__1_ADDS_BKPT                (0x00100000)          /* adds BKPT */
#define SCS_ISAR0_DIVIDE_INSTRS__M                         (0x0f000000)          /* Divide instructions */
#define SCS_ISAR0_DIVIDE_INSTRS__0                         (0x00000000)          /* no divide instructions present */
#define SCS_ISAR0_DIVIDE_INSTRS__1                         (0x01000000)          /* adds SDIV, UDIV (v1 quotient only result) */

/* SCS_ID_ISAR1 Control Bits */
#define SCS_ID_ISAR1_ETEND_INSRS__M                        (0x0000f000)          /* Extend instructions. Note that the shift options on these instructions are also  */
                                                                                 /* controlled by the WithShifts_instrs attribute. */
#define SCS_ID_ISAR1_ETEND_INSRS__0                        (0x00000000)          /* no scalar (i.e. non-SIMD) sign/zero-extend instructions present */
#define SCS_ID_ISAR1_ETEND_INSRS__1                        (0x00001000)          /* adds SXTB, SXTH, UXTB, UXTH */
#define SCS_ID_ISAR1_ETEND_INSRS__2                        (0x00002000)          /* N/A */
#define SCS_ID_ISAR1_ETEND_INSRS__2_N_A                    (0x00002000)          /* N/A */
#define SCS_ID_ISAR1_IFTHEN_INSTRS__M                      (0x000f0000)          /* IfThen instructions */
#define SCS_ID_ISAR1_IFTHEN_INSTRS__0                      (0x00000000)          /* IT instructions not present */
#define SCS_ID_ISAR1_IFTHEN_INSTRS__1                      (0x00010000)          /* adds IT instructions (and IT bits in PSRs) */
#define SCS_ID_ISAR1_IMMEDIATE_INSTRS__M                   (0x00f00000)          /* Immediate instructions */
#define SCS_ID_ISAR1_IMMEDIATE_INSTRS__0                   (0x00000000)          /* no special immediate-generating instructions present */
#define SCS_ID_ISAR1_IMMEDIATE_INSTRS__1                   (0x00100000)          /* adds ADDW, MOVW, MOVT, SUBW */
#define SCS_ID_ISAR1_INTERWORK_INSTRS__M                   (0x0f000000)          /* Interwork instructions */
#define SCS_ID_ISAR1_INTERWORK_INSTRS__0                   (0x00000000)          /* no interworking instructions supported */
#define SCS_ID_ISAR1_INTERWORK_INSTRS__1                   (0x01000000)          /* adds BX (and T bit in PSRs) */
#define SCS_ID_ISAR1_INTERWORK_INSTRS__2                   (0x02000000)          /* adds BLX, and PC loads have BX-like behavior */
#define SCS_ID_ISAR1_INTERWORK_INSTRS__3                   (0x03000000)          /* N/A */
#define SCS_ID_ISAR1_INTERWORK_INSTRS__3_N_A               (0x03000000)          /* N/A */

/* SCS_ID_ISAR2 Control Bits */
#define SCS_ID_ISAR2_LOADSTORE_INSTRS__M                   (0x0000000f)          /* LoadStore instructions */
#define SCS_ID_ISAR2_LOADSTORE_INSTRS__0                   (0x00000000)          /* no additional normal load/store instructions present */
#define SCS_ID_ISAR2_LOADSTORE_INSTRS__1                   (0x00000001)          /* adds LDRD/STRD */
#define SCS_ID_ISAR2_LOADSTORE_INSTRS__1_ADDS_LDRD_STRD    (0x00000001)          /* adds LDRD/STRD */
#define SCS_ID_ISAR2_MEMHINT_INSTRS__M                     (0x000000f0)          /* MemoryHint instructions */
#define SCS_ID_ISAR2_MEMHINT_INSTRS__0                     (0x00000000)          /* no memory hint instructions presen */
#define SCS_ID_ISAR2_MEMHINT_INSTRS__1                     (0x00000010)          /* adds PLD */
#define SCS_ID_ISAR2_MEMHINT_INSTRS__1_ADDS_PLD            (0x00000010)          /* adds PLD */
#define SCS_ID_ISAR2_MEMHINT_INSTRS__2                     (0x00000020)          /* adds PLD (ie a repeat on value 1) */
#define SCS_ID_ISAR2_MEMHINT_INSTRS__3                     (0x00000030)          /* adds PLI */
#define SCS_ID_ISAR2_MEMHINT_INSTRS__3_ADDS_PLI            (0x00000030)          /* adds PLI */
#define SCS_ID_ISAR2_MULTIACCESSINT_INSTRS__M              (0x00000f00)          /* Multi-Access interruptible instructions */
#define SCS_ID_ISAR2_MULTIACCESSINT_INSTRS__0              (0x00000000)          /* the (LDM/STM) instructions are non-interruptible */
#define SCS_ID_ISAR2_MULTIACCESSINT_INSTRS__1              (0x00000100)          /* the (LDM/STM) instructions are restartable */
#define SCS_ID_ISAR2_MULTIACCESSINT_INSTRS__2              (0x00000200)          /* the (LDM/STM) instructions are continuable */
#define SCS_ID_ISAR2_MULT_INSTRS__M                        (0x0000f000)          /* Multiply instructions */
#define SCS_ID_ISAR2_MULT_INSTRS__0                        (0x00000000)          /* only MUL present */
#define SCS_ID_ISAR2_MULT_INSTRS__1                        (0x00001000)          /* adds MLA */
#define SCS_ID_ISAR2_MULT_INSTRS__1_ADDS_MLA               (0x00001000)          /* adds MLA */
#define SCS_ID_ISAR2_MULT_INSTRS__2                        (0x00002000)          /* adds MLS */
#define SCS_ID_ISAR2_MULT_INSTRS__2_ADDS_MLS               (0x00002000)          /* adds MLS */
#define SCS_ID_ISAR2_MULTS_INSTRS__M                       (0x000f0000)          /* Multiply instructions (advanced, signed) */
#define SCS_ID_ISAR2_MULTS_INSTRS__0                       (0x00000000)          /* no signed multiply instructions present */
#define SCS_ID_ISAR2_MULTS_INSTRS__1                       (0x00010000)          /* adds SMULL, SMLAL */
#define SCS_ID_ISAR2_MULTS_INSTRS__2                       (0x00020000)          /* N/A */
#define SCS_ID_ISAR2_MULTS_INSTRS__2_N_A                   (0x00020000)          /* N/A */
#define SCS_ID_ISAR2_MULTS_INSTRS__3                       (0x00030000)          /* N/A */
#define SCS_ID_ISAR2_MULTS_INSTRS__3_N_A                   (0x00030000)          /* N/A */
#define SCS_ID_ISAR2_MULTU_INSTRS__M                       (0x00f00000)          /* Multiply instructions (advanced, unsigned) */
#define SCS_ID_ISAR2_MULTU_INSTRS__0                       (0x00000000)          /* no unsigned multiply instructions present */
#define SCS_ID_ISAR2_MULTU_INSTRS__1                       (0x00100000)          /* adds UMULL, UMLAL */
#define SCS_ID_ISAR2_MULTU_INSTRS__2                       (0x00200000)          /* N/A */
#define SCS_ID_ISAR2_MULTU_INSTRS__2_N_A                   (0x00200000)          /* N/A */
#define SCS_ID_ISAR2_REVERSAL_INSTRS__M                    (0xf0000000)          /* Reversal instructions */
#define SCS_ID_ISAR2_REVERSAL_INSTRS__0                    (0x00000000)          /* no reversal instructions present */
#define SCS_ID_ISAR2_REVERSAL_INSTRS__1                    (0x10000000)          /* adds REV, REV16, REVSH */
#define SCS_ID_ISAR2_REVERSAL_INSTRS__2                    (0x20000000)          /* adds RBIT */
#define SCS_ID_ISAR2_REVERSAL_INSTRS__2_ADDS_RBIT          (0x20000000)          /* adds RBIT */

/* SCS_ID_ISAR3 Control Bits */
#define SCS_ID_ISAR3_SATRUATE_INSTRS__M                    (0x0000000f)          /* Saturate instructions */
#define SCS_ID_ISAR3_SATRUATE_INSTRS__0                    (0x00000000)          /* no non-SIMD saturate instructions present */
#define SCS_ID_ISAR3_SATRUATE_INSTRS__1                    (0x00000001)          /* N/A */
#define SCS_ID_ISAR3_SATRUATE_INSTRS__1_N_A                (0x00000001)          /* N/A */
#define SCS_ID_ISAR3_SIMD_INSTRS__M                        (0x000000f0)          /* SIMD instructions */
#define SCS_ID_ISAR3_SIMD_INSTRS__0                        (0x00000000)          /* no SIMD instructions present */
#define SCS_ID_ISAR3_SIMD_INSTRS__1                        (0x00000010)          /* adds SSAT, USAT (and the Q flag in the PSRs) */
#define SCS_ID_ISAR3_SIMD_INSTRS__3                        (0x00000030)          /* N/A */
#define SCS_ID_ISAR3_SIMD_INSTRS__3_N_A                    (0x00000030)          /* N/A */
#define SCS_ID_ISAR3_SVC_INSTRS__M                         (0x00000f00)          /* SVC instructions */
#define SCS_ID_ISAR3_SVC_INSTRS__0                         (0x00000000)          /* no SVC (SWI) instructions present */
#define SCS_ID_ISAR3_SVC_INSTRS__1                         (0x00000100)          /* adds SVC (SWI) */
#define SCS_ID_ISAR3_SVC_INSTRS__1_ADDS_SVC__SWI           (0x00000100)          /* adds SVC (SWI) */
#define SCS_ID_ISAR3_SYNCPRIM_INSTRS__M                    (0x0000f000)          /* SyncPrim instructions. Note there are no LDREXD or STREXD in ARMv7-M. This  */
                                                                                 /* attribute is used in conjunction with the SyncPrim_instrs_frac attribute in  */
                                                                                 /* ID_ISAR4[23:20]. */
#define SCS_ID_ISAR3_SYNCPRIM_INSTRS__0                    (0x00000000)          /* no synchronization primitives present */
#define SCS_ID_ISAR3_SYNCPRIM_INSTRS__1                    (0x00001000)          /* adds LDREX, STREX */
#define SCS_ID_ISAR3_SYNCPRIM_INSTRS__2                    (0x00002000)          /* adds LDREXB, LDREXH, LDREXD, STREXB, STREXH, STREXD, CLREX(N/A) */
#define SCS_ID_ISAR3_TABBRANCH_INSTRS__M                   (0x000f0000)          /* TableBranch instructions */
#define SCS_ID_ISAR3_TABBRANCH_INSTRS__0                   (0x00000000)          /* no table-branch instructions present */
#define SCS_ID_ISAR3_TABBRANCH_INSTRS__1                   (0x00010000)          /* adds TBB, TBH */
#define SCS_ID_ISAR3_TABBRANCH_INSTRS__1_ADDS_TBB__TBH     (0x00010000)          /* adds TBB, TBH */
#define SCS_ID_ISAR3_THUMBCOPY_INSTRS__M                   (0x00f00000)          /* ThumbCopy instructions */
#define SCS_ID_ISAR3_THUMBCOPY_INSTRS__0                   (0x00000000)          /* Thumb MOV(register) instruction does not allow low reg -> low reg */
#define SCS_ID_ISAR3_THUMBCOPY_INSTRS__1                   (0x00100000)          /* adds Thumb MOV(register) low reg -> low reg and the CPY alias */
#define SCS_ID_ISAR3_TRUENOP_INSTRS__M                     (0x0f000000)          /* TrueNOP instructions */
#define SCS_ID_ISAR3_TRUENOP_INSTRS__0                     (0x00000000)          /* true NOP instructions not present - that is, NOP instructions with no register dependencies */
#define SCS_ID_ISAR3_TRUENOP_INSTRS__1                     (0x01000000)          /* adds "true NOP", and the capability of additional "NOP compatible hints" */

/* SCS_ID_ISAR4 Control Bits */
#define SCS_ID_ISAR4_UNPRIV_INSTRS__M                      (0x0000000f)          /* Unprivileged instructions */
#define SCS_ID_ISAR4_UNPRIV_INSTRS__0                      (0x00000000)          /* no "T variant" instructions exist */
#define SCS_ID_ISAR4_UNPRIV_INSTRS__1                      (0x00000001)          /* adds LDRBT, LDRT, STRBT, STRT */
#define SCS_ID_ISAR4_UNPRIV_INSTRS__2                      (0x00000002)          /* adds LDRHT, LDRSBT, LDRSHT, STRHT */
#define SCS_ID_ISAR4_WITHSHIFTS_INSTRS__M                  (0x000000f0)          /* WithShift instructions. Note that all additions only apply in cases where the  */
                                                                                 /* encoding supports them - e.g. there is no difference between levels 3 and 4 in  */
                                                                                 /* the Thumb-2 instruction set. Also note that MOV instructions with shift options  */
                                                                                 /* should instead be treated as ASR, LSL, LSR, ROR or RRX instructions. */
#define SCS_ID_ISAR4_WITHSHIFTS_INSTRS__0                  (0x00000000)          /* non-zero shifts only support MOV and shift instructions (see notes) */
#define SCS_ID_ISAR4_WITHSHIFTS_INSTRS__1                  (0x00000010)          /* shifts of loads/stores over the range LSL 0-3 */
#define SCS_ID_ISAR4_WITHSHIFTS_INSTRS__3                  (0x00000030)          /* adds other constant shift options. */
#define SCS_ID_ISAR4_WITHSHIFTS_INSTRS__4                  (0x00000040)          /* adds register-controlled shift options. */
#define SCS_ID_ISAR4_WRITEBACK_INSTRS__M                   (0x00000f00)          /* Writeback instructions */
#define SCS_ID_ISAR4_WRITEBACK_INSTRS__0                   (0x00000000)          /* only non-writeback addressing modes present, except that LDMIA/STMDB/PUSH/POP instructions support writeback addressing. */
#define SCS_ID_ISAR4_WRITEBACK_INSTRS__1                   (0x00000100)          /* adds all currently-defined writeback addressing modes (ARMv7, Thumb-2) */
#define SCS_ID_ISAR4_BARRIER_INSTRS__M                     (0x000f0000)          /* Barrier instructions */
#define SCS_ID_ISAR4_BARRIER_INSTRS__0                     (0x00000000)          /* no barrier instructions supported */
#define SCS_ID_ISAR4_BARRIER_INSTRS__1                     (0x00010000)          /* adds DMB, DSB, ISB barrier instructions */
#define SCS_ID_ISAR4_SYNCPRIM_INSTRS_FRAC__M               (0x00f00000)          /* SyncPrim_instrs_frac */
#define SCS_ID_ISAR4_SYNCPRIM_INSTRS_FRAC__0               (0x00000000)          /* no additional support */
#define SCS_ID_ISAR4_SYNCPRIM_INSTRS_FRAC__3               (0x00300000)          /* adds CLREX, LDREXB, STREXB, LDREXH, STREXH */
#define SCS_ID_ISAR4_PSR_M_INSTRS__M                       (0x0f000000)          /* PSR_M_instrs */
#define SCS_ID_ISAR4_PSR_M_INSTRS__0                       (0x00000000)          /* instructions not present */
#define SCS_ID_ISAR4_PSR_M_INSTRS__1                       (0x01000000)          /* adds CPS, MRS, and MSR instructions (M-profile forms) */

/* SCS_DHCSR Control Bits */
#define SCS_DHCSR_C_DEBUGEN                                (0x00000001)          /* Enables debug. This can only be written by AHB-AP and not by the core. It is  */
                                                                                 /* ignored when written by the core, which cannot set or clear it. The core must  */
                                                                                 /* write a 1 to it when writing C_HALT to halt itself. */
#define SCS_DHCSR_C_HALT                                   (0x00000002)          /* Halts the core. This bit is set automatically when the core Halts. For example  */
                                                                                 /* Breakpoint. This bit clears on core reset. This bit can only be written if  */
                                                                                 /* C_DEBUGEN is 1, otherwise it is ignored. When setting this bit to 1, C_DEBUGEN  */
                                                                                 /* must also be written to 1 in the same value (value[1:0] is 2'b11). The core can  */
                                                                                 /* halt itself, but only if C_DEBUGEN is already 1 and only if it writes with b11). */
#define SCS_DHCSR_C_STEP                                   (0x00000004)          /* Steps the core in halted debug. When C_DEBUGEN = 0, this bit has no effect. Must */
                                                                                 /* only be modified when the processor is halted (S_HALT == 1). */
#define SCS_DHCSR_C_MASKINTS                               (0x00000008)          /* Mask interrupts when stepping or running in halted debug. Does not affect NMI,  */
                                                                                 /* which is not maskable. Must only be modified when the processor is halted ( */
                                                                                 /* S_HALT == 1). Also does not affect fault exceptions and SVC caused by execution  */
                                                                                 /* of the instructions. CMASKINTS must be set or cleared before halt is released.  */
                                                                                 /* This means that the writes to set or clear C_MASKINTS and to set or clear C_HALT */
                                                                                 /* must be separate. */
#define SCS_DHCSR_C_SNAPSTALL                              (0x00000020)          /* If the core is stalled on a load/store operation the stall ceases and the  */
                                                                                 /* instruction is forced to complete. This enables Halting debug to gain control of */
                                                                                 /* the core. It can only be set if: C_DEBUGEN = 1 and C_HALT = 1. The core reads  */
                                                                                 /* S_RETIRE_ST as 0. This indicates that no instruction has advanced. This prevents */
                                                                                 /* misuse. The bus state is Unpredictable when this is used. S_RETIRE can detect  */
                                                                                 /* core stalls on load/store operations. */
#define SCS_DHCSR_S_REGRDY                                 (0x00010000)          /* Register Read/Write on the Debug Core Register Selector register is available.  */
                                                                                 /* Last transfer is complete. */
#define SCS_DHCSR_S_HALT                                   (0x00020000)          /* The core is in debug state when S_HALT is set. */
#define SCS_DHCSR_S_SLEEP                                  (0x00040000)          /* Indicates that the core is sleeping (WFI, WFE, or SLEEP-ON-EXIT). Must use  */
                                                                                 /* C_HALT to gain control or wait for interrupt to wake-up. */
#define SCS_DHCSR_S_LOCKUP                                 (0x00080000)          /* Reads as one if the core is running (not halted) and a lockup condition is  */
                                                                                 /* present. */
#define SCS_DHCSR_S_RETIRE_ST                              (0x01000000)          /* Indicates that an instruction has completed since last read. This is a sticky  */
                                                                                 /* bit that clears on read. This determines if the core is stalled on a load/store  */
                                                                                 /* or fetch. */
#define SCS_DHCSR_S_RESET_ST                               (0x02000000)          /* Indicates that the core has been reset, or is now being reset, since the last  */
                                                                                 /* time this bit was read. This a sticky bit that clears on read. So, reading twice */
                                                                                 /* and getting 1 then 0 means it was reset in the past. Reading twice and getting  */
                                                                                 /* 1 both times means that it is being reset now (held in reset still). */

/* SCS_DCRSR Control Bits */
#define SCS_DCRSR_REGSEL__M                                (0x0000001f)          /* Register select */
#define SCS_DCRSR_REGSEL__0                                (0x00000000)          /* R11 */
#define SCS_DCRSR_REGSEL__0_R11                            (0x00000000)          /* R11 */
#define SCS_DCRSR_REGSEL__0                                (0x00000000)          /* R0 */
#define SCS_DCRSR_REGSEL__0_R0                             (0x00000000)          /* R0 */
#define SCS_DCRSR_REGSEL__1                                (0x00000001)          /* R1 */
#define SCS_DCRSR_REGSEL__1_R1                             (0x00000001)          /* R1 */
#define SCS_DCRSR_REGSEL__2                                (0x00000002)          /* R2 */
#define SCS_DCRSR_REGSEL__2_R2                             (0x00000002)          /* R2 */
#define SCS_DCRSR_REGSEL__3                                (0x00000003)          /* R3 */
#define SCS_DCRSR_REGSEL__3_R3                             (0x00000003)          /* R3 */
#define SCS_DCRSR_REGSEL__4                                (0x00000004)          /* R4 */
#define SCS_DCRSR_REGSEL__4_R4                             (0x00000004)          /* R4 */
#define SCS_DCRSR_REGSEL__5                                (0x00000005)          /* R5 */
#define SCS_DCRSR_REGSEL__5_R5                             (0x00000005)          /* R5 */
#define SCS_DCRSR_REGSEL__6                                (0x00000006)          /* R6 */
#define SCS_DCRSR_REGSEL__6_R6                             (0x00000006)          /* R6 */
#define SCS_DCRSR_REGSEL__7                                (0x00000007)          /* R7 */
#define SCS_DCRSR_REGSEL__7_R7                             (0x00000007)          /* R7 */
#define SCS_DCRSR_REGSEL__8                                (0x00000008)          /* R8 */
#define SCS_DCRSR_REGSEL__8_R8                             (0x00000008)          /* R8 */
#define SCS_DCRSR_REGSEL__9                                (0x00000009)          /* R9 */
#define SCS_DCRSR_REGSEL__9_R9                             (0x00000009)          /* R9 */
#define SCS_DCRSR_REGSEL__10                               (0x0000000a)          /* R10 */
#define SCS_DCRSR_REGSEL__10_R10                           (0x0000000a)          /* R10 */
#define SCS_DCRSR_REGSEL__12                               (0x0000000c)          /* R12 */
#define SCS_DCRSR_REGSEL__12_R12                           (0x0000000c)          /* R12 */
#define SCS_DCRSR_REGSEL__13                               (0x0000000d)          /* Current SP */
#define SCS_DCRSR_REGSEL__13_CURRENT_SP                    (0x0000000d)          /* Current SP */
#define SCS_DCRSR_REGSEL__14                               (0x0000000e)          /* LR */
#define SCS_DCRSR_REGSEL__14_LR                            (0x0000000e)          /* LR */
#define SCS_DCRSR_REGSEL__15                               (0x0000000f)          /* DebugReturnAddress */
#define SCS_DCRSR_REGSEL__16                               (0x00000010)          /* xPSR/flags, execution state information, and exception number */
#define SCS_DCRSR_REGSEL__17                               (0x00000011)          /* MSP (Main SP) */
#define SCS_DCRSR_REGSEL__17_MSP__MAIN_SP                  (0x00000011)          /* MSP (Main SP) */
#define SCS_DCRSR_REGSEL__18                               (0x00000012)          /* PSP (Process SP) */
#define SCS_DCRSR_REGSEL__18_PSP__PROCESS_SP               (0x00000012)          /* PSP (Process SP) */
#define SCS_DCRSR_REGSEL__20                               (0x00000014)          /* CONTROL bits [31:24], FAULTMASK bits [23:16], BASEPRI bits [15:8], PRIMASK bits [7:0] */
#define SCS_DCRSR_REGWNR                                   (0x00010000)          /* Write = 1, Read = 0 */

/* SCS_DEMCR Control Bits */
#define SCS_DEMCR_VC_CORERESET                             (0x00000001)          /* Reset Vector Catch. Halt running system if Core reset occurs. */
#define SCS_DEMCR_VC_MMERR                                 (0x00000010)          /* Debug trap on Memory Management faults. */
#define SCS_DEMCR_VC_NOCPERR                               (0x00000020)          /* Debug trap on Usage Fault access to Coprocessor that is not present or marked as */
                                                                                 /* not present in CAR register. */
#define SCS_DEMCR_VC_CHKERR                                (0x00000040)          /* Debug trap on Usage Fault enabled checking errors. */
#define SCS_DEMCR_VC_STATERR                               (0x00000080)          /* Debug trap on Usage Fault state errors. */
#define SCS_DEMCR_VC_BUSERR                                (0x00000100)          /* Debug Trap on normal Bus error. */
#define SCS_DEMCR_VC_INTERR                                (0x00000200)          /* Debug Trap on interrupt/exception service errors. These are a subset of other  */
                                                                                 /* faults and catches before BUSERR or HARDERR. */
#define SCS_DEMCR_VC_HARDERR                               (0x00000400)          /* Debug trap on Hard Fault. */
#define SCS_DEMCR_MON_EN                                   (0x00010000)          /* Enable the debug monitor. When enabled, the System handler priority register  */
                                                                                 /* controls its priority level. If disabled, then all debug events go to Hard fault */
                                                                                 /* . C_DEBUGEN in the Debug Halting Control and Statue register overrides this bit. */
                                                                                 /* Vector catching is semi-synchronous. When a matching event is seen, a Halt is  */
                                                                                 /* requested. Because the processor can only halt on an instruction boundary, it  */
                                                                                 /* must wait until the next instruction boundary. As a result, it stops on the  */
                                                                                 /* first instruction of the exception handler. However, two special cases exist  */
                                                                                 /* when a vector catch has triggered: 1. If a fault is taken during vectoring,  */
                                                                                 /* vector read or stack push error, the halt occurs on the corresponding fault  */
                                                                                 /* handler, for the vector error or stack push. 2. If a late arriving interrupt  */
                                                                                 /* comes in during vectoring, it is not taken. That is, an implementation that  */
                                                                                 /* supports the late arrival optimization must suppress it in this case. */
#define SCS_DEMCR_MON_PEND                                 (0x00020000)          /* Pend the monitor to activate when priority permits. This can wake up the monitor */
                                                                                 /* through the AHB-AP port. It is the equivalent to C_HALT for Monitor debug. This */
                                                                                 /* register does not reset on a system reset. It is only reset by a power-on reset */
                                                                                 /* . Software in the reset handler or later, or by the DAP must enable the debug  */
                                                                                 /* monitor. */
#define SCS_DEMCR_MON_STEP                                 (0x00040000)          /* When MON_EN = 1, this steps the core. When MON_EN = 0, this bit is ignored. This */
                                                                                 /* is the equivalent to C_STEP. Interrupts are only stepped according to the  */
                                                                                 /* priority of the monitor and settings of PRIMASK, FAULTMASK, or BASEPRI. */
#define SCS_DEMCR_MON_REQ                                  (0x00080000)          /* This enables the monitor to identify how it wakes up. This bit clears on a Core  */
                                                                                 /* Reset. */
#define SCS_DEMCR_TRCENA                                   (0x01000000)          /* This bit must be set to 1 to enable use of the trace and debug blocks: Data  */
                                                                                 /* Watchpoint and Trace (DWT), Instrumentation Trace Macrocell (ITM), Embedded  */
                                                                                 /* Trace Macrocell (ETM), Trace Port Interface Unit (TPIU). This enables control of */
                                                                                 /* power usage unless tracing is required. The application can enable this, for  */
                                                                                 /* ITM use, or use by a debugger. Note that if no debug or trace components are  */
                                                                                 /* present in the implementation then it is not possible to set TRCENA. */

/* SCS_STIR Control Bits */
#define SCS_STIR_INTID__M                                  (0x000001ff)          /* Interrupt ID field. Writing a value to the INTID field is the same as manually  */
                                                                                 /* pending an interrupt by setting the corresponding interrupt bit in an Interrupt  */
                                                                                 /* Set Pending Register. */


//*****************************************************************************
// SYSCTL Registers
//*****************************************************************************
typedef struct {
  union {    /* SYSCTL_REBOOT_CTL Register */
    __IO uint32_t reg;
    struct { /* SYSCTL_REBOOT_CTL Control Bits */
      __O  uint32_t REBOOT              	:	1;  /* Write 1 initiates a Reboot of the device */
      __I  uint32_t RESERVED_2          	:	7;  /* Reserved */
      __O  uint32_t WKEY                	:	8;  /* Key to enable writes to bit 0. Bit 0 is written only if WKEY is 69h in the same  */
                                                /* write cycle. */
      __I  uint32_t RESERVED_1          	:	16;  /* Reserved */

    } bit;
  } REBOOT_CTL;
  union {    /* SYSCTL_NMI_CTLSTAT Register */
    __IO uint32_t reg;
    struct { /* SYSCTL_NMI_CTLSTAT Control Bits */
      __IO uint32_t CS_SRC              	:	1;  /*  */
      __IO uint32_t PSS_SRC             	:	1;  /*  */
      __IO uint32_t PCM_SRC             	:	1;  /*  */
      __IO uint32_t PIN_SRC             	:	1;  /* Setting this bit to 1 prevents the	RSTn pin from being used as a reset. An NMI  */
                                                /* is triggered by the pin only if a negative edge is	detected. */
      __I  uint32_t RESERVED_2          	:	12;  /* Reserved */
      __I  uint32_t CS_FLG              	:	1;  /* This flag gets auto-cleared when the corresponding source flag in the CS is	 */
                                                /* cleared */
      __I  uint32_t PSS_FLG             	:	1;  /* This flag gets auto-cleared when the corresponding source flag in the PSS is	 */
                                                /* cleared */
      __I  uint32_t PCM_FLG             	:	1;  /* This flag gets auto-cleared when the corresponding source flag in the PCM is	 */
                                                /* cleared */
      __IO uint32_t PIN_FLG             	:	1;  /*  */
      __I  uint32_t RESERVED_1          	:	12;  /* Reserved */

    } bit;
  } NMI_CTLSTAT;
  union {    /* SYSCTL_WDT_RSTCTL Register */
    __IO uint32_t reg;
    struct { /* SYSCTL_WDT_RSTCTL Control Bits */
      __IO uint32_t TIMEOUT             	:	1;  /*  */
      __IO uint32_t VIOLATION           	:	1;  /*  */
      __I  uint32_t RESERVED_1          	:	30;  /* Reserved */

    } bit;
  } WDT_RSTCTL;
  union {    /* SYSCTL_PERI_HALTCTL Register */
    __IO uint32_t reg;
    struct { /* SYSCTL_PERI_HALTCTL Control Bits */
      __IO uint32_t T16_0               	:	1;  /*  */
      __IO uint32_t T16_1               	:	1;  /*  */
      __IO uint32_t T16_2               	:	1;  /*  */
      __IO uint32_t T16_3               	:	1;  /*  */
      __IO uint32_t T32_0               	:	1;  /*  */
      __IO uint32_t EUA0                	:	1;  /*  */
      __IO uint32_t EUA1                	:	1;  /*  */
      __IO uint32_t EUA2                	:	1;  /*  */
      __IO uint32_t EUA3                	:	1;  /*  */
      __IO uint32_t EUB0                	:	1;  /*  */
      __IO uint32_t EUB1                	:	1;  /*  */
      __IO uint32_t EUB2                	:	1;  /*  */
      __IO uint32_t EUB3                	:	1;  /*  */
      __IO uint32_t ADC                 	:	1;  /*  */
      __IO uint32_t WDT                 	:	1;  /*  */
      __IO uint32_t DMA                 	:	1;  /*  */
      __I  uint32_t RESERVED_1          	:	16;  /* Reserved */

    } bit;
  } PERI_HALTCTL;
  uint32_t SRAM_SIZE; /* This register reflects the size of the SRAM available on the device. */
  union {    /* SYSCTL_SRAM_BANKEN Register */
    __IO uint32_t reg;
    struct { /* SYSCTL_SRAM_BANKEN Control Bits */
      __I  uint32_t BNK0_EN             	:	1;  /* When 1, enables Bank0 of the SRAM */
      __IO uint32_t BNK1_EN             	:	1;  /* When set to 1, bank enable bits for all banks below this bank are set to 1 as	 */
                                                /* well. For further details, see . */
      __IO uint32_t BNK2_EN             	:	1;  /* When set to 1, bank enable bits for all banks below this bank are set to 1 as	 */
                                                /* well. For further details, see . */
      __IO uint32_t BNK3_EN             	:	1;  /* When set to 1, bank enable bits for all banks below this bank are set to 1 as	 */
                                                /* well. For further details, see . */
      __IO uint32_t BNK4_EN             	:	1;  /* When set to 1, bank enable bits for all banks below this bank are set to 1 as	 */
                                                /* well. For further details, see . */
      __IO uint32_t BNK5_EN             	:	1;  /* When set to 1, bank enable bits for all banks below this bank are set to 1 as	 */
                                                /* well. For further details, see . */
      __IO uint32_t BNK6_EN             	:	1;  /* When set to 1, bank enable bits for all banks below this bank are set to 1 as	 */
                                                /* well. For further details, see . */
      __IO uint32_t BNK7_EN             	:	1;  /* When set to 1, bank enable bits for all banks below this bank are set to 1 as	 */
                                                /* well. For further details, see . */
      __I  uint32_t RESERVED_2          	:	8;  /* Reserved */
      __I  uint32_t SRAM_RDY            	:	1;  /*  */
      __I  uint32_t RESERVED_1          	:	15;  /* Reserved */

    } bit;
  } SRAM_BANKEN;
  union {    /* SYSCTL_SRAM_BANKRET Register */
    __IO uint32_t reg;
    struct { /* SYSCTL_SRAM_BANKRET Control Bits */
      __I  uint32_t BNK0_RET            	:	1;  /* Bank0 is always retained in DSL0, DSL1 and SD0 modes of operation */
      __IO uint32_t BNK1_RET            	:	1;  /*  */
      __IO uint32_t BNK2_RET            	:	1;  /*  */
      __IO uint32_t BNK3_RET            	:	1;  /*  */
      __IO uint32_t BNK4_RET            	:	1;  /*  */
      __IO uint32_t BNK5_RET            	:	1;  /*  */
      __IO uint32_t BNK6_RET            	:	1;  /*  */
      __IO uint32_t BNK7_RET            	:	1;  /*  */
      __I  uint32_t RESERVED_2          	:	8;  /* Reserved */
      __I  uint32_t SRAM_RDY            	:	1;  /*  */
      __I  uint32_t RESERVED_1          	:	15;  /* Reserved */

    } bit;
  } SRAM_BANKRET;
  uint8_t RESERVED0[4];
  uint32_t FLUSER_SIZE; /* The contents of this register are a reflection of the register  */
                                                                                 /* SYS_FLUSER_SIZECFG_REG. */
  uint8_t RESERVED1[12];
  union {    /* SYSCTL_DIOGLTFLTCTL Register */
    __IO uint32_t reg;
    struct { /* SYSCTL_DIOGLTFLTCTL Control Bits */
      __IO uint32_t GLTCH_EN            	:	1;  /*  */
      __I  uint32_t RESERVED_1          	:	31;  /* Reserved */

    } bit;
  } DIOGLTFLTCTL;
  uint8_t RESERVED2[12];
  union {    /* SYSCTL_SECDATA_UNLOCK Register */
    __IO uint32_t reg;
    struct { /* SYSCTL_SECDATA_UNLOCK Control Bits */
      __IO uint32_t UNLKEY              	:	16;  /* Unlock Key, which requests for secure data region to be unlocked for data	 */
                                                /* accesses  Read back : 0xA596 when unlocked, 0h when locked  Write to unlock :  */
                                                /* 0x695A (only unlocks the same region as that of the code	writing to this  */
                                                /* register) Other writes : Lock  */
      __I  uint32_t RESERVED_1          	:	16;  /* Reserved */

    } bit;
  } SECDATA_UNLOCK;
} SYSCTL_Type;


/* SYSCTL_REBOOT_CTL Control Bits */
#define SYSCTL_REBOOT_CTL_REBOOT                           (0x00000001)          /* Write 1 initiates a Reboot of the device */
#define SYSCTL_REBOOT_CTL_RESERVED_2__M                    (0x000000fe)          /* Reserved. Reads return 0h  */
#define SYSCTL_REBOOT_CTL_WKEY__M                          (0x0000ff00)          /* Key to enable writes to bit 0. Bit 0 is written only if WKEY is 69h in the same  */
                                                                                 /* write cycle. */
#define SYSCTL_REBOOT_CTL_RESERVED_1__M                    (0xffff0000)          /* Reserved. Reads return 0h  */

/* SYSCTL_NMI_CTLSTAT Control Bits */
#define SYSCTL_NMI_CTLSTAT_CS_SRC                          (0x00000001)          /*  */
#define SYSCTL_NMI_CTLSTAT_PSS_SRC                         (0x00000002)          /*  */
#define SYSCTL_NMI_CTLSTAT_PCM_SRC                         (0x00000004)          /*  */
#define SYSCTL_NMI_CTLSTAT_PIN_SRC                         (0x00000008)          /* Setting this bit to 1 prevents the	RSTn pin from being used as a reset. An NMI  */
                                                                                 /* is triggered by the pin only if a negative edge is	detected. */
#define SYSCTL_NMI_CTLSTAT_RESERVED_2__M                   (0x0000fff0)          /* Reserved. Reads return 0h  */
#define SYSCTL_NMI_CTLSTAT_CS_FLG                          (0x00010000)          /* This flag gets auto-cleared when the corresponding source flag in the CS is	 */
                                                                                 /* cleared */
#define SYSCTL_NMI_CTLSTAT_PSS_FLG                         (0x00020000)          /* This flag gets auto-cleared when the corresponding source flag in the PSS is	 */
                                                                                 /* cleared */
#define SYSCTL_NMI_CTLSTAT_PCM_FLG                         (0x00040000)          /* This flag gets auto-cleared when the corresponding source flag in the PCM is	 */
                                                                                 /* cleared */
#define SYSCTL_NMI_CTLSTAT_PIN_FLG                         (0x00080000)          /*  */
#define SYSCTL_NMI_CTLSTAT_RESERVED_1__M                   (0xfff00000)          /* Reserved. Reads return 0h  */

/* SYSCTL_WDT_RSTCTL Control Bits */
#define SYSCTL_WDT_RSTCTL_TIMEOUT                          (0x00000001)          /*  */
#define SYSCTL_WDT_RSTCTL_VIOLATION                        (0x00000002)          /*  */
#define SYSCTL_WDT_RSTCTL_RESERVED_1__M                    (0xfffffffc)          /* Reserved. Reads return 0h  */

/* SYSCTL_PERI_HALTCTL Control Bits */
#define SYSCTL_PERI_HALTCTL_T16_0                          (0x00000001)          /*  */
#define SYSCTL_PERI_HALTCTL_T16_1                          (0x00000002)          /*  */
#define SYSCTL_PERI_HALTCTL_T16_2                          (0x00000004)          /*  */
#define SYSCTL_PERI_HALTCTL_T16_3                          (0x00000008)          /*  */
#define SYSCTL_PERI_HALTCTL_T32_0                          (0x00000010)          /*  */
#define SYSCTL_PERI_HALTCTL_EUA0                           (0x00000020)          /*  */
#define SYSCTL_PERI_HALTCTL_EUA1                           (0x00000040)          /*  */
#define SYSCTL_PERI_HALTCTL_EUA2                           (0x00000080)          /*  */
#define SYSCTL_PERI_HALTCTL_EUA3                           (0x00000100)          /*  */
#define SYSCTL_PERI_HALTCTL_EUB0                           (0x00000200)          /*  */
#define SYSCTL_PERI_HALTCTL_EUB1                           (0x00000400)          /*  */
#define SYSCTL_PERI_HALTCTL_EUB2                           (0x00000800)          /*  */
#define SYSCTL_PERI_HALTCTL_EUB3                           (0x00001000)          /*  */
#define SYSCTL_PERI_HALTCTL_ADC                            (0x00002000)          /*  */
#define SYSCTL_PERI_HALTCTL_WDT                            (0x00004000)          /*  */
#define SYSCTL_PERI_HALTCTL_DMA                            (0x00008000)          /*  */
#define SYSCTL_PERI_HALTCTL_RESERVED_1__M                  (0xffff0000)          /* Reserved. Reads return 0h  */

/* SYSCTL_SRAM_BANKEN Control Bits */
#define SYSCTL_SRAM_BANKEN_BNK0_EN                         (0x00000001)          /* When 1, enables Bank0 of the SRAM */
#define SYSCTL_SRAM_BANKEN_BNK1_EN                         (0x00000002)          /* When set to 1, bank enable bits for all banks below this bank are set to 1 as	 */
                                                                                 /* well. For further details, see . */
#define SYSCTL_SRAM_BANKEN_BNK2_EN                         (0x00000004)          /* When set to 1, bank enable bits for all banks below this bank are set to 1 as	 */
                                                                                 /* well. For further details, see . */
#define SYSCTL_SRAM_BANKEN_BNK3_EN                         (0x00000008)          /* When set to 1, bank enable bits for all banks below this bank are set to 1 as	 */
                                                                                 /* well. For further details, see . */
#define SYSCTL_SRAM_BANKEN_BNK4_EN                         (0x00000010)          /* When set to 1, bank enable bits for all banks below this bank are set to 1 as	 */
                                                                                 /* well. For further details, see . */
#define SYSCTL_SRAM_BANKEN_BNK5_EN                         (0x00000020)          /* When set to 1, bank enable bits for all banks below this bank are set to 1 as	 */
                                                                                 /* well. For further details, see . */
#define SYSCTL_SRAM_BANKEN_BNK6_EN                         (0x00000040)          /* When set to 1, bank enable bits for all banks below this bank are set to 1 as	 */
                                                                                 /* well. For further details, see . */
#define SYSCTL_SRAM_BANKEN_BNK7_EN                         (0x00000080)          /* When set to 1, bank enable bits for all banks below this bank are set to 1 as	 */
                                                                                 /* well. For further details, see . */
#define SYSCTL_SRAM_BANKEN_RESERVED_2__M                   (0x0000ff00)          /* Reserved. Reads return 0h  */
#define SYSCTL_SRAM_BANKEN_SRAM_RDY                        (0x00010000)          /*  */
#define SYSCTL_SRAM_BANKEN_RESERVED_1__M                   (0xfffe0000)          /* Reserved. Reads return 0h  */

/* SYSCTL_SRAM_BANKRET Control Bits */
#define SYSCTL_SRAM_BANKRET_BNK0_RET                       (0x00000001)          /* Bank0 is always retained in DSL0, DSL1 and SD0 modes of operation */
#define SYSCTL_SRAM_BANKRET_BNK1_RET                       (0x00000002)          /*  */
#define SYSCTL_SRAM_BANKRET_BNK2_RET                       (0x00000004)          /*  */
#define SYSCTL_SRAM_BANKRET_BNK3_RET                       (0x00000008)          /*  */
#define SYSCTL_SRAM_BANKRET_BNK4_RET                       (0x00000010)          /*  */
#define SYSCTL_SRAM_BANKRET_BNK5_RET                       (0x00000020)          /*  */
#define SYSCTL_SRAM_BANKRET_BNK6_RET                       (0x00000040)          /*  */
#define SYSCTL_SRAM_BANKRET_BNK7_RET                       (0x00000080)          /*  */
#define SYSCTL_SRAM_BANKRET_RESERVED_2__M                  (0x0000ff00)          /* Reserved. Reads return 0h  */
#define SYSCTL_SRAM_BANKRET_SRAM_RDY                       (0x00010000)          /*  */
#define SYSCTL_SRAM_BANKRET_RESERVED_1__M                  (0xfffe0000)          /* Reserved. Reads return 0h  */

/* SYSCTL_DIOGLTFLTCTL Control Bits */
#define SYSCTL_DIOGLTFLTCTL_GLTCH_EN                       (0x00000001)          /*  */
#define SYSCTL_DIOGLTFLTCTL_RESERVED_1__M                  (0xfffffffe)          /* Reserved. Always reads 0h */

/* SYSCTL_SECDATA_UNLOCK Control Bits */
#define SYSCTL_SECDATA_UNLOCK_UNLKEY__M                    (0x0000ffff)          /* Unlock Key, which requests for secure data region to be unlocked for data	 */
                                                                                 /* accesses  Read back : 0xA596 when unlocked, 0h when locked  Write to unlock :  */
                                                                                 /* 0x695A (only unlocks the same region as that of the code	writing to this  */
                                                                                 /* register) Other writes : Lock  */
#define SYSCTL_SECDATA_UNLOCK_RESERVED_1__M                (0xffff0000)          /* Reserved. Reads return 0h  */


//*****************************************************************************
// T32 Registers
//*****************************************************************************
typedef struct {
  uint32_t LOAD1; /* The T32LOAD1 register is a 32-bit register containing the value from which the  */
                                                                                 /* counter is to decrement. This is the value used to reload the counter when  */
                                                                                 /* Periodic mode is enabled, and the current count reaches zero. When this register */
                                                                                 /* is written to directly, the current count is immediately reset to the new value */
                                                                                 /* at the next rising edge of TIMCLK that is enabled by TIMCLKEN. The value in  */
                                                                                 /* this register is also overwritten if the T32BGLOAD1 register is written to, but  */
                                                                                 /* the current count is not immediately affected. */
  uint32_t VALUE1; /* Timer 1 Current Value Register */
  union {    /* T32_CONTROL1 Register */
    __IO uint32_t reg;
    struct { /* T32_CONTROL1 Control Bits */
      __IO uint32_t ONESHOT             	:	1;  /* Selects one-shot or wrapping counter mode */
      __IO uint32_t SIZE                	:	1;  /* Selects 16 or 32 bit counter operation */
      __IO uint32_t PRESCALE            	:	2;  /* Prescale bits */
      __I  uint32_t RESERVED_2          	:	1;  /* Reserved */
      __IO uint32_t IE                  	:	1;  /* Interrupt enable bit */
      __IO uint32_t MODE                	:	1;  /* Mode bit */
      __IO uint32_t ENABLE              	:	1;  /* Enable bit */
      __I  uint32_t RESERVED_1          	:	24;  /* Reserved */

    } bit;
  } CONTROL1;
  uint32_t INTCLR1; /* Timer 1 Interrupt Clear Register */
  union {    /* T32_RIS1 Register */
    __IO uint32_t reg;
    struct { /* T32_RIS1 Control Bits */
      __I  uint32_t RAW_IFG             	:	1;  /* Raw interrupt status from the counter */
      __I  uint32_t RESERVED_1          	:	31;  /* Reserved */

    } bit;
  } RIS1;
  union {    /* T32_MIS1 Register */
    __IO uint32_t reg;
    struct { /* T32_MIS1 Control Bits */
      __I  uint32_t IFG                 	:	1;  /* Enabled interrupt status from the counter */
      __I  uint32_t RESERVED_1          	:	31;  /* Reserved */

    } bit;
  } MIS1;
  uint32_t BGLOAD1; /* This register provides an alternative method of accessing the T32LOAD1 Register. */
                                                                                 /* The difference is that writes to T32BGLOAD1 do not cause the counter to  */
                                                                                 /* immediately restart from the new value. */
  uint8_t RESERVED0[4];
  uint32_t LOAD2; /* The T32LOAD2 register is a 32-bit register containing the value from which the  */
                                                                                 /* counter is to decrement. This is the value used to reload the counter when  */
                                                                                 /* Periodic mode is enabled, and the current count reaches zero. When this register */
                                                                                 /* is written to directly, the current count is immediately reset to the new value */
                                                                                 /* at the next rising edge of TIMCLK that is enabled by TIMCLKEN. The value in  */
                                                                                 /* this register is also overwritten if the T32BGLOAD2 register is written to, but  */
                                                                                 /* the current count is not immediately affected. */
  uint32_t VALUE2; /* Timer 2 Current Value Register */
  union {    /* T32_CONTROL2 Register */
    __IO uint32_t reg;
    struct { /* T32_CONTROL2 Control Bits */
      __IO uint32_t ONESHOT             	:	1;  /* Selects one-shot or wrapping counter mode */
      __IO uint32_t SIZE                	:	1;  /* Selects 16 or 32 bit counter operation */
      __IO uint32_t PRESCALE            	:	2;  /* Prescale bits */
      __I  uint32_t RESERVED_2          	:	1;  /* Reserved */
      __IO uint32_t IE                  	:	1;  /* Interrupt enable bit */
      __IO uint32_t MODE                	:	1;  /* Mode bit */
      __IO uint32_t ENABLE              	:	1;  /* Enable bit */
      __I  uint32_t RESERVED_1          	:	24;  /* Reserved */

    } bit;
  } CONTROL2;
  uint32_t INTCLR2; /* Timer 2 Interrupt Clear Register */
  union {    /* T32_RIS2 Register */
    __IO uint32_t reg;
    struct { /* T32_RIS2 Control Bits */
      __I  uint32_t RAW_IFG             	:	1;  /* Raw interrupt status from the counter */
      __I  uint32_t RESERVED_1          	:	31;  /* Reserved */

    } bit;
  } RIS2;
  union {    /* T32_MIS2 Register */
    __IO uint32_t reg;
    struct { /* T32_MIS2 Control Bits */
      __I  uint32_t IFG                 	:	1;  /* Enabled interrupt status from the counter */
      __I  uint32_t RESERVED_1          	:	31;  /* Reserved */

    } bit;
  } MIS2;
  uint32_t BGLOAD2; /* This register provides an alternative method of accessing the T32LOAD2 Register. */
                                                                                 /* The difference is that writes to T32BGLOAD2 do not cause the counter to  */
                                                                                 /* immediately restart from the new value. */
} T32_Type;


/* T32_CONTROL1 Control Bits */
#define T32_CONTROL1_ONESHOT                               (0x00000001)          /* Selects one-shot or wrapping counter mode */
#define T32_CONTROL1_SIZE                                  (0x00000002)          /* Selects 16 or 32 bit counter operation */
#define T32_CONTROL1_PRESCALE__M                           (0x0000000c)          /* Prescale bits */
#define T32_CONTROL1_PRESCALE__0                           (0x00000000)          /* 0 stages of prescale, clock is divided by 1 */
#define T32_CONTROL1_PRESCALE__1                           (0x00000004)          /* 4 stages of prescale, clock is divided by 16 */
#define T32_CONTROL1_PRESCALE__2                           (0x00000008)          /* 8 stages of prescale, clock is divided by 256 */
#define T32_CONTROL1_RESERVED_2                            (0x00000010)          /* Reserved */
#define T32_CONTROL1_IE                                    (0x00000020)          /* Interrupt enable bit */
#define T32_CONTROL1_MODE                                  (0x00000040)          /* Mode bit */
#define T32_CONTROL1_ENABLE                                (0x00000080)          /* Enable bit */
#define T32_CONTROL1_RESERVED_1__M                         (0xffffff00)          /* Reserved */

/* T32_RIS1 Control Bits */
#define T32_RIS1_RAW_IFG                                   (0x00000001)          /* Raw interrupt status from the counter */
#define T32_RIS1_RESERVED_1__M                             (0xfffffffe)          /* Reserved */

/* T32_MIS1 Control Bits */
#define T32_MIS1_IFG                                       (0x00000001)          /* Enabled interrupt status from the counter */
#define T32_MIS1_RESERVED_1__M                             (0xfffffffe)          /* Reserved */

/* T32_CONTROL2 Control Bits */
#define T32_CONTROL2_ONESHOT                               (0x00000001)          /* Selects one-shot or wrapping counter mode */
#define T32_CONTROL2_SIZE                                  (0x00000002)          /* Selects 16 or 32 bit counter operation */
#define T32_CONTROL2_PRESCALE__M                           (0x0000000c)          /* Prescale bits */
#define T32_CONTROL2_PRESCALE__0                           (0x00000000)          /* 0 stages of prescale, clock is divided by 1 */
#define T32_CONTROL2_PRESCALE__1                           (0x00000004)          /* 4 stages of prescale, clock is divided by 16 */
#define T32_CONTROL2_PRESCALE__2                           (0x00000008)          /* 8 stages of prescale, clock is divided by 256 */
#define T32_CONTROL2_RESERVED_2                            (0x00000010)          /* Reserved */
#define T32_CONTROL2_IE                                    (0x00000020)          /* Interrupt enable bit */
#define T32_CONTROL2_MODE                                  (0x00000040)          /* Mode bit */
#define T32_CONTROL2_ENABLE                                (0x00000080)          /* Enable bit */
#define T32_CONTROL2_RESERVED_1__M                         (0xffffff00)          /* Reserved */

/* T32_RIS2 Control Bits */
#define T32_RIS2_RAW_IFG                                   (0x00000001)          /* Raw interrupt status from the counter */
#define T32_RIS2_RESERVED_1__M                             (0xfffffffe)          /* Reserved */

/* T32_MIS2 Control Bits */
#define T32_MIS2_IFG                                       (0x00000001)          /* Enabled interrupt status from the counter */
#define T32_MIS2_RESERVED_1__M                             (0xfffffffe)          /* Reserved */


//*****************************************************************************
// TA0 Registers
//*****************************************************************************
typedef struct {
  union {    /* TA0_CTL Register */
    __IO uint16_t reg;
    struct { /* TA0_CTL Control Bits */
      __IO uint16_t TAIFG               	:	1;  /* Timer_A interrupt flag */
      __IO uint16_t TAIE                	:	1;  /* Timer_A interrupt enable. This bit enables the TAIFG interrupt request. */
      __IO uint16_t TACLR               	:	1;  /* Timer_A clear. Setting this bit resets TAxR, the timer clock divider logic, and  */
                                                /* the count direction. The TACLR bit is automatically reset and is always read as  */
                                                /* zero. */
      __IO uint16_t RESERVED_2          	:	1;  /* Reserved */
      __IO uint16_t MC                  	:	2;  /* Mode control. Setting MCx = 00h when Timer_A is not in use conserves power. */
      __IO uint16_t ID                  	:	2;  /* Input divider. These bits along with the TAIDEX bits select the divider for the  */
                                                /* input clock.   */
      __IO uint16_t TASSEL              	:	2;  /* Timer_A clock source select   */
      __IO uint16_t RESERVED_1          	:	6;  /* Reserved */

    } bit;
  } CTL;
  union {    /* TA0_CCTL Register */
    __IO uint16_t reg;
    struct { /* TA0_CCTL Control Bits */
      __IO uint16_t CCIFG               	:	1;  /* Capture/compare interrupt flag */
      __IO uint16_t COV                 	:	1;  /* Capture overflow. This bit indicates a capture overflow occurred. COV must be  */
                                                /* reset with software. */
      __IO uint16_t OUT                 	:	1;  /* Output. For output mode 0, this bit directly controls the state of the output. */
      __I  uint16_t CCI                 	:	1;  /* Capture/compare input. The selected input signal can be read by this bit. */
      __IO uint16_t CCIE                	:	1;  /* Capture/compare interrupt enable. This bit enables the interrupt request of	the  */
                                                /* corresponding CCIFG flag. */
      __IO uint16_t OUTMOD              	:	3;  /* Output mode. Modes 2, 3, 6, and 7 are not useful for TAxCCR0 because EQUx =	EQU0 */
                                                /* . */
      __IO uint16_t CAP                 	:	1;  /* Capture mode */
      __I  uint16_t RESERVED_1          	:	1;  /* Reserved */
      __IO uint16_t SCCI                	:	1;  /* Synchronized capture/compare input. The selected CCI input signal is latched  */
                                                /* with the EQUx signal and can be read via this bit. */
      __IO uint16_t SCS                 	:	1;  /* Synchronize capture source. This bit is used to synchronize the capture input	 */
                                                /* signal with the timer clock.   */
      __IO uint16_t CCIS                	:	2;  /* Capture/compare input select. These bits select the TAxCCR0 input signal. See	 */
                                                /* the device-specific data sheet for specific signal connections.   */
      __IO uint16_t CM                  	:	2;  /* Capture mode */

    } bit;
  } CCTL;
  uint8_t RESERVED0[12];
  uint16_t R; /* Timer_Ax Counter */
  uint16_t CCR; /* Timer_Ax Capture/Compare 0 to Timer_Ax Capture/Compare 6 */
  uint8_t RESERVED1[12];
  union {    /* TA0_EX0 Register */
    __IO uint16_t reg;
    struct { /* TA0_EX0 Control Bits */
      __IO uint16_t TAIDEX              	:	3;  /* Input divider expansion. These bits along with the ID bits select the divider	 */
                                                /* for the input clock. */
      __I  uint16_t RESERVED_1          	:	13;  /* Reserved */

    } bit;
  } EX0;
  uint8_t RESERVED2[12];
  uint16_t IV; /* Timer_Ax Interrupt Vector */
} TA0_Type;


/* TA0_CTL Control Bits */
#define TA0_CTL_TAIFG                                      (0x0001)              /* Timer_A interrupt flag */
#define TA0_CTL_TAIE                                       (0x0002)              /* Timer_A interrupt enable. This bit enables the TAIFG interrupt request. */
#define TA0_CTL_TACLR                                      (0x0004)              /* Timer_A clear. Setting this bit resets TAxR, the timer clock divider logic, and  */
                                                                                 /* the count direction. The TACLR bit is automatically reset and is always read as  */
                                                                                 /* zero. */
#define TA0_CTL_RESERVED_2                                 (0x0008)              /* Reserved */
#define TA0_CTL_MC__M                                      (0x0030)              /* Mode control. Setting MCx = 00h when Timer_A is not in use conserves power. */
#define TA0_CTL_MC__0                                      (0x0000)              /* Stop mode: Timer is halted */
#define TA0_CTL_MC__1                                      (0x0010)              /* Up mode: Timer counts up to TAxCCR0 */
#define TA0_CTL_MC__2                                      (0x0020)              /* Continuous mode: Timer counts up to 0FFFFh */
#define TA0_CTL_MC__3                                      (0x0030)              /* Up/down mode: Timer counts up to TAxCCR0 then down to 0000h */
#define TA0_CTL_ID__M                                      (0x00c0)              /* Input divider. These bits along with the TAIDEX bits select the divider for the  */
                                                                                 /* input clock.   */
#define TA0_CTL_ID__0                                      (0x0000)              /* /1 */
#define TA0_CTL_ID__0__1                                   (0x0000)              /* /1 */
#define TA0_CTL_ID__1                                      (0x0040)              /* /2 */
#define TA0_CTL_ID__1__2                                   (0x0040)              /* /2 */
#define TA0_CTL_ID__2                                      (0x0080)              /* /4 */
#define TA0_CTL_ID__2__4                                   (0x0080)              /* /4 */
#define TA0_CTL_ID__3                                      (0x00c0)              /* /8 */
#define TA0_CTL_ID__3__8                                   (0x00c0)              /* /8 */
#define TA0_CTL_TASSEL__M                                  (0x0300)              /* Timer_A clock source select   */
#define TA0_CTL_TASSEL__0                                  (0x0000)              /* TAxCLK */
#define TA0_CTL_TASSEL__0_TAXCLK                           (0x0000)              /* TAxCLK */
#define TA0_CTL_TASSEL__1                                  (0x0100)              /* ACLK */
#define TA0_CTL_TASSEL__1_ACLK                             (0x0100)              /* ACLK */
#define TA0_CTL_TASSEL__2                                  (0x0200)              /* SMCLK */
#define TA0_CTL_TASSEL__2_SMCLK                            (0x0200)              /* SMCLK */
#define TA0_CTL_TASSEL__3                                  (0x0300)              /* INCLK */
#define TA0_CTL_TASSEL__3_INCLK                            (0x0300)              /* INCLK */
#define TA0_CTL_RESERVED_1__M                              (0xfc00)              /* Reserved */

/* TA0_CCTL Control Bits */
#define TA0_CCTL_CCIFG                                     (0x0001)              /* Capture/compare interrupt flag */
#define TA0_CCTL_COV                                       (0x0002)              /* Capture overflow. This bit indicates a capture overflow occurred. COV must be  */
                                                                                 /* reset with software. */
#define TA0_CCTL_OUT                                       (0x0004)              /* Output. For output mode 0, this bit directly controls the state of the output. */
#define TA0_CCTL_CCI                                       (0x0008)              /* Capture/compare input. The selected input signal can be read by this bit. */
#define TA0_CCTL_CCIE                                      (0x0010)              /* Capture/compare interrupt enable. This bit enables the interrupt request of	the  */
                                                                                 /* corresponding CCIFG flag. */
#define TA0_CCTL_OUTMOD__M                                 (0x00e0)              /* Output mode. Modes 2, 3, 6, and 7 are not useful for TAxCCR0 because EQUx =	EQU0 */
                                                                                 /* . */
#define TA0_CCTL_OUTMOD__0                                 (0x0000)              /* OUT bit value */
#define TA0_CCTL_OUTMOD__0_OUT_BIT_VALUE                   (0x0000)              /* OUT bit value */
#define TA0_CCTL_OUTMOD__1                                 (0x0020)              /* Set */
#define TA0_CCTL_OUTMOD__1_SET                             (0x0020)              /* Set */
#define TA0_CCTL_OUTMOD__2                                 (0x0040)              /* Toggle/reset */
#define TA0_CCTL_OUTMOD__2_TOGGLE_RESET                    (0x0040)              /* Toggle/reset */
#define TA0_CCTL_OUTMOD__3                                 (0x0060)              /* Set/reset */
#define TA0_CCTL_OUTMOD__3_SET_RESET                       (0x0060)              /* Set/reset */
#define TA0_CCTL_OUTMOD__4                                 (0x0080)              /* Toggle */
#define TA0_CCTL_OUTMOD__4_TOGGLE                          (0x0080)              /* Toggle */
#define TA0_CCTL_OUTMOD__5                                 (0x00a0)              /* Reset */
#define TA0_CCTL_OUTMOD__5_RESET                           (0x00a0)              /* Reset */
#define TA0_CCTL_OUTMOD__6                                 (0x00c0)              /* Toggle/set */
#define TA0_CCTL_OUTMOD__6_TOGGLE_SET                      (0x00c0)              /* Toggle/set */
#define TA0_CCTL_OUTMOD__7                                 (0x00e0)              /* Reset/set */
#define TA0_CCTL_OUTMOD__7_RESET_SET                       (0x00e0)              /* Reset/set */
#define TA0_CCTL_CAP                                       (0x0100)              /* Capture mode */
#define TA0_CCTL_RESERVED_1                                (0x0200)              /* Reserved. Reads as 0. */
#define TA0_CCTL_SCCI                                      (0x0400)              /* Synchronized capture/compare input. The selected CCI input signal is latched  */
                                                                                 /* with the EQUx signal and can be read via this bit. */
#define TA0_CCTL_SCS                                       (0x0800)              /* Synchronize capture source. This bit is used to synchronize the capture input	 */
                                                                                 /* signal with the timer clock.   */
#define TA0_CCTL_CCIS__M                                   (0x3000)              /* Capture/compare input select. These bits select the TAxCCR0 input signal. See	 */
                                                                                 /* the device-specific data sheet for specific signal connections.   */
#define TA0_CCTL_CCIS__0                                   (0x0000)              /* CCIxA */
#define TA0_CCTL_CCIS__0_CCIXA                             (0x0000)              /* CCIxA */
#define TA0_CCTL_CCIS__1                                   (0x1000)              /* CCIxB */
#define TA0_CCTL_CCIS__1_CCIXB                             (0x1000)              /* CCIxB */
#define TA0_CCTL_CCIS__2                                   (0x2000)              /* GND */
#define TA0_CCTL_CCIS__2_GND                               (0x2000)              /* GND */
#define TA0_CCTL_CCIS__3                                   (0x3000)              /* VCC */
#define TA0_CCTL_CCIS__3_VCC                               (0x3000)              /* VCC */
#define TA0_CCTL_CM__M                                     (0xc000)              /* Capture mode */
#define TA0_CCTL_CM__0                                     (0x0000)              /* No capture */
#define TA0_CCTL_CM__0_NO_CAPTURE                          (0x0000)              /* No capture */
#define TA0_CCTL_CM__1                                     (0x4000)              /* Capture on rising edge */
#define TA0_CCTL_CM__2                                     (0x8000)              /* Capture on falling edge */
#define TA0_CCTL_CM__3                                     (0xc000)              /* Capture on both rising and falling edges */

/* TA0_EX0 Control Bits */
#define TA0_EX0_TAIDEX__M                                  (0x0007)              /* Input divider expansion. These bits along with the ID bits select the divider	 */
                                                                                 /* for the input clock. */
#define TA0_EX0_TAIDEX__0                                  (0x0000)              /* Divide by 1 */
#define TA0_EX0_TAIDEX__0_DIVIDE_BY_1                      (0x0000)              /* Divide by 1 */
#define TA0_EX0_TAIDEX__1                                  (0x0001)              /* Divide by 2 */
#define TA0_EX0_TAIDEX__1_DIVIDE_BY_2                      (0x0001)              /* Divide by 2 */
#define TA0_EX0_TAIDEX__2                                  (0x0002)              /* Divide by 3 */
#define TA0_EX0_TAIDEX__2_DIVIDE_BY_3                      (0x0002)              /* Divide by 3 */
#define TA0_EX0_TAIDEX__3                                  (0x0003)              /* Divide by 4 */
#define TA0_EX0_TAIDEX__3_DIVIDE_BY_4                      (0x0003)              /* Divide by 4 */
#define TA0_EX0_TAIDEX__4                                  (0x0004)              /* Divide by 5 */
#define TA0_EX0_TAIDEX__4_DIVIDE_BY_5                      (0x0004)              /* Divide by 5 */
#define TA0_EX0_TAIDEX__5                                  (0x0005)              /* Divide by 6 */
#define TA0_EX0_TAIDEX__5_DIVIDE_BY_6                      (0x0005)              /* Divide by 6 */
#define TA0_EX0_TAIDEX__6                                  (0x0006)              /* Divide by 7 */
#define TA0_EX0_TAIDEX__6_DIVIDE_BY_7                      (0x0006)              /* Divide by 7 */
#define TA0_EX0_TAIDEX__7                                  (0x0007)              /* Divide by 8 */
#define TA0_EX0_TAIDEX__7_DIVIDE_BY_8                      (0x0007)              /* Divide by 8 */
#define TA0_EX0_RESERVED_1__M                              (0xfff8)              /* Reserved. Reads as 0. */


//*****************************************************************************
// TA1 Registers
//*****************************************************************************
typedef struct {
  union {    /* TA1_CTL Register */
    __IO uint16_t reg;
    struct { /* TA1_CTL Control Bits */
      __IO uint16_t TAIFG               	:	1;  /* Timer_A interrupt flag */
      __IO uint16_t TAIE                	:	1;  /* Timer_A interrupt enable. This bit enables the TAIFG interrupt request. */
      __IO uint16_t TACLR               	:	1;  /* Timer_A clear. Setting this bit resets TAxR, the timer clock divider logic, and  */
                                                /* the count direction. The TACLR bit is automatically reset and is always read as  */
                                                /* zero. */
      __IO uint16_t RESERVED_2          	:	1;  /* Reserved */
      __IO uint16_t MC                  	:	2;  /* Mode control. Setting MCx = 00h when Timer_A is not in use conserves power. */
      __IO uint16_t ID                  	:	2;  /* Input divider. These bits along with the TAIDEX bits select the divider for the  */
                                                /* input clock.   */
      __IO uint16_t TASSEL              	:	2;  /* Timer_A clock source select   */
      __IO uint16_t RESERVED_1          	:	6;  /* Reserved */

    } bit;
  } CTL;
  union {    /* TA1_CCTL Register */
    __IO uint16_t reg;
    struct { /* TA1_CCTL Control Bits */
      __IO uint16_t CCIFG               	:	1;  /* Capture/compare interrupt flag */
      __IO uint16_t COV                 	:	1;  /* Capture overflow. This bit indicates a capture overflow occurred. COV must be  */
                                                /* reset with software. */
      __IO uint16_t OUT                 	:	1;  /* Output. For output mode 0, this bit directly controls the state of the output. */
      __I  uint16_t CCI                 	:	1;  /* Capture/compare input. The selected input signal can be read by this bit. */
      __IO uint16_t CCIE                	:	1;  /* Capture/compare interrupt enable. This bit enables the interrupt request of	the  */
                                                /* corresponding CCIFG flag. */
      __IO uint16_t OUTMOD              	:	3;  /* Output mode. Modes 2, 3, 6, and 7 are not useful for TAxCCR0 because EQUx =	EQU0 */
                                                /* . */
      __IO uint16_t CAP                 	:	1;  /* Capture mode */
      __I  uint16_t RESERVED_1          	:	1;  /* Reserved */
      __IO uint16_t SCCI                	:	1;  /* Synchronized capture/compare input. The selected CCI input signal is latched  */
                                                /* with the EQUx signal and can be read via this bit. */
      __IO uint16_t SCS                 	:	1;  /* Synchronize capture source. This bit is used to synchronize the capture input	 */
                                                /* signal with the timer clock.   */
      __IO uint16_t CCIS                	:	2;  /* Capture/compare input select. These bits select the TAxCCR0 input signal. See	 */
                                                /* the device-specific data sheet for specific signal connections.   */
      __IO uint16_t CM                  	:	2;  /* Capture mode */

    } bit;
  } CCTL;
  uint8_t RESERVED0[12];
  uint16_t R; /* Timer_Ax Counter */
  uint16_t CCR; /* Timer_Ax Capture/Compare 0 to Timer_Ax Capture/Compare 6 */
  uint8_t RESERVED1[12];
  union {    /* TA1_EX0 Register */
    __IO uint16_t reg;
    struct { /* TA1_EX0 Control Bits */
      __IO uint16_t TAIDEX              	:	3;  /* Input divider expansion. These bits along with the ID bits select the divider	 */
                                                /* for the input clock. */
      __I  uint16_t RESERVED_1          	:	13;  /* Reserved */

    } bit;
  } EX0;
  uint8_t RESERVED2[12];
  uint16_t IV; /* Timer_Ax Interrupt Vector */
} TA1_Type;


/* TA1_CTL Control Bits */
#define TA1_CTL_TAIFG                                      (0x0001)              /* Timer_A interrupt flag */
#define TA1_CTL_TAIE                                       (0x0002)              /* Timer_A interrupt enable. This bit enables the TAIFG interrupt request. */
#define TA1_CTL_TACLR                                      (0x0004)              /* Timer_A clear. Setting this bit resets TAxR, the timer clock divider logic, and  */
                                                                                 /* the count direction. The TACLR bit is automatically reset and is always read as  */
                                                                                 /* zero. */
#define TA1_CTL_RESERVED_2                                 (0x0008)              /* Reserved */
#define TA1_CTL_MC__M                                      (0x0030)              /* Mode control. Setting MCx = 00h when Timer_A is not in use conserves power. */
#define TA1_CTL_MC__0                                      (0x0000)              /* Stop mode: Timer is halted */
#define TA1_CTL_MC__1                                      (0x0010)              /* Up mode: Timer counts up to TAxCCR0 */
#define TA1_CTL_MC__2                                      (0x0020)              /* Continuous mode: Timer counts up to 0FFFFh */
#define TA1_CTL_MC__3                                      (0x0030)              /* Up/down mode: Timer counts up to TAxCCR0 then down to 0000h */
#define TA1_CTL_ID__M                                      (0x00c0)              /* Input divider. These bits along with the TAIDEX bits select the divider for the  */
                                                                                 /* input clock.   */
#define TA1_CTL_ID__0                                      (0x0000)              /* /1 */
#define TA1_CTL_ID__0__1                                   (0x0000)              /* /1 */
#define TA1_CTL_ID__1                                      (0x0040)              /* /2 */
#define TA1_CTL_ID__1__2                                   (0x0040)              /* /2 */
#define TA1_CTL_ID__2                                      (0x0080)              /* /4 */
#define TA1_CTL_ID__2__4                                   (0x0080)              /* /4 */
#define TA1_CTL_ID__3                                      (0x00c0)              /* /8 */
#define TA1_CTL_ID__3__8                                   (0x00c0)              /* /8 */
#define TA1_CTL_TASSEL__M                                  (0x0300)              /* Timer_A clock source select   */
#define TA1_CTL_TASSEL__0                                  (0x0000)              /* TAxCLK */
#define TA1_CTL_TASSEL__0_TAXCLK                           (0x0000)              /* TAxCLK */
#define TA1_CTL_TASSEL__1                                  (0x0100)              /* ACLK */
#define TA1_CTL_TASSEL__1_ACLK                             (0x0100)              /* ACLK */
#define TA1_CTL_TASSEL__2                                  (0x0200)              /* SMCLK */
#define TA1_CTL_TASSEL__2_SMCLK                            (0x0200)              /* SMCLK */
#define TA1_CTL_TASSEL__3                                  (0x0300)              /* INCLK */
#define TA1_CTL_TASSEL__3_INCLK                            (0x0300)              /* INCLK */
#define TA1_CTL_RESERVED_1__M                              (0xfc00)              /* Reserved */

/* TA1_CCTL Control Bits */
#define TA1_CCTL_CCIFG                                     (0x0001)              /* Capture/compare interrupt flag */
#define TA1_CCTL_COV                                       (0x0002)              /* Capture overflow. This bit indicates a capture overflow occurred. COV must be  */
                                                                                 /* reset with software. */
#define TA1_CCTL_OUT                                       (0x0004)              /* Output. For output mode 0, this bit directly controls the state of the output. */
#define TA1_CCTL_CCI                                       (0x0008)              /* Capture/compare input. The selected input signal can be read by this bit. */
#define TA1_CCTL_CCIE                                      (0x0010)              /* Capture/compare interrupt enable. This bit enables the interrupt request of	the  */
                                                                                 /* corresponding CCIFG flag. */
#define TA1_CCTL_OUTMOD__M                                 (0x00e0)              /* Output mode. Modes 2, 3, 6, and 7 are not useful for TAxCCR0 because EQUx =	EQU0 */
                                                                                 /* . */
#define TA1_CCTL_OUTMOD__0                                 (0x0000)              /* OUT bit value */
#define TA1_CCTL_OUTMOD__0_OUT_BIT_VALUE                   (0x0000)              /* OUT bit value */
#define TA1_CCTL_OUTMOD__1                                 (0x0020)              /* Set */
#define TA1_CCTL_OUTMOD__1_SET                             (0x0020)              /* Set */
#define TA1_CCTL_OUTMOD__2                                 (0x0040)              /* Toggle/reset */
#define TA1_CCTL_OUTMOD__2_TOGGLE_RESET                    (0x0040)              /* Toggle/reset */
#define TA1_CCTL_OUTMOD__3                                 (0x0060)              /* Set/reset */
#define TA1_CCTL_OUTMOD__3_SET_RESET                       (0x0060)              /* Set/reset */
#define TA1_CCTL_OUTMOD__4                                 (0x0080)              /* Toggle */
#define TA1_CCTL_OUTMOD__4_TOGGLE                          (0x0080)              /* Toggle */
#define TA1_CCTL_OUTMOD__5                                 (0x00a0)              /* Reset */
#define TA1_CCTL_OUTMOD__5_RESET                           (0x00a0)              /* Reset */
#define TA1_CCTL_OUTMOD__6                                 (0x00c0)              /* Toggle/set */
#define TA1_CCTL_OUTMOD__6_TOGGLE_SET                      (0x00c0)              /* Toggle/set */
#define TA1_CCTL_OUTMOD__7                                 (0x00e0)              /* Reset/set */
#define TA1_CCTL_OUTMOD__7_RESET_SET                       (0x00e0)              /* Reset/set */
#define TA1_CCTL_CAP                                       (0x0100)              /* Capture mode */
#define TA1_CCTL_RESERVED_1                                (0x0200)              /* Reserved. Reads as 0. */
#define TA1_CCTL_SCCI                                      (0x0400)              /* Synchronized capture/compare input. The selected CCI input signal is latched  */
                                                                                 /* with the EQUx signal and can be read via this bit. */
#define TA1_CCTL_SCS                                       (0x0800)              /* Synchronize capture source. This bit is used to synchronize the capture input	 */
                                                                                 /* signal with the timer clock.   */
#define TA1_CCTL_CCIS__M                                   (0x3000)              /* Capture/compare input select. These bits select the TAxCCR0 input signal. See	 */
                                                                                 /* the device-specific data sheet for specific signal connections.   */
#define TA1_CCTL_CCIS__0                                   (0x0000)              /* CCIxA */
#define TA1_CCTL_CCIS__0_CCIXA                             (0x0000)              /* CCIxA */
#define TA1_CCTL_CCIS__1                                   (0x1000)              /* CCIxB */
#define TA1_CCTL_CCIS__1_CCIXB                             (0x1000)              /* CCIxB */
#define TA1_CCTL_CCIS__2                                   (0x2000)              /* GND */
#define TA1_CCTL_CCIS__2_GND                               (0x2000)              /* GND */
#define TA1_CCTL_CCIS__3                                   (0x3000)              /* VCC */
#define TA1_CCTL_CCIS__3_VCC                               (0x3000)              /* VCC */
#define TA1_CCTL_CM__M                                     (0xc000)              /* Capture mode */
#define TA1_CCTL_CM__0                                     (0x0000)              /* No capture */
#define TA1_CCTL_CM__0_NO_CAPTURE                          (0x0000)              /* No capture */
#define TA1_CCTL_CM__1                                     (0x4000)              /* Capture on rising edge */
#define TA1_CCTL_CM__2                                     (0x8000)              /* Capture on falling edge */
#define TA1_CCTL_CM__3                                     (0xc000)              /* Capture on both rising and falling edges */

/* TA1_EX0 Control Bits */
#define TA1_EX0_TAIDEX__M                                  (0x0007)              /* Input divider expansion. These bits along with the ID bits select the divider	 */
                                                                                 /* for the input clock. */
#define TA1_EX0_TAIDEX__0                                  (0x0000)              /* Divide by 1 */
#define TA1_EX0_TAIDEX__0_DIVIDE_BY_1                      (0x0000)              /* Divide by 1 */
#define TA1_EX0_TAIDEX__1                                  (0x0001)              /* Divide by 2 */
#define TA1_EX0_TAIDEX__1_DIVIDE_BY_2                      (0x0001)              /* Divide by 2 */
#define TA1_EX0_TAIDEX__2                                  (0x0002)              /* Divide by 3 */
#define TA1_EX0_TAIDEX__2_DIVIDE_BY_3                      (0x0002)              /* Divide by 3 */
#define TA1_EX0_TAIDEX__3                                  (0x0003)              /* Divide by 4 */
#define TA1_EX0_TAIDEX__3_DIVIDE_BY_4                      (0x0003)              /* Divide by 4 */
#define TA1_EX0_TAIDEX__4                                  (0x0004)              /* Divide by 5 */
#define TA1_EX0_TAIDEX__4_DIVIDE_BY_5                      (0x0004)              /* Divide by 5 */
#define TA1_EX0_TAIDEX__5                                  (0x0005)              /* Divide by 6 */
#define TA1_EX0_TAIDEX__5_DIVIDE_BY_6                      (0x0005)              /* Divide by 6 */
#define TA1_EX0_TAIDEX__6                                  (0x0006)              /* Divide by 7 */
#define TA1_EX0_TAIDEX__6_DIVIDE_BY_7                      (0x0006)              /* Divide by 7 */
#define TA1_EX0_TAIDEX__7                                  (0x0007)              /* Divide by 8 */
#define TA1_EX0_TAIDEX__7_DIVIDE_BY_8                      (0x0007)              /* Divide by 8 */
#define TA1_EX0_RESERVED_1__M                              (0xfff8)              /* Reserved. Reads as 0. */


//*****************************************************************************
// TA2 Registers
//*****************************************************************************
typedef struct {
  union {    /* TA2_CTL Register */
    __IO uint16_t reg;
    struct { /* TA2_CTL Control Bits */
      __IO uint16_t TAIFG               	:	1;  /* Timer_A interrupt flag */
      __IO uint16_t TAIE                	:	1;  /* Timer_A interrupt enable. This bit enables the TAIFG interrupt request. */
      __IO uint16_t TACLR               	:	1;  /* Timer_A clear. Setting this bit resets TAxR, the timer clock divider logic, and  */
                                                /* the count direction. The TACLR bit is automatically reset and is always read as  */
                                                /* zero. */
      __IO uint16_t RESERVED_2          	:	1;  /* Reserved */
      __IO uint16_t MC                  	:	2;  /* Mode control. Setting MCx = 00h when Timer_A is not in use conserves power. */
      __IO uint16_t ID                  	:	2;  /* Input divider. These bits along with the TAIDEX bits select the divider for the  */
                                                /* input clock.   */
      __IO uint16_t TASSEL              	:	2;  /* Timer_A clock source select   */
      __IO uint16_t RESERVED_1          	:	6;  /* Reserved */

    } bit;
  } CTL;
  union {    /* TA2_CCTL Register */
    __IO uint16_t reg;
    struct { /* TA2_CCTL Control Bits */
      __IO uint16_t CCIFG               	:	1;  /* Capture/compare interrupt flag */
      __IO uint16_t COV                 	:	1;  /* Capture overflow. This bit indicates a capture overflow occurred. COV must be  */
                                                /* reset with software. */
      __IO uint16_t OUT                 	:	1;  /* Output. For output mode 0, this bit directly controls the state of the output. */
      __I  uint16_t CCI                 	:	1;  /* Capture/compare input. The selected input signal can be read by this bit. */
      __IO uint16_t CCIE                	:	1;  /* Capture/compare interrupt enable. This bit enables the interrupt request of	the  */
                                                /* corresponding CCIFG flag. */
      __IO uint16_t OUTMOD              	:	3;  /* Output mode. Modes 2, 3, 6, and 7 are not useful for TAxCCR0 because EQUx =	EQU0 */
                                                /* . */
      __IO uint16_t CAP                 	:	1;  /* Capture mode */
      __I  uint16_t RESERVED_1          	:	1;  /* Reserved */
      __IO uint16_t SCCI                	:	1;  /* Synchronized capture/compare input. The selected CCI input signal is latched  */
                                                /* with the EQUx signal and can be read via this bit. */
      __IO uint16_t SCS                 	:	1;  /* Synchronize capture source. This bit is used to synchronize the capture input	 */
                                                /* signal with the timer clock.   */
      __IO uint16_t CCIS                	:	2;  /* Capture/compare input select. These bits select the TAxCCR0 input signal. See	 */
                                                /* the device-specific data sheet for specific signal connections.   */
      __IO uint16_t CM                  	:	2;  /* Capture mode */

    } bit;
  } CCTL;
  uint8_t RESERVED0[12];
  uint16_t R; /* Timer_Ax Counter */
  uint16_t CCR; /* Timer_Ax Capture/Compare 0 to Timer_Ax Capture/Compare 6 */
  uint8_t RESERVED1[12];
  union {    /* TA2_EX0 Register */
    __IO uint16_t reg;
    struct { /* TA2_EX0 Control Bits */
      __IO uint16_t TAIDEX              	:	3;  /* Input divider expansion. These bits along with the ID bits select the divider	 */
                                                /* for the input clock. */
      __I  uint16_t RESERVED_1          	:	13;  /* Reserved */

    } bit;
  } EX0;
  uint8_t RESERVED2[12];
  uint16_t IV; /* Timer_Ax Interrupt Vector */
} TA2_Type;


/* TA2_CTL Control Bits */
#define TA2_CTL_TAIFG                                      (0x0001)              /* Timer_A interrupt flag */
#define TA2_CTL_TAIE                                       (0x0002)              /* Timer_A interrupt enable. This bit enables the TAIFG interrupt request. */
#define TA2_CTL_TACLR                                      (0x0004)              /* Timer_A clear. Setting this bit resets TAxR, the timer clock divider logic, and  */
                                                                                 /* the count direction. The TACLR bit is automatically reset and is always read as  */
                                                                                 /* zero. */
#define TA2_CTL_RESERVED_2                                 (0x0008)              /* Reserved */
#define TA2_CTL_MC__M                                      (0x0030)              /* Mode control. Setting MCx = 00h when Timer_A is not in use conserves power. */
#define TA2_CTL_MC__0                                      (0x0000)              /* Stop mode: Timer is halted */
#define TA2_CTL_MC__1                                      (0x0010)              /* Up mode: Timer counts up to TAxCCR0 */
#define TA2_CTL_MC__2                                      (0x0020)              /* Continuous mode: Timer counts up to 0FFFFh */
#define TA2_CTL_MC__3                                      (0x0030)              /* Up/down mode: Timer counts up to TAxCCR0 then down to 0000h */
#define TA2_CTL_ID__M                                      (0x00c0)              /* Input divider. These bits along with the TAIDEX bits select the divider for the  */
                                                                                 /* input clock.   */
#define TA2_CTL_ID__0                                      (0x0000)              /* /1 */
#define TA2_CTL_ID__0__1                                   (0x0000)              /* /1 */
#define TA2_CTL_ID__1                                      (0x0040)              /* /2 */
#define TA2_CTL_ID__1__2                                   (0x0040)              /* /2 */
#define TA2_CTL_ID__2                                      (0x0080)              /* /4 */
#define TA2_CTL_ID__2__4                                   (0x0080)              /* /4 */
#define TA2_CTL_ID__3                                      (0x00c0)              /* /8 */
#define TA2_CTL_ID__3__8                                   (0x00c0)              /* /8 */
#define TA2_CTL_TASSEL__M                                  (0x0300)              /* Timer_A clock source select   */
#define TA2_CTL_TASSEL__0                                  (0x0000)              /* TAxCLK */
#define TA2_CTL_TASSEL__0_TAXCLK                           (0x0000)              /* TAxCLK */
#define TA2_CTL_TASSEL__1                                  (0x0100)              /* ACLK */
#define TA2_CTL_TASSEL__1_ACLK                             (0x0100)              /* ACLK */
#define TA2_CTL_TASSEL__2                                  (0x0200)              /* SMCLK */
#define TA2_CTL_TASSEL__2_SMCLK                            (0x0200)              /* SMCLK */
#define TA2_CTL_TASSEL__3                                  (0x0300)              /* INCLK */
#define TA2_CTL_TASSEL__3_INCLK                            (0x0300)              /* INCLK */
#define TA2_CTL_RESERVED_1__M                              (0xfc00)              /* Reserved */

/* TA2_CCTL Control Bits */
#define TA2_CCTL_CCIFG                                     (0x0001)              /* Capture/compare interrupt flag */
#define TA2_CCTL_COV                                       (0x0002)              /* Capture overflow. This bit indicates a capture overflow occurred. COV must be  */
                                                                                 /* reset with software. */
#define TA2_CCTL_OUT                                       (0x0004)              /* Output. For output mode 0, this bit directly controls the state of the output. */
#define TA2_CCTL_CCI                                       (0x0008)              /* Capture/compare input. The selected input signal can be read by this bit. */
#define TA2_CCTL_CCIE                                      (0x0010)              /* Capture/compare interrupt enable. This bit enables the interrupt request of	the  */
                                                                                 /* corresponding CCIFG flag. */
#define TA2_CCTL_OUTMOD__M                                 (0x00e0)              /* Output mode. Modes 2, 3, 6, and 7 are not useful for TAxCCR0 because EQUx =	EQU0 */
                                                                                 /* . */
#define TA2_CCTL_OUTMOD__0                                 (0x0000)              /* OUT bit value */
#define TA2_CCTL_OUTMOD__0_OUT_BIT_VALUE                   (0x0000)              /* OUT bit value */
#define TA2_CCTL_OUTMOD__1                                 (0x0020)              /* Set */
#define TA2_CCTL_OUTMOD__1_SET                             (0x0020)              /* Set */
#define TA2_CCTL_OUTMOD__2                                 (0x0040)              /* Toggle/reset */
#define TA2_CCTL_OUTMOD__2_TOGGLE_RESET                    (0x0040)              /* Toggle/reset */
#define TA2_CCTL_OUTMOD__3                                 (0x0060)              /* Set/reset */
#define TA2_CCTL_OUTMOD__3_SET_RESET                       (0x0060)              /* Set/reset */
#define TA2_CCTL_OUTMOD__4                                 (0x0080)              /* Toggle */
#define TA2_CCTL_OUTMOD__4_TOGGLE                          (0x0080)              /* Toggle */
#define TA2_CCTL_OUTMOD__5                                 (0x00a0)              /* Reset */
#define TA2_CCTL_OUTMOD__5_RESET                           (0x00a0)              /* Reset */
#define TA2_CCTL_OUTMOD__6                                 (0x00c0)              /* Toggle/set */
#define TA2_CCTL_OUTMOD__6_TOGGLE_SET                      (0x00c0)              /* Toggle/set */
#define TA2_CCTL_OUTMOD__7                                 (0x00e0)              /* Reset/set */
#define TA2_CCTL_OUTMOD__7_RESET_SET                       (0x00e0)              /* Reset/set */
#define TA2_CCTL_CAP                                       (0x0100)              /* Capture mode */
#define TA2_CCTL_RESERVED_1                                (0x0200)              /* Reserved. Reads as 0. */
#define TA2_CCTL_SCCI                                      (0x0400)              /* Synchronized capture/compare input. The selected CCI input signal is latched  */
                                                                                 /* with the EQUx signal and can be read via this bit. */
#define TA2_CCTL_SCS                                       (0x0800)              /* Synchronize capture source. This bit is used to synchronize the capture input	 */
                                                                                 /* signal with the timer clock.   */
#define TA2_CCTL_CCIS__M                                   (0x3000)              /* Capture/compare input select. These bits select the TAxCCR0 input signal. See	 */
                                                                                 /* the device-specific data sheet for specific signal connections.   */
#define TA2_CCTL_CCIS__0                                   (0x0000)              /* CCIxA */
#define TA2_CCTL_CCIS__0_CCIXA                             (0x0000)              /* CCIxA */
#define TA2_CCTL_CCIS__1                                   (0x1000)              /* CCIxB */
#define TA2_CCTL_CCIS__1_CCIXB                             (0x1000)              /* CCIxB */
#define TA2_CCTL_CCIS__2                                   (0x2000)              /* GND */
#define TA2_CCTL_CCIS__2_GND                               (0x2000)              /* GND */
#define TA2_CCTL_CCIS__3                                   (0x3000)              /* VCC */
#define TA2_CCTL_CCIS__3_VCC                               (0x3000)              /* VCC */
#define TA2_CCTL_CM__M                                     (0xc000)              /* Capture mode */
#define TA2_CCTL_CM__0                                     (0x0000)              /* No capture */
#define TA2_CCTL_CM__0_NO_CAPTURE                          (0x0000)              /* No capture */
#define TA2_CCTL_CM__1                                     (0x4000)              /* Capture on rising edge */
#define TA2_CCTL_CM__2                                     (0x8000)              /* Capture on falling edge */
#define TA2_CCTL_CM__3                                     (0xc000)              /* Capture on both rising and falling edges */

/* TA2_EX0 Control Bits */
#define TA2_EX0_TAIDEX__M                                  (0x0007)              /* Input divider expansion. These bits along with the ID bits select the divider	 */
                                                                                 /* for the input clock. */
#define TA2_EX0_TAIDEX__0                                  (0x0000)              /* Divide by 1 */
#define TA2_EX0_TAIDEX__0_DIVIDE_BY_1                      (0x0000)              /* Divide by 1 */
#define TA2_EX0_TAIDEX__1                                  (0x0001)              /* Divide by 2 */
#define TA2_EX0_TAIDEX__1_DIVIDE_BY_2                      (0x0001)              /* Divide by 2 */
#define TA2_EX0_TAIDEX__2                                  (0x0002)              /* Divide by 3 */
#define TA2_EX0_TAIDEX__2_DIVIDE_BY_3                      (0x0002)              /* Divide by 3 */
#define TA2_EX0_TAIDEX__3                                  (0x0003)              /* Divide by 4 */
#define TA2_EX0_TAIDEX__3_DIVIDE_BY_4                      (0x0003)              /* Divide by 4 */
#define TA2_EX0_TAIDEX__4                                  (0x0004)              /* Divide by 5 */
#define TA2_EX0_TAIDEX__4_DIVIDE_BY_5                      (0x0004)              /* Divide by 5 */
#define TA2_EX0_TAIDEX__5                                  (0x0005)              /* Divide by 6 */
#define TA2_EX0_TAIDEX__5_DIVIDE_BY_6                      (0x0005)              /* Divide by 6 */
#define TA2_EX0_TAIDEX__6                                  (0x0006)              /* Divide by 7 */
#define TA2_EX0_TAIDEX__6_DIVIDE_BY_7                      (0x0006)              /* Divide by 7 */
#define TA2_EX0_TAIDEX__7                                  (0x0007)              /* Divide by 8 */
#define TA2_EX0_TAIDEX__7_DIVIDE_BY_8                      (0x0007)              /* Divide by 8 */
#define TA2_EX0_RESERVED_1__M                              (0xfff8)              /* Reserved. Reads as 0. */


//*****************************************************************************
// TA3 Registers
//*****************************************************************************
typedef struct {
  union {    /* TA3_CTL Register */
    __IO uint16_t reg;
    struct { /* TA3_CTL Control Bits */
      __IO uint16_t TAIFG               	:	1;  /* Timer_A interrupt flag */
      __IO uint16_t TAIE                	:	1;  /* Timer_A interrupt enable. This bit enables the TAIFG interrupt request. */
      __IO uint16_t TACLR               	:	1;  /* Timer_A clear. Setting this bit resets TAxR, the timer clock divider logic, and  */
                                                /* the count direction. The TACLR bit is automatically reset and is always read as  */
                                                /* zero. */
      __IO uint16_t RESERVED_2          	:	1;  /* Reserved */
      __IO uint16_t MC                  	:	2;  /* Mode control. Setting MCx = 00h when Timer_A is not in use conserves power. */
      __IO uint16_t ID                  	:	2;  /* Input divider. These bits along with the TAIDEX bits select the divider for the  */
                                                /* input clock.   */
      __IO uint16_t TASSEL              	:	2;  /* Timer_A clock source select   */
      __IO uint16_t RESERVED_1          	:	6;  /* Reserved */

    } bit;
  } CTL;
  union {    /* TA3_CCTL Register */
    __IO uint16_t reg;
    struct { /* TA3_CCTL Control Bits */
      __IO uint16_t CCIFG               	:	1;  /* Capture/compare interrupt flag */
      __IO uint16_t COV                 	:	1;  /* Capture overflow. This bit indicates a capture overflow occurred. COV must be  */
                                                /* reset with software. */
      __IO uint16_t OUT                 	:	1;  /* Output. For output mode 0, this bit directly controls the state of the output. */
      __I  uint16_t CCI                 	:	1;  /* Capture/compare input. The selected input signal can be read by this bit. */
      __IO uint16_t CCIE                	:	1;  /* Capture/compare interrupt enable. This bit enables the interrupt request of	the  */
                                                /* corresponding CCIFG flag. */
      __IO uint16_t OUTMOD              	:	3;  /* Output mode. Modes 2, 3, 6, and 7 are not useful for TAxCCR0 because EQUx =	EQU0 */
                                                /* . */
      __IO uint16_t CAP                 	:	1;  /* Capture mode */
      __I  uint16_t RESERVED_1          	:	1;  /* Reserved */
      __IO uint16_t SCCI                	:	1;  /* Synchronized capture/compare input. The selected CCI input signal is latched  */
                                                /* with the EQUx signal and can be read via this bit. */
      __IO uint16_t SCS                 	:	1;  /* Synchronize capture source. This bit is used to synchronize the capture input	 */
                                                /* signal with the timer clock.   */
      __IO uint16_t CCIS                	:	2;  /* Capture/compare input select. These bits select the TAxCCR0 input signal. See	 */
                                                /* the device-specific data sheet for specific signal connections.   */
      __IO uint16_t CM                  	:	2;  /* Capture mode */

    } bit;
  } CCTL;
  uint8_t RESERVED0[12];
  uint16_t R; /* Timer_Ax Counter */
  uint16_t CCR; /* Timer_Ax Capture/Compare 0 to Timer_Ax Capture/Compare 6 */
  uint8_t RESERVED1[12];
  union {    /* TA3_EX0 Register */
    __IO uint16_t reg;
    struct { /* TA3_EX0 Control Bits */
      __IO uint16_t TAIDEX              	:	3;  /* Input divider expansion. These bits along with the ID bits select the divider	 */
                                                /* for the input clock. */
      __I  uint16_t RESERVED_1          	:	13;  /* Reserved */

    } bit;
  } EX0;
  uint8_t RESERVED2[12];
  uint16_t IV; /* Timer_Ax Interrupt Vector */
} TA3_Type;


/* TA3_CTL Control Bits */
#define TA3_CTL_TAIFG                                      (0x0001)              /* Timer_A interrupt flag */
#define TA3_CTL_TAIE                                       (0x0002)              /* Timer_A interrupt enable. This bit enables the TAIFG interrupt request. */
#define TA3_CTL_TACLR                                      (0x0004)              /* Timer_A clear. Setting this bit resets TAxR, the timer clock divider logic, and  */
                                                                                 /* the count direction. The TACLR bit is automatically reset and is always read as  */
                                                                                 /* zero. */
#define TA3_CTL_RESERVED_2                                 (0x0008)              /* Reserved */
#define TA3_CTL_MC__M                                      (0x0030)              /* Mode control. Setting MCx = 00h when Timer_A is not in use conserves power. */
#define TA3_CTL_MC__0                                      (0x0000)              /* Stop mode: Timer is halted */
#define TA3_CTL_MC__1                                      (0x0010)              /* Up mode: Timer counts up to TAxCCR0 */
#define TA3_CTL_MC__2                                      (0x0020)              /* Continuous mode: Timer counts up to 0FFFFh */
#define TA3_CTL_MC__3                                      (0x0030)              /* Up/down mode: Timer counts up to TAxCCR0 then down to 0000h */
#define TA3_CTL_ID__M                                      (0x00c0)              /* Input divider. These bits along with the TAIDEX bits select the divider for the  */
                                                                                 /* input clock.   */
#define TA3_CTL_ID__0                                      (0x0000)              /* /1 */
#define TA3_CTL_ID__0__1                                   (0x0000)              /* /1 */
#define TA3_CTL_ID__1                                      (0x0040)              /* /2 */
#define TA3_CTL_ID__1__2                                   (0x0040)              /* /2 */
#define TA3_CTL_ID__2                                      (0x0080)              /* /4 */
#define TA3_CTL_ID__2__4                                   (0x0080)              /* /4 */
#define TA3_CTL_ID__3                                      (0x00c0)              /* /8 */
#define TA3_CTL_ID__3__8                                   (0x00c0)              /* /8 */
#define TA3_CTL_TASSEL__M                                  (0x0300)              /* Timer_A clock source select   */
#define TA3_CTL_TASSEL__0                                  (0x0000)              /* TAxCLK */
#define TA3_CTL_TASSEL__0_TAXCLK                           (0x0000)              /* TAxCLK */
#define TA3_CTL_TASSEL__1                                  (0x0100)              /* ACLK */
#define TA3_CTL_TASSEL__1_ACLK                             (0x0100)              /* ACLK */
#define TA3_CTL_TASSEL__2                                  (0x0200)              /* SMCLK */
#define TA3_CTL_TASSEL__2_SMCLK                            (0x0200)              /* SMCLK */
#define TA3_CTL_TASSEL__3                                  (0x0300)              /* INCLK */
#define TA3_CTL_TASSEL__3_INCLK                            (0x0300)              /* INCLK */
#define TA3_CTL_RESERVED_1__M                              (0xfc00)              /* Reserved */

/* TA3_CCTL Control Bits */
#define TA3_CCTL_CCIFG                                     (0x0001)              /* Capture/compare interrupt flag */
#define TA3_CCTL_COV                                       (0x0002)              /* Capture overflow. This bit indicates a capture overflow occurred. COV must be  */
                                                                                 /* reset with software. */
#define TA3_CCTL_OUT                                       (0x0004)              /* Output. For output mode 0, this bit directly controls the state of the output. */
#define TA3_CCTL_CCI                                       (0x0008)              /* Capture/compare input. The selected input signal can be read by this bit. */
#define TA3_CCTL_CCIE                                      (0x0010)              /* Capture/compare interrupt enable. This bit enables the interrupt request of	the  */
                                                                                 /* corresponding CCIFG flag. */
#define TA3_CCTL_OUTMOD__M                                 (0x00e0)              /* Output mode. Modes 2, 3, 6, and 7 are not useful for TAxCCR0 because EQUx =	EQU0 */
                                                                                 /* . */
#define TA3_CCTL_OUTMOD__0                                 (0x0000)              /* OUT bit value */
#define TA3_CCTL_OUTMOD__0_OUT_BIT_VALUE                   (0x0000)              /* OUT bit value */
#define TA3_CCTL_OUTMOD__1                                 (0x0020)              /* Set */
#define TA3_CCTL_OUTMOD__1_SET                             (0x0020)              /* Set */
#define TA3_CCTL_OUTMOD__2                                 (0x0040)              /* Toggle/reset */
#define TA3_CCTL_OUTMOD__2_TOGGLE_RESET                    (0x0040)              /* Toggle/reset */
#define TA3_CCTL_OUTMOD__3                                 (0x0060)              /* Set/reset */
#define TA3_CCTL_OUTMOD__3_SET_RESET                       (0x0060)              /* Set/reset */
#define TA3_CCTL_OUTMOD__4                                 (0x0080)              /* Toggle */
#define TA3_CCTL_OUTMOD__4_TOGGLE                          (0x0080)              /* Toggle */
#define TA3_CCTL_OUTMOD__5                                 (0x00a0)              /* Reset */
#define TA3_CCTL_OUTMOD__5_RESET                           (0x00a0)              /* Reset */
#define TA3_CCTL_OUTMOD__6                                 (0x00c0)              /* Toggle/set */
#define TA3_CCTL_OUTMOD__6_TOGGLE_SET                      (0x00c0)              /* Toggle/set */
#define TA3_CCTL_OUTMOD__7                                 (0x00e0)              /* Reset/set */
#define TA3_CCTL_OUTMOD__7_RESET_SET                       (0x00e0)              /* Reset/set */
#define TA3_CCTL_CAP                                       (0x0100)              /* Capture mode */
#define TA3_CCTL_RESERVED_1                                (0x0200)              /* Reserved. Reads as 0. */
#define TA3_CCTL_SCCI                                      (0x0400)              /* Synchronized capture/compare input. The selected CCI input signal is latched  */
                                                                                 /* with the EQUx signal and can be read via this bit. */
#define TA3_CCTL_SCS                                       (0x0800)              /* Synchronize capture source. This bit is used to synchronize the capture input	 */
                                                                                 /* signal with the timer clock.   */
#define TA3_CCTL_CCIS__M                                   (0x3000)              /* Capture/compare input select. These bits select the TAxCCR0 input signal. See	 */
                                                                                 /* the device-specific data sheet for specific signal connections.   */
#define TA3_CCTL_CCIS__0                                   (0x0000)              /* CCIxA */
#define TA3_CCTL_CCIS__0_CCIXA                             (0x0000)              /* CCIxA */
#define TA3_CCTL_CCIS__1                                   (0x1000)              /* CCIxB */
#define TA3_CCTL_CCIS__1_CCIXB                             (0x1000)              /* CCIxB */
#define TA3_CCTL_CCIS__2                                   (0x2000)              /* GND */
#define TA3_CCTL_CCIS__2_GND                               (0x2000)              /* GND */
#define TA3_CCTL_CCIS__3                                   (0x3000)              /* VCC */
#define TA3_CCTL_CCIS__3_VCC                               (0x3000)              /* VCC */
#define TA3_CCTL_CM__M                                     (0xc000)              /* Capture mode */
#define TA3_CCTL_CM__0                                     (0x0000)              /* No capture */
#define TA3_CCTL_CM__0_NO_CAPTURE                          (0x0000)              /* No capture */
#define TA3_CCTL_CM__1                                     (0x4000)              /* Capture on rising edge */
#define TA3_CCTL_CM__2                                     (0x8000)              /* Capture on falling edge */
#define TA3_CCTL_CM__3                                     (0xc000)              /* Capture on both rising and falling edges */

/* TA3_EX0 Control Bits */
#define TA3_EX0_TAIDEX__M                                  (0x0007)              /* Input divider expansion. These bits along with the ID bits select the divider	 */
                                                                                 /* for the input clock. */
#define TA3_EX0_TAIDEX__0                                  (0x0000)              /* Divide by 1 */
#define TA3_EX0_TAIDEX__0_DIVIDE_BY_1                      (0x0000)              /* Divide by 1 */
#define TA3_EX0_TAIDEX__1                                  (0x0001)              /* Divide by 2 */
#define TA3_EX0_TAIDEX__1_DIVIDE_BY_2                      (0x0001)              /* Divide by 2 */
#define TA3_EX0_TAIDEX__2                                  (0x0002)              /* Divide by 3 */
#define TA3_EX0_TAIDEX__2_DIVIDE_BY_3                      (0x0002)              /* Divide by 3 */
#define TA3_EX0_TAIDEX__3                                  (0x0003)              /* Divide by 4 */
#define TA3_EX0_TAIDEX__3_DIVIDE_BY_4                      (0x0003)              /* Divide by 4 */
#define TA3_EX0_TAIDEX__4                                  (0x0004)              /* Divide by 5 */
#define TA3_EX0_TAIDEX__4_DIVIDE_BY_5                      (0x0004)              /* Divide by 5 */
#define TA3_EX0_TAIDEX__5                                  (0x0005)              /* Divide by 6 */
#define TA3_EX0_TAIDEX__5_DIVIDE_BY_6                      (0x0005)              /* Divide by 6 */
#define TA3_EX0_TAIDEX__6                                  (0x0006)              /* Divide by 7 */
#define TA3_EX0_TAIDEX__6_DIVIDE_BY_7                      (0x0006)              /* Divide by 7 */
#define TA3_EX0_TAIDEX__7                                  (0x0007)              /* Divide by 8 */
#define TA3_EX0_TAIDEX__7_DIVIDE_BY_8                      (0x0007)              /* Divide by 8 */
#define TA3_EX0_RESERVED_1__M                              (0xfff8)              /* Reserved. Reads as 0. */


//*****************************************************************************
// UDMA Registers
//*****************************************************************************
typedef struct {
  uint32_t STAT; /* DMA Status */
  uint32_t CFG; /* DMA Configuration */
  uint32_t CTLBASE; /* Channel control data base pointer */
  uint32_t ALTBASE; /* Channel alternate control data base pointer */
  uint32_t WAITSTAT; /* Channel wait on request status */
  uint32_t SWREQ; /* Channel software request */
  uint32_t USEBURSTSET; /* Channel useburst set */
  uint32_t USEBURSTCLR; /* Channel useburst clear */
  uint32_t REQMASKSET; /* Channel request mask set */
  uint32_t REQMASKCLR; /* Channel request mask clear */
  uint32_t ENASET; /* Channel enable set */
  uint32_t ENACLR; /* Channel enable clear */
  uint32_t ALTSET; /* Channel primary-alternate set */
  uint32_t ALTCLR; /* Channel primary-alternate clear */
  uint32_t PRIOSET; /* Channel priority set */
  uint32_t PRIOCLR; /* Channel priority clear */
  uint8_t RESERVED0[12];
  uint32_t ERRCLR; /* Bus error clear */
  uint8_t RESERVED1[3968];
  uint32_t PERIPH_ID_4; /* Peripheral identification 4 */
  uint8_t RESERVED2[12];
  uint32_t PERIPH_ID_0; /* Peripheral identification 0 */
  uint32_t PERIPH_ID_1; /* Peripheral identification 1 */
  uint32_t PERIPH_ID_2; /* Peripheral identification 2 */
  uint32_t PERIPH_ID_3; /* Peripheral identification 3 */
  uint32_t PCELL_ID_0; /* PrimeCell identification 0 */
  uint32_t PCELL_ID_1; /* PrimeCell identification 1 */
  uint32_t PCELL_ID_2; /* PrimeCell identification 2 */
  uint32_t PCELL_ID_3; /* PrimeCell identification 3 */
} UDMA_Type;



//*****************************************************************************
// WDT_A Registers
//*****************************************************************************
typedef struct {
  uint8_t RESERVED0[12];
  union {    /* WDT_A_CTL Register */
    __IO uint16_t reg;
    struct { /* WDT_A_CTL Control Bits */
      __IO uint16_t IS                  	:	3;  /* Watchdog timer interval select. These bits select the watchdog timer interval to */
                                                /* generate either a WDT interrupt or a WDT reset . */
      __O  uint16_t CNTCL               	:	1;  /* Watchdog timer counter clear. Setting WDTCNTCL = 1 clears the count value to  */
                                                /* 0000h. This bit always reads 0h   */
      __IO uint16_t TMSEL               	:	1;  /* Watchdog timer mode select   */
      __IO uint16_t SSEL                	:	2;  /* Watchdog timer clock source select   */
      __IO uint16_t HOLD                	:	1;  /* Watchdog timer hold. This bit stops the watchdog timer. Setting WDTHOLD = 1	when */
                                                /* the WDT is not in use conserves power.   */
      __IO uint16_t PW                  	:	8;  /* Watchdog timer password. Always read as 069h. Must be written as 05Ah, or the	WD */
                                                /* will generate a reset */

    } bit;
  } CTL;
} WDT_A_Type;


/* WDT_A_CTL Control Bits */
#define WDT_A_CTL_IS__M                                    (0x0007)              /* Watchdog timer interval select. These bits select the watchdog timer interval to */
                                                                                 /* generate either a WDT interrupt or a WDT reset . */
#define WDT_A_CTL_IS__0                                    (0x0000)              /* Watchdog clock source / (2^(31)) (18:12:16 at 32.768 kHz) */
#define WDT_A_CTL_IS__1                                    (0x0001)              /* Watchdog clock source /(2^(27)) (01:08:16 at 32.768 kHz) */
#define WDT_A_CTL_IS__2                                    (0x0002)              /* Watchdog clock source /(2^(23)) (00:04:16 at 32.768 kHz) */
#define WDT_A_CTL_IS__3                                    (0x0003)              /* Watchdog clock source /(2^(19)) (00:00:16 at 32.768 kHz) */
#define WDT_A_CTL_IS__4                                    (0x0004)              /* Watchdog clock source /(2^(15)) (1 s at 32.768 kHz) */
#define WDT_A_CTL_IS__5                                    (0x0005)              /* Watchdog clock source / (2^(13)) (250 ms at 32.768 kHz) */
#define WDT_A_CTL_IS__6                                    (0x0006)              /* Watchdog clock source / (2^(9)) (15.625 ms at 32.768 kHz) */
#define WDT_A_CTL_IS__7                                    (0x0007)              /* Watchdog clock source / (2^(6)) (1.95 ms at 32.768 kHz) */
#define WDT_A_CTL_CNTCL                                    (0x0008)              /* Watchdog timer counter clear. Setting WDTCNTCL = 1 clears the count value to  */
                                                                                 /* 0000h. This bit always reads 0h   */
#define WDT_A_CTL_TMSEL                                    (0x0010)              /* Watchdog timer mode select   */
#define WDT_A_CTL_SSEL__M                                  (0x0060)              /* Watchdog timer clock source select   */
#define WDT_A_CTL_SSEL__0                                  (0x0000)              /* SMCLK */
#define WDT_A_CTL_SSEL__0_SMCLK                            (0x0000)              /* SMCLK */
#define WDT_A_CTL_SSEL__1                                  (0x0020)              /* ACLK */
#define WDT_A_CTL_SSEL__1_ACLK                             (0x0020)              /* ACLK */
#define WDT_A_CTL_SSEL__2                                  (0x0040)              /* VLOCLK */
#define WDT_A_CTL_SSEL__2_VLOCLK                           (0x0040)              /* VLOCLK */
#define WDT_A_CTL_SSEL__3                                  (0x0060)              /* BCLK */
#define WDT_A_CTL_SSEL__3_BCLK                             (0x0060)              /* BCLK */
#define WDT_A_CTL_HOLD                                     (0x0080)              /* Watchdog timer hold. This bit stops the watchdog timer. Setting WDTHOLD = 1	when */
                                                                                 /* the WDT is not in use conserves power.   */
#define WDT_A_CTL_PW__M                                    (0xff00)              /* Watchdog timer password. Always read as 069h. Must be written as 05Ah, or the	WD */
                                                                                 /* will generate a reset */

/* Pre-defined bitfield values */
#define WDT_A_PW_VAL                                       (0x5A00)              /* WDT Key Value for WDT write access */

//*****************************************************************************
// Peripheral memory map and struct for MSP432Ware
//*****************************************************************************
#define __MCU_HAS_ADC14__                                                        /* Module is available */
#define __MCU_HAS_AES256__                                                       /* Module is available */
#define __MCU_HAS_CAPTIO0__                                                      /* Module is available */
#define __MCU_HAS_CAPTIO1__                                                      /* Module is available */
#define __MCU_HAS_COMP_E0__                                                      /* Module is available */
#define __MCU_HAS_COMP_E1__                                                      /* Module is available */
#define __MCU_HAS_CRC32__                                                        /* Module is available */
#define __MCU_HAS_CS__                                                           /* Module is available */
#define __MCU_HAS_DDDS__                                                         /* Module is available */
#define __MCU_HAS_DMA__                                                          /* Module is available */
#define __MCU_HAS_EUSCI_A0_SPI__                                                 /* Module is available */
#define __MCU_HAS_EUSCI_A0_UART__                                                /* Module is available */
#define __MCU_HAS_EUSCI_A1_SPI__                                                 /* Module is available */
#define __MCU_HAS_EUSCI_A1_UART__                                                /* Module is available */
#define __MCU_HAS_EUSCI_A2_SPI__                                                 /* Module is available */
#define __MCU_HAS_EUSCI_A2_UART__                                                /* Module is available */
#define __MCU_HAS_EUSCI_A3_SPI__                                                 /* Module is available */
#define __MCU_HAS_EUSCI_A3_UART__                                                /* Module is available */
#define __MCU_HAS_EUSCI_B0_I2C__                                                 /* Module is available */
#define __MCU_HAS_EUSCI_B0_SPI__                                                 /* Module is available */
#define __MCU_HAS_EUSCI_B1_I2C__                                                 /* Module is available */
#define __MCU_HAS_EUSCI_B1_SPI__                                                 /* Module is available */
#define __MCU_HAS_EUSCI_B2_I2C__                                                 /* Module is available */
#define __MCU_HAS_EUSCI_B2_SPI__                                                 /* Module is available */
#define __MCU_HAS_EUSCI_B3_I2C__                                                 /* Module is available */
#define __MCU_HAS_EUSCI_B3_SPI__                                                 /* Module is available */
#define __MCU_HAS_FLCTL__                                                        /* Module is available */
#define __MCU_HAS_FPB__                                                          /* Module is available */
#define __MCU_HAS_P1__                                                           /* Module is available */
#define __MCU_HAS_P10__                                                          /* Module is available */
#define __MCU_HAS_P2__                                                           /* Module is available */
#define __MCU_HAS_P3__                                                           /* Module is available */
#define __MCU_HAS_P4__                                                           /* Module is available */
#define __MCU_HAS_P5__                                                           /* Module is available */
#define __MCU_HAS_P6__                                                           /* Module is available */
#define __MCU_HAS_P7__                                                           /* Module is available */
#define __MCU_HAS_P8__                                                           /* Module is available */
#define __MCU_HAS_P9__                                                           /* Module is available */
#define __MCU_HAS_PA__                                                           /* Module is available */
#define __MCU_HAS_PB__                                                           /* Module is available */
#define __MCU_HAS_PC__                                                           /* Module is available */
#define __MCU_HAS_PCM__                                                          /* Module is available */
#define __MCU_HAS_PD__                                                           /* Module is available */
#define __MCU_HAS_PE__                                                           /* Module is available */
#define __MCU_HAS_PJ__                                                           /* Module is available */
#define __MCU_HAS_PMAP__                                                         /* Module is available */
#define __MCU_HAS_PSS__                                                          /* Module is available */
#define __MCU_HAS_REF_A__                                                        /* Module is available */
#define __MCU_HAS_RSTCTL__                                                       /* Module is available */
#define __MCU_HAS_RTC_C__                                                        /* Module is available */
#define __MCU_HAS_SCS__                                                          /* Module is available */
#define __MCU_HAS_SYSCTL__                                                       /* Module is available */
#define __MCU_HAS_T32__                                                          /* Module is available */
#define __MCU_HAS_TA0__                                                          /* Module is available */
#define __MCU_HAS_TA1__                                                          /* Module is available */
#define __MCU_HAS_TA2__                                                          /* Module is available */
#define __MCU_HAS_TA3__                                                          /* Module is available */
#define __MCU_HAS_UDMA__                                                         /* Module is available */
#define __MCU_HAS_WDT_A__                                                        /* Module is available */

#define __ADC14_BASE__                                     (0x40012000)          /* Base address of module registers */
#define __AES256_BASE__                                    (0x40003C00)          /* Base address of module registers */
#define __CAPTIO0_BASE__                                   (0x40005400)          /* Base address of module registers */
#define __CAPTIO1_BASE__                                   (0x40005800)          /* Base address of module registers */
#define __COMP_E0_BASE__                                   (0x40003400)          /* Base address of module registers */
#define __COMP_E1_BASE__                                   (0x40003800)          /* Base address of module registers */
#define __CRC32_BASE__                                     (0x40004000)          /* Base address of module registers */
#define __CS_BASE__                                        (0x40010400)          /* Base address of module registers */
#define __DDDS_BASE__                                      (0x00203000)          /* Base address of module registers */
#define __DMA_BASE__                                       (0x4000E000)          /* Base address of module registers */
#define __EUSCI_A0_SPI_BASE__                              (0x40001000)          /* Base address of module registers */
#define __EUSCI_A0_UART_BASE__                             (0x40001000)          /* Base address of module registers */
#define __EUSCI_A1_SPI_BASE__                              (0x40001400)          /* Base address of module registers */
#define __EUSCI_A1_UART_BASE__                             (0x40001400)          /* Base address of module registers */
#define __EUSCI_A2_SPI_BASE__                              (0x40001800)          /* Base address of module registers */
#define __EUSCI_A2_UART_BASE__                             (0x40001800)          /* Base address of module registers */
#define __EUSCI_A3_SPI_BASE__                              (0x40001C00)          /* Base address of module registers */
#define __EUSCI_A3_UART_BASE__                             (0x40001C00)          /* Base address of module registers */
#define __EUSCI_B0_I2C_BASE__                              (0x40002000)          /* Base address of module registers */
#define __EUSCI_B0_SPI_BASE__                              (0x40002000)          /* Base address of module registers */
#define __EUSCI_B1_I2C_BASE__                              (0x40002400)          /* Base address of module registers */
#define __EUSCI_B1_SPI_BASE__                              (0x40002400)          /* Base address of module registers */
#define __EUSCI_B2_I2C_BASE__                              (0x40002800)          /* Base address of module registers */
#define __EUSCI_B2_SPI_BASE__                              (0x40002800)          /* Base address of module registers */
#define __EUSCI_B3_I2C_BASE__                              (0x40002C00)          /* Base address of module registers */
#define __EUSCI_B3_SPI_BASE__                              (0x40002C00)          /* Base address of module registers */
#define __FLCTL_BASE__                                     (0x40011000)          /* Base address of module registers */
#define __FPB_BASE__                                       (0xE0002000)          /* Base address of module registers */
#define __P1_BASE__                                        (0x40004C00)          /* Base address of module registers */
#define __P10_BASE__                                       (0x40004C81)          /* Base address of module registers */
#define __P2_BASE__                                        (0x40004C01)          /* Base address of module registers */
#define __P3_BASE__                                        (0x40004C20)          /* Base address of module registers */
#define __P4_BASE__                                        (0x40004C21)          /* Base address of module registers */
#define __P5_BASE__                                        (0x40004C40)          /* Base address of module registers */
#define __P6_BASE__                                        (0x40004C41)          /* Base address of module registers */
#define __P7_BASE__                                        (0x40004C60)          /* Base address of module registers */
#define __P8_BASE__                                        (0x40004C61)          /* Base address of module registers */
#define __P9_BASE__                                        (0x40004C80)          /* Base address of module registers */
#define __PA_BASE__                                        (0x40004C00)          /* Base address of module registers */
#define __PB_BASE__                                        (0x40004C20)          /* Base address of module registers */
#define __PC_BASE__                                        (0x40004C40)          /* Base address of module registers */
#define __PCM_BASE__                                       (0x40010000)          /* Base address of module registers */
#define __PD_BASE__                                        (0x40004C60)          /* Base address of module registers */
#define __PE_BASE__                                        (0x40004C80)          /* Base address of module registers */
#define __PJ_BASE__                                        (0x40004D20)          /* Base address of module registers */
#define __PMAP_BASE__                                      (0x40005000)          /* Base address of module registers */
#define __PSS_BASE__                                       (0x40010800)          /* Base address of module registers */
#define __REF_A_BASE__                                     (0x40003000)          /* Base address of module registers */
#define __RSTCTL_BASE__                                    (0xE0042000)          /* Base address of module registers */
#define __RTC_C_BASE__                                     (0x40004400)          /* Base address of module registers */
#define __SCS_BASE__                                       (0xE000E000)          /* Base address of module registers */
#define __SYSCTL_BASE__                                    (0xE0043000)          /* Base address of module registers */
#define __T32_BASE__                                       (0x4000C000)          /* Base address of module registers */
#define __TA0_BASE__                                       (0x40000000)          /* Base address of module registers */
#define __TA1_BASE__                                       (0x40000400)          /* Base address of module registers */
#define __TA2_BASE__                                       (0x40000800)          /* Base address of module registers */
#define __TA3_BASE__                                       (0x40000C00)          /* Base address of module registers */
#define __UDMA_BASE__                                      (0x4000F000)          /* Base address of module registers */
#define __WDT_A_BASE__                                     (0x40004800)          /* Base address of module registers */

#define ADC14                                              ((ADC14_Type *) __ADC14_BASE__)
#define AES256                                             ((AES256_Type *) __AES256_BASE__)
#define CAPTIO0                                            ((CAPTIO0_Type *) __CAPTIO0_BASE__)
#define CAPTIO1                                            ((CAPTIO1_Type *) __CAPTIO1_BASE__)
#define COMP_E0                                            ((COMP_E0_Type *) __COMP_E0_BASE__)
#define COMP_E1                                            ((COMP_E1_Type *) __COMP_E1_BASE__)
#define CRC32                                              ((CRC32_Type *) __CRC32_BASE__)
#define CS                                                 ((CS_Type *) __CS_BASE__)     
#define DDDS                                               ((DDDS_Type *) __DDDS_BASE__) 
#define DMA                                                ((DMA_Type *) __DMA_BASE__)   
#define EUSCI_A0_SPI                                       ((EUSCI_A0_SPI_Type *) __EUSCI_A0_SPI_BASE__)
#define EUSCI_A0_UART                                      ((EUSCI_A0_UART_Type *) __EUSCI_A0_UART_BASE__)
#define EUSCI_A1_SPI                                       ((EUSCI_A1_SPI_Type *) __EUSCI_A1_SPI_BASE__)
#define EUSCI_A1_UART                                      ((EUSCI_A1_UART_Type *) __EUSCI_A1_UART_BASE__)
#define EUSCI_A2_SPI                                       ((EUSCI_A2_SPI_Type *) __EUSCI_A2_SPI_BASE__)
#define EUSCI_A2_UART                                      ((EUSCI_A2_UART_Type *) __EUSCI_A2_UART_BASE__)
#define EUSCI_A3_SPI                                       ((EUSCI_A3_SPI_Type *) __EUSCI_A3_SPI_BASE__)
#define EUSCI_A3_UART                                      ((EUSCI_A3_UART_Type *) __EUSCI_A3_UART_BASE__)
#define EUSCI_B0_I2C                                       ((EUSCI_B0_I2C_Type *) __EUSCI_B0_I2C_BASE__)
#define EUSCI_B0_SPI                                       ((EUSCI_B0_SPI_Type *) __EUSCI_B0_SPI_BASE__)
#define EUSCI_B1_I2C                                       ((EUSCI_B1_I2C_Type *) __EUSCI_B1_I2C_BASE__)
#define EUSCI_B1_SPI                                       ((EUSCI_B1_SPI_Type *) __EUSCI_B1_SPI_BASE__)
#define EUSCI_B2_I2C                                       ((EUSCI_B2_I2C_Type *) __EUSCI_B2_I2C_BASE__)
#define EUSCI_B2_SPI                                       ((EUSCI_B2_SPI_Type *) __EUSCI_B2_SPI_BASE__)
#define EUSCI_B3_I2C                                       ((EUSCI_B3_I2C_Type *) __EUSCI_B3_I2C_BASE__)
#define EUSCI_B3_SPI                                       ((EUSCI_B3_SPI_Type *) __EUSCI_B3_SPI_BASE__)
#define FLCTL                                              ((FLCTL_Type *) __FLCTL_BASE__)
#define FPB                                                ((FPB_Type *) __FPB_BASE__)   
#define P1                                                 ((P1_Type *) __P1_BASE__)     
#define P10                                                ((P10_Type *) __P10_BASE__)   
#define P2                                                 ((P2_Type *) __P2_BASE__)     
#define P3                                                 ((P3_Type *) __P3_BASE__)     
#define P4                                                 ((P4_Type *) __P4_BASE__)     
#define P5                                                 ((P5_Type *) __P5_BASE__)     
#define P6                                                 ((P6_Type *) __P6_BASE__)     
#define P7                                                 ((P7_Type *) __P7_BASE__)     
#define P8                                                 ((P8_Type *) __P8_BASE__)     
#define P9                                                 ((P9_Type *) __P9_BASE__)     
#define PA                                                 ((PA_Type *) __PA_BASE__)     
#define PB                                                 ((PB_Type *) __PB_BASE__)     
#define PC                                                 ((PC_Type *) __PC_BASE__)     
#define PCM                                                ((PCM_Type *) __PCM_BASE__)   
#define PD                                                 ((PD_Type *) __PD_BASE__)     
#define PE                                                 ((PE_Type *) __PE_BASE__)     
#define PJ                                                 ((PJ_Type *) __PJ_BASE__)     
#define PMAP                                               ((PMAP_Type *) __PMAP_BASE__) 
#define PSS                                                ((PSS_Type *) __PSS_BASE__)   
#define REF_A                                              ((REF_A_Type *) __REF_A_BASE__)
#define RSTCTL                                             ((RSTCTL_Type *) __RSTCTL_BASE__)
#define RTC_C                                              ((RTC_C_Type *) __RTC_C_BASE__)
#define SCS                                                ((SCS_Type *) __SCS_BASE__)   
#define SYSCTL                                             ((SYSCTL_Type *) __SYSCTL_BASE__)
#define T32                                                ((T32_Type *) __T32_BASE__)   
#define TA0                                                ((TA0_Type *) __TA0_BASE__)   
#define TA1                                                ((TA1_Type *) __TA1_BASE__)   
#define TA2                                                ((TA2_Type *) __TA2_BASE__)   
#define TA3                                                ((TA3_Type *) __TA3_BASE__)   
#define UDMA                                               ((UDMA_Type *) __UDMA_BASE__) 
#define WDT_A                                              ((WDT_A_Type *) __WDT_A_BASE__)


#endif // __MSP432P401R_CMSIS_H
